using System;
using System.Threading;
using GHI.OSHW.Hardware.LowLevel;
using Microsoft.SPOT;

namespace Cerberus_PWM
{
    class CerberusSine
    {
        ///////////////////////////////////////////////////////////////////////////////////////////////////
        // Coded by Dat - GHI Electronics Code Share Page...                                             //
        // https://www.ghielectronics.com/community/codeshare/entry/465                                  //
        // This program is example on how to make analog signal out put using register class.            //
        ///////////////////////////////////////////////////////////////////////////////////////////////////

        // RCC
        const UInt32 RCC_BASE = 0x40023800;

        // Offsets
        const UInt32 AHB1ENR_OFFSET = 0x30;
        const UInt32 APB1ENR_OFFSET = 0x40;

        // GPIOx Offsets
        const UInt32 MODER_OFFSET = 0x00;
        const UInt32 PUPDR_OFFSET = 0x0C;

        // GPIOA
        const UInt32 PERIPH_BASE = 0x40000000;
        const UInt32 AHB1PERIPH_BASE = (PERIPH_BASE + 0x00020000);
        const UInt32 GPIOA_BASE = (AHB1PERIPH_BASE + 0x0000);

        //DAC
        const UInt32 DAC_BASE = (PERIPH_BASE + 0x7400);
        const UInt32 DAC_CR_OFFSET = 0x00;
        const UInt32 DAC_DHR12R1_OFFSET = 0x8;
        const UInt32 DAC_DHR12R2_OFFSET = 0x14;

        // Register
        static Register RCC_AHB1ENR = new Register(RCC_BASE + AHB1ENR_OFFSET);
        static Register RCC_APB1ENR = new Register(RCC_BASE + APB1ENR_OFFSET);
        static Register GPIOx_MODER = new Register(GPIOA_BASE + MODER_OFFSET);
        static Register GPIOx_PUPDR = new Register(GPIOA_BASE + PUPDR_OFFSET);
        static Register DAC_CR = new Register(DAC_BASE + DAC_CR_OFFSET);
        static Register DAC_DHR12R1 = new Register(DAC_BASE + DAC_DHR12R1_OFFSET); // 12-bit right aligned channel 1
        static Register DAC_DHR12R2 = new Register(DAC_BASE + DAC_DHR12R2_OFFSET); // 12-bit right aligned channel 2

        public static void CerberusSineGen()
        {
            //GPIOA clock enable (to be used with DAC) 
            //Bit 0 GPIOAEN: IO port A clock enable
            RCC_AHB1ENR.SetBits(1 << 0);
            //Bit 29 DACEN: DAC interface clock enable
            RCC_APB1ENR.SetBits(1 << 29);

            //11b: Analog mode 
            //Pin 4 is analog
            GPIOx_MODER.SetBits(3 << 8);
            //Pin 5 is analog
            GPIOx_MODER.SetBits(3 << 10);

            //00: No pull-up, pull-down
            GPIOx_PUPDR.ClearBits(1 << 0);
            GPIOx_PUPDR.ClearBits(1 << 1);

            // Reset DAC
            DAC_CR.Write(0);

            // Enable DAC channel 1
            DAC_CR.SetBits(1 << 0);
            // Enable DAC channel 2
            DAC_CR.SetBits(1 << 16);
            // Scale
            SetLinearScale(0, 100);
            // Set 2.5V on channel 1 PA4
            SetVol(1, 75);
            // Set 1.6V on channel 2 PA5
            SetVol(2, 50);
            Thread.Sleep(Timeout.Infinite);
        }

        static int min = 0x0000;
        static int max = 0x0FFF;

        static void SetLinearScale(int minVal, int maxVal)
        {
            min = minVal;
            max = maxVal;
        }

        static void SetVol(int chanel, int value)
        {
            int resMax = 0x0FFF - 1;
            value = (resMax * (value - min)) / (max - min);
            Register DAC_DHR12Rx = new Register(DAC_BASE + (chanel == 1 ? DAC_DHR12R1_OFFSET : DAC_DHR12R2_OFFSET));
            DAC_DHR12Rx.Write((UInt32)value);
        }
    }


}
