using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;

namespace Cerberus_PWM
{
    public sealed class PWMChannel : PWM
    {
        #region PWM Pin Outs...
        // X3...
        // Pin 7
        // Pin 8
        // Pin 9

        // X4...
        // Pin 7
        // Pin 8
        // Pin 9

        #endregion

        #region Below does not work...
        // X5...
        // Pin 7
        // Pin 8
        // Pin 9

        // X6...
        // Pin 7
        // Pin 8
        // Pin 9

        // X8...
        // Pin 7
        #endregion

        /////////////////////////////////////////////////////////////////////////
        // See for more information:                                           //
        // http://msdn.microsoft.com/en-us/library/jj633523(v=vs.102).aspx     //
        // http://msdn.microsoft.com/en-us/library/jj646655(v=vs.102).aspx     //
        /////////////////////////////////////////////////////////////////////////

        /////////////////////////////////////////////////////////////// 
        // References the Library: "Microsoft.SPOT.Hardware.PWM"     //
        // Add using statement: using Microsoft.SPOT.Hardware;       //
        ///////////////////////////////////////////////////////////////

        /// <summary>
        /// Get or Set PWMChannel's Frequency using the inhetited member 'Frequency'...
        /// </summary>
        private uint ChannelFrequency
        {
            get { return (uint)Frequency; }
            set { Frequency = value; }
        }

        /// <summary>
        /// Get or Set the PWMChannel's Duty Cycle using the inhetited member 'DutyCycle'...
        /// </summary>
        private double ChannelDutyCycle
        {
            get { return DutyCycle; }
            set { DutyCycle = value; }
        }

        /// <summary>
        /// Channel 1 - Set for output on Pins 8 on X3.
        /// </summary>
        public static PWMChannel Channel_1 = new PWMChannel(Cpu.PWMChannel.PWM_0, 10, 0, GlobalVariable.InvertChannel_1);

        /// <summary>
        /// Channel 2 - Set for output on Pins 9 on X3.
        /// </summary>
        public static PWMChannel Channel_2 = new PWMChannel(Cpu.PWMChannel.PWM_2, 10, 0, GlobalVariable.InvertChannel_2);

        /// <summary>
        /// Channel 3 - Set for output on Pins 8 on X4.
        /// </summary>
        public static PWMChannel Channel_3 = new PWMChannel(Cpu.PWMChannel.PWM_3, 10, 0, GlobalVariable.InvertChannel_3);

        /// <summary>
        /// Channel 4 - Set for output on Pins 9 on X4.
        /// </summary>
        public static PWMChannel Channel_4 = new PWMChannel(Cpu.PWMChannel.PWM_5, 10, 0, GlobalVariable.InvertChannel_4);

        /// <summary>
        /// Stop Set Parameters and Start PWM on the desired Configuration...
        /// </summary>
        /// <param name="frequency_Hz">The Frequency in Hertz</param>
        /// <param name="dutyCycle">Duty Cycle expressed in Percentage EG: 0.5 gives 50% Duty Cycle of the Frequency</param>
        /// <param name="invert">Inverts the Output Pin</param>
        public void Configure(uint frequency_Hz, double dutyCycle)
        {
            this.Stop();

            ChannelFrequency = frequency_Hz;
            ChannelDutyCycle = dutyCycle;

            this.Start();
        }

        /// <summary>
        /// All Stop! Stops the Channel's PWM...
        /// </summary>
        public void ChannelStop()
        {
            this.Stop();
        }

        /// <summary>
        /// Standard Contructor, explicit PWM Inheritance.
        /// </summary>
        /// <param name="channel">The Cpu.PWMChannel Channel</param>
        /// <param name="frequency_Hz">The Frequency in Hertz</param>
        /// <param name="dutyCycle">Duty Cycle expressed in Percentage EG: 0.5 gives 50% Duty Cycle of the Frequency</param>
        /// <param name="invert">Inverts the Output Pin</param>
        private PWMChannel(Cpu.PWMChannel channel, uint frequency_Hz, double dutyCycle, bool invert) : base(channel, frequency_Hz, dutyCycle, invert)
        {
            Debug.Print("PWMChannel Class Instantiated - First Constructor called...");
        }

        /// <summary>
        /// Standard Contructor, explicit PWM Inheritance.
        /// </summary>
        /// <param name="channel">The Cpu.PWMChannel Channel</param>
        /// <param name="period">The period of the pulse in microseconds.</param>
        /// <param name="duration">The duration of the pulse in microseconds. This value should be a fraction of the period parameter.</param>
        /// <param name="scale">PWM.ScaleFactor Enumeration - Provides scale factor values for duration and period.</param>
        /// <param name="invert">Inverts the Output Pin</param>
        private PWMChannel(Cpu.PWMChannel channel, uint period, uint duration, PWM.ScaleFactor scale, bool invert) : base(channel, period, duration, scale, invert)
        {
            Debug.Print("PWMChannel Class Instantiated - First Constructor called...");
        }

    } // END of Class...
} // END of Namespace...