﻿// System...
using System;
using System.Threading;
using System.Collections;

// Microsoft...
using Microsoft.SPOT;
using Microsoft.SPOT.Touch;
using Microsoft.SPOT.Hardware;
using Microsoft.SPOT.Presentation;
using Microsoft.SPOT.Presentation.Media;
using Microsoft.SPOT.Presentation.Shapes;
using Microsoft.SPOT.Presentation.Controls;

// Gadgeteer...
using GT = Gadgeteer;
using Gadgeteer.Networking;
using GTM = Gadgeteer.Modules;
using Gadgeteer.Modules.GHIElectronics;
using GHI.Processor;
using System.Text;

namespace Cerberus_PWM
{
    public partial class Program
    {

        private Timer timer = new Timer();

        void ProgramStarted()
        {
            Debug.Print("Program Started");

            // https://www.ghielectronics.com/docs/149/device-to-pc-communication
            int baudRate = 921600; // 4800, 9600, 14400, 19200, 28800, 38400, 57600, 115200, 230400, 460800, 921600
            int dataBits = 8;
            this.usbSerial.Configure(baudRate, GT.SocketInterfaces.SerialParity.None, GT.SocketInterfaces.SerialStopBits.One, dataBits, GT.SocketInterfaces.HardwareFlowControl.NotRequired);
            this.usbSerial.Port.LineReceived += this.Port_LineReceived;
            this.usbSerial.Port.NewLine = "<";

            // Listen for the PWM Control on another Thread...
            Thread PWMControlThread = new Thread(PWMControl);
            PWMControlThread.Start();

            // Listen for the PWM Reporting on another Thread...
            Thread CurrentSensingThread = new Thread(CurrentSensing);
            CurrentSensingThread.Start();

            // Show the Settings on the desired Registers...
            //GetSettings getSettings = new GetSettings();
            //getSettings.GetRegisterSettings(STM32F4xx.GPIOB_BASE, STM32F4xx.TIM3_BASE);
        }


        #region Serial Rx Event Handler...

        
        
        void Port_LineReceived(GT.SocketInterfaces.Serial sender, string line)
        {
            
            // <1|0002000|0.5|1>

            #region Get Serial Data...

            // Get the Data Length...
            int count = sender.BytesToRead;

            // Set a new RXD Buffer for the Incomming Data...
            byte[] RXD = new byte[count];

            // Real all the Data...
            sender.Read(RXD, 0, count);

            // Get the RXD Data and convert to String...
            string RXData = new String(System.Text.Encoding.UTF8.GetChars(RXD), 0, RXD.Length);

            this.usbSerial.Port.WriteLine(RXData);
            
            #endregion

            #region Process Serial Data Received...

            // Seperate the Frequency and Duty Cycle into an Array...
            string[] values = RXData.Split('|');

            // Get the Output Timer and Channel...
            string _OutputPort = values[0].Substring(0, 1);

            // Get the Frequency's ready for Parsing...
            string _Frequency = values[1].Substring(0, 7);

            // Get the Duty Cycle's ready for Parsing...
            string _Duty = values[2].Substring(0, 3);

            // Get the Duty Divider ready for Parsing...
            string _Divider = values[3].Substring(0, 3);

            // Get the Mode ready for Parsing...
            string _Mode = values[4].Substring(0, 1);



            CPU.Port OutputPort = CPU.Parse(_OutputPort);
            double frequency = double.Parse(_Frequency);
            double duty = double.Parse(_Duty);
            int divider = int.Parse(_Divider);
            Output.Mode OutputMode = Output.Parse(_Mode);

            duty = duty / divider;


            bool debug = true;

            if (debug)
            {
                Debug.Print("_OutputPort: " + _OutputPort);
                Debug.Print("_Frequency: " + frequency);
                Debug.Print("_Duty: " + duty);
                Debug.Print("_Mode: " + _Mode);
            }

            // Set Master PWM...
            timer.SetPWM(OutputPort, frequency, duty, false, OutputMode);


            #endregion

        }



        #endregion



        #region Application Threads...

        void PWMControl()
        {
            #region Fields...

            double frequency = 7000;
            double dutyCycle = 0;

            #endregion

            #region Set PWM...

            // Set Master PWM...
            timer.SetPWM(CPU.Port.T1Ch1, frequency, dutyCycle, true, Output.Mode.SingleChannel);

            #endregion

            #region Degree Loop...

            int Offset = 0;
            bool debuging = false;

            if (debuging)
            {
                // Loop through 180 Degrees of phase shift...
                for (int i = 1; i <= 360; i++)
                {
                    Debug.Print("Degree of OffSet: " + i + " degrees.");

                    int OffSet = 0;

                    if (i > Offset)
                    {
                        OffSet = (i - Offset);
                    }
                    else
                    {
                        OffSet = (Offset + i);
                    }

                    // timer.SetSlaveOffSet(CPU.Output.T1Ch1, i);

                    frequency = frequency + 100;

                    timer.SetPWM(CPU.Port.T1Ch1, frequency, dutyCycle, false, Output.Mode.SingleChannel);

                    Thread.Sleep(1000);

                }
            }

            #endregion

            #region Thread Loop...

            while (true)
            {
                // Thread.Sleep(30000);
            }

            #endregion
        }

        void CurrentSensing()
        {
            // Fields...
            double Current = 0;
            double MaxCurrent = 0;
            double MaxFrequency = 0;
            double FrequencyDrift = 0;

            // Thread Loop...
            while (true)
            {
                // Current Reset...
                Current = 0;

                // Take several Current Readings for a Mean Current...
                for (int i = 0; i < 7; i++)
                {
                    // Take Current Reading's...
                    //Current += currentACS712.ReadACCurrent();
                    // Debug.Print("AC Current: " + current_ACS712.Read_AC_Current());

                    // Sleep Here...
                    Thread.Sleep(100);
                }

                // Calculate Mean Current...
                double MeanCurrent = 0;

                MeanCurrent = (Current / 7);

                // MeanCurrent = current_ACS712.Read_AC_Current();

                //Debug.Print("Mean Current: " + MeanCurrent);

                // Get the current Frequency...
                double Frequency = timer.GetFrequency(CPU.Pin.Pin7_X4);

                // Sleep Here...
                Thread.Sleep(500);
            }
        }

        #endregion



        #region Helper Methods...

        private uint GetFrequencyPercentage(uint Frequency)
        {
            ////////////////////////////////////////////////////////////////////////
            // Typically returning One Percent of the current frequency...        //
            ////////////////////////////////////////////////////////////////////////
            if (Frequency < 100)
            {
                return 1;
            }

            uint F = 0;

            F = ((Frequency * 1) / 100);

            return F;
        }

        private double GetDutyCyclePercentage(double DutyCycle)
        {
            double D = 0;

            D = (DutyCycle * 100);

            return ((int)D);
        }

        #endregion



    } // END of Class...

} // END of Namespace...
