using System;
using Microsoft.SPOT;
using GHI.Processor;

namespace Cerberus_PWM
{
    /// <summary>
    /// CMSIS Cortex-M4 Device Peripheral Access Layer Header File. 
    /// This file contains all the peripheral register's definitions, bits 
    /// definitions and memory mapping for STM32F4xx devices.
    /// Converted to C# by Chris Sykes...
    /// </summary>
    public class STM32F4xx
    {
        /// <summary>
        ///   FLASH(up to 1 MB) base address in the alias region                         
        /// </summary>
        public static UInt32 FLASH_BASE = 0x08000000;

        /// <summary>
        ///   CCM(core coupled memory) data RAM(64 KB) base address in the alias region  
        /// </summary>
        public static UInt32 CCMDATARAM_BASE = 0x10000000;

        /// <summary>
        ///   SRAM1(112 KB) base address in the alias region                             
        /// </summary>
        public static UInt32 SRAM1_BASE = 0x20000000;

        /// <summary>
        ///   SRAM2(16 KB) base address in the alias region                              
        /// </summary>
        public static UInt32 SRAM2_BASE = 0x2001C000;

        /// <summary>
        ///   Peripheral base address in the alias region                                
        /// </summary>
        public static UInt32 PERIPH_BASE = 0x40000000;

        /// <summary>
        ///   Backup SRAM(4 KB) base address in the alias region                         
        /// </summary>
        public static UInt32 BKPSRAM_BASE = 0x40024000;

        /// <summary>
        ///   FSMC registers base address                                                
        /// </summary>
        public static UInt32 FSMC_R_BASE = 0xA0000000;

        /// <summary>
        ///   CCM(core coupled memory) data RAM(64 KB) base address in the bit-band region  
        /// </summary>
        public static UInt32 CCMDATARAM_BB_BASE = 0x12000000;

        /// <summary>
        ///   SRAM1(112 KB) base address in the bit-band region                             
        /// </summary>
        public static UInt32 SRAM1_BB_BASE = 0x22000000;

        /// <summary>
        ///   SRAM2(16 KB) base address in the bit-band region                              
        /// </summary>
        public static UInt32 SRAM2_BB_BASE = 0x2201C000;

        /// <summary>
        ///   Peripheral base address in the bit-band region                                
        /// </summary>
        public static UInt32 PERIPH_BB_BASE = 0x42000000;

        /// <summary>
        ///   Backup SRAM(4 KB) base address in the bit-band region                         
        /// </summary>
        public static UInt32 BKPSRAM_BB_BASE = 0x42024000;

        /// <summary>
        /// Debug MCU registers base address
        /// </summary>
        public static UInt32 DBGMCU_BASE = 0xE0042000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 SRAM_BASE = SRAM1_BASE;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 SRAM_BB_BASE = SRAM1_BB_BASE;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 APB1PERIPH_BASE = PERIPH_BASE;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 APB2PERIPH_BASE = (PERIPH_BASE + 0x00010000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 AHB1PERIPH_BASE = (PERIPH_BASE + 0x00020000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 AHB2PERIPH_BASE = (PERIPH_BASE + 0x10000000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM2_BASE = (APB1PERIPH_BASE + 0x0000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM3_BASE = (APB1PERIPH_BASE + 0x0400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM4_BASE = (APB1PERIPH_BASE + 0x0800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM5_BASE = (APB1PERIPH_BASE + 0x0C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM6_BASE = (APB1PERIPH_BASE + 0x1000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM7_BASE = (APB1PERIPH_BASE + 0x1400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM12_BASE = (APB1PERIPH_BASE + 0x1800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM13_BASE = (APB1PERIPH_BASE + 0x1C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM14_BASE = (APB1PERIPH_BASE + 0x2000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BASE = (APB1PERIPH_BASE + 0x2800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 WWDG_BASE = (APB1PERIPH_BASE + 0x2C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 IWDG_BASE = (APB1PERIPH_BASE + 0x3000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 I2S2ext_BASE = (APB1PERIPH_BASE + 0x3400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 SPI2_BASE = (APB1PERIPH_BASE + 0x3800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 SPI3_BASE = (APB1PERIPH_BASE + 0x3C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 I2S3ext_BASE = (APB1PERIPH_BASE + 0x4000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 USART2_BASE = (APB1PERIPH_BASE + 0x4400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 USART3_BASE = (APB1PERIPH_BASE + 0x4800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 UART4_BASE = (APB1PERIPH_BASE + 0x4C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 UART5_BASE = (APB1PERIPH_BASE + 0x5000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 I2C1_BASE = (APB1PERIPH_BASE + 0x5400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 I2C2_BASE = (APB1PERIPH_BASE + 0x5800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 I2C3_BASE = (APB1PERIPH_BASE + 0x5C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CAN1_BASE = (APB1PERIPH_BASE + 0x6400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CAN2_BASE = (APB1PERIPH_BASE + 0x6800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 PWR_BASE = (APB1PERIPH_BASE + 0x7000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DAC_BASE = (APB1PERIPH_BASE + 0x7400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM1_BASE = (APB2PERIPH_BASE + 0x0000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM8_BASE = (APB2PERIPH_BASE + 0x0400);

        /// <summary>
        /// // (APB2PERIPH_BASE + 0x1000) Cerberus does not use USART1 but
        /// </summary>
        public static UInt32 USART1_BASE = (APB2PERIPH_BASE + 0x1400);

        /// <summary>
        /// // Cerbuino requires USART6 but it can think it is USART1
        /// </summary>
        public static UInt32 USART6_BASE = (APB2PERIPH_BASE + 0x1400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 ADC1_BASE = (APB2PERIPH_BASE + 0x2000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 ADC2_BASE = (APB2PERIPH_BASE + 0x2100);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 ADC3_BASE = (APB2PERIPH_BASE + 0x2200);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 ADC_BASE = (APB2PERIPH_BASE + 0x2300);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 SDIO_BASE = (APB2PERIPH_BASE + 0x2C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 SPI1_BASE = (APB2PERIPH_BASE + 0x3000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 SYSCFG_BASE = (APB2PERIPH_BASE + 0x3800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 EXTI_BASE = (APB2PERIPH_BASE + 0x3C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM9_BASE = (APB2PERIPH_BASE + 0x4000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM10_BASE = (APB2PERIPH_BASE + 0x4400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM11_BASE = (APB2PERIPH_BASE + 0x4800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIOA_BASE = (AHB1PERIPH_BASE + 0x0000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIOB_BASE = (AHB1PERIPH_BASE + 0x0400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIOC_BASE = (AHB1PERIPH_BASE + 0x0800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIOD_BASE = (AHB1PERIPH_BASE + 0x0C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIOE_BASE = (AHB1PERIPH_BASE + 0x1000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIOF_BASE = (AHB1PERIPH_BASE + 0x1400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIOG_BASE = (AHB1PERIPH_BASE + 0x1800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIOH_BASE = (AHB1PERIPH_BASE + 0x1C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIOI_BASE = (AHB1PERIPH_BASE + 0x2000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRC_BASE = (AHB1PERIPH_BASE + 0x3000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_BASE = (AHB1PERIPH_BASE + 0x3800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_R_BASE = (AHB1PERIPH_BASE + 0x3C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA1_BASE = (AHB1PERIPH_BASE + 0x6000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA1_Stream0_BASE = (DMA1_BASE + 0x010);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA1_Stream1_BASE = (DMA1_BASE + 0x028);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA1_Stream2_BASE = (DMA1_BASE + 0x040);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA1_Stream3_BASE = (DMA1_BASE + 0x058);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA1_Stream4_BASE = (DMA1_BASE + 0x070);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA1_Stream5_BASE = (DMA1_BASE + 0x088);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA1_Stream6_BASE = (DMA1_BASE + 0x0A0);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA1_Stream7_BASE = (DMA1_BASE + 0x0B8);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA2_BASE = (AHB1PERIPH_BASE + 0x6400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA2_Stream0_BASE = (DMA2_BASE + 0x010);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA2_Stream1_BASE = (DMA2_BASE + 0x028);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA2_Stream2_BASE = (DMA2_BASE + 0x040);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA2_Stream3_BASE = (DMA2_BASE + 0x058);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA2_Stream4_BASE = (DMA2_BASE + 0x070);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA2_Stream5_BASE = (DMA2_BASE + 0x088);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA2_Stream6_BASE = (DMA2_BASE + 0x0A0);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA2_Stream7_BASE = (DMA2_BASE + 0x0B8);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 ETH_BASE = (AHB1PERIPH_BASE + 0x8000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 ETH_MAC_BASE = (ETH_BASE);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 ETH_MMC_BASE = (ETH_BASE + 0x0100);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 ETH_PTP_BASE = (ETH_BASE + 0x0700);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 ETH_DMA_BASE = (ETH_BASE + 0x1000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_BASE = (AHB2PERIPH_BASE + 0x50000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_BASE = (AHB2PERIPH_BASE + 0x60000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_BASE = (AHB2PERIPH_BASE + 0x60400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RNG_BASE = (AHB2PERIPH_BASE + 0x60800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FSMC_Bank1_R_BASE = (FSMC_R_BASE + 0x0000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FSMC_Bank1E_R_BASE = (FSMC_R_BASE + 0x0104);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FSMC_Bank2_R_BASE = (FSMC_R_BASE + 0x0060);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FSMC_Bank3_R_BASE = (FSMC_R_BASE + 0x0080);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FSMC_Bank4_R_BASE = (FSMC_R_BASE + 0x00A0);


    }


} // END of Namespace...