﻿using System;
using System.Threading;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;

using GHI.OSHW.Hardware.LowLevel;

namespace Cerberus_PWM
{
    class Timer8_Ch1
    {
        #region Class Variables...

         private bool debug = false;

         private UInt32 Frequency = 0;

         private double DutyCycle = 0;

         private UInt32 Prescalar = 0;

         private bool InvertPin = false;

         #endregion

         /// <summary>
         /// Set the PWM with the desired Params...
         /// </summary>
         /// <param name="Frequency">Frequency of the PWM</param>
         /// <param name="DutyCycle">Double - 0.0 - 1 - Controls the Duty Cycle</param>
         /// <param name="InvertPin">Invert Chanel</param>
         /// <returns>bool true of false.</returns>
         public bool SetPWM(uint Frequency, double DutyCycle, bool InvertPin)
         {
             // Set Task Result...
             bool success = true;

             try
             {
                 // Set the Frequency...
                 this.Frequency = Frequency;

                 // Set the DutyCycle...
                 this.DutyCycle = DutyCycle;

                 // Set Pin Iinvert...
                 this.InvertPin = InvertPin;

                 // Set the Prescalar...
                 this.Prescalar = 1;

                 // Calculate the Frequency...
                 CalculateFrequency(STM32F4xx.TIM8_BASE);

                 // Calculate the Duty Cycle...
                 this.DutyCycle = (this.Frequency * this.DutyCycle);

                 // Set the Clocks...
                 ConfRCC();

                 // Configure GPIO Port...
                 ConfGPIO(STM32F4xx.GPIOC_BASE);

                 // Configure the Timer...
                 ConfTimer(STM32F4xx.TIM8_BASE);
             }
             catch
             {
                 success = false;
             }

             // Return Success...
             return success;
         }

         /// <summary>
         /// Not really necessary, much better way to do this can be achieved here...
         /// </summary>
         /// <param name="TIMx_BASE">The timer BASE to configure.</param>
         /// <returns>bool true of false.</returns>
         private bool CalculateFrequency(UInt32 TIMx_BASE)
         {
             UInt32 SYSTEM_APB1_CLOCK_HZ = 42000000;   // 42MHz
             UInt32 SYSTEM_APB2_CLOCK_HZ = 84000000;   // 84MHz

             if (TIMx_BASE == STM32F4xx.TIM1_BASE | TIMx_BASE == STM32F4xx.TIM8_BASE)
             {
                 this.Prescalar = (1);

                 this.Frequency = (SYSTEM_APB2_CLOCK_HZ / this.Frequency);
             }
             else
             {
                 //this.Frequency = (SYSTEM_APB1_CLOCK_HZ / this.Frequency);
             }

             // The prescaler can divide the counter clock frequency by any factor between 1 and 65536.

             return true;
         }

         /// <summary>
         /// Enable RCC, Reset and clock control.
         /// </summary>
         private void ConfRCC()
         {
             //////////////////////////////////////////////////////////////////////////////////////////////////////////
             // Reset and clock control for STM32F405xx/07xx and STM32F415xx/17xx(RCC)                               //
             //////////////////////////////////////////////////////////////////////////////////////////////////////////

             #region OFFSET's...

             UInt32 CR_OFFSET = 0x00;
             UInt32 PLLCFGR_OFFSET = 0x04;
             UInt32 CFGR_OFFSET = 0x08;
             UInt32 CIR_OFFSET = 0x0C;
             UInt32 AHB1RSTR_OFFSET = 0x10;
             UInt32 AHB2RSTR_OFFSET = 0x14;
             UInt32 AHB3RSTR_OFFSET = 0x18;
             UInt32 APB1RSTR_OFFSET = 0x20;
             UInt32 APB2RSTR_OFFSET = 0x24;
             UInt32 AHB1ENR_OFFSET = 0x30;
             UInt32 AHB2ENR_OFFSET = 0x34;
             UInt32 AHB3ENR_OFFSET = 0x38;
             UInt32 APB1ENR_OFFSET = 0x40;
             UInt32 APB2ENR_OFFSET = 0x44;
             UInt32 AHB1LPENR_OFFSET = 0x50;
             UInt32 AHB2LPENR_OFFSET = 0x54;
             UInt32 AHB3LPENR_OFFSET = 0x58;
             UInt32 APB1LPENR_OFFSET = 0x60;
             UInt32 APB2LPENR_OFFSET = 0x64;
             UInt32 BDCR_OFFSET = 0x70;
             UInt32 CSR_OFFSET = 0x74;
             UInt32 SSCGR_OFFSET = 0x80;
             UInt32 PLLI2SCFGR_OFFSET = 0x84;

             #endregion

             #region Registers...

             // RCC clock control register (RCC_CR)
             // Address offset: 0x00
             // Reset value: 0x0000 XX83 where X is undefined.
             // Access: no wait state, word, half-word and byte access
             Register CR = new Register(STM32F4xx.RCC_BASE + CR_OFFSET);

             // RCC PLL configuration register (RCC_PLLCFGR)
             // Address offset: 0x04
             // Reset value: 0x2400 3010
             // Access: no wait state, word, half-word and byte access.
             // This register is used to configure the PLL clock outputs according to the formulas:
             // • f(VCO clock) = f(PLL clock input) × (PLLN / PLLM)
             // • f(PLL general clock output) = f(VCO clock) / PLLP
             // • f(USB OTG FS, SDIO, RNG clock output) = f(VCO clock) / PLLQ
             Register PLLCFGR = new Register(STM32F4xx.RCC_BASE + PLLCFGR_OFFSET);

             // RCC clock configuration register (RCC_CFGR)
             // Address offset: 0x08
             // Reset value: 0x0000 0000
             // Access: 0 ≤ wait state ≤ 2, word, half-word and byte access
             // 1 or 2 wait states inserted only if the access occurs during a clock source switch.
             Register CFGR = new Register(STM32F4xx.RCC_BASE + CFGR_OFFSET);

             // RCC clock interrupt register (RCC_CIR)
             // Address offset: 0x0C
             // Reset value: 0x0000 0000
             // Access: no wait state, word, half-word and byte access
             Register CIR = new Register(STM32F4xx.RCC_BASE + CIR_OFFSET);

             // RCC AHB1 peripheral reset register (RCC_AHB1RSTR)
             // Address offset: 0x10
             // Reset value: 0x0000 0000
             // Access: no wait state, word, half-word and byte access.
             Register AHB1RSTR = new Register(STM32F4xx.RCC_BASE + AHB1RSTR_OFFSET);

             // RCC AHB2 peripheral reset register (RCC_AHB2RSTR)
             // Address offset: 0x14
             // Reset value: 0x0000 0000
             // Access: no wait state, word, half-word and byte access
             Register AHB2RSTR = new Register(STM32F4xx.RCC_BASE + AHB2RSTR_OFFSET);

             // RCC AHB3 peripheral reset register (RCC_AHB3RSTR)
             // Address offset: 0x18
             // Reset value: 0x0000 0000
             // Access: no wait state, word, half-word and byte access.
             Register AHB3RSTR = new Register(STM32F4xx.RCC_BASE + AHB3RSTR_OFFSET);

             // RCC APB1 peripheral reset register (RCC_APB1RSTR)
             // Address offset: 0x20
             // Reset value: 0x0000 0000
             // Access: no wait state, word, half-word and byte access.
             Register APB1RSTR = new Register(STM32F4xx.RCC_BASE + APB1RSTR_OFFSET);

             // RCC APB2 peripheral reset register (RCC_APB2RSTR)
             // Address offset: 0x24
             // Reset value: 0x0000 0000
             // Access: no wait state, word, half-word and byte access.
             Register APB2RSTR = new Register(STM32F4xx.RCC_BASE + APB2RSTR_OFFSET);

             // RCC AHB1 peripheral clock register (RCC_AHB1ENR)
             // Address offset: 0x30
             // Reset value: 0x0010 0000
             // Access: no wait state, word, half-word and byte access.
             Register AHB1ENR = new Register(STM32F4xx.RCC_BASE + AHB1ENR_OFFSET);

             // RCC AHB2 peripheral clock enable register (RCC_AHB2ENR)
             // Address offset: 0x34
             // Reset value: 0x0000 0000
             // Access: no wait state, word, half-word and byte access.
             Register AHB2ENR = new Register(STM32F4xx.RCC_BASE + AHB2ENR_OFFSET);

             // RCC AHB3 peripheral clock enable register (RCC_AHB3ENR)
             // Address offset: 0x38
             // Reset value: 0x0000 0000
             // Access: no wait state, word, half-word and byte access.
             Register AHB3ENR = new Register(STM32F4xx.RCC_BASE + AHB3ENR_OFFSET);

             // RCC APB1 peripheral clock enable register (RCC_APB1ENR)
             // Address offset: 0x40
             // Reset value: 0x0000 0000
             // Access: no wait state, word, half-word and byte access.
             // See: P.181
             Register APB1ENR = new Register(STM32F4xx.RCC_BASE + APB1ENR_OFFSET);

             // RCC APB2 peripheral clock enable register (RCC_APB2ENR)
             // Address offset: 0x44
             // Reset value: 0x0000 0000
             // Access: no wait state, word, half-word and byte access.
             // See: P.185
             Register APB2ENR = new Register(STM32F4xx.RCC_BASE + APB2ENR_OFFSET);

             // RCC AHB1 peripheral clock enable in low power mode register
             // (RCC_AHB1LPENR)
             // Address offset: 0x50
             // Reset value: 0x7E67 91FF
             // Access: no wait state, word, half-word and byte access.
             Register AHB1LPENR = new Register(STM32F4xx.RCC_BASE + AHB1LPENR_OFFSET);

             // RCC AHB2 peripheral clock enable in low power mode register
             // (RCC_AHB2LPENR)
             // Address offset: 0x54
             // Reset value: 0x0000 00F1
             // Access: no wait state, word, half-word and byte access.
             Register AHB2LPENR = new Register(STM32F4xx.RCC_BASE + AHB2LPENR_OFFSET);

             // RCC AHB3 peripheral clock enable in low power mode register
             // (RCC_AHB3LPENR)
             // Address offset: 0x58
             // Reset value: 0x0000 0001
             // Access: no wait state, word, half-word and byte access.
             Register AHB3LPENR = new Register(STM32F4xx.RCC_BASE + AHB3LPENR_OFFSET);

             // RCC APB1 peripheral clock enable in low power mode register
             // (RCC_APB1LPENR)
             // Address offset: 0x60
             // Reset value: 0x36FE C9FF
             // Access: no wait state, word, half-word and byte access.
             Register APB1LPENR = new Register(STM32F4xx.RCC_BASE + APB1LPENR_OFFSET);

             // RCC APB2 peripheral clock enabled in low power mode register
             // (RCC_APB2LPENR)
             // Address offset: 0x64
             // Reset value: 0x0007 5F33
             // Access: no wait state, word, half-word and byte access.
             Register APB2LPENR = new Register(STM32F4xx.RCC_BASE + APB2LPENR_OFFSET);

             // RCC Backup domain control register (RCC_BDCR)
             // Address offset: 0x70
             // Reset value: 0x0000 0000, reset by Backup domain reset.
             // Access: 0 ≤ wait state ≤ 3, word, half-word and byte access
             // Wait states are inserted in case of successive accesses to this register.
             Register BDCR = new Register(STM32F4xx.RCC_BASE + BDCR_OFFSET);

             // RCC clock control & status register (RCC_CSR)
             // Address offset: 0x74
             // Reset value: 0x0E00 0000, reset by system reset, except reset flags by power reset only.
             // Access: 0 ≤ wait state ≤ 3, word, half-word and byte access
             // Wait states are inserted in case of successive accesses to this register.
             Register CSR = new Register(STM32F4xx.RCC_BASE + CSR_OFFSET);

             // RCC spread spectrum clock generation register (RCC_SSCGR)
             // Address offset: 0x80
             // Reset value: 0x0000 0000
             // Access: no wait state, word, half-word and byte access.
             // The spread spectrum clock generation is available only for the main PLL.
             // The RCC_SSCGR register must be written either before the main PLL is enabled or after
             // the main PLL disabled.
             // Note: For full details about PLL spread spectrum clock generation (SSCG) characteristics, refer to
             // the “Electrical characteristics” section in your device datasheet.
             Register SSCGR = new Register(STM32F4xx.RCC_BASE + SSCGR_OFFSET);

             // RCC PLLI2S configuration register (RCC_PLLI2SCFGR)
             // Address offset: 0x84
             // Reset value: 0x2000 3000
             // Access: no wait state, word, half-word and byte access.
             // This register is used to configure the PLLI2S clock outputs according to the formulas:
             // f(VCO clock) = f(PLLI2S clock input) × (PLLI2SN / PLLM)
             // f(PLL I2S clock output) = f(VCO clock) / PLLI2SR
             Register PLLI2SCFGR = new Register(STM32F4xx.RCC_BASE + PLLI2SCFGR_OFFSET);

             #endregion

             //////////////////////////////////////////////////////////////////////////////////////////////////////////
             // Set the Clock with RCC... See P.178 and P.238 and P.507 - Advanced-control timers (TIM1&TIM8)        //
             //////////////////////////////////////////////////////////////////////////////////////////////////////////
             // Pulse lengths and waveform periods can be modulated from a few microseconds to several
             // milliseconds using the timer prescaler and the RCC clock controller prescalers.

             // This is not nessescary as these bits are already set from what I can see...
             AHB1ENR.SetBits(1 << 2);  //   <--- GPIOCEN BIT 2 Enable...
             APB2ENR.SetBits(1 << 1);  //   <--- TIM8EN BIT 1 Enable...

             #region Print Values...

             if (debug)
             {
                 // Print to Debug window current config...
                 // Read Value after the change...
                 Print("AHB1ENR", AHB1ENR); //   <--- Print change...
                 Print("APB2ENR", APB2ENR); //   <--- Print change...
             }

             #endregion
         }

         /// <summary>
         /// Configure GPIO Ports.
         /// Each of the 15 GPIOx Pins on each GPIO Port (GPIOA through I) is mapped to the MODER/AF/OSPEEDR/PUPDR and so on.
         /// This allows the control of each Pin, on each port to have Multiplexed Functions.
         /// Example: For GPIOC, MODER6 is mapped to Pin PA6, AF3 set on this Port in the AFRL (because the Pin is less than 7) sets TIM8 CH1.
         /// See: STM32F405VG User Manual P.278 and STM32F405VG P.62 for mappings.
         /// </summary>
         /// <param name="GPIOx_BASE">The Register BASE Address: 'STM32F4xx.GPIOC_BASE'</param>
         private void ConfGPIO(UInt32 GPIOx_BASE)
         {
             //////////////////////////////////////////////////////////////////////////////////////////////////////////
             // General-purpose I/O (GPIO) for STM32F405xx/07xx and STM32F415xx/17xx(RCC)                            //
             //////////////////////////////////////////////////////////////////////////////////////////////////////////

             #region OFFSET's...

             UInt32 MODER_OFFSET = 0x00;
             UInt32 OTYPER_OFFSET = 0x04;
             UInt32 OSPEEDR_OFFSET = 0x08;
             UInt32 PUPDR_OFFSET = 0x0C;
             UInt32 IDR_OFFSET = 0x10;
             UInt32 ODR_OFFSET = 0x14;
             UInt32 BSRR_OFFSET = 0x18;
             // UInt32 BSRRH_OFFSET = 0x1A; <--- ?
             UInt32 LCKR_OFFSET = 0x1C;

             #endregion

             #region Register's...

             // GPIO port mode register (GPIOx_MODER) (x = A..I/J/K)
             // Address offset: 0x00
             // Reset values:
             // • 0xA800 0000 for port A
             // • 0x0000 0280 for port B
             // • 0x0000 0000 for other ports
             Register MODER = new Register(GPIOx_BASE + MODER_OFFSET);

             // GPIO port output type register (GPIOx_OTYPER)
             // (x = A..I/J/K)
             // Address offset: 0x04
             // Reset value: 0x0000 0000
             Register OTYPER = new Register(GPIOx_BASE + OTYPER_OFFSET);

             // GPIO port output speed register (GPIOx_OSPEEDR)
             // (x = A..I/J/K)
             // Address offset: 0x08
             // Reset values:
             // • 0x0000 00C0 for port B
             // • 0x0000 0000 for other ports
             Register OSPEEDR = new Register(GPIOx_BASE + OSPEEDR_OFFSET);

             // GPIO port pull-up/pull-down register (GPIOx_PUPDR)
             // (x = A..I/J/K)
             // Address offset: 0x0C
             // Reset values:
             // • 0x6400 0000 for port A
             // • 0x0000 0100 for port B
             // • 0x0000 0000 for other ports
             Register PUPDR = new Register(GPIOx_BASE + PUPDR_OFFSET);

             // GPIO port input data register (GPIOx_IDR) (x = A..I/J/K)
             // Address offset: 0x10
             // Reset value: 0x0000 XXXX (where X means undefined)
             Register IDR = new Register(GPIOx_BASE + IDR_OFFSET);

             // GPIO port output data register (GPIOx_ODR) (x = A..I/J/K)
             // Address offset: 0x14
             // Reset value: 0x0000 0000
             Register ODR = new Register(GPIOx_BASE + ODR_OFFSET);

             // GPIO port bit set/reset register (GPIOx_BSRR) (x = A..I/J/K)
             // Address offset: 0x18
             // Reset value: 0x0000 0000
             Register BSRR = new Register(GPIOx_BASE + BSRR_OFFSET);

             // GPIO port configuration lock register (GPIOx_LCKR)
             // (x = A..I/J/K)
             // This register is used to lock the configuration of the port bits when a correct write sequence
             // is applied to bit 16 (LCKK). The value of bits [15:0] is used to lock the configuration of the
             // GPIO. During the write sequence, the value of LCKR[15:0] must not change. When the
             // LOCK sequence has been applied on a port bit, the value of this port bit can no longer be
             // modified until the next reset.
             // Note: A specific write sequence is used to write to the GPIOx_LCKR register. Only word access
             // (32-bit long) is allowed during this write sequence.
             // Each lock bit freezes a specific configuration register (control and alternate function
             // registers).
             // Address offset: 0x1C
             // Reset value: 0x0000 0000
             // Access: 32-bit word only, read/write register
             Register LCKR = new Register(GPIOx_BASE + LCKR_OFFSET);

             // Alternate Functions...
             // GPIO alternate function low register (GPIOx_AFRL) (x = A..I/J/K) - AFR[0]
             // Address offset: 0x20
             // Reset value: 0x0000 0000
             // GPIO alternate function high register (GPIOx_AFRH) - AFR[1]
             // (x = A..I/J/K)
             // Address offset: 0x24
             // Reset value: 0x0000 0000
             Register[] AFR = new Register[] { new Register(GPIOx_BASE + 0x20), new Register(GPIOx_BASE + 0x24) };

             #endregion

             //////////////////////////////////////////////////////////////////////////////////////////////////////////
             // See: P.283 - GPIO alternate function low register
             // AFRLy: Alternate function selection for port x bit y (y = 0..7)
             // These bits are written by software to configure alternate function I/Os
             // AFRLy selection:
             // 0000: AF0 (system)
             // 0001: AF1 (TIM1/TIM2)
             // 0010: AF2 (TIM3..5)
             // 0011: AF3 (TIM8..11)
             // 0100: AF4 (I2C1..3)
             // 0101: AF5 (SPI1/SPI2)
             // 0110: AF6 (SPI3)
             // 0111: AF7 (USART1..3)
             // 1000: AF8 (USART4..6)
             // 1001: AF9 (CAN1/CAN2, TIM12..14)
             // 1010: AF10 (OTG_FS, OTG_HS)
             // 1011: AF11 (ETH)
             // 1100: AF12 (FSMC, SDIO, OTG_HS(1))
             // 1101: AF13 (DCMI)
             // 1110: AF14
             // 1111: AF15 (EVENTOUT)
             // See: Figure 26. Selecting an alternate function
             // Also See: Table 9. Alternate function mapping of STM32F405VG.pdf
             if (AFR[0].Read() == 0)
             {
                 AFR[0].Write(0x3000000);
             }
             else
             {
                 AFR[0].SetBits(3 << 24); //   <--- Set Bits 24 and 25 to 11 - AFRL Register - Enables AF3 on AFRL6 this Maps to Pin GPIOC 6 (PC6) which is Pin 5 on X3...
             }

             //////////////////////////////////////////////////////////////////////////////////////////////////////////
             // Configure CPU Pins... See: P.271 - I/O port control registers...                                     //
             //////////////////////////////////////////////////////////////////////////////////////////////////////////
             // Each of the GPIOs has four 32-bit memory-mapped control registers (GPIOx_MODER,
             // GPIOx_OTYPER, GPIOx_OSPEEDR, GPIOx_PUPDR) to configure up to 16 I/Os. The
             // GPIOx_MODER register is used to select the I/O direction (input, output, AF, analog).
             // MODERy[1:0]: Port x configuration bits (y = 0..15)
             // These bits are written by software to configure the I/O direction mode.
             // 00: Input (reset state)
             // 01: General purpose output mode
             // 10: Alternate function mode
             // 11: Analog mode
             // MODER.ClearBits(1 << 12);
             MODER.SetBits(2 << 12); //   <--- Set Bits 12 and 13 to 10 - MODER6 this Maps to Pin GPIOC 6 (PC6) which is Pin 5 on X3...

             //////////////////////////////////////////////////////////////////////////////////////////////////////////
             // See: P.279 - GPIO port output type register
             // Bits 15:0 OTy: Port x configuration bits (y = 0..15)
             // These bits are written by software to configure the output type of the I/O port.
             // 0: Output push-pull (reset state)
             // 1: Output open-drain
             OTYPER.ClearBits(1 << 6); //   <--- Set Bit 6 to 0 - OT6 this Maps to Pin GPIOC 6 (PC6) which is Pin 5 on X3...

             //////////////////////////////////////////////////////////////////////////////////////////////////////////
             // See: P.279 - GPIO port output speed register
             // OSPEEDRy[1:0]: Port x configuration bits (y = 0..15)
             // These bits are written by software to configure the I/O output speed.
             // 00: Low speed
             // 01: Medium speed
             // 10: Fast speed
             // 11: High speed
             // Note: Refer to the
             OSPEEDR.SetBits(3 << 12); //   <--- Set Bits 12 and 13 to 11 - OSPEEDR6 this Maps to Pin GPIOC 6 (PC6) which is Pin 5 on X3...

             //////////////////////////////////////////////////////////////////////////////////////////////////////////
             // See: P.280 - GPIO port pull-up/pull-down register (GPIOx_PUPDR) 
             // (x = A..I/J/K) - Address offset: 0x0C
             // Reset values: 
             // • 0x6400 0000 for port A 
             // • 0x0000 0100 for port B 
             // • 0x0000 0000 for other ports
             // The GPIOx_PUPDR register is used to select the pull-up/pull-down whatever the I/O direction.
             PUPDR.ClearBits(3 << 12); //   <--- Set Bits 12 and 13 to 11 - PUPDR6 this Maps to Pin GPIOC 6 (PC6) which is Pin 5 on X3...

             #region Print Values...

             if (debug)
             {
                 // Print to Debug window current config...
                 Print("AFR[0]", AFR[0]); //   <--- Print change...
                 Print("MODER", MODER); //   <--- Print change...
                 Print("OTYPER", OTYPER); //   <--- Print change...
                 Print("OSPEEDR", OSPEEDR); //   <--- Print change...
                 Print("PUPDR", PUPDR); //   <--- Print change...
             }

             #endregion
         }

         /// <summary>
         /// Timer Configuration.
         /// </summary>
         /// <param name="TIMx_BASE">The Register BASE Address: 'STM32F4xx.TIM8_BASE'</param>
         private void ConfTimer(UInt32 TIMx_BASE)
         {
             //////////////////////////////////////////////////////////////////////////////////////////////////////////
             // General-purpose I/O (GPIO) for STM32F405xx/07xx and STM32F415xx/17xx(RCC)                            //
             //////////////////////////////////////////////////////////////////////////////////////////////////////////
             // Good Resources:
             // Table 42. DMA1 request mapping - See: P.304
             // Table 43. DMA2 request mapping - See: P.305
             // Advanced-control timers (TIM1&TIM8) - See: P.507

             #region OFFSET's...

             UInt32 CR1_OFFSET = 0x00;
             UInt32 CR2_OFFSET = 0x04;
             UInt32 SMCR_OFFSET = 0x08;
             UInt32 DIER_OFFSET = 0x0C;
             UInt32 SR_OFFSET = 0x10;
             UInt32 EGR_OFFSET = 0x14;
             UInt32 CCMR1_OFFSET = 0x18;
             UInt32 CCMR2_OFFSET = 0x1C;
             UInt32 CCER_OFFSET = 0x20;
             UInt32 CNT_OFFSET = 0x24;
             UInt32 PSC_OFFSET = 0x28;
             UInt32 ARR_OFFSET = 0x2C;
             UInt32 RCR_OFFSET = 0x30;
             UInt32 CCR1_OFFSET = 0x34;
             UInt32 CCR2_OFFSET = 0x38;
             UInt32 CCR3_OFFSET = 0x3C;
             UInt32 CCR4_OFFSET = 0x40;
             UInt32 BDTR_OFFSET = 0x44;
             UInt32 DCR_OFFSET = 0x48;
             UInt32 DMAR_OFFSET = 0x4C;
             UInt32 OR_OFFSET = 0x50;

             #endregion

             #region Registers...

             // TIM1&TIM8 control register 1 (TIMx_CR1)
             // Address offset: 0x00
             // Reset value: 0x0000
             Register CR1 = new Register(TIMx_BASE + CR1_OFFSET);

             // TIM1&TIM8 control register 2 (TIMx_CR2)
             // Address offset: 0x04
             // Reset value: 0x0000
             Register CR2 = new Register(TIMx_BASE + CR2_OFFSET);

             // TIM1&TIM8 slave mode control register (TIMx_SMCR)
             // Address offset: 0x08
             // Reset value: 0x0000
             Register SMCR = new Register(TIMx_BASE + SMCR_OFFSET);

             // TIM1&TIM8 DMA/interrupt enable register (TIMx_DIER)
             // Address offset: 0x0C
             // Reset value: 0x0000
             Register DIER = new Register(TIMx_BASE + DIER_OFFSET);

             // TIM1&TIM8 status register (TIMx_SR)
             // Address offset: 0x10
             // Reset value: 0x0000
             Register SR = new Register(TIMx_BASE + SR_OFFSET);

             // TIM1&TIM8 event generation register (TIMx_EGR)
             // Address offset: 0x14
             // Reset value: 0x0000
             Register EGR = new Register(TIMx_BASE + EGR_OFFSET);

             // TIM1&TIM8 capture/compare mode register 1 (TIMx_CCMR1)
             // Address offset: 0x18
             // Reset value: 0x0000
             Register CCMR1 = new Register(TIMx_BASE + CCMR1_OFFSET);

             // TIM1&TIM8 capture/compare mode register 2 (TIMx_CCMR2)
             // Address offset: 0x1C
             // Reset value: 0x0000
             // Refer to the above CCMR1 register description.
             Register CCMR2 = new Register(TIMx_BASE + CCMR2_OFFSET);

             // TIM1&TIM8 capture/compare enable register (TIMx_CCER)
             // Address offset: 0x20
             // Reset value: 0x0000
             Register CCER = new Register(TIMx_BASE + CCER_OFFSET);

             // TIM1&TIM8 counter (TIMx_CNT)
             // Address offset: 0x24
             // Reset value: 0x0000
             Register CNT = new Register(TIMx_BASE + CNT_OFFSET);

             // TIM1&TIM8 prescaler (TIMx_PSC)
             // Address offset: 0x28
             // Reset value: 0x0000
             Register PSC = new Register(TIMx_BASE + PSC_OFFSET);

             // TIM1&TIM8 auto-reload register (TIMx_ARR)
             // Address offset: 0x2C
             // Reset value: 0x0000
             Register ARR = new Register(TIMx_BASE + ARR_OFFSET);

             // TIM1&TIM8 repetition counter register (TIMx_RCR)
             // Address offset: 0x30
             // Reset value: 0x0000
             Register RCR = new Register(TIMx_BASE + RCR_OFFSET);

             // TIM1&TIM8 capture/compare register 1 (TIMx_CCR1)
             // Address offset: 0x34
             // Reset value: 0x0000
             Register CCR1 = new Register(TIMx_BASE + CCR1_OFFSET);

             // TIM1&TIM8 capture/compare register 2 (TIMx_CCR2)
             // Address offset: 0x38
             // Reset value: 0x0000
             Register CCR2 = new Register(TIMx_BASE + CCR2_OFFSET);

             // TIM1&TIM8 capture/compare register 3 (TIMx_CCR3)
             // Address offset: 0x3C
             // Reset value: 0x0000
             Register CCR3 = new Register(TIMx_BASE + CCR3_OFFSET);

             // TIM1&TIM8 capture/compare register 4 (TIMx_CCR4)
             // Address offset: 0x40
             // Reset value: 0x0000
             Register CCR4 = new Register(TIMx_BASE + CCR4_OFFSET);

             // TIM1&TIM8 break and dead-time register (TIMx_BDTR)
             // Address offset: 0x44
             // Reset value: 0x0000
             Register BDTR = new Register(TIMx_BASE + BDTR_OFFSET);

             // TIM1&TIM8 DMA control register (TIMx_DCR)
             // Address offset: 0x48
             // Reset value: 0x0000
             Register DCR = new Register(TIMx_BASE + DCR_OFFSET);

             // TIM1&TIM8 DMA address for full transfer (TIMx_DMAR)
             // Address offset: 0x4C
             // Reset value: 0x0000
             Register DMAR = new Register(TIMx_BASE + DMAR_OFFSET);

             #endregion

             #region Print Values...

             if (debug)
             {
                 // Print to Debug window current config...
                 Debug.Print("************************************************************************");
                 Print("CR1", CR1);
                 Print("CR2", CR2);
                 Print("SMCR", SMCR);
                 Print("DIER", DIER);
                 Print("SR", SR);
                 Print("EGR", EGR);
                 Print("CCMR1", CCMR1);
                 Print("CCMR2", CCMR2);
                 Print("CCER", CCER);
                 Print("CNT", CNT);
                 Print("PSC", PSC);
                 Print("ARR", ARR);
                 Print("RCR", RCR);
                 Print("CCR1", CCR1);
                 Print("CCR2", CCR2);
                 Print("CCR3", CCR3);
                 Print("CCR4", CCR4);
                 Print("BDTR", BDTR);
                 Print("DCR", DCR);
                 Print("DMAR", DMAR);
                 Debug.Print("************************************************************************");
             }

             #endregion

             ARR.Write(this.Frequency);

             PSC.Write(this.Prescalar);

             CCR1.Write((UInt16)this.DutyCycle);

             ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
             // See: P.560 - Bit 3 OC1PE: Output Compare 1 preload enable                                                              //
             ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
             // 0: Preload register on TIMx_CCR1 disabled. TIMx_CCR1 can be written at anytime, the
             // new value is taken in account immediately.
             // 1: Preload register on TIMx_CCR1 enabled. Read/Write operations access the preload
             // register. TIMx_CCR1 preload value is loaded in the active register at each update event.
             // Note: 1: These bits can not be modified as long as LOCK level 3 has been programmed
             // (LOCK bits in TIMx_BDTR register) and CC1S=’00’ (the channel is configured in output).
             // 2: The PWM mode can be used without validating the preload register only in one
             // pulse mode (OPM bit set in TIMx_CR1 register). Else the behavior is not guaranteed.
             CCMR1.ClearBits(1 << 3); //   <--- Bit 3 OC1PE: Output Compare 1 preload enable

             ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
             // See: P.560 - TIM1&TIM8 capture/compare mode register 1 (TIMx_CCMR1) - Bits 6:4 OC1M: Output Compare 1 mode             //
             ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
             // OC1M - Bits: 4,5 and 6. OC2M - Bits: 12, 13 and 14:
             // These bits define the behavior of the output reference signal OC1REF from which OC1 and
             // OC1N are derived. OC1REF is active high whereas OC1 and OC1N active level depends on CC1P and CC1NP bits:
             //
             //      000: Frozen - The comparison between the output compare register TIMx_CCR1 and the
             //           counter TIMx_CNT has no effect on the outputs.(this mode is used to generate a timing base).
             //
             //      001: Set channel 1 to active level on match. OC1REF signal is forced high when the counter
             //           TIMx_CNT matches the capture/compare register 1 (TIMx_CCR1).
             //
             //      010: Set channel 1 to inactive level on match. OC1REF signal is forced low when the
             //           counter TIMx_CNT matches the capture/compare register 1 (TIMx_CCR1).
             //
             //      011: Toggle - OC1REF toggles when TIMx_CNT=TIMx_CCR1.
             //      100: Force inactive level - OC1REF is forced low.
             //      101: Force active level - OC1REF is forced high.
             //      110: PWM mode 1 - In upcounting, channel 1 is active as long as TIMx_CNT<TIMx_CCR1
             //           else inactive. In downcounting, channel 1 is inactive (OC1REF=‘0’) as long as TIMx_CNT>TIMx_CCR1 else active (OC1REF=’1’).
             //
             //      111: PWM mode 2 - In upcounting, channel 1 is inactive as long as TIMx_CNT<TIMx_CCR1
             //           else active. In downcounting, channel 1 is active as long as TIMx_CNT>TIMx_CCR1 else inactive.
             // 
             // Note: 1: 
             //         These bits can not be modified as long as LOCK level 3 has been programmed
             //         (LOCK bits in TIMx_BDTR register) and CC1S=’00’ (the channel is configured in output).
             //      2: 
             //         In PWM mode 1 or 2, the OCREF level changes only when the result of the
             //         comparison changes or when the output compare mode switches from “frozen” mode to “PWM” mode.
             //      3: 
             //         On channels having a complementary output, this bit field is preloaded. If the CCPC
             //         bit is set in the TIMx_CR2 register then the OC1M active bits take the new value from
             //         the preloaded bits only when a COM event is generated.
             CCMR1.SetBits(7 << 5); //   <--- Bits 6:4 OC1M: Output Compare 1 mode
             CCMR1.SetBits(1 << 12); //   <--- Bits 6:4 OC2M: Output Compare 1 mode
             CCMR1.SetBits(3 << 14); //   <--- Bits 6:4 OC2M: Output Compare 1 mode



             ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
             // See: P.565 - capture/compare enable register (TIMx_CCER)                                                               //
             ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
             // Bit 1 CC1P: Capture/Compare 1 output polarity
             // CC1 channel configured as output:
             // 0: OC1 active high
             // 1: OC1 active low
             // CC1 channel configured as input:
             // CC1NP/CC1P bits select the active polarity of TI1FP1 and TI2FP1 for trigger or capture operations.
             // 00: non-inverted/rising edge
             // The circuit is sensitive to TIxFP1 rising edge (capture or trigger operations in reset, external
             // clock or trigger mode), TIxFP1 is not inverted (trigger operation in gated mode or encoder mode).
             // 01: inverted/falling edge
             // The circuit is sensitive to TIxFP1 falling edge (capture or trigger operations in reset, external
             // clock or trigger mode), TIxFP1 is inverted (trigger operation in gated mode or encoder mode).
             // 10: reserved, do not use this configuration.
             // 11: non-inverted/both edges
             if (InvertPin)
             {
                 CCER.SetBits(1 << 1);  //   <--- Bit 1 CC1P: Capture/Compare 1 output polarity
                 CCER.SetBits(1 << 3);  //   <--- Bit 1 CC1NP: Capture/Compare 1 output polarity
             }
             else
             {
                 CCER.ClearBits(1 << 1);  //   <--- Bit 1 CC1P: Capture/Compare 1 output polarity
                 CCER.ClearBits(1 << 3);  //   <--- Bit 1 CC1NP: Capture/Compare 1 output polarity
             }

             ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
             // Bits 1:0 CC1S: Capture/Compare 1 selection                                                                             //
             ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
             // This bit-field defines the direction of the channel (input/output) as well as the used input.
             //      00: CC1 channel is configured as output
             //      01: CC1 channel is configured as input, IC1 is mapped on TI1
             //      10: CC1 channel is configured as input, IC1 is mapped on TI2
             //      11: CC1 channel is configured as input, IC1 is mapped on TRC. This mode is working only if
             // an internal trigger input is selected through TS bit (TIMx_SMCR register)
             // Note: CC1S bits are writable only when the channel is OFF (CC1E = ‘0’ in TIMx_CCER).
             CCER.SetBits(1 << 0);  //   <--- Bit 0 CC1E: Capture/Compare 1 output enable

             ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
             // TIM1&TIM8 control register 1 (TIMx_CR1) - See: P.548                                                                   //
             ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
             //CR1.SetBits(1 << 2); //   <--- Bit 2 URS: Update request source
             //CR1.SetBits(1 << 7); //   <--- Bit 7 ARPE: Auto-reload preload enable
             BDTR.SetBits(1 << 15); //   <--- Bit 15 MOE: Main output enable
             CR1.SetBits(1 << 0); //   <--- Bit 0 CEN: Counter enable

             #region Print Values...

             if (debug)
             {
                 // Print to Debug window new config...
                 Print("CR1", CR1);
                 Print("CR2", CR2);
                 Print("SMCR", SMCR);
                 Print("DIER", DIER);
                 Print("SR", SR);
                 Print("EGR", EGR);
                 Print("CCMR1", CCMR1);
                 Print("CCMR2", CCMR2);
                 Print("CCER", CCER);
                 Print("CNT", CNT);
                 Print("PSC", PSC);
                 Print("ARR", ARR);
                 Print("RCR", RCR);
                 Print("CCR1", CCR1);
                 Print("CCR2", CCR2);
                 Print("CCR3", CCR3);
                 Print("CCR4", CCR4);
                 Print("BDTR", BDTR);
                 Print("DCR", DCR);
                 Print("DMAR", DMAR);
             }

             #endregion
         }

         /// <summary>
         /// Debug.Print the Register Name and the current Address...
         /// </summary>
         /// <param name="Name">String Register Display Name: 'CNT' </param>
         /// <param name="Address">The Register to be read: 'CNT'</param>
         private void Print(String Name, Register Register)
         {
             Debug.Print(Name + ": " + Register.Read());
         }
    }
}
