using System;
using Microsoft.SPOT;
using GHI.OSHW.Hardware.LowLevel;

namespace Cerberus_PWM
{
    /// <summary>
    /// enum IRQn
    /// </summary>
    public enum IRQn
    {
        /// <summary>
        /// 2 Non Maskable Interrupt
        /// </summary>
        NonMaskableInt_IRQn = -14,

        /// <summary>
        /// 4 Cortex-M4 Memory Management Interrupt 
        /// </summary>
        MemoryManagement_IRQn = -12,

        /// <summary>
        /// 5 Cortex-M4 Bus Fault Interrupt 
        /// </summary>
        BusFault_IRQn = -11,

        /// <summary>
        /// 6 Cortex-M4 Usage Fault Interrupt 
        /// </summary>
        UsageFault_IRQn = -10,

        /// <summary>
        /// 11 Cortex-M4 SV Call Interrupt
        /// </summary>
        SVCall_IRQn = -5,

        /// <summary>
        /// 12 Cortex-M4 Debug Monitor Interrupt
        /// </summary>
        DebugMonitor_IRQn = -4,

        /// <summary>
        /// 14 Cortex-M4 Pend SV Interrupt
        /// </summary>
        PendSV_IRQn = -2,

        /// <summary>
        /// 15 Cortex-M4 System Tick Interrupt
        /// </summary>
        SysTick_IRQn = -1,

        /// <summary>
        /// Window WatchDog Interrupt 
        /// </summary>
        WWDG_IRQn = 0,

        /// <summary>
        /// PVD through EXTI Line detection Interrupt 
        /// </summary>
        PVD_IRQn = 1,

        /// <summary>
        /// Tamper and TimeStamp interrupts through the EXTI line 
        /// </summary>
        TAMP_STAMP_IRQn = 2,

        /// <summary>
        /// RTC Wakeup interrupt through the EXTI line
        /// </summary>
        RTC_WKUP_IRQn = 3,

        /// <summary>
        /// FLASH global Interrupt
        /// </summary>
        FLASH_IRQn = 4,

        /// <summary>
        /// RCC global Interrupt
        /// </summary>
        RCC_IRQn = 5,

        /// <summary>
        /// EXTI Line0 Interrupt
        /// </summary>
        EXTI0_IRQn = 6,

        /// <summary>
        /// EXTI Line1 Interrupt
        /// </summary>
        EXTI1_IRQn = 7,

        /// <summary>
        /// EXTI Line2 Interrupt
        /// </summary>
        EXTI2_IRQn = 8,

        /// <summary>
        /// EXTI Line3 Interrupt
        /// </summary>
        EXTI3_IRQn = 9,

        /// <summary>
        /// EXTI Line4 Interrupt
        /// </summary>
        EXTI4_IRQn = 10,

        /// <summary>
        /// DMA1 Stream 0 global Interrupt
        /// </summary>
        DMA1_Stream0_IRQn = 11,

        /// <summary>
        /// DMA1 Stream 1 global Interrupt
        /// </summary>
        DMA1_Stream1_IRQn = 12,

        /// <summary>
        /// DMA1 Stream 2 global Interrupt
        /// </summary>
        DMA1_Stream2_IRQn = 13,

        /// <summary>
        /// DMA1 Stream 3 global Interrupt
        /// </summary>
        DMA1_Stream3_IRQn = 14,

        /// <summary>
        /// DMA1 Stream 4 global Interrupt
        /// </summary>
        DMA1_Stream4_IRQn = 15,

        /// <summary>
        /// DMA1 Stream 5 global Interrupt
        /// </summary>
        DMA1_Stream5_IRQn = 16,

        /// <summary>
        /// DMA1 Stream 6 global Interrupt
        /// </summary>
        DMA1_Stream6_IRQn = 17,

        /// <summary>
        /// ADC1, ADC2 and ADC3 global Interrupts 
        /// </summary>
        ADC_IRQn = 18,

        /// <summary>
        /// CAN1 TX Interrupt 
        /// </summary>
        CAN1_TX_IRQn = 19,

        /// <summary>
        /// CAN1 RX0 Interrupt
        /// </summary>
        CAN1_RX0_IRQn = 20,

        /// <summary>
        /// CAN1 RX1 Interrupt
        /// </summary>
        CAN1_RX1_IRQn = 21,

        /// <summary>
        /// CAN1 SCE Interrupt
        /// </summary>
        CAN1_SCE_IRQn = 22,

        /// <summary>
        /// External Line[9:5] Interrupts 
        /// </summary>
        EXTI9_5_IRQn = 23,

        /// <summary>
        /// TIM1 Break interrupt and TIM9 global interrupt
        /// </summary>
        TIM1_BRK_TIM9_IRQn = 24,

        /// <summary>
        /// TIM1 Update Interrupt and TIM10 global interrupt
        /// </summary>
        TIM1_UP_TIM10_IRQn = 25,

        /// <summary>
        /// TIM1 Trigger and Commutation Interrupt and TIM11 global interrupt 
        /// </summary>
        TIM1_TRG_COM_TIM11_IRQn = 26,

        /// <summary>
        /// TIM1 Capture Compare Interrupt
        /// </summary>
        TIM1_CC_IRQn = 27,

        /// <summary>
        /// TIM2 global Interrupt 
        /// </summary>
        TIM2_IRQn = 28,

        /// <summary>
        /// TIM3 global Interrupt 
        /// </summary>
        TIM3_IRQn = 29,

        /// <summary>
        /// TIM4 global Interrupt 
        /// </summary>
        TIM4_IRQn = 30,

        /// <summary>
        /// I2C1 Event Interrupt
        /// </summary>
        I2C1_EV_IRQn = 31,

        /// <summary>
        /// I2C1 Error Interrupt
        /// </summary>
        I2C1_ER_IRQn = 32,

        /// <summary>
        /// I2C2 Event Interrupt
        /// </summary>
        I2C2_EV_IRQn = 33,

        /// <summary>
        /// I2C2 Error Interrupt
        /// </summary>
        I2C2_ER_IRQn = 34,

        /// <summary>
        /// SPI1 global Interrupt 
        /// </summary>
        SPI1_IRQn = 35,

        /// <summary>
        /// SPI2 global Interrupt 
        /// </summary>
        SPI2_IRQn = 36,

        /// <summary>
        /// USART1 global Interrupt 
        /// </summary>
        USART1_IRQn = 37,

        /// <summary>
        /// USART2 global Interrupt 
        /// </summary>
        USART2_IRQn = 38,

        /// <summary>
        /// USART3 global Interrupt 
        /// </summary>
        USART3_IRQn = 39,

        /// <summary>
        /// External Line[15:10] Interrupts 
        /// </summary>
        EXTI15_10_IRQn = 40,

        /// <summary>
        /// RTC Alarm (A and B) through EXTI Line Interrupt 
        /// </summary>
        RTC_Alarm_IRQn = 41,

        /// <summary>
        /// USB OTG FS Wakeup through EXTI line interrupt 
        /// </summary>
        OTG_FS_WKUP_IRQn = 42,

        /// <summary>
        /// TIM8 Break Interrupt and TIM12 global interrupt 
        /// </summary>
        TIM8_BRK_TIM12_IRQn = 43,

        /// <summary>
        /// TIM8 Update Interrupt and TIM13 global interrupt
        /// </summary>
        TIM8_UP_TIM13_IRQn = 44,

        /// <summary>
        /// TIM8 Trigger and Commutation Interrupt and TIM14 global interrupt 
        /// </summary>
        TIM8_TRG_COM_TIM14_IRQn = 45,

        /// <summary>
        /// TIM8 Capture Compare Interrupt
        /// </summary>
        TIM8_CC_IRQn = 46,

        /// <summary>
        /// DMA1 Stream7 Interrupt
        /// </summary>
        DMA1_Stream7_IRQn = 47,

        /// <summary>
        /// FSMC global Interrupt 
        /// </summary>
        FSMC_IRQn = 48,

        /// <summary>
        /// SDIO global Interrupt 
        /// </summary>
        SDIO_IRQn = 49,

        /// <summary>
        /// TIM5 global Interrupt 
        /// </summary>
        TIM5_IRQn = 50,

        /// <summary>
        /// SPI3 global Interrupt 
        /// </summary>
        SPI3_IRQn = 51,

        /// <summary>
        /// UART4 global Interrupt
        /// </summary>
        UART4_IRQn = 52,

        /// <summary>
        /// UART5 global Interrupt
        /// </summary>
        UART5_IRQn = 53,

        /// <summary>
        /// TIM6 global and DAC1&2 underrun errorinterrupts 
        /// </summary>
        TIM6_DAC_IRQn = 54,

        /// <summary>
        /// TIM7 global interrupt 
        /// </summary>
        TIM7_IRQn = 55,

        /// <summary>
        /// DMA2 Stream 0 global Interrupt
        /// </summary>
        DMA2_Stream0_IRQn = 56,

        /// <summary>
        /// DMA2 Stream 1 global Interrupt
        /// </summary>
        DMA2_Stream1_IRQn = 57,

        /// <summary>
        /// DMA2 Stream 2 global Interrupt
        /// </summary>
        DMA2_Stream2_IRQn = 58,

        /// <summary>
        /// DMA2 Stream 3 global Interrupt
        /// </summary>
        DMA2_Stream3_IRQn = 59,

        /// <summary>
        /// DMA2 Stream 4 global Interrupt
        /// </summary>
        DMA2_Stream4_IRQn = 60,

        /// <summary>
        /// Ethernet global Interrupt 
        /// </summary>
        ETH_IRQn = 61,

        /// <summary>
        /// Ethernet Wakeup through EXTI line Interrupt 
        /// </summary>
        ETH_WKUP_IRQn = 62,

        /// <summary>
        /// CAN2 TX Interrupt 
        /// </summary>
        CAN2_TX_IRQn = 63,

        /// <summary>
        /// CAN2 RX0 Interrupt
        /// </summary>
        CAN2_RX0_IRQn = 64,

        /// <summary>
        /// CAN2 RX1 Interrupt
        /// </summary>
        CAN2_RX1_IRQn = 65,

        /// <summary>
        /// CAN2 SCE Interrupt
        /// </summary>
        CAN2_SCE_IRQn = 66,

        /// <summary>
        /// USB OTG FS global Interrupt 
        /// </summary>
        OTG_FS_IRQn = 67,

        /// <summary>
        /// DMA2 Stream 5 global interrupt
        /// </summary>
        DMA2_Stream5_IRQn = 68,

        /// <summary>
        /// DMA2 Stream 6 global interrupt
        /// </summary>
        DMA2_Stream6_IRQn = 69,

        /// <summary>
        /// DMA2 Stream 7 global interrupt
        /// </summary>
        DMA2_Stream7_IRQn = 70,

        /// <summary>
        /// USART6 global interrupt
        /// </summary>
        USART6_IRQn = 71,

        /// <summary>
        /// I2C3 event interrupt
        /// </summary>
        I2C3_EV_IRQn = 72,

        /// <summary>
        /// I2C3 error interrupt
        /// </summary>
        I2C3_ER_IRQn = 73,

        /// <summary>
        /// USB OTG HS End Point 1 Out global interrupt 
        /// </summary>
        OTG_HS_EP1_OUT_IRQn = 74,

        /// <summary>
        /// USB OTG HS End Point 1 In global interrupt
        /// </summary>
        OTG_HS_EP1_IN_IRQn = 75,

        /// <summary>
        /// USB OTG HS Wakeup through EXTI interrupt
        /// </summary>
        OTG_HS_WKUP_IRQn = 76,

        /// <summary>
        /// USB OTG HS global interrupt 
        /// </summary>
        OTG_HS_IRQn = 77,

        /// <summary>
        /// DCMI global interrupt 
        /// </summary>
        DCMI_IRQn = 78,

        /// <summary>
        /// CRYP crypto global interrupt
        /// </summary>
        CRYP_IRQn = 79,

        /// <summary>
        /// Hash and Rng global interrupt 
        /// </summary>
        HASH_RNG_IRQn = 80,

        /// <summary>
        /// FPU global interrupt
        /// </summary>
        FPU_IRQn = 81

    } // END of Enum...

    /// <summary>
    /// Advanced-control timers (TIM1&TIM8) - See: P.507
    /// also General-purpose timers (TIM2 to TIM5) - See: P.576
    /// also General-purpose timers (TIM9 to TIM14) - See: P.636 
    /// and also Basic timers (TIM6&TIM7) See: P.680
    /// </summary>
    public sealed class CPUTimer
    {
        // 17.3.20 Timer synchronization
        // The TIM timers are linked together internally for timer synchronization or chaining. Refer to
        // Section 18.3.15: Timer synchronization on page 605 for details.

        #region Private's...

        private uint _CR1 = 0x00;
        private uint _RESERVED0 = 0x02;
        private uint _CR2 = 0x04;
        private uint _RESERVED1 = 0x06;
        private uint _SMCR = 0x08;
        private uint _RESERVED2 = 0x0A;
        private uint _DIER = 0x0C;
        private uint _RESERVED3 = 0x0E;
        private uint _SR = 0x10;
        private uint _RESERVED4 = 0x12;
        private uint _EGR = 0x14;
        private uint _RESERVED5 = 0x16;
        private uint _CCMR1 = 0x18;
        private uint _RESERVED6 = 0x1A;
        private uint _CCMR2 = 0x1C;
        private uint _RESERVED7 = 0x1E;
        private uint _CCER = 0x20;
        private uint _RESERVED8 = 0x22;
        private uint _CNT = 0x24;
        private uint _PSC = 0x28;
        private uint _RESERVED9 = 0x2A;
        private uint _ARR = 0x2C;
        private uint _RCR = 0x30;
        private uint _RESERVED10 = 0x32;
        private uint _CCR1 = 0x34;
        private uint _CCR2 = 0x38;
        private uint _CCR3 = 0x3C;
        private uint _CCR4 = 0x40;
        private uint _BDTR = 0x44;
        private uint _RESERVED11 = 0x46;
        private uint _DCR = 0x48;
        private uint _RESERVED12 = 0x4A;
        private uint _DMAR = 0x4C;
        private uint _RESERVED13 = 0x4E;
        private uint _OR = 0x50;
        private uint _RESERVED14 = 0x52;

        #endregion

        #region Properties...

        /// <summary>
        /// TIM control register 1, Address offset: 0x00
        /// </summary>
        public uint CR1 { get { return _CR1; } set { _CR1 = value; } }

        /// <summary>
        /// Reserved, 0x02
        /// </summary>
        public uint RESERVED0 { get { return _RESERVED0; } }

        /// <summary>
        /// TIM control register 2, Address offset: 0x04 
        /// </summary>
        public uint CR2 { get { return _CR2; } set { _CR2 = value; } }

        /// <summary>
        /// Reserved, 0x06 
        /// </summary>
        public uint RESERVED1 { get { return _RESERVED1; } }

        /// <summary>
        /// TIM slave mode control register, Address offset: 0x08 
        /// </summary>
        public uint SMCR { get { return _SMCR; } set { _SMCR = value; } }

        /// <summary>
        /// Reserved, 0x0A 
        /// </summary>
        public uint RESERVED2 { get { return _RESERVED2; } }

        /// <summary>
        /// TIM DMA/interrupt enable register, Address offset: 0x0C
        /// </summary>
        public uint DIER { get { return _DIER; } set { _DIER = value; } }

        /// <summary>
        /// Reserved, 0x0E
        /// </summary>
        public uint RESERVED3 { get { return _RESERVED3; } }

        /// <summary>
        /// TIM status register, Address offset: 0x10
        /// </summary>
        public uint SR { get { return _SR; } set { _SR = value; } }

        /// <summary>
        /// Reserved, 0x12 
        /// </summary>
        public uint RESERVED4 { get { return _RESERVED4; } }

        /// <summary>
        /// TIM event generation register, Address offset: 0x14 
        /// </summary>
        public uint EGR { get { return _EGR; } set { _EGR = value; } }

        /// <summary>
        /// Reserved, 0x16
        /// </summary>
        public uint RESERVED5 { get { return _RESERVED5; } }

        /// <summary>
        /// TIM capture/compare mode register 1, Address offset: 0x18 
        /// </summary>
        public uint CCMR1 { get { return _CCMR1; } set { _CCMR1 = value; } }

        /// <summary>
        /// Reserved, 0x1A
        /// </summary>
        public uint RESERVED6 { get { return _RESERVED6; } }

        /// <summary>
        /// TIM capture/compare mode register 2, Address offset: 0x1C 
        /// </summary>
        public uint CCMR2 { get { return _CCMR2; } set { _CCMR2 = value; } }

        /// <summary>
        /// Reserved, 0x1E
        /// </summary>
        public uint RESERVED7 { get { return _RESERVED7; } }

        /// <summary>
        /// TIM capture/compare enable register, Address offset: 0x20 
        /// </summary>
        public uint CCER { get { return _CCER; } set { _CCER = value; } }

        /// <summary>
        /// Reserved, 0x22 
        /// </summary>
        public uint RESERVED8 { get { return _RESERVED8; } }

        /// <summary>
        /// TIM counter register, Address offset: 0x24 
        /// </summary>
        public uint CNT { get { return _CNT; } set { _CNT = value; } }

        /// <summary>
        /// TIM prescaler, Address offset: 0x28 
        /// </summary>
        public uint PSC { get { return _PSC; } set { _PSC = value; } }

        /// <summary>
        /// Reserved, 0x2A 
        /// </summary>
        public uint RESERVED9 { get { return _RESERVED9; } }

        /// <summary>
        /// TIM auto-reload register, Address offset: 0x2C 
        /// </summary>
        public uint ARR { get { return _ARR; } set { _ARR = value; } }

        /// <summary>
        /// TIM repetition counter register, Address offset: 0x30
        /// </summary>
        public uint RCR { get { return _RCR; } set { _RCR = value; } }

        /// <summary>
        /// Reserved, 0x32
        /// </summary>
        public uint RESERVED10 { get { return _RESERVED10; } }

        /// <summary>
        /// TIM capture/compare register 1, Address offset: 0x34 
        /// </summary>
        public uint CCR1 { get { return _CCR1; } set { _CCR1 = value; } }

        /// <summary>
        /// TIM capture/compare register 2, Address offset: 0x38
        /// </summary>
        public uint CCR2 { get { return _CCR2; } set { _CCR2 = value; } }

        /// <summary>
        /// TIM capture/compare register 3, Address offset: 0x3C
        /// </summary>
        public uint CCR3 { get { return _CCR3; } set { _CCR3 = value; } }

        /// <summary>
        /// TIM capture/compare register 4, Address offset: 0x40 
        /// </summary>
        public uint CCR4 { get { return _CCR4; } set { _CCR4 = value; } }

        /// <summary>
        /// TIM break and dead-time register, Address offset: 0x44
        /// </summary>
        public uint BDTR { get { return _BDTR; } set { _BDTR = value; } }

        /// <summary>
        /// Reserved, 0x46 
        /// </summary>
        public uint RESERVED11 { get { return _RESERVED11; } }

        /// <summary>
        /// TIM DMA control register, Address offset: 0x48
        /// </summary>
        public uint DCR { get { return _DCR; } set { _DCR = value; } }

        /// <summary>
        /// Reserved, 0x4A
        /// </summary>
        public uint RESERVED12 { get { return _RESERVED12; } }

        /// <summary>
        /// TIM DMA address for full transfer, Address offset: 0x4C
        /// </summary>
        public uint DMAR { get { return _DMAR; } set { _DMAR = value; } }

        /// <summary>
        /// Reserved, 0x4E 
        /// </summary>
        public uint RESERVED13 { get { return _RESERVED13; } }

        /// <summary>
        /// TIM option register, Address offset: 0x50
        /// </summary>
        public uint OR { get { return _OR; } set { _OR = value; } }

        /// <summary>
        /// Reserved, 0x52
        /// </summary>
        public uint RESERVED14 { get { return _RESERVED14; } }

        #endregion

        #region Timer Registers...

        ////////////////////////////////////////////////////////////////////////////////
        // See P.574 on the TIM1 and TIM8 Register table...                           //
        ////////////////////////////////////////////////////////////////////////////////
        /// <summary>
        /// control register 1 (TIMx_CR1) - Address offset: 0x00 - Reset value: 0x0000
        /// </summary>
        public Register CR1_Register;

        /// <summary>
        /// control register 2 (TIMx_CR2) - Address offset: 0x04 - Reset value: 0x0000
        /// </summary>
        public Register CR2_Register;

        /// <summary>
        /// slave mode control register (TIMx_SMCR) - Address offset: 0x08 - Reset value: 0x0000
        /// </summary>
        public Register SMCR_Register;

        /// <summary>
        /// DMA/interrupt enable register (TIMx_DIER) - Address offset: 0x0C - Reset value: 0x0000
        /// </summary>
        public Register DIER_Register;

        /// <summary>
        /// status register (TIMx_SR) - Address offset: 0x10 - Reset value: 0x0000
        /// </summary>
        public Register SR_Register;

        /// <summary>
        /// event generation register (TIMx_EGR) - Address offset: 0x14 - Reset value: 0x0000
        /// </summary>
        public Register EGR_Register;

        /// <summary>
        /// capture/compare mode register 1 (TIMx_CCMR1) - Address offset: 0x18 - Reset value: 0x0000
        /// </summary>
        public Register CCMR1_Register;

        /// <summary>
        /// capture/compare mode register 2 (TIMx_CCMR2) - Address offset: 0x1C - Reset value: 0x0000
        /// </summary>
        public Register CCMR2_Register;

        /// <summary>
        /// capture/compare enable register (TIMx_CCER) - Address offset: 0x20 - Reset value: 0x0000
        /// </summary>
        public Register CCER_Register;

        /// <summary>
        /// counter (TIMx_CNT) - Address offset: 0x24 - Reset value: 0x0000
        /// </summary>
        public Register CNT_Register;

        /// <summary>
        /// prescaler (TIMx_PSC) - Address offset: 0x28 - Reset value: 0x0000
        /// </summary>
        public Register PSC_Register;

        /// <summary>
        /// auto-reload register (TIMx_ARR) - Address offset: 0x2C - Reset value: 0x0000
        /// </summary>
        public Register ARR_Register;

        /// <summary>
        /// repetition counter register (TIMx_RCR) - Address offset: 0x30 - Reset value: 0x0000
        /// </summary>
        public Register RCR_Register;

        /// <summary>
        /// Capture/compare register 1 (TIMx_CCR1) - Address offset: 0x34 - Reset value: 0x0000
        /// </summary>
        public Register CCR1_Register;

        /// <summary>
        /// Capture/compare register 2 (TIMx_CCR2) - Address offset: 0x38 - Reset value: 0x0000
        /// </summary>
        public Register CCR2_Register;

        /// <summary>
        /// Capture/compare register 3 (TIMx_CCR3) - Address offset: 0x3C - Reset value: 0x0000
        /// </summary>
        public Register CCR3_Register;

        /// <summary>
        /// Capture/compare register 4 (TIMx_CCR4) - Address offset: 0x40 - Reset value: 0x0000
        /// </summary>
        public Register CCR4_Register;

        /// <summary>
        /// Break and dead-time register (TIMx_BDTR) - Address offset: 0x44 - Reset value: 0x0000
        /// </summary>
        public Register BDTR_Register;

        /// <summary>
        /// DMA control register (TIMx_DCR) - Address offset: 0x48 - Reset value: 0x0000
        /// </summary>
        public Register DCR_Register;

        /// <summary>
        /// DMA address for full transfer (TIMx_DMAR) - Address offset: 0x4C - Reset value: 0x0000
        /// </summary>
        public Register DMAR_Register;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////
        // System Timer Configuration                                                         //
        // STM32F4_32B_TIMER = 5                                                              //
        // STM32F4_16B_TIMER = 12                                                             //
        ////////////////////////////////////////////////////////////////////////////////////////
        // public static CPUTimer TIMER1 = new CPUTimer(STM32F4xx.TIMER1_BASE);
        // public static CPUTimer TIMER2 = new CPUTimer(STM32F4xx.TIMER2_BASE);
        // public static CPUTimer TIMER3 = new CPUTimer(STM32F4xx.TIMER3_BASE);
        // public static CPUTimer TIMER4 = new CPUTimer(STM32F4xx.TIMER4_BASE);
        // public static CPUTimer TIMER5 = new CPUTimer(STM32F4xx.TIMER5_BASE);
        // public static CPUTimer TIMER6 = new CPUTimer(STM32F4xx.TIMER6_BASE);
        // public static CPUTimer TIMER7 = new CPUTimer(STM32F4xx.TIMER7_BASE);

        /// <summary>
        /// Provids Access to the Timer 8 Configuration Registers...
        /// See: P.548 for TIM1&TIM8 registers...
        /// </summary>
        public static CPUTimer TIMER8 = new CPUTimer(STM32F4xx.TIM8_BASE);

        // public static CPUTimer TIMER9 = new CPUTimer(STM32F4xx.TIMER9_BASE);
        // public static CPUTimer TIMER10 = new CPUTimer(STM32F4xx.TIMER10_BASE);
        // public static CPUTimer TIMER11 = new CPUTimer(STM32F4xx.TIMER11_BASE);
        // public static Timer TIMER12 = new Timer(STM32F4xx.TIMER12_BASE);
        // public static CPUTimer TIMER13 = new CPUTimer(STM32F4xx.TIMER13_BASE);
        // public static CPUTimer TIMER14 = new CPUTimer(STM32F4xx.TIMER14_BASE);

        public CPUTimer(UInt32 TIMx_BASE)
        {
            // See: P.611 for TIM2 to TIM5 registers...

            CR1_Register = new Register(TIMx_BASE + CR1);
            CR2_Register = new Register(TIMx_BASE + CR2);
            SMCR_Register = new Register(TIMx_BASE + SMCR);
            DIER_Register = new Register(TIMx_BASE + DIER);
            SR_Register = new Register(TIMx_BASE + SR);
            EGR_Register = new Register(TIMx_BASE + EGR);
            CCMR1_Register = new Register(TIMx_BASE + CCMR1);
            CCMR2_Register = new Register(TIMx_BASE + CCMR2);
            CCER_Register = new Register(TIMx_BASE + CCER);
            CNT_Register = new Register(TIMx_BASE + CNT);
            PSC_Register = new Register(TIMx_BASE + PSC);
            ARR_Register = new Register(TIMx_BASE + ARR);
            RCR_Register = new Register(TIMx_BASE + RCR);
            CCR1_Register = new Register(TIMx_BASE + CCR1);
            CCR2_Register = new Register(TIMx_BASE + CCR2);
            CCR3_Register = new Register(TIMx_BASE + CCR3);
            CCR4_Register = new Register(TIMx_BASE + CCR4);
            BDTR_Register = new Register(TIMx_BASE + BDTR);
            DCR_Register = new Register(TIMx_BASE + DCR);
            DMAR_Register = new Register(TIMx_BASE + DMAR);
        }

    } // END of Class...

    /// <summary>
    /// Real-time clock (RTC) - See: P.780
    /// </summary>
    public static class RTC
    {
        #region Private's...

        /// <summary>
        /// RTC time register,Address offset: 0x00 
        /// </summary>
        private static uint _TR = 0x00;

        /// <summary>
        /// RTC date register,Address offset: 0x04 
        /// </summary>
        private static uint _DR = 0x04;

        /// <summary>
        /// RTC control register, Address offset: 0x08 
        /// </summary>
        private static uint _CR = 0x08;

        /// <summary>
        /// RTC initialization and status register, Address offset: 0x0C 
        /// </summary>
        private static uint _ISR = 0x0C;

        /// <summary>
        /// RTC prescaler register, Address offset: 0x10 
        /// </summary>
        private static uint _PRER = 0x10;

        /// <summary>
        /// RTC wakeup timer register,Address offset: 0x14 
        /// </summary>
        private static uint _WUTR = 0x14;

        /// <summary>
        /// RTC calibration register, Address offset: 0x18 
        /// </summary>
        private static uint _CALIBR = 0x18;

        /// <summary>
        /// RTC alarm A register, Address offset: 0x1C 
        /// </summary>
        private static uint _ALRMAR = 0x1C;

        /// <summary>
        /// RTC alarm B register, Address offset: 0x20 
        /// </summary>
        private static uint _ALRMBR = 0x20;

        /// <summary>
        /// RTC write protection register,Address offset: 0x24 
        /// </summary>
        private static uint _WPR = 0x24;

        /// <summary>
        /// RTC sub second register,Address offset: 0x28 
        /// </summary>
        private static uint _SSR = 0x28;

        /// <summary>
        /// RTC shift control register, Address offset: 0x2C 
        /// </summary>
        private static uint _SHIFTR = 0x2C;

        /// <summary>
        /// RTC time stamp time register, Address offset: 0x30 
        /// </summary>
        private static uint _TSTR = 0x30;

        /// <summary>
        /// RTC time stamp date register, Address offset: 0x34 
        /// </summary>
        private static uint _TSDR = 0x34;

        /// <summary>
        /// RTC time-stamp sub second register, Address offset: 0x38 
        /// </summary>
        private static uint _TSSSR = 0x38;

        /// <summary>
        /// RTC calibration register, Address offset: 0x3C 
        /// </summary>
        private static uint _CALR = 0x3C;

        /// <summary>
        /// RTC tamper and alternate function configuration register, Address offset: 0x40 
        /// </summary>
        private static uint _TAFCR = 0x40;

        /// <summary>
        ///  RTC alarm A sub second register,Address offset: 0x44 
        /// </summary>
        private static uint _ALRMASSR = 0x44;

        /// <summary>
        ///  RTC alarm B sub second register,Address offset: 0x48 
        /// </summary>
        private static uint _ALRMBSSR = 0x48;

        /// <summary>
        /// RTC backup register 1,Address offset: 0x50 
        /// </summary>
        private static uint _BKP0R = 0x50;

        /// <summary>
        /// RTC backup register 1,Address offset: 0x54 
        /// </summary>
        private static uint _BKP1R = 0x54;

        /// <summary>
        /// RTC backup register 2,Address offset: 0x58 
        /// </summary>
        private static uint _BKP2R = 0x58;

        /// <summary>
        /// RTC backup register 3,Address offset: 0x5C 
        /// </summary>
        private static uint _BKP3R = 0x5C;

        /// <summary>
        /// RTC backup register 4,Address offset: 0x60 
        /// </summary>
        private static uint _BKP4R = 0x60;

        /// <summary>
        /// RTC backup register 5,Address offset: 0x64 
        /// </summary>
        private static uint _BKP5R = 0x64;

        /// <summary>
        /// RTC backup register 6,Address offset: 0x68 
        /// </summary>
        private static uint _BKP6R = 0x68;

        /// <summary>
        /// RTC backup register 7,Address offset: 0x6C 
        /// </summary>
        private static uint _BKP7R = 0x6C;

        /// <summary>
        /// RTC backup register 8,Address offset: 0x70 
        /// </summary>
        private static uint _BKP8R = 0x70;

        /// <summary>
        /// RTC backup register 9,Address offset: 0x74 
        /// </summary>
        private static uint _BKP9R = 0x74;

        /// <summary>
        /// RTC backup register 10, Address offset: 0x78 
        /// </summary>
        private static uint _BKP10R = 0x78;

        /// <summary>
        /// RTC backup register 11, Address offset: 0x7C 
        /// </summary>
        private static uint _BKP11R = 0x7C;

        /// <summary>
        /// RTC backup register 12, Address offset: 0x80 
        /// </summary>
        private static uint _BKP12R = 0x80;

        /// <summary>
        /// RTC backup register 13, Address offset: 0x84 
        /// </summary>
        private static uint _BKP13R = 0x84;

        /// <summary>
        /// RTC backup register 14, Address offset: 0x88 
        /// </summary>
        private static uint _BKP14R = 0x88;

        /// <summary>
        /// RTC backup register 15, Address offset: 0x8C 
        /// </summary>
        private static uint _BKP15R = 0x8C;

        /// <summary>
        /// RTC backup register 16, Address offset: 0x90 
        /// </summary>
        private static uint _BKP16R = 0x90;

        /// <summary>
        /// RTC backup register 17, Address offset: 0x94 
        /// </summary>
        private static uint _BKP17R = 0x94;

        /// <summary>
        /// RTC backup register 18, Address offset: 0x98 
        /// </summary>
        private static uint _BKP18R = 0x98;

        /// <summary>
        /// RTC backup register 19, Address offset: 0x9C 
        /// </summary>
        private static uint _BKP19R = 0x9C;

        #endregion

        #region Properties...

        /// <summary>
        /// RTC time register,Address offset: 0x00 
        /// </summary>
        public static uint TR { get { return _TR; } set { _TR = value; } }

        /// <summary>
        /// RTC date register,Address offset: 0x04 
        /// </summary>
        public static uint DR { get { return _DR; } set { _DR = value; } }

        /// <summary>
        /// RTC control register, Address offset: 0x08 
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// RTC initialization and status register, Address offset: 0x0C 
        /// </summary>
        public static uint ISR { get { return _ISR; } set { _ISR = value; } }

        /// <summary>
        /// RTC prescaler register, Address offset: 0x10 
        /// </summary>
        public static uint PRER { get { return _PRER; } set { _PRER = value; } }

        /// <summary>
        /// RTC wakeup timer register,Address offset: 0x14 
        /// </summary>
        public static uint WUTR { get { return _WUTR; } set { _WUTR = value; } }

        /// <summary>
        /// RTC calibration register, Address offset: 0x18 
        /// </summary>
        public static uint CALIBR { get { return _CALIBR; } set { _CALIBR = value; } }

        /// <summary>
        /// RTC alarm A register, Address offset: 0x1C 
        /// </summary>
        public static uint ALRMAR { get { return _ALRMAR; } set { _ALRMAR = value; } }

        /// <summary>
        /// RTC alarm B register, Address offset: 0x20 
        /// </summary>
        public static uint ALRMBR { get { return _ALRMBR; } set { _ALRMBR = value; } }

        /// <summary>
        /// RTC write protection register,Address offset: 0x24 
        /// </summary>
        public static uint WPR { get { return _WPR; } set { _WPR = value; } }

        /// <summary>
        /// RTC sub second register,Address offset: 0x28 
        /// </summary>
        public static uint SSR { get { return _SSR; } set { _SSR = value; } }

        /// <summary>
        /// RTC shift control register, Address offset: 0x2C 
        /// </summary>
        public static uint SHIFTR { get { return _SHIFTR; } set { _SHIFTR = value; } }

        /// <summary>
        /// RTC time stamp time register, Address offset: 0x30 
        /// </summary>
        public static uint TSTR { get { return _TSTR; } set { _TSTR = value; } }

        /// <summary>
        /// RTC time stamp date register, Address offset: 0x34 
        /// </summary>
        public static uint TSDR { get { return _TSDR; } set { _TSDR = value; } }

        /// <summary>
        /// RTC time-stamp sub second register, Address offset: 0x38 
        /// </summary>
        public static uint TSSSR { get { return _TSSSR; } set { _TSSSR = value; } }

        /// <summary>
        /// RTC calibration register, Address offset: 0x3C 
        /// </summary>
        public static uint CALR { get { return _CALR; } set { _CALR = value; } }

        /// <summary>
        /// RTC tamper and alternate function configuration register, Address offset: 0x40 
        /// </summary>
        public static uint TAFCR { get { return _TAFCR; } set { _TAFCR = value; } }

        /// <summary>
        ///  RTC alarm A sub second register,Address offset: 0x44 
        /// </summary>
        public static uint ALRMASSR { get { return _ALRMASSR; } set { _ALRMASSR = value; } }

        /// <summary>
        ///  RTC alarm B sub second register,Address offset: 0x48 
        /// </summary>
        public static uint ALRMBSSR { get { return _ALRMBSSR; } set { _ALRMBSSR = value; } }

        /// <summary>
        /// RTC backup register 1,Address offset: 0x50 
        /// </summary>
        public static uint BKP0R { get { return _BKP0R; } set { _BKP0R = value; } }

        /// <summary>
        /// RTC backup register 1,Address offset: 0x54 
        /// </summary>
        public static uint BKP1R { get { return _BKP1R; } set { _BKP1R = value; } }

        /// <summary>
        /// RTC backup register 2,Address offset: 0x58 
        /// </summary>
        public static uint BKP2R { get { return _BKP2R; } set { _BKP2R = value; } }

        /// <summary>
        /// RTC backup register 3,Address offset: 0x5C 
        /// </summary>
        public static uint BKP3R { get { return _BKP3R; } set { _BKP3R = value; } }

        /// <summary>
        /// RTC backup register 4,Address offset: 0x60 
        /// </summary>
        public static uint BKP4R { get { return _BKP4R; } set { _BKP4R = value; } }

        /// <summary>
        /// RTC backup register 5,Address offset: 0x64 
        /// </summary>
        public static uint BKP5R { get { return _BKP5R; } set { _BKP5R = value; } }

        /// <summary>
        /// RTC backup register 6,Address offset: 0x68 
        /// </summary>
        public static uint BKP6R { get { return _BKP6R; } set { _BKP6R = value; } }

        /// <summary>
        /// RTC backup register 7,Address offset: 0x6C 
        /// </summary>
        public static uint BKP7R { get { return _BKP7R; } set { _BKP7R = value; } }

        /// <summary>
        /// RTC backup register 8,Address offset: 0x70 
        /// </summary>
        public static uint BKP8R { get { return _BKP8R; } set { _BKP8R = value; } }

        /// <summary>
        /// RTC backup register 9,Address offset: 0x74 
        /// </summary>
        public static uint BKP9R { get { return _BKP9R; } set { _BKP9R = value; } }

        /// <summary>
        /// RTC backup register 10, Address offset: 0x78 
        /// </summary>
        public static uint BKP10R { get { return _BKP10R; } set { _BKP10R = value; } }

        /// <summary>
        /// RTC backup register 11, Address offset: 0x7C 
        /// </summary>
        public static uint BKP11R { get { return _BKP11R; } set { _BKP11R = value; } }

        /// <summary>
        /// RTC backup register 12, Address offset: 0x80 
        /// </summary>
        public static uint BKP12R { get { return _BKP12R; } set { _BKP12R = value; } }

        /// <summary>
        /// RTC backup register 13, Address offset: 0x84 
        /// </summary>
        public static uint BKP13R { get { return _BKP13R; } set { _BKP13R = value; } }

        /// <summary>
        /// RTC backup register 14, Address offset: 0x88 
        /// </summary>
        public static uint BKP14R { get { return _BKP14R; } set { _BKP14R = value; } }

        /// <summary>
        /// RTC backup register 15, Address offset: 0x8C 
        /// </summary>
        public static uint BKP15R { get { return _BKP15R; } set { _BKP15R = value; } }

        /// <summary>
        /// RTC backup register 16, Address offset: 0x90 
        /// </summary>
        public static uint BKP16R { get { return _BKP16R; } set { _BKP16R = value; } }

        /// <summary>
        /// RTC backup register 17, Address offset: 0x94 
        /// </summary>
        public static uint BKP17R { get { return _BKP17R; } set { _BKP17R = value; } }

        /// <summary>
        /// RTC backup register 18, Address offset: 0x98 
        /// </summary>
        public static uint BKP18R { get { return _BKP18R; } set { _BKP18R = value; } }

        /// <summary>
        /// RTC backup register 19, Address offset: 0x9C 
        /// </summary>
        public static uint BKP19R { get { return _BKP19R; } set { _BKP19R = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// RTC time register,Address offset: 0x00 
        /// </summary>
        public static Register TR_Register;

        /// <summary>
        /// RTC date register,Address offset: 0x04 
        /// </summary>
        public static Register DR_Register;

        /// <summary>
        /// RTC control register, Address offset: 0x08 
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// RTC initialization and status register, Address offset: 0x0C 
        /// </summary>
        public static Register ISR_Register;

        /// <summary>
        /// RTC prescaler register, Address offset: 0x10 
        /// </summary>
        public static Register PRER_Register;

        /// <summary>
        /// RTC wakeup timer register,Address offset: 0x14 
        /// </summary>
        public static Register WUTR_Register;

        /// <summary>
        /// RTC calibration register, Address offset: 0x18 
        /// </summary>
        public static Register CALIBR_Register;

        /// <summary>
        /// RTC alarm A register, Address offset: 0x1C 
        /// </summary>
        public static Register ALRMAR_Register;

        /// <summary>
        /// RTC alarm B register, Address offset: 0x20 
        /// </summary>
        public static Register ALRMBR_Register;

        /// <summary>
        /// RTC write protection register,Address offset: 0x24 
        /// </summary>
        public static Register WPR_Register;

        /// <summary>
        /// RTC sub second register,Address offset: 0x28 
        /// </summary>
        public static Register SSR_Register;

        /// <summary>
        /// RTC shift control register, Address offset: 0x2C 
        /// </summary>
        public static Register SHIFTR_Register;

        /// <summary>
        /// RTC time stamp time register, Address offset: 0x30 
        /// </summary>
        public static Register TSTR_Register;

        /// <summary>
        /// RTC time stamp date register, Address offset: 0x34 
        /// </summary>
        public static Register TSDR_Register;

        /// <summary>
        /// RTC time-stamp sub second register, Address offset: 0x38 
        /// </summary>
        public static Register TSSSR_Register;

        /// <summary>
        /// RTC calibration register, Address offset: 0x3C 
        /// </summary>
        public static Register CALR_Register;

        /// <summary>
        /// RTC tamper and alternate function configuration register, Address offset: 0x40 
        /// </summary>
        public static Register TAFCR_Register;

        /// <summary>
        ///  RTC alarm A sub second register,Address offset: 0x44 
        /// </summary>
        public static Register ALRMASSR_Register;

        /// <summary>
        ///  RTC alarm B sub second register,Address offset: 0x48 
        /// </summary>
        public static Register ALRMBSSR_Register;

        /// <summary>
        /// RTC backup register 1,Address offset: 0x50 
        /// </summary>
        public static Register BKP0R_Register;

        /// <summary>
        /// RTC backup register 1,Address offset: 0x54 
        /// </summary>
        public static Register BKP1R_Register;

        /// <summary>
        /// RTC backup register 2,Address offset: 0x58 
        /// </summary>
        public static Register BKP2R_Register;

        /// <summary>
        /// RTC backup register 3,Address offset: 0x5C 
        /// </summary>
        public static Register BKP3R_Register;

        /// <summary>
        /// RTC backup register 4,Address offset: 0x60 
        /// </summary>
        public static Register BKP4R_Register;

        /// <summary>
        /// RTC backup register 5,Address offset: 0x64 
        /// </summary>
        public static Register BKP5R_Register;

        /// <summary>
        /// RTC backup register 6,Address offset: 0x68 
        /// </summary>
        public static Register BKP6R_Register;

        /// <summary>
        /// RTC backup register 7,Address offset: 0x6C 
        /// </summary>
        public static Register BKP7R_Register;

        /// <summary>
        /// RTC backup register 8,Address offset: 0x70 
        /// </summary>
        public static Register BKP8R_Register;

        /// <summary>
        /// RTC backup register 9,Address offset: 0x74 
        /// </summary>
        public static Register BKP9R_Register;

        /// <summary>
        /// RTC backup register 10, Address offset: 0x78 
        /// </summary>
        public static Register BKP10R_Register;

        /// <summary>
        /// RTC backup register 11, Address offset: 0x7C 
        /// </summary>
        public static Register BKP11R_Register;

        /// <summary>
        /// RTC backup register 12, Address offset: 0x80 
        /// </summary>
        public static Register BKP12R_Register;

        /// <summary>
        /// RTC backup register 13, Address offset: 0x84 
        /// </summary>
        public static Register BKP13R_Register;

        /// <summary>
        /// RTC backup register 14, Address offset: 0x88 
        /// </summary>
        public static Register BKP14R_Register;

        /// <summary>
        /// RTC backup register 15, Address offset: 0x8C 
        /// </summary>
        public static Register BKP15R_Register;

        /// <summary>
        /// RTC backup register 16, Address offset: 0x90 
        /// </summary>
        public static Register BKP16R_Register;

        /// <summary>
        /// RTC backup register 17, Address offset: 0x94 
        /// </summary>
        public static Register BKP17R_Register;

        /// <summary>
        /// RTC backup register 18, Address offset: 0x98 
        /// </summary>
        public static Register BKP18R_Register;

        /// <summary>
        /// RTC backup register 19, Address offset: 0x9C 
        /// </summary>
        public static Register BKP19R_Register;

        #endregion

        static RTC()
        {
            TR_Register = new Register(STM32F4xx.RTC_BASE + TR);
            DR_Register = new Register(STM32F4xx.RTC_BASE + DR);
            CR_Register = new Register(STM32F4xx.RTC_BASE + CR);
            ISR_Register = new Register(STM32F4xx.RTC_BASE + ISR);
            PRER_Register = new Register(STM32F4xx.RTC_BASE + PRER);
            WUTR_Register = new Register(STM32F4xx.RTC_BASE + WUTR);
            CALIBR_Register = new Register(STM32F4xx.RTC_BASE + CALIBR);
            ALRMAR_Register = new Register(STM32F4xx.RTC_BASE + ALRMAR);
            ALRMBR_Register = new Register(STM32F4xx.RTC_BASE + ALRMBR);
            WPR_Register = new Register(STM32F4xx.RTC_BASE + WPR);
            SSR_Register = new Register(STM32F4xx.RTC_BASE + SSR);
            SHIFTR_Register = new Register(STM32F4xx.RTC_BASE + SHIFTR);
            TSTR_Register = new Register(STM32F4xx.RTC_BASE + TSTR);
            TSDR_Register = new Register(STM32F4xx.RTC_BASE + TSDR);
            TSSSR_Register = new Register(STM32F4xx.RTC_BASE + TSSSR);
            CALR_Register = new Register(STM32F4xx.RTC_BASE + CALR);
            TAFCR_Register = new Register(STM32F4xx.RTC_BASE + TAFCR);
            ALRMASSR_Register = new Register(STM32F4xx.RTC_BASE + ALRMASSR);
            ALRMBSSR_Register = new Register(STM32F4xx.RTC_BASE + ALRMBSSR);
            BKP0R_Register = new Register(STM32F4xx.RTC_BASE + BKP0R);
            BKP1R_Register = new Register(STM32F4xx.RTC_BASE + BKP1R);
            BKP2R_Register = new Register(STM32F4xx.RTC_BASE + BKP2R);
            BKP3R_Register = new Register(STM32F4xx.RTC_BASE + BKP3R);
            BKP4R_Register = new Register(STM32F4xx.RTC_BASE + BKP4R);
            BKP5R_Register = new Register(STM32F4xx.RTC_BASE + BKP5R);
            BKP6R_Register = new Register(STM32F4xx.RTC_BASE + BKP6R);
            BKP7R_Register = new Register(STM32F4xx.RTC_BASE + BKP7R);
            BKP8R_Register = new Register(STM32F4xx.RTC_BASE + BKP8R);
            BKP9R_Register = new Register(STM32F4xx.RTC_BASE + BKP9R);
            BKP10R_Register = new Register(STM32F4xx.RTC_BASE + BKP10R);
            BKP11R_Register = new Register(STM32F4xx.RTC_BASE + BKP11R);
            BKP12R_Register = new Register(STM32F4xx.RTC_BASE + BKP12R);
            BKP13R_Register = new Register(STM32F4xx.RTC_BASE + BKP13R);
            BKP14R_Register = new Register(STM32F4xx.RTC_BASE + BKP14R);
            BKP15R_Register = new Register(STM32F4xx.RTC_BASE + BKP15R);
            BKP16R_Register = new Register(STM32F4xx.RTC_BASE + BKP16R);
            BKP17R_Register = new Register(STM32F4xx.RTC_BASE + BKP17R);
            BKP18R_Register = new Register(STM32F4xx.RTC_BASE + BKP18R);
            BKP19R_Register = new Register(STM32F4xx.RTC_BASE + BKP19R);
        }
    } // END of Class...

    /// <summary>
    /// Window watchdog (WWDG) - See: P.697
    /// </summary>
    public static class WWDG
    {
        #region Private's...

        /// <summary>
        /// WWDG Control register, Address offset: 0x00 
        /// </summary>
        private static uint _CR = 0x00;

        /// <summary>
        /// WWDG Configuration register, Address offset: 0x04 
        /// </summary>
        private static uint _CFR = 0x04;

        /// <summary>
        /// WWDG Status register,Address offset: 0x08 
        /// </summary>
        private static uint _SR = 0x08;

        #endregion

        #region Properties...

        /// <summary>
        /// WWDG Control register, Address offset: 0x00 
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// WWDG Configuration register, Address offset: 0x04 
        /// </summary>
        public static uint CFR { get { return _CFR; } set { _CFR = value; } }

        /// <summary>
        /// WWDG Status register,Address offset: 0x08 
        /// </summary>
        public static uint SR { get { return _SR; } set { _SR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// WWDG Control register, Address offset: 0x00 
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// WWDG Configuration register, Address offset: 0x04 
        /// </summary>
        public static Register CFR_Register;

        /// <summary>
        /// WWDG Status register,Address offset: 0x08 
        /// </summary>
        public static Register SR_Register;

        #endregion

        static WWDG()
        {
            CR_Register = new Register(STM32F4xx.WWDG_BASE + CR);
            CFR_Register = new Register(STM32F4xx.WWDG_BASE + CFR);
            SR_Register = new Register(STM32F4xx.WWDG_BASE + SR);
        }
    } // END of Class...

    /// <summary>
    /// Independent watchdog (IWDG) - See: P.691
    /// </summary>
    public static class IWDG
    {
        #region Private's...

        /// <summary>
        /// IWDG Key register, Address offset: 0x00 
        /// </summary>
        private static uint _KR = 0x00;

        /// <summary>
        /// IWDG Prescaler register, Address offset: 0x04 
        /// </summary>
        private static uint _PR = 0x04;

        /// <summary>
        /// IWDG Reload register,Address offset: 0x08 
        /// </summary>
        private static uint _RLR = 0x08;

        /// <summary>
        /// IWDG Status register,Address offset: 0x0C 
        /// </summary>
        private static uint _SR = 0x0C;

        #endregion

        #region Properties...

        /// <summary>
        /// IWDG Key register, Address offset: 0x00 
        /// </summary>
        public static uint KR { get { return _KR; } set { _KR = value; } }

        /// <summary>
        /// IWDG Prescaler register, Address offset: 0x04 
        /// </summary>
        public static uint PR { get { return _PR; } set { _PR = value; } }

        /// <summary>
        /// IWDG Reload register,Address offset: 0x08 
        /// </summary>
        public static uint RLR { get { return _RLR; } set { _RLR = value; } }

        /// <summary>
        /// IWDG Status register,Address offset: 0x0C 
        /// </summary>
        public static uint SR { get { return _SR; } set { _SR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// IWDG Key register, Address offset: 0x00 
        /// </summary>
        public static Register KR_Register;

        /// <summary>
        /// IWDG Prescaler register, Address offset: 0x04 
        /// </summary>
        public static Register PR_Register;

        /// <summary>
        /// IWDG Reload register,Address offset: 0x08 
        /// </summary>
        public static Register RLR_Register;

        /// <summary>
        /// IWDG Status register,Address offset: 0x0C 
        /// </summary>
        public static Register SR_Register;

        #endregion

        static IWDG()
        {
            KR_Register = new Register(STM32F4xx.IWDG_BASE + KR);
            PR_Register = new Register(STM32F4xx.IWDG_BASE + PR);
            RLR_Register = new Register(STM32F4xx.IWDG_BASE + RLR);
            SR_Register = new Register(STM32F4xx.IWDG_BASE + SR);
        }
    } // END of Class...

    /// <summary>
    /// Serial peripheral interface (SPI) - See: P.857
    /// </summary>
    public sealed class SPI
    {
        #region Private's...

        /// <summary>
        /// SPI control register 1 (not used in I2S mode),Address offset: 0x00 
        /// </summary>
        private uint _CR1 = 0x00;

        /// <summary>
        /// SPI control register 2, Address offset: 0x04 
        /// </summary>
        private uint _CR2 = 0x04;

        /// <summary>
        /// SPI status register,Address offset: 0x08 
        /// </summary>
        private uint _SR = 0x08;

        /// <summary>
        /// SPI data register,Address offset: 0x0C 
        /// </summary>
        private uint _DR = 0x0C;

        /// <summary>
        /// SPI CRC polynomial register (not used in I2S mode), Address offset: 0x10 
        /// </summary>
        private uint _CRCPR = 0x10;

        /// <summary>
        /// SPI RX CRC register (not used in I2S mode), Address offset: 0x14 
        /// </summary>
        private uint _RXCRCR = 0x14;

        /// <summary>
        /// SPI TX CRC register (not used in I2S mode), Address offset: 0x18 
        /// </summary>
        private uint _TXCRCR = 0x18;

        /// <summary>
        /// SPI_I2S configuration register, Address offset: 0x1C 
        /// </summary>
        private uint _I2SCFGR = 0x1C;

        /// <summary>
        /// SPI_I2S prescaler register, Address offset: 0x20 
        /// </summary>
        private uint _I2SPR = 0x20;

        #endregion

        #region Properties...

        /// <summary>
        /// SPI control register 1 (not used in I2S mode),Address offset: 0x00 
        /// </summary>
        public uint CR1 { get { return _CR1; } set { _CR1 = value; } }

        /// <summary>
        /// SPI control register 2, Address offset: 0x04 
        /// </summary>
        public uint CR2 { get { return _CR2; } set { _CR2 = value; } }

        /// <summary>
        /// SPI status register,Address offset: 0x08 
        /// </summary>
        public uint SR { get { return _SR; } set { _SR = value; } }

        /// <summary>
        /// SPI data register,Address offset: 0x0C 
        /// </summary>
        public uint DR { get { return _DR; } set { _DR = value; } }

        /// <summary>
        /// SPI CRC polynomial register (not used in I2S mode), Address offset: 0x10 
        /// </summary>
        public uint CRCPR { get { return _CRCPR; } set { _CRCPR = value; } }

        /// <summary>
        /// SPI RX CRC register (not used in I2S mode), Address offset: 0x14 
        /// </summary>
        public uint RXCRCR { get { return _RXCRCR; } set { _RXCRCR = value; } }

        /// <summary>
        /// SPI TX CRC register (not used in I2S mode), Address offset: 0x18 
        /// </summary>
        public uint TXCRCR { get { return _TXCRCR; } set { _TXCRCR = value; } }

        /// <summary>
        /// SPI_I2S configuration register, Address offset: 0x1C 
        /// </summary>
        public uint I2SCFGR { get { return _I2SCFGR; } set { _I2SCFGR = value; } }

        /// <summary>
        /// SPI_I2S prescaler register, Address offset: 0x20 
        /// </summary>
        public uint I2SPR { get { return _I2SPR; } set { _I2SPR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// SPI control register 1 (not used in I2S mode),Address offset: 0x00 
        /// </summary>
        public Register CR1_Register;

        /// <summary>
        /// SPI control register 2, Address offset: 0x04 
        /// </summary>
        public Register CR2_Register;

        /// <summary>
        /// SPI status register,Address offset: 0x08 
        /// </summary>
        public Register SR_Register;

        /// <summary>
        /// SPI data register,Address offset: 0x0C 
        /// </summary>
        public Register DR_Register;

        /// <summary>
        /// SPI CRC polynomial register (not used in I2S mode), Address offset: 0x10 
        /// </summary>
        public Register CRCPR_Register;

        /// <summary>
        /// SPI RX CRC register (not used in I2S mode), Address offset: 0x14 
        /// </summary>
        public Register RXCRCR_Register;

        /// <summary>
        /// SPI TX CRC register (not used in I2S mode), Address offset: 0x18 
        /// </summary>
        public Register TXCRCR_Register;

        /// <summary>
        /// SPI_I2S configuration register, Address offset: 0x1C 
        /// </summary>
        public Register I2SCFGR_Register;

        /// <summary>
        /// SPI_I2S prescaler register, Address offset: 0x20 
        /// </summary>
        public Register I2SPR_Register;

        #endregion

        public static SPI SPI1 = new SPI(STM32F4xx.SPI1_BASE);
        public static SPI SPI2 = new SPI(STM32F4xx.SPI2_BASE);
        public static SPI SPI3 = new SPI(STM32F4xx.SPI3_BASE);

        public SPI(UInt32 XX_BASE)
        {
            CR1_Register = new Register(XX_BASE + CR1);
            CR2_Register = new Register(XX_BASE + CR2);
            SR_Register = new Register(XX_BASE + SR);
            DR_Register = new Register(XX_BASE + DR);
            CRCPR_Register = new Register(XX_BASE + CRCPR);
            RXCRCR_Register = new Register(XX_BASE + RXCRCR);
            TXCRCR_Register = new Register(XX_BASE + TXCRCR);
            I2SCFGR_Register = new Register(XX_BASE + I2SCFGR);
            I2SPR_Register = new Register(XX_BASE + I2SPR);
        }
    } // END of Class...

    /// <summary>
    /// Universal synchronous asynchronous receiver transmitter (USART) - See: P.951
    /// </summary>
    public sealed class USART
    {
        #region Private's...

        /// <summary>
        /// USART Status register, Address offset: 0x00 
        /// </summary>
        private uint _SR = 0x00;

        /// <summary>
        /// USART Data register, Address offset: 0x04 
        /// </summary>
        private uint _DR = 0x04;

        /// <summary>
        /// USART Baud rate register,Address offset: 0x08 
        /// </summary>
        private uint _BRR = 0x08;

        /// <summary>
        /// USART Control register 1,Address offset: 0x0C 
        /// </summary>
        private uint _CR1 = 0x0C;

        /// <summary>
        /// USART Control register 2,Address offset: 0x10 
        /// </summary>
        private uint _CR2 = 0x10;

        /// <summary>
        /// USART Control register 3,Address offset: 0x14 
        /// </summary>
        private uint _CR3 = 0x14;

        /// <summary>
        /// USART Guard time and prescaler register, Address offset: 0x18 
        /// </summary>
        private uint _GTPR = 0x18;

        #endregion

        #region Properties...

        /// <summary>
        /// USART Status register, Address offset: 0x00 
        /// </summary>
        public uint SR { get { return _SR; } set { _SR = value; } }

        /// <summary>
        /// USART Data register, Address offset: 0x04 
        /// </summary>
        public uint DR { get { return _DR; } set { _DR = value; } }

        /// <summary>
        /// USART Baud rate register,Address offset: 0x08 
        /// </summary>
        public uint BRR { get { return _BRR; } set { _BRR = value; } }

        /// <summary>
        /// USART Control register 1,Address offset: 0x0C 
        /// </summary>
        public uint CR1 { get { return _CR1; } set { _CR1 = value; } }

        /// <summary>
        /// USART Control register 2,Address offset: 0x10 
        /// </summary>
        public uint CR2 { get { return _CR2; } set { _CR2 = value; } }

        /// <summary>
        /// USART Control register 3,Address offset: 0x14 
        /// </summary>
        public uint CR3 { get { return _CR3; } set { _CR3 = value; } }

        /// <summary>
        /// USART Guard time and prescaler register, Address offset: 0x18 
        /// </summary>
        public uint GTPR { get { return _GTPR; } set { _GTPR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// USART Status register, Address offset: 0x00 
        /// </summary>
        public Register SR_Register;

        /// <summary>
        /// USART Data register, Address offset: 0x04 
        /// </summary>
        public Register DR_Register;

        /// <summary>
        /// USART Baud rate register,Address offset: 0x08 
        /// </summary>
        public Register BRR_Register;

        /// <summary>
        /// USART Control register 1,Address offset: 0x0C 
        /// </summary>
        public Register CR1_Register;

        /// <summary>
        /// USART Control register 2,Address offset: 0x10 
        /// </summary>
        public Register CR2_Register;

        /// <summary>
        /// USART Control register 3,Address offset: 0x14 
        /// </summary>
        public Register CR3_Register;

        /// <summary>
        /// USART Guard time and prescaler register, Address offset: 0x18 
        /// </summary>
        public Register GTPR_Register;

        #endregion

        public static USART USART1 = new USART(STM32F4xx.USART1_BASE);
        public static USART USART2 = new USART(STM32F4xx.USART2_BASE);
        public static USART USART3 = new USART(STM32F4xx.USART3_BASE);
        public static USART UART4 = new USART(STM32F4xx.UART4_BASE);
        public static USART UART5 = new USART(STM32F4xx.UART5_BASE);
        public static USART USART6 = new USART(STM32F4xx.USART6_BASE);

        public USART(UInt32 XX_BASE)
        {
            SR_Register = new Register(XX_BASE + SR);
            DR_Register = new Register(XX_BASE + DR);
            BRR_Register = new Register(XX_BASE + BRR);
            CR1_Register = new Register(XX_BASE + CR1);
            CR2_Register = new Register(XX_BASE + CR2);
            CR3_Register = new Register(XX_BASE + CR3);
            GTPR_Register = new Register(XX_BASE + GTPR);
        }
    }// END of Class...

    /// <summary>
    /// Inter-integrated circuit (I2C) interface - See: P.820
    /// </summary>
    public sealed class I2C
    {
        #region Private's...

        /// <summary>
        /// I2C Control register 1, Address offset: 0x00 
        /// </summary>
        private uint _CR1 = 0x00;

        /// <summary>
        /// I2C Control register 2, Address offset: 0x04 
        /// </summary>
        private uint _CR2 = 0x04;

        /// <summary>
        /// I2C Own address register 1, Address offset: 0x08 
        /// </summary>
        private uint _OAR1 = 0x08;

        /// <summary>
        /// I2C Own address register 2, Address offset: 0x0C 
        /// </summary>
        private uint _OAR2 = 0x0C;

        /// <summary>
        /// I2C Data register,Address offset: 0x10 
        /// </summary>
        private uint _DR = 0x10;

        /// <summary>
        /// I2C Status register 1,Address offset: 0x14 
        /// </summary>
        private uint _SR1 = 0x14;

        /// <summary>
        /// I2C Status register 2,Address offset: 0x18 
        /// </summary>
        private uint _SR2 = 0x18;

        /// <summary>
        /// I2C Clock control register, Address offset: 0x1C 
        /// </summary>
        private uint _CCR = 0x1C;

        /// <summary>
        /// I2C TRISE register, Address offset: 0x20 
        /// </summary>
        private uint _TRISE = 0x20;

        #endregion

        #region Properties...

        /// <summary>
        /// I2C Control register 1, Address offset: 0x00 
        /// </summary>
        public uint CR1 { get { return _CR1; } set { _CR1 = value; } }

        /// <summary>
        /// I2C Control register 2, Address offset: 0x04 
        /// </summary>
        public uint CR2 { get { return _CR2; } set { _CR2 = value; } }

        /// <summary>
        /// I2C Own address register 1, Address offset: 0x08 
        /// </summary>
        public uint OAR1 { get { return _OAR1; } set { _OAR1 = value; } }

        /// <summary>
        /// I2C Own address register 2, Address offset: 0x0C 
        /// </summary>
        public uint OAR2 { get { return _OAR2; } set { _OAR2 = value; } }

        /// <summary>
        /// I2C Data register,Address offset: 0x10 
        /// </summary>
        public uint DR { get { return _DR; } set { _DR = value; } }

        /// <summary>
        /// I2C Status register 1,Address offset: 0x14 
        /// </summary>
        public uint SR1 { get { return _SR1; } set { _SR1 = value; } }

        /// <summary>
        /// I2C Status register 2,Address offset: 0x18 
        /// </summary>
        public uint SR2 { get { return _SR2; } set { _SR2 = value; } }

        /// <summary>
        /// I2C Clock control register, Address offset: 0x1C 
        /// </summary>
        public uint CCR { get { return _CCR; } set { _CCR = value; } }

        /// <summary>
        /// I2C TRISE register, Address offset: 0x20 
        /// </summary>
        public uint TRISE { get { return _TRISE; } set { _TRISE = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// I2C Control register 1, Address offset: 0x00 
        /// </summary>
        public Register CR1_Register;

        /// <summary>
        /// I2C Control register 2, Address offset: 0x04 
        /// </summary>
        public Register CR2_Register;

        /// <summary>
        /// I2C Own address register 1, Address offset: 0x08 
        /// </summary>
        public Register OAR1_Register;

        /// <summary>
        /// I2C Own address register 2, Address offset: 0x0C 
        /// </summary>
        public Register OAR2_Register;

        /// <summary>
        /// I2C Data register,Address offset: 0x10 
        /// </summary>
        public Register DR_Register;

        /// <summary>
        /// I2C Status register 1,Address offset: 0x14 
        /// </summary>
        public Register SR1_Register;

        /// <summary>
        /// I2C Status register 2,Address offset: 0x18 
        /// </summary>
        public Register SR2_Register;

        /// <summary>
        /// I2C Clock control register, Address offset: 0x1C 
        /// </summary>
        public Register CCR_Register;

        /// <summary>
        /// I2C TRISE register, Address offset: 0x20 
        /// </summary>
        public Register TRISE_Register;

        #endregion

        public static I2C I2C1 = new I2C(STM32F4xx.I2C1_BASE);
        public static I2C I2C2 = new I2C(STM32F4xx.I2C2_BASE);
        public static I2C I2C3 = new I2C(STM32F4xx.I2C3_BASE);
        public static I2C I2S2ext = new I2C(STM32F4xx.I2S2ext_BASE);
        public static I2C I2S3ext = new I2C(STM32F4xx.I2S3ext_BASE);

        public I2C(UInt32 XX_BASE)
        {
            CR1_Register = new Register(XX_BASE + CR1);
            CR2_Register = new Register(XX_BASE + CR2);
            OAR1_Register = new Register(XX_BASE + OAR1);
            OAR2_Register = new Register(XX_BASE + OAR2);
            DR_Register = new Register(XX_BASE + DR);
            SR1_Register = new Register(XX_BASE + SR1);
            SR2_Register = new Register(XX_BASE + SR2);
            CCR_Register = new Register(XX_BASE + CCR);
            TRISE_Register = new Register(XX_BASE + TRISE);
        }
    } // END of Class...

    /// <summary>
    /// Controller area network (bxCAN) - See: P.1059
    /// </summary>
    public sealed class CAN
    {
        #region Private's...


        /// <summary>
        /// CAN master control register, Address offset: 0x00
        /// </summary>
        private uint _MCR = 0x00;


        /// <summary>
        /// CAN master status register,Address offset: 0x04
        /// </summary>
        private uint _MSR = 0x04;


        /// <summary>
        /// CAN transmit status register,Address offset: 0x08
        /// </summary>
        private uint _TSR = 0x08;


        /// <summary>
        /// CAN receive FIFO 0 register, Address offset: 0x0C
        /// </summary>
        private uint _RF0R = 0x0C;


        /// <summary>
        /// CAN receive FIFO 1 register, Address offset: 0x10
        /// </summary>
        private uint _RF1R = 0x10;


        /// <summary>
        /// CAN interrupt enable register, Address offset: 0x14
        /// </summary>
        private uint _IER = 0x14;


        /// <summary>
        /// CAN error status register, Address offset: 0x18
        /// </summary>
        private uint _ESR = 0x18;


        /// <summary>
        /// CAN bit timing register, Address offset: 0x1C
        /// </summary>
        private uint _BTR = 0x1C;


        /// <summary>
        /// CAN filter master register,Address offset: 0x200 
        /// </summary>
        private uint _FMR = 0x200;


        /// <summary>
        /// CAN filter mode register,Address offset: 0x204 
        /// </summary>
        private uint _FM1R = 0x204;


        /// <summary>
        /// CAN filter scale register, Address offset: 0x20C 
        /// </summary>
        private uint _FS1R = 0x20C;


        /// <summary>
        /// CAN filter FIFO assignment register, Address offset: 0x214 
        /// </summary>
        private uint _FFA1R = 0x214;


        /// <summary>
        /// CAN filter activation register,Address offset: 0x21C 
        /// </summary>
        private uint _FA1R = 0x21C;


        #endregion

        #region Properties...


        /// <summary>
        /// CAN master control register, Address offset: 0x00
        /// </summary>
        public uint MCR { get { return _MCR; } set { _MCR = value; } }


        /// <summary>
        /// CAN master status register,Address offset: 0x04
        /// </summary>
        public uint MSR { get { return _MSR; } set { _MSR = value; } }


        /// <summary>
        /// CAN transmit status register,Address offset: 0x08
        /// </summary>
        public uint TSR { get { return _TSR; } set { _TSR = value; } }


        /// <summary>
        /// CAN receive FIFO 0 register, Address offset: 0x0C
        /// </summary>
        public uint RF0R { get { return _RF0R; } set { _RF0R = value; } }


        /// <summary>
        /// CAN receive FIFO 1 register, Address offset: 0x10
        /// </summary>
        public uint RF1R { get { return _RF1R; } set { _RF1R = value; } }


        /// <summary>
        /// CAN interrupt enable register, Address offset: 0x14
        /// </summary>
        public uint IER { get { return _IER; } set { _IER = value; } }


        /// <summary>
        /// CAN error status register, Address offset: 0x18
        /// </summary>
        public uint ESR { get { return _ESR; } set { _ESR = value; } }


        /// <summary>
        /// CAN bit timing register, Address offset: 0x1C
        /// </summary>
        public uint BTR { get { return _BTR; } set { _BTR = value; } }


        /// <summary>
        /// CAN filter master register,Address offset: 0x200 
        /// </summary>
        public uint FMR { get { return _FMR; } set { _FMR = value; } }


        /// <summary>
        /// CAN filter mode register,Address offset: 0x204 
        /// </summary>
        public uint FM1R { get { return _FM1R; } set { _FM1R = value; } }


        /// <summary>
        /// CAN filter scale register, Address offset: 0x20C 
        /// </summary>
        public uint FS1R { get { return _FS1R; } set { _FS1R = value; } }


        /// <summary>
        /// CAN filter FIFO assignment register, Address offset: 0x214 
        /// </summary>
        public uint FFA1R { get { return _FFA1R; } set { _FFA1R = value; } }


        /// <summary>
        /// CAN filter activation register,Address offset: 0x21C 
        /// </summary>
        public uint FA1R { get { return _FA1R; } set { _FA1R = value; } }


        #endregion

        #region Registers...


        /// <summary>
        /// CAN master control register, Address offset: 0x00
        /// </summary>
        public Register MCR_Register;


        /// <summary>
        /// CAN master status register,Address offset: 0x04
        /// </summary>
        public Register MSR_Register;


        /// <summary>
        /// CAN transmit status register,Address offset: 0x08
        /// </summary>
        public Register TSR_Register;


        /// <summary>
        /// CAN receive FIFO 0 register, Address offset: 0x0C
        /// </summary>
        public Register RF0R_Register;


        /// <summary>
        /// CAN receive FIFO 1 register, Address offset: 0x10
        /// </summary>
        public Register RF1R_Register;


        /// <summary>
        /// CAN interrupt enable register, Address offset: 0x14
        /// </summary>
        public Register IER_Register;


        /// <summary>
        /// CAN error status register, Address offset: 0x18
        /// </summary>
        public Register ESR_Register;


        /// <summary>
        /// CAN bit timing register, Address offset: 0x1C
        /// </summary>
        public Register BTR_Register;


        /// <summary>
        /// CAN filter master register,Address offset: 0x200 
        /// </summary>
        public Register FMR_Register;


        /// <summary>
        /// CAN filter mode register,Address offset: 0x204 
        /// </summary>
        public Register FM1R_Register;


        /// <summary>
        /// CAN filter scale register, Address offset: 0x20C 
        /// </summary>
        public Register FS1R_Register;


        /// <summary>
        /// CAN filter FIFO assignment register, Address offset: 0x214 
        /// </summary>
        public Register FFA1R_Register;


        /// <summary>
        /// CAN filter activation register,Address offset: 0x21C 
        /// </summary>
        public Register FA1R_Register;


        #endregion

        public static CAN CAN1 = new CAN(STM32F4xx.CAN1_BASE);
        public static CAN CAN2 = new CAN(STM32F4xx.CAN2_BASE);

        public CAN(UInt32 XX_BASE)
        {
            MCR_Register = new Register(XX_BASE + MCR);
            MSR_Register = new Register(XX_BASE + MSR);
            TSR_Register = new Register(XX_BASE + TSR);
            RF0R_Register = new Register(XX_BASE + RF0R);
            RF1R_Register = new Register(XX_BASE + RF1R);
            IER_Register = new Register(XX_BASE + IER);
            ESR_Register = new Register(XX_BASE + ESR);
            BTR_Register = new Register(XX_BASE + BTR);
            FMR_Register = new Register(XX_BASE + FMR);
            FM1R_Register = new Register(XX_BASE + FM1R);
            FS1R_Register = new Register(XX_BASE + FS1R);
            FFA1R_Register = new Register(XX_BASE + FFA1R);
            FA1R_Register = new Register(XX_BASE + FA1R);
        }
    } // END of Class...

    /// <summary>
    /// Power controller (PWR) - See: P.115
    /// </summary>
    public static class PWR
    {
        #region Private's...

        /// <summary>
        /// PWR power control register,Address offset: 0x00 
        /// </summary>
        private static uint _CR = 0x00;

        /// <summary>
        /// PWR power control/status register, Address offset: 0x04 
        /// </summary>
        private static uint _CSR = 0x04;

        #endregion

        #region Properties...

        /// <summary>
        /// PWR power control register,Address offset: 0x00 
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// PWR power control/status register, Address offset: 0x04 
        /// </summary>
        public static uint CSR { get { return _CSR; } set { _CSR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// PWR power control register,Address offset: 0x00 
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// PWR power control/status register, Address offset: 0x04 
        /// </summary>
        public static Register CSR_Register;

        #endregion

        static PWR()
        {
            CR_Register = new Register(STM32F4xx.PWR_BASE + CR);
            CSR_Register = new Register(STM32F4xx.PWR_BASE + CSR);
        }
    } // END of Class...

    /// <summary>
    /// Analog-to-digital converter (ADC) - See: P.385
    /// </summary>
    public sealed class ADC
    {
        #region Privates...

        private UInt32 _CSR = STM32F4xx.ADC1_BASE + 0x300;
        private UInt32 _CCR = STM32F4xx.ADC1_BASE + 0x304;
        private UInt32 _CDR = STM32F4xx.ADC1_BASE + 0x308;

        private UInt32 _SR = 0x00;
        private UInt32 _CR1 = 0x04;
        private UInt32 _CR2 = 0x08;
        private UInt32 _SMPR1 = 0x0C;
        private UInt32 _SMPR2 = 0x10;
        private UInt32 _JOFR1 = 0x14;
        private UInt32 _JOFR2 = 0x18;
        private UInt32 _JOFR3 = 0x1C;
        private UInt32 _JOFR4 = 0x20;
        private UInt32 _HTR = 0x24;
        private UInt32 _LTR = 0x28;
        private UInt32 _SQR1 = 0x2C;
        private UInt32 _SQR2 = 0x30;
        private UInt32 _SQR3 = 0x34;
        private UInt32 _JSQR = 0x38;
        private UInt32 _JDR1 = 0x3C;
        private UInt32 _JDR2 = 0x40;
        private UInt32 _JDR3 = 0x44;
        private UInt32 _JDR4 = 0x48;
        private UInt32 _DR = 0x4C;

        #endregion

        #region Properties...

        /// <summary>
        /// ADC Common status register, Address offset: ADC1 base address + 0x300 
        /// </summary>
        public UInt32 CSR { get { return _CSR; } set { _CSR = value; } }

        /// <summary>
        /// ADC common control register, Address offset: ADC1 base address + 0x304
        /// </summary>
        public UInt32 CCR { get { return _CCR; } set { _CCR = value; } }

        /// <summary>
        /// ADC common regular data register for dual AND triple modes, Address offset: ADC1 base address + 0x308
        /// </summary>
        public UInt32 CDR { get { return _CDR; } set { _CDR = value; } }

        /// <summary>
        /// ADC status register, Address offset: 0x00
        /// </summary>
        public UInt32 SR { get { return _SR; } set { _SR = value; } }

        /// <summary>
        /// ADC control register 1, Address offset: 0x04
        /// </summary>
        public UInt32 CR1 { get { return _CR1; } set { _CR1 = value; } }

        /// <summary>
        /// ADC control register 2, Address offset: 0x08
        /// </summary>
        public UInt32 CR2 { get { return _CR2; } set { _CR2 = value; } }

        /// <summary>
        /// ADC sample time register 1, Address offset: 0x0C
        /// </summary>
        public UInt32 SMPR1 { get { return _SMPR1; } set { _SMPR1 = value; } }

        /// <summary>
        /// ADC sample time register 2, Address offset: 0x10 
        /// </summary>
        public UInt32 SMPR2 { get { return _SMPR2; } set { _SMPR2 = value; } }

        /// <summary>
        /// ADC injected channel data offset register 1, Address offset: 0x14
        /// </summary>
        public UInt32 JOFR1 { get { return _JOFR1; } set { _JOFR1 = value; } }

        /// <summary>
        /// ADC injected channel data offset register 2, Address offset: 0x18
        /// </summary>
        public UInt32 JOFR2 { get { return _JOFR2; } set { _JOFR2 = value; } }

        /// <summary>
        /// ADC injected channel data offset register 3, Address offset: 0x1C
        /// </summary>
        public UInt32 JOFR3 { get { return _JOFR3; } set { _JOFR3 = value; } }

        /// <summary>
        /// ADC injected channel data offset register 4, Address offset: 0x20 
        /// </summary>
        public UInt32 JOFR4 { get { return _JOFR4; } set { _JOFR4 = value; } }

        /// <summary>
        /// ADC watchdog higher threshold register, Address offset: 0x24
        /// </summary>
        public UInt32 HTR { get { return _HTR; } set { _HTR = value; } }

        /// <summary>
        /// ADC watchdog lower threshold register, Address offset: 0x28
        /// </summary>
        public UInt32 LTR { get { return _LTR; } set { _LTR = value; } }

        /// <summary>
        /// ADC regular sequence register 1, Address offset: 0x2C
        /// </summary>
        public UInt32 SQR1 { get { return _SQR1; } set { _SQR1 = value; } }

        /// <summary>
        /// ADC regular sequence register 2, Address offset: 0x30
        /// </summary>
        public UInt32 SQR2 { get { return _SQR2; } set { _SQR2 = value; } }

        /// <summary>
        /// ADC regular sequence register 3, Address offset: 0x34 
        /// </summary>
        public UInt32 SQR3 { get { return _SQR3; } set { _SQR3 = value; } }

        /// <summary>
        /// ADC injected sequence register, Address offset: 0x38
        /// </summary>
        public UInt32 JSQR { get { return _JSQR; } set { _JSQR = value; } }

        /// <summary>
        /// ADC injected data register 1, Address offset: 0x3C
        /// </summary>
        public UInt32 JDR1 { get { return _JDR1; } set { _JDR1 = value; } }

        /// <summary>
        /// ADC injected data register 2, Address offset: 0x40
        /// </summary>
        public UInt32 JDR2 { get { return _JDR2; } set { _JDR2 = value; } }

        /// <summary>
        /// ADC injected data register 3, Address offset: 0x44
        /// </summary>
        public UInt32 JDR3 { get { return _JDR3; } set { _JDR3 = value; } }

        /// <summary>
        /// ADC injected data register 4, Address offset: 0x48
        /// </summary>
        public UInt32 JDR4 { get { return _JDR4; } set { _JDR4 = value; } }

        /// <summary>
        /// ADC regular data register, Address offset: 0x4C
        /// </summary>
        public UInt32 DR { get { return _DR; } set { _DR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// ADC Common status register, Address offset: ADC1 base address + 0x300 
        /// </summary>
        public Register CSR_Register;

        /// <summary>
        /// ADC common control register, Address offset: ADC1 base address + 0x304
        /// </summary>
        public Register CCR_Register;

        /// <summary>
        /// ADC common regular data register for dual AND triple modes, Address offset: ADC1 base address + 0x308
        /// </summary>
        public Register CDR_Register;

        /// <summary>
        /// ADC status register, Address offset: 0x00
        /// </summary>
        public Register SR_Register;

        /// <summary>
        /// ADC control register 1, Address offset: 0x04
        /// </summary>
        public Register CR1_Register;

        /// <summary>
        /// ADC control register 2, Address offset: 0x08
        /// </summary>
        public Register CR2_Register;

        /// <summary>
        /// ADC sample time register 1, Address offset: 0x0C
        /// </summary>
        public Register SMPR1_Register;

        /// <summary>
        /// ADC sample time register 2, Address offset: 0x10 
        /// </summary>
        public Register SMPR2_Register;

        /// <summary>
        /// ADC injected channel data offset register 1, Address offset: 0x14
        /// </summary>
        public Register JOFR1_Register;

        /// <summary>
        /// ADC injected channel data offset register 2, Address offset: 0x18
        /// </summary>
        public Register JOFR2_Register;

        /// <summary>
        /// ADC injected channel data offset register 3, Address offset: 0x1C
        /// </summary>
        public Register JOFR3_Register;

        /// <summary>
        /// ADC injected channel data offset register 4, Address offset: 0x20 
        /// </summary>
        public Register JOFR4_Register;

        /// <summary>
        /// ADC watchdog higher threshold register, Address offset: 0x24
        /// </summary>
        public Register HTR_Register;

        /// <summary>
        /// ADC watchdog lower threshold register, Address offset: 0x28
        /// </summary>
        public Register LTR_Register;

        /// <summary>
        /// ADC regular sequence register 1, Address offset: 0x2C
        /// </summary>
        public Register SQR1_Register;

        /// <summary>
        /// ADC regular sequence register 2, Address offset: 0x30
        /// </summary>
        public Register SQR2_Register;

        /// <summary>
        /// ADC regular sequence register 3, Address offset: 0x34 
        /// </summary>
        public Register SQR3_Register;

        /// <summary>
        /// ADC injected sequence register, Address offset: 0x38
        /// </summary>
        public Register JSQR_Register;

        /// <summary>
        /// ADC injected data register 1, Address offset: 0x3C
        /// </summary>
        public Register JDR1_Register;

        /// <summary>
        /// ADC injected data register 2, Address offset: 0x40
        /// </summary>
        public Register JDR2_Register;

        /// <summary>
        /// ADC injected data register 3, Address offset: 0x44
        /// </summary>
        public Register JDR3_Register;

        /// <summary>
        /// ADC injected data register 4, Address offset: 0x48
        /// </summary>
        public Register JDR4_Register;

        /// <summary>
        /// ADC regular data register, Address offset: 0x4C
        /// </summary>
        public Register DR_Register;

        #endregion

        public static ADC ADC1 = new ADC(STM32F4xx.ADC1_BASE);
        public static ADC ADC2 = new ADC(STM32F4xx.ADC2_BASE);
        public static ADC ADC3 = new ADC(STM32F4xx.ADC3_BASE);

        public ADC(UInt32 ADCx_BASE)
        {
            CSR_Register = new Register(STM32F4xx.ADC1_BASE + CSR);
            CCR_Register = new Register(STM32F4xx.ADC1_BASE + CCR);
            CDR_Register = new Register(STM32F4xx.ADC1_BASE + CDR);

            SR_Register = new Register(ADCx_BASE + SR);
            CR1_Register = new Register(ADCx_BASE + CR1);
            CR2_Register = new Register(ADCx_BASE + CR2);
            SMPR1_Register = new Register(ADCx_BASE + SMPR1);
            SMPR2_Register = new Register(ADCx_BASE + SMPR2);
            JOFR1_Register = new Register(ADCx_BASE + JOFR1);
            JOFR2_Register = new Register(ADCx_BASE + JOFR2);
            JOFR3_Register = new Register(ADCx_BASE + JOFR3);
            JOFR4_Register = new Register(ADCx_BASE + JOFR4);
            HTR_Register = new Register(ADCx_BASE + HTR);
            LTR_Register = new Register(ADCx_BASE + LTR);
            SQR1_Register = new Register(ADCx_BASE + SQR1);
            SQR2_Register = new Register(ADCx_BASE + SQR2);
            SQR3_Register = new Register(ADCx_BASE + SQR3);
            JSQR_Register = new Register(ADCx_BASE + JSQR);
            JDR1_Register = new Register(ADCx_BASE + JDR1);
            JDR2_Register = new Register(ADCx_BASE + JDR2);
            JDR3_Register = new Register(ADCx_BASE + JDR3);
            JDR4_Register = new Register(ADCx_BASE + JDR4);
            DR_Register = new Register(ADCx_BASE + DR);
        }

    } // END of Class...

    /// <summary>
    /// Digital-to-analog converter (DAC) - See: P.430
    /// </summary>
    public static class DAC
    {
        #region Private's...

        private static byte _CR = 0x00;
        private static byte _SWTRIGR = 0x04;
        private static byte _DHR12R1 = 0x08;
        private static byte _DHR12L1 = 0x0C;
        private static byte _DHR8R1 = 0x10;
        private static byte _DHR12R2 = 0x14;
        private static byte _DHR12L2 = 0x18;
        private static byte _DHR8R2 = 0x1C;
        private static byte _DHR12RD = 0x20;
        private static byte _DHR12LD = 0x24;
        private static byte _DHR8RD = 0x28;
        private static byte _DOR1 = 0x2C;
        private static byte _DOR2 = 0x30;
        private static byte _SR = 0x34;

        #endregion

        #region Properties...

        /// <summary>
        /// DAC control register, Address offset: 0x00
        /// </summary>
        public static byte CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// DAC software trigger register, Address offset: 0x04
        /// </summary>
        public static byte SWTRIGR { get { return _SWTRIGR; } set { _SWTRIGR = value; } }

        /// <summary>
        /// DAC channel1 12-bit right-aligned data holding register, Address offset: 0x08
        /// </summary>
        public static byte DHR12R1 { get { return _DHR12R1; } set { _DHR12R1 = value; } }

        /// <summary>
        /// DAC channel1 12-bit left aligned data holding register,  Address offset: 0x0C
        /// </summary>
        public static byte DHR12L1 { get { return _DHR12L1; } set { _DHR12L1 = value; } }

        /// <summary>
        /// DAC channel1 8-bit right aligned data holding register,  Address offset: 0x10
        /// </summary>
        public static byte DHR8R1 { get { return _DHR8R1; } set { _DHR8R1 = value; } }

        /// <summary>
        /// DAC channel2 12-bit right aligned data holding register, Address offset: 0x14
        /// </summary>
        public static byte DHR12R2 { get { return _DHR12R2; } set { _DHR12R2 = value; } }

        /// <summary>
        /// DAC channel2 12-bit left aligned data holding register,  Address offset: 0x18
        /// </summary>
        public static byte DHR12L2 { get { return _DHR12L2; } set { _DHR12L2 = value; } }

        /// <summary>
        /// DAC channel2 8-bit right-aligned data holding register, Address offset: 0x1C
        /// </summary>
        public static byte DHR8R2 { get { return _DHR8R2; } set { _DHR8R2 = value; } }

        /// <summary>
        /// Dual DAC 12-bit right-aligned data holding register, Address offset: 0x20
        /// </summary>
        public static byte DHR12RD { get { return _DHR12RD; } set { _DHR12RD = value; } }

        /// <summary>
        /// DUAL DAC 12-bit left aligned data holding register, Address offset: 0x24
        /// </summary>
        public static byte DHR12LD { get { return _DHR12LD; } set { _DHR12LD = value; } }

        /// <summary>
        /// DUAL DAC 8-bit right aligned data holding register, Address offset: 0x28
        /// </summary>
        public static byte DHR8RD { get { return _DHR8RD; } set { _DHR8RD = value; } }

        /// <summary>
        /// DAC channel1 data output register, Address offset: 0x2C
        /// </summary>
        public static byte DOR1 { get { return _DOR1; } set { _DOR1 = value; } }

        /// <summary>
        /// DAC channel2 data output register, Address offset: 0x30
        /// </summary>
        public static byte DOR2 { get { return _DOR2; } set { _DOR2 = value; } }

        /// <summary>
        /// DAC status register, Address offset: 0x34
        /// </summary>
        public static byte SR { get { return _SR; } set { _SR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// DAC control register, Address offset: 0x00
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// DAC software trigger register, Address offset: 0x04
        /// </summary>
        public static Register SWTRIGR_Register;

        /// <summary>
        /// DAC channel1 12-bit right-aligned data holding register, Address offset: 0x08
        /// </summary>
        public static Register DHR12R1_Register;

        /// <summary>
        /// DAC channel1 12-bit left aligned data holding register,  Address offset: 0x0C
        /// </summary>
        public static Register DHR12L1_Register;

        /// <summary>
        /// DAC channel1 8-bit right aligned data holding register,  Address offset: 0x10
        /// </summary>
        public static Register DHR8R1_Register;

        /// <summary>
        /// DAC channel2 12-bit right aligned data holding register, Address offset: 0x14
        /// </summary>
        public static Register DHR12R2_Register;

        /// <summary>
        /// DAC channel2 12-bit left aligned data holding register,  Address offset: 0x18
        /// </summary>
        public static Register DHR12L2_Register;

        /// <summary>
        /// DAC channel2 8-bit right-aligned data holding register, Address offset: 0x1C
        /// </summary>
        public static Register DHR8R2_Register;

        /// <summary>
        /// Dual DAC 12-bit right-aligned data holding register, Address offset: 0x20
        /// </summary>
        public static Register DHR12RD_Register;

        /// <summary>
        /// DUAL DAC 12-bit left aligned data holding register, Address offset: 0x24
        /// </summary>
        public static Register DHR12LD_Register;

        /// <summary>
        /// DUAL DAC 8-bit right aligned data holding register, Address offset: 0x28
        /// </summary>
        public static Register DHR8RD_Register;

        /// <summary>
        /// DAC channel1 data output register, Address offset: 0x2C
        /// </summary>
        public static Register DOR1_Register;

        /// <summary>
        /// DAC channel2 data output register, Address offset: 0x30
        /// </summary>
        public static Register DOR2_Register;

        /// <summary>
        /// DAC status register, Address offset: 0x34
        /// </summary>
        public static Register SR_Register;

        #endregion

        static DAC()
        {
            CR_Register = new Register(STM32F4xx.DAC_BASE + CR);
            SWTRIGR_Register = new Register(STM32F4xx.DAC_BASE + SWTRIGR);
            DHR12R1_Register = new Register(STM32F4xx.DAC_BASE + DHR12R1);
            DHR12L1_Register = new Register(STM32F4xx.DAC_BASE + DHR12L1);
            DHR8R1_Register = new Register(STM32F4xx.DAC_BASE + DHR8R1);
            DHR12R2_Register = new Register(STM32F4xx.DAC_BASE + DHR12R2);
            DHR12L2_Register = new Register(STM32F4xx.DAC_BASE + DHR12L2);
            DHR8R2_Register = new Register(STM32F4xx.DAC_BASE + DHR8R2);
            DHR12RD_Register = new Register(STM32F4xx.DAC_BASE + DHR12RD);
            DHR12LD_Register = new Register(STM32F4xx.DAC_BASE + DHR12LD);
            DHR8RD_Register = new Register(STM32F4xx.DAC_BASE + DHR8RD);
            DOR1_Register = new Register(STM32F4xx.DAC_BASE + DOR1);
            DOR2_Register = new Register(STM32F4xx.DAC_BASE + DOR2);
            SR_Register = new Register(STM32F4xx.DAC_BASE + SR);
        }

    } // END of Class...

    /// <summary>
    /// Secure digital input/output interface (SDIO) - See: P.1003
    /// </summary>
    public static class SDIO
    {
        #region Private's...

        /// <summary>
        /// SDIO power control register,Address offset: 0x00 
        /// </summary>
        private static uint _POWER = 0x00;

        /// <summary>
        /// SDI clock control register, Address offset: 0x04 
        /// </summary>
        private static uint _CLKCR = 0x04;

        /// <summary>
        /// SDIO argument register, Address offset: 0x08 
        /// </summary>
        private static uint _ARG = 0x08;

        /// <summary>
        /// SDIO command register,Address offset: 0x0C 
        /// </summary>
        private static uint _CMD = 0x0C;

        /// <summary>
        /// SDIO data timer register, Address offset: 0x24 
        /// </summary>
        private static uint _DTIMER = 0x24;

        /// <summary>
        /// SDIO data length register,Address offset: 0x28 
        /// </summary>
        private static uint _DLEN = 0x28;

        /// <summary>
        /// SDIO data control register, Address offset: 0x2C 
        /// </summary>
        private static uint _DCTRL = 0x2C;

        /// <summary>
        /// SDIO interrupt clear register,Address offset: 0x38 
        /// </summary>
        private static uint _ICR = 0x38;

        /// <summary>
        /// SDIO mask register, Address offset: 0x3C 
        /// </summary>
        private static uint _MASK = 0x3C;

        /// <summary>
        /// SDIO data FIFO register,Address offset: 0x80 
        /// </summary>
        private static uint _FIFO = 0x80;

        #endregion

        #region Properties...

        /// <summary>
        /// SDIO power control register,Address offset: 0x00 
        /// </summary>
        public static uint POWER { get { return _POWER; } set { _POWER = value; } }

        /// <summary>
        /// SDI clock control register, Address offset: 0x04 
        /// </summary>
        public static uint CLKCR { get { return _CLKCR; } set { _CLKCR = value; } }

        /// <summary>
        /// SDIO argument register, Address offset: 0x08 
        /// </summary>
        public static uint ARG { get { return _ARG; } set { _ARG = value; } }

        /// <summary>
        /// SDIO command register,Address offset: 0x0C 
        /// </summary>
        public static uint CMD { get { return _CMD; } set { _CMD = value; } }

        /// <summary>
        /// SDIO data timer register, Address offset: 0x24 
        /// </summary>
        public static uint DTIMER { get { return _DTIMER; } set { _DTIMER = value; } }

        /// <summary>
        /// SDIO data length register,Address offset: 0x28 
        /// </summary>
        public static uint DLEN { get { return _DLEN; } set { _DLEN = value; } }

        /// <summary>
        /// SDIO data control register, Address offset: 0x2C 
        /// </summary>
        public static uint DCTRL { get { return _DCTRL; } set { _DCTRL = value; } }

        /// <summary>
        /// SDIO interrupt clear register,Address offset: 0x38 
        /// </summary>
        public static uint ICR { get { return _ICR; } set { _ICR = value; } }

        /// <summary>
        /// SDIO mask register, Address offset: 0x3C 
        /// </summary>
        public static uint MASK { get { return _MASK; } set { _MASK = value; } }

        /// <summary>
        /// SDIO data FIFO register,Address offset: 0x80 
        /// </summary>
        public static uint FIFO { get { return _FIFO; } set { _FIFO = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// SDIO power control register,Address offset: 0x00 
        /// </summary>
        public static Register POWER_Register;

        /// <summary>
        /// SDI clock control register, Address offset: 0x04 
        /// </summary>
        public static Register CLKCR_Register;

        /// <summary>
        /// SDIO argument register, Address offset: 0x08 
        /// </summary>
        public static Register ARG_Register;

        /// <summary>
        /// SDIO command register,Address offset: 0x0C 
        /// </summary>
        public static Register CMD_Register;

        /// <summary>
        /// SDIO data timer register, Address offset: 0x24 
        /// </summary>
        public static Register DTIMER_Register;

        /// <summary>
        /// SDIO data length register,Address offset: 0x28 
        /// </summary>
        public static Register DLEN_Register;

        /// <summary>
        /// SDIO data control register, Address offset: 0x2C 
        /// </summary>
        public static Register DCTRL_Register;

        /// <summary>
        /// SDIO interrupt clear register,Address offset: 0x38 
        /// </summary>
        public static Register ICR_Register;

        /// <summary>
        /// SDIO mask register, Address offset: 0x3C 
        /// </summary>
        public static Register MASK_Register;

        /// <summary>
        /// SDIO data FIFO register,Address offset: 0x80 
        /// </summary>
        public static Register FIFO_Register;

        #endregion

        static SDIO()
        {
            POWER_Register = new Register(STM32F4xx.SDIO_BASE + POWER);
            CLKCR_Register = new Register(STM32F4xx.SDIO_BASE + CLKCR);
            ARG_Register = new Register(STM32F4xx.SDIO_BASE + ARG);
            CMD_Register = new Register(STM32F4xx.SDIO_BASE + CMD);
            DTIMER_Register = new Register(STM32F4xx.SDIO_BASE + DTIMER);
            DLEN_Register = new Register(STM32F4xx.SDIO_BASE + DLEN);
            DCTRL_Register = new Register(STM32F4xx.SDIO_BASE + DCTRL);
            ICR_Register = new Register(STM32F4xx.SDIO_BASE + ICR);
            MASK_Register = new Register(STM32F4xx.SDIO_BASE + MASK);
            FIFO_Register = new Register(STM32F4xx.SDIO_BASE + FIFO);
        }
    } // END of Class...

    /// <summary>
    /// System configuration controller (SYSCFG) - See: P.286
    /// </summary>
    public static class SYSCFG
    {
        #region Private's...

        /// <summary>
        /// SYSCFG memory remap register,Address offset: 0x00
        /// </summary>
        private static uint _MEMRMP = 0x00;

        /// <summary>
        /// SYSCFG peripheral mode configuration register, Address offset: 0x04
        /// </summary>
        private static uint _PMC = 0x04;

        /// <summary>
        /// SYSCFG external interrupt configuration registers, Address offset: 0x08-0x14 
        /// </summary>
        // private static uint _EXTICR[4] =  0x08-0x14 ;

        /// <summary>
        /// SYSCFG Compensation cell control register, Address offset: 0x20
        /// </summary>
        private static uint _CMPCR = 0x20;

        #endregion

        #region Properties...

        /// <summary>
        /// SYSCFG memory remap register,Address offset: 0x00
        /// </summary>
        public static uint MEMRMP { get { return _MEMRMP; } set { _MEMRMP = value; } }

        /// <summary>
        /// SYSCFG peripheral mode configuration register, Address offset: 0x04
        /// </summary>
        public static uint PMC { get { return _PMC; } set { _PMC = value; } }

        /// <summary>
        /// SYSCFG external interrupt configuration registers, Address offset: 0x08-0x14 
        /// </summary>
        // public static uint EXTICR[4]  { get { return _EXTICR[4]; } set { _EXTICR[4] = value; } }

        /// <summary>
        /// SYSCFG Compensation cell control register, Address offset: 0x20
        /// </summary>
        public static uint CMPCR { get { return _CMPCR; } set { _CMPCR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// SYSCFG memory remap register,Address offset: 0x00
        /// </summary>
        public static Register MEMRMP_Register;

        /// <summary>
        /// SYSCFG peripheral mode configuration register, Address offset: 0x04
        /// </summary>
        public static Register PMC_Register;

        /// <summary>
        /// SYSCFG external interrupt configuration registers, Address offset: 0x08-0x14 
        /// </summary>
        // public static Register EXTICR[4]_Register;

        /// <summary>
        /// SYSCFG Compensation cell control register, Address offset: 0x20
        /// </summary>
        public static Register CMPCR_Register;

        #endregion

        static SYSCFG()
        {
            MEMRMP_Register = new Register(STM32F4xx.SYSCFG_BASE + MEMRMP);
            PMC_Register = new Register(STM32F4xx.SYSCFG_BASE + PMC);
            // EXTICR[4]_Register = new Register(STM32F4xx.SYSCFG_BASE + EXTICR[4]);
            CMPCR_Register = new Register(STM32F4xx.SYSCFG_BASE + CMPCR);
        }
    } // END of Class...

    /// <summary>
    /// Interrupts and events - External interrupt/event controller (EXTI) - See: P.368
    /// </summary>
    public static class EXTI
    {
        #region Private's...

        /// <summary>
        /// EXTI Interrupt mask register,Address offset: 0x00 
        /// </summary>
        private static uint _IMR = 0x00;

        /// <summary>
        /// EXTI Event mask register,Address offset: 0x04 
        /// </summary>
        private static uint _EMR = 0x04;

        /// <summary>
        /// EXTI Rising trigger selection register,Address offset: 0x08 
        /// </summary>
        private static uint _RTSR = 0x08;

        /// <summary>
        /// EXTI Falling trigger selection register, Address offset: 0x0C 
        /// </summary>
        private static uint _FTSR = 0x0C;

        /// <summary>
        /// EXTI Software interrupt event register,Address offset: 0x10 
        /// </summary>
        private static uint _SWIER = 0x10;

        /// <summary>
        /// EXTI Pending register, Address offset: 0x14 
        /// </summary>
        private static uint _PR = 0x14;

        #endregion

        #region Properties...

        /// <summary>
        /// EXTI Interrupt mask register,Address offset: 0x00 
        /// </summary>
        public static uint IMR { get { return _IMR; } set { _IMR = value; } }

        /// <summary>
        /// EXTI Event mask register,Address offset: 0x04 
        /// </summary>
        public static uint EMR { get { return _EMR; } set { _EMR = value; } }

        /// <summary>
        /// EXTI Rising trigger selection register,Address offset: 0x08 
        /// </summary>
        public static uint RTSR { get { return _RTSR; } set { _RTSR = value; } }

        /// <summary>
        /// EXTI Falling trigger selection register, Address offset: 0x0C 
        /// </summary>
        public static uint FTSR { get { return _FTSR; } set { _FTSR = value; } }

        /// <summary>
        /// EXTI Software interrupt event register,Address offset: 0x10 
        /// </summary>
        public static uint SWIER { get { return _SWIER; } set { _SWIER = value; } }

        /// <summary>
        /// EXTI Pending register, Address offset: 0x14 
        /// </summary>
        public static uint PR { get { return _PR; } set { _PR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// EXTI Interrupt mask register,Address offset: 0x00 
        /// </summary>
        public static Register IMR_Register;

        /// <summary>
        /// EXTI Event mask register,Address offset: 0x04 
        /// </summary>
        public static Register EMR_Register;

        /// <summary>
        /// EXTI Rising trigger selection register,Address offset: 0x08 
        /// </summary>
        public static Register RTSR_Register;

        /// <summary>
        /// EXTI Falling trigger selection register, Address offset: 0x0C 
        /// </summary>
        public static Register FTSR_Register;

        /// <summary>
        /// EXTI Software interrupt event register,Address offset: 0x10 
        /// </summary>
        public static Register SWIER_Register;

        /// <summary>
        /// EXTI Pending register, Address offset: 0x14 
        /// </summary>
        public static Register PR_Register;

        #endregion

        static EXTI()
        {
            IMR_Register = new Register(STM32F4xx.EXTI_BASE + IMR);
            EMR_Register = new Register(STM32F4xx.EXTI_BASE + EMR);
            RTSR_Register = new Register(STM32F4xx.EXTI_BASE + RTSR);
            FTSR_Register = new Register(STM32F4xx.EXTI_BASE + FTSR);
            SWIER_Register = new Register(STM32F4xx.EXTI_BASE + SWIER);
            PR_Register = new Register(STM32F4xx.EXTI_BASE + PR);
        }
    } // END of Class...

    /// <summary>
    /// General-purpose I/Os (GPIO) - See: P.265
    /// </summary>
    public sealed class GPIO
    {
        #region Private's...

        private UInt32 _MODER = 0x00;
        private UInt32 _OTYPER = 0x04;
        private UInt32 _OSPEEDR = 0x08;
        private UInt32 _PUPDR = 0x0C;
        private UInt32 _IDR = 0x10;
        private UInt32 _ODR = 0x14;
        private UInt32 _BSRRL = 0x18;
        private UInt32 _BSRRH = 0x1A;
        private UInt32 _LCKR = 0x1C;

        // Alternate Function Number 
        private UInt32[] AFR = new UInt32[] { 0x12, 0x12, 0x22, 0x22, 0x22, 0, 0, 0x32, 0x32, 0x32, 0x32, 0x92, 0x92, 0x92 }; // AF1/2/3/9}

        #endregion

        #region Properties...

        /// <summary>
        /// GPIO port mode register, Address offset: 0x00
        /// </summary>
        public UInt32 MODER { get { return _MODER; } set { _MODER = value; } }

        /// <summary>
        /// GPIO port output type register, Address offset: 0x04
        /// </summary>
        public UInt32 OTYPER { get { return _OTYPER; } set { _OTYPER = value; } }

        /// <summary>
        /// GPIO port output speed register, Address offset: 0x08
        /// </summary>
        public UInt32 OSPEEDR { get { return _OSPEEDR; } set { _OSPEEDR = value; } }

        /// <summary>
        /// GPIO port pull-up/pull-down register, Address offset: 0x0C
        /// </summary>
        public UInt32 PUPDR { get { return _PUPDR; } set { _PUPDR = value; } }

        /// <summary>
        /// GPIO port input data register, Address offset: 0x10 
        /// </summary>
        public UInt32 IDR { get { return _IDR; } set { _IDR = value; } }

        /// <summary>
        /// GPIO port output data register, Address offset: 0x14
        /// </summary>
        public UInt32 ODR { get { return _ODR; } set { _ODR = value; } }

        /// <summary>
        /// GPIO port bit set/reset low register, Address offset: 0x18
        /// </summary>
        public UInt32 BSRRL { get { return _BSRRL; } set { _BSRRL = value; } }

        /// <summary>
        /// GPIO port bit set/reset high register, Address offset: 0x1A
        /// </summary>
        public UInt32 BSRRH { get { return _BSRRH; } set { _BSRRH = value; } }

        /// <summary>
        /// GPIO port configuration lock register, Address offset: 0x1C
        /// </summary>
        public UInt32 LCKR { get { return _LCKR; } set { _LCKR = value; } }

        /// <summary>
        /// GPIO alternate function registers, Address offset: 0x20-0x24
        /// </summary>
        // public UInt32 AFR { get { return _AFR; } set { _AFR = value; } }    

        #endregion

        #region Registers...

        /// <summary>
        /// GPIO port mode register, Address offset: 0x00
        /// </summary>
        public Register MODER_Register;

        /// <summary>
        /// GPIO port output type register, Address offset: 0x04
        /// </summary>
        public Register OTYPER_Register;

        /// <summary>
        /// GPIO port output speed register, Address offset: 0x08
        /// </summary>
        public Register OSPEEDR_Register;

        /// <summary>
        /// GPIO port pull-up/pull-down register, Address offset: 0x0C
        /// </summary>
        public Register PUPDR_Register;

        /// <summary>
        /// GPIO port input data register, Address offset: 0x10 
        /// </summary>
        public Register IDR_Register;

        /// <summary>
        /// GPIO port output data register, Address offset: 0x14
        /// </summary>
        public Register ODR_Register;

        /// <summary>
        /// GPIO port bit set/reset low register, Address offset: 0x18
        /// </summary>
        public Register BSRRL_Register;

        /// <summary>
        /// GPIO port bit set/reset high register, Address offset: 0x1A
        /// </summary>
        public Register BSRRH_Register;

        /// <summary>
        /// GPIO port configuration lock register, Address offset: 0x1C
        /// </summary>
        public Register LCKR_Register;

        /// <summary>
        /// GPIO alternate function registers, Address offset: 0x20-0x24
        /// </summary>
        // public Register AFR[2];    

        #endregion

        /// <summary>
        /// GPIOA. Class Type for defining the GPIOA Register Types...
        /// </summary>
        public static GPIO GPIOA = new GPIO(STM32F4xx.GPIOA_BASE);

        /// <summary>
        /// GPIOB. Class Type for defining the GPIOB Register Types...
        /// </summary>
        public static GPIO GPIOB = new GPIO(STM32F4xx.GPIOB_BASE);

        /// <summary>
        /// GPIOB. Class Type for defining the GPIOB Register Types...
        /// </summary>
        public static GPIO GPIOC = new GPIO(STM32F4xx.GPIOC_BASE);

        /// <summary>
        /// GPIOB. Class Type for defining the GPIOB Register Types...
        /// </summary>
        public static GPIO GPIOD = new GPIO(STM32F4xx.GPIOD_BASE);

        /// <summary>
        /// GPIOB. Class Type for defining the GPIOB Register Types...
        /// </summary>
        public static GPIO GPIOE = new GPIO(STM32F4xx.GPIOE_BASE);

        /// <summary>
        /// GPIOB. Class Type for defining the GPIOB Register Types...
        /// </summary>
        public static GPIO GPIOF = new GPIO(STM32F4xx.GPIOF_BASE);

        /// <summary>
        /// GPIOB. Class Type for defining the GPIOB Register Types...
        /// </summary>
        public static GPIO GPIOH = new GPIO(STM32F4xx.GPIOG_BASE);

        /// <summary>
        /// GPIOB. Class Type for defining the GPIOB Register Types...
        /// </summary>
        public static GPIO GPIOH = new GPIO(STM32F4xx.GPIOH_BASE);

        /// <summary>
        /// GPIOB. Class Type for defining the GPIOB Register Types...
        /// </summary>
        public static GPIO GPIOI = new GPIO(STM32F4xx.GPIOI_BASE);


        public GPIO(UInt32 GPIOx_BASE)
        {
            MODER_Register = new Register(GPIOx_BASE + MODER);
            OTYPER_Register = new Register(GPIOx_BASE + OTYPER);
            OSPEEDR_Register = new Register(GPIOx_BASE + OSPEEDR);
            PUPDR_Register = new Register(GPIOx_BASE + PUPDR);
            IDR_Register = new Register(GPIOx_BASE + IDR);
            ODR_Register = new Register(GPIOx_BASE + ODR);
            BSRRL_Register = new Register(GPIOx_BASE + BSRRL);
            BSRRH_Register = new Register(GPIOx_BASE + BSRRH);
            LCKR_Register = new Register(GPIOx_BASE + LCKR);
        }
    } // END of Class...

    /// <summary>
    /// CRC calculation unit - See: P.112
    /// </summary>
    public static class CRC
    {
        #region Private's...

        /// <summary>
        /// CRC Data register, Address offset: 0x00 
        /// </summary>
        private static uint _DR = 0x00;

        /// <summary>
        /// CRC Control register,Address offset: 0x08 
        /// </summary>
        private static uint _CR = 0x08;

        #endregion

        #region Properties...

        /// <summary>
        /// CRC Data register, Address offset: 0x00 
        /// </summary>
        public static uint DR { get { return _DR; } set { _DR = value; } }

        /// <summary>
        /// CRC Control register,Address offset: 0x08 
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// CRC Data register, Address offset: 0x00 
        /// </summary>
        public static Register DR_Register;

        /// <summary>
        /// CRC Control register,Address offset: 0x08 
        /// </summary>
        public static Register CR_Register;

        #endregion

        static CRC()
        {
            DR_Register = new Register(STM32F4xx.CRC_BASE + DR);
            CR_Register = new Register(STM32F4xx.CRC_BASE + CR);
        }
    } // END of Class...

    /// <summary>
    /// Reset and clock control for STM32F42xxx and STM32F43xxx (RCC) - See: P.147
    /// </summary>
    public static class RCC
    {
        #region Private's...

        private static uint _CR = 0x00;
        private static uint _PLLCFGR = 0x04;
        private static uint _CFGR = 0x08;
        private static uint _CIR = 0x0C;
        private static uint _AHB1RSTR = 0x10;
        private static uint _AHB2RSTR = 0x14;
        private static uint _AHB3RSTR = 0x18;
        //private uint _RESERVED0 = 0x1C;
        private static uint _APB1RSTR = 0x20;
        private static uint _APB2RSTR = 0x24;
        //private uint _RESERVED1 = 0x28 - 0x2C;
        private static uint _AHB1ENR = 0x30;
        private static uint _AHB2ENR = 0x34;
        private static uint _AHB3ENR = 0x38;
        //private uint _RESERVED2 = 0x3C;
        private static uint _APB1ENR = 0x40;
        private static uint _APB2ENR = 0x44;
        //private uint _RESERVED3 = 0x48 - 0x4C;
        private static uint _AHB1LPENR = 0x50;
        private static uint _AHB2LPENR = 0x54;
        private static uint _AHB3LPENR = 0x58;
        private static uint _RESERVED4 = 0x5C;
        private static uint _APB1LPENR = 0x60;
        private static uint _APB2LPENR = 0x64;
        //private uint _RESERVED5 = 0x68 - 0x6C;
        private static uint _BDCR = 0x70;
        private static uint _CSR = 0x74;
        //private uint _RESERVED6 = 0x78 - 0x7C;
        private static uint _SSCGR = 0x80;
        private static uint _PLLI2SCFGR = 0x84;

        #endregion

        #region Properties...

        /// <summary>
        /// RCC clock control register, Address offset: 0x00
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// RCC PLL configuration register, Address offset: 0x04
        /// </summary>
        public static uint PLLCFGR { get { return _PLLCFGR; } set { _PLLCFGR = value; } }

        /// <summary>
        /// RCC clock configuration register, Address offset: 0x08
        /// </summary>
        public static uint CFGR { get { return _CFGR; } set { _CFGR = value; } }

        /// <summary>
        /// RCC clock interrupt register, Address offset: 0x0C
        /// </summary>
        public static uint CIR { get { return _CIR; } set { _CIR = value; } }

        /// <summary>
        /// RCC AHB1 peripheral reset register, Address offset: 0x10
        /// </summary>
        public static uint AHB1RSTR { get { return _AHB1RSTR; } set { _AHB1RSTR = value; } }

        /// <summary>
        /// RCC AHB2 peripheral reset register, Address offset: 0x14
        /// </summary>
        public static uint AHB2RSTR { get { return _AHB2RSTR; } set { _AHB2RSTR = value; } }

        /// <summary>
        /// RCC AHB3 peripheral reset register, Address offset: 0x18
        /// </summary>
        public static uint AHB3RSTR { get { return _AHB3RSTR; } set { _AHB3RSTR = value; } }

        /// <summary>
        /// Reserved, 0x1C
        /// </summary>
        // public uint RESERVED0 { get { return _RESERVED0; } set { _RESERVED0 = value; } }       

        /// <summary>
        /// RCC APB1 peripheral reset register, Address offset: 0x20
        /// </summary>
        public static uint APB1RSTR { get { return _APB1RSTR; } set { _APB1RSTR = value; } }

        /// <summary>
        /// RCC APB2 peripheral reset register, Address offset: 0x24
        /// </summary>
        public static uint APB2RSTR { get { return _APB2RSTR; } set { _APB2RSTR = value; } }

        //public uint RESERVED1 { get { return _RESERVED1; } set { _RESERVED1 = value; } }  Reserved, 0x28-0x2C

        /// <summary>
        /// RCC AHB1 peripheral clock register, Address offset: 0x30
        /// </summary>
        public static uint AHB1ENR { get { return _AHB1ENR; } set { _AHB1ENR = value; } }

        /// <summary>
        /// RCC AHB2 peripheral clock register, Address offset: 0x34
        /// </summary>
        public static uint AHB2ENR { get { return _AHB2ENR; } set { _AHB2ENR = value; } }

        /// <summary>
        /// RCC AHB3 peripheral clock register, Address offset: 0x38
        /// </summary>
        public static uint AHB3ENR { get { return _AHB3ENR; } set { _AHB3ENR = value; } }

        /// <summary>
        ///  Reserved, 0x3C
        /// </summary>
        // public uint RESERVED2 { get { return _RESERVED2; } set { _RESERVED2 = value; } }          

        /// <summary>
        /// RCC APB1 peripheral clock enable register, Address offset: 0x40
        /// </summary>
        public static uint APB1ENR { get { return _APB1ENR; } set { _APB1ENR = value; } }

        /// <summary>
        /// RCC APB2 peripheral clock enable register, Address offset: 0x44
        /// </summary>
        public static uint APB2ENR { get { return _APB2ENR; } set { _APB2ENR = value; } }

        /// <summary>
        /// Reserved, 0x48-0x4C
        /// </summary>
        //public uint RESERVED3 { get { return _RESERVED3; } set { _RESERVED3 = value; } }   

        /// <summary>
        /// RCC AHB1 peripheral clock enable in low power mode register, Address offset: 0x50
        /// </summary>
        public static uint AHB1LPENR { get { return _AHB1LPENR; } set { _AHB1LPENR = value; } }

        /// <summary>
        /// RCC AHB2 peripheral clock enable in low power mode register, Address offset: 0x54
        /// </summary>
        public static uint AHB2LPENR { get { return _AHB2LPENR; } set { _AHB2LPENR = value; } }

        /// <summary>
        /// RCC AHB3 peripheral clock enable in low power mode register, Address offset: 0x58
        /// </summary>
        public static uint AHB3LPENR { get { return _AHB3LPENR; } set { _AHB3LPENR = value; } }

        /// <summary>
        /// Reserved, 0x5C
        /// </summary>
        // public uint RESERVED4 { get { return _RESERVED4; } set { _RESERVED4 = value; } }             

        /// <summary>
        /// RCC APB1 peripheral clock enable in low power mode register, Address offset: 0x60
        /// </summary>
        public static uint APB1LPENR { get { return _APB1LPENR; } set { _APB1LPENR = value; } }

        /// <summary>
        /// RCC APB2 peripheral clock enable in low power mode register, Address offset: 0x64
        /// </summary>
        public static uint APB2LPENR { get { return _APB2LPENR; } set { _APB2LPENR = value; } }

        /// <summary>
        /// Reserved, 0x68-0x6C
        /// </summary>
        //public uint RESERVED5 { get { return _RESERVED5; } set { _RESERVED5 = value; } }           

        /// <summary>
        /// RCC Backup domain control register, Address offset: 0x70 
        /// </summary>
        public static uint BDCR { get { return _BDCR; } set { _BDCR = value; } }

        /// <summary>
        /// RCC clock control & status register, Address offset: 0x74
        /// </summary>
        public static uint CSR { get { return _CSR; } set { _CSR = value; } }

        /// <summary>
        /// Reserved, 0x78-0x7C 
        /// </summary>
        // public uint RESERVED6 { get { return _RESERVED6; } set { _RESERVED6 = value; } }            

        /// <summary>
        /// RCC spread spectrum clock generation register, Address offset: 0x80
        /// </summary>
        public static uint SSCGR { get { return _SSCGR; } set { _SSCGR = value; } }

        /// <summary>
        /// RCC PLLI2S configuration register, Address offset: 0x84
        /// </summary>
        public static uint PLLI2SCFGR { get { return _PLLI2SCFGR; } set { _PLLI2SCFGR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// RCC clock control register, Address offset: 0x00
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// RCC PLL configuration register, Address offset: 0x04
        /// </summary>
        public static Register PLLCFGR_Register;

        /// <summary>
        /// RCC clock configuration register, Address offset: 0x08
        /// </summary>
        public static Register CFGR_Register;

        /// <summary>
        /// RCC clock interrupt register, Address offset: 0x0C
        /// </summary>
        public static Register CIR_Register;

        /// <summary>
        /// RCC AHB1 peripheral reset register, Address offset: 0x10
        /// </summary>
        public static Register AHB1RSTR_Register;

        /// <summary>
        /// RCC AHB2 peripheral reset register, Address offset: 0x14
        /// </summary>
        public static Register AHB2RSTR_Register;

        /// <summary>
        /// RCC AHB3 peripheral reset register, Address offset: 0x18
        /// </summary>
        public static Register AHB3RSTR_Register;

        /// <summary>
        /// RCC APB1 peripheral reset register, Address offset: 0x20
        /// </summary>
        public static Register APB1RSTR_Register;

        /// <summary>
        /// RCC APB2 peripheral reset register, Address offset: 0x24
        /// </summary>
        public static Register APB2RSTR_Register;

        /// <summary>
        /// RCC AHB1 peripheral clock register, Address offset: 0x30
        /// </summary>
        public static Register AHB1ENR_Register;

        /// <summary>
        /// RCC AHB2 peripheral clock register, Address offset: 0x34
        /// </summary>
        public static Register AHB2ENR_Register;

        /// <summary>
        /// RCC AHB3 peripheral clock register, Address offset: 0x38
        /// </summary>
        public static Register AHB3ENR_Register;

        /// <summary>
        /// RCC APB1 peripheral clock enable register, Address offset: 0x40
        /// </summary>
        public static Register APB1ENR_Register;

        /// <summary>
        /// RCC APB2 peripheral clock enable register, Address offset: 0x44
        /// </summary>
        public static Register APB2ENR_Register;

        /// <summary>
        /// RCC AHB1 peripheral clock enable in low power mode register, Address offset: 0x50
        /// </summary>
        public static Register AHB1LPENR_Register;

        /// <summary>
        /// RCC AHB2 peripheral clock enable in low power mode register, Address offset: 0x54
        /// </summary>
        public static Register AHB2LPENR_Register;

        /// <summary>
        /// RCC AHB3 peripheral clock enable in low power mode register, Address offset: 0x58
        /// </summary>
        public static Register AHB3LPENR_Register;

        /// <summary>
        /// RCC APB1 peripheral clock enable in low power mode register, Address offset: 0x60
        /// </summary>
        public static Register APB1LPENR_Register;

        /// <summary>
        /// RCC APB2 peripheral clock enable in low power mode register, Address offset: 0x64
        /// </summary>
        public static Register APB2LPENR_Register;

        /// <summary>
        /// RCC Backup domain control register, Address offset: 0x70
        /// </summary>
        public static Register BDCR_Register;

        /// <summary>
        /// RCC clock control and status register, Address offset: 0x74
        /// </summary>
        public static Register CSR_Register;

        /// <summary>
        /// RCC spread spectrum clock generation register, Address offset: 0x80
        /// </summary>
        public static Register SSCGR_Register;

        /// <summary>
        /// RCC PLLI2S configuration register, Address offset: 0x84
        /// </summary>
        public static Register PLLI2SCFGR_Register;

        #endregion

        /// <summary>
        /// Reset and Counter Control. Class Type for defining the Reset and Counter Control Registers...
        /// P.159 for RCC registers...
        /// </summary>

        static RCC()
        {
            CR_Register = new Register(STM32F4xx.RCC_BASE + CR);
            PLLCFGR_Register = new Register(STM32F4xx.RCC_BASE + PLLCFGR);
            CFGR_Register = new Register(STM32F4xx.RCC_BASE + CFGR);
            CIR_Register = new Register(STM32F4xx.RCC_BASE + CIR);
            AHB1RSTR_Register = new Register(STM32F4xx.RCC_BASE + AHB1RSTR);
            AHB2RSTR_Register = new Register(STM32F4xx.RCC_BASE + AHB2RSTR);
            AHB3RSTR_Register = new Register(STM32F4xx.RCC_BASE + AHB3RSTR);
            APB1RSTR_Register = new Register(STM32F4xx.RCC_BASE + APB1RSTR);
            APB2RSTR_Register = new Register(STM32F4xx.RCC_BASE + APB2RSTR);
            AHB1ENR_Register = new Register(STM32F4xx.RCC_BASE + AHB1ENR);
            AHB2ENR_Register = new Register(STM32F4xx.RCC_BASE + AHB2ENR);
            AHB3ENR_Register = new Register(STM32F4xx.RCC_BASE + AHB3ENR);
            APB1ENR_Register = new Register(STM32F4xx.RCC_BASE + APB1ENR);
            APB2ENR_Register = new Register(STM32F4xx.RCC_BASE + APB2ENR);
            AHB1LPENR_Register = new Register(STM32F4xx.RCC_BASE + AHB1LPENR);
            AHB2LPENR_Register = new Register(STM32F4xx.RCC_BASE + AHB2LPENR);
            AHB3LPENR_Register = new Register(STM32F4xx.RCC_BASE + AHB3LPENR);
            APB1LPENR_Register = new Register(STM32F4xx.RCC_BASE + APB1LPENR);
            APB2LPENR_Register = new Register(STM32F4xx.RCC_BASE + APB2LPENR);
            BDCR_Register = new Register(STM32F4xx.RCC_BASE + BDCR);
            CSR_Register = new Register(STM32F4xx.RCC_BASE + CSR);
            SSCGR_Register = new Register(STM32F4xx.RCC_BASE + SSCGR);
            PLLI2SCFGR_Register = new Register(STM32F4xx.RCC_BASE + PLLI2SCFGR);
        }

    } // END of Class...

    /// <summary>
    /// Embedded Flash memory in STM32F42xxx and STM32F43xxx - See: P.76
    /// </summary>
    public static class FLASH
    {
        #region Private's...

        /// <summary>
        /// FLASH access control register, Address offset: 0x00 
        /// </summary>
        private static uint _ACR = 0x00;

        /// <summary>
        /// FLASH key register,Address offset: 0x04 
        /// </summary>
        private static uint _KEYR = 0x04;

        /// <summary>
        /// FLASH option key register, Address offset: 0x08 
        /// </summary>
        private static uint _OPTKEYR = 0x08;

        /// <summary>
        /// FLASH status register, Address offset: 0x0C 
        /// </summary>
        private static uint _SR = 0x0C;

        /// <summary>
        /// FLASH control register,Address offset: 0x10 
        /// </summary>
        private static uint _CR = 0x10;

        /// <summary>
        /// FLASH option control register, Address offset: 0x14 
        /// </summary>
        private static uint _OPTCR = 0x14;

        #endregion

        #region Properties...

        /// <summary>
        /// FLASH access control register, Address offset: 0x00 
        /// </summary>
        public static uint ACR { get { return _ACR; } set { _ACR = value; } }

        /// <summary>
        /// FLASH key register,Address offset: 0x04 
        /// </summary>
        public static uint KEYR { get { return _KEYR; } set { _KEYR = value; } }

        /// <summary>
        /// FLASH option key register, Address offset: 0x08 
        /// </summary>
        public static uint OPTKEYR { get { return _OPTKEYR; } set { _OPTKEYR = value; } }

        /// <summary>
        /// FLASH status register, Address offset: 0x0C 
        /// </summary>
        public static uint SR { get { return _SR; } set { _SR = value; } }

        /// <summary>
        /// FLASH control register,Address offset: 0x10 
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// FLASH option control register, Address offset: 0x14 
        /// </summary>
        public static uint OPTCR { get { return _OPTCR; } set { _OPTCR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// FLASH access control register, Address offset: 0x00 
        /// </summary>
        public static Register ACR_Register;

        /// <summary>
        /// FLASH key register,Address offset: 0x04 
        /// </summary>
        public static Register KEYR_Register;

        /// <summary>
        /// FLASH option key register, Address offset: 0x08 
        /// </summary>
        public static Register OPTKEYR_Register;

        /// <summary>
        /// FLASH status register, Address offset: 0x0C 
        /// </summary>
        public static Register SR_Register;

        /// <summary>
        /// FLASH control register,Address offset: 0x10 
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// FLASH option control register, Address offset: 0x14 
        /// </summary>
        public static Register OPTCR_Register;

        #endregion

        static FLASH()
        {
            ACR_Register = new Register(STM32F4xx.FLASH_BASE + ACR);
            KEYR_Register = new Register(STM32F4xx.FLASH_BASE + KEYR);
            OPTKEYR_Register = new Register(STM32F4xx.FLASH_BASE + OPTKEYR);
            SR_Register = new Register(STM32F4xx.FLASH_BASE + SR);
            CR_Register = new Register(STM32F4xx.FLASH_BASE + CR);
            OPTCR_Register = new Register(STM32F4xx.FLASH_BASE + OPTCR);
        }
    } // END of Class...

    /// <summary>
    /// DMA registers - See: P.321
    /// </summary>
    public sealed class DMA
    {
        #region Private's...

        /// <summary>
        /// DMA low interrupt status register,Address offset: 0x00 
        /// </summary>
        private uint _LISR = 0x00;

        /// <summary>
        /// DMA high interrupt status register, Address offset: 0x04 
        /// </summary>
        private uint _HISR = 0x04;

        /// <summary>
        /// DMA low interrupt flag clear register,Address offset: 0x08 
        /// </summary>
        private uint _LIFCR = 0x08;

        /// <summary>
        /// DMA high interrupt flag clear register, Address offset: 0x0C 
        /// </summary>
        private uint _HIFCR = 0x0C;

        #endregion

        #region Properties...

        /// <summary>
        /// DMA low interrupt status register,Address offset: 0x00 
        /// </summary>
        public uint LISR { get { return _LISR; } set { _LISR = value; } }

        /// <summary>
        /// DMA high interrupt status register, Address offset: 0x04 
        /// </summary>
        public uint HISR { get { return _HISR; } set { _HISR = value; } }

        /// <summary>
        /// DMA low interrupt flag clear register,Address offset: 0x08 
        /// </summary>
        public uint LIFCR { get { return _LIFCR; } set { _LIFCR = value; } }

        /// <summary>
        /// DMA high interrupt flag clear register, Address offset: 0x0C 
        /// </summary>
        public uint HIFCR { get { return _HIFCR; } set { _HIFCR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// DMA low interrupt status register,Address offset: 0x00 
        /// </summary>
        public Register LISR_Register;

        /// <summary>
        /// DMA high interrupt status register, Address offset: 0x04 
        /// </summary>
        public Register HISR_Register;

        /// <summary>
        /// DMA low interrupt flag clear register,Address offset: 0x08 
        /// </summary>
        public Register LIFCR_Register;

        /// <summary>
        /// DMA high interrupt flag clear register, Address offset: 0x0C 
        /// </summary>
        public Register HIFCR_Register;

        #endregion

        public static DMA DMA1 = new DMA(STM32F4xx.DMA1_BASE);
        public static DMA DMA2 = new DMA(STM32F4xx.DMA2_BASE);

        public DMA(UInt32 XX_BASE)
        {
            LISR_Register = new Register(XX_BASE + LISR);
            HISR_Register = new Register(XX_BASE + HISR);
            LIFCR_Register = new Register(XX_BASE + LIFCR);
            HIFCR_Register = new Register(XX_BASE + HIFCR);
        }
    } // END of Class...

    /// <summary>
    /// DMA registers - See: P.321
    /// </summary>
    public sealed class DMA_Stream
    {
        #region Private's...

        /// <summary>
        /// Address offset: 0x10 + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        private uint _CR = (0x10 + 0x18);

        /// <summary>
        /// Address offset: 0x14 + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        private uint _NDTR = (0x14 + 0x18);

        /// <summary>
        /// Address offset: 0x18 + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        private uint _PAR = (0x18 + 0x18);

        /// <summary>
        /// Address offset: 0x1C + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        private uint _M0AR = (0x1C + 0x18);

        /// <summary>
        /// Address offset: 0x20 + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        private uint _M1AR = (0x20 + 0x18);

        /// <summary>
        /// Address offset: 0x24 + 0x24  stream number - Reset value: 0x0000 0021
        /// </summary>
        private uint _FCR = (0x24 + 0x24);

        #endregion

        #region Properties...

        /// <summary>
        /// Address offset: 0x10 + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        public uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// Address offset: 0x14 + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        public uint NDTR { get { return _NDTR; } set { _NDTR = value; } }

        /// <summary>
        /// Address offset: 0x18 + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        public uint PAR { get { return _PAR; } set { _PAR = value; } }

        /// <summary>
        /// Address offset: 0x1C + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        public uint M0AR { get { return _M0AR; } set { _M0AR = value; } }

        /// <summary>
        /// Address offset: 0x20 + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        public uint M1AR { get { return _M1AR; } set { _M1AR = value; } }

        /// <summary>
        /// Address offset: 0x24 + 0x24  stream number - Reset value: 0x0000 0021
        /// </summary>
        public uint FCR { get { return _FCR; } set { _FCR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// DMA stream x configuration register - Address offset: 0x10 + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        public Register CR_Register;

        /// <summary>
        /// DMA stream x number of data register - Address offset: 0x14 + 0x18  stream number - Reset value: 0x0000 0000 
        /// </summary>
        public Register NDTR_Register;

        /// <summary>
        /// DMA stream x peripheral address register - Address offset: 0x18 + 0x18  stream number - Reset value: 0x0000 0000 
        /// </summary>
        public Register PAR_Register;

        /// <summary>
        /// DMA stream x memory 0 address register - Address offset: 0x1C + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        public Register M0AR_Register;

        /// <summary>
        /// DMA stream x memory 1 address register - Address offset: 0x20 + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        public Register M1AR_Register;

        /// <summary>
        /// DMA stream x FIFO control register - Address offset: 0x24 + 0x24  stream number - Reset value: 0x0000 0021
        /// </summary>
        public Register FCR_Register;

        #endregion

        public static DMA_Stream DMA1_Stream0 = new DMA_Stream(STM32F4xx.DMA1_Stream0_BASE, 0);
        public static DMA_Stream DMA1_Stream1 = new DMA_Stream(STM32F4xx.DMA1_Stream1_BASE, 1);
        public static DMA_Stream DMA1_Stream2 = new DMA_Stream(STM32F4xx.DMA1_Stream2_BASE, 2);
        public static DMA_Stream DMA1_Stream3 = new DMA_Stream(STM32F4xx.DMA1_Stream3_BASE, 3);
        public static DMA_Stream DMA1_Stream4 = new DMA_Stream(STM32F4xx.DMA1_Stream4_BASE, 4);
        public static DMA_Stream DMA1_Stream5 = new DMA_Stream(STM32F4xx.DMA1_Stream5_BASE, 5);
        public static DMA_Stream DMA1_Stream6 = new DMA_Stream(STM32F4xx.DMA1_Stream6_BASE, 6);
        public static DMA_Stream DMA1_Stream7 = new DMA_Stream(STM32F4xx.DMA1_Stream7_BASE, 7);

        public static DMA_Stream DMA2_Stream0 = new DMA_Stream(STM32F4xx.DMA2_Stream0_BASE, 0);
        public static DMA_Stream DMA2_Stream1 = new DMA_Stream(STM32F4xx.DMA2_Stream0_BASE, 1);
        public static DMA_Stream DMA2_Stream2 = new DMA_Stream(STM32F4xx.DMA2_Stream1_BASE, 2);
        public static DMA_Stream DMA2_Stream3 = new DMA_Stream(STM32F4xx.DMA2_Stream3_BASE, 3);
        public static DMA_Stream DMA2_Stream4 = new DMA_Stream(STM32F4xx.DMA2_Stream4_BASE, 4);
        public static DMA_Stream DMA2_Stream5 = new DMA_Stream(STM32F4xx.DMA2_Stream5_BASE, 5);
        public static DMA_Stream DMA2_Stream6 = new DMA_Stream(STM32F4xx.DMA2_Stream6_BASE, 6);
        public static DMA_Stream DMA2_Stream7 = new DMA_Stream(STM32F4xx.DMA2_Stream7_BASE, 7);

        public DMA_Stream(UInt32 XX_BASE, uint streamnumber)
        {
            CR_Register = new Register(XX_BASE + (CR * streamnumber));
            NDTR_Register = new Register(XX_BASE + (NDTR * streamnumber));
            PAR_Register = new Register(XX_BASE + (PAR * streamnumber));
            M0AR_Register = new Register(XX_BASE + (M0AR * streamnumber));
            M1AR_Register = new Register(XX_BASE + (M1AR * streamnumber));
            FCR_Register = new Register(XX_BASE + (FCR * streamnumber));
        }
    } // END of Class...

    /// <summary>
    /// Ethernet (ETH): media access control (MAC) with DMA controller - See: P.1104
    /// </summary>
    public static class ETH
    {
        #region Private's...

        /// <summary>
        /// See: P.1173 - Ethernet MAC configuration register - Address offset: 0x0000 - Reset value: 0x0000 8000
        /// </summary>
        private static uint _MACCR = 0x0000;

        /// <summary>
        /// See: P.1177 - Ethernet MAC frame filter register - Address offset: 0x0004 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACFFR = 0x0004;

        /// <summary>
        /// See: P.1178 - Ethernet MAC hash table high register - Address offset: 0x0008 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACHTHR = 0x0008;

        /// <summary>
        /// See P.1179 Ethernet MAC hash table low register - Address offset: 0x000C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACHTLR = 0x000C;

        /// <summary>
        /// See: P.1179 - Ethernet MAC MII address register - Address offset: 0x0010 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACMIIAR = 0x0010;

        /// <summary>
        /// See: P.1180 - Ethernet MAC MII data register - Address offset: 0x0014 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACMIIDR = 0x0014;

        /// <summary>
        /// See: P.1181 - Ethernet MAC flow control register - Address offset: 0x0018 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACFCR = 0x0018;

        /// <summary>
        ///  See: P.1182 - Ethernet MAC VLAN tag register - Address offset: 0x001C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACVLANTR = 0x001C;

        /// <summary>
        /// See: P.1183 - Ethernet MAC remote wakeup frame filter register - Address offset: 0x0028 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACRWUFFR = 0x0028;

        /// <summary>
        /// See: P.1184 - Ethernet MAC PMT control and status register - Address offset: 0x002C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACPMTCSR = 0x002C;

        /// <1185>......

        /// <summary>
        /// See: P.1187 - Ethernet MAC interrupt status register - Address offset: 0x0038 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACSR = 0x0038;

        /// <summary>
        /// See: P.1188 - Ethernet MAC interrupt mask register - Address offset: 0x003C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACIMR = 0x003C;

        /// <summary>
        /// See: P.1188 - Ethernet MAC address 0 high register - Address offset: 0x0040 - Reset value: 0x8000 FFFF
        /// </summary>
        private static uint _MACA0HR = 0x0040;

        /// <summary>
        /// See: P.1189 - Ethernet MAC address 0 low register - Address offset: 0x0044 - Reset value: 0xFFFF FFFF
        /// </summary>
        private static uint _MACA0LR = 0x0044;

        /// <summary>
        /// See: P.1189 - Ethernet MAC address 1 high register - Address offset: 0x0048 - Reset value: 0x0000 FFFF
        /// </summary>
        private static uint _MACA1HR = 0x0048;

        /// <summary>
        /// See: P.1190 - Ethernet MAC address1 low register - Address offset: 0x004C - Reset value: 0xFFFF FFFF
        /// </summary>
        private static uint _MACA1LR = 0x004C;

        /// <summary>
        /// See: P.1190 - Ethernet MAC address 2 high register - Address offset: 0x0050 - Reset value: 0x0000 FFFF
        /// </summary>
        private static uint _MACA2HR = 0x0050;

        /// <summary>
        /// See: P.1191 - Ethernet MAC address 2 low register - Address offset: 0x0054 - Reset value: 0xFFFF FFFF
        /// </summary>
        private static uint _MACA2LR = 0x0054;

        /// <summary>
        /// See: P.1191 - Ethernet MAC address 3 high register - Address offset: 0x0058 - Reset value: 0x0000 FFFF
        /// </summary>
        private static uint _MACA3HR = 0x0058;

        /// <summary>
        /// See: P.1192 - Ethernet MAC address 3 low register - Address offset: 0x005C - Reset value: 0xFFFF FFFF
        /// </summary>
        private static uint _MACA3LR = 0x005C;

        /// <summary>
        /// See: P.1193 - Ethernet MMC control register - Address offset: 0x0100 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCCR = 0x0100;

        /// <summary>
        /// See: P.1193 - Ethernet MMC receive interrupt register - Address offset: 0x0104 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCRIR = 0x0104;

        /// <summary>
        /// See: P.1194 - Ethernet MMC transmit interrupt register - Address offset: 0x0108 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCTIR = 0x0108;

        /// <summary>
        /// See: P.1195 - Ethernet MMC receive interrupt mask register - Address offset: 0x010C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCRIMR = 0x010C;

        /// <summary>
        /// See: P.1195 - Ethernet MMC transmit interrupt mask register - Address offset: 0x0110 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCTIMR = 0x0110;

        /// <summary>
        /// See: P.1196 - Ethernet MMC transmitted good frames after a single collision counter register - Address offset: 0x014C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCTGFSCCR = 0x014C;

        /// <summary>
        /// See: P.1196 - Ethernet MMC transmitted good frames after more than a single collision counter register - Address offset: 0x0150 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCTGFMSCCR = 0x0150;

        /// <summary>
        /// See: P.1197 - Ethernet MMC transmitted good frames counter register - Address offset: 0x0168 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCTGFCR = 0x0168;

        /// <summary>
        /// See: P.1197 - Ethernet MMC received frames with CRC error counter register - Address offset: 0x0194 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCRFCECR = 0x0194;

        /// <summary>
        /// See: P.1197 - Ethernet MMC received frames with alignment error counter register - Address offset: 0x0198 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCRFAECR = 0x0198;

        /// <summary>
        /// See: P.1198 - MMC received good unicast frames counter register - Address offset: 0x01C4 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCRGUFCR = 0x01C4;

        /// <summary>
        /// See: P.1198 - Ethernet PTP time stamp control register - Address offset: 0x0700 - Reset value: 0x0000 00002000
        /// </summary>
        private static uint _PTPTSCR = 0x0700;

        /// <summary>
        /// See: P.1200 - Ethernet PTP subsecond increment register - Address offset: 0x0704 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _PTPSSIR = 0x0704;

        /// <summary>
        /// See: P.1201 - Ethernet PTP time stamp high register - Address offset: 0x0708 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _PTPTSHR = 0x0708;

        /// <summary>
        /// See: P.1201 - Ethernet PTP time stamp low register - Address offset: 0x070C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _PTPTSLR = 0x070C;

        /// <summary>
        /// See: P.1201 - Ethernet PTP time stamp high update register - Address offset: 0x0710 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _PTPTSHUR = 0x0710;

        /// <summary>
        /// See: P.1203 - Ethernet PTP time stamp low update register - Address offset: 0x0714 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _PTPTSLUR = 0x0714;

        /// <summary>
        /// See: P.1203 - Ethernet PTP time stamp addend register - Address offset: 0x0718 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _PTPTSAR = 0x0718;

        /// <summary>
        /// See: P.1204 - Ethernet PTP target time high register - Address offset: 0x071C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _PTPTTHR = 0x071C;

        /// <summary>
        /// See: P.1204 - Ethernet PTP target time low register - Address offset: 0x0720 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _PTPTTLR = 0x0720;

        /// <summary>
        /// See: P.1204 - Ethernet PTP time stamp status register - Address offset: 0x0728 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _PTPTSSR = 0x0728;

        /// <1205>

        /// <summary>
        /// See: P.1205 - Ethernet DMA bus mode register - Address offset: 0x1000 - Reset value: 0x0002 0101
        /// </summary>
        private static uint _DMABMR = 0x1000;

        /// <summary>
        /// See: P.1207 - Ethernet DMA transmit poll demand register - Address offset: 0x1004 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMATPDR = 0x1004;

        /// <summary>
        /// See: P.1208 - EHERNET DMA receive poll demand register - Address offset: 0x1008 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMARPDR = 0x1008;

        /// <summary>
        /// See: P.1208 - Ethernet DMA receive descriptor list address register - Address offset: 0x100C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMARDLAR = 0x100C;

        /// <summary>
        /// See: P.1209 - Ethernet DMA transmit descriptor list address register - Address offset: 0x1010 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMATDLAR = 0x1010;

        /// <summary>
        /// See: P.1209 - Ethernet DMA status register - Address offset: 0x1014 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMASR = 0x1014;

        /// <summary>
        /// See: P.1212 - Ethernet DMA operation mode register - Address offset: 0x1018 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMAOMR = 0x1018;

        /// <summary>
        /// See: P.1215 - Ethernet DMA interrupt enable register - Address offset: 0x101C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMAIER = 0x101C;

        /// <summary>
        /// See: P.1217 - Ethernet DMA missed frame and buffer overflow counter register - Address offset: 0x1020 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMAMFBOCR = 0x1020;

        /// <summary>
        /// See: P.1217 - Ethernet DMA receive status watchdog timer register - Address offset: 0x1024 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMARSWTR = 0x1024;

        /// <summary>
        /// See: P.1218 - Ethernet DMA current host transmit descriptor register - Address offset: 0x1048 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMACHTDR = 0x1048;

        /// <summary>
        /// See: P.1218 - Ethernet DMA current host receive descriptor register - Address offset: 0x104C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMACHRDR = 0x104C;

        /// <summary>
        /// See: P.1218 - Ethernet DMA current host transmit buffer address register - Address offset: 0x1050 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMACHTBAR = 0x1050;

        /// <summary>
        /// See: P.1219 - Ethernet DMA current host receive buffer address register - Address offset: 0x1054 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMACHRBAR = 0x1054;

        #endregion

        #region Properties...

        // <summary>
        /// See: P.1173 - Ethernet MAC configuration register - Address offset: 0x0000 - Reset value: 0x0000 8000
        /// </summary>
        private static uint MACCR { get { return _MACCR; } set { _MACCR = value; } }

        /// <summary>
        /// See: P.1177 - Ethernet MAC frame filter register - Address offset: 0x0004 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACFFR { get { return _MACFFR; } set { _MACFFR = value; } }

        /// <summary>
        /// See: P.1178 - Ethernet MAC hash table high register - Address offset: 0x0008 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACHTHR { get { return _MACHTHR; } set { _MACHTHR = value; } }

        /// <summary>
        /// See P.1179 Ethernet MAC hash table low register - Address offset: 0x000C - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACHTLR { get { return _MACHTLR; } set { _MACHTLR = value; } }

        /// <summary>
        /// See: P.1179 - Ethernet MAC MII address register - Address offset: 0x0010 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACMIIAR { get { return _MACMIIAR; } set { _MACMIIAR = value; } }

        /// <summary>
        /// See: P.1180 - Ethernet MAC MII data register - Address offset: 0x0014 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACMIIDR { get { return _MACMIIDR; } set { _MACMIIDR = value; } }

        /// <summary>
        /// See: P.1181 - Ethernet MAC flow control register - Address offset: 0x0018 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACFCR { get { return _MACFCR; } set { _MACFCR = value; } }

        /// <summary>
        ///  See: P.1182 - Ethernet MAC VLAN tag register - Address offset: 0x001C - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACVLANTR { get { return _MACVLANTR; } set { _MACVLANTR = value; } }

        /// <summary>
        /// See: P.1183 - Ethernet MAC remote wakeup frame filter register - Address offset: 0x0028 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACRWUFFR { get { return _MACRWUFFR; } set { _MACRWUFFR = value; } }

        /// <summary>
        /// See: P.1184 - Ethernet MAC PMT control and status register - Address offset: 0x002C - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACPMTCSR { get { return _MACPMTCSR; } set { _MACPMTCSR = value; } }

        /// <summary>
        /// See: P.1187 - Ethernet MAC interrupt status register - Address offset: 0x0038 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACSR { get { return _MACSR; } set { _MACSR = value; } }

        /// <summary>
        /// See: P.1188 - Ethernet MAC interrupt mask register - Address offset: 0x003C - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACIMR { get { return _MACIMR; } set { _MACIMR = value; } }

        /// <summary>
        /// See: P.1188 - Ethernet MAC address 0 high register - Address offset: 0x0040 - Reset value: 0x8000 FFFF
        /// </summary>
        private static uint MACA0HR { get { return _MACA0HR; } set { _MACA0HR = value; } }

        /// <summary>
        /// See: P.1189 - Ethernet MAC address 0 low register - Address offset: 0x0044 - Reset value: 0xFFFF FFFF
        /// </summary>
        private static uint MACA0LR { get { return _MACA0LR; } set { _MACA0LR = value; } }

        /// <summary>
        /// See: P.1189 - Ethernet MAC address 1 high register - Address offset: 0x0048 - Reset value: 0x0000 FFFF
        /// </summary>
        private static uint MACA1HR { get { return _MACA1HR; } set { _MACA1HR = value; } }

        /// <summary>
        /// See: P.1190 - Ethernet MAC address1 low register - Address offset: 0x004C - Reset value: 0xFFFF FFFF
        /// </summary>
        private static uint MACA1LR { get { return _MACA1LR; } set { _MACA1LR = value; } }

        /// <summary>
        /// See: P.1190 - Ethernet MAC address 2 high register - Address offset: 0x0050 - Reset value: 0x0000 FFFF
        /// </summary>
        private static uint MACA2HR { get { return _MACA2HR; } set { _MACA2HR = value; } }

        /// <summary>
        /// See: P.1191 - Ethernet MAC address 2 low register - Address offset: 0x0054 - Reset value: 0xFFFF FFFF
        /// </summary>
        private static uint MACA2LR { get { return _MACA2LR; } set { _MACA2LR = value; } }

        /// <summary>
        /// See: P.1191 - Ethernet MAC address 3 high register - Address offset: 0x0058 - Reset value: 0x0000 FFFF
        /// </summary>
        private static uint MACA3HR { get { return _MACA3HR; } set { _MACA3HR = value; } }

        /// <summary>
        /// See: P.1192 - Ethernet MAC address 3 low register - Address offset: 0x005C - Reset value: 0xFFFF FFFF
        /// </summary>
        private static uint MACA3LR { get { return _MACA3LR; } set { _MACA3LR = value; } }

        /// <summary>
        /// See: P.1193 - Ethernet MMC control register - Address offset: 0x0100 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCCR { get { return _MMCCR; } set { _MMCCR = value; } }

        /// <summary>
        /// See: P.1193 - Ethernet MMC receive interrupt register - Address offset: 0x0104 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCRIR { get { return _MMCRIR; } set { _MMCRIR = value; } }

        /// <summary>
        /// See: P.1194 - Ethernet MMC transmit interrupt register - Address offset: 0x0108 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCTIR { get { return _MMCTIR; } set { _MMCTIR = value; } }

        /// <summary>
        /// See: P.1195 - Ethernet MMC receive interrupt mask register - Address offset: 0x010C - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCRIMR { get { return _MMCRIMR; } set { _MMCRIMR = value; } }

        /// <summary>
        /// See: P.1195 - Ethernet MMC transmit interrupt mask register - Address offset: 0x0110 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCTIMR { get { return _MMCTIMR; } set { _MMCTIMR = value; } }

        /// <summary>
        /// See: P.1196 - Ethernet MMC transmitted good frames after a single collision counter register - Address offset: 0x014C - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCTGFSCCR { get { return _MMCTGFSCCR; } set { _MMCTGFSCCR = value; } }

        /// <summary>
        /// See: P.1196 - Ethernet MMC transmitted good frames after more than a single collision counter register - Address offset: 0x0150 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCTGFMSCCR { get { return _MMCTGFMSCCR; } set { _MMCTGFMSCCR = value; } }

        /// <summary>
        /// See: P.1197 - Ethernet MMC transmitted good frames counter register - Address offset: 0x0168 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCTGFCR { get { return _MMCTGFCR; } set { _MMCTGFCR = value; } }

        /// <summary>
        /// See: P.1197 - Ethernet MMC received frames with CRC error counter register - Address offset: 0x0194 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCRFCECR { get { return _MMCRFCECR; } set { _MMCRFCECR = value; } }

        /// <summary>
        /// See: P.1197 - Ethernet MMC received frames with alignment error counter register - Address offset: 0x0198 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCRFAECR { get { return _MMCRFAECR; } set { _MMCRFAECR = value; } }

        /// <summary>
        /// See: P.1198 - MMC received good unicast frames counter register - Address offset: 0x01C4 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCRGUFCR { get { return _MMCRGUFCR; } set { _MMCRGUFCR = value; } }

        /// <summary>
        /// See: P.1198 - Ethernet PTP time stamp control register - Address offset: 0x0700 - Reset value: 0x0000 00002000
        /// </summary>
        private static uint PTPTSCR { get { return _PTPTSCR; } set { _PTPTSCR = value; } }

        /// <summary>
        /// See: P.1200 - Ethernet PTP subsecond increment register - Address offset: 0x0704 - Reset value: 0x0000 0000
        /// </summary>
        private static uint PTPSSIR { get { return _PTPSSIR; } set { _PTPSSIR = value; } }

        /// <summary>
        /// See: P.1201 - Ethernet PTP time stamp high register - Address offset: 0x0708 - Reset value: 0x0000 0000
        /// </summary>
        private static uint PTPTSHR { get { return _PTPTSHR; } set { _PTPTSHR = value; } }

        /// <summary>
        /// See: P.1201 - Ethernet PTP time stamp low register - Address offset: 0x070C - Reset value: 0x0000 0000
        /// </summary>
        private static uint PTPTSLR { get { return _PTPTSLR; } set { _PTPTSLR = value; } }

        /// <summary>
        /// See: P.1201 - Ethernet PTP time stamp high update register - Address offset: 0x0710 - Reset value: 0x0000 0000
        /// </summary>
        private static uint PTPTSHUR { get { return _PTPTSHUR; } set { _PTPTSHUR = value; } }

        /// <summary>
        /// See: P.1203 - Ethernet PTP time stamp low update register - Address offset: 0x0714 - Reset value: 0x0000 0000
        /// </summary>
        private static uint PTPTSLUR { get { return _PTPTSLUR; } set { _PTPTSLUR = value; } }

        /// <summary>
        /// See: P.1203 - Ethernet PTP time stamp addend register - Address offset: 0x0718 - Reset value: 0x0000 0000
        /// </summary>
        private static uint PTPTSAR { get { return _PTPTSAR; } set { _PTPTSAR = value; } }

        /// <summary>
        /// See: P.1204 - Ethernet PTP target time high register - Address offset: 0x071C - Reset value: 0x0000 0000
        /// </summary>
        private static uint PTPTTHR { get { return _PTPTTHR; } set { _PTPTTHR = value; } }

        /// <summary>
        /// See: P.1204 - Ethernet PTP target time low register - Address offset: 0x0720 - Reset value: 0x0000 0000
        /// </summary>
        private static uint PTPTTLR { get { return _PTPTTLR; } set { _PTPTTLR = value; } }

        /// <summary>
        /// See: P.1204 - Ethernet PTP time stamp status register - Address offset: 0x0728 - Reset value: 0x0000 0000
        /// </summary>
        private static uint PTPTSSR { get { return _PTPTSSR; } set { _PTPTSSR = value; } }

        /// <summary>
        /// See: P.1205 - Ethernet DMA bus mode register - Address offset: 0x1000 - Reset value: 0x0002 0101
        /// </summary>
        private static uint DMABMR { get { return _DMABMR; } set { _DMABMR = value; } }

        /// <summary>
        /// See: P.1207 - Ethernet DMA transmit poll demand register - Address offset: 0x1004 - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMATPDR { get { return _DMATPDR; } set { _DMATPDR = value; } }

        /// <summary>
        /// See: P.1208 - EHERNET DMA receive poll demand register - Address offset: 0x1008 - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMARPDR { get { return _DMARPDR; } set { _DMARPDR = value; } }

        /// <summary>
        /// See: P.1208 - Ethernet DMA receive descriptor list address register - Address offset: 0x100C - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMARDLAR { get { return _DMARDLAR; } set { _DMARDLAR = value; } }

        /// <summary>
        /// See: P.1209 - Ethernet DMA transmit descriptor list address register - Address offset: 0x1010 - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMATDLAR { get { return _DMATDLAR; } set { _DMATDLAR = value; } }

        /// <summary>
        /// See: P.1209 - Ethernet DMA status register - Address offset: 0x1014 - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMASR { get { return _DMASR; } set { _DMASR = value; } }

        /// <summary>
        /// See: P.1212 - Ethernet DMA operation mode register - Address offset: 0x1018 - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMAOMR { get { return _DMAOMR; } set { _DMAOMR = value; } }

        /// <summary>
        /// See: P.1215 - Ethernet DMA interrupt enable register - Address offset: 0x101C - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMAIER { get { return _DMAIER; } set { _DMAIER = value; } }

        /// <summary>
        /// See: P.1217 - Ethernet DMA missed frame and buffer overflow counter register - Address offset: 0x1020 - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMAMFBOCR { get { return _DMAMFBOCR; } set { _DMAMFBOCR = value; } }

        /// <summary>
        /// See: P.1217 - Ethernet DMA receive status watchdog timer register - Address offset: 0x1024 - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMARSWTR { get { return _DMARSWTR; } set { _DMARSWTR = value; } }

        /// <summary>
        /// See: P.1218 - Ethernet DMA current host transmit descriptor register - Address offset: 0x1048 - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMACHTDR { get { return _DMACHTDR; } set { _DMACHTDR = value; } }

        /// <summary>
        /// See: P.1218 - Ethernet DMA current host receive descriptor register - Address offset: 0x104C - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMACHRDR { get { return _DMACHRDR; } set { _DMACHRDR = value; } }

        /// <summary>
        /// See: P.1218 - Ethernet DMA current host transmit buffer address register - Address offset: 0x1050 - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMACHTBAR { get { return _DMACHTBAR; } set { _DMACHTBAR = value; } }

        /// <summary>
        /// See: P.1219 - Ethernet DMA current host receive buffer address register - Address offset: 0x1054 - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMACHRBAR { get { return _DMACHRBAR; } set { _DMACHRBAR = value; } }

        #endregion

        #region Registers...

        //// <summary>
        /// See: P.1173 - Ethernet MAC configuration register - Address offset: 0x0000 - Reset value: 0x0000 8000
        /// </summary>
        public static Register MACCR_Register;

        /// <summary>
        /// See: P.1177 - Ethernet MAC frame filter register - Address offset: 0x0004 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACFFR_Register;

        /// <summary>
        /// See: P.1178 - Ethernet MAC hash table high register - Address offset: 0x0008 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACHTHR_Register;

        /// <summary>
        /// See P.1179 Ethernet MAC hash table low register - Address offset: 0x000C - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACHTLR_Register;

        /// <summary>
        /// See: P.1179 - Ethernet MAC MII address register - Address offset: 0x0010 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACMIIAR_Register;

        /// <summary>
        /// See: P.1180 - Ethernet MAC MII data register - Address offset: 0x0014 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACMIIDR_Register;

        /// <summary>
        /// See: P.1181 - Ethernet MAC flow control register - Address offset: 0x0018 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACFCR_Register;

        /// <summary>
        ///  See: P.1182 - Ethernet MAC VLAN tag register - Address offset: 0x001C - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACVLANTR_Register;

        /// <summary>
        /// See: P.1183 - Ethernet MAC remote wakeup frame filter register - Address offset: 0x0028 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACRWUFFR_Register;

        /// <summary>
        /// See: P.1184 - Ethernet MAC PMT control and status register - Address offset: 0x002C - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACPMTCSR_Register;

        /// <summary>
        /// See: P.1187 - Ethernet MAC interrupt status register - Address offset: 0x0038 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACSR_Register;

        /// <summary>
        /// See: P.1188 - Ethernet MAC interrupt mask register - Address offset: 0x003C - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACIMR_Register;

        /// <summary>
        /// See: P.1188 - Ethernet MAC address 0 high register - Address offset: 0x0040 - Reset value: 0x8000 FFFF
        /// </summary>
        public static Register MACA0HR_Register;

        /// <summary>
        /// See: P.1189 - Ethernet MAC address 0 low register - Address offset: 0x0044 - Reset value: 0xFFFF FFFF
        /// </summary>
        public static Register MACA0LR_Register;

        /// <summary>
        /// See: P.1189 - Ethernet MAC address 1 high register - Address offset: 0x0048 - Reset value: 0x0000 FFFF
        /// </summary>
        public static Register MACA1HR_Register;

        /// <summary>
        /// See: P.1190 - Ethernet MAC address1 low register - Address offset: 0x004C - Reset value: 0xFFFF FFFF
        /// </summary>
        public static Register MACA1LR_Register;

        /// <summary>
        /// See: P.1190 - Ethernet MAC address 2 high register - Address offset: 0x0050 - Reset value: 0x0000 FFFF
        /// </summary>
        public static Register MACA2HR_Register;

        /// <summary>
        /// See: P.1191 - Ethernet MAC address 2 low register - Address offset: 0x0054 - Reset value: 0xFFFF FFFF
        /// </summary>
        public static Register MACA2LR_Register;

        /// <summary>
        /// See: P.1191 - Ethernet MAC address 3 high register - Address offset: 0x0058 - Reset value: 0x0000 FFFF
        /// </summary>
        public static Register MACA3HR_Register;

        /// <summary>
        /// See: P.1192 - Ethernet MAC address 3 low register - Address offset: 0x005C - Reset value: 0xFFFF FFFF
        /// </summary>
        public static Register MACA3LR_Register;

        /// <summary>
        /// See: P.1193 - Ethernet MMC control register - Address offset: 0x0100 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCCR_Register;

        /// <summary>
        /// See: P.1193 - Ethernet MMC receive interrupt register - Address offset: 0x0104 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCRIR_Register;

        /// <summary>
        /// See: P.1194 - Ethernet MMC transmit interrupt register - Address offset: 0x0108 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCTIR_Register;

        /// <summary>
        /// See: P.1195 - Ethernet MMC receive interrupt mask register - Address offset: 0x010C - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCRIMR_Register;

        /// <summary>
        /// See: P.1195 - Ethernet MMC transmit interrupt mask register - Address offset: 0x0110 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCTIMR_Register;

        /// <summary>
        /// See: P.1196 - Ethernet MMC transmitted good frames after a single collision counter register - Address offset: 0x014C - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCTGFSCCR_Register;

        /// <summary>
        /// See: P.1196 - Ethernet MMC transmitted good frames after more than a single collision counter register - Address offset: 0x0150 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCTGFMSCCR_Register;

        /// <summary>
        /// See: P.1197 - Ethernet MMC transmitted good frames counter register - Address offset: 0x0168 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCTGFCR_Register;

        /// <summary>
        /// See: P.1197 - Ethernet MMC received frames with CRC error counter register - Address offset: 0x0194 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCRFCECR_Register;

        /// <summary>
        /// See: P.1197 - Ethernet MMC received frames with alignment error counter register - Address offset: 0x0198 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCRFAECR_Register;

        /// <summary>
        /// See: P.1198 - MMC received good unicast frames counter register - Address offset: 0x01C4 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCRGUFCR_Register;

        /// <summary>
        /// See: P.1198 - Ethernet PTP time stamp control register - Address offset: 0x0700 - Reset value: 0x0000 00002000
        /// </summary>
        public static Register PTPTSCR_Register;

        /// <summary>
        /// See: P.1200 - Ethernet PTP subsecond increment register - Address offset: 0x0704 - Reset value: 0x0000 0000
        /// </summary>
        public static Register PTPSSIR_Register;

        /// <summary>
        /// See: P.1201 - Ethernet PTP time stamp high register - Address offset: 0x0708 - Reset value: 0x0000 0000
        /// </summary>
        public static Register PTPTSHR_Register;

        /// <summary>
        /// See: P.1201 - Ethernet PTP time stamp low register - Address offset: 0x070C - Reset value: 0x0000 0000
        /// </summary>
        public static Register PTPTSLR_Register;

        /// <summary>
        /// See: P.1201 - Ethernet PTP time stamp high update register - Address offset: 0x0710 - Reset value: 0x0000 0000
        /// </summary>
        public static Register PTPTSHUR_Register;

        /// <summary>
        /// See: P.1203 - Ethernet PTP time stamp low update register - Address offset: 0x0714 - Reset value: 0x0000 0000
        /// </summary>
        public static Register PTPTSLUR_Register;

        /// <summary>
        /// See: P.1203 - Ethernet PTP time stamp addend register - Address offset: 0x0718 - Reset value: 0x0000 0000
        /// </summary>
        public static Register PTPTSAR_Register;

        /// <summary>
        /// See: P.1204 - Ethernet PTP target time high register - Address offset: 0x071C - Reset value: 0x0000 0000
        /// </summary>
        public static Register PTPTTHR_Register;

        /// <summary>
        /// See: P.1204 - Ethernet PTP target time low register - Address offset: 0x0720 - Reset value: 0x0000 0000
        /// </summary>
        public static Register PTPTTLR_Register;

        /// <summary>
        /// See: P.1204 - Ethernet PTP time stamp status register - Address offset: 0x0728 - Reset value: 0x0000 0000
        /// </summary>
        public static Register PTPTSSR_Register;

        /// <summary>
        /// See: P.1205 - Ethernet DMA bus mode register - Address offset: 0x1000 - Reset value: 0x0002 0101
        /// </summary>
        public static Register DMABMR_Register;

        /// <summary>
        /// See: P.1207 - Ethernet DMA transmit poll demand register - Address offset: 0x1004 - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMATPDR_Register;

        /// <summary>
        /// See: P.1208 - EHERNET DMA receive poll demand register - Address offset: 0x1008 - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMARPDR_Register;

        /// <summary>
        /// See: P.1208 - Ethernet DMA receive descriptor list address register - Address offset: 0x100C - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMARDLAR_Register;

        /// <summary>
        /// See: P.1209 - Ethernet DMA transmit descriptor list address register - Address offset: 0x1010 - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMATDLAR_Register;

        /// <summary>
        /// See: P.1209 - Ethernet DMA status register - Address offset: 0x1014 - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMASR_Register;

        /// <summary>
        /// See: P.1212 - Ethernet DMA operation mode register - Address offset: 0x1018 - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMAOMR_Register;

        /// <summary>
        /// See: P.1215 - Ethernet DMA interrupt enable register - Address offset: 0x101C - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMAIER_Register;

        /// <summary>
        /// See: P.1217 - Ethernet DMA missed frame and buffer overflow counter register - Address offset: 0x1020 - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMAMFBOCR_Register;

        /// <summary>
        /// See: P.1217 - Ethernet DMA receive status watchdog timer register - Address offset: 0x1024 - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMARSWTR_Register;

        /// <summary>
        /// See: P.1218 - Ethernet DMA current host transmit descriptor register - Address offset: 0x1048 - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMACHTDR_Register;

        /// <summary>
        /// See: P.1218 - Ethernet DMA current host receive descriptor register - Address offset: 0x104C - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMACHRDR_Register;

        /// <summary>
        /// See: P.1218 - Ethernet DMA current host transmit buffer address register - Address offset: 0x1050 - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMACHTBAR_Register;

        /// <summary>
        /// See: P.1219 - Ethernet DMA current host receive buffer address register - Address offset: 0x1054 - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMACHRBAR_Register;

        #endregion

        static ETH()
        {
            MACCR_Register = new Register(STM32F4xx.ETH_BASE + MACCR);
            MACFFR_Register = new Register(STM32F4xx.ETH_BASE + MACFFR);
            MACHTHR_Register = new Register(STM32F4xx.ETH_BASE + MACHTHR);
            MACHTLR_Register = new Register(STM32F4xx.ETH_BASE + MACHTLR);
            MACMIIAR_Register = new Register(STM32F4xx.ETH_BASE + MACMIIAR);
            MACMIIDR_Register = new Register(STM32F4xx.ETH_BASE + MACMIIDR);
            MACFCR_Register = new Register(STM32F4xx.ETH_BASE + MACFCR);
            MACVLANTR_Register = new Register(STM32F4xx.ETH_BASE + MACVLANTR);
            MACRWUFFR_Register = new Register(STM32F4xx.ETH_BASE + MACRWUFFR);
            MACPMTCSR_Register = new Register(STM32F4xx.ETH_BASE + MACPMTCSR);
            MACSR_Register = new Register(STM32F4xx.ETH_BASE + MACSR);
            MACIMR_Register = new Register(STM32F4xx.ETH_BASE + MACIMR);
            MACA0HR_Register = new Register(STM32F4xx.ETH_BASE + MACA0HR);
            MACA0LR_Register = new Register(STM32F4xx.ETH_BASE + MACA0LR);
            MACA1HR_Register = new Register(STM32F4xx.ETH_BASE + MACA1HR);
            MACA1LR_Register = new Register(STM32F4xx.ETH_BASE + MACA1LR);
            MACA2HR_Register = new Register(STM32F4xx.ETH_BASE + MACA2HR);
            MACA2LR_Register = new Register(STM32F4xx.ETH_BASE + MACA2LR);
            MACA3HR_Register = new Register(STM32F4xx.ETH_BASE + MACA3HR);
            MACA3LR_Register = new Register(STM32F4xx.ETH_BASE + MACA3LR);
            MMCCR_Register = new Register(STM32F4xx.ETH_BASE + MMCCR);
            MMCRIR_Register = new Register(STM32F4xx.ETH_BASE + MMCRIR);
            MMCTIR_Register = new Register(STM32F4xx.ETH_BASE + MMCTIR);
            MMCRIMR_Register = new Register(STM32F4xx.ETH_BASE + MMCRIMR);
            MMCTIMR_Register = new Register(STM32F4xx.ETH_BASE + MMCTIMR);
            MMCTGFSCCR_Register = new Register(STM32F4xx.ETH_BASE + MMCTGFSCCR);
            MMCTGFMSCCR_Register = new Register(STM32F4xx.ETH_BASE + MMCTGFMSCCR);
            MMCTGFCR_Register = new Register(STM32F4xx.ETH_BASE + MMCTGFCR);
            MMCRFCECR_Register = new Register(STM32F4xx.ETH_BASE + MMCRFCECR);
            MMCRFAECR_Register = new Register(STM32F4xx.ETH_BASE + MMCRFAECR);
            MMCRGUFCR_Register = new Register(STM32F4xx.ETH_BASE + MMCRGUFCR);
            PTPTSCR_Register = new Register(STM32F4xx.ETH_BASE + PTPTSCR);
            PTPSSIR_Register = new Register(STM32F4xx.ETH_BASE + PTPSSIR);
            PTPTSHR_Register = new Register(STM32F4xx.ETH_BASE + PTPTSHR);
            PTPTSLR_Register = new Register(STM32F4xx.ETH_BASE + PTPTSLR);
            PTPTSHUR_Register = new Register(STM32F4xx.ETH_BASE + PTPTSHUR);
            PTPTSLUR_Register = new Register(STM32F4xx.ETH_BASE + PTPTSLUR);
            PTPTSAR_Register = new Register(STM32F4xx.ETH_BASE + PTPTSAR);
            PTPTTHR_Register = new Register(STM32F4xx.ETH_BASE + PTPTTHR);
            PTPTTLR_Register = new Register(STM32F4xx.ETH_BASE + PTPTTLR);
            // RESERVED8_Register = new Register(STM32F4xx.ETH_BASE + RESERVED8);
            PTPTSSR_Register = new Register(STM32F4xx.ETH_BASE + PTPTSSR);
            DMABMR_Register = new Register(STM32F4xx.ETH_BASE + DMABMR);
            DMATPDR_Register = new Register(STM32F4xx.ETH_BASE + DMATPDR);
            DMARPDR_Register = new Register(STM32F4xx.ETH_BASE + DMARPDR);
            DMARDLAR_Register = new Register(STM32F4xx.ETH_BASE + DMARDLAR);
            DMATDLAR_Register = new Register(STM32F4xx.ETH_BASE + DMATDLAR);
            DMASR_Register = new Register(STM32F4xx.ETH_BASE + DMASR);
            DMAOMR_Register = new Register(STM32F4xx.ETH_BASE + DMAOMR);
            DMAIER_Register = new Register(STM32F4xx.ETH_BASE + DMAIER);
            DMAMFBOCR_Register = new Register(STM32F4xx.ETH_BASE + DMAMFBOCR);
            DMARSWTR_Register = new Register(STM32F4xx.ETH_BASE + DMARSWTR);
            DMACHTDR_Register = new Register(STM32F4xx.ETH_BASE + DMACHTDR);
            DMACHRDR_Register = new Register(STM32F4xx.ETH_BASE + DMACHRDR);
            DMACHTBAR_Register = new Register(STM32F4xx.ETH_BASE + DMACHTBAR);
            DMACHRBAR_Register = new Register(STM32F4xx.ETH_BASE + DMACHRBAR);
        }
    } // END of Class...

    /// <summary>
    /// Digital camera interface (DCMI) - See: P.452
    /// </summary>
    public static class DCMI
    {
        #region Private's...

        /// <summary>
        /// DCMI control register 1, Address offset: 0x00 
        /// </summary>
        private static uint _CR = 0x00;

        /// <summary>
        /// DCMI status register,Address offset: 0x04 
        /// </summary>
        private static uint _SR = 0x04;

        /// <summary>
        /// DCMI raw interrupt status register,Address offset: 0x08 
        /// </summary>
        private static uint _RISR = 0x08;

        /// <summary>
        /// DCMI interrupt enable register,Address offset: 0x0C 
        /// </summary>
        private static uint _IER = 0x0C;

        /// <summary>
        /// DCMI masked interrupt status register, Address offset: 0x10 
        /// </summary>
        private static uint _MISR = 0x10;

        /// <summary>
        /// DCMI interrupt clear register, Address offset: 0x14 
        /// </summary>
        private static uint _ICR = 0x14;

        /// <summary>
        /// DCMI embedded synchronization code register, Address offset: 0x18 
        /// </summary>
        private static uint _ESCR = 0x18;

        /// <summary>
        /// DCMI embedded synchronization unmask register, Address offset: 0x1C 
        /// </summary>
        private static uint _ESUR = 0x1C;

        /// <summary>
        /// DCMI crop window start,Address offset: 0x20 
        /// </summary>
        private static uint _CWSTRTR = 0x20;

        /// <summary>
        /// DCMI crop window size, Address offset: 0x24 
        /// </summary>
        private static uint _CWSIZER = 0x24;

        /// <summary>
        /// DCMI data register,Address offset: 0x28 
        /// </summary>
        private static uint _DR = 0x28;

        #endregion

        #region Properties...

        /// <summary>
        /// DCMI control register 1, Address offset: 0x00 
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// DCMI status register,Address offset: 0x04 
        /// </summary>
        public static uint SR { get { return _SR; } set { _SR = value; } }

        /// <summary>
        /// DCMI raw interrupt status register,Address offset: 0x08 
        /// </summary>
        public static uint RISR { get { return _RISR; } set { _RISR = value; } }

        /// <summary>
        /// DCMI interrupt enable register,Address offset: 0x0C 
        /// </summary>
        public static uint IER { get { return _IER; } set { _IER = value; } }

        /// <summary>
        /// DCMI masked interrupt status register, Address offset: 0x10 
        /// </summary>
        public static uint MISR { get { return _MISR; } set { _MISR = value; } }

        /// <summary>
        /// DCMI interrupt clear register, Address offset: 0x14 
        /// </summary>
        public static uint ICR { get { return _ICR; } set { _ICR = value; } }

        /// <summary>
        /// DCMI embedded synchronization code register, Address offset: 0x18 
        /// </summary>
        public static uint ESCR { get { return _ESCR; } set { _ESCR = value; } }

        /// <summary>
        /// DCMI embedded synchronization unmask register, Address offset: 0x1C 
        /// </summary>
        public static uint ESUR { get { return _ESUR; } set { _ESUR = value; } }

        /// <summary>
        /// DCMI crop window start,Address offset: 0x20 
        /// </summary>
        public static uint CWSTRTR { get { return _CWSTRTR; } set { _CWSTRTR = value; } }

        /// <summary>
        /// DCMI crop window size, Address offset: 0x24 
        /// </summary>
        public static uint CWSIZER { get { return _CWSIZER; } set { _CWSIZER = value; } }

        /// <summary>
        /// DCMI data register,Address offset: 0x28 
        /// </summary>
        public static uint DR { get { return _DR; } set { _DR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// DCMI control register 1, Address offset: 0x00 
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// DCMI status register,Address offset: 0x04 
        /// </summary>
        public static Register SR_Register;

        /// <summary>
        /// DCMI raw interrupt status register,Address offset: 0x08 
        /// </summary>
        public static Register RISR_Register;

        /// <summary>
        /// DCMI interrupt enable register,Address offset: 0x0C 
        /// </summary>
        public static Register IER_Register;

        /// <summary>
        /// DCMI masked interrupt status register, Address offset: 0x10 
        /// </summary>
        public static Register MISR_Register;

        /// <summary>
        /// DCMI interrupt clear register, Address offset: 0x14 
        /// </summary>
        public static Register ICR_Register;

        /// <summary>
        /// DCMI embedded synchronization code register, Address offset: 0x18 
        /// </summary>
        public static Register ESCR_Register;

        /// <summary>
        /// DCMI embedded synchronization unmask register, Address offset: 0x1C 
        /// </summary>
        public static Register ESUR_Register;

        /// <summary>
        /// DCMI crop window start,Address offset: 0x20 
        /// </summary>
        public static Register CWSTRTR_Register;

        /// <summary>
        /// DCMI crop window size, Address offset: 0x24 
        /// </summary>
        public static Register CWSIZER_Register;

        /// <summary>
        /// DCMI data register,Address offset: 0x28 
        /// </summary>
        public static Register DR_Register;

        #endregion

        static DCMI()
        {
            CR_Register = new Register(STM32F4xx.DCMI_BASE + CR);
            SR_Register = new Register(STM32F4xx.DCMI_BASE + SR);
            RISR_Register = new Register(STM32F4xx.DCMI_BASE + RISR);
            IER_Register = new Register(STM32F4xx.DCMI_BASE + IER);
            MISR_Register = new Register(STM32F4xx.DCMI_BASE + MISR);
            ICR_Register = new Register(STM32F4xx.DCMI_BASE + ICR);
            ESCR_Register = new Register(STM32F4xx.DCMI_BASE + ESCR);
            ESUR_Register = new Register(STM32F4xx.DCMI_BASE + ESUR);
            CWSTRTR_Register = new Register(STM32F4xx.DCMI_BASE + CWSTRTR);
            CWSIZER_Register = new Register(STM32F4xx.DCMI_BASE + CWSIZER);
            DR_Register = new Register(STM32F4xx.DCMI_BASE + DR);
        }
    } // END of Class...

    /// <summary>
    /// Cryptographic processor (CRYP) - See: P.704
    /// </summary>
    public static class CRYP
    {
        #region Private's...

        /// <summary>
        /// CRYP control register,Address offset: 0x00 
        /// </summary>
        private static uint _CR = 0x00;

        /// <summary>
        /// CRYP status register, Address offset: 0x04 
        /// </summary>
        private static uint _SR = 0x04;

        /// <summary>
        /// CRYP data input register, Address offset: 0x08 
        /// </summary>
        private static uint _DR = 0x08;

        /// <summary>
        /// CRYP data output register,Address offset: 0x0C 
        /// </summary>
        private static uint _DOUT = 0x0C;

        /// <summary>
        /// CRYP DMA control register,Address offset: 0x10 
        /// </summary>
        private static uint _DMACR = 0x10;

        /// <summary>
        /// CRYP interrupt mask set/clear register, Address offset: 0x14 
        /// </summary>
        private static uint _IMSCR = 0x14;

        /// <summary>
        /// CRYP raw interrupt status register, Address offset: 0x18 
        /// </summary>
        private static uint _RISR = 0x18;

        /// <summary>
        /// CRYP masked interrupt status register,Address offset: 0x1C 
        /// </summary>
        private static uint _MISR = 0x1C;

        /// <summary>
        /// CRYP key leftregister 0,Address offset: 0x20 
        /// </summary>
        private static uint _K0LR = 0x20;

        /// <summary>
        /// CRYP key right register 0,Address offset: 0x24 
        /// </summary>
        private static uint _K0RR = 0x24;

        /// <summary>
        /// CRYP key leftregister 1,Address offset: 0x28 
        /// </summary>
        private static uint _K1LR = 0x28;

        /// <summary>
        /// CRYP key right register 1,Address offset: 0x2C 
        /// </summary>
        private static uint _K1RR = 0x2C;

        /// <summary>
        /// CRYP key leftregister 2,Address offset: 0x30 
        /// </summary>
        private static uint _K2LR = 0x30;

        /// <summary>
        /// CRYP key right register 2,Address offset: 0x34 
        /// </summary>
        private static uint _K2RR = 0x34;

        /// <summary>
        /// CRYP key leftregister 3,Address offset: 0x38 
        /// </summary>
        private static uint _K3LR = 0x38;

        /// <summary>
        /// CRYP key right register 3,Address offset: 0x3C 
        /// </summary>
        private static uint _K3RR = 0x3C;

        /// <summary>
        /// CRYP initialization vector left-wordregister 0, Address offset: 0x40 
        /// </summary>
        private static uint _IV0LR = 0x40;

        /// <summary>
        /// CRYP initialization vector right-word register 0, Address offset: 0x44 
        /// </summary>
        private static uint _IV0RR = 0x44;

        /// <summary>
        /// CRYP initialization vector left-wordregister 1, Address offset: 0x48 
        /// </summary>
        private static uint _IV1LR = 0x48;

        /// <summary>
        /// CRYP initialization vector right-word register 1, Address offset: 0x4C 
        /// </summary>
        private static uint _IV1RR = 0x4C;

        #endregion

        #region Properties...

        /// <summary>
        /// CRYP control register,Address offset: 0x00 
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// CRYP status register, Address offset: 0x04 
        /// </summary>
        public static uint SR { get { return _SR; } set { _SR = value; } }

        /// <summary>
        /// CRYP data input register, Address offset: 0x08 
        /// </summary>
        public static uint DR { get { return _DR; } set { _DR = value; } }

        /// <summary>
        /// CRYP data output register,Address offset: 0x0C 
        /// </summary>
        public static uint DOUT { get { return _DOUT; } set { _DOUT = value; } }

        /// <summary>
        /// CRYP DMA control register,Address offset: 0x10 
        /// </summary>
        public static uint DMACR { get { return _DMACR; } set { _DMACR = value; } }

        /// <summary>
        /// CRYP interrupt mask set/clear register, Address offset: 0x14 
        /// </summary>
        public static uint IMSCR { get { return _IMSCR; } set { _IMSCR = value; } }

        /// <summary>
        /// CRYP raw interrupt status register, Address offset: 0x18 
        /// </summary>
        public static uint RISR { get { return _RISR; } set { _RISR = value; } }

        /// <summary>
        /// CRYP masked interrupt status register,Address offset: 0x1C 
        /// </summary>
        public static uint MISR { get { return _MISR; } set { _MISR = value; } }

        /// <summary>
        /// CRYP key leftregister 0,Address offset: 0x20 
        /// </summary>
        public static uint K0LR { get { return _K0LR; } set { _K0LR = value; } }

        /// <summary>
        /// CRYP key right register 0,Address offset: 0x24 
        /// </summary>
        public static uint K0RR { get { return _K0RR; } set { _K0RR = value; } }

        /// <summary>
        /// CRYP key leftregister 1,Address offset: 0x28 
        /// </summary>
        public static uint K1LR { get { return _K1LR; } set { _K1LR = value; } }

        /// <summary>
        /// CRYP key right register 1,Address offset: 0x2C 
        /// </summary>
        public static uint K1RR { get { return _K1RR; } set { _K1RR = value; } }

        /// <summary>
        /// CRYP key leftregister 2,Address offset: 0x30 
        /// </summary>
        public static uint K2LR { get { return _K2LR; } set { _K2LR = value; } }

        /// <summary>
        /// CRYP key right register 2,Address offset: 0x34 
        /// </summary>
        public static uint K2RR { get { return _K2RR; } set { _K2RR = value; } }

        /// <summary>
        /// CRYP key leftregister 3,Address offset: 0x38 
        /// </summary>
        public static uint K3LR { get { return _K3LR; } set { _K3LR = value; } }

        /// <summary>
        /// CRYP key right register 3,Address offset: 0x3C 
        /// </summary>
        public static uint K3RR { get { return _K3RR; } set { _K3RR = value; } }

        /// <summary>
        /// CRYP initialization vector left-wordregister 0, Address offset: 0x40 
        /// </summary>
        public static uint IV0LR { get { return _IV0LR; } set { _IV0LR = value; } }

        /// <summary>
        /// CRYP initialization vector right-word register 0, Address offset: 0x44 
        /// </summary>
        public static uint IV0RR { get { return _IV0RR; } set { _IV0RR = value; } }

        /// <summary>
        /// CRYP initialization vector left-wordregister 1, Address offset: 0x48 
        /// </summary>
        public static uint IV1LR { get { return _IV1LR; } set { _IV1LR = value; } }

        /// <summary>
        /// CRYP initialization vector right-word register 1, Address offset: 0x4C 
        /// </summary>
        public static uint IV1RR { get { return _IV1RR; } set { _IV1RR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// CRYP control register,Address offset: 0x00 
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// CRYP status register, Address offset: 0x04 
        /// </summary>
        public static Register SR_Register;

        /// <summary>
        /// CRYP data input register, Address offset: 0x08 
        /// </summary>
        public static Register DR_Register;

        /// <summary>
        /// CRYP data output register,Address offset: 0x0C 
        /// </summary>
        public static Register DOUT_Register;

        /// <summary>
        /// CRYP DMA control register,Address offset: 0x10 
        /// </summary>
        public static Register DMACR_Register;

        /// <summary>
        /// CRYP interrupt mask set/clear register, Address offset: 0x14 
        /// </summary>
        public static Register IMSCR_Register;

        /// <summary>
        /// CRYP raw interrupt status register, Address offset: 0x18 
        /// </summary>
        public static Register RISR_Register;

        /// <summary>
        /// CRYP masked interrupt status register,Address offset: 0x1C 
        /// </summary>
        public static Register MISR_Register;

        /// <summary>
        /// CRYP key leftregister 0,Address offset: 0x20 
        /// </summary>
        public static Register K0LR_Register;

        /// <summary>
        /// CRYP key right register 0,Address offset: 0x24 
        /// </summary>
        public static Register K0RR_Register;

        /// <summary>
        /// CRYP key leftregister 1,Address offset: 0x28 
        /// </summary>
        public static Register K1LR_Register;

        /// <summary>
        /// CRYP key right register 1,Address offset: 0x2C 
        /// </summary>
        public static Register K1RR_Register;

        /// <summary>
        /// CRYP key leftregister 2,Address offset: 0x30 
        /// </summary>
        public static Register K2LR_Register;

        /// <summary>
        /// CRYP key right register 2,Address offset: 0x34 
        /// </summary>
        public static Register K2RR_Register;

        /// <summary>
        /// CRYP key leftregister 3,Address offset: 0x38 
        /// </summary>
        public static Register K3LR_Register;

        /// <summary>
        /// CRYP key right register 3,Address offset: 0x3C 
        /// </summary>
        public static Register K3RR_Register;

        /// <summary>
        /// CRYP initialization vector left-wordregister 0, Address offset: 0x40 
        /// </summary>
        public static Register IV0LR_Register;

        /// <summary>
        /// CRYP initialization vector right-word register 0, Address offset: 0x44 
        /// </summary>
        public static Register IV0RR_Register;

        /// <summary>
        /// CRYP initialization vector left-wordregister 1, Address offset: 0x48 
        /// </summary>
        public static Register IV1LR_Register;

        /// <summary>
        /// CRYP initialization vector right-word register 1, Address offset: 0x4C 
        /// </summary>
        public static Register IV1RR_Register;

        #endregion

        static CRYP()
        {
            CR_Register = new Register(STM32F4xx.CRYP_BASE + CR);
            SR_Register = new Register(STM32F4xx.CRYP_BASE + SR);
            DR_Register = new Register(STM32F4xx.CRYP_BASE + DR);
            DOUT_Register = new Register(STM32F4xx.CRYP_BASE + DOUT);
            DMACR_Register = new Register(STM32F4xx.CRYP_BASE + DMACR);
            IMSCR_Register = new Register(STM32F4xx.CRYP_BASE + IMSCR);
            RISR_Register = new Register(STM32F4xx.CRYP_BASE + RISR);
            MISR_Register = new Register(STM32F4xx.CRYP_BASE + MISR);
            K0LR_Register = new Register(STM32F4xx.CRYP_BASE + K0LR);
            K0RR_Register = new Register(STM32F4xx.CRYP_BASE + K0RR);
            K1LR_Register = new Register(STM32F4xx.CRYP_BASE + K1LR);
            K1RR_Register = new Register(STM32F4xx.CRYP_BASE + K1RR);
            K2LR_Register = new Register(STM32F4xx.CRYP_BASE + K2LR);
            K2RR_Register = new Register(STM32F4xx.CRYP_BASE + K2RR);
            K3LR_Register = new Register(STM32F4xx.CRYP_BASE + K3LR);
            K3RR_Register = new Register(STM32F4xx.CRYP_BASE + K3RR);
            IV0LR_Register = new Register(STM32F4xx.CRYP_BASE + IV0LR);
            IV0RR_Register = new Register(STM32F4xx.CRYP_BASE + IV0RR);
            IV1LR_Register = new Register(STM32F4xx.CRYP_BASE + IV1LR);
            IV1RR_Register = new Register(STM32F4xx.CRYP_BASE + IV1RR);
        }
    } // END of Class...

    /// <summary>
    /// Hash processor (HASH) - See: P.755
    /// </summary>
    public static class HASH
    {
        #region Private's...

        /// <summary>
        /// HASH control register,Address offset: 0x00
        /// </summary>
        private static uint _CR = 0x00;

        /// <summary>
        /// HASH data input register, Address offset: 0x04
        /// </summary>
        private static uint _DIN = 0x04;

        /// <summary>
        /// HASH start register,Address offset: 0x08
        /// </summary>
        private static uint _STR = 0x08;

        /// <summary>
        /// HASH digest registers,Address offset: 0x0C-0x1C 
        /// </summary>
        // private static uint _HR[5] =  0x0C-0x1C ;

        /// <summary>
        /// HASH interrupt enable register, Address offset: 0x20
        /// </summary>
        private static uint _IMR = 0x20;

        /// <summary>
        /// HASH status register, Address offset: 0x24
        /// </summary>
        private static uint _SR = 0x24;

        /// <summary>
        /// HASH context swap registers,Address offset: 0x0F8-0x1C0 
        /// </summary>
        // private static uint _CSR[51] =  0x0F8-0x1C0 ;

        #endregion

        #region Properties...

        /// <summary>
        /// HASH control register,Address offset: 0x00
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// HASH data input register, Address offset: 0x04
        /// </summary>
        public static uint DIN { get { return _DIN; } set { _DIN = value; } }

        /// <summary>
        /// HASH start register,Address offset: 0x08
        /// </summary>
        public static uint STR { get { return _STR; } set { _STR = value; } }

        /// <summary>
        /// HASH digest registers,Address offset: 0x0C-0x1C 
        /// </summary>
        // public static uint HR[5]  { get { return _HR[5]; } set { _HR[5] = value; } }

        /// <summary>
        /// HASH interrupt enable register, Address offset: 0x20
        /// </summary>
        public static uint IMR { get { return _IMR; } set { _IMR = value; } }

        /// <summary>
        /// HASH status register, Address offset: 0x24
        /// </summary>
        public static uint SR { get { return _SR; } set { _SR = value; } }

        /// <summary>
        /// HASH context swap registers,Address offset: 0x0F8-0x1C0 
        /// </summary>
        // public static uint CSR[51]  { get { return _CSR[51]; } set { _CSR[51] = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// HASH control register,Address offset: 0x00
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// HASH data input register, Address offset: 0x04
        /// </summary>
        public static Register DIN_Register;

        /// <summary>
        /// HASH start register,Address offset: 0x08
        /// </summary>
        public static Register STR_Register;

        /// <summary>
        /// HASH digest registers,Address offset: 0x0C-0x1C 
        /// </summary>
        // public static Register HR[5]_Register;

        /// <summary>
        /// HASH interrupt enable register, Address offset: 0x20
        /// </summary>
        public static Register IMR_Register;

        /// <summary>
        /// HASH status register, Address offset: 0x24
        /// </summary>
        public static Register SR_Register;

        /// <summary>
        /// HASH context swap registers,Address offset: 0x0F8-0x1C0 
        /// </summary>
        // public Register CSR[51]_Register;

        #endregion

        static HASH()
        {
            CR_Register = new Register(STM32F4xx.HASH_BASE + CR);
            DIN_Register = new Register(STM32F4xx.HASH_BASE + DIN);
            STR_Register = new Register(STM32F4xx.HASH_BASE + STR);
            //HR[5]_Register = new Register(STM32F4xx.HASH_BASE + HR[5]);
            IMR_Register = new Register(STM32F4xx.HASH_BASE + IMR);
            SR_Register = new Register(STM32F4xx.HASH_BASE + SR);
            //CSR[51]_Register = new Register(STM32F4xx.HASH_BASE + CSR[51]);
        }
    } // END of Class...

    /// <summary>
    /// Random number generator (RNG) - See: P.750
    /// </summary>
    public static class RNG
    {
        #region Private's...

        /// <summary>
        /// RNG control register, Address offset: 0x00 
        /// </summary>
        private static uint _CR = 0x00;

        /// <summary>
        /// RNG status register,Address offset: 0x04 
        /// </summary>
        private static uint _SR = 0x04;

        /// <summary>
        /// RNG data register,Address offset: 0x08 
        /// </summary>
        private static uint _DR = 0x08;

        #endregion

        #region Properties...

        /// <summary>
        /// RNG control register, Address offset: 0x00 
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// RNG status register,Address offset: 0x04 
        /// </summary>
        public static uint SR { get { return _SR; } set { _SR = value; } }

        /// <summary>
        /// RNG data register,Address offset: 0x08 
        /// </summary>
        public static uint DR { get { return _DR; } set { _DR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// RNG control register, Address offset: 0x00 
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// RNG status register,Address offset: 0x04 
        /// </summary>
        public static Register SR_Register;

        /// <summary>
        /// RNG data register,Address offset: 0x08 
        /// </summary>
        public static Register DR_Register;

        #endregion

        static RNG()
        {
            CR_Register = new Register(STM32F4xx.RNG_BASE + CR);
            SR_Register = new Register(STM32F4xx.RNG_BASE + SR);
            DR_Register = new Register(STM32F4xx.RNG_BASE + DR);
        }
    } // END of Class...

    /// <summary>
    /// Flexible static memory controller (FSMC) - See: P.1522
    /// NOT Implimented...
    /// </summary>
    public static class FSMC_Bank1
    {
        #region Private's...

        /// <summary>
        /// NOR/PSRAM chip-select control register(BCR) and chip-select timing register(BTR), Address offset: 0x00-1C
        /// </summary>
        // private static uint _BTCR[8] =  0x00-1C;

        #endregion

        #region Properties...

        /// <summary>
        /// NOR/PSRAM chip-select control register(BCR) and chip-select timing register(BTR), Address offset: 0x00-1C
        /// </summary>
        // public static uint BTCR[8]  { get { return _BTCR[8]; } set { _BTCR[8] = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// NOR/PSRAM chip-select control register(BCR) and chip-select timing register(BTR), Address offset: 0x00-1C
        /// </summary>
        // public static Register BTCR[8]_Register;

        #endregion

        static FSMC_Bank1()
        {
            // BTCR[8]_Register = new Register(STM32F4xx.FSMC_Bank1_R_BASE + BTCR[8]);
        }
    } // END of Class...

    /// <summary>
    /// Flexible static memory controller (FSMC) - See: P.1522
    /// NOT Implimented...
    /// </summary>
    public static class FSMC_Bank1E
    {
        #region Private's...

        /// <summary>
        /// NOR/PSRAM write timing registers, Address offset: 0x104-0x11C 
        /// </summary>
        // private static uint _BWTR[7] =  0x104-0x11C ;

        #endregion

        #region Properties...

        /// <summary>
        /// NOR/PSRAM write timing registers, Address offset: 0x104-0x11C 
        /// </summary>
        // public static uint BWTR[7]  { get { return _BWTR[7]; } set { _BWTR[7] = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// NOR/PSRAM write timing registers, Address offset: 0x104-0x11C 
        /// </summary>
        // public static Register BWTR[7]_Register;

        #endregion

        static FSMC_Bank1E()
        {
            // BWTR[7]_Register = new Register(XX_BASE + BWTR[7]);
        }
    } // END of Class...

    /// <summary>
    /// Flexible static memory controller (FSMC) - See: P.1522
    /// </summary>
    public static class FSMC_Bank2
    {
        #region Private's...

        /// <summary>
        /// NAND Flash control register 2, Address offset: 0x60 
        /// </summary>
        private static uint _PCR2 = 0x60;

        /// <summary>
        /// NAND Flash FIFO status and interrupt register 2, Address offset: 0x64 
        /// </summary>
        private static uint _SR2 = 0x64;

        /// <summary>
        /// NAND Flash Common memory space timing register 2,Address offset: 0x68 
        /// </summary>
        private static uint _PMEM2 = 0x68;

        /// <summary>
        /// NAND Flash Attribute memory space timing register 2, Address offset: 0x6C 
        /// </summary>
        private static uint _PATT2 = 0x6C;

        /// <summary>
        /// NAND Flash ECC result registers 2, Address offset: 0x74 
        /// </summary>
        private static uint _ECCR2 = 0x74;

        #endregion

        #region Properties...

        /// <summary>
        /// NAND Flash control register 2, Address offset: 0x60 
        /// </summary>
        public static uint PCR2 { get { return _PCR2; } set { _PCR2 = value; } }

        /// <summary>
        /// NAND Flash FIFO status and interrupt register 2, Address offset: 0x64 
        /// </summary>
        public static uint SR2 { get { return _SR2; } set { _SR2 = value; } }

        /// <summary>
        /// NAND Flash Common memory space timing register 2,Address offset: 0x68 
        /// </summary>
        public static uint PMEM2 { get { return _PMEM2; } set { _PMEM2 = value; } }

        /// <summary>
        /// NAND Flash Attribute memory space timing register 2, Address offset: 0x6C 
        /// </summary>
        public static uint PATT2 { get { return _PATT2; } set { _PATT2 = value; } }

        /// <summary>
        /// NAND Flash ECC result registers 2, Address offset: 0x74 
        /// </summary>
        public static uint ECCR2 { get { return _ECCR2; } set { _ECCR2 = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// NAND Flash control register 2, Address offset: 0x60 
        /// </summary>
        public static Register PCR2_Register;

        /// <summary>
        /// NAND Flash FIFO status and interrupt register 2, Address offset: 0x64 
        /// </summary>
        public static Register SR2_Register;

        /// <summary>
        /// NAND Flash Common memory space timing register 2,Address offset: 0x68 
        /// </summary>
        public static Register PMEM2_Register;

        /// <summary>
        /// NAND Flash Attribute memory space timing register 2, Address offset: 0x6C 
        /// </summary>
        public static Register PATT2_Register;

        /// <summary>
        /// NAND Flash ECC result registers 2, Address offset: 0x74 
        /// </summary>
        public static Register ECCR2_Register;

        #endregion

        static FSMC_Bank2()
        {
            PCR2_Register = new Register(STM32F4xx.FSMC_Bank2_R_BASE + PCR2);
            SR2_Register = new Register(STM32F4xx.FSMC_Bank2_R_BASE + SR2);
            PMEM2_Register = new Register(STM32F4xx.FSMC_Bank2_R_BASE + PMEM2);
            PATT2_Register = new Register(STM32F4xx.FSMC_Bank2_R_BASE + PATT2);
            ECCR2_Register = new Register(STM32F4xx.FSMC_Bank2_R_BASE + ECCR2);
        }
    } // END of Class...

    /// <summary>
    /// Flexible static memory controller (FSMC) - See: P.1522
    /// </summary>
    public static class FSMC_Bank3
    {
        #region Private's...

        /// <summary>
        /// NAND Flash control register 3, Address offset: 0x80 
        /// </summary>
        private static uint _PCR3 = 0x80;

        /// <summary>
        /// NAND Flash FIFO status and interrupt register 3, Address offset: 0x84 
        /// </summary>
        private static uint _SR3 = 0x84;

        /// <summary>
        /// NAND Flash Common memory space timing register 3,Address offset: 0x88 
        /// </summary>
        private static uint _PMEM3 = 0x88;

        /// <summary>
        /// NAND Flash Attribute memory space timing register 3, Address offset: 0x8C 
        /// </summary>
        private static uint _PATT3 = 0x8C;

        /// <summary>
        /// NAND Flash ECC result registers 3, Address offset: 0x94 
        /// </summary>
        private static uint _ECCR3 = 0x94;

        #endregion

        #region Properties...

        /// <summary>
        /// NAND Flash control register 3, Address offset: 0x80 
        /// </summary>
        public static uint PCR3 { get { return _PCR3; } set { _PCR3 = value; } }

        /// <summary>
        /// NAND Flash FIFO status and interrupt register 3, Address offset: 0x84 
        /// </summary>
        public static uint SR3 { get { return _SR3; } set { _SR3 = value; } }

        /// <summary>
        /// NAND Flash Common memory space timing register 3,Address offset: 0x88 
        /// </summary>
        public static uint PMEM3 { get { return _PMEM3; } set { _PMEM3 = value; } }

        /// <summary>
        /// NAND Flash Attribute memory space timing register 3, Address offset: 0x8C 
        /// </summary>
        public static uint PATT3 { get { return _PATT3; } set { _PATT3 = value; } }

        /// <summary>
        /// NAND Flash ECC result registers 3, Address offset: 0x94 
        /// </summary>
        public static uint ECCR3 { get { return _ECCR3; } set { _ECCR3 = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// NAND Flash control register 3, Address offset: 0x80 
        /// </summary>
        public static Register PCR3_Register;

        /// <summary>
        /// NAND Flash FIFO status and interrupt register 3, Address offset: 0x84 
        /// </summary>
        public static Register SR3_Register;

        /// <summary>
        /// NAND Flash Common memory space timing register 3,Address offset: 0x88 
        /// </summary>
        public static Register PMEM3_Register;

        /// <summary>
        /// NAND Flash Attribute memory space timing register 3, Address offset: 0x8C 
        /// </summary>
        public static Register PATT3_Register;

        /// <summary>
        /// NAND Flash ECC result registers 3, Address offset: 0x94 
        /// </summary>
        public static Register ECCR3_Register;

        #endregion

        static FSMC_Bank3()
        {
            PCR3_Register = new Register(STM32F4xx.FSMC_Bank3_R_BASE + PCR3);
            SR3_Register = new Register(STM32F4xx.FSMC_Bank3_R_BASE + SR3);
            PMEM3_Register = new Register(STM32F4xx.FSMC_Bank3_R_BASE + PMEM3);
            PATT3_Register = new Register(STM32F4xx.FSMC_Bank3_R_BASE + PATT3);
            ECCR3_Register = new Register(STM32F4xx.FSMC_Bank3_R_BASE + ECCR3);
        }
    } // END of Class...

    /// <summary>
    /// Flexible static memory controller (FSMC) - See: P.1522
    /// </summary>
    public static class FSMC_Bank4
    {
        #region Private's...

        /// <summary>
        /// PC Cardcontrol register 4, Address offset: 0xA0 
        /// </summary>
        private static uint _PCR4 = 0xA0;

        /// <summary>
        /// PC CardFIFO status and interrupt register 4, Address offset: 0xA4 
        /// </summary>
        private static uint _SR4 = 0xA4;

        /// <summary>
        /// PC CardCommon memory space timing register 4,Address offset: 0xA8 
        /// </summary>
        private static uint _PMEM4 = 0xA8;

        /// <summary>
        /// PC CardAttribute memory space timing register 4, Address offset: 0xAC 
        /// </summary>
        private static uint _PATT4 = 0xAC;

        /// <summary>
        /// PC CardI/O space timing register 4,Address offset: 0xB0 
        /// </summary>
        private static uint _PIO4 = 0xB0;

        #endregion

        #region Properties...

        /// <summary>
        /// PC Cardcontrol register 4, Address offset: 0xA0 
        /// </summary>
        public static uint PCR4 { get { return _PCR4; } set { _PCR4 = value; } }

        /// <summary>
        /// PC CardFIFO status and interrupt register 4, Address offset: 0xA4 
        /// </summary>
        public static uint SR4 { get { return _SR4; } set { _SR4 = value; } }

        /// <summary>
        /// PC CardCommon memory space timing register 4,Address offset: 0xA8 
        /// </summary>
        public static uint PMEM4 { get { return _PMEM4; } set { _PMEM4 = value; } }

        /// <summary>
        /// PC CardAttribute memory space timing register 4, Address offset: 0xAC 
        /// </summary>
        public static uint PATT4 { get { return _PATT4; } set { _PATT4 = value; } }

        /// <summary>
        /// PC CardI/O space timing register 4,Address offset: 0xB0 
        /// </summary>
        public static uint PIO4 { get { return _PIO4; } set { _PIO4 = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// PC Cardcontrol register 4, Address offset: 0xA0 
        /// </summary>
        public static Register PCR4_Register;

        /// <summary>
        /// PC CardFIFO status and interrupt register 4, Address offset: 0xA4 
        /// </summary>
        public static Register SR4_Register;

        /// <summary>
        /// PC CardCommon memory space timing register 4,Address offset: 0xA8 
        /// </summary>
        public static Register PMEM4_Register;

        /// <summary>
        /// PC CardAttribute memory space timing register 4, Address offset: 0xAC 
        /// </summary>
        public static Register PATT4_Register;

        /// <summary>
        /// PC CardI/O space timing register 4,Address offset: 0xB0 
        /// </summary>
        public static Register PIO4_Register;

        #endregion

        static FSMC_Bank4()
        {
            PCR4_Register = new Register(STM32F4xx.FSMC_Bank4_R_BASE + PCR4);
            SR4_Register = new Register(STM32F4xx.FSMC_Bank4_R_BASE + SR4);
            PMEM4_Register = new Register(STM32F4xx.FSMC_Bank4_R_BASE + PMEM4);
            PATT4_Register = new Register(STM32F4xx.FSMC_Bank4_R_BASE + PATT4);
            PIO4_Register = new Register(STM32F4xx.FSMC_Bank4_R_BASE + PIO4);
        }
    } // END of Class...

    /// <summary>
    /// MCU debug component (DBGMCU) - See: P.1680
    /// </summary>
    public sealed class DBGMCU
    {
        #region Private's...

        /// <summary>
        /// MCU device ID code, Address offset: 0x00 
        /// </summary>
        private static uint _IDCODE = 0x00;

        /// <summary>
        /// Debug MCU configuration register, Address offset: 0x04 
        /// </summary>
        private static uint _CR = 0x04;

        /// <summary>
        /// Debug MCU APB1 freeze register, Address offset: 0x08 
        /// </summary>
        private static uint _APB1FZ = 0x08;

        /// <summary>
        /// Debug MCU APB2 freeze register, Address offset: 0x0C 
        /// </summary>
        private static uint _APB2FZ = 0x0C;

        #endregion

        #region Properties...

        /// <summary>
        /// MCU device ID code, Address offset: 0x00 
        /// </summary>
        public static uint IDCODE { get { return _IDCODE; } set { _IDCODE = value; } }

        /// <summary>
        /// Debug MCU configuration register, Address offset: 0x04 
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// Debug MCU APB1 freeze register, Address offset: 0x08 
        /// </summary>
        public static uint APB1FZ { get { return _APB1FZ; } set { _APB1FZ = value; } }

        /// <summary>
        /// Debug MCU APB2 freeze register, Address offset: 0x0C 
        /// </summary>
        public static uint APB2FZ { get { return _APB2FZ; } set { _APB2FZ = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// MCU device ID code, Address offset: 0x00 
        /// </summary>
        public static Register IDCODE_Register;

        /// <summary>
        /// Debug MCU configuration register, Address offset: 0x04 
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// Debug MCU APB1 freeze register, Address offset: 0x08 
        /// </summary>
        public static Register APB1FZ_Register;

        /// <summary>
        /// Debug MCU APB2 freeze register, Address offset: 0x0C 
        /// </summary>
        public static Register APB2FZ_Register;

        #endregion

        public DBGMCU()
        {
            IDCODE_Register = new Register(STM32F4xx.DBGMCU_BASE + IDCODE);
            CR_Register = new Register(STM32F4xx.DBGMCU_BASE + CR);
            APB1FZ_Register = new Register(STM32F4xx.DBGMCU_BASE + APB1FZ);
            APB2FZ_Register = new Register(STM32F4xx.DBGMCU_BASE + APB2FZ);
        }
    } // END of Class...


    /// <summary>
    /// CMSIS Cortex-M4 Device Peripheral Access Layer Header File. 
    /// This file contains all the peripheral register's definitions, bits 
    /// definitions and memory mapping for STM32F4xx devices.
    /// Converted to C# by Chris Sykes...
    /// </summary>
    public class STM32F4xx
    {
        /*-------------------------------------------------------------(BASE)-------------------------------------------------------------*/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 SRAM_BASE = SRAM1_BASE;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 SRAM_BB_BASE = SRAM1_BB_BASE;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 APB1PERIPH_BASE = PERIPH_BASE;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 APB2PERIPH_BASE = (PERIPH_BASE + 0x00010000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 AHB1PERIPH_BASE = (PERIPH_BASE + 0x00020000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 AHB2PERIPH_BASE = (PERIPH_BASE + 0x10000000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM2_BASE = (APB1PERIPH_BASE + 0x0000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM3_BASE = (APB1PERIPH_BASE + 0x0400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM4_BASE = (APB1PERIPH_BASE + 0x0800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM5_BASE = (APB1PERIPH_BASE + 0x0C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM6_BASE = (APB1PERIPH_BASE + 0x1000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM7_BASE = (APB1PERIPH_BASE + 0x1400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM12_BASE = (APB1PERIPH_BASE + 0x1800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM13_BASE = (APB1PERIPH_BASE + 0x1C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM14_BASE = (APB1PERIPH_BASE + 0x2000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BASE = (APB1PERIPH_BASE + 0x2800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 WWDG_BASE = (APB1PERIPH_BASE + 0x2C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 IWDG_BASE = (APB1PERIPH_BASE + 0x3000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 I2S2ext_BASE = (APB1PERIPH_BASE + 0x3400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 SPI2_BASE = (APB1PERIPH_BASE + 0x3800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 SPI3_BASE = (APB1PERIPH_BASE + 0x3C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 I2S3ext_BASE = (APB1PERIPH_BASE + 0x4000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 USART2_BASE = (APB1PERIPH_BASE + 0x4400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 USART3_BASE = (APB1PERIPH_BASE + 0x4800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 UART4_BASE = (APB1PERIPH_BASE + 0x4C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 UART5_BASE = (APB1PERIPH_BASE + 0x5000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 I2C1_BASE = (APB1PERIPH_BASE + 0x5400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 I2C2_BASE = (APB1PERIPH_BASE + 0x5800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 I2C3_BASE = (APB1PERIPH_BASE + 0x5C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CAN1_BASE = (APB1PERIPH_BASE + 0x6400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CAN2_BASE = (APB1PERIPH_BASE + 0x6800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 PWR_BASE = (APB1PERIPH_BASE + 0x7000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DAC_BASE = (APB1PERIPH_BASE + 0x7400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM1_BASE = (APB2PERIPH_BASE + 0x0000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM8_BASE = (APB2PERIPH_BASE + 0x0400);

        /// <summary>
        /// // (APB2PERIPH_BASE + 0x1000) Cerberus does not use USART1 but
        /// </summary>
        public static UInt32 USART1_BASE = (APB2PERIPH_BASE + 0x1400);

        /// <summary>
        /// // Cerbuino requires USART6 but it can think it is USART1
        /// </summary>
        public static UInt32 USART6_BASE = (APB2PERIPH_BASE + 0x1400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 ADC1_BASE = (APB2PERIPH_BASE + 0x2000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 ADC2_BASE = (APB2PERIPH_BASE + 0x2100);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 ADC3_BASE = (APB2PERIPH_BASE + 0x2200);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 ADC_BASE = (APB2PERIPH_BASE + 0x2300);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 SDIO_BASE = (APB2PERIPH_BASE + 0x2C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 SPI1_BASE = (APB2PERIPH_BASE + 0x3000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 SYSCFG_BASE = (APB2PERIPH_BASE + 0x3800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 EXTI_BASE = (APB2PERIPH_BASE + 0x3C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM9_BASE = (APB2PERIPH_BASE + 0x4000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM10_BASE = (APB2PERIPH_BASE + 0x4400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 TIM11_BASE = (APB2PERIPH_BASE + 0x4800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIOA_BASE = (AHB1PERIPH_BASE + 0x0000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIOB_BASE = (AHB1PERIPH_BASE + 0x0400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIOC_BASE = (AHB1PERIPH_BASE + 0x0800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIOD_BASE = (AHB1PERIPH_BASE + 0x0C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIOE_BASE = (AHB1PERIPH_BASE + 0x1000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIOF_BASE = (AHB1PERIPH_BASE + 0x1400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIOG_BASE = (AHB1PERIPH_BASE + 0x1800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIOH_BASE = (AHB1PERIPH_BASE + 0x1C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIOI_BASE = (AHB1PERIPH_BASE + 0x2000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRC_BASE = (AHB1PERIPH_BASE + 0x3000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_BASE = (AHB1PERIPH_BASE + 0x3800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_R_BASE = (AHB1PERIPH_BASE + 0x3C00);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA1_BASE = (AHB1PERIPH_BASE + 0x6000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA1_Stream0_BASE = (DMA1_BASE + 0x010);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA1_Stream1_BASE = (DMA1_BASE + 0x028);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA1_Stream2_BASE = (DMA1_BASE + 0x040);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA1_Stream3_BASE = (DMA1_BASE + 0x058);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA1_Stream4_BASE = (DMA1_BASE + 0x070);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA1_Stream5_BASE = (DMA1_BASE + 0x088);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA1_Stream6_BASE = (DMA1_BASE + 0x0A0);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA1_Stream7_BASE = (DMA1_BASE + 0x0B8);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA2_BASE = (AHB1PERIPH_BASE + 0x6400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA2_Stream0_BASE = (DMA2_BASE + 0x010);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA2_Stream1_BASE = (DMA2_BASE + 0x028);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA2_Stream2_BASE = (DMA2_BASE + 0x040);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA2_Stream3_BASE = (DMA2_BASE + 0x058);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA2_Stream4_BASE = (DMA2_BASE + 0x070);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA2_Stream5_BASE = (DMA2_BASE + 0x088);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA2_Stream6_BASE = (DMA2_BASE + 0x0A0);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA2_Stream7_BASE = (DMA2_BASE + 0x0B8);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 ETH_BASE = (AHB1PERIPH_BASE + 0x8000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 ETH_MAC_BASE = (ETH_BASE);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 ETH_MMC_BASE = (ETH_BASE + 0x0100);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 ETH_PTP_BASE = (ETH_BASE + 0x0700);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 ETH_DMA_BASE = (ETH_BASE + 0x1000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_BASE = (AHB2PERIPH_BASE + 0x50000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_BASE = (AHB2PERIPH_BASE + 0x60000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_BASE = (AHB2PERIPH_BASE + 0x60400);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RNG_BASE = (AHB2PERIPH_BASE + 0x60800);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FSMC_Bank1_R_BASE = (FSMC_R_BASE + 0x0000);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FSMC_Bank1E_R_BASE = (FSMC_R_BASE + 0x0104);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FSMC_Bank2_R_BASE = (FSMC_R_BASE + 0x0060);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FSMC_Bank3_R_BASE = (FSMC_R_BASE + 0x0080);

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FSMC_Bank4_R_BASE = (FSMC_R_BASE + 0x00A0);


        /*-------------------------------------------------------------(Code)-------------------------------------------------------------*/



        /**
          * @}
          */

        /** @addtogroup Peripheral_memory_map
          * @{
          */
        /// <summary>
        ///   FLASH(up to 1 MB) base address in the alias region                         
        /// </summary>
        public static UInt32 FLASH_BASE = 0x08000000;

        /// <summary>
        ///   CCM(core coupled memory) data RAM(64 KB) base address in the alias region  
        /// </summary>
        public static UInt32 CCMDATARAM_BASE = 0x10000000;

        /// <summary>
        ///   SRAM1(112 KB) base address in the alias region                             
        /// </summary>
        public static UInt32 SRAM1_BASE = 0x20000000;

        /// <summary>
        ///   SRAM2(16 KB) base address in the alias region                              
        /// </summary>
        public static UInt32 SRAM2_BASE = 0x2001C000;

        /// <summary>
        ///   Peripheral base address in the alias region                                
        /// </summary>
        public static UInt32 PERIPH_BASE = 0x40000000;

        /// <summary>
        ///   Backup SRAM(4 KB) base address in the alias region                         
        /// </summary>
        public static UInt32 BKPSRAM_BASE = 0x40024000;

        /// <summary>
        ///   FSMC registers base address                                                
        /// </summary>
        public static UInt32 FSMC_R_BASE = 0xA0000000;


        /// <summary>
        ///   CCM(core coupled memory) data RAM(64 KB) base address in the bit-band region  
        /// </summary>
        public static UInt32 CCMDATARAM_BB_BASE = 0x12000000;

        /// <summary>
        ///   SRAM1(112 KB) base address in the bit-band region                             
        /// </summary>
        public static UInt32 SRAM1_BB_BASE = 0x22000000;

        /// <summary>
        ///   SRAM2(16 KB) base address in the bit-band region                              
        /// </summary>
        public static UInt32 SRAM2_BB_BASE = 0x2201C000;

        /// <summary>
        ///   Peripheral base address in the bit-band region                                
        /// </summary>
        public static UInt32 PERIPH_BB_BASE = 0x42000000;

        /// <summary>
        ///   Backup SRAM(4 KB) base address in the bit-band region                         
        /// </summary>
        public static UInt32 BKPSRAM_BB_BASE = 0x42024000;


        /* Legacy defines */


        /*!< Peripheral memory map */

        /*!< APB1 peripherals */

        /*!< APB2 peripherals */

        /*!< AHB1 peripherals */

        /*!< AHB2 peripherals */

        /*!< FSMC Bankx registers base address */

        /* Debug MCU registers base address */
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_BASE = 0xE0042000;


        /**
          * @}
          */

        /** @addtogroup Peripheral_declaration
          * @{
          */

        /**
          * @}
          */

        /** @addtogroup Exported_constants
          * @{
          */

        /** @addtogroup Peripheral_Registers_Bits_Definition
        * @{
        */

        /******************************************************************************/
        /*                         Peripheral Registers_Bits_Definition               */
        /******************************************************************************/

        /******************************************************************************/
        /*                                                                            */
        /*                        Analog to Digital Converter                         */
        /*                                                                            */
        /******************************************************************************/
        /********************  Bit definition for ADC_SR register  ********************/
        /// <summary>
        ///  Analog watchdog flag 
        /// </summary>
        public static byte ADC_SR_AWD = 0x01;

        /// <summary>
        ///  End of conversion 
        /// </summary>
        public static byte ADC_SR_EOC = 0x02;

        /// <summary>
        ///  Injected channel end of conversion 
        /// </summary>
        public static byte ADC_SR_JEOC = 0x04;

        /// <summary>
        ///  Injected channel Start flag 
        /// </summary>
        public static byte ADC_SR_JSTRT = 0x08;

        /// <summary>
        ///  Regular channel Start flag 
        /// </summary>
        public static byte ADC_SR_STRT = 0x10;

        /// <summary>
        ///  Overrun flag 
        /// </summary>
        public static byte ADC_SR_OVR = 0x20;


        /*******************  Bit definition for ADC_CR1 register  ********************/
        /// <summary>
        ///  AWDCH[4:0] bits (Analog watchdog channel select bits) 
        /// </summary>
        public static UInt32 ADC_CR1_AWDCH = 0x0000001F;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_CR1_AWDCH_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_CR1_AWDCH_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_CR1_AWDCH_2 = 0x00000004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_CR1_AWDCH_3 = 0x00000008;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_CR1_AWDCH_4 = 0x00000010;

        /// <summary>
        ///  Interrupt enable for EOC 
        /// </summary>
        public static UInt32 ADC_CR1_EOCIE = 0x00000020;

        /// <summary>
        ///  AAnalog Watchdog interrupt enable 
        /// </summary>
        public static UInt32 ADC_CR1_AWDIE = 0x00000040;

        /// <summary>
        ///  Interrupt enable for injected channels 
        /// </summary>
        public static UInt32 ADC_CR1_JEOCIE = 0x00000080;

        /// <summary>
        ///  Scan mode 
        /// </summary>
        public static UInt32 ADC_CR1_SCAN = 0x00000100;

        /// <summary>
        ///  Enable the watchdog on a single channel in scan mode 
        /// </summary>
        public static UInt32 ADC_CR1_AWDSGL = 0x00000200;

        /// <summary>
        ///  Automatic injected group conversion 
        /// </summary>
        public static UInt32 ADC_CR1_JAUTO = 0x00000400;

        /// <summary>
        ///  Discontinuous mode on regular channels 
        /// </summary>
        public static UInt32 ADC_CR1_DISCEN = 0x00000800;

        /// <summary>
        ///  Discontinuous mode on injected channels 
        /// </summary>
        public static UInt32 ADC_CR1_JDISCEN = 0x00001000;

        /// <summary>
        ///  DISCNUM[2:0] bits (Discontinuous mode channel count) 
        /// </summary>
        public static UInt32 ADC_CR1_DISCNUM = 0x0000E000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_CR1_DISCNUM_0 = 0x00002000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_CR1_DISCNUM_1 = 0x00004000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_CR1_DISCNUM_2 = 0x00008000;

        /// <summary>
        ///  Analog watchdog enable on injected channels 
        /// </summary>
        public static UInt32 ADC_CR1_JAWDEN = 0x00400000;

        /// <summary>
        ///  Analog watchdog enable on regular channels 
        /// </summary>
        public static UInt32 ADC_CR1_AWDEN = 0x00800000;

        /// <summary>
        ///  RES[2:0] bits (Resolution) 
        /// </summary>
        public static UInt32 ADC_CR1_RES = 0x03000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_CR1_RES_0 = 0x01000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_CR1_RES_1 = 0x02000000;

        /// <summary>
        ///  overrun interrupt enable 
        /// </summary>
        public static UInt32 ADC_CR1_OVRIE = 0x04000000;


        /*******************  Bit definition for ADC_CR2 register  ********************/
        /// <summary>
        ///  A/D Converter ON / OFF 
        /// </summary>
        public static UInt32 ADC_CR2_ADON = 0x00000001;

        /// <summary>
        ///  Continuous Conversion 
        /// </summary>
        public static UInt32 ADC_CR2_CONT = 0x00000002;

        /// <summary>
        ///  Direct Memory access mode 
        /// </summary>
        public static UInt32 ADC_CR2_DMA = 0x00000100;

        /// <summary>
        ///  DMA disable selection (Single ADC) 
        /// </summary>
        public static UInt32 ADC_CR2_DDS = 0x00000200;

        /// <summary>
        ///  End of conversion selection 
        /// </summary>
        public static UInt32 ADC_CR2_EOCS = 0x00000400;

        /// <summary>
        ///  Data Alignment 
        /// </summary>
        public static UInt32 ADC_CR2_ALIGN = 0x00000800;

        /// <summary>
        ///  JEXTSEL[3:0] bits (External event select for injected group) 
        /// </summary>
        public static UInt32 ADC_CR2_JEXTSEL = 0x000F0000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_CR2_JEXTSEL_0 = 0x00010000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_CR2_JEXTSEL_1 = 0x00020000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_CR2_JEXTSEL_2 = 0x00040000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_CR2_JEXTSEL_3 = 0x00080000;

        /// <summary>
        ///  JEXTEN[1:0] bits (External Trigger Conversion mode for injected channelsp) 
        /// </summary>
        public static UInt32 ADC_CR2_JEXTEN = 0x00300000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_CR2_JEXTEN_0 = 0x00100000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_CR2_JEXTEN_1 = 0x00200000;

        /// <summary>
        ///  Start Conversion of injected channels 
        /// </summary>
        public static UInt32 ADC_CR2_JSWSTART = 0x00400000;

        /// <summary>
        ///  EXTSEL[3:0] bits (External Event Select for regular group) 
        /// </summary>
        public static UInt32 ADC_CR2_EXTSEL = 0x0F000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_CR2_EXTSEL_0 = 0x01000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_CR2_EXTSEL_1 = 0x02000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_CR2_EXTSEL_2 = 0x04000000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_CR2_EXTSEL_3 = 0x08000000;

        /// <summary>
        ///  EXTEN[1:0] bits (External Trigger Conversion mode for regular channelsp) 
        /// </summary>
        public static UInt32 ADC_CR2_EXTEN = 0x30000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_CR2_EXTEN_0 = 0x10000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_CR2_EXTEN_1 = 0x20000000;

        /// <summary>
        ///  Start Conversion of regular channels 
        /// </summary>
        public static UInt32 ADC_CR2_SWSTART = 0x40000000;


        /******************  Bit definition for ADC_SMPR1 register  *******************/
        /// <summary>
        ///  SMP10[2:0] bits (Channel 10 Sample time selection) 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP10 = 0x00000007;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP10_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP10_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP10_2 = 0x00000004;

        /// <summary>
        ///  SMP11[2:0] bits (Channel 11 Sample time selection) 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP11 = 0x00000038;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP11_0 = 0x00000008;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP11_1 = 0x00000010;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP11_2 = 0x00000020;

        /// <summary>
        ///  SMP12[2:0] bits (Channel 12 Sample time selection) 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP12 = 0x000001C0;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP12_0 = 0x00000040;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP12_1 = 0x00000080;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP12_2 = 0x00000100;

        /// <summary>
        ///  SMP13[2:0] bits (Channel 13 Sample time selection) 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP13 = 0x00000E00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP13_0 = 0x00000200;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP13_1 = 0x00000400;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP13_2 = 0x00000800;

        /// <summary>
        ///  SMP14[2:0] bits (Channel 14 Sample time selection) 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP14 = 0x00007000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP14_0 = 0x00001000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP14_1 = 0x00002000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP14_2 = 0x00004000;

        /// <summary>
        ///  SMP15[2:0] bits (Channel 15 Sample time selection) 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP15 = 0x00038000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP15_0 = 0x00008000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP15_1 = 0x00010000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP15_2 = 0x00020000;

        /// <summary>
        ///  SMP16[2:0] bits (Channel 16 Sample time selection) 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP16 = 0x001C0000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP16_0 = 0x00040000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP16_1 = 0x00080000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP16_2 = 0x00100000;

        /// <summary>
        ///  SMP17[2:0] bits (Channel 17 Sample time selection) 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP17 = 0x00E00000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP17_0 = 0x00200000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP17_1 = 0x00400000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP17_2 = 0x00800000;

        /// <summary>
        ///  SMP18[2:0] bits (Channel 18 Sample time selection) 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP18 = 0x07000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP18_0 = 0x01000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP18_1 = 0x02000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SMPR1_SMP18_2 = 0x04000000;


        /******************  Bit definition for ADC_SMPR2 register  *******************/
        /// <summary>
        ///  SMP0[2:0] bits (Channel 0 Sample time selection) 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP0 = 0x00000007;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP0_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP0_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP0_2 = 0x00000004;

        /// <summary>
        ///  SMP1[2:0] bits (Channel 1 Sample time selection) 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP1 = 0x00000038;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP1_0 = 0x00000008;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP1_1 = 0x00000010;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP1_2 = 0x00000020;

        /// <summary>
        ///  SMP2[2:0] bits (Channel 2 Sample time selection) 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP2 = 0x000001C0;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP2_0 = 0x00000040;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP2_1 = 0x00000080;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP2_2 = 0x00000100;

        /// <summary>
        ///  SMP3[2:0] bits (Channel 3 Sample time selection) 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP3 = 0x00000E00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP3_0 = 0x00000200;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP3_1 = 0x00000400;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP3_2 = 0x00000800;

        /// <summary>
        ///  SMP4[2:0] bits (Channel 4 Sample time selection) 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP4 = 0x00007000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP4_0 = 0x00001000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP4_1 = 0x00002000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP4_2 = 0x00004000;

        /// <summary>
        ///  SMP5[2:0] bits (Channel 5 Sample time selection) 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP5 = 0x00038000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP5_0 = 0x00008000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP5_1 = 0x00010000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP5_2 = 0x00020000;

        /// <summary>
        ///  SMP6[2:0] bits (Channel 6 Sample time selection) 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP6 = 0x001C0000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP6_0 = 0x00040000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP6_1 = 0x00080000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP6_2 = 0x00100000;

        /// <summary>
        ///  SMP7[2:0] bits (Channel 7 Sample time selection) 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP7 = 0x00E00000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP7_0 = 0x00200000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP7_1 = 0x00400000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP7_2 = 0x00800000;

        /// <summary>
        ///  SMP8[2:0] bits (Channel 8 Sample time selection) 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP8 = 0x07000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP8_0 = 0x01000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP8_1 = 0x02000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP8_2 = 0x04000000;

        /// <summary>
        ///  SMP9[2:0] bits (Channel 9 Sample time selection) 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP9 = 0x38000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP9_0 = 0x08000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP9_1 = 0x10000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SMPR2_SMP9_2 = 0x20000000;


        /******************  Bit definition for ADC_JOFR1 register  *******************/
        /// <summary>
        ///  Data offset for injected channel 1 
        /// </summary>
        public static UInt16 ADC_JOFR1_JOFFSET1 = 0x0FFF;


        /******************  Bit definition for ADC_JOFR2 register  *******************/
        /// <summary>
        ///  Data offset for injected channel 2 
        /// </summary>
        public static UInt16 ADC_JOFR2_JOFFSET2 = 0x0FFF;


        /******************  Bit definition for ADC_JOFR3 register  *******************/
        /// <summary>
        ///  Data offset for injected channel 3 
        /// </summary>
        public static UInt16 ADC_JOFR3_JOFFSET3 = 0x0FFF;


        /******************  Bit definition for ADC_JOFR4 register  *******************/
        /// <summary>
        ///  Data offset for injected channel 4 
        /// </summary>
        public static UInt16 ADC_JOFR4_JOFFSET4 = 0x0FFF;


        /*******************  Bit definition for ADC_HTR register  ********************/
        /// <summary>
        ///  Analog watchdog high threshold 
        /// </summary>
        public static UInt16 ADC_HTR_HT = 0x0FFF;


        /*******************  Bit definition for ADC_LTR register  ********************/
        /// <summary>
        ///  Analog watchdog low threshold 
        /// </summary>
        public static UInt16 ADC_LTR_LT = 0x0FFF;


        /*******************  Bit definition for ADC_SQR1 register  *******************/
        /// <summary>
        ///  SQ13[4:0] bits (13th conversion in regular sequence) 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ13 = 0x0000001F;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ13_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ13_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ13_2 = 0x00000004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ13_3 = 0x00000008;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ13_4 = 0x00000010;

        /// <summary>
        ///  SQ14[4:0] bits (14th conversion in regular sequence) 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ14 = 0x000003E0;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ14_0 = 0x00000020;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ14_1 = 0x00000040;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ14_2 = 0x00000080;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ14_3 = 0x00000100;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ14_4 = 0x00000200;

        /// <summary>
        ///  SQ15[4:0] bits (15th conversion in regular sequence) 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ15 = 0x00007C00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ15_0 = 0x00000400;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ15_1 = 0x00000800;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ15_2 = 0x00001000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ15_3 = 0x00002000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ15_4 = 0x00004000;

        /// <summary>
        ///  SQ16[4:0] bits (16th conversion in regular sequence) 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ16 = 0x000F8000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ16_0 = 0x00008000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ16_1 = 0x00010000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ16_2 = 0x00020000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ16_3 = 0x00040000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_SQR1_SQ16_4 = 0x00080000;

        /// <summary>
        ///  L[3:0] bits (Regular channel sequence length) 
        /// </summary>
        public static UInt32 ADC_SQR1_L = 0x00F00000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SQR1_L_0 = 0x00100000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SQR1_L_1 = 0x00200000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SQR1_L_2 = 0x00400000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_SQR1_L_3 = 0x00800000;


        /*******************  Bit definition for ADC_SQR2 register  *******************/
        /// <summary>
        ///  SQ7[4:0] bits (7th conversion in regular sequence) 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ7 = 0x0000001F;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ7_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ7_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ7_2 = 0x00000004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ7_3 = 0x00000008;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ7_4 = 0x00000010;

        /// <summary>
        ///  SQ8[4:0] bits (8th conversion in regular sequence) 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ8 = 0x000003E0;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ8_0 = 0x00000020;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ8_1 = 0x00000040;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ8_2 = 0x00000080;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ8_3 = 0x00000100;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ8_4 = 0x00000200;

        /// <summary>
        ///  SQ9[4:0] bits (9th conversion in regular sequence) 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ9 = 0x00007C00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ9_0 = 0x00000400;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ9_1 = 0x00000800;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ9_2 = 0x00001000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ9_3 = 0x00002000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ9_4 = 0x00004000;

        /// <summary>
        ///  SQ10[4:0] bits (10th conversion in regular sequence) 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ10 = 0x000F8000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ10_0 = 0x00008000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ10_1 = 0x00010000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ10_2 = 0x00020000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ10_3 = 0x00040000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ10_4 = 0x00080000;

        /// <summary>
        ///  SQ11[4:0] bits (11th conversion in regular sequence) 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ11 = 0x01F00000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ11_0 = 0x00100000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ11_1 = 0x00200000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ11_2 = 0x00400000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ11_3 = 0x00800000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ11_4 = 0x01000000;

        /// <summary>
        ///  SQ12[4:0] bits (12th conversion in regular sequence) 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ12 = 0x3E000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ12_0 = 0x02000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ12_1 = 0x04000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ12_2 = 0x08000000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ12_3 = 0x10000000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_SQR2_SQ12_4 = 0x20000000;


        /*******************  Bit definition for ADC_SQR3 register  *******************/
        /// <summary>
        ///  SQ1[4:0] bits (1st conversion in regular sequence) 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ1 = 0x0000001F;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ1_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ1_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ1_2 = 0x00000004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ1_3 = 0x00000008;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ1_4 = 0x00000010;

        /// <summary>
        ///  SQ2[4:0] bits (2nd conversion in regular sequence) 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ2 = 0x000003E0;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ2_0 = 0x00000020;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ2_1 = 0x00000040;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ2_2 = 0x00000080;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ2_3 = 0x00000100;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ2_4 = 0x00000200;

        /// <summary>
        ///  SQ3[4:0] bits (3rd conversion in regular sequence) 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ3 = 0x00007C00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ3_0 = 0x00000400;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ3_1 = 0x00000800;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ3_2 = 0x00001000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ3_3 = 0x00002000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ3_4 = 0x00004000;

        /// <summary>
        ///  SQ4[4:0] bits (4th conversion in regular sequence) 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ4 = 0x000F8000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ4_0 = 0x00008000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ4_1 = 0x00010000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ4_2 = 0x00020000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ4_3 = 0x00040000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ4_4 = 0x00080000;

        /// <summary>
        ///  SQ5[4:0] bits (5th conversion in regular sequence) 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ5 = 0x01F00000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ5_0 = 0x00100000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ5_1 = 0x00200000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ5_2 = 0x00400000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ5_3 = 0x00800000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ5_4 = 0x01000000;

        /// <summary>
        ///  SQ6[4:0] bits (6th conversion in regular sequence) 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ6 = 0x3E000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ6_0 = 0x02000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ6_1 = 0x04000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ6_2 = 0x08000000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ6_3 = 0x10000000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_SQR3_SQ6_4 = 0x20000000;


        /*******************  Bit definition for ADC_JSQR register  *******************/
        /// <summary>
        ///  JSQ1[4:0] bits (1st conversion in injected sequence)   
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ1 = 0x0000001F;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ1_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ1_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ1_2 = 0x00000004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ1_3 = 0x00000008;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ1_4 = 0x00000010;

        /// <summary>
        ///  JSQ2[4:0] bits (2nd conversion in injected sequence) 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ2 = 0x000003E0;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ2_0 = 0x00000020;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ2_1 = 0x00000040;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ2_2 = 0x00000080;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ2_3 = 0x00000100;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ2_4 = 0x00000200;

        /// <summary>
        ///  JSQ3[4:0] bits (3rd conversion in injected sequence) 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ3 = 0x00007C00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ3_0 = 0x00000400;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ3_1 = 0x00000800;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ3_2 = 0x00001000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ3_3 = 0x00002000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ3_4 = 0x00004000;

        /// <summary>
        ///  JSQ4[4:0] bits (4th conversion in injected sequence) 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ4 = 0x000F8000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ4_0 = 0x00008000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ4_1 = 0x00010000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ4_2 = 0x00020000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ4_3 = 0x00040000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_JSQR_JSQ4_4 = 0x00080000;

        /// <summary>
        ///  JL[1:0] bits (Injected Sequence length) 
        /// </summary>
        public static UInt32 ADC_JSQR_JL = 0x00300000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_JSQR_JL_0 = 0x00100000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_JSQR_JL_1 = 0x00200000;


        /*******************  Bit definition for ADC_JDR1 register  *******************/
        /// <summary>
        ///  Injected data 
        /// </summary>
        public static UInt16 ADC_JDR1_JDATA = 0xFFFF;


        /*******************  Bit definition for ADC_JDR2 register  *******************/
        /// <summary>
        ///  Injected data 
        /// </summary>
        public static UInt16 ADC_JDR2_JDATA = 0xFFFF;


        /*******************  Bit definition for ADC_JDR3 register  *******************/
        /// <summary>
        ///  Injected data 
        /// </summary>
        public static UInt16 ADC_JDR3_JDATA = 0xFFFF;


        /*******************  Bit definition for ADC_JDR4 register  *******************/
        /// <summary>
        ///  Injected data 
        /// </summary>
        public static UInt16 ADC_JDR4_JDATA = 0xFFFF;


        /********************  Bit definition for ADC_DR register  ********************/
        /// <summary>
        ///  Regular data 
        /// </summary>
        public static UInt32 ADC_DR_DATA = 0x0000FFFF;

        /// <summary>
        ///  ADC2 data 
        /// </summary>
        public static UInt32 ADC_DR_ADC2DATA = 0xFFFF0000;


        /*******************  Bit definition for ADC_CSR register  ********************/
        /// <summary>
        ///  ADC1 Analog watchdog flag 
        /// </summary>
        public static UInt32 ADC_CSR_AWD1 = 0x00000001;

        /// <summary>
        ///  ADC1 End of conversion 
        /// </summary>
        public static UInt32 ADC_CSR_EOC1 = 0x00000002;

        /// <summary>
        ///  ADC1 Injected channel end of conversion 
        /// </summary>
        public static UInt32 ADC_CSR_JEOC1 = 0x00000004;

        /// <summary>
        ///  ADC1 Injected channel Start flag 
        /// </summary>
        public static UInt32 ADC_CSR_JSTRT1 = 0x00000008;

        /// <summary>
        ///  ADC1 Regular channel Start flag 
        /// </summary>
        public static UInt32 ADC_CSR_STRT1 = 0x00000010;

        /// <summary>
        ///  ADC1 DMA overrun  flag 
        /// </summary>
        public static UInt32 ADC_CSR_DOVR1 = 0x00000020;

        /// <summary>
        ///  ADC2 Analog watchdog flag 
        /// </summary>
        public static UInt32 ADC_CSR_AWD2 = 0x00000100;

        /// <summary>
        ///  ADC2 End of conversion 
        /// </summary>
        public static UInt32 ADC_CSR_EOC2 = 0x00000200;

        /// <summary>
        ///  ADC2 Injected channel end of conversion 
        /// </summary>
        public static UInt32 ADC_CSR_JEOC2 = 0x00000400;

        /// <summary>
        ///  ADC2 Injected channel Start flag 
        /// </summary>
        public static UInt32 ADC_CSR_JSTRT2 = 0x00000800;

        /// <summary>
        ///  ADC2 Regular channel Start flag 
        /// </summary>
        public static UInt32 ADC_CSR_STRT2 = 0x00001000;

        /// <summary>
        ///  ADC2 DMA overrun  flag 
        /// </summary>
        public static UInt32 ADC_CSR_DOVR2 = 0x00002000;

        /// <summary>
        ///  ADC3 Analog watchdog flag 
        /// </summary>
        public static UInt32 ADC_CSR_AWD3 = 0x00010000;

        /// <summary>
        ///  ADC3 End of conversion 
        /// </summary>
        public static UInt32 ADC_CSR_EOC3 = 0x00020000;

        /// <summary>
        ///  ADC3 Injected channel end of conversion 
        /// </summary>
        public static UInt32 ADC_CSR_JEOC3 = 0x00040000;

        /// <summary>
        ///  ADC3 Injected channel Start flag 
        /// </summary>
        public static UInt32 ADC_CSR_JSTRT3 = 0x00080000;

        /// <summary>
        ///  ADC3 Regular channel Start flag 
        /// </summary>
        public static UInt32 ADC_CSR_STRT3 = 0x00100000;

        /// <summary>
        ///  ADC3 DMA overrun  flag 
        /// </summary>
        public static UInt32 ADC_CSR_DOVR3 = 0x00200000;


        /*******************  Bit definition for ADC_CCR register  ********************/
        /// <summary>
        ///  MULTI[4:0] bits (Multi-ADC mode selection)   
        /// </summary>
        public static UInt32 ADC_CCR_MULTI = 0x0000001F;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_CCR_MULTI_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_CCR_MULTI_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_CCR_MULTI_2 = 0x00000004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_CCR_MULTI_3 = 0x00000008;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 ADC_CCR_MULTI_4 = 0x00000010;

        /// <summary>
        ///  DELAY[3:0] bits (Delay between 2 sampling phases)   
        /// </summary>
        public static UInt32 ADC_CCR_DELAY = 0x00000F00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_CCR_DELAY_0 = 0x00000100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_CCR_DELAY_1 = 0x00000200;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 ADC_CCR_DELAY_2 = 0x00000400;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 ADC_CCR_DELAY_3 = 0x00000800;

        /// <summary>
        ///  DMA disable selection (Multi-ADC mode) 
        /// </summary>
        public static UInt32 ADC_CCR_DDS = 0x00002000;

        /// <summary>
        ///  DMA[1:0] bits (Direct Memory Access mode for multimode)   
        /// </summary>
        public static UInt32 ADC_CCR_DMA = 0x0000C000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_CCR_DMA_0 = 0x00004000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_CCR_DMA_1 = 0x00008000;

        /// <summary>
        ///  ADCPRE[1:0] bits (ADC prescaler)   
        /// </summary>
        public static UInt32 ADC_CCR_ADCPRE = 0x00030000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 ADC_CCR_ADCPRE_0 = 0x00010000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 ADC_CCR_ADCPRE_1 = 0x00020000;

        /// <summary>
        ///  VBAT Enable 
        /// </summary>
        public static UInt32 ADC_CCR_VBATE = 0x00400000;

        /// <summary>
        ///  Temperature Sensor and VREFINT Enable 
        /// </summary>
        public static UInt32 ADC_CCR_TSVREFE = 0x00800000;


        /*******************  Bit definition for ADC_CDR register  ********************/
        /// <summary>
        ///  1st data of a pair of regular conversions 
        /// </summary>
        public static UInt32 ADC_CDR_DATA1 = 0x0000FFFF;

        /// <summary>
        ///  2nd data of a pair of regular conversions 
        /// </summary>
        public static UInt32 ADC_CDR_DATA2 = 0xFFFF0000;


        /******************************************************************************/
        /*                                                                            */
        /*                         Controller Area Network                            */
        /*                                                                            */
        /******************************************************************************/
        /*!<CAN control and status registers */
        /*******************  Bit definition for CAN_MCR register  ********************/
        /// <summary>
        ///  Initialization Request 
        /// </summary>
        public static UInt16 CAN_MCR_INRQ = 0x0001;

        /// <summary>
        ///  Sleep Mode Request 
        /// </summary>
        public static UInt16 CAN_MCR_SLEEP = 0x0002;

        /// <summary>
        ///  Transmit FIFO Priority 
        /// </summary>
        public static UInt16 CAN_MCR_TXFP = 0x0004;

        /// <summary>
        ///  Receive FIFO Locked Mode 
        /// </summary>
        public static UInt16 CAN_MCR_RFLM = 0x0008;

        /// <summary>
        ///  No Automatic Retransmission 
        /// </summary>
        public static UInt16 CAN_MCR_NART = 0x0010;

        /// <summary>
        ///  Automatic Wakeup Mode 
        /// </summary>
        public static UInt16 CAN_MCR_AWUM = 0x0020;

        /// <summary>
        ///  Automatic Bus-Off Management 
        /// </summary>
        public static UInt16 CAN_MCR_ABOM = 0x0040;

        /// <summary>
        ///  Time Triggered Communication Mode 
        /// </summary>
        public static UInt16 CAN_MCR_TTCM = 0x0080;

        /// <summary>
        ///  bxCAN software master reset 
        /// </summary>
        public static UInt16 CAN_MCR_RESET = 0x8000;


        /*******************  Bit definition for CAN_MSR register  ********************/
        /// <summary>
        ///  Initialization Acknowledge 
        /// </summary>
        public static UInt16 CAN_MSR_INAK = 0x0001;

        /// <summary>
        ///  Sleep Acknowledge 
        /// </summary>
        public static UInt16 CAN_MSR_SLAK = 0x0002;

        /// <summary>
        ///  Error Interrupt 
        /// </summary>
        public static UInt16 CAN_MSR_ERRI = 0x0004;

        /// <summary>
        ///  Wakeup Interrupt 
        /// </summary>
        public static UInt16 CAN_MSR_WKUI = 0x0008;

        /// <summary>
        ///  Sleep Acknowledge Interrupt 
        /// </summary>
        public static UInt16 CAN_MSR_SLAKI = 0x0010;

        /// <summary>
        ///  Transmit Mode 
        /// </summary>
        public static UInt16 CAN_MSR_TXM = 0x0100;

        /// <summary>
        ///  Receive Mode 
        /// </summary>
        public static UInt16 CAN_MSR_RXM = 0x0200;

        /// <summary>
        ///  Last Sample Point 
        /// </summary>
        public static UInt16 CAN_MSR_SAMP = 0x0400;

        /// <summary>
        ///  CAN Rx Signal 
        /// </summary>
        public static UInt16 CAN_MSR_RX = 0x0800;


        /*******************  Bit definition for CAN_TSR register  ********************/
        /// <summary>
        ///  Request Completed Mailbox0 
        /// </summary>
        public static UInt32 CAN_TSR_RQCP0 = 0x00000001;

        /// <summary>
        ///  Transmission OK of Mailbox0 
        /// </summary>
        public static UInt32 CAN_TSR_TXOK0 = 0x00000002;

        /// <summary>
        ///  Arbitration Lost for Mailbox0 
        /// </summary>
        public static UInt32 CAN_TSR_ALST0 = 0x00000004;

        /// <summary>
        ///  Transmission Error of Mailbox0 
        /// </summary>
        public static UInt32 CAN_TSR_TERR0 = 0x00000008;

        /// <summary>
        ///  Abort Request for Mailbox0 
        /// </summary>
        public static UInt32 CAN_TSR_ABRQ0 = 0x00000080;

        /// <summary>
        ///  Request Completed Mailbox1 
        /// </summary>
        public static UInt32 CAN_TSR_RQCP1 = 0x00000100;

        /// <summary>
        ///  Transmission OK of Mailbox1 
        /// </summary>
        public static UInt32 CAN_TSR_TXOK1 = 0x00000200;

        /// <summary>
        ///  Arbitration Lost for Mailbox1 
        /// </summary>
        public static UInt32 CAN_TSR_ALST1 = 0x00000400;

        /// <summary>
        ///  Transmission Error of Mailbox1 
        /// </summary>
        public static UInt32 CAN_TSR_TERR1 = 0x00000800;

        /// <summary>
        ///  Abort Request for Mailbox 1 
        /// </summary>
        public static UInt32 CAN_TSR_ABRQ1 = 0x00008000;

        /// <summary>
        ///  Request Completed Mailbox2 
        /// </summary>
        public static UInt32 CAN_TSR_RQCP2 = 0x00010000;

        /// <summary>
        ///  Transmission OK of Mailbox 2 
        /// </summary>
        public static UInt32 CAN_TSR_TXOK2 = 0x00020000;

        /// <summary>
        ///  Arbitration Lost for mailbox 2 
        /// </summary>
        public static UInt32 CAN_TSR_ALST2 = 0x00040000;

        /// <summary>
        ///  Transmission Error of Mailbox 2 
        /// </summary>
        public static UInt32 CAN_TSR_TERR2 = 0x00080000;

        /// <summary>
        ///  Abort Request for Mailbox 2 
        /// </summary>
        public static UInt32 CAN_TSR_ABRQ2 = 0x00800000;

        /// <summary>
        ///  Mailbox Code 
        /// </summary>
        public static UInt32 CAN_TSR_CODE = 0x03000000;


        /// <summary>
        ///  TME[2:0] bits 
        /// </summary>
        public static UInt32 CAN_TSR_TME = 0x1C000000;

        /// <summary>
        ///  Transmit Mailbox 0 Empty 
        /// </summary>
        public static UInt32 CAN_TSR_TME0 = 0x04000000;

        /// <summary>
        ///  Transmit Mailbox 1 Empty 
        /// </summary>
        public static UInt32 CAN_TSR_TME1 = 0x08000000;

        /// <summary>
        ///  Transmit Mailbox 2 Empty 
        /// </summary>
        public static UInt32 CAN_TSR_TME2 = 0x10000000;


        /// <summary>
        ///  LOW[2:0] bits 
        /// </summary>
        public static UInt32 CAN_TSR_LOW = 0xE0000000;

        /// <summary>
        ///  Lowest Priority Flag for Mailbox 0 
        /// </summary>
        public static UInt32 CAN_TSR_LOW0 = 0x20000000;

        /// <summary>
        ///  Lowest Priority Flag for Mailbox 1 
        /// </summary>
        public static UInt32 CAN_TSR_LOW1 = 0x40000000;

        /// <summary>
        ///  Lowest Priority Flag for Mailbox 2 
        /// </summary>
        public static UInt32 CAN_TSR_LOW2 = 0x80000000;


        /*******************  Bit definition for CAN_RF0R register  *******************/
        /// <summary>
        ///  FIFO 0 Message Pending 
        /// </summary>
        public static byte CAN_RF0R_FMP0 = 0x03;

        /// <summary>
        ///  FIFO 0 Full 
        /// </summary>
        public static byte CAN_RF0R_FULL0 = 0x08;

        /// <summary>
        ///  FIFO 0 Overrun 
        /// </summary>
        public static byte CAN_RF0R_FOVR0 = 0x10;

        /// <summary>
        ///  Release FIFO 0 Output Mailbox 
        /// </summary>
        public static byte CAN_RF0R_RFOM0 = 0x20;


        /*******************  Bit definition for CAN_RF1R register  *******************/
        /// <summary>
        ///  FIFO 1 Message Pending 
        /// </summary>
        public static byte CAN_RF1R_FMP1 = 0x03;

        /// <summary>
        ///  FIFO 1 Full 
        /// </summary>
        public static byte CAN_RF1R_FULL1 = 0x08;

        /// <summary>
        ///  FIFO 1 Overrun 
        /// </summary>
        public static byte CAN_RF1R_FOVR1 = 0x10;

        /// <summary>
        ///  Release FIFO 1 Output Mailbox 
        /// </summary>
        public static byte CAN_RF1R_RFOM1 = 0x20;


        /********************  Bit definition for CAN_IER register  *******************/
        /// <summary>
        ///  Transmit Mailbox Empty Interrupt Enable 
        /// </summary>
        public static UInt32 CAN_IER_TMEIE = 0x00000001;

        /// <summary>
        ///  FIFO Message Pending Interrupt Enable 
        /// </summary>
        public static UInt32 CAN_IER_FMPIE0 = 0x00000002;

        /// <summary>
        ///  FIFO Full Interrupt Enable 
        /// </summary>
        public static UInt32 CAN_IER_FFIE0 = 0x00000004;

        /// <summary>
        ///  FIFO Overrun Interrupt Enable 
        /// </summary>
        public static UInt32 CAN_IER_FOVIE0 = 0x00000008;

        /// <summary>
        ///  FIFO Message Pending Interrupt Enable 
        /// </summary>
        public static UInt32 CAN_IER_FMPIE1 = 0x00000010;

        /// <summary>
        ///  FIFO Full Interrupt Enable 
        /// </summary>
        public static UInt32 CAN_IER_FFIE1 = 0x00000020;

        /// <summary>
        ///  FIFO Overrun Interrupt Enable 
        /// </summary>
        public static UInt32 CAN_IER_FOVIE1 = 0x00000040;

        /// <summary>
        ///  Error Warning Interrupt Enable 
        /// </summary>
        public static UInt32 CAN_IER_EWGIE = 0x00000100;

        /// <summary>
        ///  Error Passive Interrupt Enable 
        /// </summary>
        public static UInt32 CAN_IER_EPVIE = 0x00000200;

        /// <summary>
        ///  Bus-Off Interrupt Enable 
        /// </summary>
        public static UInt32 CAN_IER_BOFIE = 0x00000400;

        /// <summary>
        ///  Last Error Code Interrupt Enable 
        /// </summary>
        public static UInt32 CAN_IER_LECIE = 0x00000800;

        /// <summary>
        ///  Error Interrupt Enable 
        /// </summary>
        public static UInt32 CAN_IER_ERRIE = 0x00008000;

        /// <summary>
        ///  Wakeup Interrupt Enable 
        /// </summary>
        public static UInt32 CAN_IER_WKUIE = 0x00010000;

        /// <summary>
        ///  Sleep Interrupt Enable 
        /// </summary>
        public static UInt32 CAN_IER_SLKIE = 0x00020000;


        /********************  Bit definition for CAN_ESR register  *******************/
        /// <summary>
        ///  Error Warning Flag 
        /// </summary>
        public static UInt32 CAN_ESR_EWGF = 0x00000001;

        /// <summary>
        ///  Error Passive Flag 
        /// </summary>
        public static UInt32 CAN_ESR_EPVF = 0x00000002;

        /// <summary>
        ///  Bus-Off Flag 
        /// </summary>
        public static UInt32 CAN_ESR_BOFF = 0x00000004;


        /// <summary>
        ///  LEC[2:0] bits (Last Error Code) 
        /// </summary>
        public static UInt32 CAN_ESR_LEC = 0x00000070;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 CAN_ESR_LEC_0 = 0x00000010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 CAN_ESR_LEC_1 = 0x00000020;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 CAN_ESR_LEC_2 = 0x00000040;


        /// <summary>
        ///  Least significant byte of the 9-bit Transmit Error Counter 
        /// </summary>
        public static UInt32 CAN_ESR_TEC = 0x00FF0000;

        /// <summary>
        ///  Receive Error Counter 
        /// </summary>
        public static UInt32 CAN_ESR_REC = 0xFF000000;


        /*******************  Bit definition for CAN_BTR register  ********************/
        /// <summary>
        ///  Baud Rate Prescaler 
        /// </summary>
        public static UInt32 CAN_BTR_BRP = 0x000003FF;

        /// <summary>
        ///  Time Segment 1 
        /// </summary>
        public static UInt32 CAN_BTR_TS1 = 0x000F0000;

        /// <summary>
        ///  Time Segment 2 
        /// </summary>
        public static UInt32 CAN_BTR_TS2 = 0x00700000;

        /// <summary>
        ///  Resynchronization Jump Width 
        /// </summary>
        public static UInt32 CAN_BTR_SJW = 0x03000000;

        /// <summary>
        ///  Loop Back Mode (Debug) 
        /// </summary>
        public static UInt32 CAN_BTR_LBKM = 0x40000000;

        /// <summary>
        ///  Silent Mode 
        /// </summary>
        public static UInt32 CAN_BTR_SILM = 0x80000000;


        /*!<Mailbox registers */
        /******************  Bit definition for CAN_TI0R register  ********************/
        /// <summary>
        ///  Transmit Mailbox Request 
        /// </summary>
        public static UInt32 CAN_TI0R_TXRQ = 0x00000001;

        /// <summary>
        ///  Remote Transmission Request 
        /// </summary>
        public static UInt32 CAN_TI0R_RTR = 0x00000002;

        /// <summary>
        ///  Identifier Extension 
        /// </summary>
        public static UInt32 CAN_TI0R_IDE = 0x00000004;

        /// <summary>
        ///  Extended Identifier 
        /// </summary>
        public static UInt32 CAN_TI0R_EXID = 0x001FFFF8;

        /// <summary>
        ///  Standard Identifier or Extended Identifier 
        /// </summary>
        public static UInt32 CAN_TI0R_STID = 0xFFE00000;


        /******************  Bit definition for CAN_TDT0R register  *******************/
        /// <summary>
        ///  Data Length Code 
        /// </summary>
        public static UInt32 CAN_TDT0R_DLC = 0x0000000F;

        /// <summary>
        ///  Transmit Global Time 
        /// </summary>
        public static UInt32 CAN_TDT0R_TGT = 0x00000100;

        /// <summary>
        ///  Message Time Stamp 
        /// </summary>
        public static UInt32 CAN_TDT0R_TIME = 0xFFFF0000;


        /******************  Bit definition for CAN_TDL0R register  *******************/
        /// <summary>
        ///  Data byte 0 
        /// </summary>
        public static UInt32 CAN_TDL0R_DATA0 = 0x000000FF;

        /// <summary>
        ///  Data byte 1 
        /// </summary>
        public static UInt32 CAN_TDL0R_DATA1 = 0x0000FF00;

        /// <summary>
        ///  Data byte 2 
        /// </summary>
        public static UInt32 CAN_TDL0R_DATA2 = 0x00FF0000;

        /// <summary>
        ///  Data byte 3 
        /// </summary>
        public static UInt32 CAN_TDL0R_DATA3 = 0xFF000000;


        /******************  Bit definition for CAN_TDH0R register  *******************/
        /// <summary>
        ///  Data byte 4 
        /// </summary>
        public static UInt32 CAN_TDH0R_DATA4 = 0x000000FF;

        /// <summary>
        ///  Data byte 5 
        /// </summary>
        public static UInt32 CAN_TDH0R_DATA5 = 0x0000FF00;

        /// <summary>
        ///  Data byte 6 
        /// </summary>
        public static UInt32 CAN_TDH0R_DATA6 = 0x00FF0000;

        /// <summary>
        ///  Data byte 7 
        /// </summary>
        public static UInt32 CAN_TDH0R_DATA7 = 0xFF000000;


        /*******************  Bit definition for CAN_TI1R register  *******************/
        /// <summary>
        ///  Transmit Mailbox Request 
        /// </summary>
        public static UInt32 CAN_TI1R_TXRQ = 0x00000001;

        /// <summary>
        ///  Remote Transmission Request 
        /// </summary>
        public static UInt32 CAN_TI1R_RTR = 0x00000002;

        /// <summary>
        ///  Identifier Extension 
        /// </summary>
        public static UInt32 CAN_TI1R_IDE = 0x00000004;

        /// <summary>
        ///  Extended Identifier 
        /// </summary>
        public static UInt32 CAN_TI1R_EXID = 0x001FFFF8;

        /// <summary>
        ///  Standard Identifier or Extended Identifier 
        /// </summary>
        public static UInt32 CAN_TI1R_STID = 0xFFE00000;


        /*******************  Bit definition for CAN_TDT1R register  ******************/
        /// <summary>
        ///  Data Length Code 
        /// </summary>
        public static UInt32 CAN_TDT1R_DLC = 0x0000000F;

        /// <summary>
        ///  Transmit Global Time 
        /// </summary>
        public static UInt32 CAN_TDT1R_TGT = 0x00000100;

        /// <summary>
        ///  Message Time Stamp 
        /// </summary>
        public static UInt32 CAN_TDT1R_TIME = 0xFFFF0000;


        /*******************  Bit definition for CAN_TDL1R register  ******************/
        /// <summary>
        ///  Data byte 0 
        /// </summary>
        public static UInt32 CAN_TDL1R_DATA0 = 0x000000FF;

        /// <summary>
        ///  Data byte 1 
        /// </summary>
        public static UInt32 CAN_TDL1R_DATA1 = 0x0000FF00;

        /// <summary>
        ///  Data byte 2 
        /// </summary>
        public static UInt32 CAN_TDL1R_DATA2 = 0x00FF0000;

        /// <summary>
        ///  Data byte 3 
        /// </summary>
        public static UInt32 CAN_TDL1R_DATA3 = 0xFF000000;


        /*******************  Bit definition for CAN_TDH1R register  ******************/
        /// <summary>
        ///  Data byte 4 
        /// </summary>
        public static UInt32 CAN_TDH1R_DATA4 = 0x000000FF;

        /// <summary>
        ///  Data byte 5 
        /// </summary>
        public static UInt32 CAN_TDH1R_DATA5 = 0x0000FF00;

        /// <summary>
        ///  Data byte 6 
        /// </summary>
        public static UInt32 CAN_TDH1R_DATA6 = 0x00FF0000;

        /// <summary>
        ///  Data byte 7 
        /// </summary>
        public static UInt32 CAN_TDH1R_DATA7 = 0xFF000000;


        /*******************  Bit definition for CAN_TI2R register  *******************/
        /// <summary>
        ///  Transmit Mailbox Request 
        /// </summary>
        public static UInt32 CAN_TI2R_TXRQ = 0x00000001;

        /// <summary>
        ///  Remote Transmission Request 
        /// </summary>
        public static UInt32 CAN_TI2R_RTR = 0x00000002;

        /// <summary>
        ///  Identifier Extension 
        /// </summary>
        public static UInt32 CAN_TI2R_IDE = 0x00000004;

        /// <summary>
        ///  Extended identifier 
        /// </summary>
        public static UInt32 CAN_TI2R_EXID = 0x001FFFF8;

        /// <summary>
        ///  Standard Identifier or Extended Identifier 
        /// </summary>
        public static UInt32 CAN_TI2R_STID = 0xFFE00000;


        /*******************  Bit definition for CAN_TDT2R register  ******************/
        /// <summary>
        ///  Data Length Code 
        /// </summary>
        public static UInt32 CAN_TDT2R_DLC = 0x0000000F;

        /// <summary>
        ///  Transmit Global Time 
        /// </summary>
        public static UInt32 CAN_TDT2R_TGT = 0x00000100;

        /// <summary>
        ///  Message Time Stamp 
        /// </summary>
        public static UInt32 CAN_TDT2R_TIME = 0xFFFF0000;


        /*******************  Bit definition for CAN_TDL2R register  ******************/
        /// <summary>
        ///  Data byte 0 
        /// </summary>
        public static UInt32 CAN_TDL2R_DATA0 = 0x000000FF;

        /// <summary>
        ///  Data byte 1 
        /// </summary>
        public static UInt32 CAN_TDL2R_DATA1 = 0x0000FF00;

        /// <summary>
        ///  Data byte 2 
        /// </summary>
        public static UInt32 CAN_TDL2R_DATA2 = 0x00FF0000;

        /// <summary>
        ///  Data byte 3 
        /// </summary>
        public static UInt32 CAN_TDL2R_DATA3 = 0xFF000000;


        /*******************  Bit definition for CAN_TDH2R register  ******************/
        /// <summary>
        ///  Data byte 4 
        /// </summary>
        public static UInt32 CAN_TDH2R_DATA4 = 0x000000FF;

        /// <summary>
        ///  Data byte 5 
        /// </summary>
        public static UInt32 CAN_TDH2R_DATA5 = 0x0000FF00;

        /// <summary>
        ///  Data byte 6 
        /// </summary>
        public static UInt32 CAN_TDH2R_DATA6 = 0x00FF0000;

        /// <summary>
        ///  Data byte 7 
        /// </summary>
        public static UInt32 CAN_TDH2R_DATA7 = 0xFF000000;


        /*******************  Bit definition for CAN_RI0R register  *******************/
        /// <summary>
        ///  Remote Transmission Request 
        /// </summary>
        public static UInt32 CAN_RI0R_RTR = 0x00000002;

        /// <summary>
        ///  Identifier Extension 
        /// </summary>
        public static UInt32 CAN_RI0R_IDE = 0x00000004;

        /// <summary>
        ///  Extended Identifier 
        /// </summary>
        public static UInt32 CAN_RI0R_EXID = 0x001FFFF8;

        /// <summary>
        ///  Standard Identifier or Extended Identifier 
        /// </summary>
        public static UInt32 CAN_RI0R_STID = 0xFFE00000;


        /*******************  Bit definition for CAN_RDT0R register  ******************/
        /// <summary>
        ///  Data Length Code 
        /// </summary>
        public static UInt32 CAN_RDT0R_DLC = 0x0000000F;

        /// <summary>
        ///  Filter Match Index 
        /// </summary>
        public static UInt32 CAN_RDT0R_FMI = 0x0000FF00;

        /// <summary>
        ///  Message Time Stamp 
        /// </summary>
        public static UInt32 CAN_RDT0R_TIME = 0xFFFF0000;


        /*******************  Bit definition for CAN_RDL0R register  ******************/
        /// <summary>
        ///  Data byte 0 
        /// </summary>
        public static UInt32 CAN_RDL0R_DATA0 = 0x000000FF;

        /// <summary>
        ///  Data byte 1 
        /// </summary>
        public static UInt32 CAN_RDL0R_DATA1 = 0x0000FF00;

        /// <summary>
        ///  Data byte 2 
        /// </summary>
        public static UInt32 CAN_RDL0R_DATA2 = 0x00FF0000;

        /// <summary>
        ///  Data byte 3 
        /// </summary>
        public static UInt32 CAN_RDL0R_DATA3 = 0xFF000000;


        /*******************  Bit definition for CAN_RDH0R register  ******************/
        /// <summary>
        ///  Data byte 4 
        /// </summary>
        public static UInt32 CAN_RDH0R_DATA4 = 0x000000FF;

        /// <summary>
        ///  Data byte 5 
        /// </summary>
        public static UInt32 CAN_RDH0R_DATA5 = 0x0000FF00;

        /// <summary>
        ///  Data byte 6 
        /// </summary>
        public static UInt32 CAN_RDH0R_DATA6 = 0x00FF0000;

        /// <summary>
        ///  Data byte 7 
        /// </summary>
        public static UInt32 CAN_RDH0R_DATA7 = 0xFF000000;


        /*******************  Bit definition for CAN_RI1R register  *******************/
        /// <summary>
        ///  Remote Transmission Request 
        /// </summary>
        public static UInt32 CAN_RI1R_RTR = 0x00000002;

        /// <summary>
        ///  Identifier Extension 
        /// </summary>
        public static UInt32 CAN_RI1R_IDE = 0x00000004;

        /// <summary>
        ///  Extended identifier 
        /// </summary>
        public static UInt32 CAN_RI1R_EXID = 0x001FFFF8;

        /// <summary>
        ///  Standard Identifier or Extended Identifier 
        /// </summary>
        public static UInt32 CAN_RI1R_STID = 0xFFE00000;


        /*******************  Bit definition for CAN_RDT1R register  ******************/
        /// <summary>
        ///  Data Length Code 
        /// </summary>
        public static UInt32 CAN_RDT1R_DLC = 0x0000000F;

        /// <summary>
        ///  Filter Match Index 
        /// </summary>
        public static UInt32 CAN_RDT1R_FMI = 0x0000FF00;

        /// <summary>
        ///  Message Time Stamp 
        /// </summary>
        public static UInt32 CAN_RDT1R_TIME = 0xFFFF0000;


        /*******************  Bit definition for CAN_RDL1R register  ******************/
        /// <summary>
        ///  Data byte 0 
        /// </summary>
        public static UInt32 CAN_RDL1R_DATA0 = 0x000000FF;

        /// <summary>
        ///  Data byte 1 
        /// </summary>
        public static UInt32 CAN_RDL1R_DATA1 = 0x0000FF00;

        /// <summary>
        ///  Data byte 2 
        /// </summary>
        public static UInt32 CAN_RDL1R_DATA2 = 0x00FF0000;

        /// <summary>
        ///  Data byte 3 
        /// </summary>
        public static UInt32 CAN_RDL1R_DATA3 = 0xFF000000;


        /*******************  Bit definition for CAN_RDH1R register  ******************/
        /// <summary>
        ///  Data byte 4 
        /// </summary>
        public static UInt32 CAN_RDH1R_DATA4 = 0x000000FF;

        /// <summary>
        ///  Data byte 5 
        /// </summary>
        public static UInt32 CAN_RDH1R_DATA5 = 0x0000FF00;

        /// <summary>
        ///  Data byte 6 
        /// </summary>
        public static UInt32 CAN_RDH1R_DATA6 = 0x00FF0000;

        /// <summary>
        ///  Data byte 7 
        /// </summary>
        public static UInt32 CAN_RDH1R_DATA7 = 0xFF000000;


        /*!<CAN filter registers */
        /*******************  Bit definition for CAN_FMR register  ********************/
        /// <summary>
        ///  Filter Init Mode 
        /// </summary>
        public static byte CAN_FMR_FINIT = 0x01;


        /*******************  Bit definition for CAN_FM1R register  *******************/
        /// <summary>
        ///  Filter Mode 
        /// </summary>
        public static UInt16 CAN_FM1R_FBM = 0x3FFF;

        /// <summary>
        ///  Filter Init Mode bit 0 
        /// </summary>
        public static UInt16 CAN_FM1R_FBM0 = 0x0001;

        /// <summary>
        ///  Filter Init Mode bit 1 
        /// </summary>
        public static UInt16 CAN_FM1R_FBM1 = 0x0002;

        /// <summary>
        ///  Filter Init Mode bit 2 
        /// </summary>
        public static UInt16 CAN_FM1R_FBM2 = 0x0004;

        /// <summary>
        ///  Filter Init Mode bit 3 
        /// </summary>
        public static UInt16 CAN_FM1R_FBM3 = 0x0008;

        /// <summary>
        ///  Filter Init Mode bit 4 
        /// </summary>
        public static UInt16 CAN_FM1R_FBM4 = 0x0010;

        /// <summary>
        ///  Filter Init Mode bit 5 
        /// </summary>
        public static UInt16 CAN_FM1R_FBM5 = 0x0020;

        /// <summary>
        ///  Filter Init Mode bit 6 
        /// </summary>
        public static UInt16 CAN_FM1R_FBM6 = 0x0040;

        /// <summary>
        ///  Filter Init Mode bit 7 
        /// </summary>
        public static UInt16 CAN_FM1R_FBM7 = 0x0080;

        /// <summary>
        ///  Filter Init Mode bit 8 
        /// </summary>
        public static UInt16 CAN_FM1R_FBM8 = 0x0100;

        /// <summary>
        ///  Filter Init Mode bit 9 
        /// </summary>
        public static UInt16 CAN_FM1R_FBM9 = 0x0200;

        /// <summary>
        ///  Filter Init Mode bit 10 
        /// </summary>
        public static UInt16 CAN_FM1R_FBM10 = 0x0400;

        /// <summary>
        ///  Filter Init Mode bit 11 
        /// </summary>
        public static UInt16 CAN_FM1R_FBM11 = 0x0800;

        /// <summary>
        ///  Filter Init Mode bit 12 
        /// </summary>
        public static UInt16 CAN_FM1R_FBM12 = 0x1000;

        /// <summary>
        ///  Filter Init Mode bit 13 
        /// </summary>
        public static UInt16 CAN_FM1R_FBM13 = 0x2000;


        /*******************  Bit definition for CAN_FS1R register  *******************/
        /// <summary>
        ///  Filter Scale Configuration 
        /// </summary>
        public static UInt16 CAN_FS1R_FSC = 0x3FFF;

        /// <summary>
        ///  Filter Scale Configuration bit 0 
        /// </summary>
        public static UInt16 CAN_FS1R_FSC0 = 0x0001;

        /// <summary>
        ///  Filter Scale Configuration bit 1 
        /// </summary>
        public static UInt16 CAN_FS1R_FSC1 = 0x0002;

        /// <summary>
        ///  Filter Scale Configuration bit 2 
        /// </summary>
        public static UInt16 CAN_FS1R_FSC2 = 0x0004;

        /// <summary>
        ///  Filter Scale Configuration bit 3 
        /// </summary>
        public static UInt16 CAN_FS1R_FSC3 = 0x0008;

        /// <summary>
        ///  Filter Scale Configuration bit 4 
        /// </summary>
        public static UInt16 CAN_FS1R_FSC4 = 0x0010;

        /// <summary>
        ///  Filter Scale Configuration bit 5 
        /// </summary>
        public static UInt16 CAN_FS1R_FSC5 = 0x0020;

        /// <summary>
        ///  Filter Scale Configuration bit 6 
        /// </summary>
        public static UInt16 CAN_FS1R_FSC6 = 0x0040;

        /// <summary>
        ///  Filter Scale Configuration bit 7 
        /// </summary>
        public static UInt16 CAN_FS1R_FSC7 = 0x0080;

        /// <summary>
        ///  Filter Scale Configuration bit 8 
        /// </summary>
        public static UInt16 CAN_FS1R_FSC8 = 0x0100;

        /// <summary>
        ///  Filter Scale Configuration bit 9 
        /// </summary>
        public static UInt16 CAN_FS1R_FSC9 = 0x0200;

        /// <summary>
        ///  Filter Scale Configuration bit 10 
        /// </summary>
        public static UInt16 CAN_FS1R_FSC10 = 0x0400;

        /// <summary>
        ///  Filter Scale Configuration bit 11 
        /// </summary>
        public static UInt16 CAN_FS1R_FSC11 = 0x0800;

        /// <summary>
        ///  Filter Scale Configuration bit 12 
        /// </summary>
        public static UInt16 CAN_FS1R_FSC12 = 0x1000;

        /// <summary>
        ///  Filter Scale Configuration bit 13 
        /// </summary>
        public static UInt16 CAN_FS1R_FSC13 = 0x2000;


        /******************  Bit definition for CAN_FFA1R register  *******************/
        /// <summary>
        ///  Filter FIFO Assignment 
        /// </summary>
        public static UInt16 CAN_FFA1R_FFA = 0x3FFF;

        /// <summary>
        ///  Filter FIFO Assignment for Filter 0 
        /// </summary>
        public static UInt16 CAN_FFA1R_FFA0 = 0x0001;

        /// <summary>
        ///  Filter FIFO Assignment for Filter 1 
        /// </summary>
        public static UInt16 CAN_FFA1R_FFA1 = 0x0002;

        /// <summary>
        ///  Filter FIFO Assignment for Filter 2 
        /// </summary>
        public static UInt16 CAN_FFA1R_FFA2 = 0x0004;

        /// <summary>
        ///  Filter FIFO Assignment for Filter 3 
        /// </summary>
        public static UInt16 CAN_FFA1R_FFA3 = 0x0008;

        /// <summary>
        ///  Filter FIFO Assignment for Filter 4 
        /// </summary>
        public static UInt16 CAN_FFA1R_FFA4 = 0x0010;

        /// <summary>
        ///  Filter FIFO Assignment for Filter 5 
        /// </summary>
        public static UInt16 CAN_FFA1R_FFA5 = 0x0020;

        /// <summary>
        ///  Filter FIFO Assignment for Filter 6 
        /// </summary>
        public static UInt16 CAN_FFA1R_FFA6 = 0x0040;

        /// <summary>
        ///  Filter FIFO Assignment for Filter 7 
        /// </summary>
        public static UInt16 CAN_FFA1R_FFA7 = 0x0080;

        /// <summary>
        ///  Filter FIFO Assignment for Filter 8 
        /// </summary>
        public static UInt16 CAN_FFA1R_FFA8 = 0x0100;

        /// <summary>
        ///  Filter FIFO Assignment for Filter 9 
        /// </summary>
        public static UInt16 CAN_FFA1R_FFA9 = 0x0200;

        /// <summary>
        ///  Filter FIFO Assignment for Filter 10 
        /// </summary>
        public static UInt16 CAN_FFA1R_FFA10 = 0x0400;

        /// <summary>
        ///  Filter FIFO Assignment for Filter 11 
        /// </summary>
        public static UInt16 CAN_FFA1R_FFA11 = 0x0800;

        /// <summary>
        ///  Filter FIFO Assignment for Filter 12 
        /// </summary>
        public static UInt16 CAN_FFA1R_FFA12 = 0x1000;

        /// <summary>
        ///  Filter FIFO Assignment for Filter 13 
        /// </summary>
        public static UInt16 CAN_FFA1R_FFA13 = 0x2000;


        /*******************  Bit definition for CAN_FA1R register  *******************/
        /// <summary>
        ///  Filter Active 
        /// </summary>
        public static UInt16 CAN_FA1R_FACT = 0x3FFF;

        /// <summary>
        ///  Filter 0 Active 
        /// </summary>
        public static UInt16 CAN_FA1R_FACT0 = 0x0001;

        /// <summary>
        ///  Filter 1 Active 
        /// </summary>
        public static UInt16 CAN_FA1R_FACT1 = 0x0002;

        /// <summary>
        ///  Filter 2 Active 
        /// </summary>
        public static UInt16 CAN_FA1R_FACT2 = 0x0004;

        /// <summary>
        ///  Filter 3 Active 
        /// </summary>
        public static UInt16 CAN_FA1R_FACT3 = 0x0008;

        /// <summary>
        ///  Filter 4 Active 
        /// </summary>
        public static UInt16 CAN_FA1R_FACT4 = 0x0010;

        /// <summary>
        ///  Filter 5 Active 
        /// </summary>
        public static UInt16 CAN_FA1R_FACT5 = 0x0020;

        /// <summary>
        ///  Filter 6 Active 
        /// </summary>
        public static UInt16 CAN_FA1R_FACT6 = 0x0040;

        /// <summary>
        ///  Filter 7 Active 
        /// </summary>
        public static UInt16 CAN_FA1R_FACT7 = 0x0080;

        /// <summary>
        ///  Filter 8 Active 
        /// </summary>
        public static UInt16 CAN_FA1R_FACT8 = 0x0100;

        /// <summary>
        ///  Filter 9 Active 
        /// </summary>
        public static UInt16 CAN_FA1R_FACT9 = 0x0200;

        /// <summary>
        ///  Filter 10 Active 
        /// </summary>
        public static UInt16 CAN_FA1R_FACT10 = 0x0400;

        /// <summary>
        ///  Filter 11 Active 
        /// </summary>
        public static UInt16 CAN_FA1R_FACT11 = 0x0800;

        /// <summary>
        ///  Filter 12 Active 
        /// </summary>
        public static UInt16 CAN_FA1R_FACT12 = 0x1000;

        /// <summary>
        ///  Filter 13 Active 
        /// </summary>
        public static UInt16 CAN_FA1R_FACT13 = 0x2000;


        /*******************  Bit definition for CAN_F0R1 register  *******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F0R1_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F0R1_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F0R1_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F0R1_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F0R1_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F0R1_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F0R1_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F0R1_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F0R1_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F0R1_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F0R1_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F0R1_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F0R1_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F0R1_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F0R1_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F0R1_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F0R1_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F0R1_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F0R1_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F0R1_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F0R1_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F0R1_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F0R1_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F0R1_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F0R1_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F0R1_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F0R1_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F0R1_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F0R1_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F0R1_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F0R1_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F0R1_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F1R1 register  *******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F1R1_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F1R1_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F1R1_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F1R1_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F1R1_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F1R1_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F1R1_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F1R1_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F1R1_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F1R1_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F1R1_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F1R1_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F1R1_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F1R1_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F1R1_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F1R1_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F1R1_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F1R1_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F1R1_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F1R1_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F1R1_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F1R1_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F1R1_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F1R1_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F1R1_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F1R1_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F1R1_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F1R1_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F1R1_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F1R1_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F1R1_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F1R1_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F2R1 register  *******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F2R1_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F2R1_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F2R1_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F2R1_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F2R1_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F2R1_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F2R1_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F2R1_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F2R1_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F2R1_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F2R1_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F2R1_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F2R1_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F2R1_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F2R1_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F2R1_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F2R1_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F2R1_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F2R1_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F2R1_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F2R1_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F2R1_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F2R1_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F2R1_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F2R1_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F2R1_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F2R1_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F2R1_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F2R1_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F2R1_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F2R1_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F2R1_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F3R1 register  *******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F3R1_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F3R1_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F3R1_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F3R1_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F3R1_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F3R1_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F3R1_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F3R1_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F3R1_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F3R1_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F3R1_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F3R1_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F3R1_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F3R1_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F3R1_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F3R1_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F3R1_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F3R1_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F3R1_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F3R1_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F3R1_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F3R1_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F3R1_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F3R1_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F3R1_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F3R1_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F3R1_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F3R1_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F3R1_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F3R1_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F3R1_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F3R1_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F4R1 register  *******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F4R1_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F4R1_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F4R1_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F4R1_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F4R1_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F4R1_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F4R1_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F4R1_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F4R1_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F4R1_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F4R1_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F4R1_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F4R1_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F4R1_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F4R1_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F4R1_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F4R1_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F4R1_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F4R1_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F4R1_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F4R1_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F4R1_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F4R1_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F4R1_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F4R1_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F4R1_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F4R1_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F4R1_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F4R1_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F4R1_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F4R1_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F4R1_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F5R1 register  *******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F5R1_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F5R1_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F5R1_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F5R1_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F5R1_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F5R1_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F5R1_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F5R1_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F5R1_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F5R1_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F5R1_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F5R1_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F5R1_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F5R1_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F5R1_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F5R1_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F5R1_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F5R1_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F5R1_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F5R1_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F5R1_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F5R1_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F5R1_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F5R1_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F5R1_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F5R1_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F5R1_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F5R1_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F5R1_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F5R1_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F5R1_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F5R1_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F6R1 register  *******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F6R1_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F6R1_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F6R1_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F6R1_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F6R1_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F6R1_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F6R1_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F6R1_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F6R1_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F6R1_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F6R1_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F6R1_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F6R1_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F6R1_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F6R1_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F6R1_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F6R1_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F6R1_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F6R1_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F6R1_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F6R1_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F6R1_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F6R1_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F6R1_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F6R1_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F6R1_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F6R1_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F6R1_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F6R1_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F6R1_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F6R1_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F6R1_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F7R1 register  *******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F7R1_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F7R1_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F7R1_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F7R1_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F7R1_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F7R1_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F7R1_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F7R1_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F7R1_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F7R1_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F7R1_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F7R1_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F7R1_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F7R1_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F7R1_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F7R1_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F7R1_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F7R1_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F7R1_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F7R1_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F7R1_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F7R1_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F7R1_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F7R1_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F7R1_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F7R1_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F7R1_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F7R1_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F7R1_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F7R1_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F7R1_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F7R1_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F8R1 register  *******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F8R1_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F8R1_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F8R1_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F8R1_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F8R1_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F8R1_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F8R1_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F8R1_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F8R1_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F8R1_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F8R1_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F8R1_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F8R1_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F8R1_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F8R1_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F8R1_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F8R1_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F8R1_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F8R1_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F8R1_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F8R1_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F8R1_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F8R1_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F8R1_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F8R1_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F8R1_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F8R1_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F8R1_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F8R1_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F8R1_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F8R1_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F8R1_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F9R1 register  *******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F9R1_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F9R1_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F9R1_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F9R1_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F9R1_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F9R1_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F9R1_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F9R1_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F9R1_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F9R1_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F9R1_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F9R1_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F9R1_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F9R1_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F9R1_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F9R1_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F9R1_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F9R1_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F9R1_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F9R1_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F9R1_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F9R1_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F9R1_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F9R1_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F9R1_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F9R1_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F9R1_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F9R1_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F9R1_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F9R1_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F9R1_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F9R1_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F10R1 register  ******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F10R1_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F10R1_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F10R1_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F10R1_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F10R1_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F10R1_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F10R1_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F10R1_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F10R1_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F10R1_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F10R1_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F10R1_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F10R1_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F10R1_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F10R1_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F10R1_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F10R1_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F10R1_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F10R1_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F10R1_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F10R1_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F10R1_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F10R1_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F10R1_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F10R1_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F10R1_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F10R1_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F10R1_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F10R1_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F10R1_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F10R1_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F10R1_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F11R1 register  ******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F11R1_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F11R1_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F11R1_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F11R1_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F11R1_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F11R1_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F11R1_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F11R1_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F11R1_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F11R1_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F11R1_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F11R1_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F11R1_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F11R1_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F11R1_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F11R1_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F11R1_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F11R1_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F11R1_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F11R1_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F11R1_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F11R1_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F11R1_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F11R1_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F11R1_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F11R1_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F11R1_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F11R1_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F11R1_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F11R1_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F11R1_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F11R1_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F12R1 register  ******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F12R1_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F12R1_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F12R1_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F12R1_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F12R1_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F12R1_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F12R1_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F12R1_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F12R1_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F12R1_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F12R1_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F12R1_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F12R1_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F12R1_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F12R1_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F12R1_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F12R1_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F12R1_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F12R1_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F12R1_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F12R1_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F12R1_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F12R1_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F12R1_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F12R1_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F12R1_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F12R1_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F12R1_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F12R1_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F12R1_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F12R1_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F12R1_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F13R1 register  ******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F13R1_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F13R1_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F13R1_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F13R1_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F13R1_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F13R1_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F13R1_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F13R1_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F13R1_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F13R1_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F13R1_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F13R1_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F13R1_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F13R1_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F13R1_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F13R1_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F13R1_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F13R1_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F13R1_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F13R1_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F13R1_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F13R1_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F13R1_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F13R1_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F13R1_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F13R1_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F13R1_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F13R1_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F13R1_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F13R1_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F13R1_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F13R1_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F0R2 register  *******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F0R2_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F0R2_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F0R2_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F0R2_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F0R2_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F0R2_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F0R2_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F0R2_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F0R2_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F0R2_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F0R2_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F0R2_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F0R2_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F0R2_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F0R2_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F0R2_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F0R2_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F0R2_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F0R2_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F0R2_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F0R2_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F0R2_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F0R2_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F0R2_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F0R2_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F0R2_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F0R2_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F0R2_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F0R2_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F0R2_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F0R2_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F0R2_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F1R2 register  *******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F1R2_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F1R2_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F1R2_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F1R2_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F1R2_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F1R2_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F1R2_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F1R2_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F1R2_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F1R2_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F1R2_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F1R2_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F1R2_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F1R2_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F1R2_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F1R2_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F1R2_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F1R2_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F1R2_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F1R2_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F1R2_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F1R2_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F1R2_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F1R2_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F1R2_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F1R2_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F1R2_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F1R2_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F1R2_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F1R2_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F1R2_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F1R2_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F2R2 register  *******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F2R2_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F2R2_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F2R2_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F2R2_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F2R2_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F2R2_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F2R2_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F2R2_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F2R2_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F2R2_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F2R2_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F2R2_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F2R2_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F2R2_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F2R2_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F2R2_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F2R2_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F2R2_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F2R2_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F2R2_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F2R2_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F2R2_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F2R2_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F2R2_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F2R2_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F2R2_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F2R2_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F2R2_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F2R2_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F2R2_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F2R2_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F2R2_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F3R2 register  *******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F3R2_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F3R2_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F3R2_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F3R2_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F3R2_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F3R2_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F3R2_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F3R2_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F3R2_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F3R2_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F3R2_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F3R2_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F3R2_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F3R2_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F3R2_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F3R2_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F3R2_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F3R2_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F3R2_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F3R2_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F3R2_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F3R2_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F3R2_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F3R2_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F3R2_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F3R2_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F3R2_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F3R2_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F3R2_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F3R2_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F3R2_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F3R2_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F4R2 register  *******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F4R2_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F4R2_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F4R2_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F4R2_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F4R2_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F4R2_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F4R2_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F4R2_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F4R2_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F4R2_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F4R2_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F4R2_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F4R2_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F4R2_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F4R2_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F4R2_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F4R2_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F4R2_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F4R2_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F4R2_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F4R2_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F4R2_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F4R2_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F4R2_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F4R2_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F4R2_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F4R2_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F4R2_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F4R2_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F4R2_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F4R2_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F4R2_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F5R2 register  *******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F5R2_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F5R2_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F5R2_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F5R2_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F5R2_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F5R2_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F5R2_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F5R2_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F5R2_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F5R2_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F5R2_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F5R2_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F5R2_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F5R2_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F5R2_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F5R2_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F5R2_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F5R2_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F5R2_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F5R2_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F5R2_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F5R2_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F5R2_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F5R2_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F5R2_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F5R2_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F5R2_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F5R2_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F5R2_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F5R2_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F5R2_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F5R2_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F6R2 register  *******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F6R2_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F6R2_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F6R2_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F6R2_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F6R2_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F6R2_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F6R2_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F6R2_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F6R2_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F6R2_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F6R2_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F6R2_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F6R2_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F6R2_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F6R2_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F6R2_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F6R2_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F6R2_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F6R2_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F6R2_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F6R2_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F6R2_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F6R2_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F6R2_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F6R2_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F6R2_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F6R2_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F6R2_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F6R2_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F6R2_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F6R2_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F6R2_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F7R2 register  *******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F7R2_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F7R2_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F7R2_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F7R2_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F7R2_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F7R2_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F7R2_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F7R2_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F7R2_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F7R2_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F7R2_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F7R2_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F7R2_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F7R2_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F7R2_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F7R2_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F7R2_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F7R2_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F7R2_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F7R2_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F7R2_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F7R2_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F7R2_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F7R2_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F7R2_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F7R2_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F7R2_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F7R2_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F7R2_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F7R2_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F7R2_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F7R2_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F8R2 register  *******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F8R2_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F8R2_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F8R2_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F8R2_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F8R2_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F8R2_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F8R2_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F8R2_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F8R2_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F8R2_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F8R2_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F8R2_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F8R2_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F8R2_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F8R2_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F8R2_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F8R2_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F8R2_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F8R2_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F8R2_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F8R2_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F8R2_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F8R2_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F8R2_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F8R2_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F8R2_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F8R2_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F8R2_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F8R2_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F8R2_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F8R2_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F8R2_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F9R2 register  *******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F9R2_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F9R2_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F9R2_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F9R2_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F9R2_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F9R2_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F9R2_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F9R2_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F9R2_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F9R2_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F9R2_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F9R2_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F9R2_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F9R2_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F9R2_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F9R2_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F9R2_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F9R2_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F9R2_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F9R2_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F9R2_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F9R2_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F9R2_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F9R2_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F9R2_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F9R2_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F9R2_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F9R2_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F9R2_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F9R2_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F9R2_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F9R2_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F10R2 register  ******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F10R2_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F10R2_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F10R2_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F10R2_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F10R2_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F10R2_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F10R2_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F10R2_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F10R2_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F10R2_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F10R2_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F10R2_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F10R2_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F10R2_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F10R2_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F10R2_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F10R2_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F10R2_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F10R2_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F10R2_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F10R2_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F10R2_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F10R2_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F10R2_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F10R2_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F10R2_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F10R2_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F10R2_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F10R2_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F10R2_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F10R2_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F10R2_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F11R2 register  ******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F11R2_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F11R2_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F11R2_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F11R2_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F11R2_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F11R2_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F11R2_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F11R2_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F11R2_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F11R2_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F11R2_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F11R2_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F11R2_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F11R2_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F11R2_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F11R2_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F11R2_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F11R2_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F11R2_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F11R2_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F11R2_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F11R2_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F11R2_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F11R2_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F11R2_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F11R2_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F11R2_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F11R2_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F11R2_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F11R2_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F11R2_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F11R2_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F12R2 register  ******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F12R2_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F12R2_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F12R2_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F12R2_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F12R2_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F12R2_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F12R2_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F12R2_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F12R2_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F12R2_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F12R2_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F12R2_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F12R2_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F12R2_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F12R2_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F12R2_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F12R2_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F12R2_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F12R2_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F12R2_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F12R2_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F12R2_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F12R2_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F12R2_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F12R2_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F12R2_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F12R2_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F12R2_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F12R2_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F12R2_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F12R2_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F12R2_FB31 = 0x80000000;


        /*******************  Bit definition for CAN_F13R2 register  ******************/
        /// <summary>
        ///  Filter bit 0 
        /// </summary>
        public static UInt32 CAN_F13R2_FB0 = 0x00000001;

        /// <summary>
        ///  Filter bit 1 
        /// </summary>
        public static UInt32 CAN_F13R2_FB1 = 0x00000002;

        /// <summary>
        ///  Filter bit 2 
        /// </summary>
        public static UInt32 CAN_F13R2_FB2 = 0x00000004;

        /// <summary>
        ///  Filter bit 3 
        /// </summary>
        public static UInt32 CAN_F13R2_FB3 = 0x00000008;

        /// <summary>
        ///  Filter bit 4 
        /// </summary>
        public static UInt32 CAN_F13R2_FB4 = 0x00000010;

        /// <summary>
        ///  Filter bit 5 
        /// </summary>
        public static UInt32 CAN_F13R2_FB5 = 0x00000020;

        /// <summary>
        ///  Filter bit 6 
        /// </summary>
        public static UInt32 CAN_F13R2_FB6 = 0x00000040;

        /// <summary>
        ///  Filter bit 7 
        /// </summary>
        public static UInt32 CAN_F13R2_FB7 = 0x00000080;

        /// <summary>
        ///  Filter bit 8 
        /// </summary>
        public static UInt32 CAN_F13R2_FB8 = 0x00000100;

        /// <summary>
        ///  Filter bit 9 
        /// </summary>
        public static UInt32 CAN_F13R2_FB9 = 0x00000200;

        /// <summary>
        ///  Filter bit 10 
        /// </summary>
        public static UInt32 CAN_F13R2_FB10 = 0x00000400;

        /// <summary>
        ///  Filter bit 11 
        /// </summary>
        public static UInt32 CAN_F13R2_FB11 = 0x00000800;

        /// <summary>
        ///  Filter bit 12 
        /// </summary>
        public static UInt32 CAN_F13R2_FB12 = 0x00001000;

        /// <summary>
        ///  Filter bit 13 
        /// </summary>
        public static UInt32 CAN_F13R2_FB13 = 0x00002000;

        /// <summary>
        ///  Filter bit 14 
        /// </summary>
        public static UInt32 CAN_F13R2_FB14 = 0x00004000;

        /// <summary>
        ///  Filter bit 15 
        /// </summary>
        public static UInt32 CAN_F13R2_FB15 = 0x00008000;

        /// <summary>
        ///  Filter bit 16 
        /// </summary>
        public static UInt32 CAN_F13R2_FB16 = 0x00010000;

        /// <summary>
        ///  Filter bit 17 
        /// </summary>
        public static UInt32 CAN_F13R2_FB17 = 0x00020000;

        /// <summary>
        ///  Filter bit 18 
        /// </summary>
        public static UInt32 CAN_F13R2_FB18 = 0x00040000;

        /// <summary>
        ///  Filter bit 19 
        /// </summary>
        public static UInt32 CAN_F13R2_FB19 = 0x00080000;

        /// <summary>
        ///  Filter bit 20 
        /// </summary>
        public static UInt32 CAN_F13R2_FB20 = 0x00100000;

        /// <summary>
        ///  Filter bit 21 
        /// </summary>
        public static UInt32 CAN_F13R2_FB21 = 0x00200000;

        /// <summary>
        ///  Filter bit 22 
        /// </summary>
        public static UInt32 CAN_F13R2_FB22 = 0x00400000;

        /// <summary>
        ///  Filter bit 23 
        /// </summary>
        public static UInt32 CAN_F13R2_FB23 = 0x00800000;

        /// <summary>
        ///  Filter bit 24 
        /// </summary>
        public static UInt32 CAN_F13R2_FB24 = 0x01000000;

        /// <summary>
        ///  Filter bit 25 
        /// </summary>
        public static UInt32 CAN_F13R2_FB25 = 0x02000000;

        /// <summary>
        ///  Filter bit 26 
        /// </summary>
        public static UInt32 CAN_F13R2_FB26 = 0x04000000;

        /// <summary>
        ///  Filter bit 27 
        /// </summary>
        public static UInt32 CAN_F13R2_FB27 = 0x08000000;

        /// <summary>
        ///  Filter bit 28 
        /// </summary>
        public static UInt32 CAN_F13R2_FB28 = 0x10000000;

        /// <summary>
        ///  Filter bit 29 
        /// </summary>
        public static UInt32 CAN_F13R2_FB29 = 0x20000000;

        /// <summary>
        ///  Filter bit 30 
        /// </summary>
        public static UInt32 CAN_F13R2_FB30 = 0x40000000;

        /// <summary>
        ///  Filter bit 31 
        /// </summary>
        public static UInt32 CAN_F13R2_FB31 = 0x80000000;


        /******************************************************************************/
        /*                                                                            */
        /*                          CRC calculation unit                              */
        /*                                                                            */
        /******************************************************************************/
        /*******************  Bit definition for CRC_DR register  *********************/
        /// <summary>
        ///   Data register bits 
        /// </summary>
        public static UInt32 CRC_DR_DR = 0xFFFFFFFF;



        /*******************  Bit definition for CRC_IDR register  ********************/
        /// <summary>
        ///   General-purpose 8-bit data register bits 
        /// </summary>
        public static byte CRC_IDR_IDR = 0xFF;



        /********************  Bit definition for CRC_CR register  ********************/
        /// <summary>
        ///   RESET bit 
        /// </summary>
        public static byte CRC_CR_RESET = 0x01;


        /******************************************************************************/
        /*                                                                            */
        /*                            Crypto Processor                                */
        /*                                                                            */
        /******************************************************************************/
        /******************* Bits definition for CRYP_CR register  ********************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_CR_ALGODIR = 0x00000004;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_CR_ALGOMODE = 0x00000038;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_CR_ALGOMODE_0 = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_CR_ALGOMODE_1 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_CR_ALGOMODE_2 = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_CR_ALGOMODE_TDES_ECB = 0x00000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_CR_ALGOMODE_TDES_CBC = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_CR_ALGOMODE_DES_ECB = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_CR_ALGOMODE_DES_CBC = 0x00000018;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_CR_ALGOMODE_AES_ECB = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_CR_ALGOMODE_AES_CBC = 0x00000028;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_CR_ALGOMODE_AES_CTR = 0x00000030;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_CR_ALGOMODE_AES_KEY = 0x00000038;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_CR_DATATYPE = 0x000000C0;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_CR_DATATYPE_0 = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_CR_DATATYPE_1 = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_CR_KEYSIZE = 0x00000300;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_CR_KEYSIZE_0 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_CR_KEYSIZE_1 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_CR_FFLUSH = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_CR_CRYPEN = 0x00008000;

        /****************** Bits definition for CRYP_SR register  *********************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_SR_IFEM = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_SR_IFNF = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_SR_OFNE = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_SR_OFFU = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_SR_BUSY = 0x00000010;

        /****************** Bits definition for CRYP_DMACR register  ******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_DMACR_DIEN = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_DMACR_DOEN = 0x00000002;

        /*****************  Bits definition for CRYP_IMSCR register  ******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_IMSCR_INIM = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_IMSCR_OUTIM = 0x00000002;

        /****************** Bits definition for CRYP_RISR register  *******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_RISR_OUTRIS = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_RISR_INRIS = 0x00000002;

        /****************** Bits definition for CRYP_MISR register  *******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_MISR_INMIS = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 CRYP_MISR_OUTMIS = 0x00000002;


        /******************************************************************************/
        /*                                                                            */
        /*                      Digital to Analog Converter                           */
        /*                                                                            */
        /******************************************************************************/
        /********************  Bit definition for DAC_CR register  ********************/
        /// <summary>
        ///  DAC channel1 enable 
        /// </summary>
        public static UInt32 DAC_CR_EN1 = 0x00000001;

        /// <summary>
        ///  DAC channel1 output buffer disable 
        /// </summary>
        public static UInt32 DAC_CR_BOFF1 = 0x00000002;

        /// <summary>
        ///  DAC channel1 Trigger enable 
        /// </summary>
        public static UInt32 DAC_CR_TEN1 = 0x00000004;


        /// <summary>
        ///  TSEL1[2:0] (DAC channel1 Trigger selection) 
        /// </summary>
        public static UInt32 DAC_CR_TSEL1 = 0x00000038;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 DAC_CR_TSEL1_0 = 0x00000008;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 DAC_CR_TSEL1_1 = 0x00000010;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 DAC_CR_TSEL1_2 = 0x00000020;


        /// <summary>
        ///  WAVE1[1:0] (DAC channel1 noise/triangle wave generation enable) 
        /// </summary>
        public static UInt32 DAC_CR_WAVE1 = 0x000000C0;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 DAC_CR_WAVE1_0 = 0x00000040;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 DAC_CR_WAVE1_1 = 0x00000080;


        /// <summary>
        ///  MAMP1[3:0] (DAC channel1 Mask/Amplitude selector) 
        /// </summary>
        public static UInt32 DAC_CR_MAMP1 = 0x00000F00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 DAC_CR_MAMP1_0 = 0x00000100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 DAC_CR_MAMP1_1 = 0x00000200;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 DAC_CR_MAMP1_2 = 0x00000400;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 DAC_CR_MAMP1_3 = 0x00000800;


        /// <summary>
        ///  DAC channel1 DMA enable 
        /// </summary>
        public static UInt32 DAC_CR_DMAEN1 = 0x00001000;

        /// <summary>
        ///  DAC channel2 enable 
        /// </summary>
        public static UInt32 DAC_CR_EN2 = 0x00010000;

        /// <summary>
        ///  DAC channel2 output buffer disable 
        /// </summary>
        public static UInt32 DAC_CR_BOFF2 = 0x00020000;

        /// <summary>
        ///  DAC channel2 Trigger enable 
        /// </summary>
        public static UInt32 DAC_CR_TEN2 = 0x00040000;


        /// <summary>
        ///  TSEL2[2:0] (DAC channel2 Trigger selection) 
        /// </summary>
        public static UInt32 DAC_CR_TSEL2 = 0x00380000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 DAC_CR_TSEL2_0 = 0x00080000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 DAC_CR_TSEL2_1 = 0x00100000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 DAC_CR_TSEL2_2 = 0x00200000;


        /// <summary>
        ///  WAVE2[1:0] (DAC channel2 noise/triangle wave generation enable) 
        /// </summary>
        public static UInt32 DAC_CR_WAVE2 = 0x00C00000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 DAC_CR_WAVE2_0 = 0x00400000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 DAC_CR_WAVE2_1 = 0x00800000;


        /// <summary>
        ///  MAMP2[3:0] (DAC channel2 Mask/Amplitude selector) 
        /// </summary>
        public static UInt32 DAC_CR_MAMP2 = 0x0F000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 DAC_CR_MAMP2_0 = 0x01000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 DAC_CR_MAMP2_1 = 0x02000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 DAC_CR_MAMP2_2 = 0x04000000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 DAC_CR_MAMP2_3 = 0x08000000;


        /// <summary>
        ///  DAC channel2 DMA enabled 
        /// </summary>
        public static UInt32 DAC_CR_DMAEN2 = 0x10000000;


        /*****************  Bit definition for DAC_SWTRIGR register  ******************/
        /// <summary>
        ///  DAC channel1 software trigger 
        /// </summary>
        public static byte DAC_SWTRIGR_SWTRIG1 = 0x01;

        /// <summary>
        ///  DAC channel2 software trigger 
        /// </summary>
        public static byte DAC_SWTRIGR_SWTRIG2 = 0x02;


        /*****************  Bit definition for DAC_DHR12R1 register  ******************/
        /// <summary>
        ///  DAC channel1 12-bit Right aligned data 
        /// </summary>
        public static UInt16 DAC_DHR12R1_DACC1DHR = 0x0FFF;


        /*****************  Bit definition for DAC_DHR12L1 register  ******************/
        /// <summary>
        ///  DAC channel1 12-bit Left aligned data 
        /// </summary>
        public static UInt16 DAC_DHR12L1_DACC1DHR = 0xFFF0;


        /******************  Bit definition for DAC_DHR8R1 register  ******************/
        /// <summary>
        ///  DAC channel1 8-bit Right aligned data 
        /// </summary>
        public static byte DAC_DHR8R1_DACC1DHR = 0xFF;


        /*****************  Bit definition for DAC_DHR12R2 register  ******************/
        /// <summary>
        ///  DAC channel2 12-bit Right aligned data 
        /// </summary>
        public static UInt16 DAC_DHR12R2_DACC2DHR = 0x0FFF;


        /*****************  Bit definition for DAC_DHR12L2 register  ******************/
        /// <summary>
        ///  DAC channel2 12-bit Left aligned data 
        /// </summary>
        public static UInt16 DAC_DHR12L2_DACC2DHR = 0xFFF0;


        /******************  Bit definition for DAC_DHR8R2 register  ******************/
        /// <summary>
        ///  DAC channel2 8-bit Right aligned data 
        /// </summary>
        public static byte DAC_DHR8R2_DACC2DHR = 0xFF;


        /*****************  Bit definition for DAC_DHR12RD register  ******************/
        /// <summary>
        ///  DAC channel1 12-bit Right aligned data 
        /// </summary>
        public static UInt32 DAC_DHR12RD_DACC1DHR = 0x00000FFF;

        /// <summary>
        ///  DAC channel2 12-bit Right aligned data 
        /// </summary>
        public static UInt32 DAC_DHR12RD_DACC2DHR = 0x0FFF0000;


        /*****************  Bit definition for DAC_DHR12LD register  ******************/
        /// <summary>
        ///  DAC channel1 12-bit Left aligned data 
        /// </summary>
        public static UInt32 DAC_DHR12LD_DACC1DHR = 0x0000FFF0;

        /// <summary>
        ///  DAC channel2 12-bit Left aligned data 
        /// </summary>
        public static UInt32 DAC_DHR12LD_DACC2DHR = 0xFFF00000;


        /******************  Bit definition for DAC_DHR8RD register  ******************/
        /// <summary>
        ///  DAC channel1 8-bit Right aligned data 
        /// </summary>
        public static UInt16 DAC_DHR8RD_DACC1DHR = 0x00FF;

        /// <summary>
        ///  DAC channel2 8-bit Right aligned data 
        /// </summary>
        public static UInt16 DAC_DHR8RD_DACC2DHR = 0xFF00;


        /*******************  Bit definition for DAC_DOR1 register  *******************/
        /// <summary>
        ///  DAC channel1 data output 
        /// </summary>
        public static UInt16 DAC_DOR1_DACC1DOR = 0x0FFF;


        /*******************  Bit definition for DAC_DOR2 register  *******************/
        /// <summary>
        ///  DAC channel2 data output 
        /// </summary>
        public static UInt16 DAC_DOR2_DACC2DOR = 0x0FFF;


        /********************  Bit definition for DAC_SR register  ********************/
        /// <summary>
        ///  DAC channel1 DMA underrun flag 
        /// </summary>
        public static UInt32 DAC_SR_DMAUDR1 = 0x00002000;

        /// <summary>
        ///  DAC channel2 DMA underrun flag 
        /// </summary>
        public static UInt32 DAC_SR_DMAUDR2 = 0x20000000;


        /******************************************************************************/
        /*                                                                            */
        /*                                 Debug MCU                                  */
        /*                                                                            */
        /******************************************************************************/

        /******************************************************************************/
        /*                                                                            */
        /*                                    DCMI                                    */
        /*                                                                            */
        /******************************************************************************/
        /********************  Bits definition for DCMI_CR register  ******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_CR_CAPTURE = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_CR_CM = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_CR_CROP = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_CR_JPEG = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_CR_ESS = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_CR_PCKPOL = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_CR_HSPOL = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_CR_VSPOL = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_CR_FCRC_0 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_CR_FCRC_1 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_CR_EDM_0 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_CR_EDM_1 = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_CR_CRE = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_CR_ENABLE = 0x00004000;


        /********************  Bits definition for DCMI_SR register  ******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_SR_HSYNC = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_SR_VSYNC = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_SR_FNE = 0x00000004;


        /********************  Bits definition for DCMI_RISR register  ****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_RISR_FRAME_RIS = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_RISR_OVF_RIS = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_RISR_ERR_RIS = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_RISR_VSYNC_RIS = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_RISR_LINE_RIS = 0x00000010;


        /********************  Bits definition for DCMI_IER register  *****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_IER_FRAME_IE = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_IER_OVF_IE = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_IER_ERR_IE = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_IER_VSYNC_IE = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_IER_LINE_IE = 0x00000010;


        /********************  Bits definition for DCMI_MISR register  ****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_MISR_FRAME_MIS = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_MISR_OVF_MIS = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_MISR_ERR_MIS = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_MISR_VSYNC_MIS = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_MISR_LINE_MIS = 0x00000010;


        /********************  Bits definition for DCMI_ICR register  *****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_ICR_FRAME_ISC = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_ICR_OVF_ISC = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_ICR_ERR_ISC = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_ICR_VSYNC_ISC = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DCMI_ICR_LINE_ISC = 0x00000010;


        /******************************************************************************/
        /*                                                                            */
        /*                             DMA Controller                                 */
        /*                                                                            */
        /******************************************************************************/
        /********************  Bits definition for DMA_SxCR register  *****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_CHSEL = 0x0E000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_CHSEL_0 = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_CHSEL_1 = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_CHSEL_2 = 0x08000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_MBURST = 0x01800000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_MBURST_0 = 0x00800000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_MBURST_1 = 0x01000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_PBURST = 0x00600000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_PBURST_0 = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_PBURST_1 = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_ACK = 0x00100000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_CT = 0x00080000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_DBM = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_PL = 0x00030000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_PL_0 = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_PL_1 = 0x00020000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_PINCOS = 0x00008000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_MSIZE = 0x00006000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_MSIZE_0 = 0x00002000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_MSIZE_1 = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_PSIZE = 0x00001800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_PSIZE_0 = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_PSIZE_1 = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_MINC = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_PINC = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_CIRC = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_DIR = 0x000000C0;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_DIR_0 = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_DIR_1 = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_PFCTRL = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_TCIE = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_HTIE = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_TEIE = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_DMEIE = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxCR_EN = 0x00000001;


        /********************  Bits definition for DMA_SxCNDTR register  **************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxNDT = 0x0000FFFF;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxNDT_0 = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxNDT_1 = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxNDT_2 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxNDT_3 = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxNDT_4 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxNDT_5 = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxNDT_6 = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxNDT_7 = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxNDT_8 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxNDT_9 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxNDT_10 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxNDT_11 = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxNDT_12 = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxNDT_13 = 0x00002000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxNDT_14 = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxNDT_15 = 0x00008000;


        /********************  Bits definition for DMA_SxFCR register  ****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxFCR_FEIE = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxFCR_FS = 0x00000038;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxFCR_FS_0 = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxFCR_FS_1 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxFCR_FS_2 = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxFCR_DMDIS = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxFCR_FTH = 0x00000003;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxFCR_FTH_0 = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_SxFCR_FTH_1 = 0x00000002;


        /********************  Bits definition for DMA_LISR register  *****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LISR_TCIF3 = 0x08000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LISR_HTIF3 = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LISR_TEIF3 = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LISR_DMEIF3 = 0x01000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LISR_FEIF3 = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LISR_TCIF2 = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LISR_HTIF2 = 0x00100000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LISR_TEIF2 = 0x00080000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LISR_DMEIF2 = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LISR_FEIF2 = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LISR_TCIF1 = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LISR_HTIF1 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LISR_TEIF1 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LISR_DMEIF1 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LISR_FEIF1 = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LISR_TCIF0 = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LISR_HTIF0 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LISR_TEIF0 = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LISR_DMEIF0 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LISR_FEIF0 = 0x00000001;


        /********************  Bits definition for DMA_HISR register  *****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HISR_TCIF7 = 0x08000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HISR_HTIF7 = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HISR_TEIF7 = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HISR_DMEIF7 = 0x01000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HISR_FEIF7 = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HISR_TCIF6 = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HISR_HTIF6 = 0x00100000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HISR_TEIF6 = 0x00080000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HISR_DMEIF6 = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HISR_FEIF6 = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HISR_TCIF5 = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HISR_HTIF5 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HISR_TEIF5 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HISR_DMEIF5 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HISR_FEIF5 = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HISR_TCIF4 = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HISR_HTIF4 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HISR_TEIF4 = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HISR_DMEIF4 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HISR_FEIF4 = 0x00000001;


        /********************  Bits definition for DMA_LIFCR register  ****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LIFCR_CTCIF3 = 0x08000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LIFCR_CHTIF3 = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LIFCR_CTEIF3 = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LIFCR_CDMEIF3 = 0x01000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LIFCR_CFEIF3 = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LIFCR_CTCIF2 = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LIFCR_CHTIF2 = 0x00100000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LIFCR_CTEIF2 = 0x00080000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LIFCR_CDMEIF2 = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LIFCR_CFEIF2 = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LIFCR_CTCIF1 = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LIFCR_CHTIF1 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LIFCR_CTEIF1 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LIFCR_CDMEIF1 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LIFCR_CFEIF1 = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LIFCR_CTCIF0 = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LIFCR_CHTIF0 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LIFCR_CTEIF0 = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LIFCR_CDMEIF0 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_LIFCR_CFEIF0 = 0x00000001;


        /********************  Bits definition for DMA_HIFCR  register  ****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HIFCR_CTCIF7 = 0x08000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HIFCR_CHTIF7 = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HIFCR_CTEIF7 = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HIFCR_CDMEIF7 = 0x01000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HIFCR_CFEIF7 = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HIFCR_CTCIF6 = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HIFCR_CHTIF6 = 0x00100000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HIFCR_CTEIF6 = 0x00080000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HIFCR_CDMEIF6 = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HIFCR_CFEIF6 = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HIFCR_CTCIF5 = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HIFCR_CHTIF5 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HIFCR_CTEIF5 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HIFCR_CDMEIF5 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HIFCR_CFEIF5 = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HIFCR_CTCIF4 = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HIFCR_CHTIF4 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HIFCR_CTEIF4 = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HIFCR_CDMEIF4 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DMA_HIFCR_CFEIF4 = 0x00000001;


        /******************************************************************************/
        /*                                                                            */
        /*                    External Interrupt/Event Controller                     */
        /*                                                                            */
        /******************************************************************************/
        /*******************  Bit definition for EXTI_IMR register  *******************/
        /// <summary>
        ///   Interrupt Mask on line 0 
        /// </summary>
        public static UInt32 EXTI_IMR_MR0 = 0x00000001;

        /// <summary>
        ///   Interrupt Mask on line 1 
        /// </summary>
        public static UInt32 EXTI_IMR_MR1 = 0x00000002;

        /// <summary>
        ///   Interrupt Mask on line 2 
        /// </summary>
        public static UInt32 EXTI_IMR_MR2 = 0x00000004;

        /// <summary>
        ///   Interrupt Mask on line 3 
        /// </summary>
        public static UInt32 EXTI_IMR_MR3 = 0x00000008;

        /// <summary>
        ///   Interrupt Mask on line 4 
        /// </summary>
        public static UInt32 EXTI_IMR_MR4 = 0x00000010;

        /// <summary>
        ///   Interrupt Mask on line 5 
        /// </summary>
        public static UInt32 EXTI_IMR_MR5 = 0x00000020;

        /// <summary>
        ///   Interrupt Mask on line 6 
        /// </summary>
        public static UInt32 EXTI_IMR_MR6 = 0x00000040;

        /// <summary>
        ///   Interrupt Mask on line 7 
        /// </summary>
        public static UInt32 EXTI_IMR_MR7 = 0x00000080;

        /// <summary>
        ///   Interrupt Mask on line 8 
        /// </summary>
        public static UInt32 EXTI_IMR_MR8 = 0x00000100;

        /// <summary>
        ///   Interrupt Mask on line 9 
        /// </summary>
        public static UInt32 EXTI_IMR_MR9 = 0x00000200;

        /// <summary>
        ///   Interrupt Mask on line 10 
        /// </summary>
        public static UInt32 EXTI_IMR_MR10 = 0x00000400;

        /// <summary>
        ///   Interrupt Mask on line 11 
        /// </summary>
        public static UInt32 EXTI_IMR_MR11 = 0x00000800;

        /// <summary>
        ///   Interrupt Mask on line 12 
        /// </summary>
        public static UInt32 EXTI_IMR_MR12 = 0x00001000;

        /// <summary>
        ///   Interrupt Mask on line 13 
        /// </summary>
        public static UInt32 EXTI_IMR_MR13 = 0x00002000;

        /// <summary>
        ///   Interrupt Mask on line 14 
        /// </summary>
        public static UInt32 EXTI_IMR_MR14 = 0x00004000;

        /// <summary>
        ///   Interrupt Mask on line 15 
        /// </summary>
        public static UInt32 EXTI_IMR_MR15 = 0x00008000;

        /// <summary>
        ///   Interrupt Mask on line 16 
        /// </summary>
        public static UInt32 EXTI_IMR_MR16 = 0x00010000;

        /// <summary>
        ///   Interrupt Mask on line 17 
        /// </summary>
        public static UInt32 EXTI_IMR_MR17 = 0x00020000;

        /// <summary>
        ///   Interrupt Mask on line 18 
        /// </summary>
        public static UInt32 EXTI_IMR_MR18 = 0x00040000;

        /// <summary>
        ///   Interrupt Mask on line 19 
        /// </summary>
        public static UInt32 EXTI_IMR_MR19 = 0x00080000;


        /*******************  Bit definition for EXTI_EMR register  *******************/
        /// <summary>
        ///   Event Mask on line 0 
        /// </summary>
        public static UInt32 EXTI_EMR_MR0 = 0x00000001;

        /// <summary>
        ///   Event Mask on line 1 
        /// </summary>
        public static UInt32 EXTI_EMR_MR1 = 0x00000002;

        /// <summary>
        ///   Event Mask on line 2 
        /// </summary>
        public static UInt32 EXTI_EMR_MR2 = 0x00000004;

        /// <summary>
        ///   Event Mask on line 3 
        /// </summary>
        public static UInt32 EXTI_EMR_MR3 = 0x00000008;

        /// <summary>
        ///   Event Mask on line 4 
        /// </summary>
        public static UInt32 EXTI_EMR_MR4 = 0x00000010;

        /// <summary>
        ///   Event Mask on line 5 
        /// </summary>
        public static UInt32 EXTI_EMR_MR5 = 0x00000020;

        /// <summary>
        ///   Event Mask on line 6 
        /// </summary>
        public static UInt32 EXTI_EMR_MR6 = 0x00000040;

        /// <summary>
        ///   Event Mask on line 7 
        /// </summary>
        public static UInt32 EXTI_EMR_MR7 = 0x00000080;

        /// <summary>
        ///   Event Mask on line 8 
        /// </summary>
        public static UInt32 EXTI_EMR_MR8 = 0x00000100;

        /// <summary>
        ///   Event Mask on line 9 
        /// </summary>
        public static UInt32 EXTI_EMR_MR9 = 0x00000200;

        /// <summary>
        ///   Event Mask on line 10 
        /// </summary>
        public static UInt32 EXTI_EMR_MR10 = 0x00000400;

        /// <summary>
        ///   Event Mask on line 11 
        /// </summary>
        public static UInt32 EXTI_EMR_MR11 = 0x00000800;

        /// <summary>
        ///   Event Mask on line 12 
        /// </summary>
        public static UInt32 EXTI_EMR_MR12 = 0x00001000;

        /// <summary>
        ///   Event Mask on line 13 
        /// </summary>
        public static UInt32 EXTI_EMR_MR13 = 0x00002000;

        /// <summary>
        ///   Event Mask on line 14 
        /// </summary>
        public static UInt32 EXTI_EMR_MR14 = 0x00004000;

        /// <summary>
        ///   Event Mask on line 15 
        /// </summary>
        public static UInt32 EXTI_EMR_MR15 = 0x00008000;

        /// <summary>
        ///   Event Mask on line 16 
        /// </summary>
        public static UInt32 EXTI_EMR_MR16 = 0x00010000;

        /// <summary>
        ///   Event Mask on line 17 
        /// </summary>
        public static UInt32 EXTI_EMR_MR17 = 0x00020000;

        /// <summary>
        ///   Event Mask on line 18 
        /// </summary>
        public static UInt32 EXTI_EMR_MR18 = 0x00040000;

        /// <summary>
        ///   Event Mask on line 19 
        /// </summary>
        public static UInt32 EXTI_EMR_MR19 = 0x00080000;


        /******************  Bit definition for EXTI_RTSR register  *******************/
        /// <summary>
        ///   Rising trigger event configuration bit of line 0 
        /// </summary>
        public static UInt32 EXTI_RTSR_TR0 = 0x00000001;

        /// <summary>
        ///   Rising trigger event configuration bit of line 1 
        /// </summary>
        public static UInt32 EXTI_RTSR_TR1 = 0x00000002;

        /// <summary>
        ///   Rising trigger event configuration bit of line 2 
        /// </summary>
        public static UInt32 EXTI_RTSR_TR2 = 0x00000004;

        /// <summary>
        ///   Rising trigger event configuration bit of line 3 
        /// </summary>
        public static UInt32 EXTI_RTSR_TR3 = 0x00000008;

        /// <summary>
        ///   Rising trigger event configuration bit of line 4 
        /// </summary>
        public static UInt32 EXTI_RTSR_TR4 = 0x00000010;

        /// <summary>
        ///   Rising trigger event configuration bit of line 5 
        /// </summary>
        public static UInt32 EXTI_RTSR_TR5 = 0x00000020;

        /// <summary>
        ///   Rising trigger event configuration bit of line 6 
        /// </summary>
        public static UInt32 EXTI_RTSR_TR6 = 0x00000040;

        /// <summary>
        ///   Rising trigger event configuration bit of line 7 
        /// </summary>
        public static UInt32 EXTI_RTSR_TR7 = 0x00000080;

        /// <summary>
        ///   Rising trigger event configuration bit of line 8 
        /// </summary>
        public static UInt32 EXTI_RTSR_TR8 = 0x00000100;

        /// <summary>
        ///   Rising trigger event configuration bit of line 9 
        /// </summary>
        public static UInt32 EXTI_RTSR_TR9 = 0x00000200;

        /// <summary>
        ///   Rising trigger event configuration bit of line 10 
        /// </summary>
        public static UInt32 EXTI_RTSR_TR10 = 0x00000400;

        /// <summary>
        ///   Rising trigger event configuration bit of line 11 
        /// </summary>
        public static UInt32 EXTI_RTSR_TR11 = 0x00000800;

        /// <summary>
        ///   Rising trigger event configuration bit of line 12 
        /// </summary>
        public static UInt32 EXTI_RTSR_TR12 = 0x00001000;

        /// <summary>
        ///   Rising trigger event configuration bit of line 13 
        /// </summary>
        public static UInt32 EXTI_RTSR_TR13 = 0x00002000;

        /// <summary>
        ///   Rising trigger event configuration bit of line 14 
        /// </summary>
        public static UInt32 EXTI_RTSR_TR14 = 0x00004000;

        /// <summary>
        ///   Rising trigger event configuration bit of line 15 
        /// </summary>
        public static UInt32 EXTI_RTSR_TR15 = 0x00008000;

        /// <summary>
        ///   Rising trigger event configuration bit of line 16 
        /// </summary>
        public static UInt32 EXTI_RTSR_TR16 = 0x00010000;

        /// <summary>
        ///   Rising trigger event configuration bit of line 17 
        /// </summary>
        public static UInt32 EXTI_RTSR_TR17 = 0x00020000;

        /// <summary>
        ///   Rising trigger event configuration bit of line 18 
        /// </summary>
        public static UInt32 EXTI_RTSR_TR18 = 0x00040000;

        /// <summary>
        ///   Rising trigger event configuration bit of line 19 
        /// </summary>
        public static UInt32 EXTI_RTSR_TR19 = 0x00080000;


        /******************  Bit definition for EXTI_FTSR register  *******************/
        /// <summary>
        ///   Falling trigger event configuration bit of line 0 
        /// </summary>
        public static UInt32 EXTI_FTSR_TR0 = 0x00000001;

        /// <summary>
        ///   Falling trigger event configuration bit of line 1 
        /// </summary>
        public static UInt32 EXTI_FTSR_TR1 = 0x00000002;

        /// <summary>
        ///   Falling trigger event configuration bit of line 2 
        /// </summary>
        public static UInt32 EXTI_FTSR_TR2 = 0x00000004;

        /// <summary>
        ///   Falling trigger event configuration bit of line 3 
        /// </summary>
        public static UInt32 EXTI_FTSR_TR3 = 0x00000008;

        /// <summary>
        ///   Falling trigger event configuration bit of line 4 
        /// </summary>
        public static UInt32 EXTI_FTSR_TR4 = 0x00000010;

        /// <summary>
        ///   Falling trigger event configuration bit of line 5 
        /// </summary>
        public static UInt32 EXTI_FTSR_TR5 = 0x00000020;

        /// <summary>
        ///   Falling trigger event configuration bit of line 6 
        /// </summary>
        public static UInt32 EXTI_FTSR_TR6 = 0x00000040;

        /// <summary>
        ///   Falling trigger event configuration bit of line 7 
        /// </summary>
        public static UInt32 EXTI_FTSR_TR7 = 0x00000080;

        /// <summary>
        ///   Falling trigger event configuration bit of line 8 
        /// </summary>
        public static UInt32 EXTI_FTSR_TR8 = 0x00000100;

        /// <summary>
        ///   Falling trigger event configuration bit of line 9 
        /// </summary>
        public static UInt32 EXTI_FTSR_TR9 = 0x00000200;

        /// <summary>
        ///   Falling trigger event configuration bit of line 10 
        /// </summary>
        public static UInt32 EXTI_FTSR_TR10 = 0x00000400;

        /// <summary>
        ///   Falling trigger event configuration bit of line 11 
        /// </summary>
        public static UInt32 EXTI_FTSR_TR11 = 0x00000800;

        /// <summary>
        ///   Falling trigger event configuration bit of line 12 
        /// </summary>
        public static UInt32 EXTI_FTSR_TR12 = 0x00001000;

        /// <summary>
        ///   Falling trigger event configuration bit of line 13 
        /// </summary>
        public static UInt32 EXTI_FTSR_TR13 = 0x00002000;

        /// <summary>
        ///   Falling trigger event configuration bit of line 14 
        /// </summary>
        public static UInt32 EXTI_FTSR_TR14 = 0x00004000;

        /// <summary>
        ///   Falling trigger event configuration bit of line 15 
        /// </summary>
        public static UInt32 EXTI_FTSR_TR15 = 0x00008000;

        /// <summary>
        ///   Falling trigger event configuration bit of line 16 
        /// </summary>
        public static UInt32 EXTI_FTSR_TR16 = 0x00010000;

        /// <summary>
        ///   Falling trigger event configuration bit of line 17 
        /// </summary>
        public static UInt32 EXTI_FTSR_TR17 = 0x00020000;

        /// <summary>
        ///   Falling trigger event configuration bit of line 18 
        /// </summary>
        public static UInt32 EXTI_FTSR_TR18 = 0x00040000;

        /// <summary>
        ///   Falling trigger event configuration bit of line 19 
        /// </summary>
        public static UInt32 EXTI_FTSR_TR19 = 0x00080000;


        /******************  Bit definition for EXTI_SWIER register  ******************/
        /// <summary>
        ///   Software Interrupt on line 0 
        /// </summary>
        public static UInt32 EXTI_SWIER_SWIER0 = 0x00000001;

        /// <summary>
        ///   Software Interrupt on line 1 
        /// </summary>
        public static UInt32 EXTI_SWIER_SWIER1 = 0x00000002;

        /// <summary>
        ///   Software Interrupt on line 2 
        /// </summary>
        public static UInt32 EXTI_SWIER_SWIER2 = 0x00000004;

        /// <summary>
        ///   Software Interrupt on line 3 
        /// </summary>
        public static UInt32 EXTI_SWIER_SWIER3 = 0x00000008;

        /// <summary>
        ///   Software Interrupt on line 4 
        /// </summary>
        public static UInt32 EXTI_SWIER_SWIER4 = 0x00000010;

        /// <summary>
        ///   Software Interrupt on line 5 
        /// </summary>
        public static UInt32 EXTI_SWIER_SWIER5 = 0x00000020;

        /// <summary>
        ///   Software Interrupt on line 6 
        /// </summary>
        public static UInt32 EXTI_SWIER_SWIER6 = 0x00000040;

        /// <summary>
        ///   Software Interrupt on line 7 
        /// </summary>
        public static UInt32 EXTI_SWIER_SWIER7 = 0x00000080;

        /// <summary>
        ///   Software Interrupt on line 8 
        /// </summary>
        public static UInt32 EXTI_SWIER_SWIER8 = 0x00000100;

        /// <summary>
        ///   Software Interrupt on line 9 
        /// </summary>
        public static UInt32 EXTI_SWIER_SWIER9 = 0x00000200;

        /// <summary>
        ///   Software Interrupt on line 10 
        /// </summary>
        public static UInt32 EXTI_SWIER_SWIER10 = 0x00000400;

        /// <summary>
        ///   Software Interrupt on line 11 
        /// </summary>
        public static UInt32 EXTI_SWIER_SWIER11 = 0x00000800;

        /// <summary>
        ///   Software Interrupt on line 12 
        /// </summary>
        public static UInt32 EXTI_SWIER_SWIER12 = 0x00001000;

        /// <summary>
        ///   Software Interrupt on line 13 
        /// </summary>
        public static UInt32 EXTI_SWIER_SWIER13 = 0x00002000;

        /// <summary>
        ///   Software Interrupt on line 14 
        /// </summary>
        public static UInt32 EXTI_SWIER_SWIER14 = 0x00004000;

        /// <summary>
        ///   Software Interrupt on line 15 
        /// </summary>
        public static UInt32 EXTI_SWIER_SWIER15 = 0x00008000;

        /// <summary>
        ///   Software Interrupt on line 16 
        /// </summary>
        public static UInt32 EXTI_SWIER_SWIER16 = 0x00010000;

        /// <summary>
        ///   Software Interrupt on line 17 
        /// </summary>
        public static UInt32 EXTI_SWIER_SWIER17 = 0x00020000;

        /// <summary>
        ///   Software Interrupt on line 18 
        /// </summary>
        public static UInt32 EXTI_SWIER_SWIER18 = 0x00040000;

        /// <summary>
        ///   Software Interrupt on line 19 
        /// </summary>
        public static UInt32 EXTI_SWIER_SWIER19 = 0x00080000;


        /*******************  Bit definition for EXTI_PR register  ********************/
        /// <summary>
        ///   Pending bit for line 0 
        /// </summary>
        public static UInt32 EXTI_PR_PR0 = 0x00000001;

        /// <summary>
        ///   Pending bit for line 1 
        /// </summary>
        public static UInt32 EXTI_PR_PR1 = 0x00000002;

        /// <summary>
        ///   Pending bit for line 2 
        /// </summary>
        public static UInt32 EXTI_PR_PR2 = 0x00000004;

        /// <summary>
        ///   Pending bit for line 3 
        /// </summary>
        public static UInt32 EXTI_PR_PR3 = 0x00000008;

        /// <summary>
        ///   Pending bit for line 4 
        /// </summary>
        public static UInt32 EXTI_PR_PR4 = 0x00000010;

        /// <summary>
        ///   Pending bit for line 5 
        /// </summary>
        public static UInt32 EXTI_PR_PR5 = 0x00000020;

        /// <summary>
        ///   Pending bit for line 6 
        /// </summary>
        public static UInt32 EXTI_PR_PR6 = 0x00000040;

        /// <summary>
        ///   Pending bit for line 7 
        /// </summary>
        public static UInt32 EXTI_PR_PR7 = 0x00000080;

        /// <summary>
        ///   Pending bit for line 8 
        /// </summary>
        public static UInt32 EXTI_PR_PR8 = 0x00000100;

        /// <summary>
        ///   Pending bit for line 9 
        /// </summary>
        public static UInt32 EXTI_PR_PR9 = 0x00000200;

        /// <summary>
        ///   Pending bit for line 10 
        /// </summary>
        public static UInt32 EXTI_PR_PR10 = 0x00000400;

        /// <summary>
        ///   Pending bit for line 11 
        /// </summary>
        public static UInt32 EXTI_PR_PR11 = 0x00000800;

        /// <summary>
        ///   Pending bit for line 12 
        /// </summary>
        public static UInt32 EXTI_PR_PR12 = 0x00001000;

        /// <summary>
        ///   Pending bit for line 13 
        /// </summary>
        public static UInt32 EXTI_PR_PR13 = 0x00002000;

        /// <summary>
        ///   Pending bit for line 14 
        /// </summary>
        public static UInt32 EXTI_PR_PR14 = 0x00004000;

        /// <summary>
        ///   Pending bit for line 15 
        /// </summary>
        public static UInt32 EXTI_PR_PR15 = 0x00008000;

        /// <summary>
        ///   Pending bit for line 16 
        /// </summary>
        public static UInt32 EXTI_PR_PR16 = 0x00010000;

        /// <summary>
        ///   Pending bit for line 17 
        /// </summary>
        public static UInt32 EXTI_PR_PR17 = 0x00020000;

        /// <summary>
        ///   Pending bit for line 18 
        /// </summary>
        public static UInt32 EXTI_PR_PR18 = 0x00040000;

        /// <summary>
        ///   Pending bit for line 19 
        /// </summary>
        public static UInt32 EXTI_PR_PR19 = 0x00080000;


        /******************************************************************************/
        /*                                                                            */
        /*                                    FLASH                                   */
        /*                                                                            */
        /******************************************************************************/
        /*******************  Bits definition for FLASH_ACR register  *****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_ACR_LATENCY = 0x00000007;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_ACR_LATENCY_0WS = 0x00000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_ACR_LATENCY_1WS = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_ACR_LATENCY_2WS = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_ACR_LATENCY_3WS = 0x00000003;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_ACR_LATENCY_4WS = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_ACR_LATENCY_5WS = 0x00000005;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_ACR_LATENCY_6WS = 0x00000006;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_ACR_LATENCY_7WS = 0x00000007;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_ACR_PRFTEN = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_ACR_ICEN = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_ACR_DCEN = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_ACR_ICRST = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_ACR_DCRST = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_ACR_BYTE0_ADDRESS = 0x40023C00;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_ACR_BYTE2_ADDRESS = 0x40023C03;


        /*******************  Bits definition for FLASH_SR register  ******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_SR_EOP = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_SR_SOP = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_SR_WRPERR = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_SR_PGAERR = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_SR_PGPERR = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_SR_PGSERR = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_SR_BSY = 0x00010000;


        /*******************  Bits definition for FLASH_CR register  ******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_CR_PG = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_CR_SER = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_CR_MER = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_CR_SNB_0 = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_CR_SNB_1 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_CR_SNB_2 = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_CR_SNB_3 = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_CR_PSIZE_0 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_CR_PSIZE_1 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_CR_STRT = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_CR_EOPIE = 0x01000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_CR_LOCK = 0x80000000;


        /*******************  Bits definition for FLASH_OPTCR register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_OPTLOCK = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_OPTSTRT = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_BOR_LEV_0 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_BOR_LEV_1 = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_BOR_LEV = 0x0000000C;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_WDG_SW = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_nRST_STOP = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_nRST_STDBY = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_RDP_0 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_RDP_1 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_RDP_2 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_RDP_3 = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_RDP_4 = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_RDP_5 = 0x00002000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_RDP_6 = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_RDP_7 = 0x00008000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_nWRP_0 = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_nWRP_1 = 0x00020000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_nWRP_2 = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_nWRP_3 = 0x00080000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_nWRP_4 = 0x00100000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_nWRP_5 = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_nWRP_6 = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_nWRP_7 = 0x00800000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_nWRP_8 = 0x01000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_nWRP_9 = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_nWRP_10 = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 FLASH_OPTCR_nWRP_11 = 0x08000000;


        /******************************************************************************/
        /*                                                                            */
        /*                       Flexible Static Memory Controller                    */
        /*                                                                            */
        /******************************************************************************/
        /******************  Bit definition for FSMC_BCR1 register  *******************/
        /// <summary>
        ///  Memory bank enable bit 
        /// </summary>
        public static UInt32 FSMC_BCR1_MBKEN = 0x00000001;

        /// <summary>
        ///  Address/data multiplexing enable bit 
        /// </summary>
        public static UInt32 FSMC_BCR1_MUXEN = 0x00000002;


        /// <summary>
        ///  MTYP[1:0] bits (Memory type) 
        /// </summary>
        public static UInt32 FSMC_BCR1_MTYP = 0x0000000C;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BCR1_MTYP_0 = 0x00000004;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BCR1_MTYP_1 = 0x00000008;


        /// <summary>
        ///  MWID[1:0] bits (Memory data bus width) 
        /// </summary>
        public static UInt32 FSMC_BCR1_MWID = 0x00000030;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BCR1_MWID_0 = 0x00000010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BCR1_MWID_1 = 0x00000020;


        /// <summary>
        ///  Flash access enable 
        /// </summary>
        public static UInt32 FSMC_BCR1_FACCEN = 0x00000040;

        /// <summary>
        ///  Burst enable bit 
        /// </summary>
        public static UInt32 FSMC_BCR1_BURSTEN = 0x00000100;

        /// <summary>
        ///  Wait signal polarity bit 
        /// </summary>
        public static UInt32 FSMC_BCR1_WAITPOL = 0x00000200;

        /// <summary>
        ///  Wrapped burst mode support 
        /// </summary>
        public static UInt32 FSMC_BCR1_WRAPMOD = 0x00000400;

        /// <summary>
        ///  Wait timing configuration 
        /// </summary>
        public static UInt32 FSMC_BCR1_WAITCFG = 0x00000800;

        /// <summary>
        ///  Write enable bit 
        /// </summary>
        public static UInt32 FSMC_BCR1_WREN = 0x00001000;

        /// <summary>
        ///  Wait enable bit 
        /// </summary>
        public static UInt32 FSMC_BCR1_WAITEN = 0x00002000;

        /// <summary>
        ///  Extended mode enable 
        /// </summary>
        public static UInt32 FSMC_BCR1_EXTMOD = 0x00004000;

        /// <summary>
        ///  Asynchronous wait 
        /// </summary>
        public static UInt32 FSMC_BCR1_ASYNCWAIT = 0x00008000;

        /// <summary>
        ///  Write burst enable 
        /// </summary>
        public static UInt32 FSMC_BCR1_CBURSTRW = 0x00080000;


        /******************  Bit definition for FSMC_BCR2 register  *******************/
        /// <summary>
        ///  Memory bank enable bit 
        /// </summary>
        public static UInt32 FSMC_BCR2_MBKEN = 0x00000001;

        /// <summary>
        ///  Address/data multiplexing enable bit 
        /// </summary>
        public static UInt32 FSMC_BCR2_MUXEN = 0x00000002;


        /// <summary>
        ///  MTYP[1:0] bits (Memory type) 
        /// </summary>
        public static UInt32 FSMC_BCR2_MTYP = 0x0000000C;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BCR2_MTYP_0 = 0x00000004;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BCR2_MTYP_1 = 0x00000008;


        /// <summary>
        ///  MWID[1:0] bits (Memory data bus width) 
        /// </summary>
        public static UInt32 FSMC_BCR2_MWID = 0x00000030;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BCR2_MWID_0 = 0x00000010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BCR2_MWID_1 = 0x00000020;


        /// <summary>
        ///  Flash access enable 
        /// </summary>
        public static UInt32 FSMC_BCR2_FACCEN = 0x00000040;

        /// <summary>
        ///  Burst enable bit 
        /// </summary>
        public static UInt32 FSMC_BCR2_BURSTEN = 0x00000100;

        /// <summary>
        ///  Wait signal polarity bit 
        /// </summary>
        public static UInt32 FSMC_BCR2_WAITPOL = 0x00000200;

        /// <summary>
        ///  Wrapped burst mode support 
        /// </summary>
        public static UInt32 FSMC_BCR2_WRAPMOD = 0x00000400;

        /// <summary>
        ///  Wait timing configuration 
        /// </summary>
        public static UInt32 FSMC_BCR2_WAITCFG = 0x00000800;

        /// <summary>
        ///  Write enable bit 
        /// </summary>
        public static UInt32 FSMC_BCR2_WREN = 0x00001000;

        /// <summary>
        ///  Wait enable bit 
        /// </summary>
        public static UInt32 FSMC_BCR2_WAITEN = 0x00002000;

        /// <summary>
        ///  Extended mode enable 
        /// </summary>
        public static UInt32 FSMC_BCR2_EXTMOD = 0x00004000;

        /// <summary>
        ///  Asynchronous wait 
        /// </summary>
        public static UInt32 FSMC_BCR2_ASYNCWAIT = 0x00008000;

        /// <summary>
        ///  Write burst enable 
        /// </summary>
        public static UInt32 FSMC_BCR2_CBURSTRW = 0x00080000;


        /******************  Bit definition for FSMC_BCR3 register  *******************/
        /// <summary>
        ///  Memory bank enable bit 
        /// </summary>
        public static UInt32 FSMC_BCR3_MBKEN = 0x00000001;

        /// <summary>
        ///  Address/data multiplexing enable bit 
        /// </summary>
        public static UInt32 FSMC_BCR3_MUXEN = 0x00000002;


        /// <summary>
        ///  MTYP[1:0] bits (Memory type) 
        /// </summary>
        public static UInt32 FSMC_BCR3_MTYP = 0x0000000C;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BCR3_MTYP_0 = 0x00000004;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BCR3_MTYP_1 = 0x00000008;


        /// <summary>
        ///  MWID[1:0] bits (Memory data bus width) 
        /// </summary>
        public static UInt32 FSMC_BCR3_MWID = 0x00000030;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BCR3_MWID_0 = 0x00000010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BCR3_MWID_1 = 0x00000020;


        /// <summary>
        ///  Flash access enable 
        /// </summary>
        public static UInt32 FSMC_BCR3_FACCEN = 0x00000040;

        /// <summary>
        ///  Burst enable bit 
        /// </summary>
        public static UInt32 FSMC_BCR3_BURSTEN = 0x00000100;

        /// <summary>
        ///  Wait signal polarity bit. 
        /// </summary>
        public static UInt32 FSMC_BCR3_WAITPOL = 0x00000200;

        /// <summary>
        ///  Wrapped burst mode support 
        /// </summary>
        public static UInt32 FSMC_BCR3_WRAPMOD = 0x00000400;

        /// <summary>
        ///  Wait timing configuration 
        /// </summary>
        public static UInt32 FSMC_BCR3_WAITCFG = 0x00000800;

        /// <summary>
        ///  Write enable bit 
        /// </summary>
        public static UInt32 FSMC_BCR3_WREN = 0x00001000;

        /// <summary>
        ///  Wait enable bit 
        /// </summary>
        public static UInt32 FSMC_BCR3_WAITEN = 0x00002000;

        /// <summary>
        ///  Extended mode enable 
        /// </summary>
        public static UInt32 FSMC_BCR3_EXTMOD = 0x00004000;

        /// <summary>
        ///  Asynchronous wait 
        /// </summary>
        public static UInt32 FSMC_BCR3_ASYNCWAIT = 0x00008000;

        /// <summary>
        ///  Write burst enable 
        /// </summary>
        public static UInt32 FSMC_BCR3_CBURSTRW = 0x00080000;


        /******************  Bit definition for FSMC_BCR4 register  *******************/
        /// <summary>
        ///  Memory bank enable bit 
        /// </summary>
        public static UInt32 FSMC_BCR4_MBKEN = 0x00000001;

        /// <summary>
        ///  Address/data multiplexing enable bit 
        /// </summary>
        public static UInt32 FSMC_BCR4_MUXEN = 0x00000002;


        /// <summary>
        ///  MTYP[1:0] bits (Memory type) 
        /// </summary>
        public static UInt32 FSMC_BCR4_MTYP = 0x0000000C;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BCR4_MTYP_0 = 0x00000004;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BCR4_MTYP_1 = 0x00000008;


        /// <summary>
        ///  MWID[1:0] bits (Memory data bus width) 
        /// </summary>
        public static UInt32 FSMC_BCR4_MWID = 0x00000030;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BCR4_MWID_0 = 0x00000010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BCR4_MWID_1 = 0x00000020;


        /// <summary>
        ///  Flash access enable 
        /// </summary>
        public static UInt32 FSMC_BCR4_FACCEN = 0x00000040;

        /// <summary>
        ///  Burst enable bit 
        /// </summary>
        public static UInt32 FSMC_BCR4_BURSTEN = 0x00000100;

        /// <summary>
        ///  Wait signal polarity bit 
        /// </summary>
        public static UInt32 FSMC_BCR4_WAITPOL = 0x00000200;

        /// <summary>
        ///  Wrapped burst mode support 
        /// </summary>
        public static UInt32 FSMC_BCR4_WRAPMOD = 0x00000400;

        /// <summary>
        ///  Wait timing configuration 
        /// </summary>
        public static UInt32 FSMC_BCR4_WAITCFG = 0x00000800;

        /// <summary>
        ///  Write enable bit 
        /// </summary>
        public static UInt32 FSMC_BCR4_WREN = 0x00001000;

        /// <summary>
        ///  Wait enable bit 
        /// </summary>
        public static UInt32 FSMC_BCR4_WAITEN = 0x00002000;

        /// <summary>
        ///  Extended mode enable 
        /// </summary>
        public static UInt32 FSMC_BCR4_EXTMOD = 0x00004000;

        /// <summary>
        ///  Asynchronous wait 
        /// </summary>
        public static UInt32 FSMC_BCR4_ASYNCWAIT = 0x00008000;

        /// <summary>
        ///  Write burst enable 
        /// </summary>
        public static UInt32 FSMC_BCR4_CBURSTRW = 0x00080000;


        /******************  Bit definition for FSMC_BTR1 register  ******************/
        /// <summary>
        ///  ADDSET[3:0] bits (Address setup phase duration) 
        /// </summary>
        public static UInt32 FSMC_BTR1_ADDSET = 0x0000000F;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR1_ADDSET_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR1_ADDSET_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR1_ADDSET_2 = 0x00000004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR1_ADDSET_3 = 0x00000008;


        /// <summary>
        ///  ADDHLD[3:0] bits (Address-hold phase duration) 
        /// </summary>
        public static UInt32 FSMC_BTR1_ADDHLD = 0x000000F0;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR1_ADDHLD_0 = 0x00000010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR1_ADDHLD_1 = 0x00000020;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR1_ADDHLD_2 = 0x00000040;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR1_ADDHLD_3 = 0x00000080;


        /// <summary>
        ///  DATAST [3:0] bits (Data-phase duration) 
        /// </summary>
        public static UInt32 FSMC_BTR1_DATAST = 0x0000FF00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR1_DATAST_0 = 0x00000100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR1_DATAST_1 = 0x00000200;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR1_DATAST_2 = 0x00000400;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR1_DATAST_3 = 0x00000800;


        /// <summary>
        ///  BUSTURN[3:0] bits (Bus turnaround phase duration) 
        /// </summary>
        public static UInt32 FSMC_BTR1_BUSTURN = 0x000F0000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR1_BUSTURN_0 = 0x00010000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR1_BUSTURN_1 = 0x00020000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR1_BUSTURN_2 = 0x00040000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR1_BUSTURN_3 = 0x00080000;


        /// <summary>
        ///  CLKDIV[3:0] bits (Clock divide ratio) 
        /// </summary>
        public static UInt32 FSMC_BTR1_CLKDIV = 0x00F00000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR1_CLKDIV_0 = 0x00100000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR1_CLKDIV_1 = 0x00200000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR1_CLKDIV_2 = 0x00400000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR1_CLKDIV_3 = 0x00800000;


        /// <summary>
        ///  DATLA[3:0] bits (Data latency) 
        /// </summary>
        public static UInt32 FSMC_BTR1_DATLAT = 0x0F000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR1_DATLAT_0 = 0x01000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR1_DATLAT_1 = 0x02000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR1_DATLAT_2 = 0x04000000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR1_DATLAT_3 = 0x08000000;


        /// <summary>
        ///  ACCMOD[1:0] bits (Access mode) 
        /// </summary>
        public static UInt32 FSMC_BTR1_ACCMOD = 0x30000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR1_ACCMOD_0 = 0x10000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR1_ACCMOD_1 = 0x20000000;


        /******************  Bit definition for FSMC_BTR2 register  *******************/
        /// <summary>
        ///  ADDSET[3:0] bits (Address setup phase duration) 
        /// </summary>
        public static UInt32 FSMC_BTR2_ADDSET = 0x0000000F;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR2_ADDSET_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR2_ADDSET_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR2_ADDSET_2 = 0x00000004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR2_ADDSET_3 = 0x00000008;


        /// <summary>
        ///  ADDHLD[3:0] bits (Address-hold phase duration) 
        /// </summary>
        public static UInt32 FSMC_BTR2_ADDHLD = 0x000000F0;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR2_ADDHLD_0 = 0x00000010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR2_ADDHLD_1 = 0x00000020;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR2_ADDHLD_2 = 0x00000040;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR2_ADDHLD_3 = 0x00000080;


        /// <summary>
        ///  DATAST [3:0] bits (Data-phase duration) 
        /// </summary>
        public static UInt32 FSMC_BTR2_DATAST = 0x0000FF00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR2_DATAST_0 = 0x00000100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR2_DATAST_1 = 0x00000200;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR2_DATAST_2 = 0x00000400;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR2_DATAST_3 = 0x00000800;


        /// <summary>
        ///  BUSTURN[3:0] bits (Bus turnaround phase duration) 
        /// </summary>
        public static UInt32 FSMC_BTR2_BUSTURN = 0x000F0000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR2_BUSTURN_0 = 0x00010000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR2_BUSTURN_1 = 0x00020000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR2_BUSTURN_2 = 0x00040000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR2_BUSTURN_3 = 0x00080000;


        /// <summary>
        ///  CLKDIV[3:0] bits (Clock divide ratio) 
        /// </summary>
        public static UInt32 FSMC_BTR2_CLKDIV = 0x00F00000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR2_CLKDIV_0 = 0x00100000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR2_CLKDIV_1 = 0x00200000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR2_CLKDIV_2 = 0x00400000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR2_CLKDIV_3 = 0x00800000;


        /// <summary>
        ///  DATLA[3:0] bits (Data latency) 
        /// </summary>
        public static UInt32 FSMC_BTR2_DATLAT = 0x0F000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR2_DATLAT_0 = 0x01000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR2_DATLAT_1 = 0x02000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR2_DATLAT_2 = 0x04000000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR2_DATLAT_3 = 0x08000000;


        /// <summary>
        ///  ACCMOD[1:0] bits (Access mode) 
        /// </summary>
        public static UInt32 FSMC_BTR2_ACCMOD = 0x30000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR2_ACCMOD_0 = 0x10000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR2_ACCMOD_1 = 0x20000000;


        /*******************  Bit definition for FSMC_BTR3 register  *******************/
        /// <summary>
        ///  ADDSET[3:0] bits (Address setup phase duration) 
        /// </summary>
        public static UInt32 FSMC_BTR3_ADDSET = 0x0000000F;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR3_ADDSET_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR3_ADDSET_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR3_ADDSET_2 = 0x00000004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR3_ADDSET_3 = 0x00000008;


        /// <summary>
        ///  ADDHLD[3:0] bits (Address-hold phase duration) 
        /// </summary>
        public static UInt32 FSMC_BTR3_ADDHLD = 0x000000F0;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR3_ADDHLD_0 = 0x00000010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR3_ADDHLD_1 = 0x00000020;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR3_ADDHLD_2 = 0x00000040;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR3_ADDHLD_3 = 0x00000080;


        /// <summary>
        ///  DATAST [3:0] bits (Data-phase duration) 
        /// </summary>
        public static UInt32 FSMC_BTR3_DATAST = 0x0000FF00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR3_DATAST_0 = 0x00000100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR3_DATAST_1 = 0x00000200;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR3_DATAST_2 = 0x00000400;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR3_DATAST_3 = 0x00000800;


        /// <summary>
        ///  BUSTURN[3:0] bits (Bus turnaround phase duration) 
        /// </summary>
        public static UInt32 FSMC_BTR3_BUSTURN = 0x000F0000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR3_BUSTURN_0 = 0x00010000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR3_BUSTURN_1 = 0x00020000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR3_BUSTURN_2 = 0x00040000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR3_BUSTURN_3 = 0x00080000;


        /// <summary>
        ///  CLKDIV[3:0] bits (Clock divide ratio) 
        /// </summary>
        public static UInt32 FSMC_BTR3_CLKDIV = 0x00F00000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR3_CLKDIV_0 = 0x00100000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR3_CLKDIV_1 = 0x00200000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR3_CLKDIV_2 = 0x00400000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR3_CLKDIV_3 = 0x00800000;


        /// <summary>
        ///  DATLA[3:0] bits (Data latency) 
        /// </summary>
        public static UInt32 FSMC_BTR3_DATLAT = 0x0F000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR3_DATLAT_0 = 0x01000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR3_DATLAT_1 = 0x02000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR3_DATLAT_2 = 0x04000000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR3_DATLAT_3 = 0x08000000;


        /// <summary>
        ///  ACCMOD[1:0] bits (Access mode) 
        /// </summary>
        public static UInt32 FSMC_BTR3_ACCMOD = 0x30000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR3_ACCMOD_0 = 0x10000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR3_ACCMOD_1 = 0x20000000;


        /******************  Bit definition for FSMC_BTR4 register  *******************/
        /// <summary>
        ///  ADDSET[3:0] bits (Address setup phase duration) 
        /// </summary>
        public static UInt32 FSMC_BTR4_ADDSET = 0x0000000F;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR4_ADDSET_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR4_ADDSET_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR4_ADDSET_2 = 0x00000004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR4_ADDSET_3 = 0x00000008;


        /// <summary>
        ///  ADDHLD[3:0] bits (Address-hold phase duration) 
        /// </summary>
        public static UInt32 FSMC_BTR4_ADDHLD = 0x000000F0;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR4_ADDHLD_0 = 0x00000010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR4_ADDHLD_1 = 0x00000020;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR4_ADDHLD_2 = 0x00000040;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR4_ADDHLD_3 = 0x00000080;


        /// <summary>
        ///  DATAST [3:0] bits (Data-phase duration) 
        /// </summary>
        public static UInt32 FSMC_BTR4_DATAST = 0x0000FF00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR4_DATAST_0 = 0x00000100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR4_DATAST_1 = 0x00000200;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR4_DATAST_2 = 0x00000400;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR4_DATAST_3 = 0x00000800;


        /// <summary>
        ///  BUSTURN[3:0] bits (Bus turnaround phase duration) 
        /// </summary>
        public static UInt32 FSMC_BTR4_BUSTURN = 0x000F0000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR4_BUSTURN_0 = 0x00010000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR4_BUSTURN_1 = 0x00020000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR4_BUSTURN_2 = 0x00040000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR4_BUSTURN_3 = 0x00080000;


        /// <summary>
        ///  CLKDIV[3:0] bits (Clock divide ratio) 
        /// </summary>
        public static UInt32 FSMC_BTR4_CLKDIV = 0x00F00000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR4_CLKDIV_0 = 0x00100000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR4_CLKDIV_1 = 0x00200000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR4_CLKDIV_2 = 0x00400000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR4_CLKDIV_3 = 0x00800000;


        /// <summary>
        ///  DATLA[3:0] bits (Data latency) 
        /// </summary>
        public static UInt32 FSMC_BTR4_DATLAT = 0x0F000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR4_DATLAT_0 = 0x01000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR4_DATLAT_1 = 0x02000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BTR4_DATLAT_2 = 0x04000000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BTR4_DATLAT_3 = 0x08000000;


        /// <summary>
        ///  ACCMOD[1:0] bits (Access mode) 
        /// </summary>
        public static UInt32 FSMC_BTR4_ACCMOD = 0x30000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BTR4_ACCMOD_0 = 0x10000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BTR4_ACCMOD_1 = 0x20000000;


        /******************  Bit definition for FSMC_BWTR1 register  ******************/
        /// <summary>
        ///  ADDSET[3:0] bits (Address setup phase duration) 
        /// </summary>
        public static UInt32 FSMC_BWTR1_ADDSET = 0x0000000F;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR1_ADDSET_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR1_ADDSET_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BWTR1_ADDSET_2 = 0x00000004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BWTR1_ADDSET_3 = 0x00000008;


        /// <summary>
        ///  ADDHLD[3:0] bits (Address-hold phase duration) 
        /// </summary>
        public static UInt32 FSMC_BWTR1_ADDHLD = 0x000000F0;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR1_ADDHLD_0 = 0x00000010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR1_ADDHLD_1 = 0x00000020;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BWTR1_ADDHLD_2 = 0x00000040;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BWTR1_ADDHLD_3 = 0x00000080;


        /// <summary>
        ///  DATAST [3:0] bits (Data-phase duration) 
        /// </summary>
        public static UInt32 FSMC_BWTR1_DATAST = 0x0000FF00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR1_DATAST_0 = 0x00000100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR1_DATAST_1 = 0x00000200;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BWTR1_DATAST_2 = 0x00000400;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BWTR1_DATAST_3 = 0x00000800;


        /// <summary>
        ///  CLKDIV[3:0] bits (Clock divide ratio) 
        /// </summary>
        public static UInt32 FSMC_BWTR1_CLKDIV = 0x00F00000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR1_CLKDIV_0 = 0x00100000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR1_CLKDIV_1 = 0x00200000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BWTR1_CLKDIV_2 = 0x00400000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BWTR1_CLKDIV_3 = 0x00800000;


        /// <summary>
        ///  DATLA[3:0] bits (Data latency) 
        /// </summary>
        public static UInt32 FSMC_BWTR1_DATLAT = 0x0F000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR1_DATLAT_0 = 0x01000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR1_DATLAT_1 = 0x02000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BWTR1_DATLAT_2 = 0x04000000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BWTR1_DATLAT_3 = 0x08000000;


        /// <summary>
        ///  ACCMOD[1:0] bits (Access mode) 
        /// </summary>
        public static UInt32 FSMC_BWTR1_ACCMOD = 0x30000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR1_ACCMOD_0 = 0x10000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR1_ACCMOD_1 = 0x20000000;


        /******************  Bit definition for FSMC_BWTR2 register  ******************/
        /// <summary>
        ///  ADDSET[3:0] bits (Address setup phase duration) 
        /// </summary>
        public static UInt32 FSMC_BWTR2_ADDSET = 0x0000000F;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR2_ADDSET_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR2_ADDSET_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BWTR2_ADDSET_2 = 0x00000004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BWTR2_ADDSET_3 = 0x00000008;


        /// <summary>
        ///  ADDHLD[3:0] bits (Address-hold phase duration) 
        /// </summary>
        public static UInt32 FSMC_BWTR2_ADDHLD = 0x000000F0;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR2_ADDHLD_0 = 0x00000010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR2_ADDHLD_1 = 0x00000020;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BWTR2_ADDHLD_2 = 0x00000040;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BWTR2_ADDHLD_3 = 0x00000080;


        /// <summary>
        ///  DATAST [3:0] bits (Data-phase duration) 
        /// </summary>
        public static UInt32 FSMC_BWTR2_DATAST = 0x0000FF00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR2_DATAST_0 = 0x00000100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR2_DATAST_1 = 0x00000200;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BWTR2_DATAST_2 = 0x00000400;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BWTR2_DATAST_3 = 0x00000800;


        /// <summary>
        ///  CLKDIV[3:0] bits (Clock divide ratio) 
        /// </summary>
        public static UInt32 FSMC_BWTR2_CLKDIV = 0x00F00000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR2_CLKDIV_0 = 0x00100000;

        /// <summary>
        ///  Bit 1
        /// </summary>
        public static UInt32 FSMC_BWTR2_CLKDIV_1 = 0x00200000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BWTR2_CLKDIV_2 = 0x00400000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BWTR2_CLKDIV_3 = 0x00800000;


        /// <summary>
        ///  DATLA[3:0] bits (Data latency) 
        /// </summary>
        public static UInt32 FSMC_BWTR2_DATLAT = 0x0F000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR2_DATLAT_0 = 0x01000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR2_DATLAT_1 = 0x02000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BWTR2_DATLAT_2 = 0x04000000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BWTR2_DATLAT_3 = 0x08000000;


        /// <summary>
        ///  ACCMOD[1:0] bits (Access mode) 
        /// </summary>
        public static UInt32 FSMC_BWTR2_ACCMOD = 0x30000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR2_ACCMOD_0 = 0x10000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR2_ACCMOD_1 = 0x20000000;


        /******************  Bit definition for FSMC_BWTR3 register  ******************/
        /// <summary>
        ///  ADDSET[3:0] bits (Address setup phase duration) 
        /// </summary>
        public static UInt32 FSMC_BWTR3_ADDSET = 0x0000000F;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR3_ADDSET_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR3_ADDSET_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BWTR3_ADDSET_2 = 0x00000004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BWTR3_ADDSET_3 = 0x00000008;


        /// <summary>
        ///  ADDHLD[3:0] bits (Address-hold phase duration) 
        /// </summary>
        public static UInt32 FSMC_BWTR3_ADDHLD = 0x000000F0;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR3_ADDHLD_0 = 0x00000010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR3_ADDHLD_1 = 0x00000020;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BWTR3_ADDHLD_2 = 0x00000040;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BWTR3_ADDHLD_3 = 0x00000080;


        /// <summary>
        ///  DATAST [3:0] bits (Data-phase duration) 
        /// </summary>
        public static UInt32 FSMC_BWTR3_DATAST = 0x0000FF00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR3_DATAST_0 = 0x00000100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR3_DATAST_1 = 0x00000200;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BWTR3_DATAST_2 = 0x00000400;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BWTR3_DATAST_3 = 0x00000800;


        /// <summary>
        ///  CLKDIV[3:0] bits (Clock divide ratio) 
        /// </summary>
        public static UInt32 FSMC_BWTR3_CLKDIV = 0x00F00000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR3_CLKDIV_0 = 0x00100000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR3_CLKDIV_1 = 0x00200000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BWTR3_CLKDIV_2 = 0x00400000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BWTR3_CLKDIV_3 = 0x00800000;


        /// <summary>
        ///  DATLA[3:0] bits (Data latency) 
        /// </summary>
        public static UInt32 FSMC_BWTR3_DATLAT = 0x0F000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR3_DATLAT_0 = 0x01000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR3_DATLAT_1 = 0x02000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BWTR3_DATLAT_2 = 0x04000000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BWTR3_DATLAT_3 = 0x08000000;


        /// <summary>
        ///  ACCMOD[1:0] bits (Access mode) 
        /// </summary>
        public static UInt32 FSMC_BWTR3_ACCMOD = 0x30000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR3_ACCMOD_0 = 0x10000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR3_ACCMOD_1 = 0x20000000;


        /******************  Bit definition for FSMC_BWTR4 register  ******************/
        /// <summary>
        ///  ADDSET[3:0] bits (Address setup phase duration) 
        /// </summary>
        public static UInt32 FSMC_BWTR4_ADDSET = 0x0000000F;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR4_ADDSET_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR4_ADDSET_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BWTR4_ADDSET_2 = 0x00000004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BWTR4_ADDSET_3 = 0x00000008;


        /// <summary>
        ///  ADDHLD[3:0] bits (Address-hold phase duration) 
        /// </summary>
        public static UInt32 FSMC_BWTR4_ADDHLD = 0x000000F0;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR4_ADDHLD_0 = 0x00000010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR4_ADDHLD_1 = 0x00000020;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BWTR4_ADDHLD_2 = 0x00000040;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BWTR4_ADDHLD_3 = 0x00000080;


        /// <summary>
        ///  DATAST [3:0] bits (Data-phase duration) 
        /// </summary>
        public static UInt32 FSMC_BWTR4_DATAST = 0x0000FF00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR4_DATAST_0 = 0x00000100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR4_DATAST_1 = 0x00000200;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BWTR4_DATAST_2 = 0x00000400;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BWTR4_DATAST_3 = 0x00000800;


        /// <summary>
        ///  CLKDIV[3:0] bits (Clock divide ratio) 
        /// </summary>
        public static UInt32 FSMC_BWTR4_CLKDIV = 0x00F00000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR4_CLKDIV_0 = 0x00100000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR4_CLKDIV_1 = 0x00200000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BWTR4_CLKDIV_2 = 0x00400000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BWTR4_CLKDIV_3 = 0x00800000;


        /// <summary>
        ///  DATLA[3:0] bits (Data latency) 
        /// </summary>
        public static UInt32 FSMC_BWTR4_DATLAT = 0x0F000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR4_DATLAT_0 = 0x01000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR4_DATLAT_1 = 0x02000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_BWTR4_DATLAT_2 = 0x04000000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_BWTR4_DATLAT_3 = 0x08000000;


        /// <summary>
        ///  ACCMOD[1:0] bits (Access mode) 
        /// </summary>
        public static UInt32 FSMC_BWTR4_ACCMOD = 0x30000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_BWTR4_ACCMOD_0 = 0x10000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_BWTR4_ACCMOD_1 = 0x20000000;


        /******************  Bit definition for FSMC_PCR2 register  *******************/
        /// <summary>
        ///  Wait feature enable bit 
        /// </summary>
        public static UInt32 FSMC_PCR2_PWAITEN = 0x00000002;

        /// <summary>
        ///  PC Card/NAND Flash memory bank enable bit 
        /// </summary>
        public static UInt32 FSMC_PCR2_PBKEN = 0x00000004;

        /// <summary>
        ///  Memory type 
        /// </summary>
        public static UInt32 FSMC_PCR2_PTYP = 0x00000008;


        /// <summary>
        ///  PWID[1:0] bits (NAND Flash databus width) 
        /// </summary>
        public static UInt32 FSMC_PCR2_PWID = 0x00000030;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PCR2_PWID_0 = 0x00000010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PCR2_PWID_1 = 0x00000020;


        /// <summary>
        ///  ECC computation logic enable bit 
        /// </summary>
        public static UInt32 FSMC_PCR2_ECCEN = 0x00000040;


        /// <summary>
        ///  TCLR[3:0] bits (CLE to RE delay) 
        /// </summary>
        public static UInt32 FSMC_PCR2_TCLR = 0x00001E00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PCR2_TCLR_0 = 0x00000200;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PCR2_TCLR_1 = 0x00000400;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PCR2_TCLR_2 = 0x00000800;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PCR2_TCLR_3 = 0x00001000;


        /// <summary>
        ///  TAR[3:0] bits (ALE to RE delay) 
        /// </summary>
        public static UInt32 FSMC_PCR2_TAR = 0x0001E000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PCR2_TAR_0 = 0x00002000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PCR2_TAR_1 = 0x00004000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PCR2_TAR_2 = 0x00008000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PCR2_TAR_3 = 0x00010000;


        /// <summary>
        ///  ECCPS[1:0] bits (ECC page size) 
        /// </summary>
        public static UInt32 FSMC_PCR2_ECCPS = 0x000E0000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PCR2_ECCPS_0 = 0x00020000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PCR2_ECCPS_1 = 0x00040000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PCR2_ECCPS_2 = 0x00080000;


        /******************  Bit definition for FSMC_PCR3 register  *******************/
        /// <summary>
        ///  Wait feature enable bit 
        /// </summary>
        public static UInt32 FSMC_PCR3_PWAITEN = 0x00000002;

        /// <summary>
        ///  PC Card/NAND Flash memory bank enable bit 
        /// </summary>
        public static UInt32 FSMC_PCR3_PBKEN = 0x00000004;

        /// <summary>
        ///  Memory type 
        /// </summary>
        public static UInt32 FSMC_PCR3_PTYP = 0x00000008;


        /// <summary>
        ///  PWID[1:0] bits (NAND Flash databus width) 
        /// </summary>
        public static UInt32 FSMC_PCR3_PWID = 0x00000030;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PCR3_PWID_0 = 0x00000010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PCR3_PWID_1 = 0x00000020;


        /// <summary>
        ///  ECC computation logic enable bit 
        /// </summary>
        public static UInt32 FSMC_PCR3_ECCEN = 0x00000040;


        /// <summary>
        ///  TCLR[3:0] bits (CLE to RE delay) 
        /// </summary>
        public static UInt32 FSMC_PCR3_TCLR = 0x00001E00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PCR3_TCLR_0 = 0x00000200;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PCR3_TCLR_1 = 0x00000400;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PCR3_TCLR_2 = 0x00000800;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PCR3_TCLR_3 = 0x00001000;


        /// <summary>
        ///  TAR[3:0] bits (ALE to RE delay) 
        /// </summary>
        public static UInt32 FSMC_PCR3_TAR = 0x0001E000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PCR3_TAR_0 = 0x00002000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PCR3_TAR_1 = 0x00004000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PCR3_TAR_2 = 0x00008000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PCR3_TAR_3 = 0x00010000;


        /// <summary>
        ///  ECCPS[2:0] bits (ECC page size) 
        /// </summary>
        public static UInt32 FSMC_PCR3_ECCPS = 0x000E0000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PCR3_ECCPS_0 = 0x00020000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PCR3_ECCPS_1 = 0x00040000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PCR3_ECCPS_2 = 0x00080000;


        /******************  Bit definition for FSMC_PCR4 register  *******************/
        /// <summary>
        ///  Wait feature enable bit 
        /// </summary>
        public static UInt32 FSMC_PCR4_PWAITEN = 0x00000002;

        /// <summary>
        ///  PC Card/NAND Flash memory bank enable bit 
        /// </summary>
        public static UInt32 FSMC_PCR4_PBKEN = 0x00000004;

        /// <summary>
        ///  Memory type 
        /// </summary>
        public static UInt32 FSMC_PCR4_PTYP = 0x00000008;


        /// <summary>
        ///  PWID[1:0] bits (NAND Flash databus width) 
        /// </summary>
        public static UInt32 FSMC_PCR4_PWID = 0x00000030;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PCR4_PWID_0 = 0x00000010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PCR4_PWID_1 = 0x00000020;


        /// <summary>
        ///  ECC computation logic enable bit 
        /// </summary>
        public static UInt32 FSMC_PCR4_ECCEN = 0x00000040;


        /// <summary>
        ///  TCLR[3:0] bits (CLE to RE delay) 
        /// </summary>
        public static UInt32 FSMC_PCR4_TCLR = 0x00001E00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PCR4_TCLR_0 = 0x00000200;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PCR4_TCLR_1 = 0x00000400;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PCR4_TCLR_2 = 0x00000800;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PCR4_TCLR_3 = 0x00001000;


        /// <summary>
        ///  TAR[3:0] bits (ALE to RE delay) 
        /// </summary>
        public static UInt32 FSMC_PCR4_TAR = 0x0001E000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PCR4_TAR_0 = 0x00002000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PCR4_TAR_1 = 0x00004000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PCR4_TAR_2 = 0x00008000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PCR4_TAR_3 = 0x00010000;


        /// <summary>
        ///  ECCPS[2:0] bits (ECC page size) 
        /// </summary>
        public static UInt32 FSMC_PCR4_ECCPS = 0x000E0000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PCR4_ECCPS_0 = 0x00020000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PCR4_ECCPS_1 = 0x00040000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PCR4_ECCPS_2 = 0x00080000;


        /*******************  Bit definition for FSMC_SR2 register  *******************/
        /// <summary>
        ///  Interrupt Rising Edge status 
        /// </summary>
        public static byte FSMC_SR2_IRS = 0x01;

        /// <summary>
        ///  Interrupt Level status 
        /// </summary>
        public static byte FSMC_SR2_ILS = 0x02;

        /// <summary>
        ///  Interrupt Falling Edge status 
        /// </summary>
        public static byte FSMC_SR2_IFS = 0x04;

        /// <summary>
        ///  Interrupt Rising Edge detection Enable bit 
        /// </summary>
        public static byte FSMC_SR2_IREN = 0x08;

        /// <summary>
        ///  Interrupt Level detection Enable bit 
        /// </summary>
        public static byte FSMC_SR2_ILEN = 0x10;

        /// <summary>
        ///  Interrupt Falling Edge detection Enable bit 
        /// </summary>
        public static byte FSMC_SR2_IFEN = 0x20;

        /// <summary>
        ///  FIFO empty 
        /// </summary>
        public static byte FSMC_SR2_FEMPT = 0x40;


        /*******************  Bit definition for FSMC_SR3 register  *******************/
        /// <summary>
        ///  Interrupt Rising Edge status 
        /// </summary>
        public static byte FSMC_SR3_IRS = 0x01;

        /// <summary>
        ///  Interrupt Level status 
        /// </summary>
        public static byte FSMC_SR3_ILS = 0x02;

        /// <summary>
        ///  Interrupt Falling Edge status 
        /// </summary>
        public static byte FSMC_SR3_IFS = 0x04;

        /// <summary>
        ///  Interrupt Rising Edge detection Enable bit 
        /// </summary>
        public static byte FSMC_SR3_IREN = 0x08;

        /// <summary>
        ///  Interrupt Level detection Enable bit 
        /// </summary>
        public static byte FSMC_SR3_ILEN = 0x10;

        /// <summary>
        ///  Interrupt Falling Edge detection Enable bit 
        /// </summary>
        public static byte FSMC_SR3_IFEN = 0x20;

        /// <summary>
        ///  FIFO empty 
        /// </summary>
        public static byte FSMC_SR3_FEMPT = 0x40;


        /*******************  Bit definition for FSMC_SR4 register  *******************/
        /// <summary>
        ///  Interrupt Rising Edge status 
        /// </summary>
        public static byte FSMC_SR4_IRS = 0x01;

        /// <summary>
        ///  Interrupt Level status 
        /// </summary>
        public static byte FSMC_SR4_ILS = 0x02;

        /// <summary>
        ///  Interrupt Falling Edge status 
        /// </summary>
        public static byte FSMC_SR4_IFS = 0x04;

        /// <summary>
        ///  Interrupt Rising Edge detection Enable bit 
        /// </summary>
        public static byte FSMC_SR4_IREN = 0x08;

        /// <summary>
        ///  Interrupt Level detection Enable bit 
        /// </summary>
        public static byte FSMC_SR4_ILEN = 0x10;

        /// <summary>
        ///  Interrupt Falling Edge detection Enable bit 
        /// </summary>
        public static byte FSMC_SR4_IFEN = 0x20;

        /// <summary>
        ///  FIFO empty 
        /// </summary>
        public static byte FSMC_SR4_FEMPT = 0x40;


        /******************  Bit definition for FSMC_PMEM2 register  ******************/
        /// <summary>
        ///  MEMSET2[7:0] bits (Common memory 2 setup time) 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMSET2 = 0x000000FF;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMSET2_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMSET2_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMSET2_2 = 0x00000004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMSET2_3 = 0x00000008;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMSET2_4 = 0x00000010;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMSET2_5 = 0x00000020;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMSET2_6 = 0x00000040;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMSET2_7 = 0x00000080;


        /// <summary>
        ///  MEMWAIT2[7:0] bits (Common memory 2 wait time) 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMWAIT2 = 0x0000FF00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMWAIT2_0 = 0x00000100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMWAIT2_1 = 0x00000200;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMWAIT2_2 = 0x00000400;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMWAIT2_3 = 0x00000800;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMWAIT2_4 = 0x00001000;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMWAIT2_5 = 0x00002000;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMWAIT2_6 = 0x00004000;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMWAIT2_7 = 0x00008000;


        /// <summary>
        ///  MEMHOLD2[7:0] bits (Common memory 2 hold time) 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMHOLD2 = 0x00FF0000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMHOLD2_0 = 0x00010000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMHOLD2_1 = 0x00020000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMHOLD2_2 = 0x00040000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMHOLD2_3 = 0x00080000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMHOLD2_4 = 0x00100000;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMHOLD2_5 = 0x00200000;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMHOLD2_6 = 0x00400000;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMHOLD2_7 = 0x00800000;


        /// <summary>
        ///  MEMHIZ2[7:0] bits (Common memory 2 databus HiZ time) 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMHIZ2 = 0xFF000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMHIZ2_0 = 0x01000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMHIZ2_1 = 0x02000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMHIZ2_2 = 0x04000000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMHIZ2_3 = 0x08000000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMHIZ2_4 = 0x10000000;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMHIZ2_5 = 0x20000000;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMHIZ2_6 = 0x40000000;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PMEM2_MEMHIZ2_7 = 0x80000000;


        /******************  Bit definition for FSMC_PMEM3 register  ******************/
        /// <summary>
        ///  MEMSET3[7:0] bits (Common memory 3 setup time) 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMSET3 = 0x000000FF;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMSET3_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMSET3_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMSET3_2 = 0x00000004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMSET3_3 = 0x00000008;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMSET3_4 = 0x00000010;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMSET3_5 = 0x00000020;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMSET3_6 = 0x00000040;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMSET3_7 = 0x00000080;


        /// <summary>
        ///  MEMWAIT3[7:0] bits (Common memory 3 wait time) 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMWAIT3 = 0x0000FF00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMWAIT3_0 = 0x00000100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMWAIT3_1 = 0x00000200;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMWAIT3_2 = 0x00000400;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMWAIT3_3 = 0x00000800;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMWAIT3_4 = 0x00001000;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMWAIT3_5 = 0x00002000;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMWAIT3_6 = 0x00004000;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMWAIT3_7 = 0x00008000;


        /// <summary>
        ///  MEMHOLD3[7:0] bits (Common memory 3 hold time) 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMHOLD3 = 0x00FF0000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMHOLD3_0 = 0x00010000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMHOLD3_1 = 0x00020000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMHOLD3_2 = 0x00040000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMHOLD3_3 = 0x00080000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMHOLD3_4 = 0x00100000;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMHOLD3_5 = 0x00200000;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMHOLD3_6 = 0x00400000;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMHOLD3_7 = 0x00800000;


        /// <summary>
        ///  MEMHIZ3[7:0] bits (Common memory 3 databus HiZ time) 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMHIZ3 = 0xFF000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMHIZ3_0 = 0x01000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMHIZ3_1 = 0x02000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMHIZ3_2 = 0x04000000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMHIZ3_3 = 0x08000000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMHIZ3_4 = 0x10000000;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMHIZ3_5 = 0x20000000;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMHIZ3_6 = 0x40000000;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PMEM3_MEMHIZ3_7 = 0x80000000;


        /******************  Bit definition for FSMC_PMEM4 register  ******************/
        /// <summary>
        ///  MEMSET4[7:0] bits (Common memory 4 setup time) 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMSET4 = 0x000000FF;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMSET4_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMSET4_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMSET4_2 = 0x00000004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMSET4_3 = 0x00000008;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMSET4_4 = 0x00000010;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMSET4_5 = 0x00000020;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMSET4_6 = 0x00000040;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMSET4_7 = 0x00000080;


        /// <summary>
        ///  MEMWAIT4[7:0] bits (Common memory 4 wait time) 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMWAIT4 = 0x0000FF00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMWAIT4_0 = 0x00000100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMWAIT4_1 = 0x00000200;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMWAIT4_2 = 0x00000400;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMWAIT4_3 = 0x00000800;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMWAIT4_4 = 0x00001000;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMWAIT4_5 = 0x00002000;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMWAIT4_6 = 0x00004000;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMWAIT4_7 = 0x00008000;


        /// <summary>
        ///  MEMHOLD4[7:0] bits (Common memory 4 hold time) 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMHOLD4 = 0x00FF0000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMHOLD4_0 = 0x00010000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMHOLD4_1 = 0x00020000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMHOLD4_2 = 0x00040000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMHOLD4_3 = 0x00080000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMHOLD4_4 = 0x00100000;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMHOLD4_5 = 0x00200000;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMHOLD4_6 = 0x00400000;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMHOLD4_7 = 0x00800000;


        /// <summary>
        ///  MEMHIZ4[7:0] bits (Common memory 4 databus HiZ time) 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMHIZ4 = 0xFF000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMHIZ4_0 = 0x01000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMHIZ4_1 = 0x02000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMHIZ4_2 = 0x04000000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMHIZ4_3 = 0x08000000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMHIZ4_4 = 0x10000000;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMHIZ4_5 = 0x20000000;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMHIZ4_6 = 0x40000000;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PMEM4_MEMHIZ4_7 = 0x80000000;


        /******************  Bit definition for FSMC_PATT2 register  ******************/
        /// <summary>
        ///  ATTSET2[7:0] bits (Attribute memory 2 setup time) 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTSET2 = 0x000000FF;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTSET2_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTSET2_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTSET2_2 = 0x00000004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTSET2_3 = 0x00000008;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTSET2_4 = 0x00000010;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTSET2_5 = 0x00000020;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTSET2_6 = 0x00000040;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTSET2_7 = 0x00000080;


        /// <summary>
        ///  ATTWAIT2[7:0] bits (Attribute memory 2 wait time) 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTWAIT2 = 0x0000FF00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTWAIT2_0 = 0x00000100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTWAIT2_1 = 0x00000200;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTWAIT2_2 = 0x00000400;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTWAIT2_3 = 0x00000800;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTWAIT2_4 = 0x00001000;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTWAIT2_5 = 0x00002000;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTWAIT2_6 = 0x00004000;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTWAIT2_7 = 0x00008000;


        /// <summary>
        ///  ATTHOLD2[7:0] bits (Attribute memory 2 hold time) 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTHOLD2 = 0x00FF0000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTHOLD2_0 = 0x00010000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTHOLD2_1 = 0x00020000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTHOLD2_2 = 0x00040000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTHOLD2_3 = 0x00080000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTHOLD2_4 = 0x00100000;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTHOLD2_5 = 0x00200000;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTHOLD2_6 = 0x00400000;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTHOLD2_7 = 0x00800000;


        /// <summary>
        ///  ATTHIZ2[7:0] bits (Attribute memory 2 databus HiZ time) 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTHIZ2 = 0xFF000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTHIZ2_0 = 0x01000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTHIZ2_1 = 0x02000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTHIZ2_2 = 0x04000000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTHIZ2_3 = 0x08000000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTHIZ2_4 = 0x10000000;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTHIZ2_5 = 0x20000000;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTHIZ2_6 = 0x40000000;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PATT2_ATTHIZ2_7 = 0x80000000;


        /******************  Bit definition for FSMC_PATT3 register  ******************/
        /// <summary>
        ///  ATTSET3[7:0] bits (Attribute memory 3 setup time) 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTSET3 = 0x000000FF;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTSET3_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTSET3_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTSET3_2 = 0x00000004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTSET3_3 = 0x00000008;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTSET3_4 = 0x00000010;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTSET3_5 = 0x00000020;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTSET3_6 = 0x00000040;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTSET3_7 = 0x00000080;


        /// <summary>
        ///  ATTWAIT3[7:0] bits (Attribute memory 3 wait time) 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTWAIT3 = 0x0000FF00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTWAIT3_0 = 0x00000100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTWAIT3_1 = 0x00000200;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTWAIT3_2 = 0x00000400;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTWAIT3_3 = 0x00000800;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTWAIT3_4 = 0x00001000;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTWAIT3_5 = 0x00002000;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTWAIT3_6 = 0x00004000;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTWAIT3_7 = 0x00008000;


        /// <summary>
        ///  ATTHOLD3[7:0] bits (Attribute memory 3 hold time) 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTHOLD3 = 0x00FF0000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTHOLD3_0 = 0x00010000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTHOLD3_1 = 0x00020000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTHOLD3_2 = 0x00040000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTHOLD3_3 = 0x00080000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTHOLD3_4 = 0x00100000;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTHOLD3_5 = 0x00200000;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTHOLD3_6 = 0x00400000;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTHOLD3_7 = 0x00800000;


        /// <summary>
        ///  ATTHIZ3[7:0] bits (Attribute memory 3 databus HiZ time) 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTHIZ3 = 0xFF000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTHIZ3_0 = 0x01000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTHIZ3_1 = 0x02000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTHIZ3_2 = 0x04000000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTHIZ3_3 = 0x08000000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTHIZ3_4 = 0x10000000;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTHIZ3_5 = 0x20000000;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTHIZ3_6 = 0x40000000;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PATT3_ATTHIZ3_7 = 0x80000000;


        /******************  Bit definition for FSMC_PATT4 register  ******************/
        /// <summary>
        ///  ATTSET4[7:0] bits (Attribute memory 4 setup time) 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTSET4 = 0x000000FF;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTSET4_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTSET4_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTSET4_2 = 0x00000004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTSET4_3 = 0x00000008;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTSET4_4 = 0x00000010;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTSET4_5 = 0x00000020;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTSET4_6 = 0x00000040;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTSET4_7 = 0x00000080;


        /// <summary>
        ///  ATTWAIT4[7:0] bits (Attribute memory 4 wait time) 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTWAIT4 = 0x0000FF00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTWAIT4_0 = 0x00000100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTWAIT4_1 = 0x00000200;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTWAIT4_2 = 0x00000400;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTWAIT4_3 = 0x00000800;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTWAIT4_4 = 0x00001000;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTWAIT4_5 = 0x00002000;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTWAIT4_6 = 0x00004000;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTWAIT4_7 = 0x00008000;


        /// <summary>
        ///  ATTHOLD4[7:0] bits (Attribute memory 4 hold time) 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTHOLD4 = 0x00FF0000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTHOLD4_0 = 0x00010000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTHOLD4_1 = 0x00020000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTHOLD4_2 = 0x00040000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTHOLD4_3 = 0x00080000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTHOLD4_4 = 0x00100000;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTHOLD4_5 = 0x00200000;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTHOLD4_6 = 0x00400000;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTHOLD4_7 = 0x00800000;


        /// <summary>
        ///  ATTHIZ4[7:0] bits (Attribute memory 4 databus HiZ time) 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTHIZ4 = 0xFF000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTHIZ4_0 = 0x01000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTHIZ4_1 = 0x02000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTHIZ4_2 = 0x04000000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTHIZ4_3 = 0x08000000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTHIZ4_4 = 0x10000000;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTHIZ4_5 = 0x20000000;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTHIZ4_6 = 0x40000000;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PATT4_ATTHIZ4_7 = 0x80000000;


        /******************  Bit definition for FSMC_PIO4 register  *******************/
        /// <summary>
        ///  IOSET4[7:0] bits (I/O 4 setup time) 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOSET4 = 0x000000FF;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOSET4_0 = 0x00000001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOSET4_1 = 0x00000002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOSET4_2 = 0x00000004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOSET4_3 = 0x00000008;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOSET4_4 = 0x00000010;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOSET4_5 = 0x00000020;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOSET4_6 = 0x00000040;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOSET4_7 = 0x00000080;


        /// <summary>
        ///  IOWAIT4[7:0] bits (I/O 4 wait time) 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOWAIT4 = 0x0000FF00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOWAIT4_0 = 0x00000100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOWAIT4_1 = 0x00000200;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOWAIT4_2 = 0x00000400;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOWAIT4_3 = 0x00000800;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOWAIT4_4 = 0x00001000;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOWAIT4_5 = 0x00002000;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOWAIT4_6 = 0x00004000;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOWAIT4_7 = 0x00008000;


        /// <summary>
        ///  IOHOLD4[7:0] bits (I/O 4 hold time) 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOHOLD4 = 0x00FF0000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOHOLD4_0 = 0x00010000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOHOLD4_1 = 0x00020000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOHOLD4_2 = 0x00040000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOHOLD4_3 = 0x00080000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOHOLD4_4 = 0x00100000;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOHOLD4_5 = 0x00200000;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOHOLD4_6 = 0x00400000;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOHOLD4_7 = 0x00800000;


        /// <summary>
        ///  IOHIZ4[7:0] bits (I/O 4 databus HiZ time) 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOHIZ4 = 0xFF000000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOHIZ4_0 = 0x01000000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOHIZ4_1 = 0x02000000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOHIZ4_2 = 0x04000000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOHIZ4_3 = 0x08000000;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOHIZ4_4 = 0x10000000;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOHIZ4_5 = 0x20000000;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOHIZ4_6 = 0x40000000;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt32 FSMC_PIO4_IOHIZ4_7 = 0x80000000;


        /******************  Bit definition for FSMC_ECCR2 register  ******************/
        /// <summary>
        ///  ECC result 
        /// </summary>
        public static UInt32 FSMC_ECCR2_ECC2 = 0xFFFFFFFF;


        /******************  Bit definition for FSMC_ECCR3 register  ******************/
        /// <summary>
        ///  ECC result 
        /// </summary>
        public static UInt32 FSMC_ECCR3_ECC3 = 0xFFFFFFFF;


        /******************************************************************************/
        /*                                                                            */
        /*                            General Purpose I/O                             */
        /*                                                                            */
        /******************************************************************************/
        /******************  Bits definition for GPIO_MODER register  *****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER0 = 0x00000003;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER0_0 = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER0_1 = 0x00000002;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER1 = 0x0000000C;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER1_0 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER1_1 = 0x00000008;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER2 = 0x00000030;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER2_0 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER2_1 = 0x00000020;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER3 = 0x000000C0;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER3_0 = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER3_1 = 0x00000080;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER4 = 0x00000300;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER4_0 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER4_1 = 0x00000200;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER5 = 0x00000C00;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER5_0 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER5_1 = 0x00000800;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER6 = 0x00003000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER6_0 = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER6_1 = 0x00002000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER7 = 0x0000C000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER7_0 = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER7_1 = 0x00008000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER8 = 0x00030000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER8_0 = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER8_1 = 0x00020000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER9 = 0x000C0000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER9_0 = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER9_1 = 0x00080000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER10 = 0x00300000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER10_0 = 0x00100000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER10_1 = 0x00200000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER11 = 0x00C00000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER11_0 = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER11_1 = 0x00800000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER12 = 0x03000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER12_0 = 0x01000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER12_1 = 0x02000000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER13 = 0x0C000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER13_0 = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER13_1 = 0x08000000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER14 = 0x30000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER14_0 = 0x10000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER14_1 = 0x20000000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER15 = 0xC0000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER15_0 = 0x40000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_MODER_MODER15_1 = 0x80000000;


        /******************  Bits definition for GPIO_OTYPER register  ****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OTYPER_OT_0 = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OTYPER_OT_1 = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OTYPER_OT_2 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OTYPER_OT_3 = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OTYPER_OT_4 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OTYPER_OT_5 = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OTYPER_OT_6 = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OTYPER_OT_7 = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OTYPER_OT_8 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OTYPER_OT_9 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OTYPER_OT_10 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OTYPER_OT_11 = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OTYPER_OT_12 = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OTYPER_OT_13 = 0x00002000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OTYPER_OT_14 = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OTYPER_OT_15 = 0x00008000;


        /******************  Bits definition for GPIO_OSPEEDR register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR0 = 0x00000003;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR0_0 = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR0_1 = 0x00000002;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR1 = 0x0000000C;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR1_0 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR1_1 = 0x00000008;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR2 = 0x00000030;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR2_0 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR2_1 = 0x00000020;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR3 = 0x000000C0;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR3_0 = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR3_1 = 0x00000080;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR4 = 0x00000300;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR4_0 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR4_1 = 0x00000200;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR5 = 0x00000C00;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR5_0 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR5_1 = 0x00000800;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR6 = 0x00003000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR6_0 = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR6_1 = 0x00002000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR7 = 0x0000C000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR7_0 = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR7_1 = 0x00008000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR8 = 0x00030000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR8_0 = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR8_1 = 0x00020000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR9 = 0x000C0000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR9_0 = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR9_1 = 0x00080000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR10 = 0x00300000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR10_0 = 0x00100000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR10_1 = 0x00200000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR11 = 0x00C00000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR11_0 = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR11_1 = 0x00800000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR12 = 0x03000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR12_0 = 0x01000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR12_1 = 0x02000000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR13 = 0x0C000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR13_0 = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR13_1 = 0x08000000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR14 = 0x30000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR14_0 = 0x10000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR14_1 = 0x20000000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR15 = 0xC0000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR15_0 = 0x40000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_OSPEEDER_OSPEEDR15_1 = 0x80000000;


        /******************  Bits definition for GPIO_PUPDR register  *****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR0 = 0x00000003;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR0_0 = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR0_1 = 0x00000002;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR1 = 0x0000000C;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR1_0 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR1_1 = 0x00000008;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR2 = 0x00000030;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR2_0 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR2_1 = 0x00000020;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR3 = 0x000000C0;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR3_0 = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR3_1 = 0x00000080;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR4 = 0x00000300;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR4_0 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR4_1 = 0x00000200;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR5 = 0x00000C00;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR5_0 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR5_1 = 0x00000800;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR6 = 0x00003000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR6_0 = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR6_1 = 0x00002000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR7 = 0x0000C000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR7_0 = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR7_1 = 0x00008000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR8 = 0x00030000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR8_0 = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR8_1 = 0x00020000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR9 = 0x000C0000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR9_0 = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR9_1 = 0x00080000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR10 = 0x00300000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR10_0 = 0x00100000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR10_1 = 0x00200000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR11 = 0x00C00000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR11_0 = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR11_1 = 0x00800000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR12 = 0x03000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR12_0 = 0x01000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR12_1 = 0x02000000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR13 = 0x0C000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR13_0 = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR13_1 = 0x08000000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR14 = 0x30000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR14_0 = 0x10000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR14_1 = 0x20000000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR15 = 0xC0000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR15_0 = 0x40000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_PUPDR_PUPDR15_1 = 0x80000000;


        /******************  Bits definition for GPIO_IDR register  *******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_IDR_IDR_0 = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_IDR_IDR_1 = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_IDR_IDR_2 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_IDR_IDR_3 = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_IDR_IDR_4 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_IDR_IDR_5 = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_IDR_IDR_6 = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_IDR_IDR_7 = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_IDR_IDR_8 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_IDR_IDR_9 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_IDR_IDR_10 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_IDR_IDR_11 = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_IDR_IDR_12 = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_IDR_IDR_13 = 0x00002000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_IDR_IDR_14 = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_IDR_IDR_15 = 0x00008000;

        /* Old GPIO_IDR register bits definition, maintained for legacy purpose */
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_IDR_0 = GPIO_IDR_IDR_0;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_IDR_1 = GPIO_IDR_IDR_1;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_IDR_2 = GPIO_IDR_IDR_2;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_IDR_3 = GPIO_IDR_IDR_3;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_IDR_4 = GPIO_IDR_IDR_4;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_IDR_5 = GPIO_IDR_IDR_5;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_IDR_6 = GPIO_IDR_IDR_6;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_IDR_7 = GPIO_IDR_IDR_7;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_IDR_8 = GPIO_IDR_IDR_8;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_IDR_9 = GPIO_IDR_IDR_9;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_IDR_10 = GPIO_IDR_IDR_10;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_IDR_11 = GPIO_IDR_IDR_11;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_IDR_12 = GPIO_IDR_IDR_12;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_IDR_13 = GPIO_IDR_IDR_13;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_IDR_14 = GPIO_IDR_IDR_14;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_IDR_15 = GPIO_IDR_IDR_15;

        /******************  Bits definition for GPIO_ODR register  *******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_ODR_ODR_0 = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_ODR_ODR_1 = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_ODR_ODR_2 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_ODR_ODR_3 = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_ODR_ODR_4 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_ODR_ODR_5 = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_ODR_ODR_6 = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_ODR_ODR_7 = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_ODR_ODR_8 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_ODR_ODR_9 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_ODR_ODR_10 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_ODR_ODR_11 = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_ODR_ODR_12 = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_ODR_ODR_13 = 0x00002000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_ODR_ODR_14 = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_ODR_ODR_15 = 0x00008000;

        /* Old GPIO_ODR register bits definition, maintained for legacy purpose */
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_ODR_0 = GPIO_ODR_ODR_0;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_ODR_1 = GPIO_ODR_ODR_1;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_ODR_2 = GPIO_ODR_ODR_2;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_ODR_3 = GPIO_ODR_ODR_3;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_ODR_4 = GPIO_ODR_ODR_4;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_ODR_5 = GPIO_ODR_ODR_5;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_ODR_6 = GPIO_ODR_ODR_6;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_ODR_7 = GPIO_ODR_ODR_7;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_ODR_8 = GPIO_ODR_ODR_8;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_ODR_9 = GPIO_ODR_ODR_9;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_ODR_10 = GPIO_ODR_ODR_10;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_ODR_11 = GPIO_ODR_ODR_11;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_ODR_12 = GPIO_ODR_ODR_12;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_ODR_13 = GPIO_ODR_ODR_13;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_ODR_14 = GPIO_ODR_ODR_14;
        /// <summary>
        /// 
        /// </summary>
        public UInt32 GPIO_OTYPER_ODR_15 = GPIO_ODR_ODR_15;

        /******************  Bits definition for GPIO_BSRR register  ******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BS_0 = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BS_1 = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BS_2 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BS_3 = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BS_4 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BS_5 = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BS_6 = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BS_7 = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BS_8 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BS_9 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BS_10 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BS_11 = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BS_12 = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BS_13 = 0x00002000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BS_14 = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BS_15 = 0x00008000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BR_0 = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BR_1 = 0x00020000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BR_2 = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BR_3 = 0x00080000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BR_4 = 0x00100000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BR_5 = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BR_6 = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BR_7 = 0x00800000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BR_8 = 0x01000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BR_9 = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BR_10 = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BR_11 = 0x08000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BR_12 = 0x10000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BR_13 = 0x20000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BR_14 = 0x40000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 GPIO_BSRR_BR_15 = 0x80000000;


        /******************************************************************************/
        /*                                                                            */
        /*                                    HASH                                    */
        /*                                                                            */
        /******************************************************************************/
        /******************  Bits definition for HASH_CR register  ********************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_CR_INIT = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_CR_DMAE = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_CR_DATATYPE = 0x00000030;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_CR_DATATYPE_0 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_CR_DATATYPE_1 = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_CR_MODE = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_CR_ALGO = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_CR_NBW = 0x00000F00;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_CR_NBW_0 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_CR_NBW_1 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_CR_NBW_2 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_CR_NBW_3 = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_CR_DINNE = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_CR_LKEY = 0x00010000;


        /******************  Bits definition for HASH_STR register  *******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_STR_NBW = 0x0000001F;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_STR_NBW_0 = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_STR_NBW_1 = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_STR_NBW_2 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_STR_NBW_3 = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_STR_NBW_4 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_STR_DCAL = 0x00000100;


        /******************  Bits definition for HASH_IMR register  *******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_IMR_DINIM = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_IMR_DCIM = 0x00000002;


        /******************  Bits definition for HASH_SR register  ********************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_SR_DINIS = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_SR_DCIS = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_SR_DMAS = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 HASH_SR_BUSY = 0x00000008;


        /******************************************************************************/
        /*                                                                            */
        /*                      Inter-integrated Circuit Interface                    */
        /*                                                                            */
        /******************************************************************************/
        /*******************  Bit definition for I2C_CR1 register  ********************/
        /// <summary>
        ///  Peripheral Enable 
        /// </summary>
        public static UInt16 I2C_CR1_PE = 0x0001;

        /// <summary>
        ///  SMBus Mode 
        /// </summary>
        public static UInt16 I2C_CR1_SMBUS = 0x0002;

        /// <summary>
        ///  SMBus Type 
        /// </summary>
        public static UInt16 I2C_CR1_SMBTYPE = 0x0008;

        /// <summary>
        ///  ARP Enable 
        /// </summary>
        public static UInt16 I2C_CR1_ENARP = 0x0010;

        /// <summary>
        ///  PEC Enable 
        /// </summary>
        public static UInt16 I2C_CR1_ENPEC = 0x0020;

        /// <summary>
        ///  General Call Enable 
        /// </summary>
        public static UInt16 I2C_CR1_ENGC = 0x0040;

        /// <summary>
        ///  Clock Stretching Disable (Slave mode) 
        /// </summary>
        public static UInt16 I2C_CR1_NOSTRETCH = 0x0080;

        /// <summary>
        ///  Start Generation 
        /// </summary>
        public static UInt16 I2C_CR1_START = 0x0100;

        /// <summary>
        ///  Stop Generation 
        /// </summary>
        public static UInt16 I2C_CR1_STOP = 0x0200;

        /// <summary>
        ///  Acknowledge Enable 
        /// </summary>
        public static UInt16 I2C_CR1_ACK = 0x0400;

        /// <summary>
        ///  Acknowledge/PEC Position (for data reception) 
        /// </summary>
        public static UInt16 I2C_CR1_POS = 0x0800;

        /// <summary>
        ///  Packet Error Checking 
        /// </summary>
        public static UInt16 I2C_CR1_PEC = 0x1000;

        /// <summary>
        ///  SMBus Alert 
        /// </summary>
        public static UInt16 I2C_CR1_ALERT = 0x2000;

        /// <summary>
        ///  Software Reset 
        /// </summary>
        public static UInt16 I2C_CR1_SWRST = 0x8000;


        /*******************  Bit definition for I2C_CR2 register  ********************/
        /// <summary>
        ///  FREQ[5:0] bits (Peripheral Clock Frequency) 
        /// </summary>
        public static UInt16 I2C_CR2_FREQ = 0x003F;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 I2C_CR2_FREQ_0 = 0x0001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 I2C_CR2_FREQ_1 = 0x0002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt16 I2C_CR2_FREQ_2 = 0x0004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt16 I2C_CR2_FREQ_3 = 0x0008;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt16 I2C_CR2_FREQ_4 = 0x0010;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt16 I2C_CR2_FREQ_5 = 0x0020;


        /// <summary>
        ///  Error Interrupt Enable 
        /// </summary>
        public static UInt16 I2C_CR2_ITERREN = 0x0100;

        /// <summary>
        ///  Event Interrupt Enable 
        /// </summary>
        public static UInt16 I2C_CR2_ITEVTEN = 0x0200;

        /// <summary>
        ///  Buffer Interrupt Enable 
        /// </summary>
        public static UInt16 I2C_CR2_ITBUFEN = 0x0400;

        /// <summary>
        ///  DMA Requests Enable 
        /// </summary>
        public static UInt16 I2C_CR2_DMAEN = 0x0800;

        /// <summary>
        ///  DMA Last Transfer 
        /// </summary>
        public static UInt16 I2C_CR2_LAST = 0x1000;


        /*******************  Bit definition for I2C_OAR1 register  *******************/
        /// <summary>
        ///  Interface Address 
        /// </summary>
        public static UInt16 I2C_OAR1_ADD1_7 = 0x00FE;

        /// <summary>
        ///  Interface Address 
        /// </summary>
        public static UInt16 I2C_OAR1_ADD8_9 = 0x0300;


        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 I2C_OAR1_ADD0 = 0x0001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 I2C_OAR1_ADD1 = 0x0002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt16 I2C_OAR1_ADD2 = 0x0004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt16 I2C_OAR1_ADD3 = 0x0008;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt16 I2C_OAR1_ADD4 = 0x0010;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt16 I2C_OAR1_ADD5 = 0x0020;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt16 I2C_OAR1_ADD6 = 0x0040;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt16 I2C_OAR1_ADD7 = 0x0080;

        /// <summary>
        ///  Bit 8 
        /// </summary>
        public static UInt16 I2C_OAR1_ADD8 = 0x0100;

        /// <summary>
        ///  Bit 9 
        /// </summary>
        public static UInt16 I2C_OAR1_ADD9 = 0x0200;


        /// <summary>
        ///  Addressing Mode (Slave mode) 
        /// </summary>
        public static UInt16 I2C_OAR1_ADDMODE = 0x8000;


        /*******************  Bit definition for I2C_OAR2 register  *******************/
        /// <summary>
        ///  Dual addressing mode enable 
        /// </summary>
        public static byte I2C_OAR2_ENDUAL = 0x01;

        /// <summary>
        ///  Interface address 
        /// </summary>
        public static byte I2C_OAR2_ADD2 = 0xFE;


        /********************  Bit definition for I2C_DR register  ********************/
        /// <summary>
        ///  8-bit Data Register 
        /// </summary>
        public static byte I2C_DR_DR = 0xFF;


        /*******************  Bit definition for I2C_SR1 register  ********************/
        /// <summary>
        ///  Start Bit (Master mode) 
        /// </summary>
        public static UInt16 I2C_SR1_SB = 0x0001;

        /// <summary>
        ///  Address sent (master mode)/matched (slave mode) 
        /// </summary>
        public static UInt16 I2C_SR1_ADDR = 0x0002;

        /// <summary>
        ///  Byte Transfer Finished 
        /// </summary>
        public static UInt16 I2C_SR1_BTF = 0x0004;

        /// <summary>
        ///  10-bit header sent (Master mode) 
        /// </summary>
        public static UInt16 I2C_SR1_ADD10 = 0x0008;

        /// <summary>
        ///  Stop detection (Slave mode) 
        /// </summary>
        public static UInt16 I2C_SR1_STOPF = 0x0010;

        /// <summary>
        ///  Data Register not Empty (receivers) 
        /// </summary>
        public static UInt16 I2C_SR1_RXNE = 0x0040;

        /// <summary>
        ///  Data Register Empty (transmitters) 
        /// </summary>
        public static UInt16 I2C_SR1_TXE = 0x0080;

        /// <summary>
        ///  Bus Error 
        /// </summary>
        public static UInt16 I2C_SR1_BERR = 0x0100;

        /// <summary>
        ///  Arbitration Lost (master mode) 
        /// </summary>
        public static UInt16 I2C_SR1_ARLO = 0x0200;

        /// <summary>
        ///  Acknowledge Failure 
        /// </summary>
        public static UInt16 I2C_SR1_AF = 0x0400;

        /// <summary>
        ///  Overrun/Underrun 
        /// </summary>
        public static UInt16 I2C_SR1_OVR = 0x0800;

        /// <summary>
        ///  PEC Error in reception 
        /// </summary>
        public static UInt16 I2C_SR1_PECERR = 0x1000;

        /// <summary>
        ///  Timeout or Tlow Error 
        /// </summary>
        public static UInt16 I2C_SR1_TIMEOUT = 0x4000;

        /// <summary>
        ///  SMBus Alert 
        /// </summary>
        public static UInt16 I2C_SR1_SMBALERT = 0x8000;


        /*******************  Bit definition for I2C_SR2 register  ********************/
        /// <summary>
        ///  Master/Slave 
        /// </summary>
        public static UInt16 I2C_SR2_MSL = 0x0001;

        /// <summary>
        ///  Bus Busy 
        /// </summary>
        public static UInt16 I2C_SR2_BUSY = 0x0002;

        /// <summary>
        ///  Transmitter/Receiver 
        /// </summary>
        public static UInt16 I2C_SR2_TRA = 0x0004;

        /// <summary>
        ///  General Call Address (Slave mode) 
        /// </summary>
        public static UInt16 I2C_SR2_GENCALL = 0x0010;

        /// <summary>
        ///  SMBus Device Default Address (Slave mode) 
        /// </summary>
        public static UInt16 I2C_SR2_SMBDEFAULT = 0x0020;

        /// <summary>
        ///  SMBus Host Header (Slave mode) 
        /// </summary>
        public static UInt16 I2C_SR2_SMBHOST = 0x0040;

        /// <summary>
        ///  Dual Flag (Slave mode) 
        /// </summary>
        public static UInt16 I2C_SR2_DUALF = 0x0080;

        /// <summary>
        ///  Packet Error Checking Register 
        /// </summary>
        public static UInt16 I2C_SR2_PEC = 0xFF00;


        /*******************  Bit definition for I2C_CCR register  ********************/
        /// <summary>
        ///  Clock Control Register in Fast/Standard mode (Master mode) 
        /// </summary>
        public static UInt16 I2C_CCR_CCR = 0x0FFF;

        /// <summary>
        ///  Fast Mode Duty Cycle 
        /// </summary>
        public static UInt16 I2C_CCR_DUTY = 0x4000;

        /// <summary>
        ///  I2C Master Mode Selection 
        /// </summary>
        public static UInt16 I2C_CCR_FS = 0x8000;


        /******************  Bit definition for I2C_TRISE register  *******************/
        /// <summary>
        ///  Maximum Rise Time in Fast/Standard mode (Master mode) 
        /// </summary>
        public static byte I2C_TRISE_TRISE = 0x3F;


        /******************************************************************************/
        /*                                                                            */
        /*                           Independent WATCHDOG                             */
        /*                                                                            */
        /******************************************************************************/
        /*******************  Bit definition for IWDG_KR register  ********************/
        /// <summary>
        ///  Key value (write only, read 0000h) 
        /// </summary>
        public static UInt16 IWDG_KR_KEY = 0xFFFF;


        /*******************  Bit definition for IWDG_PR register  ********************/
        /// <summary>
        ///  PR[2:0] (Prescaler divider) 
        /// </summary>
        public static byte IWDG_PR_PR = 0x07;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static byte IWDG_PR_PR_0 = 0x01;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static byte IWDG_PR_PR_1 = 0x02;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static byte IWDG_PR_PR_2 = 0x04;


        /*******************  Bit definition for IWDG_RLR register  *******************/
        /// <summary>
        ///  Watchdog counter reload value 
        /// </summary>
        public static UInt16 IWDG_RLR_RL = 0x0FFF;


        /*******************  Bit definition for IWDG_SR register  ********************/
        /// <summary>
        ///  Watchdog prescaler value update 
        /// </summary>
        public static byte IWDG_SR_PVU = 0x01;

        /// <summary>
        ///  Watchdog counter reload value update 
        /// </summary>
        public static byte IWDG_SR_RVU = 0x02;


        /******************************************************************************/
        /*                                                                            */
        /*                             Power Control                                  */
        /*                                                                            */
        /******************************************************************************/
        /********************  Bit definition for PWR_CR register  ********************/
        /// <summary>
        ///   Low-Power Deepsleep 
        /// </summary>
        public static UInt16 PWR_CR_LPDS = 0x0001;

        /// <summary>
        ///   Power Down Deepsleep 
        /// </summary>
        public static UInt16 PWR_CR_PDDS = 0x0002;

        /// <summary>
        ///   Clear Wakeup Flag 
        /// </summary>
        public static UInt16 PWR_CR_CWUF = 0x0004;

        /// <summary>
        ///   Clear Standby Flag 
        /// </summary>
        public static UInt16 PWR_CR_CSBF = 0x0008;

        /// <summary>
        ///   Power Voltage Detector Enable 
        /// </summary>
        public static UInt16 PWR_CR_PVDE = 0x0010;


        /// <summary>
        ///   PLS[2:0] bits (PVD Level Selection) 
        /// </summary>
        public static UInt16 PWR_CR_PLS = 0x00E0;

        /// <summary>
        ///   Bit 0 
        /// </summary>
        public static UInt16 PWR_CR_PLS_0 = 0x0020;

        /// <summary>
        ///   Bit 1 
        /// </summary>
        public static UInt16 PWR_CR_PLS_1 = 0x0040;

        /// <summary>
        ///   Bit 2 
        /// </summary>
        public static UInt16 PWR_CR_PLS_2 = 0x0080;



        /*!< PVD level configuration */
        /// <summary>
        ///   PVD level 0 
        /// </summary>
        public static UInt16 PWR_CR_PLS_LEV0 = 0x0000;

        /// <summary>
        ///   PVD level 1 
        /// </summary>
        public static UInt16 PWR_CR_PLS_LEV1 = 0x0020;

        /// <summary>
        ///   PVD level 2 
        /// </summary>
        public static UInt16 PWR_CR_PLS_LEV2 = 0x0040;

        /// <summary>
        ///   PVD level 3 
        /// </summary>
        public static UInt16 PWR_CR_PLS_LEV3 = 0x0060;

        /// <summary>
        ///   PVD level 4 
        /// </summary>
        public static UInt16 PWR_CR_PLS_LEV4 = 0x0080;

        /// <summary>
        ///   PVD level 5 
        /// </summary>
        public static UInt16 PWR_CR_PLS_LEV5 = 0x00A0;

        /// <summary>
        ///   PVD level 6 
        /// </summary>
        public static UInt16 PWR_CR_PLS_LEV6 = 0x00C0;

        /// <summary>
        ///   PVD level 7 
        /// </summary>
        public static UInt16 PWR_CR_PLS_LEV7 = 0x00E0;


        /// <summary>
        ///   Disable Backup Domain write protection 
        /// </summary>
        public static UInt16 PWR_CR_DBP = 0x0100;

        /// <summary>
        ///   Flash power down in Stop mode 
        /// </summary>
        public static UInt16 PWR_CR_FPDS = 0x0200;

        /// <summary>
        ///   Regulator voltage scaling output selection 
        /// </summary>
        public static UInt16 PWR_CR_VOS = 0x4000;

        /* Legacy define */
        /// <summary>
        /// 
        /// </summary>
        public UInt32 PWR_CR_PMODE = PWR_CR_VOS;

        /*******************  Bit definition for PWR_CSR register  ********************/
        /// <summary>
        ///   Wakeup Flag 
        /// </summary>
        public static UInt16 PWR_CSR_WUF = 0x0001;

        /// <summary>
        ///   Standby Flag 
        /// </summary>
        public static UInt16 PWR_CSR_SBF = 0x0002;

        /// <summary>
        ///   PVD Output 
        /// </summary>
        public static UInt16 PWR_CSR_PVDO = 0x0004;

        /// <summary>
        ///   Backup regulator ready 
        /// </summary>
        public static UInt16 PWR_CSR_BRR = 0x0008;

        /// <summary>
        ///   Enable WKUP pin 
        /// </summary>
        public static UInt16 PWR_CSR_EWUP = 0x0100;

        /// <summary>
        ///   Backup regulator enable 
        /// </summary>
        public static UInt16 PWR_CSR_BRE = 0x0200;

        /// <summary>
        ///   Regulator voltage scaling output selection ready 
        /// </summary>
        public static UInt16 PWR_CSR_VOSRDY = 0x4000;

        /* Legacy define */
        /// <summary>
        /// 
        /// </summary>
        public UInt32 PWR_CSR_REGRDY = PWR_CSR_VOSRDY;

        /******************************************************************************/
        /*                                                                            */
        /*                         Reset and Clock Control                            */
        /*                                                                            */
        /******************************************************************************/
        /********************  Bit definition for RCC_CR register  ********************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CR_HSION = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CR_HSIRDY = 0x00000002;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CR_HSITRIM = 0x000000F8;

        /// <summary>
        /// Bit 0 
        /// </summary>
        public static UInt32 RCC_CR_HSITRIM_0 = 0x00000008;

        /// <summary>
        /// Bit 1 
        /// </summary>
        public static UInt32 RCC_CR_HSITRIM_1 = 0x00000010;

        /// <summary>
        /// Bit 2 
        /// </summary>
        public static UInt32 RCC_CR_HSITRIM_2 = 0x00000020;

        /// <summary>
        /// Bit 3 
        /// </summary>
        public static UInt32 RCC_CR_HSITRIM_3 = 0x00000040;

        /// <summary>
        /// Bit 4 
        /// </summary>
        public static UInt32 RCC_CR_HSITRIM_4 = 0x00000080;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CR_HSICAL = 0x0000FF00;

        /// <summary>
        /// Bit 0 
        /// </summary>
        public static UInt32 RCC_CR_HSICAL_0 = 0x00000100;

        /// <summary>
        /// Bit 1 
        /// </summary>
        public static UInt32 RCC_CR_HSICAL_1 = 0x00000200;

        /// <summary>
        /// Bit 2 
        /// </summary>
        public static UInt32 RCC_CR_HSICAL_2 = 0x00000400;

        /// <summary>
        /// Bit 3 
        /// </summary>
        public static UInt32 RCC_CR_HSICAL_3 = 0x00000800;

        /// <summary>
        /// Bit 4 
        /// </summary>
        public static UInt32 RCC_CR_HSICAL_4 = 0x00001000;

        /// <summary>
        /// Bit 5 
        /// </summary>
        public static UInt32 RCC_CR_HSICAL_5 = 0x00002000;

        /// <summary>
        /// Bit 6 
        /// </summary>
        public static UInt32 RCC_CR_HSICAL_6 = 0x00004000;

        /// <summary>
        /// Bit 7 
        /// </summary>
        public static UInt32 RCC_CR_HSICAL_7 = 0x00008000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CR_HSEON = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CR_HSERDY = 0x00020000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CR_HSEBYP = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CR_CSSON = 0x00080000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CR_PLLON = 0x01000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CR_PLLRDY = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CR_PLLI2SON = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CR_PLLI2SRDY = 0x08000000;


        /********************  Bit definition for RCC_PLLCFGR register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLM = 0x0000003F;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLM_0 = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLM_1 = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLM_2 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLM_3 = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLM_4 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLM_5 = 0x00000020;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLN = 0x00007FC0;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLN_0 = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLN_1 = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLN_2 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLN_3 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLN_4 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLN_5 = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLN_6 = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLN_7 = 0x00002000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLN_8 = 0x00004000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLP = 0x00030000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLP_0 = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLP_1 = 0x00020000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLSRC = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLSRC_HSE = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLSRC_HSI = 0x00000000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLQ = 0x0F000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLQ_0 = 0x01000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLQ_1 = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLQ_2 = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLCFGR_PLLQ_3 = 0x08000000;


        /********************  Bit definition for RCC_CFGR register  ******************/
        /*!< SW configuration */
        /// <summary>
        ///   SW[1:0] bits (System clock Switch) 
        /// </summary>
        public static UInt32 RCC_CFGR_SW = 0x00000003;

        /// <summary>
        ///   Bit 0 
        /// </summary>
        public static UInt32 RCC_CFGR_SW_0 = 0x00000001;

        /// <summary>
        ///   Bit 1 
        /// </summary>
        public static UInt32 RCC_CFGR_SW_1 = 0x00000002;


        /// <summary>
        ///   HSI selected as system clock 
        /// </summary>
        public static UInt32 RCC_CFGR_SW_HSI = 0x00000000;

        /// <summary>
        ///   HSE selected as system clock 
        /// </summary>
        public static UInt32 RCC_CFGR_SW_HSE = 0x00000001;

        /// <summary>
        ///   PLL selected as system clock 
        /// </summary>
        public static UInt32 RCC_CFGR_SW_PLL = 0x00000002;


        /*!< SWS configuration */
        /// <summary>
        ///   SWS[1:0] bits (System Clock Switch Status) 
        /// </summary>
        public static UInt32 RCC_CFGR_SWS = 0x0000000C;

        /// <summary>
        ///   Bit 0 
        /// </summary>
        public static UInt32 RCC_CFGR_SWS_0 = 0x00000004;

        /// <summary>
        ///   Bit 1 
        /// </summary>
        public static UInt32 RCC_CFGR_SWS_1 = 0x00000008;


        /// <summary>
        ///   HSI oscillator used as system clock 
        /// </summary>
        public static UInt32 RCC_CFGR_SWS_HSI = 0x00000000;

        /// <summary>
        ///   HSE oscillator used as system clock 
        /// </summary>
        public static UInt32 RCC_CFGR_SWS_HSE = 0x00000004;

        /// <summary>
        ///   PLL used as system clock 
        /// </summary>
        public static UInt32 RCC_CFGR_SWS_PLL = 0x00000008;


        /*!< HPRE configuration */
        /// <summary>
        ///   HPRE[3:0] bits (AHB prescaler) 
        /// </summary>
        public static UInt32 RCC_CFGR_HPRE = 0x000000F0;

        /// <summary>
        ///   Bit 0 
        /// </summary>
        public static UInt32 RCC_CFGR_HPRE_0 = 0x00000010;

        /// <summary>
        ///   Bit 1 
        /// </summary>
        public static UInt32 RCC_CFGR_HPRE_1 = 0x00000020;

        /// <summary>
        ///   Bit 2 
        /// </summary>
        public static UInt32 RCC_CFGR_HPRE_2 = 0x00000040;

        /// <summary>
        ///   Bit 3 
        /// </summary>
        public static UInt32 RCC_CFGR_HPRE_3 = 0x00000080;


        /// <summary>
        ///   SYSCLK not divided 
        /// </summary>
        public static UInt32 RCC_CFGR_HPRE_DIV1 = 0x00000000;

        /// <summary>
        ///   SYSCLK divided by 2 
        /// </summary>
        public static UInt32 RCC_CFGR_HPRE_DIV2 = 0x00000080;

        /// <summary>
        ///   SYSCLK divided by 4 
        /// </summary>
        public static UInt32 RCC_CFGR_HPRE_DIV4 = 0x00000090;

        /// <summary>
        ///   SYSCLK divided by 8 
        /// </summary>
        public static UInt32 RCC_CFGR_HPRE_DIV8 = 0x000000A0;

        /// <summary>
        ///   SYSCLK divided by 16 
        /// </summary>
        public static UInt32 RCC_CFGR_HPRE_DIV16 = 0x000000B0;

        /// <summary>
        ///   SYSCLK divided by 64 
        /// </summary>
        public static UInt32 RCC_CFGR_HPRE_DIV64 = 0x000000C0;

        /// <summary>
        ///   SYSCLK divided by 128 
        /// </summary>
        public static UInt32 RCC_CFGR_HPRE_DIV128 = 0x000000D0;

        /// <summary>
        ///   SYSCLK divided by 256 
        /// </summary>
        public static UInt32 RCC_CFGR_HPRE_DIV256 = 0x000000E0;

        /// <summary>
        ///   SYSCLK divided by 512 
        /// </summary>
        public static UInt32 RCC_CFGR_HPRE_DIV512 = 0x000000F0;


        /*!< PPRE1 configuration */
        /// <summary>
        ///   PRE1[2:0] bits (APB1 prescaler) 
        /// </summary>
        public static UInt32 RCC_CFGR_PPRE1 = 0x00001C00;

        /// <summary>
        ///   Bit 0 
        /// </summary>
        public static UInt32 RCC_CFGR_PPRE1_0 = 0x00000400;

        /// <summary>
        ///   Bit 1 
        /// </summary>
        public static UInt32 RCC_CFGR_PPRE1_1 = 0x00000800;

        /// <summary>
        ///   Bit 2 
        /// </summary>
        public static UInt32 RCC_CFGR_PPRE1_2 = 0x00001000;


        /// <summary>
        ///   HCLK not divided 
        /// </summary>
        public static UInt32 RCC_CFGR_PPRE1_DIV1 = 0x00000000;

        /// <summary>
        ///   HCLK divided by 2 
        /// </summary>
        public static UInt32 RCC_CFGR_PPRE1_DIV2 = 0x00001000;

        /// <summary>
        ///   HCLK divided by 4 
        /// </summary>
        public static UInt32 RCC_CFGR_PPRE1_DIV4 = 0x00001400;

        /// <summary>
        ///   HCLK divided by 8 
        /// </summary>
        public static UInt32 RCC_CFGR_PPRE1_DIV8 = 0x00001800;

        /// <summary>
        ///   HCLK divided by 16 
        /// </summary>
        public static UInt32 RCC_CFGR_PPRE1_DIV16 = 0x00001C00;


        /*!< PPRE2 configuration */
        /// <summary>
        ///   PRE2[2:0] bits (APB2 prescaler) 
        /// </summary>
        public static UInt32 RCC_CFGR_PPRE2 = 0x0000E000;

        /// <summary>
        ///   Bit 0 
        /// </summary>
        public static UInt32 RCC_CFGR_PPRE2_0 = 0x00002000;

        /// <summary>
        ///   Bit 1 
        /// </summary>
        public static UInt32 RCC_CFGR_PPRE2_1 = 0x00004000;

        /// <summary>
        ///   Bit 2 
        /// </summary>
        public static UInt32 RCC_CFGR_PPRE2_2 = 0x00008000;


        /// <summary>
        ///   HCLK not divided 
        /// </summary>
        public static UInt32 RCC_CFGR_PPRE2_DIV1 = 0x00000000;

        /// <summary>
        ///   HCLK divided by 2 
        /// </summary>
        public static UInt32 RCC_CFGR_PPRE2_DIV2 = 0x00008000;

        /// <summary>
        ///   HCLK divided by 4 
        /// </summary>
        public static UInt32 RCC_CFGR_PPRE2_DIV4 = 0x0000A000;

        /// <summary>
        ///   HCLK divided by 8 
        /// </summary>
        public static UInt32 RCC_CFGR_PPRE2_DIV8 = 0x0000C000;

        /// <summary>
        ///   HCLK divided by 16 
        /// </summary>
        public static UInt32 RCC_CFGR_PPRE2_DIV16 = 0x0000E000;


        /*!< RTCPRE configuration */
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CFGR_RTCPRE = 0x001F0000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CFGR_RTCPRE_0 = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CFGR_RTCPRE_1 = 0x00020000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CFGR_RTCPRE_2 = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CFGR_RTCPRE_3 = 0x00080000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CFGR_RTCPRE_4 = 0x00100000;


        /*!< MCO1 configuration */
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CFGR_MCO1 = 0x00600000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CFGR_MCO1_0 = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CFGR_MCO1_1 = 0x00400000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CFGR_I2SSRC = 0x00800000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CFGR_MCO1PRE = 0x07000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CFGR_MCO1PRE_0 = 0x01000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CFGR_MCO1PRE_1 = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CFGR_MCO1PRE_2 = 0x04000000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CFGR_MCO2PRE = 0x38000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CFGR_MCO2PRE_0 = 0x08000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CFGR_MCO2PRE_1 = 0x10000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CFGR_MCO2PRE_2 = 0x20000000;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CFGR_MCO2 = 0xC0000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CFGR_MCO2_0 = 0x40000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CFGR_MCO2_1 = 0x80000000;


        /********************  Bit definition for RCC_CIR register  *******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CIR_LSIRDYF = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CIR_LSERDYF = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CIR_HSIRDYF = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CIR_HSERDYF = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CIR_PLLRDYF = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CIR_PLLI2SRDYF = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CIR_CSSF = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CIR_LSIRDYIE = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CIR_LSERDYIE = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CIR_HSIRDYIE = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CIR_HSERDYIE = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CIR_PLLRDYIE = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CIR_PLLI2SRDYIE = 0x00002000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CIR_LSIRDYC = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CIR_LSERDYC = 0x00020000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CIR_HSIRDYC = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CIR_HSERDYC = 0x00080000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CIR_PLLRDYC = 0x00100000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CIR_PLLI2SRDYC = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CIR_CSSC = 0x00800000;


        /********************  Bit definition for RCC_AHB1RSTR register  **************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1RSTR_GPIOARST = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1RSTR_GPIOBRST = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1RSTR_GPIOCRST = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1RSTR_GPIODRST = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1RSTR_GPIOERST = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1RSTR_GPIOFRST = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1RSTR_GPIOGRST = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1RSTR_GPIOHRST = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1RSTR_GPIOIRST = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1RSTR_CRCRST = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1RSTR_DMA1RST = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1RSTR_DMA2RST = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1RSTR_ETHMACRST = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1RSTR_OTGHRST = 0x10000000;


        /********************  Bit definition for RCC_AHB2RSTR register  **************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB2RSTR_DCMIRST = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB2RSTR_CRYPRST = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB2RSTR_HSAHRST = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB2RSTR_RNGRST = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB2RSTR_OTGFSRST = 0x00000080;


        /********************  Bit definition for RCC_AHB3RSTR register  **************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB3RSTR_FSMCRST = 0x00000001;


        /********************  Bit definition for RCC_APB1RSTR register  **************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_TIM2RST = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_TIM3RST = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_TIM4RST = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_TIM5RST = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_TIM6RST = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_TIM7RST = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_TIM12RST = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_TIM13RST = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_TIM14RST = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_WWDGEN = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_SPI2RST = 0x00008000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_SPI3RST = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_USART2RST = 0x00020000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_USART3RST = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_UART4RST = 0x00080000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_UART5RST = 0x00100000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_I2C1RST = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_I2C2RST = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_I2C3RST = 0x00800000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_CAN1RST = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_CAN2RST = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_PWRRST = 0x10000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1RSTR_DACRST = 0x20000000;


        /********************  Bit definition for RCC_APB2RSTR register  **************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2RSTR_TIM1RST = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2RSTR_TIM8RST = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2RSTR_USART1RST = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2RSTR_USART6RST = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2RSTR_ADCRST = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2RSTR_SDIORST = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2RSTR_SPI1RST = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2RSTR_SYSCFGRST = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2RSTR_TIM9RST = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2RSTR_TIM10RST = 0x00020000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2RSTR_TIM11RST = 0x00040000;

        /* Old SPI1RST bit definition, maintained for legacy purpose */
        /// <summary>
        /// 
        /// </summary>
        public UInt32 RCC_APB2RSTR_SPI1 = RCC_APB2RSTR_SPI1RST;

        /********************  Bit definition for RCC_AHB1ENR register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1ENR_GPIOAEN = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1ENR_GPIOBEN = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1ENR_GPIOCEN = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1ENR_GPIODEN = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1ENR_GPIOEEN = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1ENR_GPIOFEN = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1ENR_GPIOGEN = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1ENR_GPIOHEN = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1ENR_GPIOIEN = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1ENR_CRCEN = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1ENR_BKPSRAMEN = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1ENR_CCMDATARAMEN = 0x00100000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1ENR_DMA1EN = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1ENR_DMA2EN = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1ENR_ETHMACEN = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1ENR_ETHMACTXEN = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1ENR_ETHMACRXEN = 0x08000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1ENR_ETHMACPTPEN = 0x10000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1ENR_OTGHSEN = 0x20000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1ENR_OTGHSULPIEN = 0x40000000;


        /********************  Bit definition for RCC_AHB2ENR register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB2ENR_DCMIEN = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB2ENR_CRYPEN = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB2ENR_HASHEN = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB2ENR_RNGEN = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB2ENR_OTGFSEN = 0x00000080;


        /********************  Bit definition for RCC_AHB3ENR register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB3ENR_FSMCEN = 0x00000001;


        /********************  Bit definition for RCC_APB1ENR register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_TIM2EN = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_TIM3EN = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_TIM4EN = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_TIM5EN = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_TIM6EN = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_TIM7EN = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_TIM12EN = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_TIM13EN = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_TIM14EN = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_WWDGEN = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_SPI2EN = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_SPI3EN = 0x00008000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_USART2EN = 0x00020000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_USART3EN = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_UART4EN = 0x00080000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_UART5EN = 0x00100000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_I2C1EN = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_I2C2EN = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_I2C3EN = 0x00800000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_CAN1EN = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_CAN2EN = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_PWREN = 0x10000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1ENR_DACEN = 0x20000000;


        /********************  Bit definition for RCC_APB2ENR register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2ENR_TIM1EN = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2ENR_TIM8EN = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2ENR_USART1EN = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2ENR_USART6EN = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2ENR_ADC1EN = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2ENR_ADC2EN = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2ENR_ADC3EN = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2ENR_SDIOEN = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2ENR_SPI1EN = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2ENR_SYSCFGEN = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2ENR_TIM11EN = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2ENR_TIM10EN = 0x00020000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2ENR_TIM9EN = 0x00010000;


        /********************  Bit definition for RCC_AHB1LPENR register  *************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_GPIOALPEN = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_GPIOBLPEN = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_GPIOCLPEN = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_GPIODLPEN = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_GPIOELPEN = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_GPIOFLPEN = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_GPIOGLPEN = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_GPIOHLPEN = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_GPIOILPEN = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_CRCLPEN = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_FLITFLPEN = 0x00008000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_SRAM1LPEN = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_SRAM2LPEN = 0x00020000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_BKPSRAMLPEN = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_DMA1LPEN = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_DMA2LPEN = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_ETHMACLPEN = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_ETHMACTXLPEN = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_ETHMACRXLPEN = 0x08000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_ETHMACPTPLPEN = 0x10000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_OTGHSLPEN = 0x20000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB1LPENR_OTGHSULPILPEN = 0x40000000;


        /********************  Bit definition for RCC_AHB2LPENR register  *************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB2LPENR_DCMILPEN = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB2LPENR_CRYPLPEN = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB2LPENR_HASHLPEN = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB2LPENR_RNGLPEN = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB2LPENR_OTGFSLPEN = 0x00000080;


        /********************  Bit definition for RCC_AHB3LPENR register  *************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_AHB3LPENR_FSMCLPEN = 0x00000001;


        /********************  Bit definition for RCC_APB1LPENR register  *************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_TIM2LPEN = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_TIM3LPEN = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_TIM4LPEN = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_TIM5LPEN = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_TIM6LPEN = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_TIM7LPEN = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_TIM12LPEN = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_TIM13LPEN = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_TIM14LPEN = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_WWDGLPEN = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_SPI2LPEN = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_SPI3LPEN = 0x00008000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_USART2LPEN = 0x00020000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_USART3LPEN = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_UART4LPEN = 0x00080000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_UART5LPEN = 0x00100000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_I2C1LPEN = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_I2C2LPEN = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_I2C3LPEN = 0x00800000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_CAN1LPEN = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_CAN2LPEN = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_PWRLPEN = 0x10000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB1LPENR_DACLPEN = 0x20000000;


        /********************  Bit definition for RCC_APB2LPENR register  *************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2LPENR_TIM1LPEN = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2LPENR_TIM8LPEN = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2LPENR_USART1LPEN = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2LPENR_USART6LPEN = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2LPENR_ADC1LPEN = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2LPENR_ADC2PEN = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2LPENR_ADC3LPEN = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2LPENR_SDIOLPEN = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2LPENR_SPI1LPEN = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2LPENR_SYSCFGLPEN = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2LPENR_TIM9LPEN = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2LPENR_TIM10LPEN = 0x00020000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_APB2LPENR_TIM11LPEN = 0x00040000;


        /********************  Bit definition for RCC_BDCR register  ******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_BDCR_LSEON = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_BDCR_LSERDY = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_BDCR_LSEBYP = 0x00000004;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_BDCR_RTCSEL = 0x00000300;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_BDCR_RTCSEL_0 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_BDCR_RTCSEL_1 = 0x00000200;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_BDCR_RTCEN = 0x00008000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_BDCR_BDRST = 0x00010000;


        /********************  Bit definition for RCC_CSR register  *******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CSR_LSION = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CSR_LSIRDY = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CSR_RMVF = 0x01000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CSR_BORRSTF = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CSR_PADRSTF = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CSR_PORRSTF = 0x08000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CSR_SFTRSTF = 0x10000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CSR_WDGRSTF = 0x20000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CSR_WWDGRSTF = 0x40000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_CSR_LPWRRSTF = 0x80000000;


        /********************  Bit definition for RCC_SSCGR register  *****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_SSCGR_MODPER = 0x00001FFF;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_SSCGR_INCSTEP = 0x0FFFE000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_SSCGR_SPREADSEL = 0x40000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_SSCGR_SSCGEN = 0x80000000;


        /********************  Bit definition for RCC_PLLI2SCFGR register  ************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLI2SCFGR_PLLI2SN = 0x00007FC0;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RCC_PLLI2SCFGR_PLLI2SR = 0x70000000;


        /******************************************************************************/
        /*                                                                            */
        /*                                    RNG                                     */
        /*                                                                            */
        /******************************************************************************/
        /********************  Bits definition for RNG_CR register  *******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RNG_CR_RNGEN = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RNG_CR_IE = 0x00000008;


        /********************  Bits definition for RNG_SR register  *******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RNG_SR_DRDY = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RNG_SR_CECS = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RNG_SR_SECS = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RNG_SR_CEIS = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RNG_SR_SEIS = 0x00000040;


        /******************************************************************************/
        /*                                                                            */
        /*                           Real-Time Clock (RTC)                            */
        /*                                                                            */
        /******************************************************************************/
        /********************  Bits definition for RTC_TR register  *******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_PM = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_HT = 0x00300000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_HT_0 = 0x00100000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_HT_1 = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_HU = 0x000F0000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_HU_0 = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_HU_1 = 0x00020000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_HU_2 = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_HU_3 = 0x00080000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_MNT = 0x00007000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_MNT_0 = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_MNT_1 = 0x00002000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_MNT_2 = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_MNU = 0x00000F00;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_MNU_0 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_MNU_1 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_MNU_2 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_MNU_3 = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_ST = 0x00000070;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_ST_0 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_ST_1 = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_ST_2 = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_SU = 0x0000000F;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_SU_0 = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_SU_1 = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_SU_2 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TR_SU_3 = 0x00000008;


        /********************  Bits definition for RTC_DR register  *******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_YT = 0x00F00000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_YT_0 = 0x00100000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_YT_1 = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_YT_2 = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_YT_3 = 0x00800000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_YU = 0x000F0000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_YU_0 = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_YU_1 = 0x00020000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_YU_2 = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_YU_3 = 0x00080000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_WDU = 0x0000E000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_WDU_0 = 0x00002000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_WDU_1 = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_WDU_2 = 0x00008000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_MT = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_MU = 0x00000F00;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_MU_0 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_MU_1 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_MU_2 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_MU_3 = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_DT = 0x00000030;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_DT_0 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_DT_1 = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_DU = 0x0000000F;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_DU_0 = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_DU_1 = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_DU_2 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_DR_DU_3 = 0x00000008;


        /********************  Bits definition for RTC_CR register  *******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_COE = 0x00800000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_OSEL = 0x00600000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_OSEL_0 = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_OSEL_1 = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_POL = 0x00100000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_COSEL = 0x00080000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_BCK = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_SUB1H = 0x00020000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_ADD1H = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_TSIE = 0x00008000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_WUTIE = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_ALRBIE = 0x00002000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_ALRAIE = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_TSE = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_WUTE = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_ALRBE = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_ALRAE = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_DCE = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_FMT = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_BYPSHAD = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_REFCKON = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_TSEDGE = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_WUCKSEL = 0x00000007;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_WUCKSEL_0 = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_WUCKSEL_1 = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CR_WUCKSEL_2 = 0x00000004;


        /********************  Bits definition for RTC_ISR register  ******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ISR_RECALPF = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ISR_TAMP1F = 0x00002000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ISR_TSOVF = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ISR_TSF = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ISR_WUTF = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ISR_ALRBF = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ISR_ALRAF = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ISR_INIT = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ISR_INITF = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ISR_RSF = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ISR_INITS = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ISR_SHPF = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ISR_WUTWF = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ISR_ALRBWF = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ISR_ALRAWF = 0x00000001;


        /********************  Bits definition for RTC_PRER register  *****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_PRER_PREDIV_A = 0x007F0000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_PRER_PREDIV_S = 0x00001FFF;


        /********************  Bits definition for RTC_WUTR register  *****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_WUTR_WUT = 0x0000FFFF;


        /********************  Bits definition for RTC_CALIBR register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CALIBR_DCS = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CALIBR_DC = 0x0000001F;


        /********************  Bits definition for RTC_ALRMAR register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_MSK4 = 0x80000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_WDSEL = 0x40000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_DT = 0x30000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_DT_0 = 0x10000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_DT_1 = 0x20000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_DU = 0x0F000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_DU_0 = 0x01000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_DU_1 = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_DU_2 = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_DU_3 = 0x08000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_MSK3 = 0x00800000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_PM = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_HT = 0x00300000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_HT_0 = 0x00100000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_HT_1 = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_HU = 0x000F0000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_HU_0 = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_HU_1 = 0x00020000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_HU_2 = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_HU_3 = 0x00080000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_MSK2 = 0x00008000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_MNT = 0x00007000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_MNT_0 = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_MNT_1 = 0x00002000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_MNT_2 = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_MNU = 0x00000F00;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_MNU_0 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_MNU_1 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_MNU_2 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_MNU_3 = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_MSK1 = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_ST = 0x00000070;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_ST_0 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_ST_1 = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_ST_2 = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_SU = 0x0000000F;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_SU_0 = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_SU_1 = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_SU_2 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMAR_SU_3 = 0x00000008;


        /********************  Bits definition for RTC_ALRMBR register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_MSK4 = 0x80000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_WDSEL = 0x40000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_DT = 0x30000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_DT_0 = 0x10000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_DT_1 = 0x20000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_DU = 0x0F000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_DU_0 = 0x01000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_DU_1 = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_DU_2 = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_DU_3 = 0x08000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_MSK3 = 0x00800000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_PM = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_HT = 0x00300000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_HT_0 = 0x00100000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_HT_1 = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_HU = 0x000F0000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_HU_0 = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_HU_1 = 0x00020000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_HU_2 = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_HU_3 = 0x00080000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_MSK2 = 0x00008000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_MNT = 0x00007000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_MNT_0 = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_MNT_1 = 0x00002000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_MNT_2 = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_MNU = 0x00000F00;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_MNU_0 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_MNU_1 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_MNU_2 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_MNU_3 = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_MSK1 = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_ST = 0x00000070;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_ST_0 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_ST_1 = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_ST_2 = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_SU = 0x0000000F;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_SU_0 = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_SU_1 = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_SU_2 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBR_SU_3 = 0x00000008;


        /********************  Bits definition for RTC_WPR register  ******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_WPR_KEY = 0x000000FF;


        /********************  Bits definition for RTC_SSR register  ******************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_SSR_SS = 0x0000FFFF;


        /********************  Bits definition for RTC_SHIFTR register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_SHIFTR_SUBFS = 0x00007FFF;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_SHIFTR_ADD1S = 0x80000000;


        /********************  Bits definition for RTC_TSTR register  *****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_PM = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_HT = 0x00300000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_HT_0 = 0x00100000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_HT_1 = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_HU = 0x000F0000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_HU_0 = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_HU_1 = 0x00020000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_HU_2 = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_HU_3 = 0x00080000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_MNT = 0x00007000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_MNT_0 = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_MNT_1 = 0x00002000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_MNT_2 = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_MNU = 0x00000F00;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_MNU_0 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_MNU_1 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_MNU_2 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_MNU_3 = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_ST = 0x00000070;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_ST_0 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_ST_1 = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_ST_2 = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_SU = 0x0000000F;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_SU_0 = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_SU_1 = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_SU_2 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSTR_SU_3 = 0x00000008;


        /********************  Bits definition for RTC_TSDR register  *****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSDR_WDU = 0x0000E000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSDR_WDU_0 = 0x00002000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSDR_WDU_1 = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSDR_WDU_2 = 0x00008000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSDR_MT = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSDR_MU = 0x00000F00;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSDR_MU_0 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSDR_MU_1 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSDR_MU_2 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSDR_MU_3 = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSDR_DT = 0x00000030;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSDR_DT_0 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSDR_DT_1 = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSDR_DU = 0x0000000F;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSDR_DU_0 = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSDR_DU_1 = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSDR_DU_2 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSDR_DU_3 = 0x00000008;


        /********************  Bits definition for RTC_TSSSR register  ****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TSSSR_SS = 0x0000FFFF;


        /********************  Bits definition for RTC_CAL register  *****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CALR_CALP = 0x00008000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CALR_CALW8 = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CALR_CALW16 = 0x00002000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CALR_CALM = 0x000001FF;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CALR_CALM_0 = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CALR_CALM_1 = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CALR_CALM_2 = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CALR_CALM_3 = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CALR_CALM_4 = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CALR_CALM_5 = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CALR_CALM_6 = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CALR_CALM_7 = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_CALR_CALM_8 = 0x00000100;


        /********************  Bits definition for RTC_TAFCR register  ****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TAFCR_ALARMOUTTYPE = 0x00040000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TAFCR_TSINSEL = 0x00020000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TAFCR_TAMPINSEL = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TAFCR_TAMPPUDIS = 0x00008000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TAFCR_TAMPPRCH = 0x00006000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TAFCR_TAMPPRCH_0 = 0x00002000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TAFCR_TAMPPRCH_1 = 0x00004000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TAFCR_TAMPFLT = 0x00001800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TAFCR_TAMPFLT_0 = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TAFCR_TAMPFLT_1 = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TAFCR_TAMPFREQ = 0x00000700;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TAFCR_TAMPFREQ_0 = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TAFCR_TAMPFREQ_1 = 0x00000200;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TAFCR_TAMPFREQ_2 = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TAFCR_TAMPTS = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TAFCR_TAMPIE = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TAFCR_TAMP1TRG = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_TAFCR_TAMP1E = 0x00000001;


        /********************  Bits definition for RTC_ALRMASSR register  *************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMASSR_MASKSS = 0x0F000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMASSR_MASKSS_0 = 0x01000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMASSR_MASKSS_1 = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMASSR_MASKSS_2 = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMASSR_MASKSS_3 = 0x08000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMASSR_SS = 0x00007FFF;


        /********************  Bits definition for RTC_ALRMBSSR register  *************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBSSR_MASKSS = 0x0F000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBSSR_MASKSS_0 = 0x01000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBSSR_MASKSS_1 = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBSSR_MASKSS_2 = 0x04000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBSSR_MASKSS_3 = 0x08000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_ALRMBSSR_SS = 0x00007FFF;


        /********************  Bits definition for RTC_BKP0R register  ****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BKP0R = 0xFFFFFFFF;


        /********************  Bits definition for RTC_BKP1R register  ****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BKP1R = 0xFFFFFFFF;


        /********************  Bits definition for RTC_BKP2R register  ****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BKP2R = 0xFFFFFFFF;


        /********************  Bits definition for RTC_BKP3R register  ****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BKP3R = 0xFFFFFFFF;


        /********************  Bits definition for RTC_BKP4R register  ****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BKP4R = 0xFFFFFFFF;


        /********************  Bits definition for RTC_BKP5R register  ****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BKP5R = 0xFFFFFFFF;


        /********************  Bits definition for RTC_BKP6R register  ****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BKP6R = 0xFFFFFFFF;


        /********************  Bits definition for RTC_BKP7R register  ****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BKP7R = 0xFFFFFFFF;


        /********************  Bits definition for RTC_BKP8R register  ****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BKP8R = 0xFFFFFFFF;


        /********************  Bits definition for RTC_BKP9R register  ****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BKP9R = 0xFFFFFFFF;


        /********************  Bits definition for RTC_BKP10R register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BKP10R = 0xFFFFFFFF;


        /********************  Bits definition for RTC_BKP11R register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BKP11R = 0xFFFFFFFF;


        /********************  Bits definition for RTC_BKP12R register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BKP12R = 0xFFFFFFFF;


        /********************  Bits definition for RTC_BKP13R register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BKP13R = 0xFFFFFFFF;


        /********************  Bits definition for RTC_BKP14R register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BKP14R = 0xFFFFFFFF;


        /********************  Bits definition for RTC_BKP15R register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BKP15R = 0xFFFFFFFF;


        /********************  Bits definition for RTC_BKP16R register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BKP16R = 0xFFFFFFFF;


        /********************  Bits definition for RTC_BKP17R register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BKP17R = 0xFFFFFFFF;


        /********************  Bits definition for RTC_BKP18R register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BKP18R = 0xFFFFFFFF;


        /********************  Bits definition for RTC_BKP19R register  ***************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 RTC_BKP19R = 0xFFFFFFFF;


        /******************************************************************************/
        /*                                                                            */
        /*                          SD host Interface                                 */
        /*                                                                            */
        /******************************************************************************/
        /******************  Bit definition for SDIO_POWER register  ******************/
        /// <summary>
        ///  PWRCTRL[1:0] bits (Power supply control bits) 
        /// </summary>
        public static byte SDIO_POWER_PWRCTRL = 0x03;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static byte SDIO_POWER_PWRCTRL_0 = 0x01;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static byte SDIO_POWER_PWRCTRL_1 = 0x02;


        /******************  Bit definition for SDIO_CLKCR register  ******************/
        /// <summary>
        ///  Clock divide factor 
        /// </summary>
        public static UInt16 SDIO_CLKCR_CLKDIV = 0x00FF;

        /// <summary>
        ///  Clock enable bit 
        /// </summary>
        public static UInt16 SDIO_CLKCR_CLKEN = 0x0100;

        /// <summary>
        ///  Power saving configuration bit 
        /// </summary>
        public static UInt16 SDIO_CLKCR_PWRSAV = 0x0200;

        /// <summary>
        ///  Clock divider bypass enable bit 
        /// </summary>
        public static UInt16 SDIO_CLKCR_BYPASS = 0x0400;


        /// <summary>
        ///  WIDBUS[1:0] bits (Wide bus mode enable bit) 
        /// </summary>
        public static UInt16 SDIO_CLKCR_WIDBUS = 0x1800;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 SDIO_CLKCR_WIDBUS_0 = 0x0800;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 SDIO_CLKCR_WIDBUS_1 = 0x1000;


        /// <summary>
        ///  SDIO_CK dephasing selection bit 
        /// </summary>
        public static UInt16 SDIO_CLKCR_NEGEDGE = 0x2000;

        /// <summary>
        ///  HW Flow Control enable 
        /// </summary>
        public static UInt16 SDIO_CLKCR_HWFC_EN = 0x4000;


        /*******************  Bit definition for SDIO_ARG register  *******************/
        /// <summary>
        ///  Command argument 
        /// </summary>
        public static UInt32 SDIO_ARG_CMDARG = 0xFFFFFFFF;


        /*******************  Bit definition for SDIO_CMD register  *******************/
        /// <summary>
        ///  Command Index 
        /// </summary>
        public static UInt16 SDIO_CMD_CMDINDEX = 0x003F;


        /// <summary>
        ///  WAITRESP[1:0] bits (Wait for response bits) 
        /// </summary>
        public static UInt16 SDIO_CMD_WAITRESP = 0x00C0;

        /// <summary>
        ///   Bit 0 
        /// </summary>
        public static UInt16 SDIO_CMD_WAITRESP_0 = 0x0040;

        /// <summary>
        ///   Bit 1 
        /// </summary>
        public static UInt16 SDIO_CMD_WAITRESP_1 = 0x0080;


        /// <summary>
        ///  CPSM Waits for Interrupt Request 
        /// </summary>
        public static UInt16 SDIO_CMD_WAITINT = 0x0100;

        /// <summary>
        ///  CPSM Waits for ends of data transfer (CmdPend internal signal) 
        /// </summary>
        public static UInt16 SDIO_CMD_WAITPEND = 0x0200;

        /// <summary>
        ///  Command path state machine (CPSM) Enable bit 
        /// </summary>
        public static UInt16 SDIO_CMD_CPSMEN = 0x0400;

        /// <summary>
        ///  SD I/O suspend command 
        /// </summary>
        public static UInt16 SDIO_CMD_SDIOSUSPEND = 0x0800;

        /// <summary>
        ///  Enable CMD completion 
        /// </summary>
        public static UInt16 SDIO_CMD_ENCMDCOMPL = 0x1000;

        /// <summary>
        ///  Not Interrupt Enable 
        /// </summary>
        public static UInt16 SDIO_CMD_NIEN = 0x2000;

        /// <summary>
        ///  CE-ATA command 
        /// </summary>
        public static UInt16 SDIO_CMD_CEATACMD = 0x4000;


        /*****************  Bit definition for SDIO_RESPCMD register  *****************/
        /// <summary>
        ///  Response command index 
        /// </summary>
        public static byte SDIO_RESPCMD_RESPCMD = 0x3F;


        /******************  Bit definition for SDIO_RESP0 register  ******************/
        /// <summary>
        ///  Card Status 
        /// </summary>
        public static UInt32 SDIO_RESP0_CARDSTATUS0 = 0xFFFFFFFF;


        /******************  Bit definition for SDIO_RESP1 register  ******************/
        /// <summary>
        ///  Card Status 
        /// </summary>
        public static UInt32 SDIO_RESP1_CARDSTATUS1 = 0xFFFFFFFF;


        /******************  Bit definition for SDIO_RESP2 register  ******************/
        /// <summary>
        ///  Card Status 
        /// </summary>
        public static UInt32 SDIO_RESP2_CARDSTATUS2 = 0xFFFFFFFF;


        /******************  Bit definition for SDIO_RESP3 register  ******************/
        /// <summary>
        ///  Card Status 
        /// </summary>
        public static UInt32 SDIO_RESP3_CARDSTATUS3 = 0xFFFFFFFF;


        /******************  Bit definition for SDIO_RESP4 register  ******************/
        /// <summary>
        ///  Card Status 
        /// </summary>
        public static UInt32 SDIO_RESP4_CARDSTATUS4 = 0xFFFFFFFF;


        /******************  Bit definition for SDIO_DTIMER register  *****************/
        /// <summary>
        ///  Data timeout period. 
        /// </summary>
        public static UInt32 SDIO_DTIMER_DATATIME = 0xFFFFFFFF;


        /******************  Bit definition for SDIO_DLEN register  *******************/
        /// <summary>
        ///  Data length value 
        /// </summary>
        public static UInt32 SDIO_DLEN_DATALENGTH = 0x01FFFFFF;


        /******************  Bit definition for SDIO_DCTRL register  ******************/
        /// <summary>
        ///  Data transfer enabled bit 
        /// </summary>
        public static UInt16 SDIO_DCTRL_DTEN = 0x0001;

        /// <summary>
        ///  Data transfer direction selection 
        /// </summary>
        public static UInt16 SDIO_DCTRL_DTDIR = 0x0002;

        /// <summary>
        ///  Data transfer mode selection 
        /// </summary>
        public static UInt16 SDIO_DCTRL_DTMODE = 0x0004;

        /// <summary>
        ///  DMA enabled bit 
        /// </summary>
        public static UInt16 SDIO_DCTRL_DMAEN = 0x0008;


        /// <summary>
        ///  DBLOCKSIZE[3:0] bits (Data block size) 
        /// </summary>
        public static UInt16 SDIO_DCTRL_DBLOCKSIZE = 0x00F0;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 SDIO_DCTRL_DBLOCKSIZE_0 = 0x0010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 SDIO_DCTRL_DBLOCKSIZE_1 = 0x0020;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt16 SDIO_DCTRL_DBLOCKSIZE_2 = 0x0040;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt16 SDIO_DCTRL_DBLOCKSIZE_3 = 0x0080;


        /// <summary>
        ///  Read wait start 
        /// </summary>
        public static UInt16 SDIO_DCTRL_RWSTART = 0x0100;

        /// <summary>
        ///  Read wait stop 
        /// </summary>
        public static UInt16 SDIO_DCTRL_RWSTOP = 0x0200;

        /// <summary>
        ///  Read wait mode 
        /// </summary>
        public static UInt16 SDIO_DCTRL_RWMOD = 0x0400;

        /// <summary>
        ///  SD I/O enable functions 
        /// </summary>
        public static UInt16 SDIO_DCTRL_SDIOEN = 0x0800;


        /******************  Bit definition for SDIO_DCOUNT register  *****************/
        /// <summary>
        ///  Data count value 
        /// </summary>
        public static UInt32 SDIO_DCOUNT_DATACOUNT = 0x01FFFFFF;


        /******************  Bit definition for SDIO_STA register  ********************/
        /// <summary>
        ///  Command response received (CRC check failed) 
        /// </summary>
        public static UInt32 SDIO_STA_CCRCFAIL = 0x00000001;

        /// <summary>
        ///  Data block sent/received (CRC check failed) 
        /// </summary>
        public static UInt32 SDIO_STA_DCRCFAIL = 0x00000002;

        /// <summary>
        ///  Command response timeout 
        /// </summary>
        public static UInt32 SDIO_STA_CTIMEOUT = 0x00000004;

        /// <summary>
        ///  Data timeout 
        /// </summary>
        public static UInt32 SDIO_STA_DTIMEOUT = 0x00000008;

        /// <summary>
        ///  Transmit FIFO underrun error 
        /// </summary>
        public static UInt32 SDIO_STA_TXUNDERR = 0x00000010;

        /// <summary>
        ///  Received FIFO overrun error 
        /// </summary>
        public static UInt32 SDIO_STA_RXOVERR = 0x00000020;

        /// <summary>
        ///  Command response received (CRC check passed) 
        /// </summary>
        public static UInt32 SDIO_STA_CMDREND = 0x00000040;

        /// <summary>
        ///  Command sent (no response required) 
        /// </summary>
        public static UInt32 SDIO_STA_CMDSENT = 0x00000080;

        /// <summary>
        ///  Data end (data counter, SDIDCOUNT, is zero) 
        /// </summary>
        public static UInt32 SDIO_STA_DATAEND = 0x00000100;

        /// <summary>
        ///  Start bit not detected on all data signals in wide bus mode 
        /// </summary>
        public static UInt32 SDIO_STA_STBITERR = 0x00000200;

        /// <summary>
        ///  Data block sent/received (CRC check passed) 
        /// </summary>
        public static UInt32 SDIO_STA_DBCKEND = 0x00000400;

        /// <summary>
        ///  Command transfer in progress 
        /// </summary>
        public static UInt32 SDIO_STA_CMDACT = 0x00000800;

        /// <summary>
        ///  Data transmit in progress 
        /// </summary>
        public static UInt32 SDIO_STA_TXACT = 0x00001000;

        /// <summary>
        ///  Data receive in progress 
        /// </summary>
        public static UInt32 SDIO_STA_RXACT = 0x00002000;

        /// <summary>
        ///  Transmit FIFO Half Empty: at least 8 words can be written into the FIFO 
        /// </summary>
        public static UInt32 SDIO_STA_TXFIFOHE = 0x00004000;

        /// <summary>
        ///  Receive FIFO Half Full: there are at least 8 words in the FIFO 
        /// </summary>
        public static UInt32 SDIO_STA_RXFIFOHF = 0x00008000;

        /// <summary>
        ///  Transmit FIFO full 
        /// </summary>
        public static UInt32 SDIO_STA_TXFIFOF = 0x00010000;

        /// <summary>
        ///  Receive FIFO full 
        /// </summary>
        public static UInt32 SDIO_STA_RXFIFOF = 0x00020000;

        /// <summary>
        ///  Transmit FIFO empty 
        /// </summary>
        public static UInt32 SDIO_STA_TXFIFOE = 0x00040000;

        /// <summary>
        ///  Receive FIFO empty 
        /// </summary>
        public static UInt32 SDIO_STA_RXFIFOE = 0x00080000;

        /// <summary>
        ///  Data available in transmit FIFO 
        /// </summary>
        public static UInt32 SDIO_STA_TXDAVL = 0x00100000;

        /// <summary>
        ///  Data available in receive FIFO 
        /// </summary>
        public static UInt32 SDIO_STA_RXDAVL = 0x00200000;

        /// <summary>
        ///  SDIO interrupt received 
        /// </summary>
        public static UInt32 SDIO_STA_SDIOIT = 0x00400000;

        /// <summary>
        ///  CE-ATA command completion signal received for CMD61 
        /// </summary>
        public static UInt32 SDIO_STA_CEATAEND = 0x00800000;


        /*******************  Bit definition for SDIO_ICR register  *******************/
        /// <summary>
        ///  CCRCFAIL flag clear bit 
        /// </summary>
        public static UInt32 SDIO_ICR_CCRCFAILC = 0x00000001;

        /// <summary>
        ///  DCRCFAIL flag clear bit 
        /// </summary>
        public static UInt32 SDIO_ICR_DCRCFAILC = 0x00000002;

        /// <summary>
        ///  CTIMEOUT flag clear bit 
        /// </summary>
        public static UInt32 SDIO_ICR_CTIMEOUTC = 0x00000004;

        /// <summary>
        ///  DTIMEOUT flag clear bit 
        /// </summary>
        public static UInt32 SDIO_ICR_DTIMEOUTC = 0x00000008;

        /// <summary>
        ///  TXUNDERR flag clear bit 
        /// </summary>
        public static UInt32 SDIO_ICR_TXUNDERRC = 0x00000010;

        /// <summary>
        ///  RXOVERR flag clear bit 
        /// </summary>
        public static UInt32 SDIO_ICR_RXOVERRC = 0x00000020;

        /// <summary>
        ///  CMDREND flag clear bit 
        /// </summary>
        public static UInt32 SDIO_ICR_CMDRENDC = 0x00000040;

        /// <summary>
        ///  CMDSENT flag clear bit 
        /// </summary>
        public static UInt32 SDIO_ICR_CMDSENTC = 0x00000080;

        /// <summary>
        ///  DATAEND flag clear bit 
        /// </summary>
        public static UInt32 SDIO_ICR_DATAENDC = 0x00000100;

        /// <summary>
        ///  STBITERR flag clear bit 
        /// </summary>
        public static UInt32 SDIO_ICR_STBITERRC = 0x00000200;

        /// <summary>
        ///  DBCKEND flag clear bit 
        /// </summary>
        public static UInt32 SDIO_ICR_DBCKENDC = 0x00000400;

        /// <summary>
        ///  SDIOIT flag clear bit 
        /// </summary>
        public static UInt32 SDIO_ICR_SDIOITC = 0x00400000;

        /// <summary>
        ///  CEATAEND flag clear bit 
        /// </summary>
        public static UInt32 SDIO_ICR_CEATAENDC = 0x00800000;


        /******************  Bit definition for SDIO_MASK register  *******************/
        /// <summary>
        ///  Command CRC Fail Interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_CCRCFAILIE = 0x00000001;

        /// <summary>
        ///  Data CRC Fail Interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_DCRCFAILIE = 0x00000002;

        /// <summary>
        ///  Command TimeOut Interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_CTIMEOUTIE = 0x00000004;

        /// <summary>
        ///  Data TimeOut Interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_DTIMEOUTIE = 0x00000008;

        /// <summary>
        ///  Tx FIFO UnderRun Error Interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_TXUNDERRIE = 0x00000010;

        /// <summary>
        ///  Rx FIFO OverRun Error Interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_RXOVERRIE = 0x00000020;

        /// <summary>
        ///  Command Response Received Interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_CMDRENDIE = 0x00000040;

        /// <summary>
        ///  Command Sent Interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_CMDSENTIE = 0x00000080;

        /// <summary>
        ///  Data End Interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_DATAENDIE = 0x00000100;

        /// <summary>
        ///  Start Bit Error Interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_STBITERRIE = 0x00000200;

        /// <summary>
        ///  Data Block End Interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_DBCKENDIE = 0x00000400;

        /// <summary>
        ///  CCommand Acting Interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_CMDACTIE = 0x00000800;

        /// <summary>
        ///  Data Transmit Acting Interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_TXACTIE = 0x00001000;

        /// <summary>
        ///  Data receive acting interrupt enabled 
        /// </summary>
        public static UInt32 SDIO_MASK_RXACTIE = 0x00002000;

        /// <summary>
        ///  Tx FIFO Half Empty interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_TXFIFOHEIE = 0x00004000;

        /// <summary>
        ///  Rx FIFO Half Full interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_RXFIFOHFIE = 0x00008000;

        /// <summary>
        ///  Tx FIFO Full interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_TXFIFOFIE = 0x00010000;

        /// <summary>
        ///  Rx FIFO Full interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_RXFIFOFIE = 0x00020000;

        /// <summary>
        ///  Tx FIFO Empty interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_TXFIFOEIE = 0x00040000;

        /// <summary>
        ///  Rx FIFO Empty interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_RXFIFOEIE = 0x00080000;

        /// <summary>
        ///  Data available in Tx FIFO interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_TXDAVLIE = 0x00100000;

        /// <summary>
        ///  Data available in Rx FIFO interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_RXDAVLIE = 0x00200000;

        /// <summary>
        ///  SDIO Mode Interrupt Received interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_SDIOITIE = 0x00400000;

        /// <summary>
        ///  CE-ATA command completion signal received Interrupt Enable 
        /// </summary>
        public static UInt32 SDIO_MASK_CEATAENDIE = 0x00800000;


        /*****************  Bit definition for SDIO_FIFOCNT register  *****************/
        /// <summary>
        ///  Remaining number of words to be written to or read from the FIFO 
        /// </summary>
        public static UInt32 SDIO_FIFOCNT_FIFOCOUNT = 0x00FFFFFF;


        /******************  Bit definition for SDIO_FIFO register  *******************/
        /// <summary>
        ///  Receive and transmit FIFO data 
        /// </summary>
        public static UInt32 SDIO_FIFO_FIFODATA = 0xFFFFFFFF;


        /******************************************************************************/
        /*                                                                            */
        /*                        Serial Peripheral Interface                         */
        /*                                                                            */
        /******************************************************************************/
        /*******************  Bit definition for SPI_CR1 register  ********************/
        /// <summary>
        ///  Clock Phase 
        /// </summary>
        public static UInt16 SPI_CR1_CPHA = 0x0001;

        /// <summary>
        ///  Clock Polarity 
        /// </summary>
        public static UInt16 SPI_CR1_CPOL = 0x0002;

        /// <summary>
        ///  Master Selection 
        /// </summary>
        public static UInt16 SPI_CR1_MSTR = 0x0004;


        /// <summary>
        ///  BR[2:0] bits (Baud Rate Control) 
        /// </summary>
        public static UInt16 SPI_CR1_BR = 0x0038;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 SPI_CR1_BR_0 = 0x0008;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 SPI_CR1_BR_1 = 0x0010;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt16 SPI_CR1_BR_2 = 0x0020;


        /// <summary>
        ///  SPI Enable 
        /// </summary>
        public static UInt16 SPI_CR1_SPE = 0x0040;

        /// <summary>
        ///  Frame Format 
        /// </summary>
        public static UInt16 SPI_CR1_LSBFIRST = 0x0080;

        /// <summary>
        ///  Internal slave select 
        /// </summary>
        public static UInt16 SPI_CR1_SSI = 0x0100;

        /// <summary>
        ///  Software slave management 
        /// </summary>
        public static UInt16 SPI_CR1_SSM = 0x0200;

        /// <summary>
        ///  Receive only 
        /// </summary>
        public static UInt16 SPI_CR1_RXONLY = 0x0400;

        /// <summary>
        ///  Data Frame Format 
        /// </summary>
        public static UInt16 SPI_CR1_DFF = 0x0800;

        /// <summary>
        ///  Transmit CRC next 
        /// </summary>
        public static UInt16 SPI_CR1_CRCNEXT = 0x1000;

        /// <summary>
        ///  Hardware CRC calculation enable 
        /// </summary>
        public static UInt16 SPI_CR1_CRCEN = 0x2000;

        /// <summary>
        ///  Output enable in bidirectional mode 
        /// </summary>
        public static UInt16 SPI_CR1_BIDIOE = 0x4000;

        /// <summary>
        ///  Bidirectional data mode enable 
        /// </summary>
        public static UInt16 SPI_CR1_BIDIMODE = 0x8000;


        /*******************  Bit definition for SPI_CR2 register  ********************/
        /// <summary>
        ///  Rx Buffer DMA Enable 
        /// </summary>
        public static byte SPI_CR2_RXDMAEN = 0x01;

        /// <summary>
        ///  Tx Buffer DMA Enable 
        /// </summary>
        public static byte SPI_CR2_TXDMAEN = 0x02;

        /// <summary>
        ///  SS Output Enable 
        /// </summary>
        public static byte SPI_CR2_SSOE = 0x04;

        /// <summary>
        ///  Error Interrupt Enable 
        /// </summary>
        public static byte SPI_CR2_ERRIE = 0x20;

        /// <summary>
        ///  RX buffer Not Empty Interrupt Enable 
        /// </summary>
        public static byte SPI_CR2_RXNEIE = 0x40;

        /// <summary>
        ///  Tx buffer Empty Interrupt Enable 
        /// </summary>
        public static byte SPI_CR2_TXEIE = 0x80;


        /********************  Bit definition for SPI_SR register  ********************/
        /// <summary>
        ///  Receive buffer Not Empty 
        /// </summary>
        public static byte SPI_SR_RXNE = 0x01;

        /// <summary>
        ///  Transmit buffer Empty 
        /// </summary>
        public static byte SPI_SR_TXE = 0x02;

        /// <summary>
        ///  Channel side 
        /// </summary>
        public static byte SPI_SR_CHSIDE = 0x04;

        /// <summary>
        ///  Underrun flag 
        /// </summary>
        public static byte SPI_SR_UDR = 0x08;

        /// <summary>
        ///  CRC Error flag 
        /// </summary>
        public static byte SPI_SR_CRCERR = 0x10;

        /// <summary>
        ///  Mode fault 
        /// </summary>
        public static byte SPI_SR_MODF = 0x20;

        /// <summary>
        ///  Overrun flag 
        /// </summary>
        public static byte SPI_SR_OVR = 0x40;

        /// <summary>
        ///  Busy flag 
        /// </summary>
        public static byte SPI_SR_BSY = 0x80;


        /********************  Bit definition for SPI_DR register  ********************/
        /// <summary>
        ///  Data Register 
        /// </summary>
        public static UInt16 SPI_DR_DR = 0xFFFF;


        /*******************  Bit definition for SPI_CRCPR register  ******************/
        /// <summary>
        ///  CRC polynomial register 
        /// </summary>
        public static UInt16 SPI_CRCPR_CRCPOLY = 0xFFFF;


        /******************  Bit definition for SPI_RXCRCR register  ******************/
        /// <summary>
        ///  Rx CRC Register 
        /// </summary>
        public static UInt16 SPI_RXCRCR_RXCRC = 0xFFFF;


        /******************  Bit definition for SPI_TXCRCR register  ******************/
        /// <summary>
        ///  Tx CRC Register 
        /// </summary>
        public static UInt16 SPI_TXCRCR_TXCRC = 0xFFFF;


        /******************  Bit definition for SPI_I2SCFGR register  *****************/
        /// <summary>
        ///  Channel length (number of bits per audio channel) 
        /// </summary>
        public static UInt16 SPI_I2SCFGR_CHLEN = 0x0001;


        /// <summary>
        ///  DATLEN[1:0] bits (Data length to be transferred) 
        /// </summary>
        public static UInt16 SPI_I2SCFGR_DATLEN = 0x0006;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 SPI_I2SCFGR_DATLEN_0 = 0x0002;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 SPI_I2SCFGR_DATLEN_1 = 0x0004;


        /// <summary>
        ///  steady state clock polarity 
        /// </summary>
        public static UInt16 SPI_I2SCFGR_CKPOL = 0x0008;


        /// <summary>
        ///  I2SSTD[1:0] bits (I2S standard selection) 
        /// </summary>
        public static UInt16 SPI_I2SCFGR_I2SSTD = 0x0030;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 SPI_I2SCFGR_I2SSTD_0 = 0x0010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 SPI_I2SCFGR_I2SSTD_1 = 0x0020;


        /// <summary>
        ///  PCM frame synchronization 
        /// </summary>
        public static UInt16 SPI_I2SCFGR_PCMSYNC = 0x0080;


        /// <summary>
        ///  I2SCFG[1:0] bits (I2S configuration mode) 
        /// </summary>
        public static UInt16 SPI_I2SCFGR_I2SCFG = 0x0300;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 SPI_I2SCFGR_I2SCFG_0 = 0x0100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 SPI_I2SCFGR_I2SCFG_1 = 0x0200;


        /// <summary>
        ///  I2S Enable 
        /// </summary>
        public static UInt16 SPI_I2SCFGR_I2SE = 0x0400;

        /// <summary>
        ///  I2S mode selection 
        /// </summary>
        public static UInt16 SPI_I2SCFGR_I2SMOD = 0x0800;


        /******************  Bit definition for SPI_I2SPR register  *******************/
        /// <summary>
        ///  I2S Linear prescaler 
        /// </summary>
        public static UInt16 SPI_I2SPR_I2SDIV = 0x00FF;

        /// <summary>
        ///  Odd factor for the prescaler 
        /// </summary>
        public static UInt16 SPI_I2SPR_ODD = 0x0100;

        /// <summary>
        ///  Master Clock Output Enable 
        /// </summary>
        public static UInt16 SPI_I2SPR_MCKOE = 0x0200;


        /******************************************************************************/
        /*                                                                            */
        /*                                 SYSCFG                                     */
        /*                                                                            */
        /******************************************************************************/
        /******************  Bit definition for SYSCFG_MEMRMP register  ***************/
        /// <summary>
        ///  SYSCFG_Memory Remap Config 
        /// </summary>
        public static UInt32 SYSCFG_MEMRMP_MEM_MODE = 0x00000003;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 SYSCFG_MEMRMP_MEM_MODE_0 = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 SYSCFG_MEMRMP_MEM_MODE_1 = 0x00000002;


        /******************  Bit definition for SYSCFG_PMC register  ******************/
        /// <summary>
        ///  Ethernet PHY interface selection 
        /// </summary>
        public static UInt32 SYSCFG_PMC_MII_RMII_SEL = 0x00800000;

        /* Old MII_RMII_SEL bit definition, maintained for legacy purpose */
        /// <summary>
        /// 
        /// </summary>
        public UInt32 SYSCFG_PMC_MII_RMII = SYSCFG_PMC_MII_RMII_SEL;

        /*****************  Bit definition for SYSCFG_EXTICR1 register  ***************/
        /// <summary>
        ///  EXTI 0 configuration 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI0 = 0x000F;

        /// <summary>
        ///  EXTI 1 configuration 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI1 = 0x00F0;

        /// <summary>
        ///  EXTI 2 configuration 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI2 = 0x0F00;

        /// <summary>
        ///  EXTI 3 configuration 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI3 = 0xF000;

        /** 
          * @brief   EXTI0 configuration  
          */
        /// <summary>
        ///  PA[0] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI0_PA = 0x0000;

        /// <summary>
        ///  PB[0] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI0_PB = 0x0001;

        /// <summary>
        ///  PC[0] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI0_PC = 0x0002;

        /// <summary>
        ///  PD[0] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI0_PD = 0x0003;

        /// <summary>
        ///  PE[0] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI0_PE = 0x0004;

        /// <summary>
        ///  PF[0] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI0_PF = 0x0005;

        /// <summary>
        ///  PG[0] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI0_PG = 0x0006;

        /// <summary>
        ///  PH[0] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI0_PH = 0x0007;

        /// <summary>
        ///  PI[0] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI0_PI = 0x0008;

        /** 
          * @brief   EXTI1 configuration  
          */
        /// <summary>
        ///  PA[1] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI1_PA = 0x0000;

        /// <summary>
        ///  PB[1] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI1_PB = 0x0010;

        /// <summary>
        ///  PC[1] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI1_PC = 0x0020;

        /// <summary>
        ///  PD[1] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI1_PD = 0x0030;

        /// <summary>
        ///  PE[1] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI1_PE = 0x0040;

        /// <summary>
        ///  PF[1] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI1_PF = 0x0050;

        /// <summary>
        ///  PG[1] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI1_PG = 0x0060;

        /// <summary>
        ///  PH[1] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI1_PH = 0x0070;

        /// <summary>
        ///  PI[1] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI1_PI = 0x0080;

        /** 
          * @brief   EXTI2 configuration  
          */
        /// <summary>
        ///  PA[2] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI2_PA = 0x0000;

        /// <summary>
        ///  PB[2] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI2_PB = 0x0100;

        /// <summary>
        ///  PC[2] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI2_PC = 0x0200;

        /// <summary>
        ///  PD[2] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI2_PD = 0x0300;

        /// <summary>
        ///  PE[2] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI2_PE = 0x0400;

        /// <summary>
        ///  PF[2] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI2_PF = 0x0500;

        /// <summary>
        ///  PG[2] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI2_PG = 0x0600;

        /// <summary>
        ///  PH[2] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI2_PH = 0x0700;

        /// <summary>
        ///  PI[2] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI2_PI = 0x0800;

        /** 
          * @brief   EXTI3 configuration  
          */
        /// <summary>
        ///  PA[3] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI3_PA = 0x0000;

        /// <summary>
        ///  PB[3] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI3_PB = 0x1000;

        /// <summary>
        ///  PC[3] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI3_PC = 0x2000;

        /// <summary>
        ///  PD[3] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI3_PD = 0x3000;

        /// <summary>
        ///  PE[3] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI3_PE = 0x4000;

        /// <summary>
        ///  PF[3] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI3_PF = 0x5000;

        /// <summary>
        ///  PG[3] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI3_PG = 0x6000;

        /// <summary>
        ///  PH[3] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI3_PH = 0x7000;

        /// <summary>
        ///  PI[3] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR1_EXTI3_PI = 0x8000;


        /*****************  Bit definition for SYSCFG_EXTICR2 register  ***************/
        /// <summary>
        ///  EXTI 4 configuration 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI4 = 0x000F;

        /// <summary>
        ///  EXTI 5 configuration 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI5 = 0x00F0;

        /// <summary>
        ///  EXTI 6 configuration 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI6 = 0x0F00;

        /// <summary>
        ///  EXTI 7 configuration 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI7 = 0xF000;

        /** 
          * @brief   EXTI4 configuration  
          */
        /// <summary>
        ///  PA[4] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI4_PA = 0x0000;

        /// <summary>
        ///  PB[4] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI4_PB = 0x0001;

        /// <summary>
        ///  PC[4] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI4_PC = 0x0002;

        /// <summary>
        ///  PD[4] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI4_PD = 0x0003;

        /// <summary>
        ///  PE[4] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI4_PE = 0x0004;

        /// <summary>
        ///  PF[4] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI4_PF = 0x0005;

        /// <summary>
        ///  PG[4] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI4_PG = 0x0006;

        /// <summary>
        ///  PH[4] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI4_PH = 0x0007;

        /// <summary>
        ///  PI[4] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI4_PI = 0x0008;

        /** 
          * @brief   EXTI5 configuration  
          */
        /// <summary>
        ///  PA[5] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI5_PA = 0x0000;

        /// <summary>
        ///  PB[5] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI5_PB = 0x0010;

        /// <summary>
        ///  PC[5] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI5_PC = 0x0020;

        /// <summary>
        ///  PD[5] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI5_PD = 0x0030;

        /// <summary>
        ///  PE[5] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI5_PE = 0x0040;

        /// <summary>
        ///  PF[5] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI5_PF = 0x0050;

        /// <summary>
        ///  PG[5] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI5_PG = 0x0060;

        /// <summary>
        ///  PH[5] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI5_PH = 0x0070;

        /// <summary>
        ///  PI[5] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI5_PI = 0x0080;

        /** 
          * @brief   EXTI6 configuration  
          */
        /// <summary>
        ///  PA[6] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI6_PA = 0x0000;

        /// <summary>
        ///  PB[6] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI6_PB = 0x0100;

        /// <summary>
        ///  PC[6] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI6_PC = 0x0200;

        /// <summary>
        ///  PD[6] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI6_PD = 0x0300;

        /// <summary>
        ///  PE[6] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI6_PE = 0x0400;

        /// <summary>
        ///  PF[6] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI6_PF = 0x0500;

        /// <summary>
        ///  PG[6] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI6_PG = 0x0600;

        /// <summary>
        ///  PH[6] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI6_PH = 0x0700;

        /// <summary>
        ///  PI[6] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI6_PI = 0x0800;

        /** 
          * @brief   EXTI7 configuration  
          */
        /// <summary>
        ///  PA[7] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI7_PA = 0x0000;

        /// <summary>
        ///  PB[7] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI7_PB = 0x1000;

        /// <summary>
        ///  PC[7] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI7_PC = 0x2000;

        /// <summary>
        ///  PD[7] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI7_PD = 0x3000;

        /// <summary>
        ///  PE[7] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI7_PE = 0x4000;

        /// <summary>
        ///  PF[7] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI7_PF = 0x5000;

        /// <summary>
        ///  PG[7] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI7_PG = 0x6000;

        /// <summary>
        ///  PH[7] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI7_PH = 0x7000;

        /// <summary>
        ///  PI[7] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR2_EXTI7_PI = 0x8000;


        /*****************  Bit definition for SYSCFG_EXTICR3 register  ***************/
        /// <summary>
        ///  EXTI 8 configuration 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI8 = 0x000F;

        /// <summary>
        ///  EXTI 9 configuration 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI9 = 0x00F0;

        /// <summary>
        ///  EXTI 10 configuration 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI10 = 0x0F00;

        /// <summary>
        ///  EXTI 11 configuration 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI11 = 0xF000;


        /** 
          * @brief   EXTI8 configuration  
          */
        /// <summary>
        ///  PA[8] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI8_PA = 0x0000;

        /// <summary>
        ///  PB[8] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI8_PB = 0x0001;

        /// <summary>
        ///  PC[8] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI8_PC = 0x0002;

        /// <summary>
        ///  PD[8] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI8_PD = 0x0003;

        /// <summary>
        ///  PE[8] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI8_PE = 0x0004;

        /// <summary>
        ///  PF[8] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI8_PF = 0x0005;

        /// <summary>
        ///  PG[8] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI8_PG = 0x0006;

        /// <summary>
        ///  PH[8] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI8_PH = 0x0007;

        /// <summary>
        ///  PI[8] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI8_PI = 0x0008;

        /** 
          * @brief   EXTI9 configuration  
          */
        /// <summary>
        ///  PA[9] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI9_PA = 0x0000;

        /// <summary>
        ///  PB[9] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI9_PB = 0x0010;

        /// <summary>
        ///  PC[9] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI9_PC = 0x0020;

        /// <summary>
        ///  PD[9] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI9_PD = 0x0030;

        /// <summary>
        ///  PE[9] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI9_PE = 0x0040;

        /// <summary>
        ///  PF[9] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI9_PF = 0x0050;

        /// <summary>
        ///  PG[9] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI9_PG = 0x0060;

        /// <summary>
        ///  PH[9] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI9_PH = 0x0070;

        /// <summary>
        ///  PI[9] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI9_PI = 0x0080;

        /** 
          * @brief   EXTI10 configuration  
          */
        /// <summary>
        ///  PA[10] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI10_PA = 0x0000;

        /// <summary>
        ///  PB[10] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI10_PB = 0x0100;

        /// <summary>
        ///  PC[10] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI10_PC = 0x0200;

        /// <summary>
        ///  PD[10] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI10_PD = 0x0300;

        /// <summary>
        ///  PE[10] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI10_PE = 0x0400;

        /// <summary>
        ///  PF[10] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI10_PF = 0x0500;

        /// <summary>
        ///  PG[10] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI10_PG = 0x0600;

        /// <summary>
        ///  PH[10] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI10_PH = 0x0700;

        /// <summary>
        ///  PI[10] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI10_PI = 0x0800;

        /** 
          * @brief   EXTI11 configuration  
          */
        /// <summary>
        ///  PA[11] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI11_PA = 0x0000;

        /// <summary>
        ///  PB[11] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI11_PB = 0x1000;

        /// <summary>
        ///  PC[11] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI11_PC = 0x2000;

        /// <summary>
        ///  PD[11] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI11_PD = 0x3000;

        /// <summary>
        ///  PE[11] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI11_PE = 0x4000;

        /// <summary>
        ///  PF[11] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI11_PF = 0x5000;

        /// <summary>
        ///  PG[11] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI11_PG = 0x6000;

        /// <summary>
        ///  PH[11] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI11_PH = 0x7000;

        /// <summary>
        ///  PI[11] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI11_PI = 0x8000;


        /*****************  Bit definition for SYSCFG_EXTICR4 register  ***************/
        /// <summary>
        ///  EXTI 12 configuration 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI12 = 0x000F;

        /// <summary>
        ///  EXTI 13 configuration 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI13 = 0x00F0;

        /// <summary>
        ///  EXTI 14 configuration 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI14 = 0x0F00;

        /// <summary>
        ///  EXTI 15 configuration 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI15 = 0xF000;

        /** 
          * @brief   EXTI12 configuration  
          */
        /// <summary>
        ///  PA[12] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI12_PA = 0x0000;

        /// <summary>
        ///  PB[12] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI12_PB = 0x0001;

        /// <summary>
        ///  PC[12] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI12_PC = 0x0002;

        /// <summary>
        ///  PD[12] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI12_PD = 0x0003;

        /// <summary>
        ///  PE[12] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI12_PE = 0x0004;

        /// <summary>
        ///  PF[12] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI12_PF = 0x0005;

        /// <summary>
        ///  PG[12] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI12_PG = 0x0006;

        /// <summary>
        ///  PH[12] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI12_PH = 0x0007;

        /** 
          * @brief   EXTI13 configuration  
          */
        /// <summary>
        ///  PA[13] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI13_PA = 0x0000;

        /// <summary>
        ///  PB[13] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI13_PB = 0x0010;

        /// <summary>
        ///  PC[13] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI13_PC = 0x0020;

        /// <summary>
        ///  PD[13] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI13_PD = 0x0030;

        /// <summary>
        ///  PE[13] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI13_PE = 0x0040;

        /// <summary>
        ///  PF[13] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI13_PF = 0x0050;

        /// <summary>
        ///  PG[13] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI13_PG = 0x0060;

        /// <summary>
        ///  PH[13] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI13_PH = 0x0070;

        /** 
          * @brief   EXTI14 configuration  
          */
        /// <summary>
        ///  PA[14] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI14_PA = 0x0000;

        /// <summary>
        ///  PB[14] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI14_PB = 0x0100;

        /// <summary>
        ///  PC[14] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI14_PC = 0x0200;

        /// <summary>
        ///  PD[14] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI14_PD = 0x0300;

        /// <summary>
        ///  PE[14] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI14_PE = 0x0400;

        /// <summary>
        ///  PF[14] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI14_PF = 0x0500;

        /// <summary>
        ///  PG[14] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI14_PG = 0x0600;

        /// <summary>
        ///  PH[14] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI14_PH = 0x0700;

        /** 
          * @brief   EXTI15 configuration  
          */
        /// <summary>
        ///  PA[15] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI15_PA = 0x0000;

        /// <summary>
        ///  PB[15] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI15_PB = 0x1000;

        /// <summary>
        ///  PC[15] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI15_PC = 0x2000;

        /// <summary>
        ///  PD[15] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI15_PD = 0x3000;

        /// <summary>
        ///  PE[15] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI15_PE = 0x4000;

        /// <summary>
        ///  PF[15] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI15_PF = 0x5000;

        /// <summary>
        ///  PG[15] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR4_EXTI15_PG = 0x6000;

        /// <summary>
        ///  PH[15] pin 
        /// </summary>
        public static UInt16 SYSCFG_EXTICR3_EXTI15_PH = 0x7000;


        /******************  Bit definition for SYSCFG_CMPCR register  ****************/
        /// <summary>
        ///  Compensation cell ready flag 
        /// </summary>
        public static UInt32 SYSCFG_CMPCR_CMP_PD = 0x00000001;

        /// <summary>
        ///  Compensation cell power-down 
        /// </summary>
        public static UInt32 SYSCFG_CMPCR_READY = 0x00000100;


        /******************************************************************************/
        /*                                                                            */
        /*                                    TIM                                     */
        /*                                                                            */
        /******************************************************************************/
        /*******************  Bit definition for TIM_CR1 register  ********************/
        /// <summary>
        ///  Counter enable 
        /// </summary>
        public static UInt16 TIM_CR1_CEN = 0x0001;

        /// <summary>
        ///  Update disable 
        /// </summary>
        public static UInt16 TIM_CR1_UDIS = 0x0002;

        /// <summary>
        ///  Update request source 
        /// </summary>
        public static UInt16 TIM_CR1_URS = 0x0004;

        /// <summary>
        ///  One pulse mode 
        /// </summary>
        public static UInt16 TIM_CR1_OPM = 0x0008;

        /// <summary>
        ///  Direction 
        /// </summary>
        public static UInt16 TIM_CR1_DIR = 0x0010;


        /// <summary>
        ///  CMS[1:0] bits (Center-aligned mode selection) 
        /// </summary>
        public static UInt16 TIM_CR1_CMS = 0x0060;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_CR1_CMS_0 = 0x0020;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_CR1_CMS_1 = 0x0040;


        /// <summary>
        ///  Auto-reload preload enable 
        /// </summary>
        public static UInt16 TIM_CR1_ARPE = 0x0080;


        /// <summary>
        ///  CKD[1:0] bits (clock division) 
        /// </summary>
        public static UInt16 TIM_CR1_CKD = 0x0300;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_CR1_CKD_0 = 0x0100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_CR1_CKD_1 = 0x0200;


        /*******************  Bit definition for TIM_CR2 register  ********************/
        /// <summary>
        ///  Capture/Compare Preloaded Control 
        /// </summary>
        public static UInt16 TIM_CR2_CCPC = 0x0001;

        /// <summary>
        ///  Capture/Compare Control Update Selection 
        /// </summary>
        public static UInt16 TIM_CR2_CCUS = 0x0004;

        /// <summary>
        ///  Capture/Compare DMA Selection 
        /// </summary>
        public static UInt16 TIM_CR2_CCDS = 0x0008;


        /// <summary>
        ///  MMS[2:0] bits (Master Mode Selection) 
        /// </summary>
        public static UInt16 TIM_CR2_MMS = 0x0070;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_CR2_MMS_0 = 0x0010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_CR2_MMS_1 = 0x0020;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt16 TIM_CR2_MMS_2 = 0x0040;


        /// <summary>
        ///  TI1 Selection 
        /// </summary>
        public static UInt16 TIM_CR2_TI1S = 0x0080;

        /// <summary>
        ///  Output Idle state 1 (OC1 output) 
        /// </summary>
        public static UInt16 TIM_CR2_OIS1 = 0x0100;

        /// <summary>
        ///  Output Idle state 1 (OC1N output) 
        /// </summary>
        public static UInt16 TIM_CR2_OIS1N = 0x0200;

        /// <summary>
        ///  Output Idle state 2 (OC2 output) 
        /// </summary>
        public static UInt16 TIM_CR2_OIS2 = 0x0400;

        /// <summary>
        ///  Output Idle state 2 (OC2N output) 
        /// </summary>
        public static UInt16 TIM_CR2_OIS2N = 0x0800;

        /// <summary>
        ///  Output Idle state 3 (OC3 output) 
        /// </summary>
        public static UInt16 TIM_CR2_OIS3 = 0x1000;

        /// <summary>
        ///  Output Idle state 3 (OC3N output) 
        /// </summary>
        public static UInt16 TIM_CR2_OIS3N = 0x2000;

        /// <summary>
        ///  Output Idle state 4 (OC4 output) 
        /// </summary>
        public static UInt16 TIM_CR2_OIS4 = 0x4000;


        /*******************  Bit definition for TIM_SMCR register  *******************/
        /// <summary>
        ///  SMS[2:0] bits (Slave mode selection) 
        /// </summary>
        public static UInt16 TIM_SMCR_SMS = 0x0007;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_SMCR_SMS_0 = 0x0001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_SMCR_SMS_1 = 0x0002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt16 TIM_SMCR_SMS_2 = 0x0004;


        /// <summary>
        ///  TS[2:0] bits (Trigger selection) 
        /// </summary>
        public static UInt16 TIM_SMCR_TS = 0x0070;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_SMCR_TS_0 = 0x0010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_SMCR_TS_1 = 0x0020;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt16 TIM_SMCR_TS_2 = 0x0040;


        /// <summary>
        ///  Master/slave mode 
        /// </summary>
        public static UInt16 TIM_SMCR_MSM = 0x0080;


        /// <summary>
        ///  ETF[3:0] bits (External trigger filter) 
        /// </summary>
        public static UInt16 TIM_SMCR_ETF = 0x0F00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_SMCR_ETF_0 = 0x0100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_SMCR_ETF_1 = 0x0200;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt16 TIM_SMCR_ETF_2 = 0x0400;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt16 TIM_SMCR_ETF_3 = 0x0800;


        /// <summary>
        ///  ETPS[1:0] bits (External trigger prescaler) 
        /// </summary>
        public static UInt16 TIM_SMCR_ETPS = 0x3000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_SMCR_ETPS_0 = 0x1000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_SMCR_ETPS_1 = 0x2000;


        /// <summary>
        ///  External clock enable 
        /// </summary>
        public static UInt16 TIM_SMCR_ECE = 0x4000;

        /// <summary>
        ///  External trigger polarity 
        /// </summary>
        public static UInt16 TIM_SMCR_ETP = 0x8000;


        /*******************  Bit definition for TIM_DIER register  *******************/
        /// <summary>
        ///  Update interrupt enable 
        /// </summary>
        public static UInt16 TIM_DIER_UIE = 0x0001;

        /// <summary>
        ///  Capture/Compare 1 interrupt enable 
        /// </summary>
        public static UInt16 TIM_DIER_CC1IE = 0x0002;

        /// <summary>
        ///  Capture/Compare 2 interrupt enable 
        /// </summary>
        public static UInt16 TIM_DIER_CC2IE = 0x0004;

        /// <summary>
        ///  Capture/Compare 3 interrupt enable 
        /// </summary>
        public static UInt16 TIM_DIER_CC3IE = 0x0008;

        /// <summary>
        ///  Capture/Compare 4 interrupt enable 
        /// </summary>
        public static UInt16 TIM_DIER_CC4IE = 0x0010;

        /// <summary>
        ///  COM interrupt enable 
        /// </summary>
        public static UInt16 TIM_DIER_COMIE = 0x0020;

        /// <summary>
        ///  Trigger interrupt enable 
        /// </summary>
        public static UInt16 TIM_DIER_TIE = 0x0040;

        /// <summary>
        ///  Break interrupt enable 
        /// </summary>
        public static UInt16 TIM_DIER_BIE = 0x0080;

        /// <summary>
        ///  Update DMA request enable 
        /// </summary>
        public static UInt16 TIM_DIER_UDE = 0x0100;

        /// <summary>
        ///  Capture/Compare 1 DMA request enable 
        /// </summary>
        public static UInt16 TIM_DIER_CC1DE = 0x0200;

        /// <summary>
        ///  Capture/Compare 2 DMA request enable 
        /// </summary>
        public static UInt16 TIM_DIER_CC2DE = 0x0400;

        /// <summary>
        ///  Capture/Compare 3 DMA request enable 
        /// </summary>
        public static UInt16 TIM_DIER_CC3DE = 0x0800;

        /// <summary>
        ///  Capture/Compare 4 DMA request enable 
        /// </summary>
        public static UInt16 TIM_DIER_CC4DE = 0x1000;

        /// <summary>
        ///  COM DMA request enable 
        /// </summary>
        public static UInt16 TIM_DIER_COMDE = 0x2000;

        /// <summary>
        ///  Trigger DMA request enable 
        /// </summary>
        public static UInt16 TIM_DIER_TDE = 0x4000;


        /********************  Bit definition for TIM_SR register  ********************/
        /// <summary>
        ///  Update interrupt Flag 
        /// </summary>
        public static UInt16 TIM_SR_UIF = 0x0001;

        /// <summary>
        ///  Capture/Compare 1 interrupt Flag 
        /// </summary>
        public static UInt16 TIM_SR_CC1IF = 0x0002;

        /// <summary>
        ///  Capture/Compare 2 interrupt Flag 
        /// </summary>
        public static UInt16 TIM_SR_CC2IF = 0x0004;

        /// <summary>
        ///  Capture/Compare 3 interrupt Flag 
        /// </summary>
        public static UInt16 TIM_SR_CC3IF = 0x0008;

        /// <summary>
        ///  Capture/Compare 4 interrupt Flag 
        /// </summary>
        public static UInt16 TIM_SR_CC4IF = 0x0010;

        /// <summary>
        ///  COM interrupt Flag 
        /// </summary>
        public static UInt16 TIM_SR_COMIF = 0x0020;

        /// <summary>
        ///  Trigger interrupt Flag 
        /// </summary>
        public static UInt16 TIM_SR_TIF = 0x0040;

        /// <summary>
        ///  Break interrupt Flag 
        /// </summary>
        public static UInt16 TIM_SR_BIF = 0x0080;

        /// <summary>
        ///  Capture/Compare 1 Overcapture Flag 
        /// </summary>
        public static UInt16 TIM_SR_CC1OF = 0x0200;

        /// <summary>
        ///  Capture/Compare 2 Overcapture Flag 
        /// </summary>
        public static UInt16 TIM_SR_CC2OF = 0x0400;

        /// <summary>
        ///  Capture/Compare 3 Overcapture Flag 
        /// </summary>
        public static UInt16 TIM_SR_CC3OF = 0x0800;

        /// <summary>
        ///  Capture/Compare 4 Overcapture Flag 
        /// </summary>
        public static UInt16 TIM_SR_CC4OF = 0x1000;


        /*******************  Bit definition for TIM_EGR register  ********************/
        /// <summary>
        ///  Update Generation 
        /// </summary>
        public static byte TIM_EGR_UG = 0x01;

        /// <summary>
        ///  Capture/Compare 1 Generation 
        /// </summary>
        public static byte TIM_EGR_CC1G = 0x02;

        /// <summary>
        ///  Capture/Compare 2 Generation 
        /// </summary>
        public static byte TIM_EGR_CC2G = 0x04;

        /// <summary>
        ///  Capture/Compare 3 Generation 
        /// </summary>
        public static byte TIM_EGR_CC3G = 0x08;

        /// <summary>
        ///  Capture/Compare 4 Generation 
        /// </summary>
        public static byte TIM_EGR_CC4G = 0x10;

        /// <summary>
        ///  Capture/Compare Control Update Generation 
        /// </summary>
        public static byte TIM_EGR_COMG = 0x20;

        /// <summary>
        ///  Trigger Generation 
        /// </summary>
        public static byte TIM_EGR_TG = 0x40;

        /// <summary>
        ///  Break Generation 
        /// </summary>
        public static byte TIM_EGR_BG = 0x80;


        /******************  Bit definition for TIM_CCMR1 register  *******************/
        /// <summary>
        ///  CC1S[1:0] bits (Capture/Compare 1 Selection) 
        /// </summary>
        public static UInt16 TIM_CCMR1_CC1S = 0x0003;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_CCMR1_CC1S_0 = 0x0001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_CCMR1_CC1S_1 = 0x0002;


        /// <summary>
        ///  Output Compare 1 Fast enable 
        /// </summary>
        public static UInt16 TIM_CCMR1_OC1FE = 0x0004;

        /// <summary>
        ///  Output Compare 1 Preload enable 
        /// </summary>
        public static UInt16 TIM_CCMR1_OC1PE = 0x0008;


        /// <summary>
        ///  OC1M[2:0] bits (Output Compare 1 Mode) 
        /// </summary>
        public static UInt16 TIM_CCMR1_OC1M = 0x0070;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_CCMR1_OC1M_0 = 0x0010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_CCMR1_OC1M_1 = 0x0020;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt16 TIM_CCMR1_OC1M_2 = 0x0040;


        /// <summary>
        ///  Output Compare 1Clear Enable 
        /// </summary>
        public static UInt16 TIM_CCMR1_OC1CE = 0x0080;


        /// <summary>
        ///  CC2S[1:0] bits (Capture/Compare 2 Selection) 
        /// </summary>
        public static UInt16 TIM_CCMR1_CC2S = 0x0300;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_CCMR1_CC2S_0 = 0x0100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_CCMR1_CC2S_1 = 0x0200;


        /// <summary>
        ///  Output Compare 2 Fast enable 
        /// </summary>
        public static UInt16 TIM_CCMR1_OC2FE = 0x0400;

        /// <summary>
        ///  Output Compare 2 Preload enable 
        /// </summary>
        public static UInt16 TIM_CCMR1_OC2PE = 0x0800;


        /// <summary>
        ///  OC2M[2:0] bits (Output Compare 2 Mode) 
        /// </summary>
        public static UInt16 TIM_CCMR1_OC2M = 0x7000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_CCMR1_OC2M_0 = 0x1000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_CCMR1_OC2M_1 = 0x2000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt16 TIM_CCMR1_OC2M_2 = 0x4000;


        /// <summary>
        ///  Output Compare 2 Clear Enable 
        /// </summary>
        public static UInt16 TIM_CCMR1_OC2CE = 0x8000;


        /*----------------------------------------------------------------------------*/

        /// <summary>
        ///  IC1PSC[1:0] bits (Input Capture 1 Prescaler) 
        /// </summary>
        public static UInt16 TIM_CCMR1_IC1PSC = 0x000C;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_CCMR1_IC1PSC_0 = 0x0004;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_CCMR1_IC1PSC_1 = 0x0008;


        /// <summary>
        ///  IC1F[3:0] bits (Input Capture 1 Filter) 
        /// </summary>
        public static UInt16 TIM_CCMR1_IC1F = 0x00F0;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_CCMR1_IC1F_0 = 0x0010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_CCMR1_IC1F_1 = 0x0020;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt16 TIM_CCMR1_IC1F_2 = 0x0040;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt16 TIM_CCMR1_IC1F_3 = 0x0080;


        /// <summary>
        ///  IC2PSC[1:0] bits (Input Capture 2 Prescaler) 
        /// </summary>
        public static UInt16 TIM_CCMR1_IC2PSC = 0x0C00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_CCMR1_IC2PSC_0 = 0x0400;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_CCMR1_IC2PSC_1 = 0x0800;


        /// <summary>
        ///  IC2F[3:0] bits (Input Capture 2 Filter) 
        /// </summary>
        public static UInt16 TIM_CCMR1_IC2F = 0xF000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_CCMR1_IC2F_0 = 0x1000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_CCMR1_IC2F_1 = 0x2000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt16 TIM_CCMR1_IC2F_2 = 0x4000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt16 TIM_CCMR1_IC2F_3 = 0x8000;


        /******************  Bit definition for TIM_CCMR2 register  *******************/
        /// <summary>
        ///  CC3S[1:0] bits (Capture/Compare 3 Selection) 
        /// </summary>
        public static UInt16 TIM_CCMR2_CC3S = 0x0003;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_CCMR2_CC3S_0 = 0x0001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_CCMR2_CC3S_1 = 0x0002;


        /// <summary>
        ///  Output Compare 3 Fast enable 
        /// </summary>
        public static UInt16 TIM_CCMR2_OC3FE = 0x0004;

        /// <summary>
        ///  Output Compare 3 Preload enable 
        /// </summary>
        public static UInt16 TIM_CCMR2_OC3PE = 0x0008;


        /// <summary>
        ///  OC3M[2:0] bits (Output Compare 3 Mode) 
        /// </summary>
        public static UInt16 TIM_CCMR2_OC3M = 0x0070;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_CCMR2_OC3M_0 = 0x0010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_CCMR2_OC3M_1 = 0x0020;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt16 TIM_CCMR2_OC3M_2 = 0x0040;


        /// <summary>
        ///  Output Compare 3 Clear Enable 
        /// </summary>
        public static UInt16 TIM_CCMR2_OC3CE = 0x0080;


        /// <summary>
        ///  CC4S[1:0] bits (Capture/Compare 4 Selection) 
        /// </summary>
        public static UInt16 TIM_CCMR2_CC4S = 0x0300;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_CCMR2_CC4S_0 = 0x0100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_CCMR2_CC4S_1 = 0x0200;


        /// <summary>
        ///  Output Compare 4 Fast enable 
        /// </summary>
        public static UInt16 TIM_CCMR2_OC4FE = 0x0400;

        /// <summary>
        ///  Output Compare 4 Preload enable 
        /// </summary>
        public static UInt16 TIM_CCMR2_OC4PE = 0x0800;


        /// <summary>
        ///  OC4M[2:0] bits (Output Compare 4 Mode) 
        /// </summary>
        public static UInt16 TIM_CCMR2_OC4M = 0x7000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_CCMR2_OC4M_0 = 0x1000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_CCMR2_OC4M_1 = 0x2000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt16 TIM_CCMR2_OC4M_2 = 0x4000;


        /// <summary>
        ///  Output Compare 4 Clear Enable 
        /// </summary>
        public static UInt16 TIM_CCMR2_OC4CE = 0x8000;


        /*----------------------------------------------------------------------------*/

        /// <summary>
        ///  IC3PSC[1:0] bits (Input Capture 3 Prescaler) 
        /// </summary>
        public static UInt16 TIM_CCMR2_IC3PSC = 0x000C;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_CCMR2_IC3PSC_0 = 0x0004;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_CCMR2_IC3PSC_1 = 0x0008;


        /// <summary>
        ///  IC3F[3:0] bits (Input Capture 3 Filter) 
        /// </summary>
        public static UInt16 TIM_CCMR2_IC3F = 0x00F0;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_CCMR2_IC3F_0 = 0x0010;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_CCMR2_IC3F_1 = 0x0020;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt16 TIM_CCMR2_IC3F_2 = 0x0040;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt16 TIM_CCMR2_IC3F_3 = 0x0080;


        /// <summary>
        ///  IC4PSC[1:0] bits (Input Capture 4 Prescaler) 
        /// </summary>
        public static UInt16 TIM_CCMR2_IC4PSC = 0x0C00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_CCMR2_IC4PSC_0 = 0x0400;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_CCMR2_IC4PSC_1 = 0x0800;


        /// <summary>
        ///  IC4F[3:0] bits (Input Capture 4 Filter) 
        /// </summary>
        public static UInt16 TIM_CCMR2_IC4F = 0xF000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_CCMR2_IC4F_0 = 0x1000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_CCMR2_IC4F_1 = 0x2000;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt16 TIM_CCMR2_IC4F_2 = 0x4000;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt16 TIM_CCMR2_IC4F_3 = 0x8000;


        /*******************  Bit definition for TIM_CCER register  *******************/
        /// <summary>
        ///  Capture/Compare 1 output enable 
        /// </summary>
        public static UInt16 TIM_CCER_CC1E = 0x0001;

        /// <summary>
        ///  Capture/Compare 1 output Polarity 
        /// </summary>
        public static UInt16 TIM_CCER_CC1P = 0x0002;

        /// <summary>
        ///  Capture/Compare 1 Complementary output enable 
        /// </summary>
        public static UInt16 TIM_CCER_CC1NE = 0x0004;

        /// <summary>
        ///  Capture/Compare 1 Complementary output Polarity 
        /// </summary>
        public static UInt16 TIM_CCER_CC1NP = 0x0008;

        /// <summary>
        ///  Capture/Compare 2 output enable 
        /// </summary>
        public static UInt16 TIM_CCER_CC2E = 0x0010;

        /// <summary>
        ///  Capture/Compare 2 output Polarity 
        /// </summary>
        public static UInt16 TIM_CCER_CC2P = 0x0020;

        /// <summary>
        ///  Capture/Compare 2 Complementary output enable 
        /// </summary>
        public static UInt16 TIM_CCER_CC2NE = 0x0040;

        /// <summary>
        ///  Capture/Compare 2 Complementary output Polarity 
        /// </summary>
        public static UInt16 TIM_CCER_CC2NP = 0x0080;

        /// <summary>
        ///  Capture/Compare 3 output enable 
        /// </summary>
        public static UInt16 TIM_CCER_CC3E = 0x0100;

        /// <summary>
        ///  Capture/Compare 3 output Polarity 
        /// </summary>
        public static UInt16 TIM_CCER_CC3P = 0x0200;

        /// <summary>
        ///  Capture/Compare 3 Complementary output enable 
        /// </summary>
        public static UInt16 TIM_CCER_CC3NE = 0x0400;

        /// <summary>
        ///  Capture/Compare 3 Complementary output Polarity 
        /// </summary>
        public static UInt16 TIM_CCER_CC3NP = 0x0800;

        /// <summary>
        ///  Capture/Compare 4 output enable 
        /// </summary>
        public static UInt16 TIM_CCER_CC4E = 0x1000;

        /// <summary>
        ///  Capture/Compare 4 output Polarity 
        /// </summary>
        public static UInt16 TIM_CCER_CC4P = 0x2000;

        /// <summary>
        ///  Capture/Compare 4 Complementary output Polarity 
        /// </summary>
        public static UInt16 TIM_CCER_CC4NP = 0x8000;


        /*******************  Bit definition for TIM_CNT register  ********************/
        /// <summary>
        ///  Counter Value 
        /// </summary>
        public static UInt16 TIM_CNT_CNT = 0xFFFF;


        /*******************  Bit definition for TIM_PSC register  ********************/
        /// <summary>
        ///  Prescaler Value 
        /// </summary>
        public static UInt16 TIM_PSC_PSC = 0xFFFF;


        /*******************  Bit definition for TIM_ARR register  ********************/
        /// <summary>
        ///  actual auto-reload Value 
        /// </summary>
        public static UInt16 TIM_ARR_ARR = 0xFFFF;


        /*******************  Bit definition for TIM_RCR register  ********************/
        /// <summary>
        ///  Repetition Counter Value 
        /// </summary>
        public static byte TIM_RCR_REP = 0xFF;


        /*******************  Bit definition for TIM_CCR1 register  *******************/
        /// <summary>
        ///  Capture/Compare 1 Value 
        /// </summary>
        public static UInt16 TIM_CCR1_CCR1 = 0xFFFF;


        /*******************  Bit definition for TIM_CCR2 register  *******************/
        /// <summary>
        ///  Capture/Compare 2 Value 
        /// </summary>
        public static UInt16 TIM_CCR2_CCR2 = 0xFFFF;


        /*******************  Bit definition for TIM_CCR3 register  *******************/
        /// <summary>
        ///  Capture/Compare 3 Value 
        /// </summary>
        public static UInt16 TIM_CCR3_CCR3 = 0xFFFF;


        /*******************  Bit definition for TIM_CCR4 register  *******************/
        /// <summary>
        ///  Capture/Compare 4 Value 
        /// </summary>
        public static UInt16 TIM_CCR4_CCR4 = 0xFFFF;


        /*******************  Bit definition for TIM_BDTR register  *******************/
        /// <summary>
        ///  DTG[0:7] bits (Dead-Time Generator set-up) 
        /// </summary>
        public static UInt16 TIM_BDTR_DTG = 0x00FF;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_BDTR_DTG_0 = 0x0001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_BDTR_DTG_1 = 0x0002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt16 TIM_BDTR_DTG_2 = 0x0004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt16 TIM_BDTR_DTG_3 = 0x0008;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt16 TIM_BDTR_DTG_4 = 0x0010;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt16 TIM_BDTR_DTG_5 = 0x0020;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt16 TIM_BDTR_DTG_6 = 0x0040;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt16 TIM_BDTR_DTG_7 = 0x0080;


        /// <summary>
        ///  LOCK[1:0] bits (Lock Configuration) 
        /// </summary>
        public static UInt16 TIM_BDTR_LOCK = 0x0300;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_BDTR_LOCK_0 = 0x0100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_BDTR_LOCK_1 = 0x0200;


        /// <summary>
        ///  Off-State Selection for Idle mode 
        /// </summary>
        public static UInt16 TIM_BDTR_OSSI = 0x0400;

        /// <summary>
        ///  Off-State Selection for Run mode 
        /// </summary>
        public static UInt16 TIM_BDTR_OSSR = 0x0800;

        /// <summary>
        ///  Break enable 
        /// </summary>
        public static UInt16 TIM_BDTR_BKE = 0x1000;

        /// <summary>
        ///  Break Polarity 
        /// </summary>
        public static UInt16 TIM_BDTR_BKP = 0x2000;

        /// <summary>
        ///  Automatic Output enable 
        /// </summary>
        public static UInt16 TIM_BDTR_AOE = 0x4000;

        /// <summary>
        ///  Main Output enable 
        /// </summary>
        public static UInt16 TIM_BDTR_MOE = 0x8000;


        /*******************  Bit definition for TIM_DCR register  ********************/
        /// <summary>
        ///  DBA[4:0] bits (DMA Base Address) 
        /// </summary>
        public static UInt16 TIM_DCR_DBA = 0x001F;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_DCR_DBA_0 = 0x0001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_DCR_DBA_1 = 0x0002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt16 TIM_DCR_DBA_2 = 0x0004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt16 TIM_DCR_DBA_3 = 0x0008;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt16 TIM_DCR_DBA_4 = 0x0010;


        /// <summary>
        ///  DBL[4:0] bits (DMA Burst Length) 
        /// </summary>
        public static UInt16 TIM_DCR_DBL = 0x1F00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_DCR_DBL_0 = 0x0100;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_DCR_DBL_1 = 0x0200;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt16 TIM_DCR_DBL_2 = 0x0400;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt16 TIM_DCR_DBL_3 = 0x0800;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt16 TIM_DCR_DBL_4 = 0x1000;


        /*******************  Bit definition for TIM_DMAR register  *******************/
        /// <summary>
        ///  DMA register for burst accesses 
        /// </summary>
        public static UInt16 TIM_DMAR_DMAB = 0xFFFF;


        /*******************  Bit definition for TIM_OR register  *********************/
        /// <summary>
        ///  TI4_RMP[1:0] bits (TIM5 Input 4 remap) 
        /// </summary>
        public static UInt16 TIM_OR_TI4_RMP = 0x00C0;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_OR_TI4_RMP_0 = 0x0040;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_OR_TI4_RMP_1 = 0x0080;

        /// <summary>
        ///  ITR1_RMP[1:0] bits (TIM2 Internal trigger 1 remap) 
        /// </summary>
        public static UInt16 TIM_OR_ITR1_RMP = 0x0C00;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 TIM_OR_ITR1_RMP_0 = 0x0400;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 TIM_OR_ITR1_RMP_1 = 0x0800;



        /******************************************************************************/
        /*                                                                            */
        /*         Universal Synchronous Asynchronous Receiver Transmitter            */
        /*                                                                            */
        /******************************************************************************/
        /*******************  Bit definition for USART_SR register  *******************/
        /// <summary>
        ///  Parity Error 
        /// </summary>
        public static UInt16 USART_SR_PE = 0x0001;

        /// <summary>
        ///  Framing Error 
        /// </summary>
        public static UInt16 USART_SR_FE = 0x0002;

        /// <summary>
        ///  Noise Error Flag 
        /// </summary>
        public static UInt16 USART_SR_NE = 0x0004;

        /// <summary>
        ///  OverRun Error 
        /// </summary>
        public static UInt16 USART_SR_ORE = 0x0008;

        /// <summary>
        ///  IDLE line detected 
        /// </summary>
        public static UInt16 USART_SR_IDLE = 0x0010;

        /// <summary>
        ///  Read Data Register Not Empty 
        /// </summary>
        public static UInt16 USART_SR_RXNE = 0x0020;

        /// <summary>
        ///  Transmission Complete 
        /// </summary>
        public static UInt16 USART_SR_TC = 0x0040;

        /// <summary>
        ///  Transmit Data Register Empty 
        /// </summary>
        public static UInt16 USART_SR_TXE = 0x0080;

        /// <summary>
        ///  LIN Break Detection Flag 
        /// </summary>
        public static UInt16 USART_SR_LBD = 0x0100;

        /// <summary>
        ///  CTS Flag 
        /// </summary>
        public static UInt16 USART_SR_CTS = 0x0200;


        /*******************  Bit definition for USART_DR register  *******************/
        /// <summary>
        ///  Data value 
        /// </summary>
        public static UInt16 USART_DR_DR = 0x01FF;


        /******************  Bit definition for USART_BRR register  *******************/
        /// <summary>
        ///  Fraction of USARTDIV 
        /// </summary>
        public static UInt16 USART_BRR_DIV_Fraction = 0x000F;

        /// <summary>
        ///  Mantissa of USARTDIV 
        /// </summary>
        public static UInt16 USART_BRR_DIV_Mantissa = 0xFFF0;


        /******************  Bit definition for USART_CR1 register  *******************/
        /// <summary>
        ///  Send Break 
        /// </summary>
        public static UInt16 USART_CR1_SBK = 0x0001;

        /// <summary>
        ///  Receiver wakeup 
        /// </summary>
        public static UInt16 USART_CR1_RWU = 0x0002;

        /// <summary>
        ///  Receiver Enable 
        /// </summary>
        public static UInt16 USART_CR1_RE = 0x0004;

        /// <summary>
        ///  Transmitter Enable 
        /// </summary>
        public static UInt16 USART_CR1_TE = 0x0008;

        /// <summary>
        ///  IDLE Interrupt Enable 
        /// </summary>
        public static UInt16 USART_CR1_IDLEIE = 0x0010;

        /// <summary>
        ///  RXNE Interrupt Enable 
        /// </summary>
        public static UInt16 USART_CR1_RXNEIE = 0x0020;

        /// <summary>
        ///  Transmission Complete Interrupt Enable 
        /// </summary>
        public static UInt16 USART_CR1_TCIE = 0x0040;

        /// <summary>
        ///  PE Interrupt Enable 
        /// </summary>
        public static UInt16 USART_CR1_TXEIE = 0x0080;

        /// <summary>
        ///  PE Interrupt Enable 
        /// </summary>
        public static UInt16 USART_CR1_PEIE = 0x0100;

        /// <summary>
        ///  Parity Selection 
        /// </summary>
        public static UInt16 USART_CR1_PS = 0x0200;

        /// <summary>
        ///  Parity Control Enable 
        /// </summary>
        public static UInt16 USART_CR1_PCE = 0x0400;

        /// <summary>
        ///  Wakeup method 
        /// </summary>
        public static UInt16 USART_CR1_WAKE = 0x0800;

        /// <summary>
        ///  Word length 
        /// </summary>
        public static UInt16 USART_CR1_M = 0x1000;

        /// <summary>
        ///  USART Enable 
        /// </summary>
        public static UInt16 USART_CR1_UE = 0x2000;

        /// <summary>
        ///  USART Oversampling by 8 enable 
        /// </summary>
        public static UInt16 USART_CR1_OVER8 = 0x8000;


        /******************  Bit definition for USART_CR2 register  *******************/
        /// <summary>
        ///  Address of the USART node 
        /// </summary>
        public static UInt16 USART_CR2_ADD = 0x000F;

        /// <summary>
        ///  LIN Break Detection Length 
        /// </summary>
        public static UInt16 USART_CR2_LBDL = 0x0020;

        /// <summary>
        ///  LIN Break Detection Interrupt Enable 
        /// </summary>
        public static UInt16 USART_CR2_LBDIE = 0x0040;

        /// <summary>
        ///  Last Bit Clock pulse 
        /// </summary>
        public static UInt16 USART_CR2_LBCL = 0x0100;

        /// <summary>
        ///  Clock Phase 
        /// </summary>
        public static UInt16 USART_CR2_CPHA = 0x0200;

        /// <summary>
        ///  Clock Polarity 
        /// </summary>
        public static UInt16 USART_CR2_CPOL = 0x0400;

        /// <summary>
        ///  Clock Enable 
        /// </summary>
        public static UInt16 USART_CR2_CLKEN = 0x0800;


        /// <summary>
        ///  STOP[1:0] bits (STOP bits) 
        /// </summary>
        public static UInt16 USART_CR2_STOP = 0x3000;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 USART_CR2_STOP_0 = 0x1000;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 USART_CR2_STOP_1 = 0x2000;


        /// <summary>
        ///  LIN mode enable 
        /// </summary>
        public static UInt16 USART_CR2_LINEN = 0x4000;


        /******************  Bit definition for USART_CR3 register  *******************/
        /// <summary>
        ///  Error Interrupt Enable 
        /// </summary>
        public static UInt16 USART_CR3_EIE = 0x0001;

        /// <summary>
        ///  IrDA mode Enable 
        /// </summary>
        public static UInt16 USART_CR3_IREN = 0x0002;

        /// <summary>
        ///  IrDA Low-Power 
        /// </summary>
        public static UInt16 USART_CR3_IRLP = 0x0004;

        /// <summary>
        ///  Half-Duplex Selection 
        /// </summary>
        public static UInt16 USART_CR3_HDSEL = 0x0008;

        /// <summary>
        ///  Smartcard NACK enable 
        /// </summary>
        public static UInt16 USART_CR3_NACK = 0x0010;

        /// <summary>
        ///  Smartcard mode enable 
        /// </summary>
        public static UInt16 USART_CR3_SCEN = 0x0020;

        /// <summary>
        ///  DMA Enable Receiver 
        /// </summary>
        public static UInt16 USART_CR3_DMAR = 0x0040;

        /// <summary>
        ///  DMA Enable Transmitter 
        /// </summary>
        public static UInt16 USART_CR3_DMAT = 0x0080;

        /// <summary>
        ///  RTS Enable 
        /// </summary>
        public static UInt16 USART_CR3_RTSE = 0x0100;

        /// <summary>
        ///  CTS Enable 
        /// </summary>
        public static UInt16 USART_CR3_CTSE = 0x0200;

        /// <summary>
        ///  CTS Interrupt Enable 
        /// </summary>
        public static UInt16 USART_CR3_CTSIE = 0x0400;

        /// <summary>
        ///  USART One bit method enable 
        /// </summary>
        public static UInt16 USART_CR3_ONEBIT = 0x0800;


        /******************  Bit definition for USART_GTPR register  ******************/
        /// <summary>
        ///  PSC[7:0] bits (Prescaler value) 
        /// </summary>
        public static UInt16 USART_GTPR_PSC = 0x00FF;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 USART_GTPR_PSC_0 = 0x0001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 USART_GTPR_PSC_1 = 0x0002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt16 USART_GTPR_PSC_2 = 0x0004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt16 USART_GTPR_PSC_3 = 0x0008;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt16 USART_GTPR_PSC_4 = 0x0010;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt16 USART_GTPR_PSC_5 = 0x0020;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt16 USART_GTPR_PSC_6 = 0x0040;

        /// <summary>
        ///  Bit 7 
        /// </summary>
        public static UInt16 USART_GTPR_PSC_7 = 0x0080;


        /// <summary>
        ///  Guard time value 
        /// </summary>
        public static UInt16 USART_GTPR_GT = 0xFF00;


        /******************************************************************************/
        /*                                                                            */
        /*                            Window WATCHDOG                                 */
        /*                                                                            */
        /******************************************************************************/
        /*******************  Bit definition for WWDG_CR register  ********************/
        /// <summary>
        ///  T[6:0] bits (7-Bit counter (MSB to LSB)) 
        /// </summary>
        public static byte WWDG_CR_T = 0x7F;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static byte WWDG_CR_T0 = 0x01;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static byte WWDG_CR_T1 = 0x02;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static byte WWDG_CR_T2 = 0x04;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static byte WWDG_CR_T3 = 0x08;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static byte WWDG_CR_T4 = 0x10;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static byte WWDG_CR_T5 = 0x20;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static byte WWDG_CR_T6 = 0x40;


        /// <summary>
        ///  Activation bit 
        /// </summary>
        public static byte WWDG_CR_WDGA = 0x80;


        /*******************  Bit definition for WWDG_CFR register  *******************/
        /// <summary>
        ///  W[6:0] bits (7-bit window value) 
        /// </summary>
        public static UInt16 WWDG_CFR_W = 0x007F;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 WWDG_CFR_W0 = 0x0001;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 WWDG_CFR_W1 = 0x0002;

        /// <summary>
        ///  Bit 2 
        /// </summary>
        public static UInt16 WWDG_CFR_W2 = 0x0004;

        /// <summary>
        ///  Bit 3 
        /// </summary>
        public static UInt16 WWDG_CFR_W3 = 0x0008;

        /// <summary>
        ///  Bit 4 
        /// </summary>
        public static UInt16 WWDG_CFR_W4 = 0x0010;

        /// <summary>
        ///  Bit 5 
        /// </summary>
        public static UInt16 WWDG_CFR_W5 = 0x0020;

        /// <summary>
        ///  Bit 6 
        /// </summary>
        public static UInt16 WWDG_CFR_W6 = 0x0040;


        /// <summary>
        ///  WDGTB[1:0] bits (Timer Base) 
        /// </summary>
        public static UInt16 WWDG_CFR_WDGTB = 0x0180;

        /// <summary>
        ///  Bit 0 
        /// </summary>
        public static UInt16 WWDG_CFR_WDGTB0 = 0x0080;

        /// <summary>
        ///  Bit 1 
        /// </summary>
        public static UInt16 WWDG_CFR_WDGTB1 = 0x0100;


        /// <summary>
        ///  Early Wakeup Interrupt 
        /// </summary>
        public static UInt16 WWDG_CFR_EWI = 0x0200;


        /*******************  Bit definition for WWDG_SR register  ********************/
        /// <summary>
        ///  Early Wakeup Interrupt Flag 
        /// </summary>
        public static byte WWDG_SR_EWIF = 0x01;



        /******************************************************************************/
        /*                                                                            */
        /*                                DBG                                         */
        /*                                                                            */
        /******************************************************************************/
        /********************  Bit definition for DBGMCU_IDCODE register  *************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_IDCODE_DEV_ID = 0x00000FFF;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_IDCODE_REV_ID = 0xFFFF0000;


        /********************  Bit definition for DBGMCU_CR register  *****************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_CR_DBG_SLEEP = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_CR_DBG_STOP = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_CR_DBG_STANDBY = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_CR_TRACE_IOEN = 0x00000020;


        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_CR_TRACE_MODE = 0x000000C0;

        /// <summary>
        /// Bit 0 
        /// </summary>
        public static UInt32 DBGMCU_CR_TRACE_MODE_0 = 0x00000040;

        /// <summary>
        /// Bit 1 
        /// </summary>
        public static UInt32 DBGMCU_CR_TRACE_MODE_1 = 0x00000080;


        /********************  Bit definition for DBGMCU_APB1_FZ register  ************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_TIM2_STOP = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_TIM3_STOP = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_TIM4_STOP = 0x00000004;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_TIM5_STOP = 0x00000008;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_TIM6_STOP = 0x00000010;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_TIM7_STOP = 0x00000020;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_TIM12_STOP = 0x00000040;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_TIM13_STOP = 0x00000080;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_TIM14_STOP = 0x00000100;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_RTC_STOP = 0x00000400;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_WWDG_STOP = 0x00000800;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_IWDG_STOP = 0x00001000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_I2C1_SMBUS_TIMEOUT = 0x00200000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_I2C2_SMBUS_TIMEOUT = 0x00400000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_I2C3_SMBUS_TIMEOUT = 0x00800000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_CAN1_STOP = 0x02000000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_CAN2_STOP = 0x04000000;

        /* Old IWDGSTOP bit definition, maintained for legacy purpose */
        /// <summary>
        /// 
        /// </summary>
        public UInt32 DBGMCU_APB1_FZ_DBG_IWDEG_STOP = DBGMCU_APB1_FZ_DBG_IWDG_STOP;

        /********************  Bit definition for DBGMCU_APB2_FZ register  ************/
        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_TIM1_STOP = 0x00000001;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_TIM8_STOP = 0x00000002;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_TIM9_STOP = 0x00010000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_TIM10_STOP = 0x00020000;

        /// <summary>
        /// 
        /// </summary>
        public static UInt32 DBGMCU_APB1_FZ_DBG_TIM11_STOP = 0x00040000;


        /******************************************************************************/
        /*                                                                            */
        /*                Ethernet MAC Registers bits definitions                     */
        /*                                                                            */
        /******************************************************************************/
        /* Bit definition for Ethernet MAC Control Register register */
        /// <summary>
        ///   Watchdog disable 
        /// </summary>
        public static UInt32 ETH_MACCR_WD = 0x00800000;

        /// <summary>
        ///   Jabber disable 
        /// </summary>
        public static UInt32 ETH_MACCR_JD = 0x00400000;

        /// <summary>
        ///   Inter-frame gap 
        /// </summary>
        public static UInt32 ETH_MACCR_IFG = 0x000E0000;

        /// <summary>
        ///   Minimum IFG between frames during transmission is 96Bit 
        /// </summary>
        public static UInt32 ETH_MACCR_IFG_96Bit = 0x00000000;

        /// <summary>
        ///   Minimum IFG between frames during transmission is 88Bit 
        /// </summary>
        public static UInt32 ETH_MACCR_IFG_88Bit = 0x00020000;

        /// <summary>
        ///   Minimum IFG between frames during transmission is 80Bit 
        /// </summary>
        public static UInt32 ETH_MACCR_IFG_80Bit = 0x00040000;

        /// <summary>
        ///   Minimum IFG between frames during transmission is 72Bit 
        /// </summary>
        public static UInt32 ETH_MACCR_IFG_72Bit = 0x00060000;

        /// <summary>
        ///   Minimum IFG between frames during transmission is 64Bit         
        /// </summary>
        public static UInt32 ETH_MACCR_IFG_64Bit = 0x00080000;

        /// <summary>
        ///   Minimum IFG between frames during transmission is 56Bit 
        /// </summary>
        public static UInt32 ETH_MACCR_IFG_56Bit = 0x000A0000;

        /// <summary>
        ///   Minimum IFG between frames during transmission is 48Bit 
        /// </summary>
        public static UInt32 ETH_MACCR_IFG_48Bit = 0x000C0000;

        /// <summary>
        ///   Minimum IFG between frames during transmission is 40Bit               
        /// </summary>
        public static UInt32 ETH_MACCR_IFG_40Bit = 0x000E0000;

        /// <summary>
        ///   Carrier sense disable (during transmission) 
        /// </summary>
        public static UInt32 ETH_MACCR_CSD = 0x00010000;

        /// <summary>
        ///   Fast ethernet speed 
        /// </summary>
        public static UInt32 ETH_MACCR_FES = 0x00004000;

        /// <summary>
        ///   Receive own disable 
        /// </summary>
        public static UInt32 ETH_MACCR_ROD = 0x00002000;

        /// <summary>
        ///   loopback mode 
        /// </summary>
        public static UInt32 ETH_MACCR_LM = 0x00001000;

        /// <summary>
        ///   Duplex mode 
        /// </summary>
        public static UInt32 ETH_MACCR_DM = 0x00000800;

        /// <summary>
        ///   IP Checksum offload 
        /// </summary>
        public static UInt32 ETH_MACCR_IPCO = 0x00000400;

        /// <summary>
        ///   Retry disable 
        /// </summary>
        public static UInt32 ETH_MACCR_RD = 0x00000200;

        /// <summary>
        ///   Automatic Pad/CRC stripping 
        /// </summary>
        public static UInt32 ETH_MACCR_APCS = 0x00000080;

        /// <summary>
        ///   Back-off limit: random integer number (r) of slot time delays before rescheduling a transmission attempt during retries after a collision: 0 =< r <2^k */
        /// </summary>
        public static UInt32 ETH_MACCR_BL = 0x00000060;

        /// <summary>
        ///   k = min (n, 10) 
        /// </summary>
        public static UInt32 ETH_MACCR_BL_10 = 0x00000000;

        /// <summary>
        ///   k = min (n, 8) 
        /// </summary>
        public static UInt32 ETH_MACCR_BL_8 = 0x00000020;

        /// <summary>
        ///   k = min (n, 4) 
        /// </summary>
        public static UInt32 ETH_MACCR_BL_4 = 0x00000040;

        /// <summary>
        ///   k = min (n, 1)  
        /// </summary>
        public static UInt32 ETH_MACCR_BL_1 = 0x00000060;

        /// <summary>
        ///   Defferal check 
        /// </summary>
        public static UInt32 ETH_MACCR_DC = 0x00000010;

        /// <summary>
        ///   Transmitter enable 
        /// </summary>
        public static UInt32 ETH_MACCR_TE = 0x00000008;

        /// <summary>
        ///   Receiver enable 
        /// </summary>
        public static UInt32 ETH_MACCR_RE = 0x00000004;


        /* Bit definition for Ethernet MAC Frame Filter Register */
        /// <summary>
        ///   Receive all  
        /// </summary>
        public static UInt32 ETH_MACFFR_RA = 0x80000000;

        /// <summary>
        ///   Hash or perfect filter  
        /// </summary>
        public static UInt32 ETH_MACFFR_HPF = 0x00000400;

        /// <summary>
        ///   Source address filter enable  
        /// </summary>
        public static UInt32 ETH_MACFFR_SAF = 0x00000200;

        /// <summary>
        ///   SA inverse filtering  
        /// </summary>
        public static UInt32 ETH_MACFFR_SAIF = 0x00000100;

        /// <summary>
        ///   Pass control frames: 3 cases 
        /// </summary>
        public static UInt32 ETH_MACFFR_PCF = 0x000000C0;

        /// <summary>
        ///   MAC filters all control frames from reaching the application 
        /// </summary>
        public static UInt32 ETH_MACFFR_PCF_BlockAll = 0x00000040;

        /// <summary>
        ///   MAC forwards all control frames to application even if they fail the Address Filter 
        /// </summary>
        public static UInt32 ETH_MACFFR_PCF_ForwardAll = 0x00000080;

        /// <summary>
        ///   MAC forwards control frames that pass the Address Filter.  
        /// </summary>
        public static UInt32 ETH_MACFFR_PCF_ForwardPassedAddrFilter = 0x000000C0;

        /// <summary>
        ///   Broadcast frame disable  
        /// </summary>
        public static UInt32 ETH_MACFFR_BFD = 0x00000020;

        /// <summary>
        ///   Pass all mutlicast  
        /// </summary>
        public static UInt32 ETH_MACFFR_PAM = 0x00000010;

        /// <summary>
        ///   DA Inverse filtering  
        /// </summary>
        public static UInt32 ETH_MACFFR_DAIF = 0x00000008;

        /// <summary>
        ///   Hash multicast  
        /// </summary>
        public static UInt32 ETH_MACFFR_HM = 0x00000004;

        /// <summary>
        ///   Hash unicast 
        /// </summary>
        public static UInt32 ETH_MACFFR_HU = 0x00000002;

        /// <summary>
        ///   Promiscuous mode 
        /// </summary>
        public static UInt32 ETH_MACFFR_PM = 0x00000001;


        /* Bit definition for Ethernet MAC Hash Table High Register */
        /// <summary>
        ///   Hash table high 
        /// </summary>
        public static UInt32 ETH_MACHTHR_HTH = 0xFFFFFFFF;


        /* Bit definition for Ethernet MAC Hash Table Low Register */
        /// <summary>
        ///   Hash table low 
        /// </summary>
        public static UInt32 ETH_MACHTLR_HTL = 0xFFFFFFFF;


        /* Bit definition for Ethernet MAC MII Address Register */
        /// <summary>
        ///   Physical layer address  
        /// </summary>
        public static UInt32 ETH_MACMIIAR_PA = 0x0000F800;

        /// <summary>
        ///   MII register in the selected PHY  
        /// </summary>
        public static UInt32 ETH_MACMIIAR_MR = 0x000007C0;

        /// <summary>
        ///   CR clock range: 6 cases  
        /// </summary>
        public static UInt32 ETH_MACMIIAR_CR = 0x0000001C;

        /// <summary>
        ///   HCLK:60-100 MHz; MDC clock= HCLK/42 
        /// </summary>
        public static UInt32 ETH_MACMIIAR_CR_Div42 = 0x00000000;

        /// <summary>
        ///   HCLK:100-150 MHz; MDC clock= HCLK/62 
        /// </summary>
        public static UInt32 ETH_MACMIIAR_CR_Div62 = 0x00000004;

        /// <summary>
        ///   HCLK:20-35 MHz; MDC clock= HCLK/16 
        /// </summary>
        public static UInt32 ETH_MACMIIAR_CR_Div16 = 0x00000008;

        /// <summary>
        ///   HCLK:35-60 MHz; MDC clock= HCLK/26 
        /// </summary>
        public static UInt32 ETH_MACMIIAR_CR_Div26 = 0x0000000C;

        /// <summary>
        ///   HCLK:150-168 MHz; MDC clock= HCLK/102   
        /// </summary>
        public static UInt32 ETH_MACMIIAR_CR_Div102 = 0x00000010;

        /// <summary>
        ///   MII write  
        /// </summary>
        public static UInt32 ETH_MACMIIAR_MW = 0x00000002;

        /// <summary>
        ///   MII busy  
        /// </summary>
        public static UInt32 ETH_MACMIIAR_MB = 0x00000001;


        /* Bit definition for Ethernet MAC MII Data Register */
        /// <summary>
        ///   MII data: read/write data from/to PHY 
        /// </summary>
        public static UInt32 ETH_MACMIIDR_MD = 0x0000FFFF;


        /* Bit definition for Ethernet MAC Flow Control Register */
        /// <summary>
        ///   Pause time 
        /// </summary>
        public static UInt32 ETH_MACFCR_PT = 0xFFFF0000;

        /// <summary>
        ///   Zero-quanta pause disable 
        /// </summary>
        public static UInt32 ETH_MACFCR_ZQPD = 0x00000080;

        /// <summary>
        ///   Pause low threshold: 4 cases 
        /// </summary>
        public static UInt32 ETH_MACFCR_PLT = 0x00000030;

        /// <summary>
        ///   Pause time minus 4 slot times 
        /// </summary>
        public static UInt32 ETH_MACFCR_PLT_Minus4 = 0x00000000;

        /// <summary>
        ///   Pause time minus 28 slot times 
        /// </summary>
        public static UInt32 ETH_MACFCR_PLT_Minus28 = 0x00000010;

        /// <summary>
        ///   Pause time minus 144 slot times 
        /// </summary>
        public static UInt32 ETH_MACFCR_PLT_Minus144 = 0x00000020;

        /// <summary>
        ///   Pause time minus 256 slot times       
        /// </summary>
        public static UInt32 ETH_MACFCR_PLT_Minus256 = 0x00000030;

        /// <summary>
        ///   Unicast pause frame detect 
        /// </summary>
        public static UInt32 ETH_MACFCR_UPFD = 0x00000008;

        /// <summary>
        ///   Receive flow control enable 
        /// </summary>
        public static UInt32 ETH_MACFCR_RFCE = 0x00000004;

        /// <summary>
        ///   Transmit flow control enable 
        /// </summary>
        public static UInt32 ETH_MACFCR_TFCE = 0x00000002;

        /// <summary>
        ///   Flow control busy/backpressure activate 
        /// </summary>
        public static UInt32 ETH_MACFCR_FCBBPA = 0x00000001;


        /* Bit definition for Ethernet MAC VLAN Tag Register */
        /// <summary>
        ///   12-bit VLAN tag comparison 
        /// </summary>
        public static UInt32 ETH_MACVLANTR_VLANTC = 0x00010000;

        /// <summary>
        ///   VLAN tag identifier (for receive frames) 
        /// </summary>
        public static UInt32 ETH_MACVLANTR_VLANTI = 0x0000FFFF;


        /* Bit definition for Ethernet MAC Remote Wake-UpFrame Filter Register */
        /// <summary>
        ///   Wake-up frame filter register data 
        /// </summary>
        public static UInt32 ETH_MACRWUFFR_D = 0xFFFFFFFF;

        /* Eight sequential Writes to this address (offset 0x28) will write all Wake-UpFrame Filter Registers.
           Eight sequential Reads from this address (offset 0x28) will read all Wake-UpFrame Filter Registers. */
        /* Wake-UpFrame Filter Reg0 : Filter 0 Byte Mask
           Wake-UpFrame Filter Reg1 : Filter 1 Byte Mask
           Wake-UpFrame Filter Reg2 : Filter 2 Byte Mask
           Wake-UpFrame Filter Reg3 : Filter 3 Byte Mask
           Wake-UpFrame Filter Reg4 : RSVD - Filter3 Command - RSVD - Filter2 Command - 
                                      RSVD - Filter1 Command - RSVD - Filter0 Command
           Wake-UpFrame Filter Re5 : Filter3 Offset - Filter2 Offset - Filter1 Offset - Filter0 Offset
           Wake-UpFrame Filter Re6 : Filter1 CRC16 - Filter0 CRC16
           Wake-UpFrame Filter Re7 : Filter3 CRC16 - Filter2 CRC16 */

        /* Bit definition for Ethernet MAC PMT Control and Status Register */
        /// <summary>
        ///   Wake-Up Frame Filter Register Pointer Reset 
        /// </summary>
        public static UInt32 ETH_MACPMTCSR_WFFRPR = 0x80000000;

        /// <summary>
        ///   Global Unicast 
        /// </summary>
        public static UInt32 ETH_MACPMTCSR_GU = 0x00000200;

        /// <summary>
        ///   Wake-Up Frame Received 
        /// </summary>
        public static UInt32 ETH_MACPMTCSR_WFR = 0x00000040;

        /// <summary>
        ///   Magic Packet Received 
        /// </summary>
        public static UInt32 ETH_MACPMTCSR_MPR = 0x00000020;

        /// <summary>
        ///   Wake-Up Frame Enable 
        /// </summary>
        public static UInt32 ETH_MACPMTCSR_WFE = 0x00000004;

        /// <summary>
        ///   Magic Packet Enable 
        /// </summary>
        public static UInt32 ETH_MACPMTCSR_MPE = 0x00000002;

        /// <summary>
        ///   Power Down 
        /// </summary>
        public static UInt32 ETH_MACPMTCSR_PD = 0x00000001;


        /* Bit definition for Ethernet MAC Status Register */
        /// <summary>
        ///   Time stamp trigger status 
        /// </summary>
        public static UInt32 ETH_MACSR_TSTS = 0x00000200;

        /// <summary>
        ///   MMC transmit status 
        /// </summary>
        public static UInt32 ETH_MACSR_MMCTS = 0x00000040;

        /// <summary>
        ///   MMC receive status 
        /// </summary>
        public static UInt32 ETH_MACSR_MMMCRS = 0x00000020;

        /// <summary>
        ///   MMC status 
        /// </summary>
        public static UInt32 ETH_MACSR_MMCS = 0x00000010;

        /// <summary>
        ///   PMT status 
        /// </summary>
        public static UInt32 ETH_MACSR_PMTS = 0x00000008;


        /* Bit definition for Ethernet MAC Interrupt Mask Register */
        /// <summary>
        ///   Time stamp trigger interrupt mask 
        /// </summary>
        public static UInt32 ETH_MACIMR_TSTIM = 0x00000200;

        /// <summary>
        ///   PMT interrupt mask 
        /// </summary>
        public static UInt32 ETH_MACIMR_PMTIM = 0x00000008;


        /* Bit definition for Ethernet MAC Address0 High Register */
        /// <summary>
        ///   MAC address0 high 
        /// </summary>
        public static UInt32 ETH_MACA0HR_MACA0H = 0x0000FFFF;


        /* Bit definition for Ethernet MAC Address0 Low Register */
        /// <summary>
        ///   MAC address0 low 
        /// </summary>
        public static UInt32 ETH_MACA0LR_MACA0L = 0xFFFFFFFF;


        /* Bit definition for Ethernet MAC Address1 High Register */
        /// <summary>
        ///   Address enable 
        /// </summary>
        public static UInt32 ETH_MACA1HR_AE = 0x80000000;

        /// <summary>
        ///   Source address 
        /// </summary>
        public static UInt32 ETH_MACA1HR_SA = 0x40000000;

        /// <summary>
        ///   Mask byte control: bits to mask for comparison of the MAC Address bytes 
        /// </summary>
        public static UInt32 ETH_MACA1HR_MBC = 0x3F000000;

        /// <summary>
        ///   Mask MAC Address high reg bits [15:8] 
        /// </summary>
        public static UInt32 ETH_MACA1HR_MBC_HBits15_8 = 0x20000000;

        /// <summary>
        ///   Mask MAC Address high reg bits [7:0] 
        /// </summary>
        public static UInt32 ETH_MACA1HR_MBC_HBits7_0 = 0x10000000;

        /// <summary>
        ///   Mask MAC Address low reg bits [31:24] 
        /// </summary>
        public static UInt32 ETH_MACA1HR_MBC_LBits31_24 = 0x08000000;

        /// <summary>
        ///   Mask MAC Address low reg bits [23:16] 
        /// </summary>
        public static UInt32 ETH_MACA1HR_MBC_LBits23_16 = 0x04000000;

        /// <summary>
        ///   Mask MAC Address low reg bits [15:8] 
        /// </summary>
        public static UInt32 ETH_MACA1HR_MBC_LBits15_8 = 0x02000000;

        /// <summary>
        ///   Mask MAC Address low reg bits [7:0]  
        /// </summary>
        public static UInt32 ETH_MACA1HR_MBC_LBits7_0 = 0x01000000;

        /// <summary>
        ///   MAC address1 high 
        /// </summary>
        public static UInt32 ETH_MACA1HR_MACA1H = 0x0000FFFF;


        /* Bit definition for Ethernet MAC Address1 Low Register */
        /// <summary>
        ///   MAC address1 low 
        /// </summary>
        public static UInt32 ETH_MACA1LR_MACA1L = 0xFFFFFFFF;


        /* Bit definition for Ethernet MAC Address2 High Register */
        /// <summary>
        ///   Address enable 
        /// </summary>
        public static UInt32 ETH_MACA2HR_AE = 0x80000000;

        /// <summary>
        ///   Source address 
        /// </summary>
        public static UInt32 ETH_MACA2HR_SA = 0x40000000;

        /// <summary>
        ///   Mask byte control 
        /// </summary>
        public static UInt32 ETH_MACA2HR_MBC = 0x3F000000;

        /// <summary>
        ///   Mask MAC Address high reg bits [15:8] 
        /// </summary>
        public static UInt32 ETH_MACA2HR_MBC_HBits15_8 = 0x20000000;

        /// <summary>
        ///   Mask MAC Address high reg bits [7:0] 
        /// </summary>
        public static UInt32 ETH_MACA2HR_MBC_HBits7_0 = 0x10000000;

        /// <summary>
        ///   Mask MAC Address low reg bits [31:24] 
        /// </summary>
        public static UInt32 ETH_MACA2HR_MBC_LBits31_24 = 0x08000000;

        /// <summary>
        ///   Mask MAC Address low reg bits [23:16] 
        /// </summary>
        public static UInt32 ETH_MACA2HR_MBC_LBits23_16 = 0x04000000;

        /// <summary>
        ///   Mask MAC Address low reg bits [15:8] 
        /// </summary>
        public static UInt32 ETH_MACA2HR_MBC_LBits15_8 = 0x02000000;

        /// <summary>
        ///   Mask MAC Address low reg bits [70] 
        /// </summary>
        public static UInt32 ETH_MACA2HR_MBC_LBits7_0 = 0x01000000;

        /// <summary>
        ///   MAC address1 high 
        /// </summary>
        public static UInt32 ETH_MACA2HR_MACA2H = 0x0000FFFF;


        /* Bit definition for Ethernet MAC Address2 Low Register */
        /// <summary>
        ///   MAC address2 low 
        /// </summary>
        public static UInt32 ETH_MACA2LR_MACA2L = 0xFFFFFFFF;


        /* Bit definition for Ethernet MAC Address3 High Register */
        /// <summary>
        ///   Address enable 
        /// </summary>
        public static UInt32 ETH_MACA3HR_AE = 0x80000000;

        /// <summary>
        ///   Source address 
        /// </summary>
        public static UInt32 ETH_MACA3HR_SA = 0x40000000;

        /// <summary>
        ///   Mask byte control 
        /// </summary>
        public static UInt32 ETH_MACA3HR_MBC = 0x3F000000;

        /// <summary>
        ///   Mask MAC Address high reg bits [15:8] 
        /// </summary>
        public static UInt32 ETH_MACA3HR_MBC_HBits15_8 = 0x20000000;

        /// <summary>
        ///   Mask MAC Address high reg bits [7:0] 
        /// </summary>
        public static UInt32 ETH_MACA3HR_MBC_HBits7_0 = 0x10000000;

        /// <summary>
        ///   Mask MAC Address low reg bits [31:24] 
        /// </summary>
        public static UInt32 ETH_MACA3HR_MBC_LBits31_24 = 0x08000000;

        /// <summary>
        ///   Mask MAC Address low reg bits [23:16] 
        /// </summary>
        public static UInt32 ETH_MACA3HR_MBC_LBits23_16 = 0x04000000;

        /// <summary>
        ///   Mask MAC Address low reg bits [15:8] 
        /// </summary>
        public static UInt32 ETH_MACA3HR_MBC_LBits15_8 = 0x02000000;

        /// <summary>
        ///   Mask MAC Address low reg bits [70] 
        /// </summary>
        public static UInt32 ETH_MACA3HR_MBC_LBits7_0 = 0x01000000;

        /// <summary>
        ///   MAC address3 high 
        /// </summary>
        public static UInt32 ETH_MACA3HR_MACA3H = 0x0000FFFF;


        /* Bit definition for Ethernet MAC Address3 Low Register */
        /// <summary>
        ///   MAC address3 low 
        /// </summary>
        public static UInt32 ETH_MACA3LR_MACA3L = 0xFFFFFFFF;


        /******************************************************************************/
        /*                Ethernet MMC Registers bits definition                      */
        /******************************************************************************/

        /* Bit definition for Ethernet MMC Contol Register */
        /// <summary>
        ///   MMC counter Full-Half preset 
        /// </summary>
        public static UInt32 ETH_MMCCR_MCFHP = 0x00000020;

        /// <summary>
        ///   MMC counter preset 
        /// </summary>
        public static UInt32 ETH_MMCCR_MCP = 0x00000010;

        /// <summary>
        ///   MMC Counter Freeze 
        /// </summary>
        public static UInt32 ETH_MMCCR_MCF = 0x00000008;

        /// <summary>
        ///   Reset on Read 
        /// </summary>
        public static UInt32 ETH_MMCCR_ROR = 0x00000004;

        /// <summary>
        ///   Counter Stop Rollover 
        /// </summary>
        public static UInt32 ETH_MMCCR_CSR = 0x00000002;

        /// <summary>
        ///   Counters Reset 
        /// </summary>
        public static UInt32 ETH_MMCCR_CR = 0x00000001;


        /* Bit definition for Ethernet MMC Receive Interrupt Register */
        /// <summary>
        ///   Set when Rx good unicast frames counter reaches half the maximum value 
        /// </summary>
        public static UInt32 ETH_MMCRIR_RGUFS = 0x00020000;

        /// <summary>
        ///   Set when Rx alignment error counter reaches half the maximum value 
        /// </summary>
        public static UInt32 ETH_MMCRIR_RFAES = 0x00000040;

        /// <summary>
        ///   Set when Rx crc error counter reaches half the maximum value 
        /// </summary>
        public static UInt32 ETH_MMCRIR_RFCES = 0x00000020;


        /* Bit definition for Ethernet MMC Transmit Interrupt Register */
        /// <summary>
        ///   Set when Tx good frame count counter reaches half the maximum value 
        /// </summary>
        public static UInt32 ETH_MMCTIR_TGFS = 0x00200000;

        /// <summary>
        ///   Set when Tx good multi col counter reaches half the maximum value 
        /// </summary>
        public static UInt32 ETH_MMCTIR_TGFMSCS = 0x00008000;

        /// <summary>
        ///   Set when Tx good single col counter reaches half the maximum value 
        /// </summary>
        public static UInt32 ETH_MMCTIR_TGFSCS = 0x00004000;


        /* Bit definition for Ethernet MMC Receive Interrupt Mask Register */
        /// <summary>
        ///   Mask the interrupt when Rx good unicast frames counter reaches half the maximum value 
        /// </summary>
        public static UInt32 ETH_MMCRIMR_RGUFM = 0x00020000;

        /// <summary>
        ///   Mask the interrupt when when Rx alignment error counter reaches half the maximum value 
        /// </summary>
        public static UInt32 ETH_MMCRIMR_RFAEM = 0x00000040;

        /// <summary>
        ///   Mask the interrupt when Rx crc error counter reaches half the maximum value 
        /// </summary>
        public static UInt32 ETH_MMCRIMR_RFCEM = 0x00000020;


        /* Bit definition for Ethernet MMC Transmit Interrupt Mask Register */
        /// <summary>
        ///   Mask the interrupt when Tx good frame count counter reaches half the maximum value 
        /// </summary>
        public static UInt32 ETH_MMCTIMR_TGFM = 0x00200000;

        /// <summary>
        ///   Mask the interrupt when Tx good multi col counter reaches half the maximum value 
        /// </summary>
        public static UInt32 ETH_MMCTIMR_TGFMSCM = 0x00008000;

        /// <summary>
        ///   Mask the interrupt when Tx good single col counter reaches half the maximum value 
        /// </summary>
        public static UInt32 ETH_MMCTIMR_TGFSCM = 0x00004000;


        /* Bit definition for Ethernet MMC Transmitted Good Frames after Single Collision Counter Register */
        /// <summary>
        ///   Number of successfully transmitted frames after a single collision in Half-duplex mode. 
        /// </summary>
        public static UInt32 ETH_MMCTGFSCCR_TGFSCC = 0xFFFFFFFF;


        /* Bit definition for Ethernet MMC Transmitted Good Frames after More than a Single Collision Counter Register */
        /// <summary>
        ///   Number of successfully transmitted frames after more than a single collision in Half-duplex mode. 
        /// </summary>
        public static UInt32 ETH_MMCTGFMSCCR_TGFMSCC = 0xFFFFFFFF;


        /* Bit definition for Ethernet MMC Transmitted Good Frames Counter Register */
        /// <summary>
        ///   Number of good frames transmitted. 
        /// </summary>
        public static UInt32 ETH_MMCTGFCR_TGFC = 0xFFFFFFFF;


        /* Bit definition for Ethernet MMC Received Frames with CRC Error Counter Register */
        /// <summary>
        ///   Number of frames received with CRC error. 
        /// </summary>
        public static UInt32 ETH_MMCRFCECR_RFCEC = 0xFFFFFFFF;


        /* Bit definition for Ethernet MMC Received Frames with Alignement Error Counter Register */
        /// <summary>
        ///   Number of frames received with alignment (dribble) error 
        /// </summary>
        public static UInt32 ETH_MMCRFAECR_RFAEC = 0xFFFFFFFF;


        /* Bit definition for Ethernet MMC Received Good Unicast Frames Counter Register */
        /// <summary>
        ///   Number of good unicast frames received. 
        /// </summary>
        public static UInt32 ETH_MMCRGUFCR_RGUFC = 0xFFFFFFFF;


        /******************************************************************************/
        /*               Ethernet PTP Registers bits definition                       */
        /******************************************************************************/

        /* Bit definition for Ethernet PTP Time Stamp Contol Register */
        /// <summary>
        ///   Time stamp clock node type 
        /// </summary>
        public static UInt32 ETH_PTPTSCR_TSCNT = 0x00030000;

        /// <summary>
        ///   Time stamp snapshot for message relevant to master enable 
        /// </summary>
        public static UInt32 ETH_PTPTSSR_TSSMRME = 0x00008000;

        /// <summary>
        ///   Time stamp snapshot for event message enable 
        /// </summary>
        public static UInt32 ETH_PTPTSSR_TSSEME = 0x00004000;

        /// <summary>
        ///   Time stamp snapshot for IPv4 frames enable 
        /// </summary>
        public static UInt32 ETH_PTPTSSR_TSSIPV4FE = 0x00002000;

        /// <summary>
        ///   Time stamp snapshot for IPv6 frames enable 
        /// </summary>
        public static UInt32 ETH_PTPTSSR_TSSIPV6FE = 0x00001000;

        /// <summary>
        ///   Time stamp snapshot for PTP over ethernet frames enable 
        /// </summary>
        public static UInt32 ETH_PTPTSSR_TSSPTPOEFE = 0x00000800;

        /// <summary>
        ///   Time stamp PTP packet snooping for version2 format enable 
        /// </summary>
        public static UInt32 ETH_PTPTSSR_TSPTPPSV2E = 0x00000400;

        /// <summary>
        ///   Time stamp Sub-seconds rollover 
        /// </summary>
        public static UInt32 ETH_PTPTSSR_TSSSR = 0x00000200;

        /// <summary>
        ///   Time stamp snapshot for all received frames enable 
        /// </summary>
        public static UInt32 ETH_PTPTSSR_TSSARFE = 0x00000100;


        /// <summary>
        ///   Addend register update 
        /// </summary>
        public static UInt32 ETH_PTPTSCR_TSARU = 0x00000020;

        /// <summary>
        ///   Time stamp interrupt trigger enable 
        /// </summary>
        public static UInt32 ETH_PTPTSCR_TSITE = 0x00000010;

        /// <summary>
        ///   Time stamp update 
        /// </summary>
        public static UInt32 ETH_PTPTSCR_TSSTU = 0x00000008;

        /// <summary>
        ///   Time stamp initialize 
        /// </summary>
        public static UInt32 ETH_PTPTSCR_TSSTI = 0x00000004;

        /// <summary>
        ///   Time stamp fine or coarse update 
        /// </summary>
        public static UInt32 ETH_PTPTSCR_TSFCU = 0x00000002;

        /// <summary>
        ///   Time stamp enable 
        /// </summary>
        public static UInt32 ETH_PTPTSCR_TSE = 0x00000001;


        /* Bit definition for Ethernet PTP Sub-Second Increment Register */
        /// <summary>
        ///   System time Sub-second increment value 
        /// </summary>
        public static UInt32 ETH_PTPSSIR_STSSI = 0x000000FF;


        /* Bit definition for Ethernet PTP Time Stamp High Register */
        /// <summary>
        ///   System Time second 
        /// </summary>
        public static UInt32 ETH_PTPTSHR_STS = 0xFFFFFFFF;


        /* Bit definition for Ethernet PTP Time Stamp Low Register */
        /// <summary>
        ///   System Time Positive or negative time 
        /// </summary>
        public static UInt32 ETH_PTPTSLR_STPNS = 0x80000000;

        /// <summary>
        ///   System Time sub-seconds 
        /// </summary>
        public static UInt32 ETH_PTPTSLR_STSS = 0x7FFFFFFF;


        /* Bit definition for Ethernet PTP Time Stamp High Update Register */
        /// <summary>
        ///   Time stamp update seconds 
        /// </summary>
        public static UInt32 ETH_PTPTSHUR_TSUS = 0xFFFFFFFF;


        /* Bit definition for Ethernet PTP Time Stamp Low Update Register */
        /// <summary>
        ///   Time stamp update Positive or negative time 
        /// </summary>
        public static UInt32 ETH_PTPTSLUR_TSUPNS = 0x80000000;

        /// <summary>
        ///   Time stamp update sub-seconds 
        /// </summary>
        public static UInt32 ETH_PTPTSLUR_TSUSS = 0x7FFFFFFF;


        /* Bit definition for Ethernet PTP Time Stamp Addend Register */
        /// <summary>
        ///   Time stamp addend 
        /// </summary>
        public static UInt32 ETH_PTPTSAR_TSA = 0xFFFFFFFF;


        /* Bit definition for Ethernet PTP Target Time High Register */
        /// <summary>
        ///   Target time stamp high 
        /// </summary>
        public static UInt32 ETH_PTPTTHR_TTSH = 0xFFFFFFFF;


        /* Bit definition for Ethernet PTP Target Time Low Register */
        /// <summary>
        ///   Target time stamp low 
        /// </summary>
        public static UInt32 ETH_PTPTTLR_TTSL = 0xFFFFFFFF;


        /* Bit definition for Ethernet PTP Time Stamp Status Register */
        /// <summary>
        ///   Time stamp target time reached 
        /// </summary>
        public static UInt32 ETH_PTPTSSR_TSTTR = 0x00000020;

        /// <summary>
        ///   Time stamp seconds overflow 
        /// </summary>
        public static UInt32 ETH_PTPTSSR_TSSO = 0x00000010;


        /******************************************************************************/
        /*                 Ethernet DMA Registers bits definition                     */
        /******************************************************************************/

        /* Bit definition for Ethernet DMA Bus Mode Register */
        /// <summary>
        ///   Address-Aligned beats 
        /// </summary>
        public static UInt32 ETH_DMABMR_AAB = 0x02000000;

        /// <summary>
        ///   4xPBL mode 
        /// </summary>
        public static UInt32 ETH_DMABMR_FPM = 0x01000000;

        /// <summary>
        ///   Use separate PBL 
        /// </summary>
        public static UInt32 ETH_DMABMR_USP = 0x00800000;

        /// <summary>
        ///   RxDMA PBL 
        /// </summary>
        public static UInt32 ETH_DMABMR_RDP = 0x007E0000;

        /// <summary>
        ///   maximum number of beats to be transferred in one RxDMA transaction is 1 
        /// </summary>
        public static UInt32 ETH_DMABMR_RDP_1Beat = 0x00020000;

        /// <summary>
        ///   maximum number of beats to be transferred in one RxDMA transaction is 2 
        /// </summary>
        public static UInt32 ETH_DMABMR_RDP_2Beat = 0x00040000;

        /// <summary>
        ///   maximum number of beats to be transferred in one RxDMA transaction is 4 
        /// </summary>
        public static UInt32 ETH_DMABMR_RDP_4Beat = 0x00080000;

        /// <summary>
        ///   maximum number of beats to be transferred in one RxDMA transaction is 8 
        /// </summary>
        public static UInt32 ETH_DMABMR_RDP_8Beat = 0x00100000;

        /// <summary>
        ///   maximum number of beats to be transferred in one RxDMA transaction is 16 
        /// </summary>
        public static UInt32 ETH_DMABMR_RDP_16Beat = 0x00200000;

        /// <summary>
        ///   maximum number of beats to be transferred in one RxDMA transaction is 32                 
        /// </summary>
        public static UInt32 ETH_DMABMR_RDP_32Beat = 0x00400000;

        /// <summary>
        ///   maximum number of beats to be transferred in one RxDMA transaction is 4 
        /// </summary>
        public static UInt32 ETH_DMABMR_RDP_4xPBL_4Beat = 0x01020000;

        /// <summary>
        ///   maximum number of beats to be transferred in one RxDMA transaction is 8 
        /// </summary>
        public static UInt32 ETH_DMABMR_RDP_4xPBL_8Beat = 0x01040000;

        /// <summary>
        ///   maximum number of beats to be transferred in one RxDMA transaction is 16 
        /// </summary>
        public static UInt32 ETH_DMABMR_RDP_4xPBL_16Beat = 0x01080000;

        /// <summary>
        ///   maximum number of beats to be transferred in one RxDMA transaction is 32 
        /// </summary>
        public static UInt32 ETH_DMABMR_RDP_4xPBL_32Beat = 0x01100000;

        /// <summary>
        ///   maximum number of beats to be transferred in one RxDMA transaction is 64 
        /// </summary>
        public static UInt32 ETH_DMABMR_RDP_4xPBL_64Beat = 0x01200000;

        /// <summary>
        ///   maximum number of beats to be transferred in one RxDMA transaction is 128   
        /// </summary>
        public static UInt32 ETH_DMABMR_RDP_4xPBL_128Beat = 0x01400000;

        /// <summary>
        ///   Fixed Burst 
        /// </summary>
        public static UInt32 ETH_DMABMR_FB = 0x00010000;

        /// <summary>
        ///   Rx Tx priority ratio 
        /// </summary>
        public static UInt32 ETH_DMABMR_RTPR = 0x0000C000;

        /// <summary>
        ///   Rx Tx priority ratio 
        /// </summary>
        public static UInt32 ETH_DMABMR_RTPR_1_1 = 0x00000000;

        /// <summary>
        ///   Rx Tx priority ratio 
        /// </summary>
        public static UInt32 ETH_DMABMR_RTPR_2_1 = 0x00004000;

        /// <summary>
        ///   Rx Tx priority ratio 
        /// </summary>
        public static UInt32 ETH_DMABMR_RTPR_3_1 = 0x00008000;

        /// <summary>
        ///   Rx Tx priority ratio   
        /// </summary>
        public static UInt32 ETH_DMABMR_RTPR_4_1 = 0x0000C000;

        /// <summary>
        ///   Programmable burst length 
        /// </summary>
        public static UInt32 ETH_DMABMR_PBL = 0x00003F00;

        /// <summary>
        ///   maximum number of beats to be transferred in one TxDMA (or both) transaction is 1 
        /// </summary>
        public static UInt32 ETH_DMABMR_PBL_1Beat = 0x00000100;

        /// <summary>
        ///   maximum number of beats to be transferred in one TxDMA (or both) transaction is 2 
        /// </summary>
        public static UInt32 ETH_DMABMR_PBL_2Beat = 0x00000200;

        /// <summary>
        ///   maximum number of beats to be transferred in one TxDMA (or both) transaction is 4 
        /// </summary>
        public static UInt32 ETH_DMABMR_PBL_4Beat = 0x00000400;

        /// <summary>
        ///   maximum number of beats to be transferred in one TxDMA (or both) transaction is 8 
        /// </summary>
        public static UInt32 ETH_DMABMR_PBL_8Beat = 0x00000800;

        /// <summary>
        ///   maximum number of beats to be transferred in one TxDMA (or both) transaction is 16 
        /// </summary>
        public static UInt32 ETH_DMABMR_PBL_16Beat = 0x00001000;

        /// <summary>
        ///   maximum number of beats to be transferred in one TxDMA (or both) transaction is 32                 
        /// </summary>
        public static UInt32 ETH_DMABMR_PBL_32Beat = 0x00002000;

        /// <summary>
        ///   maximum number of beats to be transferred in one TxDMA (or both) transaction is 4 
        /// </summary>
        public static UInt32 ETH_DMABMR_PBL_4xPBL_4Beat = 0x01000100;

        /// <summary>
        ///   maximum number of beats to be transferred in one TxDMA (or both) transaction is 8 
        /// </summary>
        public static UInt32 ETH_DMABMR_PBL_4xPBL_8Beat = 0x01000200;

        /// <summary>
        ///   maximum number of beats to be transferred in one TxDMA (or both) transaction is 16 
        /// </summary>
        public static UInt32 ETH_DMABMR_PBL_4xPBL_16Beat = 0x01000400;

        /// <summary>
        ///   maximum number of beats to be transferred in one TxDMA (or both) transaction is 32 
        /// </summary>
        public static UInt32 ETH_DMABMR_PBL_4xPBL_32Beat = 0x01000800;

        /// <summary>
        ///   maximum number of beats to be transferred in one TxDMA (or both) transaction is 64 
        /// </summary>
        public static UInt32 ETH_DMABMR_PBL_4xPBL_64Beat = 0x01001000;

        /// <summary>
        ///   maximum number of beats to be transferred in one TxDMA (or both) transaction is 128 
        /// </summary>
        public static UInt32 ETH_DMABMR_PBL_4xPBL_128Beat = 0x01002000;

        /// <summary>
        ///   Enhanced Descriptor Enable 
        /// </summary>
        public static UInt32 ETH_DMABMR_EDE = 0x00000080;

        /// <summary>
        ///   Descriptor Skip Length 
        /// </summary>
        public static UInt32 ETH_DMABMR_DSL = 0x0000007C;

        /// <summary>
        ///   DMA arbitration scheme 
        /// </summary>
        public static UInt32 ETH_DMABMR_DA = 0x00000002;

        /// <summary>
        ///   Software reset 
        /// </summary>
        public static UInt32 ETH_DMABMR_SR = 0x00000001;


        /* Bit definition for Ethernet DMA Transmit Poll Demand Register */
        /// <summary>
        ///   Transmit poll demand 
        /// </summary>
        public static UInt32 ETH_DMATPDR_TPD = 0xFFFFFFFF;


        /* Bit definition for Ethernet DMA Receive Poll Demand Register */
        /// <summary>
        ///   Receive poll demand  
        /// </summary>
        public static UInt32 ETH_DMARPDR_RPD = 0xFFFFFFFF;


        /* Bit definition for Ethernet DMA Receive Descriptor List Address Register */
        /// <summary>
        ///   Start of receive list 
        /// </summary>
        public static UInt32 ETH_DMARDLAR_SRL = 0xFFFFFFFF;


        /* Bit definition for Ethernet DMA Transmit Descriptor List Address Register */
        /// <summary>
        ///   Start of transmit list 
        /// </summary>
        public static UInt32 ETH_DMATDLAR_STL = 0xFFFFFFFF;


        /* Bit definition for Ethernet DMA Status Register */
        /// <summary>
        ///   Time-stamp trigger status 
        /// </summary>
        public static UInt32 ETH_DMASR_TSTS = 0x20000000;

        /// <summary>
        ///   PMT status 
        /// </summary>
        public static UInt32 ETH_DMASR_PMTS = 0x10000000;

        /// <summary>
        ///   MMC status 
        /// </summary>
        public static UInt32 ETH_DMASR_MMCS = 0x08000000;

        /// <summary>
        ///   Error bits status 
        /// </summary>
        public static UInt32 ETH_DMASR_EBS = 0x03800000;

        /* combination with EBS[2:0] for GetFlagStatus function */
        /// <summary>
        ///   Error bits 0-data buffer, 1-desc. access 
        /// </summary>
        public static UInt32 ETH_DMASR_EBS_DescAccess = 0x02000000;

        /// <summary>
        ///   Error bits 0-write trnsf, 1-read transfr 
        /// </summary>
        public static UInt32 ETH_DMASR_EBS_ReadTransf = 0x01000000;

        /// <summary>
        ///   Error bits 0-Rx DMA, 1-Tx DMA 
        /// </summary>
        public static UInt32 ETH_DMASR_EBS_DataTransfTx = 0x00800000;

        /// <summary>
        ///   Transmit process state 
        /// </summary>
        public static UInt32 ETH_DMASR_TPS = 0x00700000;

        /// <summary>
        ///   Stopped - Reset or Stop Tx Command issued  
        /// </summary>
        public static UInt32 ETH_DMASR_TPS_Stopped = 0x00000000;

        /// <summary>
        ///   Running - fetching the Tx descriptor 
        /// </summary>
        public static UInt32 ETH_DMASR_TPS_Fetching = 0x00100000;

        /// <summary>
        ///   Running - waiting for status 
        /// </summary>
        public static UInt32 ETH_DMASR_TPS_Waiting = 0x00200000;

        /// <summary>
        ///   Running - reading the data from host memory 
        /// </summary>
        public static UInt32 ETH_DMASR_TPS_Reading = 0x00300000;

        /// <summary>
        ///   Suspended - Tx Descriptor unavailabe 
        /// </summary>
        public static UInt32 ETH_DMASR_TPS_Suspended = 0x00600000;

        /// <summary>
        ///   Running - closing Rx descriptor 
        /// </summary>
        public static UInt32 ETH_DMASR_TPS_Closing = 0x00700000;

        /// <summary>
        ///   Receive process state 
        /// </summary>
        public static UInt32 ETH_DMASR_RPS = 0x000E0000;

        /// <summary>
        ///   Stopped - Reset or Stop Rx Command issued 
        /// </summary>
        public static UInt32 ETH_DMASR_RPS_Stopped = 0x00000000;

        /// <summary>
        ///   Running - fetching the Rx descriptor 
        /// </summary>
        public static UInt32 ETH_DMASR_RPS_Fetching = 0x00020000;

        /// <summary>
        ///   Running - waiting for packet 
        /// </summary>
        public static UInt32 ETH_DMASR_RPS_Waiting = 0x00060000;

        /// <summary>
        ///   Suspended - Rx Descriptor unavailable 
        /// </summary>
        public static UInt32 ETH_DMASR_RPS_Suspended = 0x00080000;

        /// <summary>
        ///   Running - closing descriptor 
        /// </summary>
        public static UInt32 ETH_DMASR_RPS_Closing = 0x000A0000;

        /// <summary>
        ///   Running - queuing the recieve frame into host memory 
        /// </summary>
        public static UInt32 ETH_DMASR_RPS_Queuing = 0x000E0000;

        /// <summary>
        ///   Normal interrupt summary 
        /// </summary>
        public static UInt32 ETH_DMASR_NIS = 0x00010000;

        /// <summary>
        ///   Abnormal interrupt summary 
        /// </summary>
        public static UInt32 ETH_DMASR_AIS = 0x00008000;

        /// <summary>
        ///   Early receive status 
        /// </summary>
        public static UInt32 ETH_DMASR_ERS = 0x00004000;

        /// <summary>
        ///   Fatal bus error status 
        /// </summary>
        public static UInt32 ETH_DMASR_FBES = 0x00002000;

        /// <summary>
        ///   Early transmit status 
        /// </summary>
        public static UInt32 ETH_DMASR_ETS = 0x00000400;

        /// <summary>
        ///   Receive watchdog timeout status 
        /// </summary>
        public static UInt32 ETH_DMASR_RWTS = 0x00000200;

        /// <summary>
        ///   Receive process stopped status 
        /// </summary>
        public static UInt32 ETH_DMASR_RPSS = 0x00000100;

        /// <summary>
        ///   Receive buffer unavailable status 
        /// </summary>
        public static UInt32 ETH_DMASR_RBUS = 0x00000080;

        /// <summary>
        ///   Receive status 
        /// </summary>
        public static UInt32 ETH_DMASR_RS = 0x00000040;

        /// <summary>
        ///   Transmit underflow status 
        /// </summary>
        public static UInt32 ETH_DMASR_TUS = 0x00000020;

        /// <summary>
        ///   Receive overflow status 
        /// </summary>
        public static UInt32 ETH_DMASR_ROS = 0x00000010;

        /// <summary>
        ///   Transmit jabber timeout status 
        /// </summary>
        public static UInt32 ETH_DMASR_TJTS = 0x00000008;

        /// <summary>
        ///   Transmit buffer unavailable status 
        /// </summary>
        public static UInt32 ETH_DMASR_TBUS = 0x00000004;

        /// <summary>
        ///   Transmit process stopped status 
        /// </summary>
        public static UInt32 ETH_DMASR_TPSS = 0x00000002;

        /// <summary>
        ///   Transmit status 
        /// </summary>
        public static UInt32 ETH_DMASR_TS = 0x00000001;


        /* Bit definition for Ethernet DMA Operation Mode Register */
        /// <summary>
        ///   Disable Dropping of TCP/IP checksum error frames 
        /// </summary>
        public static UInt32 ETH_DMAOMR_DTCEFD = 0x04000000;

        /// <summary>
        ///   Receive store and forward 
        /// </summary>
        public static UInt32 ETH_DMAOMR_RSF = 0x02000000;

        /// <summary>
        ///   Disable flushing of received frames 
        /// </summary>
        public static UInt32 ETH_DMAOMR_DFRF = 0x01000000;

        /// <summary>
        ///   Transmit store and forward 
        /// </summary>
        public static UInt32 ETH_DMAOMR_TSF = 0x00200000;

        /// <summary>
        ///   Flush transmit FIFO 
        /// </summary>
        public static UInt32 ETH_DMAOMR_FTF = 0x00100000;

        /// <summary>
        ///   Transmit threshold control 
        /// </summary>
        public static UInt32 ETH_DMAOMR_TTC = 0x0001C000;

        /// <summary>
        ///   threshold level of the MTL Transmit FIFO is 64 Bytes 
        /// </summary>
        public static UInt32 ETH_DMAOMR_TTC_64Bytes = 0x00000000;

        /// <summary>
        ///   threshold level of the MTL Transmit FIFO is 128 Bytes 
        /// </summary>
        public static UInt32 ETH_DMAOMR_TTC_128Bytes = 0x00004000;

        /// <summary>
        ///   threshold level of the MTL Transmit FIFO is 192 Bytes 
        /// </summary>
        public static UInt32 ETH_DMAOMR_TTC_192Bytes = 0x00008000;

        /// <summary>
        ///   threshold level of the MTL Transmit FIFO is 256 Bytes 
        /// </summary>
        public static UInt32 ETH_DMAOMR_TTC_256Bytes = 0x0000C000;

        /// <summary>
        ///   threshold level of the MTL Transmit FIFO is 40 Bytes 
        /// </summary>
        public static UInt32 ETH_DMAOMR_TTC_40Bytes = 0x00010000;

        /// <summary>
        ///   threshold level of the MTL Transmit FIFO is 32 Bytes 
        /// </summary>
        public static UInt32 ETH_DMAOMR_TTC_32Bytes = 0x00014000;

        /// <summary>
        ///   threshold level of the MTL Transmit FIFO is 24 Bytes 
        /// </summary>
        public static UInt32 ETH_DMAOMR_TTC_24Bytes = 0x00018000;

        /// <summary>
        ///   threshold level of the MTL Transmit FIFO is 16 Bytes 
        /// </summary>
        public static UInt32 ETH_DMAOMR_TTC_16Bytes = 0x0001C000;

        /// <summary>
        ///   Start/stop transmission command 
        /// </summary>
        public static UInt32 ETH_DMAOMR_ST = 0x00002000;

        /// <summary>
        ///   Forward error frames 
        /// </summary>
        public static UInt32 ETH_DMAOMR_FEF = 0x00000080;

        /// <summary>
        ///   Forward undersized good frames 
        /// </summary>
        public static UInt32 ETH_DMAOMR_FUGF = 0x00000040;

        /// <summary>
        ///   receive threshold control 
        /// </summary>
        public static UInt32 ETH_DMAOMR_RTC = 0x00000018;

        /// <summary>
        ///   threshold level of the MTL Receive FIFO is 64 Bytes 
        /// </summary>
        public static UInt32 ETH_DMAOMR_RTC_64Bytes = 0x00000000;

        /// <summary>
        ///   threshold level of the MTL Receive FIFO is 32 Bytes 
        /// </summary>
        public static UInt32 ETH_DMAOMR_RTC_32Bytes = 0x00000008;

        /// <summary>
        ///   threshold level of the MTL Receive FIFO is 96 Bytes 
        /// </summary>
        public static UInt32 ETH_DMAOMR_RTC_96Bytes = 0x00000010;

        /// <summary>
        ///   threshold level of the MTL Receive FIFO is 128 Bytes 
        /// </summary>
        public static UInt32 ETH_DMAOMR_RTC_128Bytes = 0x00000018;

        /// <summary>
        ///   operate on second frame 
        /// </summary>
        public static UInt32 ETH_DMAOMR_OSF = 0x00000004;

        /// <summary>
        ///   Start/stop receive 
        /// </summary>
        public static UInt32 ETH_DMAOMR_SR = 0x00000002;


        /* Bit definition for Ethernet DMA Interrupt Enable Register */
        /// <summary>
        ///   Normal interrupt summary enable 
        /// </summary>
        public static UInt32 ETH_DMAIER_NISE = 0x00010000;

        /// <summary>
        ///   Abnormal interrupt summary enable 
        /// </summary>
        public static UInt32 ETH_DMAIER_AISE = 0x00008000;

        /// <summary>
        ///   Early receive interrupt enable 
        /// </summary>
        public static UInt32 ETH_DMAIER_ERIE = 0x00004000;

        /// <summary>
        ///   Fatal bus error interrupt enable 
        /// </summary>
        public static UInt32 ETH_DMAIER_FBEIE = 0x00002000;

        /// <summary>
        ///   Early transmit interrupt enable 
        /// </summary>
        public static UInt32 ETH_DMAIER_ETIE = 0x00000400;

        /// <summary>
        ///   Receive watchdog timeout interrupt enable 
        /// </summary>
        public static UInt32 ETH_DMAIER_RWTIE = 0x00000200;

        /// <summary>
        ///   Receive process stopped interrupt enable 
        /// </summary>
        public static UInt32 ETH_DMAIER_RPSIE = 0x00000100;

        /// <summary>
        ///   Receive buffer unavailable interrupt enable 
        /// </summary>
        public static UInt32 ETH_DMAIER_RBUIE = 0x00000080;

        /// <summary>
        ///   Receive interrupt enable 
        /// </summary>
        public static UInt32 ETH_DMAIER_RIE = 0x00000040;

        /// <summary>
        ///   Transmit Underflow interrupt enable 
        /// </summary>
        public static UInt32 ETH_DMAIER_TUIE = 0x00000020;

        /// <summary>
        ///   Receive Overflow interrupt enable 
        /// </summary>
        public static UInt32 ETH_DMAIER_ROIE = 0x00000010;

        /// <summary>
        ///   Transmit jabber timeout interrupt enable 
        /// </summary>
        public static UInt32 ETH_DMAIER_TJTIE = 0x00000008;

        /// <summary>
        ///   Transmit buffer unavailable interrupt enable 
        /// </summary>
        public static UInt32 ETH_DMAIER_TBUIE = 0x00000004;

        /// <summary>
        ///   Transmit process stopped interrupt enable 
        /// </summary>
        public static UInt32 ETH_DMAIER_TPSIE = 0x00000002;

        /// <summary>
        ///   Transmit interrupt enable 
        /// </summary>
        public static UInt32 ETH_DMAIER_TIE = 0x00000001;


        /* Bit definition for Ethernet DMA Missed Frame and Buffer Overflow Counter Register */
        /// <summary>
        ///   Overflow bit for FIFO overflow counter 
        /// </summary>
        public static UInt32 ETH_DMAMFBOCR_OFOC = 0x10000000;

        /// <summary>
        ///   Number of frames missed by the application 
        /// </summary>
        public static UInt32 ETH_DMAMFBOCR_MFA = 0x0FFE0000;

        /// <summary>
        ///   Overflow bit for missed frame counter 
        /// </summary>
        public static UInt32 ETH_DMAMFBOCR_OMFC = 0x00010000;

        /// <summary>
        ///   Number of frames missed by the controller 
        /// </summary>
        public static UInt32 ETH_DMAMFBOCR_MFC = 0x0000FFFF;


        /* Bit definition for Ethernet DMA Current Host Transmit Descriptor Register */
        /// <summary>
        ///   Host transmit descriptor address pointer 
        /// </summary>
        public static UInt32 ETH_DMACHTDR_HTDAP = 0xFFFFFFFF;


        /* Bit definition for Ethernet DMA Current Host Receive Descriptor Register */
        /// <summary>
        ///   Host receive descriptor address pointer 
        /// </summary>
        public static UInt32 ETH_DMACHRDR_HRDAP = 0xFFFFFFFF;


        /* Bit definition for Ethernet DMA Current Host Transmit Buffer Address Register */
        /// <summary>
        ///   Host transmit buffer address pointer 
        /// </summary>
        public static UInt32 ETH_DMACHTBAR_HTBAP = 0xFFFFFFFF;


        /* Bit definition for Ethernet DMA Current Host Receive Buffer Address Register */
        /// <summary>
        ///   Host receive buffer address pointer 
        /// </summary>
        public static UInt32 ETH_DMACHRBAR_HRBAP = 0xFFFFFFFF;

    }


} // END of Namespace...