using System;
using Microsoft.SPOT;

namespace Cerberus_PWM
{
    /// <summary>
    /// enum IRQn
    /// </summary>
    public enum IRQn
    {
        /// <summary>
        /// 2 Non Maskable Interrupt
        /// </summary>
        NonMaskableInt_IRQn = -14,

        /// <summary>
        /// 4 Cortex-M4 Memory Management Interrupt 
        /// </summary>
        MemoryManagement_IRQn = -12,

        /// <summary>
        /// 5 Cortex-M4 Bus Fault Interrupt 
        /// </summary>
        BusFault_IRQn = -11,

        /// <summary>
        /// 6 Cortex-M4 Usage Fault Interrupt 
        /// </summary>
        UsageFault_IRQn = -10,

        /// <summary>
        /// 11 Cortex-M4 SV Call Interrupt
        /// </summary>
        SVCall_IRQn = -5,

        /// <summary>
        /// 12 Cortex-M4 Debug Monitor Interrupt
        /// </summary>
        DebugMonitor_IRQn = -4,

        /// <summary>
        /// 14 Cortex-M4 Pend SV Interrupt
        /// </summary>
        PendSV_IRQn = -2,

        /// <summary>
        /// 15 Cortex-M4 System Tick Interrupt
        /// </summary>
        SysTick_IRQn = -1,

        /// <summary>
        /// Window WatchDog Interrupt 
        /// </summary>
        WWDG_IRQn = 0,

        /// <summary>
        /// PVD through EXTI Line detection Interrupt 
        /// </summary>
        PVD_IRQn = 1,

        /// <summary>
        /// Tamper and TimeStamp interrupts through the EXTI line 
        /// </summary>
        TAMP_STAMP_IRQn = 2,

        /// <summary>
        /// RTC Wakeup interrupt through the EXTI line
        /// </summary>
        RTC_WKUP_IRQn = 3,

        /// <summary>
        /// FLASH global Interrupt
        /// </summary>
        FLASH_IRQn = 4,

        /// <summary>
        /// RCC global Interrupt
        /// </summary>
        RCC_IRQn = 5,

        /// <summary>
        /// EXTI Line0 Interrupt
        /// </summary>
        EXTI0_IRQn = 6,

        /// <summary>
        /// EXTI Line1 Interrupt
        /// </summary>
        EXTI1_IRQn = 7,

        /// <summary>
        /// EXTI Line2 Interrupt
        /// </summary>
        EXTI2_IRQn = 8,

        /// <summary>
        /// EXTI Line3 Interrupt
        /// </summary>
        EXTI3_IRQn = 9,

        /// <summary>
        /// EXTI Line4 Interrupt
        /// </summary>
        EXTI4_IRQn = 10,

        /// <summary>
        /// DMA1 Stream 0 global Interrupt
        /// </summary>
        DMA1_Stream0_IRQn = 11,

        /// <summary>
        /// DMA1 Stream 1 global Interrupt
        /// </summary>
        DMA1_Stream1_IRQn = 12,

        /// <summary>
        /// DMA1 Stream 2 global Interrupt
        /// </summary>
        DMA1_Stream2_IRQn = 13,

        /// <summary>
        /// DMA1 Stream 3 global Interrupt
        /// </summary>
        DMA1_Stream3_IRQn = 14,

        /// <summary>
        /// DMA1 Stream 4 global Interrupt
        /// </summary>
        DMA1_Stream4_IRQn = 15,

        /// <summary>
        /// DMA1 Stream 5 global Interrupt
        /// </summary>
        DMA1_Stream5_IRQn = 16,

        /// <summary>
        /// DMA1 Stream 6 global Interrupt
        /// </summary>
        DMA1_Stream6_IRQn = 17,

        /// <summary>
        /// ADC1, ADC2 and ADC3 global Interrupts 
        /// </summary>
        ADC_IRQn = 18,

        /// <summary>
        /// CAN1 TX Interrupt 
        /// </summary>
        CAN1_TX_IRQn = 19,

        /// <summary>
        /// CAN1 RX0 Interrupt
        /// </summary>
        CAN1_RX0_IRQn = 20,

        /// <summary>
        /// CAN1 RX1 Interrupt
        /// </summary>
        CAN1_RX1_IRQn = 21,

        /// <summary>
        /// CAN1 SCE Interrupt
        /// </summary>
        CAN1_SCE_IRQn = 22,

        /// <summary>
        /// External Line[9:5] Interrupts 
        /// </summary>
        EXTI9_5_IRQn = 23,

        /// <summary>
        /// TIM1 Break interrupt and TIM9 global interrupt
        /// </summary>
        TIM1_BRK_TIM9_IRQn = 24,

        /// <summary>
        /// TIM1 Update Interrupt and TIM10 global interrupt
        /// </summary>
        TIM1_UP_TIM10_IRQn = 25,

        /// <summary>
        /// TIM1 Trigger and Commutation Interrupt and TIM11 global interrupt 
        /// </summary>
        TIM1_TRG_COM_TIM11_IRQn = 26,

        /// <summary>
        /// TIM1 Capture Compare Interrupt
        /// </summary>
        TIM1_CC_IRQn = 27,

        /// <summary>
        /// TIM2 global Interrupt 
        /// </summary>
        TIM2_IRQn = 28,

        /// <summary>
        /// TIM3 global Interrupt 
        /// </summary>
        TIM3_IRQn = 29,

        /// <summary>
        /// TIM4 global Interrupt 
        /// </summary>
        TIM4_IRQn = 30,

        /// <summary>
        /// I2C1 Event Interrupt
        /// </summary>
        I2C1_EV_IRQn = 31,

        /// <summary>
        /// I2C1 Error Interrupt
        /// </summary>
        I2C1_ER_IRQn = 32,

        /// <summary>
        /// I2C2 Event Interrupt
        /// </summary>
        I2C2_EV_IRQn = 33,

        /// <summary>
        /// I2C2 Error Interrupt
        /// </summary>
        I2C2_ER_IRQn = 34,

        /// <summary>
        /// SPI1 global Interrupt 
        /// </summary>
        SPI1_IRQn = 35,

        /// <summary>
        /// SPI2 global Interrupt 
        /// </summary>
        SPI2_IRQn = 36,

        /// <summary>
        /// USART1 global Interrupt 
        /// </summary>
        USART1_IRQn = 37,

        /// <summary>
        /// USART2 global Interrupt 
        /// </summary>
        USART2_IRQn = 38,

        /// <summary>
        /// USART3 global Interrupt 
        /// </summary>
        USART3_IRQn = 39,

        /// <summary>
        /// External Line[15:10] Interrupts 
        /// </summary>
        EXTI15_10_IRQn = 40,

        /// <summary>
        /// RTC Alarm (A and B) through EXTI Line Interrupt 
        /// </summary>
        RTC_Alarm_IRQn = 41,

        /// <summary>
        /// USB OTG FS Wakeup through EXTI line interrupt 
        /// </summary>
        OTG_FS_WKUP_IRQn = 42,

        /// <summary>
        /// TIM8 Break Interrupt and TIM12 global interrupt 
        /// </summary>
        TIM8_BRK_TIM12_IRQn = 43,

        /// <summary>
        /// TIM8 Update Interrupt and TIM13 global interrupt
        /// </summary>
        TIM8_UP_TIM13_IRQn = 44,

        /// <summary>
        /// TIM8 Trigger and Commutation Interrupt and TIM14 global interrupt 
        /// </summary>
        TIM8_TRG_COM_TIM14_IRQn = 45,

        /// <summary>
        /// TIM8 Capture Compare Interrupt
        /// </summary>
        TIM8_CC_IRQn = 46,

        /// <summary>
        /// DMA1 Stream7 Interrupt
        /// </summary>
        DMA1_Stream7_IRQn = 47,

        /// <summary>
        /// FSMC global Interrupt 
        /// </summary>
        FSMC_IRQn = 48,

        /// <summary>
        /// SDIO global Interrupt 
        /// </summary>
        SDIO_IRQn = 49,

        /// <summary>
        /// TIM5 global Interrupt 
        /// </summary>
        TIM5_IRQn = 50,

        /// <summary>
        /// SPI3 global Interrupt 
        /// </summary>
        SPI3_IRQn = 51,

        /// <summary>
        /// UART4 global Interrupt
        /// </summary>
        UART4_IRQn = 52,

        /// <summary>
        /// UART5 global Interrupt
        /// </summary>
        UART5_IRQn = 53,

        /// <summary>
        /// TIM6 global and DAC1&2 underrun errorinterrupts 
        /// </summary>
        TIM6_DAC_IRQn = 54,

        /// <summary>
        /// TIM7 global interrupt 
        /// </summary>
        TIM7_IRQn = 55,

        /// <summary>
        /// DMA2 Stream 0 global Interrupt
        /// </summary>
        DMA2_Stream0_IRQn = 56,

        /// <summary>
        /// DMA2 Stream 1 global Interrupt
        /// </summary>
        DMA2_Stream1_IRQn = 57,

        /// <summary>
        /// DMA2 Stream 2 global Interrupt
        /// </summary>
        DMA2_Stream2_IRQn = 58,

        /// <summary>
        /// DMA2 Stream 3 global Interrupt
        /// </summary>
        DMA2_Stream3_IRQn = 59,

        /// <summary>
        /// DMA2 Stream 4 global Interrupt
        /// </summary>
        DMA2_Stream4_IRQn = 60,

        /// <summary>
        /// Ethernet global Interrupt 
        /// </summary>
        ETH_IRQn = 61,

        /// <summary>
        /// Ethernet Wakeup through EXTI line Interrupt 
        /// </summary>
        ETH_WKUP_IRQn = 62,

        /// <summary>
        /// CAN2 TX Interrupt 
        /// </summary>
        CAN2_TX_IRQn = 63,

        /// <summary>
        /// CAN2 RX0 Interrupt
        /// </summary>
        CAN2_RX0_IRQn = 64,

        /// <summary>
        /// CAN2 RX1 Interrupt
        /// </summary>
        CAN2_RX1_IRQn = 65,

        /// <summary>
        /// CAN2 SCE Interrupt
        /// </summary>
        CAN2_SCE_IRQn = 66,

        /// <summary>
        /// USB OTG FS global Interrupt 
        /// </summary>
        OTG_FS_IRQn = 67,

        /// <summary>
        /// DMA2 Stream 5 global interrupt
        /// </summary>
        DMA2_Stream5_IRQn = 68,

        /// <summary>
        /// DMA2 Stream 6 global interrupt
        /// </summary>
        DMA2_Stream6_IRQn = 69,

        /// <summary>
        /// DMA2 Stream 7 global interrupt
        /// </summary>
        DMA2_Stream7_IRQn = 70,

        /// <summary>
        /// USART6 global interrupt
        /// </summary>
        USART6_IRQn = 71,

        /// <summary>
        /// I2C3 event interrupt
        /// </summary>
        I2C3_EV_IRQn = 72,

        /// <summary>
        /// I2C3 error interrupt
        /// </summary>
        I2C3_ER_IRQn = 73,

        /// <summary>
        /// USB OTG HS End Point 1 Out global interrupt 
        /// </summary>
        OTG_HS_EP1_OUT_IRQn = 74,

        /// <summary>
        /// USB OTG HS End Point 1 In global interrupt
        /// </summary>
        OTG_HS_EP1_IN_IRQn = 75,

        /// <summary>
        /// USB OTG HS Wakeup through EXTI interrupt
        /// </summary>
        OTG_HS_WKUP_IRQn = 76,

        /// <summary>
        /// USB OTG HS global interrupt 
        /// </summary>
        OTG_HS_IRQn = 77,

        /// <summary>
        /// DCMI global interrupt 
        /// </summary>
        DCMI_IRQn = 78,

        /// <summary>
        /// CRYP crypto global interrupt
        /// </summary>
        CRYP_IRQn = 79,

        /// <summary>
        /// Hash and Rng global interrupt 
        /// </summary>
        HASH_RNG_IRQn = 80,

        /// <summary>
        /// FPU global interrupt
        /// </summary>
        FPU_IRQn = 81

    } // END of Enum...

    /// <summary>
    /// Advanced-control timers (TIM1&TIM8) - See: P.507
    /// also General-purpose timers (TIM2 to TIM5) - See: P.576
    /// also General-purpose timers (TIM9 to TIM14) - See: P.636 
    /// and also Basic timers (TIM6&TIM7) See: P.680
    /// </summary>
    public sealed class CPUTimer
    {
        // 17.3.20 Timer synchronization
        // The TIM timers are linked together internally for timer synchronization or chaining. Refer to
        // Section 18.3.15: Timer synchronization on page 605 for details.

        #region Private's...

        private uint _CR1 = 0x00;
        private uint _RESERVED0 = 0x02;
        private uint _CR2 = 0x04;
        private uint _RESERVED1 = 0x06;
        private uint _SMCR = 0x08;
        private uint _RESERVED2 = 0x0A;
        private uint _DIER = 0x0C;
        private uint _RESERVED3 = 0x0E;
        private uint _SR = 0x10;
        private uint _RESERVED4 = 0x12;
        private uint _EGR = 0x14;
        private uint _RESERVED5 = 0x16;
        private uint _CCMR1 = 0x18;
        private uint _RESERVED6 = 0x1A;
        private uint _CCMR2 = 0x1C;
        private uint _RESERVED7 = 0x1E;
        private uint _CCER = 0x20;
        private uint _RESERVED8 = 0x22;
        private uint _CNT = 0x24;
        private uint _PSC = 0x28;
        private uint _RESERVED9 = 0x2A;
        private uint _ARR = 0x2C;
        private uint _RCR = 0x30;
        private uint _RESERVED10 = 0x32;
        private uint _CCR1 = 0x34;
        private uint _CCR2 = 0x38;
        private uint _CCR3 = 0x3C;
        private uint _CCR4 = 0x40;
        private uint _BDTR = 0x44;
        private uint _RESERVED11 = 0x46;
        private uint _DCR = 0x48;
        private uint _RESERVED12 = 0x4A;
        private uint _DMAR = 0x4C;
        private uint _RESERVED13 = 0x4E;
        private uint _OR = 0x50;
        private uint _RESERVED14 = 0x52;

        #endregion

        #region Properties...

        /// <summary>
        /// TIM control register 1, Address offset: 0x00
        /// </summary>
        public uint CR1 { get { return _CR1; } set { _CR1 = value; } }

        /// <summary>
        /// Reserved, 0x02
        /// </summary>
        public uint RESERVED0 { get { return _RESERVED0; } }

        /// <summary>
        /// TIM control register 2, Address offset: 0x04 
        /// </summary>
        public uint CR2 { get { return _CR2; } set { _CR2 = value; } }

        /// <summary>
        /// Reserved, 0x06 
        /// </summary>
        public uint RESERVED1 { get { return _RESERVED1; } }

        /// <summary>
        /// TIM slave mode control register, Address offset: 0x08 
        /// </summary>
        public uint SMCR { get { return _SMCR; } set { _SMCR = value; } }

        /// <summary>
        /// Reserved, 0x0A 
        /// </summary>
        public uint RESERVED2 { get { return _RESERVED2; } }

        /// <summary>
        /// TIM DMA/interrupt enable register, Address offset: 0x0C
        /// </summary>
        public uint DIER { get { return _DIER; } set { _DIER = value; } }

        /// <summary>
        /// Reserved, 0x0E
        /// </summary>
        public uint RESERVED3 { get { return _RESERVED3; } }

        /// <summary>
        /// TIM status register, Address offset: 0x10
        /// </summary>
        public uint SR { get { return _SR; } set { _SR = value; } }

        /// <summary>
        /// Reserved, 0x12 
        /// </summary>
        public uint RESERVED4 { get { return _RESERVED4; } }

        /// <summary>
        /// TIM event generation register, Address offset: 0x14 
        /// </summary>
        public uint EGR { get { return _EGR; } set { _EGR = value; } }

        /// <summary>
        /// Reserved, 0x16
        /// </summary>
        public uint RESERVED5 { get { return _RESERVED5; } }

        /// <summary>
        /// TIM capture/compare mode register 1, Address offset: 0x18 
        /// </summary>
        public uint CCMR1 { get { return _CCMR1; } set { _CCMR1 = value; } }

        /// <summary>
        /// Reserved, 0x1A
        /// </summary>
        public uint RESERVED6 { get { return _RESERVED6; } }

        /// <summary>
        /// TIM capture/compare mode register 2, Address offset: 0x1C 
        /// </summary>
        public uint CCMR2 { get { return _CCMR2; } set { _CCMR2 = value; } }

        /// <summary>
        /// Reserved, 0x1E
        /// </summary>
        public uint RESERVED7 { get { return _RESERVED7; } }

        /// <summary>
        /// TIM capture/compare enable register, Address offset: 0x20 
        /// </summary>
        public uint CCER { get { return _CCER; } set { _CCER = value; } }

        /// <summary>
        /// Reserved, 0x22 
        /// </summary>
        public uint RESERVED8 { get { return _RESERVED8; } }

        /// <summary>
        /// TIM counter register, Address offset: 0x24 
        /// </summary>
        public uint CNT { get { return _CNT; } set { _CNT = value; } }

        /// <summary>
        /// TIM prescaler, Address offset: 0x28 
        /// </summary>
        public uint PSC { get { return _PSC; } set { _PSC = value; } }

        /// <summary>
        /// Reserved, 0x2A 
        /// </summary>
        public uint RESERVED9 { get { return _RESERVED9; } }

        /// <summary>
        /// TIM auto-reload register, Address offset: 0x2C 
        /// </summary>
        public uint ARR { get { return _ARR; } set { _ARR = value; } }

        /// <summary>
        /// TIM repetition counter register, Address offset: 0x30
        /// </summary>
        public uint RCR { get { return _RCR; } set { _RCR = value; } }

        /// <summary>
        /// Reserved, 0x32
        /// </summary>
        public uint RESERVED10 { get { return _RESERVED10; } }

        /// <summary>
        /// TIM capture/compare register 1, Address offset: 0x34 
        /// </summary>
        public uint CCR1 { get { return _CCR1; } set { _CCR1 = value; } }

        /// <summary>
        /// TIM capture/compare register 2, Address offset: 0x38
        /// </summary>
        public uint CCR2 { get { return _CCR2; } set { _CCR2 = value; } }

        /// <summary>
        /// TIM capture/compare register 3, Address offset: 0x3C
        /// </summary>
        public uint CCR3 { get { return _CCR3; } set { _CCR3 = value; } }

        /// <summary>
        /// TIM capture/compare register 4, Address offset: 0x40 
        /// </summary>
        public uint CCR4 { get { return _CCR4; } set { _CCR4 = value; } }

        /// <summary>
        /// TIM break and dead-time register, Address offset: 0x44
        /// </summary>
        public uint BDTR { get { return _BDTR; } set { _BDTR = value; } }

        /// <summary>
        /// Reserved, 0x46 
        /// </summary>
        public uint RESERVED11 { get { return _RESERVED11; } }

        /// <summary>
        /// TIM DMA control register, Address offset: 0x48
        /// </summary>
        public uint DCR { get { return _DCR; } set { _DCR = value; } }

        /// <summary>
        /// Reserved, 0x4A
        /// </summary>
        public uint RESERVED12 { get { return _RESERVED12; } }

        /// <summary>
        /// TIM DMA address for full transfer, Address offset: 0x4C
        /// </summary>
        public uint DMAR { get { return _DMAR; } set { _DMAR = value; } }

        /// <summary>
        /// Reserved, 0x4E 
        /// </summary>
        public uint RESERVED13 { get { return _RESERVED13; } }

        /// <summary>
        /// TIM option register, Address offset: 0x50
        /// </summary>
        public uint OR { get { return _OR; } set { _OR = value; } }

        /// <summary>
        /// Reserved, 0x52
        /// </summary>
        public uint RESERVED14 { get { return _RESERVED14; } }

        #endregion

        #region Timer Registers...

        ////////////////////////////////////////////////////////////////////////////////
        // See P.574 on the TIM1 and TIM8 Register table...                           //
        ////////////////////////////////////////////////////////////////////////////////
        /// <summary>
        /// control register 1 (TIMx_CR1) - Address offset: 0x00 - Reset value: 0x0000
        /// </summary>
        public Register CR1_Register;

        /// <summary>
        /// control register 2 (TIMx_CR2) - Address offset: 0x04 - Reset value: 0x0000
        /// </summary>
        public Register CR2_Register;

        /// <summary>
        /// slave mode control register (TIMx_SMCR) - Address offset: 0x08 - Reset value: 0x0000
        /// </summary>
        public Register SMCR_Register;

        /// <summary>
        /// DMA/interrupt enable register (TIMx_DIER) - Address offset: 0x0C - Reset value: 0x0000
        /// </summary>
        public Register DIER_Register;

        /// <summary>
        /// status register (TIMx_SR) - Address offset: 0x10 - Reset value: 0x0000
        /// </summary>
        public Register SR_Register;

        /// <summary>
        /// event generation register (TIMx_EGR) - Address offset: 0x14 - Reset value: 0x0000
        /// </summary>
        public Register EGR_Register;

        /// <summary>
        /// capture/compare mode register 1 (TIMx_CCMR1) - Address offset: 0x18 - Reset value: 0x0000
        /// </summary>
        public Register CCMR1_Register;

        /// <summary>
        /// capture/compare mode register 2 (TIMx_CCMR2) - Address offset: 0x1C - Reset value: 0x0000
        /// </summary>
        public Register CCMR2_Register;

        /// <summary>
        /// capture/compare enable register (TIMx_CCER) - Address offset: 0x20 - Reset value: 0x0000
        /// </summary>
        public Register CCER_Register;

        /// <summary>
        /// counter (TIMx_CNT) - Address offset: 0x24 - Reset value: 0x0000
        /// </summary>
        public Register CNT_Register;

        /// <summary>
        /// prescaler (TIMx_PSC) - Address offset: 0x28 - Reset value: 0x0000
        /// </summary>
        public Register PSC_Register;

        /// <summary>
        /// auto-reload register (TIMx_ARR) - Address offset: 0x2C - Reset value: 0x0000
        /// </summary>
        public Register ARR_Register;

        /// <summary>
        /// repetition counter register (TIMx_RCR) - Address offset: 0x30 - Reset value: 0x0000
        /// </summary>
        public Register RCR_Register;

        /// <summary>
        /// Capture/compare register 1 (TIMx_CCR1) - Address offset: 0x34 - Reset value: 0x0000
        /// </summary>
        public Register CCR1_Register;

        /// <summary>
        /// Capture/compare register 2 (TIMx_CCR2) - Address offset: 0x38 - Reset value: 0x0000
        /// </summary>
        public Register CCR2_Register;

        /// <summary>
        /// Capture/compare register 3 (TIMx_CCR3) - Address offset: 0x3C - Reset value: 0x0000
        /// </summary>
        public Register CCR3_Register;

        /// <summary>
        /// Capture/compare register 4 (TIMx_CCR4) - Address offset: 0x40 - Reset value: 0x0000
        /// </summary>
        public Register CCR4_Register;

        /// <summary>
        /// Break and dead-time register (TIMx_BDTR) - Address offset: 0x44 - Reset value: 0x0000
        /// </summary>
        public Register BDTR_Register;

        /// <summary>
        /// DMA control register (TIMx_DCR) - Address offset: 0x48 - Reset value: 0x0000
        /// </summary>
        public Register DCR_Register;

        /// <summary>
        /// DMA address for full transfer (TIMx_DMAR) - Address offset: 0x4C - Reset value: 0x0000
        /// </summary>
        public Register DMAR_Register;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////
        // System Timer Configuration                                                         //
        // STM32F4_32B_TIMER = 5                                                              //
        // STM32F4_16B_TIMER = 12                                                             //
        ////////////////////////////////////////////////////////////////////////////////////////
        // public static CPUTimer TIMER1 = new CPUTimer(STM32F4xx.TIMER1_BASE);
        // public static CPUTimer TIMER2 = new CPUTimer(STM32F4xx.TIMER2_BASE);
        // public static CPUTimer TIMER3 = new CPUTimer(STM32F4xx.TIMER3_BASE);
        // public static CPUTimer TIMER4 = new CPUTimer(STM32F4xx.TIMER4_BASE);
        // public static CPUTimer TIMER5 = new CPUTimer(STM32F4xx.TIMER5_BASE);
        // public static CPUTimer TIMER6 = new CPUTimer(STM32F4xx.TIMER6_BASE);
        // public static CPUTimer TIMER7 = new CPUTimer(STM32F4xx.TIMER7_BASE);

        /// <summary>
        /// Provids Access to the Timer 8 Configuration Registers...
        /// See: P.548 for TIM1&TIM8 registers...
        /// </summary>
        public static CPUTimer TIMER8 = new CPUTimer(STM32F4xx.TIM8_BASE);

        // public static CPUTimer TIMER9 = new CPUTimer(STM32F4xx.TIMER9_BASE);
        // public static CPUTimer TIMER10 = new CPUTimer(STM32F4xx.TIMER10_BASE);
        // public static CPUTimer TIMER11 = new CPUTimer(STM32F4xx.TIMER11_BASE);
        // public static Timer TIMER12 = new Timer(STM32F4xx.TIMER12_BASE);
        // public static CPUTimer TIMER13 = new CPUTimer(STM32F4xx.TIMER13_BASE);
        // public static CPUTimer TIMER14 = new CPUTimer(STM32F4xx.TIMER14_BASE);

        public CPUTimer(UInt32 TIMx_BASE)
        {
            // See: P.611 for TIM2 to TIM5 registers...

            CR1_Register = new Register(TIMx_BASE + CR1);
            CR2_Register = new Register(TIMx_BASE + CR2);
            SMCR_Register = new Register(TIMx_BASE + SMCR);
            DIER_Register = new Register(TIMx_BASE + DIER);
            SR_Register = new Register(TIMx_BASE + SR);
            EGR_Register = new Register(TIMx_BASE + EGR);
            CCMR1_Register = new Register(TIMx_BASE + CCMR1);
            CCMR2_Register = new Register(TIMx_BASE + CCMR2);
            CCER_Register = new Register(TIMx_BASE + CCER);
            CNT_Register = new Register(TIMx_BASE + CNT);
            PSC_Register = new Register(TIMx_BASE + PSC);
            ARR_Register = new Register(TIMx_BASE + ARR);
            RCR_Register = new Register(TIMx_BASE + RCR);
            CCR1_Register = new Register(TIMx_BASE + CCR1);
            CCR2_Register = new Register(TIMx_BASE + CCR2);
            CCR3_Register = new Register(TIMx_BASE + CCR3);
            CCR4_Register = new Register(TIMx_BASE + CCR4);
            BDTR_Register = new Register(TIMx_BASE + BDTR);
            DCR_Register = new Register(TIMx_BASE + DCR);
            DMAR_Register = new Register(TIMx_BASE + DMAR);
        }

    } // END of Class...

    /// <summary>
    /// Real-time clock (RTC) - See: P.780
    /// </summary>
    public static class RTC
    {
        #region Private's...

        /// <summary>
        /// RTC time register,Address offset: 0x00 
        /// </summary>
        private static uint _TR = 0x00;

        /// <summary>
        /// RTC date register,Address offset: 0x04 
        /// </summary>
        private static uint _DR = 0x04;

        /// <summary>
        /// RTC control register, Address offset: 0x08 
        /// </summary>
        private static uint _CR = 0x08;

        /// <summary>
        /// RTC initialization and status register, Address offset: 0x0C 
        /// </summary>
        private static uint _ISR = 0x0C;

        /// <summary>
        /// RTC prescaler register, Address offset: 0x10 
        /// </summary>
        private static uint _PRER = 0x10;

        /// <summary>
        /// RTC wakeup timer register,Address offset: 0x14 
        /// </summary>
        private static uint _WUTR = 0x14;

        /// <summary>
        /// RTC calibration register, Address offset: 0x18 
        /// </summary>
        private static uint _CALIBR = 0x18;

        /// <summary>
        /// RTC alarm A register, Address offset: 0x1C 
        /// </summary>
        private static uint _ALRMAR = 0x1C;

        /// <summary>
        /// RTC alarm B register, Address offset: 0x20 
        /// </summary>
        private static uint _ALRMBR = 0x20;

        /// <summary>
        /// RTC write protection register,Address offset: 0x24 
        /// </summary>
        private static uint _WPR = 0x24;

        /// <summary>
        /// RTC sub second register,Address offset: 0x28 
        /// </summary>
        private static uint _SSR = 0x28;

        /// <summary>
        /// RTC shift control register, Address offset: 0x2C 
        /// </summary>
        private static uint _SHIFTR = 0x2C;

        /// <summary>
        /// RTC time stamp time register, Address offset: 0x30 
        /// </summary>
        private static uint _TSTR = 0x30;

        /// <summary>
        /// RTC time stamp date register, Address offset: 0x34 
        /// </summary>
        private static uint _TSDR = 0x34;

        /// <summary>
        /// RTC time-stamp sub second register, Address offset: 0x38 
        /// </summary>
        private static uint _TSSSR = 0x38;

        /// <summary>
        /// RTC calibration register, Address offset: 0x3C 
        /// </summary>
        private static uint _CALR = 0x3C;

        /// <summary>
        /// RTC tamper and alternate function configuration register, Address offset: 0x40 
        /// </summary>
        private static uint _TAFCR = 0x40;

        /// <summary>
        ///  RTC alarm A sub second register,Address offset: 0x44 
        /// </summary>
        private static uint _ALRMASSR = 0x44;

        /// <summary>
        ///  RTC alarm B sub second register,Address offset: 0x48 
        /// </summary>
        private static uint _ALRMBSSR = 0x48;

        /// <summary>
        /// RTC backup register 1,Address offset: 0x50 
        /// </summary>
        private static uint _BKP0R = 0x50;

        /// <summary>
        /// RTC backup register 1,Address offset: 0x54 
        /// </summary>
        private static uint _BKP1R = 0x54;

        /// <summary>
        /// RTC backup register 2,Address offset: 0x58 
        /// </summary>
        private static uint _BKP2R = 0x58;

        /// <summary>
        /// RTC backup register 3,Address offset: 0x5C 
        /// </summary>
        private static uint _BKP3R = 0x5C;

        /// <summary>
        /// RTC backup register 4,Address offset: 0x60 
        /// </summary>
        private static uint _BKP4R = 0x60;

        /// <summary>
        /// RTC backup register 5,Address offset: 0x64 
        /// </summary>
        private static uint _BKP5R = 0x64;

        /// <summary>
        /// RTC backup register 6,Address offset: 0x68 
        /// </summary>
        private static uint _BKP6R = 0x68;

        /// <summary>
        /// RTC backup register 7,Address offset: 0x6C 
        /// </summary>
        private static uint _BKP7R = 0x6C;

        /// <summary>
        /// RTC backup register 8,Address offset: 0x70 
        /// </summary>
        private static uint _BKP8R = 0x70;

        /// <summary>
        /// RTC backup register 9,Address offset: 0x74 
        /// </summary>
        private static uint _BKP9R = 0x74;

        /// <summary>
        /// RTC backup register 10, Address offset: 0x78 
        /// </summary>
        private static uint _BKP10R = 0x78;

        /// <summary>
        /// RTC backup register 11, Address offset: 0x7C 
        /// </summary>
        private static uint _BKP11R = 0x7C;

        /// <summary>
        /// RTC backup register 12, Address offset: 0x80 
        /// </summary>
        private static uint _BKP12R = 0x80;

        /// <summary>
        /// RTC backup register 13, Address offset: 0x84 
        /// </summary>
        private static uint _BKP13R = 0x84;

        /// <summary>
        /// RTC backup register 14, Address offset: 0x88 
        /// </summary>
        private static uint _BKP14R = 0x88;

        /// <summary>
        /// RTC backup register 15, Address offset: 0x8C 
        /// </summary>
        private static uint _BKP15R = 0x8C;

        /// <summary>
        /// RTC backup register 16, Address offset: 0x90 
        /// </summary>
        private static uint _BKP16R = 0x90;

        /// <summary>
        /// RTC backup register 17, Address offset: 0x94 
        /// </summary>
        private static uint _BKP17R = 0x94;

        /// <summary>
        /// RTC backup register 18, Address offset: 0x98 
        /// </summary>
        private static uint _BKP18R = 0x98;

        /// <summary>
        /// RTC backup register 19, Address offset: 0x9C 
        /// </summary>
        private static uint _BKP19R = 0x9C;

        #endregion

        #region Properties...

        /// <summary>
        /// RTC time register,Address offset: 0x00 
        /// </summary>
        public static uint TR { get { return _TR; } set { _TR = value; } }

        /// <summary>
        /// RTC date register,Address offset: 0x04 
        /// </summary>
        public static uint DR { get { return _DR; } set { _DR = value; } }

        /// <summary>
        /// RTC control register, Address offset: 0x08 
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// RTC initialization and status register, Address offset: 0x0C 
        /// </summary>
        public static uint ISR { get { return _ISR; } set { _ISR = value; } }

        /// <summary>
        /// RTC prescaler register, Address offset: 0x10 
        /// </summary>
        public static uint PRER { get { return _PRER; } set { _PRER = value; } }

        /// <summary>
        /// RTC wakeup timer register,Address offset: 0x14 
        /// </summary>
        public static uint WUTR { get { return _WUTR; } set { _WUTR = value; } }

        /// <summary>
        /// RTC calibration register, Address offset: 0x18 
        /// </summary>
        public static uint CALIBR { get { return _CALIBR; } set { _CALIBR = value; } }

        /// <summary>
        /// RTC alarm A register, Address offset: 0x1C 
        /// </summary>
        public static uint ALRMAR { get { return _ALRMAR; } set { _ALRMAR = value; } }

        /// <summary>
        /// RTC alarm B register, Address offset: 0x20 
        /// </summary>
        public static uint ALRMBR { get { return _ALRMBR; } set { _ALRMBR = value; } }

        /// <summary>
        /// RTC write protection register,Address offset: 0x24 
        /// </summary>
        public static uint WPR { get { return _WPR; } set { _WPR = value; } }

        /// <summary>
        /// RTC sub second register,Address offset: 0x28 
        /// </summary>
        public static uint SSR { get { return _SSR; } set { _SSR = value; } }

        /// <summary>
        /// RTC shift control register, Address offset: 0x2C 
        /// </summary>
        public static uint SHIFTR { get { return _SHIFTR; } set { _SHIFTR = value; } }

        /// <summary>
        /// RTC time stamp time register, Address offset: 0x30 
        /// </summary>
        public static uint TSTR { get { return _TSTR; } set { _TSTR = value; } }

        /// <summary>
        /// RTC time stamp date register, Address offset: 0x34 
        /// </summary>
        public static uint TSDR { get { return _TSDR; } set { _TSDR = value; } }

        /// <summary>
        /// RTC time-stamp sub second register, Address offset: 0x38 
        /// </summary>
        public static uint TSSSR { get { return _TSSSR; } set { _TSSSR = value; } }

        /// <summary>
        /// RTC calibration register, Address offset: 0x3C 
        /// </summary>
        public static uint CALR { get { return _CALR; } set { _CALR = value; } }

        /// <summary>
        /// RTC tamper and alternate function configuration register, Address offset: 0x40 
        /// </summary>
        public static uint TAFCR { get { return _TAFCR; } set { _TAFCR = value; } }

        /// <summary>
        ///  RTC alarm A sub second register,Address offset: 0x44 
        /// </summary>
        public static uint ALRMASSR { get { return _ALRMASSR; } set { _ALRMASSR = value; } }

        /// <summary>
        ///  RTC alarm B sub second register,Address offset: 0x48 
        /// </summary>
        public static uint ALRMBSSR { get { return _ALRMBSSR; } set { _ALRMBSSR = value; } }

        /// <summary>
        /// RTC backup register 1,Address offset: 0x50 
        /// </summary>
        public static uint BKP0R { get { return _BKP0R; } set { _BKP0R = value; } }

        /// <summary>
        /// RTC backup register 1,Address offset: 0x54 
        /// </summary>
        public static uint BKP1R { get { return _BKP1R; } set { _BKP1R = value; } }

        /// <summary>
        /// RTC backup register 2,Address offset: 0x58 
        /// </summary>
        public static uint BKP2R { get { return _BKP2R; } set { _BKP2R = value; } }

        /// <summary>
        /// RTC backup register 3,Address offset: 0x5C 
        /// </summary>
        public static uint BKP3R { get { return _BKP3R; } set { _BKP3R = value; } }

        /// <summary>
        /// RTC backup register 4,Address offset: 0x60 
        /// </summary>
        public static uint BKP4R { get { return _BKP4R; } set { _BKP4R = value; } }

        /// <summary>
        /// RTC backup register 5,Address offset: 0x64 
        /// </summary>
        public static uint BKP5R { get { return _BKP5R; } set { _BKP5R = value; } }

        /// <summary>
        /// RTC backup register 6,Address offset: 0x68 
        /// </summary>
        public static uint BKP6R { get { return _BKP6R; } set { _BKP6R = value; } }

        /// <summary>
        /// RTC backup register 7,Address offset: 0x6C 
        /// </summary>
        public static uint BKP7R { get { return _BKP7R; } set { _BKP7R = value; } }

        /// <summary>
        /// RTC backup register 8,Address offset: 0x70 
        /// </summary>
        public static uint BKP8R { get { return _BKP8R; } set { _BKP8R = value; } }

        /// <summary>
        /// RTC backup register 9,Address offset: 0x74 
        /// </summary>
        public static uint BKP9R { get { return _BKP9R; } set { _BKP9R = value; } }

        /// <summary>
        /// RTC backup register 10, Address offset: 0x78 
        /// </summary>
        public static uint BKP10R { get { return _BKP10R; } set { _BKP10R = value; } }

        /// <summary>
        /// RTC backup register 11, Address offset: 0x7C 
        /// </summary>
        public static uint BKP11R { get { return _BKP11R; } set { _BKP11R = value; } }

        /// <summary>
        /// RTC backup register 12, Address offset: 0x80 
        /// </summary>
        public static uint BKP12R { get { return _BKP12R; } set { _BKP12R = value; } }

        /// <summary>
        /// RTC backup register 13, Address offset: 0x84 
        /// </summary>
        public static uint BKP13R { get { return _BKP13R; } set { _BKP13R = value; } }

        /// <summary>
        /// RTC backup register 14, Address offset: 0x88 
        /// </summary>
        public static uint BKP14R { get { return _BKP14R; } set { _BKP14R = value; } }

        /// <summary>
        /// RTC backup register 15, Address offset: 0x8C 
        /// </summary>
        public static uint BKP15R { get { return _BKP15R; } set { _BKP15R = value; } }

        /// <summary>
        /// RTC backup register 16, Address offset: 0x90 
        /// </summary>
        public static uint BKP16R { get { return _BKP16R; } set { _BKP16R = value; } }

        /// <summary>
        /// RTC backup register 17, Address offset: 0x94 
        /// </summary>
        public static uint BKP17R { get { return _BKP17R; } set { _BKP17R = value; } }

        /// <summary>
        /// RTC backup register 18, Address offset: 0x98 
        /// </summary>
        public static uint BKP18R { get { return _BKP18R; } set { _BKP18R = value; } }

        /// <summary>
        /// RTC backup register 19, Address offset: 0x9C 
        /// </summary>
        public static uint BKP19R { get { return _BKP19R; } set { _BKP19R = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// RTC time register,Address offset: 0x00 
        /// </summary>
        public static Register TR_Register;

        /// <summary>
        /// RTC date register,Address offset: 0x04 
        /// </summary>
        public static Register DR_Register;

        /// <summary>
        /// RTC control register, Address offset: 0x08 
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// RTC initialization and status register, Address offset: 0x0C 
        /// </summary>
        public static Register ISR_Register;

        /// <summary>
        /// RTC prescaler register, Address offset: 0x10 
        /// </summary>
        public static Register PRER_Register;

        /// <summary>
        /// RTC wakeup timer register,Address offset: 0x14 
        /// </summary>
        public static Register WUTR_Register;

        /// <summary>
        /// RTC calibration register, Address offset: 0x18 
        /// </summary>
        public static Register CALIBR_Register;

        /// <summary>
        /// RTC alarm A register, Address offset: 0x1C 
        /// </summary>
        public static Register ALRMAR_Register;

        /// <summary>
        /// RTC alarm B register, Address offset: 0x20 
        /// </summary>
        public static Register ALRMBR_Register;

        /// <summary>
        /// RTC write protection register,Address offset: 0x24 
        /// </summary>
        public static Register WPR_Register;

        /// <summary>
        /// RTC sub second register,Address offset: 0x28 
        /// </summary>
        public static Register SSR_Register;

        /// <summary>
        /// RTC shift control register, Address offset: 0x2C 
        /// </summary>
        public static Register SHIFTR_Register;

        /// <summary>
        /// RTC time stamp time register, Address offset: 0x30 
        /// </summary>
        public static Register TSTR_Register;

        /// <summary>
        /// RTC time stamp date register, Address offset: 0x34 
        /// </summary>
        public static Register TSDR_Register;

        /// <summary>
        /// RTC time-stamp sub second register, Address offset: 0x38 
        /// </summary>
        public static Register TSSSR_Register;

        /// <summary>
        /// RTC calibration register, Address offset: 0x3C 
        /// </summary>
        public static Register CALR_Register;

        /// <summary>
        /// RTC tamper and alternate function configuration register, Address offset: 0x40 
        /// </summary>
        public static Register TAFCR_Register;

        /// <summary>
        ///  RTC alarm A sub second register,Address offset: 0x44 
        /// </summary>
        public static Register ALRMASSR_Register;

        /// <summary>
        ///  RTC alarm B sub second register,Address offset: 0x48 
        /// </summary>
        public static Register ALRMBSSR_Register;

        /// <summary>
        /// RTC backup register 1,Address offset: 0x50 
        /// </summary>
        public static Register BKP0R_Register;

        /// <summary>
        /// RTC backup register 1,Address offset: 0x54 
        /// </summary>
        public static Register BKP1R_Register;

        /// <summary>
        /// RTC backup register 2,Address offset: 0x58 
        /// </summary>
        public static Register BKP2R_Register;

        /// <summary>
        /// RTC backup register 3,Address offset: 0x5C 
        /// </summary>
        public static Register BKP3R_Register;

        /// <summary>
        /// RTC backup register 4,Address offset: 0x60 
        /// </summary>
        public static Register BKP4R_Register;

        /// <summary>
        /// RTC backup register 5,Address offset: 0x64 
        /// </summary>
        public static Register BKP5R_Register;

        /// <summary>
        /// RTC backup register 6,Address offset: 0x68 
        /// </summary>
        public static Register BKP6R_Register;

        /// <summary>
        /// RTC backup register 7,Address offset: 0x6C 
        /// </summary>
        public static Register BKP7R_Register;

        /// <summary>
        /// RTC backup register 8,Address offset: 0x70 
        /// </summary>
        public static Register BKP8R_Register;

        /// <summary>
        /// RTC backup register 9,Address offset: 0x74 
        /// </summary>
        public static Register BKP9R_Register;

        /// <summary>
        /// RTC backup register 10, Address offset: 0x78 
        /// </summary>
        public static Register BKP10R_Register;

        /// <summary>
        /// RTC backup register 11, Address offset: 0x7C 
        /// </summary>
        public static Register BKP11R_Register;

        /// <summary>
        /// RTC backup register 12, Address offset: 0x80 
        /// </summary>
        public static Register BKP12R_Register;

        /// <summary>
        /// RTC backup register 13, Address offset: 0x84 
        /// </summary>
        public static Register BKP13R_Register;

        /// <summary>
        /// RTC backup register 14, Address offset: 0x88 
        /// </summary>
        public static Register BKP14R_Register;

        /// <summary>
        /// RTC backup register 15, Address offset: 0x8C 
        /// </summary>
        public static Register BKP15R_Register;

        /// <summary>
        /// RTC backup register 16, Address offset: 0x90 
        /// </summary>
        public static Register BKP16R_Register;

        /// <summary>
        /// RTC backup register 17, Address offset: 0x94 
        /// </summary>
        public static Register BKP17R_Register;

        /// <summary>
        /// RTC backup register 18, Address offset: 0x98 
        /// </summary>
        public static Register BKP18R_Register;

        /// <summary>
        /// RTC backup register 19, Address offset: 0x9C 
        /// </summary>
        public static Register BKP19R_Register;

        #endregion

        static RTC()
        {
            TR_Register = new Register(STM32F4xx.RTC_BASE + TR);
            DR_Register = new Register(STM32F4xx.RTC_BASE + DR);
            CR_Register = new Register(STM32F4xx.RTC_BASE + CR);
            ISR_Register = new Register(STM32F4xx.RTC_BASE + ISR);
            PRER_Register = new Register(STM32F4xx.RTC_BASE + PRER);
            WUTR_Register = new Register(STM32F4xx.RTC_BASE + WUTR);
            CALIBR_Register = new Register(STM32F4xx.RTC_BASE + CALIBR);
            ALRMAR_Register = new Register(STM32F4xx.RTC_BASE + ALRMAR);
            ALRMBR_Register = new Register(STM32F4xx.RTC_BASE + ALRMBR);
            WPR_Register = new Register(STM32F4xx.RTC_BASE + WPR);
            SSR_Register = new Register(STM32F4xx.RTC_BASE + SSR);
            SHIFTR_Register = new Register(STM32F4xx.RTC_BASE + SHIFTR);
            TSTR_Register = new Register(STM32F4xx.RTC_BASE + TSTR);
            TSDR_Register = new Register(STM32F4xx.RTC_BASE + TSDR);
            TSSSR_Register = new Register(STM32F4xx.RTC_BASE + TSSSR);
            CALR_Register = new Register(STM32F4xx.RTC_BASE + CALR);
            TAFCR_Register = new Register(STM32F4xx.RTC_BASE + TAFCR);
            ALRMASSR_Register = new Register(STM32F4xx.RTC_BASE + ALRMASSR);
            ALRMBSSR_Register = new Register(STM32F4xx.RTC_BASE + ALRMBSSR);
            BKP0R_Register = new Register(STM32F4xx.RTC_BASE + BKP0R);
            BKP1R_Register = new Register(STM32F4xx.RTC_BASE + BKP1R);
            BKP2R_Register = new Register(STM32F4xx.RTC_BASE + BKP2R);
            BKP3R_Register = new Register(STM32F4xx.RTC_BASE + BKP3R);
            BKP4R_Register = new Register(STM32F4xx.RTC_BASE + BKP4R);
            BKP5R_Register = new Register(STM32F4xx.RTC_BASE + BKP5R);
            BKP6R_Register = new Register(STM32F4xx.RTC_BASE + BKP6R);
            BKP7R_Register = new Register(STM32F4xx.RTC_BASE + BKP7R);
            BKP8R_Register = new Register(STM32F4xx.RTC_BASE + BKP8R);
            BKP9R_Register = new Register(STM32F4xx.RTC_BASE + BKP9R);
            BKP10R_Register = new Register(STM32F4xx.RTC_BASE + BKP10R);
            BKP11R_Register = new Register(STM32F4xx.RTC_BASE + BKP11R);
            BKP12R_Register = new Register(STM32F4xx.RTC_BASE + BKP12R);
            BKP13R_Register = new Register(STM32F4xx.RTC_BASE + BKP13R);
            BKP14R_Register = new Register(STM32F4xx.RTC_BASE + BKP14R);
            BKP15R_Register = new Register(STM32F4xx.RTC_BASE + BKP15R);
            BKP16R_Register = new Register(STM32F4xx.RTC_BASE + BKP16R);
            BKP17R_Register = new Register(STM32F4xx.RTC_BASE + BKP17R);
            BKP18R_Register = new Register(STM32F4xx.RTC_BASE + BKP18R);
            BKP19R_Register = new Register(STM32F4xx.RTC_BASE + BKP19R);
        }
    } // END of Class...

    /// <summary>
    /// Window watchdog (WWDG) - See: P.697
    /// </summary>
    public static class WWDG
    {
        #region Private's...

        /// <summary>
        /// WWDG Control register, Address offset: 0x00 
        /// </summary>
        private static uint _CR = 0x00;

        /// <summary>
        /// WWDG Configuration register, Address offset: 0x04 
        /// </summary>
        private static uint _CFR = 0x04;

        /// <summary>
        /// WWDG Status register,Address offset: 0x08 
        /// </summary>
        private static uint _SR = 0x08;

        #endregion

        #region Properties...

        /// <summary>
        /// WWDG Control register, Address offset: 0x00 
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// WWDG Configuration register, Address offset: 0x04 
        /// </summary>
        public static uint CFR { get { return _CFR; } set { _CFR = value; } }

        /// <summary>
        /// WWDG Status register,Address offset: 0x08 
        /// </summary>
        public static uint SR { get { return _SR; } set { _SR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// WWDG Control register, Address offset: 0x00 
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// WWDG Configuration register, Address offset: 0x04 
        /// </summary>
        public static Register CFR_Register;

        /// <summary>
        /// WWDG Status register,Address offset: 0x08 
        /// </summary>
        public static Register SR_Register;

        #endregion

        static WWDG()
        {
            CR_Register = new Register(STM32F4xx.WWDG_BASE + CR);
            CFR_Register = new Register(STM32F4xx.WWDG_BASE + CFR);
            SR_Register = new Register(STM32F4xx.WWDG_BASE + SR);
        }
    } // END of Class...

    /// <summary>
    /// Independent watchdog (IWDG) - See: P.691
    /// </summary>
    public static class IWDG
    {
        #region Private's...

        /// <summary>
        /// IWDG Key register, Address offset: 0x00 
        /// </summary>
        private static uint _KR = 0x00;

        /// <summary>
        /// IWDG Prescaler register, Address offset: 0x04 
        /// </summary>
        private static uint _PR = 0x04;

        /// <summary>
        /// IWDG Reload register,Address offset: 0x08 
        /// </summary>
        private static uint _RLR = 0x08;

        /// <summary>
        /// IWDG Status register,Address offset: 0x0C 
        /// </summary>
        private static uint _SR = 0x0C;

        #endregion

        #region Properties...

        /// <summary>
        /// IWDG Key register, Address offset: 0x00 
        /// </summary>
        public static uint KR { get { return _KR; } set { _KR = value; } }

        /// <summary>
        /// IWDG Prescaler register, Address offset: 0x04 
        /// </summary>
        public static uint PR { get { return _PR; } set { _PR = value; } }

        /// <summary>
        /// IWDG Reload register,Address offset: 0x08 
        /// </summary>
        public static uint RLR { get { return _RLR; } set { _RLR = value; } }

        /// <summary>
        /// IWDG Status register,Address offset: 0x0C 
        /// </summary>
        public static uint SR { get { return _SR; } set { _SR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// IWDG Key register, Address offset: 0x00 
        /// </summary>
        public static Register KR_Register;

        /// <summary>
        /// IWDG Prescaler register, Address offset: 0x04 
        /// </summary>
        public static Register PR_Register;

        /// <summary>
        /// IWDG Reload register,Address offset: 0x08 
        /// </summary>
        public static Register RLR_Register;

        /// <summary>
        /// IWDG Status register,Address offset: 0x0C 
        /// </summary>
        public static Register SR_Register;

        #endregion

        static IWDG()
        {
            KR_Register = new Register(STM32F4xx.IWDG_BASE + KR);
            PR_Register = new Register(STM32F4xx.IWDG_BASE + PR);
            RLR_Register = new Register(STM32F4xx.IWDG_BASE + RLR);
            SR_Register = new Register(STM32F4xx.IWDG_BASE + SR);
        }
    } // END of Class...

    /// <summary>
    /// Serial peripheral interface (SPI) - See: P.857
    /// </summary>
    public sealed class SPI
    {
        #region Private's...

        /// <summary>
        /// SPI control register 1 (not used in I2S mode),Address offset: 0x00 
        /// </summary>
        private uint _CR1 = 0x00;

        /// <summary>
        /// SPI control register 2, Address offset: 0x04 
        /// </summary>
        private uint _CR2 = 0x04;

        /// <summary>
        /// SPI status register,Address offset: 0x08 
        /// </summary>
        private uint _SR = 0x08;

        /// <summary>
        /// SPI data register,Address offset: 0x0C 
        /// </summary>
        private uint _DR = 0x0C;

        /// <summary>
        /// SPI CRC polynomial register (not used in I2S mode), Address offset: 0x10 
        /// </summary>
        private uint _CRCPR = 0x10;

        /// <summary>
        /// SPI RX CRC register (not used in I2S mode), Address offset: 0x14 
        /// </summary>
        private uint _RXCRCR = 0x14;

        /// <summary>
        /// SPI TX CRC register (not used in I2S mode), Address offset: 0x18 
        /// </summary>
        private uint _TXCRCR = 0x18;

        /// <summary>
        /// SPI_I2S configuration register, Address offset: 0x1C 
        /// </summary>
        private uint _I2SCFGR = 0x1C;

        /// <summary>
        /// SPI_I2S prescaler register, Address offset: 0x20 
        /// </summary>
        private uint _I2SPR = 0x20;

        #endregion

        #region Properties...

        /// <summary>
        /// SPI control register 1 (not used in I2S mode),Address offset: 0x00 
        /// </summary>
        public uint CR1 { get { return _CR1; } set { _CR1 = value; } }

        /// <summary>
        /// SPI control register 2, Address offset: 0x04 
        /// </summary>
        public uint CR2 { get { return _CR2; } set { _CR2 = value; } }

        /// <summary>
        /// SPI status register,Address offset: 0x08 
        /// </summary>
        public uint SR { get { return _SR; } set { _SR = value; } }

        /// <summary>
        /// SPI data register,Address offset: 0x0C 
        /// </summary>
        public uint DR { get { return _DR; } set { _DR = value; } }

        /// <summary>
        /// SPI CRC polynomial register (not used in I2S mode), Address offset: 0x10 
        /// </summary>
        public uint CRCPR { get { return _CRCPR; } set { _CRCPR = value; } }

        /// <summary>
        /// SPI RX CRC register (not used in I2S mode), Address offset: 0x14 
        /// </summary>
        public uint RXCRCR { get { return _RXCRCR; } set { _RXCRCR = value; } }

        /// <summary>
        /// SPI TX CRC register (not used in I2S mode), Address offset: 0x18 
        /// </summary>
        public uint TXCRCR { get { return _TXCRCR; } set { _TXCRCR = value; } }

        /// <summary>
        /// SPI_I2S configuration register, Address offset: 0x1C 
        /// </summary>
        public uint I2SCFGR { get { return _I2SCFGR; } set { _I2SCFGR = value; } }

        /// <summary>
        /// SPI_I2S prescaler register, Address offset: 0x20 
        /// </summary>
        public uint I2SPR { get { return _I2SPR; } set { _I2SPR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// SPI control register 1 (not used in I2S mode),Address offset: 0x00 
        /// </summary>
        public Register CR1_Register;

        /// <summary>
        /// SPI control register 2, Address offset: 0x04 
        /// </summary>
        public Register CR2_Register;

        /// <summary>
        /// SPI status register,Address offset: 0x08 
        /// </summary>
        public Register SR_Register;

        /// <summary>
        /// SPI data register,Address offset: 0x0C 
        /// </summary>
        public Register DR_Register;

        /// <summary>
        /// SPI CRC polynomial register (not used in I2S mode), Address offset: 0x10 
        /// </summary>
        public Register CRCPR_Register;

        /// <summary>
        /// SPI RX CRC register (not used in I2S mode), Address offset: 0x14 
        /// </summary>
        public Register RXCRCR_Register;

        /// <summary>
        /// SPI TX CRC register (not used in I2S mode), Address offset: 0x18 
        /// </summary>
        public Register TXCRCR_Register;

        /// <summary>
        /// SPI_I2S configuration register, Address offset: 0x1C 
        /// </summary>
        public Register I2SCFGR_Register;

        /// <summary>
        /// SPI_I2S prescaler register, Address offset: 0x20 
        /// </summary>
        public Register I2SPR_Register;

        #endregion

        public static SPI SPI1 = new SPI(STM32F4xx.SPI1_BASE);
        public static SPI SPI2 = new SPI(STM32F4xx.SPI2_BASE);
        public static SPI SPI3 = new SPI(STM32F4xx.SPI3_BASE);

        public SPI(UInt32 XX_BASE)
        {
            CR1_Register = new Register(XX_BASE + CR1);
            CR2_Register = new Register(XX_BASE + CR2);
            SR_Register = new Register(XX_BASE + SR);
            DR_Register = new Register(XX_BASE + DR);
            CRCPR_Register = new Register(XX_BASE + CRCPR);
            RXCRCR_Register = new Register(XX_BASE + RXCRCR);
            TXCRCR_Register = new Register(XX_BASE + TXCRCR);
            I2SCFGR_Register = new Register(XX_BASE + I2SCFGR);
            I2SPR_Register = new Register(XX_BASE + I2SPR);
        }
    } // END of Class...

    /// <summary>
    /// Universal synchronous asynchronous receiver transmitter (USART) - See: P.951
    /// </summary>
    public sealed class USART
    {
        #region Private's...



        #endregion

        #region Properties...



        #endregion

        #region Registers...



        #endregion

        public static USART USART1 = new USART(STM32F4xx.USART1_BASE);
        public static USART USART2 = new USART(STM32F4xx.USART2_BASE);
        public static USART USART3 = new USART(STM32F4xx.USART3_BASE);
        public static USART UART4 = new USART(STM32F4xx.UART4_BASE);
        public static USART UART5 = new USART(STM32F4xx.UART5_BASE);
        public static USART USART6 = new USART(STM32F4xx.USART6_BASE);

        public USART(UInt32 XX_BASE)
        {
            // USART2 = ((USART_TypeDef *) USART2_BASE);
        }
    } // END of Class...

    /// <summary>
    /// Inter-integrated circuit (I2C) interface - See: P.820
    /// </summary>
    public sealed class I2C
    {

        #region Private's...



        #endregion

        #region Properties...



        #endregion

        #region Registers...



        #endregion

        public I2C I2C1 = new I2C(STM32F4xx.I2C1_BASE);
        public I2C I2C2 = new I2C(STM32F4xx.I2C2_BASE);
        public I2C I2C3 = new I2C(STM32F4xx.I2C3_BASE);

        public I2C(UInt32 XX_BASE)
        {
            // I2C1 = ((I2C_TypeDef *) I2C1_BASE);
        }
    } // END of Class...

    /// <summary>
    /// Controller area network (bxCAN) - See: P.1059
    /// </summary>
    public sealed class CAN
    {
        #region Private's...


        /// <summary>
        /// CAN master control register, Address offset: 0x00
        /// </summary>
        private uint _MCR = 0x00;


        /// <summary>
        /// CAN master status register,Address offset: 0x04
        /// </summary>
        private uint _MSR = 0x04;


        /// <summary>
        /// CAN transmit status register,Address offset: 0x08
        /// </summary>
        private uint _TSR = 0x08;


        /// <summary>
        /// CAN receive FIFO 0 register, Address offset: 0x0C
        /// </summary>
        private uint _RF0R = 0x0C;


        /// <summary>
        /// CAN receive FIFO 1 register, Address offset: 0x10
        /// </summary>
        private uint _RF1R = 0x10;


        /// <summary>
        /// CAN interrupt enable register, Address offset: 0x14
        /// </summary>
        private uint _IER = 0x14;


        /// <summary>
        /// CAN error status register, Address offset: 0x18
        /// </summary>
        private uint _ESR = 0x18;


        /// <summary>
        /// CAN bit timing register, Address offset: 0x1C
        /// </summary>
        private uint _BTR = 0x1C;


        /// <summary>
        /// CAN filter master register,Address offset: 0x200 
        /// </summary>
        private uint _FMR = 0x200;


        /// <summary>
        /// CAN filter mode register,Address offset: 0x204 
        /// </summary>
        private uint _FM1R = 0x204;


        /// <summary>
        /// CAN filter scale register, Address offset: 0x20C 
        /// </summary>
        private uint _FS1R = 0x20C;


        /// <summary>
        /// CAN filter FIFO assignment register, Address offset: 0x214 
        /// </summary>
        private uint _FFA1R = 0x214;


        /// <summary>
        /// CAN filter activation register,Address offset: 0x21C 
        /// </summary>
        private uint _FA1R = 0x21C;


        #endregion

        #region Properties...


        /// <summary>
        /// CAN master control register, Address offset: 0x00
        /// </summary>
        public uint MCR { get { return _MCR; } set { _MCR = value; } }


        /// <summary>
        /// CAN master status register,Address offset: 0x04
        /// </summary>
        public uint MSR { get { return _MSR; } set { _MSR = value; } }


        /// <summary>
        /// CAN transmit status register,Address offset: 0x08
        /// </summary>
        public uint TSR { get { return _TSR; } set { _TSR = value; } }


        /// <summary>
        /// CAN receive FIFO 0 register, Address offset: 0x0C
        /// </summary>
        public uint RF0R { get { return _RF0R; } set { _RF0R = value; } }


        /// <summary>
        /// CAN receive FIFO 1 register, Address offset: 0x10
        /// </summary>
        public uint RF1R { get { return _RF1R; } set { _RF1R = value; } }


        /// <summary>
        /// CAN interrupt enable register, Address offset: 0x14
        /// </summary>
        public uint IER { get { return _IER; } set { _IER = value; } }


        /// <summary>
        /// CAN error status register, Address offset: 0x18
        /// </summary>
        public uint ESR { get { return _ESR; } set { _ESR = value; } }


        /// <summary>
        /// CAN bit timing register, Address offset: 0x1C
        /// </summary>
        public uint BTR { get { return _BTR; } set { _BTR = value; } }


        /// <summary>
        /// CAN filter master register,Address offset: 0x200 
        /// </summary>
        public uint FMR { get { return _FMR; } set { _FMR = value; } }


        /// <summary>
        /// CAN filter mode register,Address offset: 0x204 
        /// </summary>
        public uint FM1R { get { return _FM1R; } set { _FM1R = value; } }


        /// <summary>
        /// CAN filter scale register, Address offset: 0x20C 
        /// </summary>
        public uint FS1R { get { return _FS1R; } set { _FS1R = value; } }


        /// <summary>
        /// CAN filter FIFO assignment register, Address offset: 0x214 
        /// </summary>
        public uint FFA1R { get { return _FFA1R; } set { _FFA1R = value; } }


        /// <summary>
        /// CAN filter activation register,Address offset: 0x21C 
        /// </summary>
        public uint FA1R { get { return _FA1R; } set { _FA1R = value; } }


        #endregion

        #region Registers...


        /// <summary>
        /// CAN master control register, Address offset: 0x00
        /// </summary>
        public Register MCR_Register;


        /// <summary>
        /// CAN master status register,Address offset: 0x04
        /// </summary>
        public Register MSR_Register;


        /// <summary>
        /// CAN transmit status register,Address offset: 0x08
        /// </summary>
        public Register TSR_Register;


        /// <summary>
        /// CAN receive FIFO 0 register, Address offset: 0x0C
        /// </summary>
        public Register RF0R_Register;


        /// <summary>
        /// CAN receive FIFO 1 register, Address offset: 0x10
        /// </summary>
        public Register RF1R_Register;


        /// <summary>
        /// CAN interrupt enable register, Address offset: 0x14
        /// </summary>
        public Register IER_Register;


        /// <summary>
        /// CAN error status register, Address offset: 0x18
        /// </summary>
        public Register ESR_Register;


        /// <summary>
        /// CAN bit timing register, Address offset: 0x1C
        /// </summary>
        public Register BTR_Register;


        /// <summary>
        /// CAN filter master register,Address offset: 0x200 
        /// </summary>
        public Register FMR_Register;


        /// <summary>
        /// CAN filter mode register,Address offset: 0x204 
        /// </summary>
        public Register FM1R_Register;


        /// <summary>
        /// CAN filter scale register, Address offset: 0x20C 
        /// </summary>
        public Register FS1R_Register;


        /// <summary>
        /// CAN filter FIFO assignment register, Address offset: 0x214 
        /// </summary>
        public Register FFA1R_Register;


        /// <summary>
        /// CAN filter activation register,Address offset: 0x21C 
        /// </summary>
        public Register FA1R_Register;


        #endregion

        public static CAN CAN1 = new CAN(STM32F4xx.CAN1_BASE);
        public static CAN CAN2 = new CAN(STM32F4xx.CAN2_BASE);

        public CAN(UInt32 XX_BASE)
        {
            MCR_Register = new Register(XX_BASE + MCR);
            MSR_Register = new Register(XX_BASE + MSR);
            TSR_Register = new Register(XX_BASE + TSR);
            RF0R_Register = new Register(XX_BASE + RF0R);
            RF1R_Register = new Register(XX_BASE + RF1R);
            IER_Register = new Register(XX_BASE + IER);
            ESR_Register = new Register(XX_BASE + ESR);
            BTR_Register = new Register(XX_BASE + BTR);
            FMR_Register = new Register(XX_BASE + FMR);
            FM1R_Register = new Register(XX_BASE + FM1R);
            FS1R_Register = new Register(XX_BASE + FS1R);
            FFA1R_Register = new Register(XX_BASE + FFA1R);
            FA1R_Register = new Register(XX_BASE + FA1R);
        }
    } // END of Class...

    /// <summary>
    /// Power controller (PWR) - See: P.115
    /// </summary>
    public static class PWR
    {
        #region Private's...

        /// <summary>
        /// PWR power control register,Address offset: 0x00 
        /// </summary>
        private static uint _CR = 0x00;

        /// <summary>
        /// PWR power control/status register, Address offset: 0x04 
        /// </summary>
        private static uint _CSR = 0x04;

        #endregion

        #region Properties...

        /// <summary>
        /// PWR power control register,Address offset: 0x00 
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// PWR power control/status register, Address offset: 0x04 
        /// </summary>
        public static uint CSR { get { return _CSR; } set { _CSR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// PWR power control register,Address offset: 0x00 
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// PWR power control/status register, Address offset: 0x04 
        /// </summary>
        public static Register CSR_Register;

        #endregion

        static PWR()
        {
            CR_Register = new Register(STM32F4xx.PWR_BASE + CR);
            CSR_Register = new Register(STM32F4xx.PWR_BASE + CSR);
        }
    } // END of Class...

    /// <summary>
    /// Analog-to-digital converter (ADC) - See: P.385
    /// </summary>
    public sealed class ADC
    {
        #region Privates...

        private UInt32 _CSR = STM32F4xx.ADC1_BASE + 0x300;
        private UInt32 _CCR = STM32F4xx.ADC1_BASE + 0x304;
        private UInt32 _CDR = STM32F4xx.ADC1_BASE + 0x308;

        private UInt32 _SR = 0x00;
        private UInt32 _CR1 = 0x04;
        private UInt32 _CR2 = 0x08;
        private UInt32 _SMPR1 = 0x0C;
        private UInt32 _SMPR2 = 0x10;
        private UInt32 _JOFR1 = 0x14;
        private UInt32 _JOFR2 = 0x18;
        private UInt32 _JOFR3 = 0x1C;
        private UInt32 _JOFR4 = 0x20;
        private UInt32 _HTR = 0x24;
        private UInt32 _LTR = 0x28;
        private UInt32 _SQR1 = 0x2C;
        private UInt32 _SQR2 = 0x30;
        private UInt32 _SQR3 = 0x34;
        private UInt32 _JSQR = 0x38;
        private UInt32 _JDR1 = 0x3C;
        private UInt32 _JDR2 = 0x40;
        private UInt32 _JDR3 = 0x44;
        private UInt32 _JDR4 = 0x48;
        private UInt32 _DR = 0x4C;

        #endregion

        #region Properties...

        /// <summary>
        /// ADC Common status register, Address offset: ADC1 base address + 0x300 
        /// </summary>
        public UInt32 CSR { get { return _CSR; } set { _CSR = value; } }

        /// <summary>
        /// ADC common control register, Address offset: ADC1 base address + 0x304
        /// </summary>
        public UInt32 CCR { get { return _CCR; } set { _CCR = value; } }

        /// <summary>
        /// ADC common regular data register for dual AND triple modes, Address offset: ADC1 base address + 0x308
        /// </summary>
        public UInt32 CDR { get { return _CDR; } set { _CDR = value; } }

        /// <summary>
        /// ADC status register, Address offset: 0x00
        /// </summary>
        public UInt32 SR { get { return _SR; } set { _SR = value; } }

        /// <summary>
        /// ADC control register 1, Address offset: 0x04
        /// </summary>
        public UInt32 CR1 { get { return _CR1; } set { _CR1 = value; } }

        /// <summary>
        /// ADC control register 2, Address offset: 0x08
        /// </summary>
        public UInt32 CR2 { get { return _CR2; } set { _CR2 = value; } }

        /// <summary>
        /// ADC sample time register 1, Address offset: 0x0C
        /// </summary>
        public UInt32 SMPR1 { get { return _SMPR1; } set { _SMPR1 = value; } }

        /// <summary>
        /// ADC sample time register 2, Address offset: 0x10 
        /// </summary>
        public UInt32 SMPR2 { get { return _SMPR2; } set { _SMPR2 = value; } }

        /// <summary>
        /// ADC injected channel data offset register 1, Address offset: 0x14
        /// </summary>
        public UInt32 JOFR1 { get { return _JOFR1; } set { _JOFR1 = value; } }

        /// <summary>
        /// ADC injected channel data offset register 2, Address offset: 0x18
        /// </summary>
        public UInt32 JOFR2 { get { return _JOFR2; } set { _JOFR2 = value; } }

        /// <summary>
        /// ADC injected channel data offset register 3, Address offset: 0x1C
        /// </summary>
        public UInt32 JOFR3 { get { return _JOFR3; } set { _JOFR3 = value; } }

        /// <summary>
        /// ADC injected channel data offset register 4, Address offset: 0x20 
        /// </summary>
        public UInt32 JOFR4 { get { return _JOFR4; } set { _JOFR4 = value; } }

        /// <summary>
        /// ADC watchdog higher threshold register, Address offset: 0x24
        /// </summary>
        public UInt32 HTR { get { return _HTR; } set { _HTR = value; } }

        /// <summary>
        /// ADC watchdog lower threshold register, Address offset: 0x28
        /// </summary>
        public UInt32 LTR { get { return _LTR; } set { _LTR = value; } }

        /// <summary>
        /// ADC regular sequence register 1, Address offset: 0x2C
        /// </summary>
        public UInt32 SQR1 { get { return _SQR1; } set { _SQR1 = value; } }

        /// <summary>
        /// ADC regular sequence register 2, Address offset: 0x30
        /// </summary>
        public UInt32 SQR2 { get { return _SQR2; } set { _SQR2 = value; } }

        /// <summary>
        /// ADC regular sequence register 3, Address offset: 0x34 
        /// </summary>
        public UInt32 SQR3 { get { return _SQR3; } set { _SQR3 = value; } }

        /// <summary>
        /// ADC injected sequence register, Address offset: 0x38
        /// </summary>
        public UInt32 JSQR { get { return _JSQR; } set { _JSQR = value; } }

        /// <summary>
        /// ADC injected data register 1, Address offset: 0x3C
        /// </summary>
        public UInt32 JDR1 { get { return _JDR1; } set { _JDR1 = value; } }

        /// <summary>
        /// ADC injected data register 2, Address offset: 0x40
        /// </summary>
        public UInt32 JDR2 { get { return _JDR2; } set { _JDR2 = value; } }

        /// <summary>
        /// ADC injected data register 3, Address offset: 0x44
        /// </summary>
        public UInt32 JDR3 { get { return _JDR3; } set { _JDR3 = value; } }

        /// <summary>
        /// ADC injected data register 4, Address offset: 0x48
        /// </summary>
        public UInt32 JDR4 { get { return _JDR4; } set { _JDR4 = value; } }

        /// <summary>
        /// ADC regular data register, Address offset: 0x4C
        /// </summary>
        public UInt32 DR { get { return _DR; } set { _DR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// ADC Common status register, Address offset: ADC1 base address + 0x300 
        /// </summary>
        public Register CSR_Register;

        /// <summary>
        /// ADC common control register, Address offset: ADC1 base address + 0x304
        /// </summary>
        public Register CCR_Register;

        /// <summary>
        /// ADC common regular data register for dual AND triple modes, Address offset: ADC1 base address + 0x308
        /// </summary>
        public Register CDR_Register;

        /// <summary>
        /// ADC status register, Address offset: 0x00
        /// </summary>
        public Register SR_Register;

        /// <summary>
        /// ADC control register 1, Address offset: 0x04
        /// </summary>
        public Register CR1_Register;

        /// <summary>
        /// ADC control register 2, Address offset: 0x08
        /// </summary>
        public Register CR2_Register;

        /// <summary>
        /// ADC sample time register 1, Address offset: 0x0C
        /// </summary>
        public Register SMPR1_Register;

        /// <summary>
        /// ADC sample time register 2, Address offset: 0x10 
        /// </summary>
        public Register SMPR2_Register;

        /// <summary>
        /// ADC injected channel data offset register 1, Address offset: 0x14
        /// </summary>
        public Register JOFR1_Register;

        /// <summary>
        /// ADC injected channel data offset register 2, Address offset: 0x18
        /// </summary>
        public Register JOFR2_Register;

        /// <summary>
        /// ADC injected channel data offset register 3, Address offset: 0x1C
        /// </summary>
        public Register JOFR3_Register;

        /// <summary>
        /// ADC injected channel data offset register 4, Address offset: 0x20 
        /// </summary>
        public Register JOFR4_Register;

        /// <summary>
        /// ADC watchdog higher threshold register, Address offset: 0x24
        /// </summary>
        public Register HTR_Register;

        /// <summary>
        /// ADC watchdog lower threshold register, Address offset: 0x28
        /// </summary>
        public Register LTR_Register;

        /// <summary>
        /// ADC regular sequence register 1, Address offset: 0x2C
        /// </summary>
        public Register SQR1_Register;

        /// <summary>
        /// ADC regular sequence register 2, Address offset: 0x30
        /// </summary>
        public Register SQR2_Register;

        /// <summary>
        /// ADC regular sequence register 3, Address offset: 0x34 
        /// </summary>
        public Register SQR3_Register;

        /// <summary>
        /// ADC injected sequence register, Address offset: 0x38
        /// </summary>
        public Register JSQR_Register;

        /// <summary>
        /// ADC injected data register 1, Address offset: 0x3C
        /// </summary>
        public Register JDR1_Register;

        /// <summary>
        /// ADC injected data register 2, Address offset: 0x40
        /// </summary>
        public Register JDR2_Register;

        /// <summary>
        /// ADC injected data register 3, Address offset: 0x44
        /// </summary>
        public Register JDR3_Register;

        /// <summary>
        /// ADC injected data register 4, Address offset: 0x48
        /// </summary>
        public Register JDR4_Register;

        /// <summary>
        /// ADC regular data register, Address offset: 0x4C
        /// </summary>
        public Register DR_Register;

        #endregion

        public static ADC ADC1 = new ADC(STM32F4xx.ADC1_BASE);
        public static ADC ADC2 = new ADC(STM32F4xx.ADC2_BASE);
        public static ADC ADC3 = new ADC(STM32F4xx.ADC3_BASE);

        public ADC(UInt32 ADCx_BASE)
        {
            CSR_Register = new Register(STM32F4xx.ADC1_BASE + CSR);
            CCR_Register = new Register(STM32F4xx.ADC1_BASE + CCR);
            CDR_Register = new Register(STM32F4xx.ADC1_BASE + CDR);

            SR_Register = new Register(ADCx_BASE + SR);
            CR1_Register = new Register(ADCx_BASE + CR1);
            CR2_Register = new Register(ADCx_BASE + CR2);
            SMPR1_Register = new Register(ADCx_BASE + SMPR1);
            SMPR2_Register = new Register(ADCx_BASE + SMPR2);
            JOFR1_Register = new Register(ADCx_BASE + JOFR1);
            JOFR2_Register = new Register(ADCx_BASE + JOFR2);
            JOFR3_Register = new Register(ADCx_BASE + JOFR3);
            JOFR4_Register = new Register(ADCx_BASE + JOFR4);
            HTR_Register = new Register(ADCx_BASE + HTR);
            LTR_Register = new Register(ADCx_BASE + LTR);
            SQR1_Register = new Register(ADCx_BASE + SQR1);
            SQR2_Register = new Register(ADCx_BASE + SQR2);
            SQR3_Register = new Register(ADCx_BASE + SQR3);
            JSQR_Register = new Register(ADCx_BASE + JSQR);
            JDR1_Register = new Register(ADCx_BASE + JDR1);
            JDR2_Register = new Register(ADCx_BASE + JDR2);
            JDR3_Register = new Register(ADCx_BASE + JDR3);
            JDR4_Register = new Register(ADCx_BASE + JDR4);
            DR_Register = new Register(ADCx_BASE + DR);
        }

    } // END of Class...

    /// <summary>
    /// Digital-to-analog converter (DAC) - See: P.430
    /// </summary>
    public static class DAC
    {
        #region Private's...

        private static byte _CR = 0x00;
        private static byte _SWTRIGR = 0x04;
        private static byte _DHR12R1 = 0x08;
        private static byte _DHR12L1 = 0x0C;
        private static byte _DHR8R1 = 0x10;
        private static byte _DHR12R2 = 0x14;
        private static byte _DHR12L2 = 0x18;
        private static byte _DHR8R2 = 0x1C;
        private static byte _DHR12RD = 0x20;
        private static byte _DHR12LD = 0x24;
        private static byte _DHR8RD = 0x28;
        private static byte _DOR1 = 0x2C;
        private static byte _DOR2 = 0x30;
        private static byte _SR = 0x34;

        #endregion

        #region Properties...

        /// <summary>
        /// DAC control register, Address offset: 0x00
        /// </summary>
        public static byte CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// DAC software trigger register, Address offset: 0x04
        /// </summary>
        public static byte SWTRIGR { get { return _SWTRIGR; } set { _SWTRIGR = value; } }

        /// <summary>
        /// DAC channel1 12-bit right-aligned data holding register, Address offset: 0x08
        /// </summary>
        public static byte DHR12R1 { get { return _DHR12R1; } set { _DHR12R1 = value; } }

        /// <summary>
        /// DAC channel1 12-bit left aligned data holding register,  Address offset: 0x0C
        /// </summary>
        public static byte DHR12L1 { get { return _DHR12L1; } set { _DHR12L1 = value; } }

        /// <summary>
        /// DAC channel1 8-bit right aligned data holding register,  Address offset: 0x10
        /// </summary>
        public static byte DHR8R1 { get { return _DHR8R1; } set { _DHR8R1 = value; } }

        /// <summary>
        /// DAC channel2 12-bit right aligned data holding register, Address offset: 0x14
        /// </summary>
        public static byte DHR12R2 { get { return _DHR12R2; } set { _DHR12R2 = value; } }

        /// <summary>
        /// DAC channel2 12-bit left aligned data holding register,  Address offset: 0x18
        /// </summary>
        public static byte DHR12L2 { get { return _DHR12L2; } set { _DHR12L2 = value; } }

        /// <summary>
        /// DAC channel2 8-bit right-aligned data holding register, Address offset: 0x1C
        /// </summary>
        public static byte DHR8R2 { get { return _DHR8R2; } set { _DHR8R2 = value; } }

        /// <summary>
        /// Dual DAC 12-bit right-aligned data holding register, Address offset: 0x20
        /// </summary>
        public static byte DHR12RD { get { return _DHR12RD; } set { _DHR12RD = value; } }

        /// <summary>
        /// DUAL DAC 12-bit left aligned data holding register, Address offset: 0x24
        /// </summary>
        public static byte DHR12LD { get { return _DHR12LD; } set { _DHR12LD = value; } }

        /// <summary>
        /// DUAL DAC 8-bit right aligned data holding register, Address offset: 0x28
        /// </summary>
        public static byte DHR8RD { get { return _DHR8RD; } set { _DHR8RD = value; } }

        /// <summary>
        /// DAC channel1 data output register, Address offset: 0x2C
        /// </summary>
        public static byte DOR1 { get { return _DOR1; } set { _DOR1 = value; } }

        /// <summary>
        /// DAC channel2 data output register, Address offset: 0x30
        /// </summary>
        public static byte DOR2 { get { return _DOR2; } set { _DOR2 = value; } }

        /// <summary>
        /// DAC status register, Address offset: 0x34
        /// </summary>
        public static byte SR { get { return _SR; } set { _SR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// DAC control register, Address offset: 0x00
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// DAC software trigger register, Address offset: 0x04
        /// </summary>
        public static Register SWTRIGR_Register;

        /// <summary>
        /// DAC channel1 12-bit right-aligned data holding register, Address offset: 0x08
        /// </summary>
        public static Register DHR12R1_Register;

        /// <summary>
        /// DAC channel1 12-bit left aligned data holding register,  Address offset: 0x0C
        /// </summary>
        public static Register DHR12L1_Register;

        /// <summary>
        /// DAC channel1 8-bit right aligned data holding register,  Address offset: 0x10
        /// </summary>
        public static Register DHR8R1_Register;

        /// <summary>
        /// DAC channel2 12-bit right aligned data holding register, Address offset: 0x14
        /// </summary>
        public static Register DHR12R2_Register;

        /// <summary>
        /// DAC channel2 12-bit left aligned data holding register,  Address offset: 0x18
        /// </summary>
        public static Register DHR12L2_Register;

        /// <summary>
        /// DAC channel2 8-bit right-aligned data holding register, Address offset: 0x1C
        /// </summary>
        public static Register DHR8R2_Register;

        /// <summary>
        /// Dual DAC 12-bit right-aligned data holding register, Address offset: 0x20
        /// </summary>
        public static Register DHR12RD_Register;

        /// <summary>
        /// DUAL DAC 12-bit left aligned data holding register, Address offset: 0x24
        /// </summary>
        public static Register DHR12LD_Register;

        /// <summary>
        /// DUAL DAC 8-bit right aligned data holding register, Address offset: 0x28
        /// </summary>
        public static Register DHR8RD_Register;

        /// <summary>
        /// DAC channel1 data output register, Address offset: 0x2C
        /// </summary>
        public static Register DOR1_Register;

        /// <summary>
        /// DAC channel2 data output register, Address offset: 0x30
        /// </summary>
        public static Register DOR2_Register;

        /// <summary>
        /// DAC status register, Address offset: 0x34
        /// </summary>
        public static Register SR_Register;

        #endregion

        static DAC()
        {
            CR_Register = new Register(STM32F4xx.DAC_BASE + CR);
            SWTRIGR_Register = new Register(STM32F4xx.DAC_BASE + SWTRIGR);
            DHR12R1_Register = new Register(STM32F4xx.DAC_BASE + DHR12R1);
            DHR12L1_Register = new Register(STM32F4xx.DAC_BASE + DHR12L1);
            DHR8R1_Register = new Register(STM32F4xx.DAC_BASE + DHR8R1);
            DHR12R2_Register = new Register(STM32F4xx.DAC_BASE + DHR12R2);
            DHR12L2_Register = new Register(STM32F4xx.DAC_BASE + DHR12L2);
            DHR8R2_Register = new Register(STM32F4xx.DAC_BASE + DHR8R2);
            DHR12RD_Register = new Register(STM32F4xx.DAC_BASE + DHR12RD);
            DHR12LD_Register = new Register(STM32F4xx.DAC_BASE + DHR12LD);
            DHR8RD_Register = new Register(STM32F4xx.DAC_BASE + DHR8RD);
            DOR1_Register = new Register(STM32F4xx.DAC_BASE + DOR1);
            DOR2_Register = new Register(STM32F4xx.DAC_BASE + DOR2);
            SR_Register = new Register(STM32F4xx.DAC_BASE + SR);
        }

    } // END of Class...

    /// <summary>
    /// Secure digital input/output interface (SDIO) - See: P.1003
    /// </summary>
    public static class SDIO
    {
        #region Private's...

        /// <summary>
        /// SDIO power control register,Address offset: 0x00 
        /// </summary>
        private static uint _POWER = 0x00;

        /// <summary>
        /// SDI clock control register, Address offset: 0x04 
        /// </summary>
        private static uint _CLKCR = 0x04;

        /// <summary>
        /// SDIO argument register, Address offset: 0x08 
        /// </summary>
        private static uint _ARG = 0x08;

        /// <summary>
        /// SDIO command register,Address offset: 0x0C 
        /// </summary>
        private static uint _CMD = 0x0C;

        /// <summary>
        /// SDIO data timer register, Address offset: 0x24 
        /// </summary>
        private static uint _DTIMER = 0x24;

        /// <summary>
        /// SDIO data length register,Address offset: 0x28 
        /// </summary>
        private static uint _DLEN = 0x28;

        /// <summary>
        /// SDIO data control register, Address offset: 0x2C 
        /// </summary>
        private static uint _DCTRL = 0x2C;

        /// <summary>
        /// SDIO interrupt clear register,Address offset: 0x38 
        /// </summary>
        private static uint _ICR = 0x38;

        /// <summary>
        /// SDIO mask register, Address offset: 0x3C 
        /// </summary>
        private static uint _MASK = 0x3C;

        /// <summary>
        /// SDIO data FIFO register,Address offset: 0x80 
        /// </summary>
        private static uint _FIFO = 0x80;

        #endregion

        #region Properties...

        /// <summary>
        /// SDIO power control register,Address offset: 0x00 
        /// </summary>
        public static uint POWER { get { return _POWER; } set { _POWER = value; } }

        /// <summary>
        /// SDI clock control register, Address offset: 0x04 
        /// </summary>
        public static uint CLKCR { get { return _CLKCR; } set { _CLKCR = value; } }

        /// <summary>
        /// SDIO argument register, Address offset: 0x08 
        /// </summary>
        public static uint ARG { get { return _ARG; } set { _ARG = value; } }

        /// <summary>
        /// SDIO command register,Address offset: 0x0C 
        /// </summary>
        public static uint CMD { get { return _CMD; } set { _CMD = value; } }

        /// <summary>
        /// SDIO data timer register, Address offset: 0x24 
        /// </summary>
        public static uint DTIMER { get { return _DTIMER; } set { _DTIMER = value; } }

        /// <summary>
        /// SDIO data length register,Address offset: 0x28 
        /// </summary>
        public static uint DLEN { get { return _DLEN; } set { _DLEN = value; } }

        /// <summary>
        /// SDIO data control register, Address offset: 0x2C 
        /// </summary>
        public static uint DCTRL { get { return _DCTRL; } set { _DCTRL = value; } }

        /// <summary>
        /// SDIO interrupt clear register,Address offset: 0x38 
        /// </summary>
        public static uint ICR { get { return _ICR; } set { _ICR = value; } }

        /// <summary>
        /// SDIO mask register, Address offset: 0x3C 
        /// </summary>
        public static uint MASK { get { return _MASK; } set { _MASK = value; } }

        /// <summary>
        /// SDIO data FIFO register,Address offset: 0x80 
        /// </summary>
        public static uint FIFO { get { return _FIFO; } set { _FIFO = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// SDIO power control register,Address offset: 0x00 
        /// </summary>
        public static Register POWER_Register;

        /// <summary>
        /// SDI clock control register, Address offset: 0x04 
        /// </summary>
        public static Register CLKCR_Register;

        /// <summary>
        /// SDIO argument register, Address offset: 0x08 
        /// </summary>
        public static Register ARG_Register;

        /// <summary>
        /// SDIO command register,Address offset: 0x0C 
        /// </summary>
        public static Register CMD_Register;

        /// <summary>
        /// SDIO data timer register, Address offset: 0x24 
        /// </summary>
        public static Register DTIMER_Register;

        /// <summary>
        /// SDIO data length register,Address offset: 0x28 
        /// </summary>
        public static Register DLEN_Register;

        /// <summary>
        /// SDIO data control register, Address offset: 0x2C 
        /// </summary>
        public static Register DCTRL_Register;

        /// <summary>
        /// SDIO interrupt clear register,Address offset: 0x38 
        /// </summary>
        public static Register ICR_Register;

        /// <summary>
        /// SDIO mask register, Address offset: 0x3C 
        /// </summary>
        public static Register MASK_Register;

        /// <summary>
        /// SDIO data FIFO register,Address offset: 0x80 
        /// </summary>
        public static Register FIFO_Register;

        #endregion

        static SDIO()
        {
            POWER_Register = new Register(STM32F4xx.SDIO_BASE + POWER);
            CLKCR_Register = new Register(STM32F4xx.SDIO_BASE + CLKCR);
            ARG_Register = new Register(STM32F4xx.SDIO_BASE + ARG);
            CMD_Register = new Register(STM32F4xx.SDIO_BASE + CMD);
            DTIMER_Register = new Register(STM32F4xx.SDIO_BASE + DTIMER);
            DLEN_Register = new Register(STM32F4xx.SDIO_BASE + DLEN);
            DCTRL_Register = new Register(STM32F4xx.SDIO_BASE + DCTRL);
            ICR_Register = new Register(STM32F4xx.SDIO_BASE + ICR);
            MASK_Register = new Register(STM32F4xx.SDIO_BASE + MASK);
            FIFO_Register = new Register(STM32F4xx.SDIO_BASE + FIFO);
        }
    } // END of Class...

    /// <summary>
    /// System configuration controller (SYSCFG) - See: P.286
    /// </summary>
    public static class SYSCFG
    {
        #region Private's...

        /// <summary>
        /// SYSCFG memory remap register,Address offset: 0x00
        /// </summary>
        private static uint _MEMRMP = 0x00;

        /// <summary>
        /// SYSCFG peripheral mode configuration register, Address offset: 0x04
        /// </summary>
        private static uint _PMC = 0x04;

        /// <summary>
        /// SYSCFG external interrupt configuration registers, Address offset: 0x08-0x14 
        /// </summary>
        // private static uint _EXTICR[4] =  0x08-0x14 ;

        /// <summary>
        /// SYSCFG Compensation cell control register, Address offset: 0x20
        /// </summary>
        private static uint _CMPCR = 0x20;

        #endregion

        #region Properties...

        /// <summary>
        /// SYSCFG memory remap register,Address offset: 0x00
        /// </summary>
        public static uint MEMRMP { get { return _MEMRMP; } set { _MEMRMP = value; } }

        /// <summary>
        /// SYSCFG peripheral mode configuration register, Address offset: 0x04
        /// </summary>
        public static uint PMC { get { return _PMC; } set { _PMC = value; } }

        /// <summary>
        /// SYSCFG external interrupt configuration registers, Address offset: 0x08-0x14 
        /// </summary>
        // public static uint EXTICR[4]  { get { return _EXTICR[4]; } set { _EXTICR[4] = value; } }

        /// <summary>
        /// SYSCFG Compensation cell control register, Address offset: 0x20
        /// </summary>
        public static uint CMPCR { get { return _CMPCR; } set { _CMPCR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// SYSCFG memory remap register,Address offset: 0x00
        /// </summary>
        public static Register MEMRMP_Register;

        /// <summary>
        /// SYSCFG peripheral mode configuration register, Address offset: 0x04
        /// </summary>
        public static Register PMC_Register;

        /// <summary>
        /// SYSCFG external interrupt configuration registers, Address offset: 0x08-0x14 
        /// </summary>
        // public static Register EXTICR[4]_Register;

        /// <summary>
        /// SYSCFG Compensation cell control register, Address offset: 0x20
        /// </summary>
        public static Register CMPCR_Register;

        #endregion

        static SYSCFG()
        {
            MEMRMP_Register = new Register(STM32F4xx.SYSCFG_BASE + MEMRMP);
            PMC_Register = new Register(STM32F4xx.SYSCFG_BASE + PMC);
            // EXTICR[4]_Register = new Register(STM32F4xx.SYSCFG_BASE + EXTICR[4]);
            CMPCR_Register = new Register(STM32F4xx.SYSCFG_BASE + CMPCR);
        }
    } // END of Class...

    /// <summary>
    /// Interrupts and events - External interrupt/event controller (EXTI) - See: P.368
    /// </summary>
    public static class EXTI
    {
        #region Private's...

        /// <summary>
        /// EXTI Interrupt mask register,Address offset: 0x00 
        /// </summary>
        private static uint _IMR = 0x00;

        /// <summary>
        /// EXTI Event mask register,Address offset: 0x04 
        /// </summary>
        private static uint _EMR = 0x04;

        /// <summary>
        /// EXTI Rising trigger selection register,Address offset: 0x08 
        /// </summary>
        private static uint _RTSR = 0x08;

        /// <summary>
        /// EXTI Falling trigger selection register, Address offset: 0x0C 
        /// </summary>
        private static uint _FTSR = 0x0C;

        /// <summary>
        /// EXTI Software interrupt event register,Address offset: 0x10 
        /// </summary>
        private static uint _SWIER = 0x10;

        /// <summary>
        /// EXTI Pending register, Address offset: 0x14 
        /// </summary>
        private static uint _PR = 0x14;

        #endregion

        #region Properties...

        /// <summary>
        /// EXTI Interrupt mask register,Address offset: 0x00 
        /// </summary>
        public static uint IMR { get { return _IMR; } set { _IMR = value; } }

        /// <summary>
        /// EXTI Event mask register,Address offset: 0x04 
        /// </summary>
        public static uint EMR { get { return _EMR; } set { _EMR = value; } }

        /// <summary>
        /// EXTI Rising trigger selection register,Address offset: 0x08 
        /// </summary>
        public static uint RTSR { get { return _RTSR; } set { _RTSR = value; } }

        /// <summary>
        /// EXTI Falling trigger selection register, Address offset: 0x0C 
        /// </summary>
        public static uint FTSR { get { return _FTSR; } set { _FTSR = value; } }

        /// <summary>
        /// EXTI Software interrupt event register,Address offset: 0x10 
        /// </summary>
        public static uint SWIER { get { return _SWIER; } set { _SWIER = value; } }

        /// <summary>
        /// EXTI Pending register, Address offset: 0x14 
        /// </summary>
        public static uint PR { get { return _PR; } set { _PR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// EXTI Interrupt mask register,Address offset: 0x00 
        /// </summary>
        public static Register IMR_Register;

        /// <summary>
        /// EXTI Event mask register,Address offset: 0x04 
        /// </summary>
        public static Register EMR_Register;

        /// <summary>
        /// EXTI Rising trigger selection register,Address offset: 0x08 
        /// </summary>
        public static Register RTSR_Register;

        /// <summary>
        /// EXTI Falling trigger selection register, Address offset: 0x0C 
        /// </summary>
        public static Register FTSR_Register;

        /// <summary>
        /// EXTI Software interrupt event register,Address offset: 0x10 
        /// </summary>
        public static Register SWIER_Register;

        /// <summary>
        /// EXTI Pending register, Address offset: 0x14 
        /// </summary>
        public static Register PR_Register;

        #endregion

        static EXTI()
        {
            IMR_Register = new Register(STM32F4xx.EXTI_BASE + IMR);
            EMR_Register = new Register(STM32F4xx.EXTI_BASE + EMR);
            RTSR_Register = new Register(STM32F4xx.EXTI_BASE + RTSR);
            FTSR_Register = new Register(STM32F4xx.EXTI_BASE + FTSR);
            SWIER_Register = new Register(STM32F4xx.EXTI_BASE + SWIER);
            PR_Register = new Register(STM32F4xx.EXTI_BASE + PR);
        }
    } // END of Class...

    /// <summary>
    /// General-purpose I/Os (GPIO) - See: P.265
    /// </summary>
    public sealed class GPIO
    {
        #region Private's...

        private UInt32 _MODER = 0x00;
        private UInt32 _OTYPER = 0x04;
        private UInt32 _OSPEEDR = 0x08;
        private UInt32 _PUPDR = 0x0C;
        private UInt32 _IDR = 0x10;
        private UInt32 _ODR = 0x14;
        private UInt32 _BSRRL = 0x18;
        private UInt32 _BSRRH = 0x1A;
        private UInt32 _LCKR = 0x1C;

        // Alternate Function Number 
        private UInt32[] AFR = new UInt32[] { 0x12, 0x12, 0x22, 0x22, 0x22, 0, 0, 0x32, 0x32, 0x32, 0x32, 0x92, 0x92, 0x92 }; // AF1/2/3/9}

        #endregion

        #region Properties...

        /// <summary>
        /// GPIO port mode register, Address offset: 0x00
        /// </summary>
        public UInt32 MODER { get { return _MODER; } set { _MODER = value; } }

        /// <summary>
        /// GPIO port output type register, Address offset: 0x04
        /// </summary>
        public UInt32 OTYPER { get { return _OTYPER; } set { _OTYPER = value; } }

        /// <summary>
        /// GPIO port output speed register, Address offset: 0x08
        /// </summary>
        public UInt32 OSPEEDR { get { return _OSPEEDR; } set { _OSPEEDR = value; } }

        /// <summary>
        /// GPIO port pull-up/pull-down register, Address offset: 0x0C
        /// </summary>
        public UInt32 PUPDR { get { return _PUPDR; } set { _PUPDR = value; } }

        /// <summary>
        /// GPIO port input data register, Address offset: 0x10 
        /// </summary>
        public UInt32 IDR { get { return _IDR; } set { _IDR = value; } }

        /// <summary>
        /// GPIO port output data register, Address offset: 0x14
        /// </summary>
        public UInt32 ODR { get { return _ODR; } set { _ODR = value; } }

        /// <summary>
        /// GPIO port bit set/reset low register, Address offset: 0x18
        /// </summary>
        public UInt32 BSRRL { get { return _BSRRL; } set { _BSRRL = value; } }

        /// <summary>
        /// GPIO port bit set/reset high register, Address offset: 0x1A
        /// </summary>
        public UInt32 BSRRH { get { return _BSRRH; } set { _BSRRH = value; } }

        /// <summary>
        /// GPIO port configuration lock register, Address offset: 0x1C
        /// </summary>
        public UInt32 LCKR { get { return _LCKR; } set { _LCKR = value; } }

        /// <summary>
        /// GPIO alternate function registers, Address offset: 0x20-0x24
        /// </summary>
        // public UInt32 AFR { get { return _AFR; } set { _AFR = value; } }    

        #endregion

        #region Registers...

        /// <summary>
        /// GPIO port mode register, Address offset: 0x00
        /// </summary>
        public Register MODER_Register;

        /// <summary>
        /// GPIO port output type register, Address offset: 0x04
        /// </summary>
        public Register OTYPER_Register;

        /// <summary>
        /// GPIO port output speed register, Address offset: 0x08
        /// </summary>
        public Register OSPEEDR_Register;

        /// <summary>
        /// GPIO port pull-up/pull-down register, Address offset: 0x0C
        /// </summary>
        public Register PUPDR_Register;

        /// <summary>
        /// GPIO port input data register, Address offset: 0x10 
        /// </summary>
        public Register IDR_Register;

        /// <summary>
        /// GPIO port output data register, Address offset: 0x14
        /// </summary>
        public Register ODR_Register;

        /// <summary>
        /// GPIO port bit set/reset low register, Address offset: 0x18
        /// </summary>
        public Register BSRRL_Register;

        /// <summary>
        /// GPIO port bit set/reset high register, Address offset: 0x1A
        /// </summary>
        public Register BSRRH_Register;

        /// <summary>
        /// GPIO port configuration lock register, Address offset: 0x1C
        /// </summary>
        public Register LCKR_Register;

        /// <summary>
        /// GPIO alternate function registers, Address offset: 0x20-0x24
        /// </summary>
        // public Register AFR[2];    

        #endregion

        /// <summary>
        /// GPIOA. Class Type for defining the GPIOA Register Types...
        /// </summary>
        public static GPIO GPIOA = new GPIO(STM32F4xx.GPIOA_BASE);

        /// <summary>
        /// GPIOB. Class Type for defining the GPIOB Register Types...
        /// </summary>
        public static GPIO GPIOB = new GPIO(STM32F4xx.GPIOB_BASE);

        public GPIO(UInt32 GPIOx_BASE)
        {
            MODER_Register = new Register(GPIOx_BASE + MODER);
            OTYPER_Register = new Register(GPIOx_BASE + OTYPER);
            OSPEEDR_Register = new Register(GPIOx_BASE + OSPEEDR);
            PUPDR_Register = new Register(GPIOx_BASE + PUPDR);
            IDR_Register = new Register(GPIOx_BASE + IDR);
            ODR_Register = new Register(GPIOx_BASE + ODR);
            BSRRL_Register = new Register(GPIOx_BASE + BSRRL);
            BSRRH_Register = new Register(GPIOx_BASE + BSRRH);
            LCKR_Register = new Register(GPIOx_BASE + LCKR);
        }
    } // END of Class...

    /// <summary>
    /// CRC calculation unit - See: P.112
    /// </summary>
    public static class CRC
    {
        #region Private's...

        /// <summary>
        /// CRC Data register, Address offset: 0x00 
        /// </summary>
        private static uint _DR = 0x00;

        /// <summary>
        /// CRC Control register,Address offset: 0x08 
        /// </summary>
        private static uint _CR = 0x08;

        #endregion

        #region Properties...

        /// <summary>
        /// CRC Data register, Address offset: 0x00 
        /// </summary>
        public static uint DR { get { return _DR; } set { _DR = value; } }

        /// <summary>
        /// CRC Control register,Address offset: 0x08 
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// CRC Data register, Address offset: 0x00 
        /// </summary>
        public static Register DR_Register;

        /// <summary>
        /// CRC Control register,Address offset: 0x08 
        /// </summary>
        public static Register CR_Register;

        #endregion

        static CRC()
        {
            DR_Register = new Register(STM32F4xx.CRC_BASE + DR);
            CR_Register = new Register(STM32F4xx.CRC_BASE + CR);
        }
    } // END of Class...

    /// <summary>
    /// Reset and clock control for STM32F42xxx and STM32F43xxx (RCC) - See: P.147
    /// </summary>
    public static class RCC
    {
        #region Private's...

        private static uint _CR = 0x00;
        private static uint _PLLCFGR = 0x04;
        private static uint _CFGR = 0x08;
        private static uint _CIR = 0x0C;
        private static uint _AHB1RSTR = 0x10;
        private static uint _AHB2RSTR = 0x14;
        private static uint _AHB3RSTR = 0x18;
        //private uint _RESERVED0 = 0x1C;
        private static uint _APB1RSTR = 0x20;
        private static uint _APB2RSTR = 0x24;
        //private uint _RESERVED1 = 0x28 - 0x2C;
        private static uint _AHB1ENR = 0x30;
        private static uint _AHB2ENR = 0x34;
        private static uint _AHB3ENR = 0x38;
        //private uint _RESERVED2 = 0x3C;
        private static uint _APB1ENR = 0x40;
        private static uint _APB2ENR = 0x44;
        //private uint _RESERVED3 = 0x48 - 0x4C;
        private static uint _AHB1LPENR = 0x50;
        private static uint _AHB2LPENR = 0x54;
        private static uint _AHB3LPENR = 0x58;
        private static uint _RESERVED4 = 0x5C;
        private static uint _APB1LPENR = 0x60;
        private static uint _APB2LPENR = 0x64;
        //private uint _RESERVED5 = 0x68 - 0x6C;
        private static uint _BDCR = 0x70;
        private static uint _CSR = 0x74;
        //private uint _RESERVED6 = 0x78 - 0x7C;
        private static uint _SSCGR = 0x80;
        private static uint _PLLI2SCFGR = 0x84;

        #endregion

        #region Properties...

        /// <summary>
        /// RCC clock control register, Address offset: 0x00
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// RCC PLL configuration register, Address offset: 0x04
        /// </summary>
        public static uint PLLCFGR { get { return _PLLCFGR; } set { _PLLCFGR = value; } }

        /// <summary>
        /// RCC clock configuration register, Address offset: 0x08
        /// </summary>
        public static uint CFGR { get { return _CFGR; } set { _CFGR = value; } }

        /// <summary>
        /// RCC clock interrupt register, Address offset: 0x0C
        /// </summary>
        public static uint CIR { get { return _CIR; } set { _CIR = value; } }

        /// <summary>
        /// RCC AHB1 peripheral reset register, Address offset: 0x10
        /// </summary>
        public static uint AHB1RSTR { get { return _AHB1RSTR; } set { _AHB1RSTR = value; } }

        /// <summary>
        /// RCC AHB2 peripheral reset register, Address offset: 0x14
        /// </summary>
        public static uint AHB2RSTR { get { return _AHB2RSTR; } set { _AHB2RSTR = value; } }

        /// <summary>
        /// RCC AHB3 peripheral reset register, Address offset: 0x18
        /// </summary>
        public static uint AHB3RSTR { get { return _AHB3RSTR; } set { _AHB3RSTR = value; } }

        /// <summary>
        /// Reserved, 0x1C
        /// </summary>
        // public uint RESERVED0 { get { return _RESERVED0; } set { _RESERVED0 = value; } }       

        /// <summary>
        /// RCC APB1 peripheral reset register, Address offset: 0x20
        /// </summary>
        public static uint APB1RSTR { get { return _APB1RSTR; } set { _APB1RSTR = value; } }

        /// <summary>
        /// RCC APB2 peripheral reset register, Address offset: 0x24
        /// </summary>
        public static uint APB2RSTR { get { return _APB2RSTR; } set { _APB2RSTR = value; } }

        //public uint RESERVED1 { get { return _RESERVED1; } set { _RESERVED1 = value; } }  Reserved, 0x28-0x2C

        /// <summary>
        /// RCC AHB1 peripheral clock register, Address offset: 0x30
        /// </summary>
        public static uint AHB1ENR { get { return _AHB1ENR; } set { _AHB1ENR = value; } }

        /// <summary>
        /// RCC AHB2 peripheral clock register, Address offset: 0x34
        /// </summary>
        public static uint AHB2ENR { get { return _AHB2ENR; } set { _AHB2ENR = value; } }

        /// <summary>
        /// RCC AHB3 peripheral clock register, Address offset: 0x38
        /// </summary>
        public static uint AHB3ENR { get { return _AHB3ENR; } set { _AHB3ENR = value; } }

        /// <summary>
        ///  Reserved, 0x3C
        /// </summary>
        // public uint RESERVED2 { get { return _RESERVED2; } set { _RESERVED2 = value; } }          

        /// <summary>
        /// RCC APB1 peripheral clock enable register, Address offset: 0x40
        /// </summary>
        public static uint APB1ENR { get { return _APB1ENR; } set { _APB1ENR = value; } }

        /// <summary>
        /// RCC APB2 peripheral clock enable register, Address offset: 0x44
        /// </summary>
        public static uint APB2ENR { get { return _APB2ENR; } set { _APB2ENR = value; } }

        /// <summary>
        /// Reserved, 0x48-0x4C
        /// </summary>
        //public uint RESERVED3 { get { return _RESERVED3; } set { _RESERVED3 = value; } }   

        /// <summary>
        /// RCC AHB1 peripheral clock enable in low power mode register, Address offset: 0x50
        /// </summary>
        public static uint AHB1LPENR { get { return _AHB1LPENR; } set { _AHB1LPENR = value; } }

        /// <summary>
        /// RCC AHB2 peripheral clock enable in low power mode register, Address offset: 0x54
        /// </summary>
        public static uint AHB2LPENR { get { return _AHB2LPENR; } set { _AHB2LPENR = value; } }

        /// <summary>
        /// RCC AHB3 peripheral clock enable in low power mode register, Address offset: 0x58
        /// </summary>
        public static uint AHB3LPENR { get { return _AHB3LPENR; } set { _AHB3LPENR = value; } }

        /// <summary>
        /// Reserved, 0x5C
        /// </summary>
        // public uint RESERVED4 { get { return _RESERVED4; } set { _RESERVED4 = value; } }             

        /// <summary>
        /// RCC APB1 peripheral clock enable in low power mode register, Address offset: 0x60
        /// </summary>
        public static uint APB1LPENR { get { return _APB1LPENR; } set { _APB1LPENR = value; } }

        /// <summary>
        /// RCC APB2 peripheral clock enable in low power mode register, Address offset: 0x64
        /// </summary>
        public static uint APB2LPENR { get { return _APB2LPENR; } set { _APB2LPENR = value; } }

        /// <summary>
        /// Reserved, 0x68-0x6C
        /// </summary>
        //public uint RESERVED5 { get { return _RESERVED5; } set { _RESERVED5 = value; } }           

        /// <summary>
        /// RCC Backup domain control register, Address offset: 0x70 
        /// </summary>
        public static uint BDCR { get { return _BDCR; } set { _BDCR = value; } }

        /// <summary>
        /// RCC clock control & status register, Address offset: 0x74
        /// </summary>
        public static uint CSR { get { return _CSR; } set { _CSR = value; } }

        /// <summary>
        /// Reserved, 0x78-0x7C 
        /// </summary>
        // public uint RESERVED6 { get { return _RESERVED6; } set { _RESERVED6 = value; } }            

        /// <summary>
        /// RCC spread spectrum clock generation register, Address offset: 0x80
        /// </summary>
        public static uint SSCGR { get { return _SSCGR; } set { _SSCGR = value; } }

        /// <summary>
        /// RCC PLLI2S configuration register, Address offset: 0x84
        /// </summary>
        public static uint PLLI2SCFGR { get { return _PLLI2SCFGR; } set { _PLLI2SCFGR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// RCC clock control register, Address offset: 0x00
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// RCC PLL configuration register, Address offset: 0x04
        /// </summary>
        public static Register PLLCFGR_Register;

        /// <summary>
        /// RCC clock configuration register, Address offset: 0x08
        /// </summary>
        public static Register CFGR_Register;

        /// <summary>
        /// RCC clock interrupt register, Address offset: 0x0C
        /// </summary>
        public static Register CIR_Register;

        /// <summary>
        /// RCC AHB1 peripheral reset register, Address offset: 0x10
        /// </summary>
        public static Register AHB1RSTR_Register;

        /// <summary>
        /// RCC AHB2 peripheral reset register, Address offset: 0x14
        /// </summary>
        public static Register AHB2RSTR_Register;

        /// <summary>
        /// RCC AHB3 peripheral reset register, Address offset: 0x18
        /// </summary>
        public static Register AHB3RSTR_Register;

        /// <summary>
        /// RCC APB1 peripheral reset register, Address offset: 0x20
        /// </summary>
        public static Register APB1RSTR_Register;

        /// <summary>
        /// RCC APB2 peripheral reset register, Address offset: 0x24
        /// </summary>
        public static Register APB2RSTR_Register;

        /// <summary>
        /// RCC AHB1 peripheral clock register, Address offset: 0x30
        /// </summary>
        public static Register AHB1ENR_Register;

        /// <summary>
        /// RCC AHB2 peripheral clock register, Address offset: 0x34
        /// </summary>
        public static Register AHB2ENR_Register;

        /// <summary>
        /// RCC AHB3 peripheral clock register, Address offset: 0x38
        /// </summary>
        public static Register AHB3ENR_Register;

        /// <summary>
        /// RCC APB1 peripheral clock enable register, Address offset: 0x40
        /// </summary>
        public static Register APB1ENR_Register;

        /// <summary>
        /// RCC APB2 peripheral clock enable register, Address offset: 0x44
        /// </summary>
        public static Register APB2ENR_Register;

        /// <summary>
        /// RCC AHB1 peripheral clock enable in low power mode register, Address offset: 0x50
        /// </summary>
        public static Register AHB1LPENR_Register;

        /// <summary>
        /// RCC AHB2 peripheral clock enable in low power mode register, Address offset: 0x54
        /// </summary>
        public static Register AHB2LPENR_Register;

        /// <summary>
        /// RCC AHB3 peripheral clock enable in low power mode register, Address offset: 0x58
        /// </summary>
        public static Register AHB3LPENR_Register;

        /// <summary>
        /// RCC APB1 peripheral clock enable in low power mode register, Address offset: 0x60
        /// </summary>
        public static Register APB1LPENR_Register;

        /// <summary>
        /// RCC APB2 peripheral clock enable in low power mode register, Address offset: 0x64
        /// </summary>
        public static Register APB2LPENR_Register;

        /// <summary>
        /// RCC Backup domain control register, Address offset: 0x70
        /// </summary>
        public static Register BDCR_Register;

        /// <summary>
        /// RCC clock control and status register, Address offset: 0x74
        /// </summary>
        public static Register CSR_Register;

        /// <summary>
        /// RCC spread spectrum clock generation register, Address offset: 0x80
        /// </summary>
        public static Register SSCGR_Register;

        /// <summary>
        /// RCC PLLI2S configuration register, Address offset: 0x84
        /// </summary>
        public static Register PLLI2SCFGR_Register;

        #endregion

        /// <summary>
        /// Reset and Counter Control. Class Type for defining the Reset and Counter Control Registers...
        /// P.159 for RCC registers...
        /// </summary>

        static RCC()
        {
            CR_Register = new Register(STM32F4xx.RCC_BASE + CR);
            PLLCFGR_Register = new Register(STM32F4xx.RCC_BASE + PLLCFGR);
            CFGR_Register = new Register(STM32F4xx.RCC_BASE + CFGR);
            CIR_Register = new Register(STM32F4xx.RCC_BASE + CIR);
            AHB1RSTR_Register = new Register(STM32F4xx.RCC_BASE + AHB1RSTR);
            AHB2RSTR_Register = new Register(STM32F4xx.RCC_BASE + AHB2RSTR);
            AHB3RSTR_Register = new Register(STM32F4xx.RCC_BASE + AHB3RSTR);
            APB1RSTR_Register = new Register(STM32F4xx.RCC_BASE + APB1RSTR);
            APB2RSTR_Register = new Register(STM32F4xx.RCC_BASE + APB2RSTR);
            AHB1ENR_Register = new Register(STM32F4xx.RCC_BASE + AHB1ENR);
            AHB2ENR_Register = new Register(STM32F4xx.RCC_BASE + AHB2ENR);
            AHB3ENR_Register = new Register(STM32F4xx.RCC_BASE + AHB3ENR);
            APB1ENR_Register = new Register(STM32F4xx.RCC_BASE + APB1ENR);
            APB2ENR_Register = new Register(STM32F4xx.RCC_BASE + APB2ENR);
            AHB1LPENR_Register = new Register(STM32F4xx.RCC_BASE + AHB1LPENR);
            AHB2LPENR_Register = new Register(STM32F4xx.RCC_BASE + AHB2LPENR);
            AHB3LPENR_Register = new Register(STM32F4xx.RCC_BASE + AHB3LPENR);
            APB1LPENR_Register = new Register(STM32F4xx.RCC_BASE + APB1LPENR);
            APB2LPENR_Register = new Register(STM32F4xx.RCC_BASE + APB2LPENR);
            BDCR_Register = new Register(STM32F4xx.RCC_BASE + BDCR);
            CSR_Register = new Register(STM32F4xx.RCC_BASE + CSR);
            SSCGR_Register = new Register(STM32F4xx.RCC_BASE + SSCGR);
            PLLI2SCFGR_Register = new Register(STM32F4xx.RCC_BASE + PLLI2SCFGR);
        }

    } // END of Class...

    /// <summary>
    /// Embedded Flash memory in STM32F42xxx and STM32F43xxx - See: P.76
    /// </summary>
    public static class FLASH
    {
        #region Private's...

        /// <summary>
        /// FLASH access control register, Address offset: 0x00 
        /// </summary>
        private static uint _ACR = 0x00;

        /// <summary>
        /// FLASH key register,Address offset: 0x04 
        /// </summary>
        private static uint _KEYR = 0x04;

        /// <summary>
        /// FLASH option key register, Address offset: 0x08 
        /// </summary>
        private static uint _OPTKEYR = 0x08;

        /// <summary>
        /// FLASH status register, Address offset: 0x0C 
        /// </summary>
        private static uint _SR = 0x0C;

        /// <summary>
        /// FLASH control register,Address offset: 0x10 
        /// </summary>
        private static uint _CR = 0x10;

        /// <summary>
        /// FLASH option control register, Address offset: 0x14 
        /// </summary>
        private static uint _OPTCR = 0x14;

        #endregion

        #region Properties...

        /// <summary>
        /// FLASH access control register, Address offset: 0x00 
        /// </summary>
        public static uint ACR { get { return _ACR; } set { _ACR = value; } }

        /// <summary>
        /// FLASH key register,Address offset: 0x04 
        /// </summary>
        public static uint KEYR { get { return _KEYR; } set { _KEYR = value; } }

        /// <summary>
        /// FLASH option key register, Address offset: 0x08 
        /// </summary>
        public static uint OPTKEYR { get { return _OPTKEYR; } set { _OPTKEYR = value; } }

        /// <summary>
        /// FLASH status register, Address offset: 0x0C 
        /// </summary>
        public static uint SR { get { return _SR; } set { _SR = value; } }

        /// <summary>
        /// FLASH control register,Address offset: 0x10 
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// FLASH option control register, Address offset: 0x14 
        /// </summary>
        public static uint OPTCR { get { return _OPTCR; } set { _OPTCR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// FLASH access control register, Address offset: 0x00 
        /// </summary>
        public static Register ACR_Register;

        /// <summary>
        /// FLASH key register,Address offset: 0x04 
        /// </summary>
        public static Register KEYR_Register;

        /// <summary>
        /// FLASH option key register, Address offset: 0x08 
        /// </summary>
        public static Register OPTKEYR_Register;

        /// <summary>
        /// FLASH status register, Address offset: 0x0C 
        /// </summary>
        public static Register SR_Register;

        /// <summary>
        /// FLASH control register,Address offset: 0x10 
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// FLASH option control register, Address offset: 0x14 
        /// </summary>
        public static Register OPTCR_Register;

        #endregion

        static FLASH()
        {
            ACR_Register = new Register(STM32F4xx.FLASH_BASE + ACR);
            KEYR_Register = new Register(STM32F4xx.FLASH_BASE + KEYR);
            OPTKEYR_Register = new Register(STM32F4xx.FLASH_BASE + OPTKEYR);
            SR_Register = new Register(STM32F4xx.FLASH_BASE + SR);
            CR_Register = new Register(STM32F4xx.FLASH_BASE + CR);
            OPTCR_Register = new Register(STM32F4xx.FLASH_BASE + OPTCR);
        }
    } // END of Class...

    /// <summary>
    /// DMA registers - See: P.321
    /// </summary>
    public sealed class DMA
    {
        #region Private's...

        /// <summary>
        /// DMA low interrupt status register,Address offset: 0x00 
        /// </summary>
        private uint _LISR = 0x00;

        /// <summary>
        /// DMA high interrupt status register, Address offset: 0x04 
        /// </summary>
        private uint _HISR = 0x04;

        /// <summary>
        /// DMA low interrupt flag clear register,Address offset: 0x08 
        /// </summary>
        private uint _LIFCR = 0x08;

        /// <summary>
        /// DMA high interrupt flag clear register, Address offset: 0x0C 
        /// </summary>
        private uint _HIFCR = 0x0C;

        #endregion

        #region Properties...

        /// <summary>
        /// DMA low interrupt status register,Address offset: 0x00 
        /// </summary>
        public uint LISR { get { return _LISR; } set { _LISR = value; } }

        /// <summary>
        /// DMA high interrupt status register, Address offset: 0x04 
        /// </summary>
        public uint HISR { get { return _HISR; } set { _HISR = value; } }

        /// <summary>
        /// DMA low interrupt flag clear register,Address offset: 0x08 
        /// </summary>
        public uint LIFCR { get { return _LIFCR; } set { _LIFCR = value; } }

        /// <summary>
        /// DMA high interrupt flag clear register, Address offset: 0x0C 
        /// </summary>
        public uint HIFCR { get { return _HIFCR; } set { _HIFCR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// DMA low interrupt status register,Address offset: 0x00 
        /// </summary>
        public Register LISR_Register;

        /// <summary>
        /// DMA high interrupt status register, Address offset: 0x04 
        /// </summary>
        public Register HISR_Register;

        /// <summary>
        /// DMA low interrupt flag clear register,Address offset: 0x08 
        /// </summary>
        public Register LIFCR_Register;

        /// <summary>
        /// DMA high interrupt flag clear register, Address offset: 0x0C 
        /// </summary>
        public Register HIFCR_Register;

        #endregion

        public static DMA DMA1 = new DMA(STM32F4xx.DMA1_BASE);
        public static DMA DMA2 = new DMA(STM32F4xx.DMA2_BASE);

        public DMA(UInt32 XX_BASE)
        {
            LISR_Register = new Register(XX_BASE + LISR);
            HISR_Register = new Register(XX_BASE + HISR);
            LIFCR_Register = new Register(XX_BASE + LIFCR);
            HIFCR_Register = new Register(XX_BASE + HIFCR);
        }
    } // END of Class...

    /// <summary>
    /// DMA registers - See: P.321
    /// </summary>
    public sealed class DMA_Stream
    {
        #region Private's...

        /// <summary>
        /// Address offset: 0x10 + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        private uint _CR = (0x10 + 0x18);

        /// <summary>
        /// Address offset: 0x14 + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        private uint _NDTR = (0x14 + 0x18);

        /// <summary>
        /// Address offset: 0x18 + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        private uint _PAR = (0x18 + 0x18);

        /// <summary>
        /// Address offset: 0x1C + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        private uint _M0AR = (0x1C + 0x18);

        /// <summary>
        /// Address offset: 0x20 + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        private uint _M1AR = (0x20 + 0x18);

        /// <summary>
        /// Address offset: 0x24 + 0x24  stream number - Reset value: 0x0000 0021
        /// </summary>
        private uint _FCR = (0x24 + 0x24);

        #endregion

        #region Properties...

        /// <summary>
        /// Address offset: 0x10 + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        public uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// Address offset: 0x14 + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        public uint NDTR { get { return _NDTR; } set { _NDTR = value; } }

        /// <summary>
        /// Address offset: 0x18 + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        public uint PAR { get { return _PAR; } set { _PAR = value; } }

        /// <summary>
        /// Address offset: 0x1C + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        public uint M0AR { get { return _M0AR; } set { _M0AR = value; } }

        /// <summary>
        /// Address offset: 0x20 + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        public uint M1AR { get { return _M1AR; } set { _M1AR = value; } }

        /// <summary>
        /// Address offset: 0x24 + 0x24  stream number - Reset value: 0x0000 0021
        /// </summary>
        public uint FCR { get { return _FCR; } set { _FCR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// DMA stream x configuration register - Address offset: 0x10 + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        public Register CR_Register;

        /// <summary>
        /// DMA stream x number of data register - Address offset: 0x14 + 0x18  stream number - Reset value: 0x0000 0000 
        /// </summary>
        public Register NDTR_Register;

        /// <summary>
        /// DMA stream x peripheral address register - Address offset: 0x18 + 0x18  stream number - Reset value: 0x0000 0000 
        /// </summary>
        public Register PAR_Register;

        /// <summary>
        /// DMA stream x memory 0 address register - Address offset: 0x1C + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        public Register M0AR_Register;

        /// <summary>
        /// DMA stream x memory 1 address register - Address offset: 0x20 + 0x18  stream number - Reset value: 0x0000 0000
        /// </summary>
        public Register M1AR_Register;

        /// <summary>
        /// DMA stream x FIFO control register - Address offset: 0x24 + 0x24  stream number - Reset value: 0x0000 0021
        /// </summary>
        public Register FCR_Register;

        #endregion

        public static DMA_Stream DMA1_Stream0 = new DMA_Stream(STM32F4xx.DMA1_Stream0_BASE, 0);
        public static DMA_Stream DMA1_Stream1 = new DMA_Stream(STM32F4xx.DMA1_Stream1_BASE, 1);
        public static DMA_Stream DMA1_Stream2 = new DMA_Stream(STM32F4xx.DMA1_Stream2_BASE, 2);
        public static DMA_Stream DMA1_Stream3 = new DMA_Stream(STM32F4xx.DMA1_Stream3_BASE, 3);
        public static DMA_Stream DMA1_Stream4 = new DMA_Stream(STM32F4xx.DMA1_Stream4_BASE, 4);
        public static DMA_Stream DMA1_Stream5 = new DMA_Stream(STM32F4xx.DMA1_Stream5_BASE, 5);
        public static DMA_Stream DMA1_Stream6 = new DMA_Stream(STM32F4xx.DMA1_Stream6_BASE, 6);
        public static DMA_Stream DMA1_Stream7 = new DMA_Stream(STM32F4xx.DMA1_Stream7_BASE, 7);

        public static DMA_Stream DMA2_Stream0 = new DMA_Stream(STM32F4xx.DMA2_Stream0_BASE, 0);
        public static DMA_Stream DMA2_Stream1 = new DMA_Stream(STM32F4xx.DMA2_Stream0_BASE, 1);
        public static DMA_Stream DMA2_Stream2 = new DMA_Stream(STM32F4xx.DMA2_Stream1_BASE, 2);
        public static DMA_Stream DMA2_Stream3 = new DMA_Stream(STM32F4xx.DMA2_Stream3_BASE, 3);
        public static DMA_Stream DMA2_Stream4 = new DMA_Stream(STM32F4xx.DMA2_Stream4_BASE, 4);
        public static DMA_Stream DMA2_Stream5 = new DMA_Stream(STM32F4xx.DMA2_Stream5_BASE, 5);
        public static DMA_Stream DMA2_Stream6 = new DMA_Stream(STM32F4xx.DMA2_Stream6_BASE, 6);
        public static DMA_Stream DMA2_Stream7 = new DMA_Stream(STM32F4xx.DMA2_Stream7_BASE, 7);

        public DMA_Stream(UInt32 XX_BASE, uint streamnumber)
        {
            CR_Register = new Register(XX_BASE + (CR * streamnumber));
            NDTR_Register = new Register(XX_BASE + (NDTR * streamnumber));
            PAR_Register = new Register(XX_BASE + (PAR * streamnumber));
            M0AR_Register = new Register(XX_BASE + (M0AR * streamnumber));
            M1AR_Register = new Register(XX_BASE + (M1AR * streamnumber));
            FCR_Register = new Register(XX_BASE + (FCR * streamnumber));
        }
    } // END of Class...

    /// <summary>
    /// Ethernet (ETH): media access control (MAC) with DMA controller - See: P.1104
    /// </summary>
    public static class ETH
    {
        #region Private's...

        /// <summary>
        /// See: P.1173 - Ethernet MAC configuration register - Address offset: 0x0000 - Reset value: 0x0000 8000
        /// </summary>
        private static uint _MACCR = 0x0000;

        /// <summary>
        /// See: P.1177 - Ethernet MAC frame filter register - Address offset: 0x0004 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACFFR = 0x0004;

        /// <summary>
        /// See: P.1178 - Ethernet MAC hash table high register - Address offset: 0x0008 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACHTHR = 0x0008;

        /// <summary>
        /// See P.1179 Ethernet MAC hash table low register - Address offset: 0x000C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACHTLR = 0x000C;

        /// <summary>
        /// See: P.1179 - Ethernet MAC MII address register - Address offset: 0x0010 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACMIIAR = 0x0010;

        /// <summary>
        /// See: P.1180 - Ethernet MAC MII data register - Address offset: 0x0014 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACMIIDR = 0x0014;

        /// <summary>
        /// See: P.1181 - Ethernet MAC flow control register - Address offset: 0x0018 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACFCR = 0x0018;

        /// <summary>
        ///  See: P.1182 - Ethernet MAC VLAN tag register - Address offset: 0x001C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACVLANTR = 0x001C;

        /// <summary>
        /// See: P.1183 - Ethernet MAC remote wakeup frame filter register - Address offset: 0x0028 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACRWUFFR = 0x0028;

        /// <summary>
        /// See: P.1184 - Ethernet MAC PMT control and status register - Address offset: 0x002C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACPMTCSR = 0x002C;

        /// <1185>......

        /// <summary>
        /// See: P.1187 - Ethernet MAC interrupt status register - Address offset: 0x0038 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACSR = 0x0038;

        /// <summary>
        /// See: P.1188 - Ethernet MAC interrupt mask register - Address offset: 0x003C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MACIMR = 0x003C;

        /// <summary>
        /// See: P.1188 - Ethernet MAC address 0 high register - Address offset: 0x0040 - Reset value: 0x8000 FFFF
        /// </summary>
        private static uint _MACA0HR = 0x0040;

        /// <summary>
        /// See: P.1189 - Ethernet MAC address 0 low register - Address offset: 0x0044 - Reset value: 0xFFFF FFFF
        /// </summary>
        private static uint _MACA0LR = 0x0044;

        /// <summary>
        /// See: P.1189 - Ethernet MAC address 1 high register - Address offset: 0x0048 - Reset value: 0x0000 FFFF
        /// </summary>
        private static uint _MACA1HR = 0x0048;

        /// <summary>
        /// See: P.1190 - Ethernet MAC address1 low register - Address offset: 0x004C - Reset value: 0xFFFF FFFF
        /// </summary>
        private static uint _MACA1LR = 0x004C;

        /// <summary>
        /// See: P.1190 - Ethernet MAC address 2 high register - Address offset: 0x0050 - Reset value: 0x0000 FFFF
        /// </summary>
        private static uint _MACA2HR = 0x0050;

        /// <summary>
        /// See: P.1191 - Ethernet MAC address 2 low register - Address offset: 0x0054 - Reset value: 0xFFFF FFFF
        /// </summary>
        private static uint _MACA2LR = 0x0054;

        /// <summary>
        /// See: P.1191 - Ethernet MAC address 3 high register - Address offset: 0x0058 - Reset value: 0x0000 FFFF
        /// </summary>
        private static uint _MACA3HR = 0x0058;

        /// <summary>
        /// See: P.1192 - Ethernet MAC address 3 low register - Address offset: 0x005C - Reset value: 0xFFFF FFFF
        /// </summary>
        private static uint _MACA3LR = 0x005C;

        /// <summary>
        /// See: P.1193 - Ethernet MMC control register - Address offset: 0x0100 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCCR = 0x0100;

        /// <summary>
        /// See: P.1193 - Ethernet MMC receive interrupt register - Address offset: 0x0104 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCRIR = 0x0104;

        /// <summary>
        /// See: P.1194 - Ethernet MMC transmit interrupt register - Address offset: 0x0108 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCTIR = 0x0108;

        /// <summary>
        /// See: P.1195 - Ethernet MMC receive interrupt mask register - Address offset: 0x010C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCRIMR = 0x010C;

        /// <summary>
        /// See: P.1195 - Ethernet MMC transmit interrupt mask register - Address offset: 0x0110 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCTIMR = 0x0110;

        /// <summary>
        /// See: P.1196 - Ethernet MMC transmitted good frames after a single collision counter register - Address offset: 0x014C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCTGFSCCR = 0x014C;

        /// <summary>
        /// See: P.1196 - Ethernet MMC transmitted good frames after more than a single collision counter register - Address offset: 0x0150 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCTGFMSCCR = 0x0150;

        /// <summary>
        /// See: P.1197 - Ethernet MMC transmitted good frames counter register - Address offset: 0x0168 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCTGFCR = 0x0168;

        /// <summary>
        /// See: P.1197 - Ethernet MMC received frames with CRC error counter register - Address offset: 0x0194 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCRFCECR = 0x0194;

        /// <summary>
        /// See: P.1197 - Ethernet MMC received frames with alignment error counter register - Address offset: 0x0198 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCRFAECR = 0x0198;

        /// <summary>
        /// See: P.1198 - MMC received good unicast frames counter register - Address offset: 0x01C4 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _MMCRGUFCR = 0x01C4;

        /// <summary>
        /// See: P.1198 - Ethernet PTP time stamp control register - Address offset: 0x0700 - Reset value: 0x0000 00002000
        /// </summary>
        private static uint _PTPTSCR = 0x0700;

        /// <summary>
        /// See: P.1200 - Ethernet PTP subsecond increment register - Address offset: 0x0704 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _PTPSSIR = 0x0704;

        /// <summary>
        /// See: P.1201 - Ethernet PTP time stamp high register - Address offset: 0x0708 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _PTPTSHR = 0x0708;

        /// <summary>
        /// See: P.1201 - Ethernet PTP time stamp low register - Address offset: 0x070C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _PTPTSLR = 0x070C;

        /// <summary>
        /// See: P.1201 - Ethernet PTP time stamp high update register - Address offset: 0x0710 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _PTPTSHUR = 0x0710;

        /// <summary>
        /// See: P.1203 - Ethernet PTP time stamp low update register - Address offset: 0x0714 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _PTPTSLUR = 0x0714;

        /// <summary>
        /// See: P.1203 - Ethernet PTP time stamp addend register - Address offset: 0x0718 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _PTPTSAR = 0x0718;

        /// <summary>
        /// See: P.1204 - Ethernet PTP target time high register - Address offset: 0x071C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _PTPTTHR = 0x071C;

        /// <summary>
        /// See: P.1204 - Ethernet PTP target time low register - Address offset: 0x0720 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _PTPTTLR = 0x0720;

        /// <summary>
        /// See: P.1204 - Ethernet PTP time stamp status register - Address offset: 0x0728 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _PTPTSSR = 0x0728;

        /// <1205>

        /// <summary>
        /// See: P.1205 - Ethernet DMA bus mode register - Address offset: 0x1000 - Reset value: 0x0002 0101
        /// </summary>
        private static uint _DMABMR = 0x1000;

        /// <summary>
        /// See: P.1207 - Ethernet DMA transmit poll demand register - Address offset: 0x1004 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMATPDR = 0x1004;

        /// <summary>
        /// See: P.1208 - EHERNET DMA receive poll demand register - Address offset: 0x1008 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMARPDR = 0x1008;

        /// <summary>
        /// See: P.1208 - Ethernet DMA receive descriptor list address register - Address offset: 0x100C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMARDLAR = 0x100C;

        /// <summary>
        /// See: P.1209 - Ethernet DMA transmit descriptor list address register - Address offset: 0x1010 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMATDLAR = 0x1010;

        /// <summary>
        /// See: P.1209 - Ethernet DMA status register - Address offset: 0x1014 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMASR = 0x1014;

        /// <summary>
        /// See: P.1212 - Ethernet DMA operation mode register - Address offset: 0x1018 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMAOMR = 0x1018;

        /// <summary>
        /// See: P.1215 - Ethernet DMA interrupt enable register - Address offset: 0x101C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMAIER = 0x101C;

        /// <summary>
        /// See: P.1217 - Ethernet DMA missed frame and buffer overflow counter register - Address offset: 0x1020 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMAMFBOCR = 0x1020;

        /// <summary>
        /// See: P.1217 - Ethernet DMA receive status watchdog timer register - Address offset: 0x1024 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMARSWTR = 0x1024;

        /// <summary>
        /// See: P.1218 - Ethernet DMA current host transmit descriptor register - Address offset: 0x1048 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMACHTDR = 0x1048;

        /// <summary>
        /// See: P.1218 - Ethernet DMA current host receive descriptor register - Address offset: 0x104C - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMACHRDR = 0x104C;

        /// <summary>
        /// See: P.1218 - Ethernet DMA current host transmit buffer address register - Address offset: 0x1050 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMACHTBAR = 0x1050;

        /// <summary>
        /// See: P.1219 - Ethernet DMA current host receive buffer address register - Address offset: 0x1054 - Reset value: 0x0000 0000
        /// </summary>
        private static uint _DMACHRBAR = 0x1054;

        #endregion

        #region Properties...

        // <summary>
        /// See: P.1173 - Ethernet MAC configuration register - Address offset: 0x0000 - Reset value: 0x0000 8000
        /// </summary>
        private static uint MACCR { get { return _MACCR; } set { _MACCR = value; } }

        /// <summary>
        /// See: P.1177 - Ethernet MAC frame filter register - Address offset: 0x0004 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACFFR { get { return _MACFFR; } set { _MACFFR = value; } }

        /// <summary>
        /// See: P.1178 - Ethernet MAC hash table high register - Address offset: 0x0008 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACHTHR { get { return _MACHTHR; } set { _MACHTHR = value; } }

        /// <summary>
        /// See P.1179 Ethernet MAC hash table low register - Address offset: 0x000C - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACHTLR { get { return _MACHTLR; } set { _MACHTLR = value; } }

        /// <summary>
        /// See: P.1179 - Ethernet MAC MII address register - Address offset: 0x0010 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACMIIAR { get { return _MACMIIAR; } set { _MACMIIAR = value; } }

        /// <summary>
        /// See: P.1180 - Ethernet MAC MII data register - Address offset: 0x0014 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACMIIDR { get { return _MACMIIDR; } set { _MACMIIDR = value; } }

        /// <summary>
        /// See: P.1181 - Ethernet MAC flow control register - Address offset: 0x0018 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACFCR { get { return _MACFCR; } set { _MACFCR = value; } }

        /// <summary>
        ///  See: P.1182 - Ethernet MAC VLAN tag register - Address offset: 0x001C - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACVLANTR { get { return _MACVLANTR; } set { _MACVLANTR = value; } }

        /// <summary>
        /// See: P.1183 - Ethernet MAC remote wakeup frame filter register - Address offset: 0x0028 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACRWUFFR { get { return _MACRWUFFR; } set { _MACRWUFFR = value; } }

        /// <summary>
        /// See: P.1184 - Ethernet MAC PMT control and status register - Address offset: 0x002C - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACPMTCSR { get { return _MACPMTCSR; } set { _MACPMTCSR = value; } }

        /// <summary>
        /// See: P.1187 - Ethernet MAC interrupt status register - Address offset: 0x0038 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACSR { get { return _MACSR; } set { _MACSR = value; } }

        /// <summary>
        /// See: P.1188 - Ethernet MAC interrupt mask register - Address offset: 0x003C - Reset value: 0x0000 0000
        /// </summary>
        private static uint MACIMR { get { return _MACIMR; } set { _MACIMR = value; } }

        /// <summary>
        /// See: P.1188 - Ethernet MAC address 0 high register - Address offset: 0x0040 - Reset value: 0x8000 FFFF
        /// </summary>
        private static uint MACA0HR { get { return _MACA0HR; } set { _MACA0HR = value; } }

        /// <summary>
        /// See: P.1189 - Ethernet MAC address 0 low register - Address offset: 0x0044 - Reset value: 0xFFFF FFFF
        /// </summary>
        private static uint MACA0LR { get { return _MACA0LR; } set { _MACA0LR = value; } }

        /// <summary>
        /// See: P.1189 - Ethernet MAC address 1 high register - Address offset: 0x0048 - Reset value: 0x0000 FFFF
        /// </summary>
        private static uint MACA1HR { get { return _MACA1HR; } set { _MACA1HR = value; } }

        /// <summary>
        /// See: P.1190 - Ethernet MAC address1 low register - Address offset: 0x004C - Reset value: 0xFFFF FFFF
        /// </summary>
        private static uint MACA1LR { get { return _MACA1LR; } set { _MACA1LR = value; } }

        /// <summary>
        /// See: P.1190 - Ethernet MAC address 2 high register - Address offset: 0x0050 - Reset value: 0x0000 FFFF
        /// </summary>
        private static uint MACA2HR { get { return _MACA2HR; } set { _MACA2HR = value; } }

        /// <summary>
        /// See: P.1191 - Ethernet MAC address 2 low register - Address offset: 0x0054 - Reset value: 0xFFFF FFFF
        /// </summary>
        private static uint MACA2LR { get { return _MACA2LR; } set { _MACA2LR = value; } }

        /// <summary>
        /// See: P.1191 - Ethernet MAC address 3 high register - Address offset: 0x0058 - Reset value: 0x0000 FFFF
        /// </summary>
        private static uint MACA3HR { get { return _MACA3HR; } set { _MACA3HR = value; } }

        /// <summary>
        /// See: P.1192 - Ethernet MAC address 3 low register - Address offset: 0x005C - Reset value: 0xFFFF FFFF
        /// </summary>
        private static uint MACA3LR { get { return _MACA3LR; } set { _MACA3LR = value; } }

        /// <summary>
        /// See: P.1193 - Ethernet MMC control register - Address offset: 0x0100 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCCR { get { return _MMCCR; } set { _MMCCR = value; } }

        /// <summary>
        /// See: P.1193 - Ethernet MMC receive interrupt register - Address offset: 0x0104 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCRIR { get { return _MMCRIR; } set { _MMCRIR = value; } }

        /// <summary>
        /// See: P.1194 - Ethernet MMC transmit interrupt register - Address offset: 0x0108 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCTIR { get { return _MMCTIR; } set { _MMCTIR = value; } }

        /// <summary>
        /// See: P.1195 - Ethernet MMC receive interrupt mask register - Address offset: 0x010C - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCRIMR { get { return _MMCRIMR; } set { _MMCRIMR = value; } }

        /// <summary>
        /// See: P.1195 - Ethernet MMC transmit interrupt mask register - Address offset: 0x0110 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCTIMR { get { return _MMCTIMR; } set { _MMCTIMR = value; } }

        /// <summary>
        /// See: P.1196 - Ethernet MMC transmitted good frames after a single collision counter register - Address offset: 0x014C - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCTGFSCCR { get { return _MMCTGFSCCR; } set { _MMCTGFSCCR = value; } }

        /// <summary>
        /// See: P.1196 - Ethernet MMC transmitted good frames after more than a single collision counter register - Address offset: 0x0150 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCTGFMSCCR { get { return _MMCTGFMSCCR; } set { _MMCTGFMSCCR = value; } }

        /// <summary>
        /// See: P.1197 - Ethernet MMC transmitted good frames counter register - Address offset: 0x0168 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCTGFCR { get { return _MMCTGFCR; } set { _MMCTGFCR = value; } }

        /// <summary>
        /// See: P.1197 - Ethernet MMC received frames with CRC error counter register - Address offset: 0x0194 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCRFCECR { get { return _MMCRFCECR; } set { _MMCRFCECR = value; } }

        /// <summary>
        /// See: P.1197 - Ethernet MMC received frames with alignment error counter register - Address offset: 0x0198 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCRFAECR { get { return _MMCRFAECR; } set { _MMCRFAECR = value; } }

        /// <summary>
        /// See: P.1198 - MMC received good unicast frames counter register - Address offset: 0x01C4 - Reset value: 0x0000 0000
        /// </summary>
        private static uint MMCRGUFCR { get { return _MMCRGUFCR; } set { _MMCRGUFCR = value; } }

        /// <summary>
        /// See: P.1198 - Ethernet PTP time stamp control register - Address offset: 0x0700 - Reset value: 0x0000 00002000
        /// </summary>
        private static uint PTPTSCR { get { return _PTPTSCR; } set { _PTPTSCR = value; } }

        /// <summary>
        /// See: P.1200 - Ethernet PTP subsecond increment register - Address offset: 0x0704 - Reset value: 0x0000 0000
        /// </summary>
        private static uint PTPSSIR { get { return _PTPSSIR; } set { _PTPSSIR = value; } }

        /// <summary>
        /// See: P.1201 - Ethernet PTP time stamp high register - Address offset: 0x0708 - Reset value: 0x0000 0000
        /// </summary>
        private static uint PTPTSHR { get { return _PTPTSHR; } set { _PTPTSHR = value; } }

        /// <summary>
        /// See: P.1201 - Ethernet PTP time stamp low register - Address offset: 0x070C - Reset value: 0x0000 0000
        /// </summary>
        private static uint PTPTSLR { get { return _PTPTSLR; } set { _PTPTSLR = value; } }

        /// <summary>
        /// See: P.1201 - Ethernet PTP time stamp high update register - Address offset: 0x0710 - Reset value: 0x0000 0000
        /// </summary>
        private static uint PTPTSHUR { get { return _PTPTSHUR; } set { _PTPTSHUR = value; } }

        /// <summary>
        /// See: P.1203 - Ethernet PTP time stamp low update register - Address offset: 0x0714 - Reset value: 0x0000 0000
        /// </summary>
        private static uint PTPTSLUR { get { return _PTPTSLUR; } set { _PTPTSLUR = value; } }

        /// <summary>
        /// See: P.1203 - Ethernet PTP time stamp addend register - Address offset: 0x0718 - Reset value: 0x0000 0000
        /// </summary>
        private static uint PTPTSAR { get { return _PTPTSAR; } set { _PTPTSAR = value; } }

        /// <summary>
        /// See: P.1204 - Ethernet PTP target time high register - Address offset: 0x071C - Reset value: 0x0000 0000
        /// </summary>
        private static uint PTPTTHR { get { return _PTPTTHR; } set { _PTPTTHR = value; } }

        /// <summary>
        /// See: P.1204 - Ethernet PTP target time low register - Address offset: 0x0720 - Reset value: 0x0000 0000
        /// </summary>
        private static uint PTPTTLR { get { return _PTPTTLR; } set { _PTPTTLR = value; } }

        /// <summary>
        /// See: P.1204 - Ethernet PTP time stamp status register - Address offset: 0x0728 - Reset value: 0x0000 0000
        /// </summary>
        private static uint PTPTSSR { get { return _PTPTSSR; } set { _PTPTSSR = value; } }

        /// <summary>
        /// See: P.1205 - Ethernet DMA bus mode register - Address offset: 0x1000 - Reset value: 0x0002 0101
        /// </summary>
        private static uint DMABMR { get { return _DMABMR; } set { _DMABMR = value; } }

        /// <summary>
        /// See: P.1207 - Ethernet DMA transmit poll demand register - Address offset: 0x1004 - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMATPDR { get { return _DMATPDR; } set { _DMATPDR = value; } }

        /// <summary>
        /// See: P.1208 - EHERNET DMA receive poll demand register - Address offset: 0x1008 - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMARPDR { get { return _DMARPDR; } set { _DMARPDR = value; } }

        /// <summary>
        /// See: P.1208 - Ethernet DMA receive descriptor list address register - Address offset: 0x100C - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMARDLAR { get { return _DMARDLAR; } set { _DMARDLAR = value; } }

        /// <summary>
        /// See: P.1209 - Ethernet DMA transmit descriptor list address register - Address offset: 0x1010 - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMATDLAR { get { return _DMATDLAR; } set { _DMATDLAR = value; } }

        /// <summary>
        /// See: P.1209 - Ethernet DMA status register - Address offset: 0x1014 - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMASR { get { return _DMASR; } set { _DMASR = value; } }

        /// <summary>
        /// See: P.1212 - Ethernet DMA operation mode register - Address offset: 0x1018 - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMAOMR { get { return _DMAOMR; } set { _DMAOMR = value; } }

        /// <summary>
        /// See: P.1215 - Ethernet DMA interrupt enable register - Address offset: 0x101C - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMAIER { get { return _DMAIER; } set { _DMAIER = value; } }

        /// <summary>
        /// See: P.1217 - Ethernet DMA missed frame and buffer overflow counter register - Address offset: 0x1020 - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMAMFBOCR { get { return _DMAMFBOCR; } set { _DMAMFBOCR = value; } }

        /// <summary>
        /// See: P.1217 - Ethernet DMA receive status watchdog timer register - Address offset: 0x1024 - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMARSWTR { get { return _DMARSWTR; } set { _DMARSWTR = value; } }

        /// <summary>
        /// See: P.1218 - Ethernet DMA current host transmit descriptor register - Address offset: 0x1048 - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMACHTDR { get { return _DMACHTDR; } set { _DMACHTDR = value; } }

        /// <summary>
        /// See: P.1218 - Ethernet DMA current host receive descriptor register - Address offset: 0x104C - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMACHRDR { get { return _DMACHRDR; } set { _DMACHRDR = value; } }

        /// <summary>
        /// See: P.1218 - Ethernet DMA current host transmit buffer address register - Address offset: 0x1050 - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMACHTBAR { get { return _DMACHTBAR; } set { _DMACHTBAR = value; } }

        /// <summary>
        /// See: P.1219 - Ethernet DMA current host receive buffer address register - Address offset: 0x1054 - Reset value: 0x0000 0000
        /// </summary>
        private static uint DMACHRBAR { get { return _DMACHRBAR; } set { _DMACHRBAR = value; } }

        #endregion

        #region Registers...

        //// <summary>
        /// See: P.1173 - Ethernet MAC configuration register - Address offset: 0x0000 - Reset value: 0x0000 8000
        /// </summary>
        public static Register MACCR_Register;

        /// <summary>
        /// See: P.1177 - Ethernet MAC frame filter register - Address offset: 0x0004 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACFFR_Register;

        /// <summary>
        /// See: P.1178 - Ethernet MAC hash table high register - Address offset: 0x0008 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACHTHR_Register;

        /// <summary>
        /// See P.1179 Ethernet MAC hash table low register - Address offset: 0x000C - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACHTLR_Register;

        /// <summary>
        /// See: P.1179 - Ethernet MAC MII address register - Address offset: 0x0010 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACMIIAR_Register;

        /// <summary>
        /// See: P.1180 - Ethernet MAC MII data register - Address offset: 0x0014 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACMIIDR_Register;

        /// <summary>
        /// See: P.1181 - Ethernet MAC flow control register - Address offset: 0x0018 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACFCR_Register;

        /// <summary>
        ///  See: P.1182 - Ethernet MAC VLAN tag register - Address offset: 0x001C - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACVLANTR_Register;

        /// <summary>
        /// See: P.1183 - Ethernet MAC remote wakeup frame filter register - Address offset: 0x0028 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACRWUFFR_Register;

        /// <summary>
        /// See: P.1184 - Ethernet MAC PMT control and status register - Address offset: 0x002C - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACPMTCSR_Register;

        /// <summary>
        /// See: P.1187 - Ethernet MAC interrupt status register - Address offset: 0x0038 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACSR_Register;

        /// <summary>
        /// See: P.1188 - Ethernet MAC interrupt mask register - Address offset: 0x003C - Reset value: 0x0000 0000
        /// </summary>
        public static Register MACIMR_Register;

        /// <summary>
        /// See: P.1188 - Ethernet MAC address 0 high register - Address offset: 0x0040 - Reset value: 0x8000 FFFF
        /// </summary>
        public static Register MACA0HR_Register;

        /// <summary>
        /// See: P.1189 - Ethernet MAC address 0 low register - Address offset: 0x0044 - Reset value: 0xFFFF FFFF
        /// </summary>
        public static Register MACA0LR_Register;

        /// <summary>
        /// See: P.1189 - Ethernet MAC address 1 high register - Address offset: 0x0048 - Reset value: 0x0000 FFFF
        /// </summary>
        public static Register MACA1HR_Register;

        /// <summary>
        /// See: P.1190 - Ethernet MAC address1 low register - Address offset: 0x004C - Reset value: 0xFFFF FFFF
        /// </summary>
        public static Register MACA1LR_Register;

        /// <summary>
        /// See: P.1190 - Ethernet MAC address 2 high register - Address offset: 0x0050 - Reset value: 0x0000 FFFF
        /// </summary>
        public static Register MACA2HR_Register;

        /// <summary>
        /// See: P.1191 - Ethernet MAC address 2 low register - Address offset: 0x0054 - Reset value: 0xFFFF FFFF
        /// </summary>
        public static Register MACA2LR_Register;

        /// <summary>
        /// See: P.1191 - Ethernet MAC address 3 high register - Address offset: 0x0058 - Reset value: 0x0000 FFFF
        /// </summary>
        public static Register MACA3HR_Register;

        /// <summary>
        /// See: P.1192 - Ethernet MAC address 3 low register - Address offset: 0x005C - Reset value: 0xFFFF FFFF
        /// </summary>
        public static Register MACA3LR_Register;

        /// <summary>
        /// See: P.1193 - Ethernet MMC control register - Address offset: 0x0100 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCCR_Register;

        /// <summary>
        /// See: P.1193 - Ethernet MMC receive interrupt register - Address offset: 0x0104 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCRIR_Register;

        /// <summary>
        /// See: P.1194 - Ethernet MMC transmit interrupt register - Address offset: 0x0108 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCTIR_Register;

        /// <summary>
        /// See: P.1195 - Ethernet MMC receive interrupt mask register - Address offset: 0x010C - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCRIMR_Register;

        /// <summary>
        /// See: P.1195 - Ethernet MMC transmit interrupt mask register - Address offset: 0x0110 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCTIMR_Register;

        /// <summary>
        /// See: P.1196 - Ethernet MMC transmitted good frames after a single collision counter register - Address offset: 0x014C - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCTGFSCCR_Register;

        /// <summary>
        /// See: P.1196 - Ethernet MMC transmitted good frames after more than a single collision counter register - Address offset: 0x0150 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCTGFMSCCR_Register;

        /// <summary>
        /// See: P.1197 - Ethernet MMC transmitted good frames counter register - Address offset: 0x0168 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCTGFCR_Register;

        /// <summary>
        /// See: P.1197 - Ethernet MMC received frames with CRC error counter register - Address offset: 0x0194 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCRFCECR_Register;

        /// <summary>
        /// See: P.1197 - Ethernet MMC received frames with alignment error counter register - Address offset: 0x0198 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCRFAECR_Register;

        /// <summary>
        /// See: P.1198 - MMC received good unicast frames counter register - Address offset: 0x01C4 - Reset value: 0x0000 0000
        /// </summary>
        public static Register MMCRGUFCR_Register;

        /// <summary>
        /// See: P.1198 - Ethernet PTP time stamp control register - Address offset: 0x0700 - Reset value: 0x0000 00002000
        /// </summary>
        public static Register PTPTSCR_Register;

        /// <summary>
        /// See: P.1200 - Ethernet PTP subsecond increment register - Address offset: 0x0704 - Reset value: 0x0000 0000
        /// </summary>
        public static Register PTPSSIR_Register;

        /// <summary>
        /// See: P.1201 - Ethernet PTP time stamp high register - Address offset: 0x0708 - Reset value: 0x0000 0000
        /// </summary>
        public static Register PTPTSHR_Register;

        /// <summary>
        /// See: P.1201 - Ethernet PTP time stamp low register - Address offset: 0x070C - Reset value: 0x0000 0000
        /// </summary>
        public static Register PTPTSLR_Register;

        /// <summary>
        /// See: P.1201 - Ethernet PTP time stamp high update register - Address offset: 0x0710 - Reset value: 0x0000 0000
        /// </summary>
        public static Register PTPTSHUR_Register;

        /// <summary>
        /// See: P.1203 - Ethernet PTP time stamp low update register - Address offset: 0x0714 - Reset value: 0x0000 0000
        /// </summary>
        public static Register PTPTSLUR_Register;

        /// <summary>
        /// See: P.1203 - Ethernet PTP time stamp addend register - Address offset: 0x0718 - Reset value: 0x0000 0000
        /// </summary>
        public static Register PTPTSAR_Register;

        /// <summary>
        /// See: P.1204 - Ethernet PTP target time high register - Address offset: 0x071C - Reset value: 0x0000 0000
        /// </summary>
        public static Register PTPTTHR_Register;

        /// <summary>
        /// See: P.1204 - Ethernet PTP target time low register - Address offset: 0x0720 - Reset value: 0x0000 0000
        /// </summary>
        public static Register PTPTTLR_Register;

        /// <summary>
        /// See: P.1204 - Ethernet PTP time stamp status register - Address offset: 0x0728 - Reset value: 0x0000 0000
        /// </summary>
        public static Register PTPTSSR_Register;

        /// <summary>
        /// See: P.1205 - Ethernet DMA bus mode register - Address offset: 0x1000 - Reset value: 0x0002 0101
        /// </summary>
        public static Register DMABMR_Register;

        /// <summary>
        /// See: P.1207 - Ethernet DMA transmit poll demand register - Address offset: 0x1004 - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMATPDR_Register;

        /// <summary>
        /// See: P.1208 - EHERNET DMA receive poll demand register - Address offset: 0x1008 - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMARPDR_Register;

        /// <summary>
        /// See: P.1208 - Ethernet DMA receive descriptor list address register - Address offset: 0x100C - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMARDLAR_Register;

        /// <summary>
        /// See: P.1209 - Ethernet DMA transmit descriptor list address register - Address offset: 0x1010 - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMATDLAR_Register;

        /// <summary>
        /// See: P.1209 - Ethernet DMA status register - Address offset: 0x1014 - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMASR_Register;

        /// <summary>
        /// See: P.1212 - Ethernet DMA operation mode register - Address offset: 0x1018 - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMAOMR_Register;

        /// <summary>
        /// See: P.1215 - Ethernet DMA interrupt enable register - Address offset: 0x101C - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMAIER_Register;

        /// <summary>
        /// See: P.1217 - Ethernet DMA missed frame and buffer overflow counter register - Address offset: 0x1020 - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMAMFBOCR_Register;

        /// <summary>
        /// See: P.1217 - Ethernet DMA receive status watchdog timer register - Address offset: 0x1024 - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMARSWTR_Register;

        /// <summary>
        /// See: P.1218 - Ethernet DMA current host transmit descriptor register - Address offset: 0x1048 - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMACHTDR_Register;

        /// <summary>
        /// See: P.1218 - Ethernet DMA current host receive descriptor register - Address offset: 0x104C - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMACHRDR_Register;

        /// <summary>
        /// See: P.1218 - Ethernet DMA current host transmit buffer address register - Address offset: 0x1050 - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMACHTBAR_Register;

        /// <summary>
        /// See: P.1219 - Ethernet DMA current host receive buffer address register - Address offset: 0x1054 - Reset value: 0x0000 0000
        /// </summary>
        public static Register DMACHRBAR_Register;

        #endregion

        static ETH()
        {
            MACCR_Register = new Register(STM32F4xx.ETH_BASE + MACCR);
            MACFFR_Register = new Register(STM32F4xx.ETH_BASE + MACFFR);
            MACHTHR_Register = new Register(STM32F4xx.ETH_BASE + MACHTHR);
            MACHTLR_Register = new Register(STM32F4xx.ETH_BASE + MACHTLR);
            MACMIIAR_Register = new Register(STM32F4xx.ETH_BASE + MACMIIAR);
            MACMIIDR_Register = new Register(STM32F4xx.ETH_BASE + MACMIIDR);
            MACFCR_Register = new Register(STM32F4xx.ETH_BASE + MACFCR);
            MACVLANTR_Register = new Register(STM32F4xx.ETH_BASE + MACVLANTR);
            MACRWUFFR_Register = new Register(STM32F4xx.ETH_BASE + MACRWUFFR);
            MACPMTCSR_Register = new Register(STM32F4xx.ETH_BASE + MACPMTCSR);
            MACSR_Register = new Register(STM32F4xx.ETH_BASE + MACSR);
            MACIMR_Register = new Register(STM32F4xx.ETH_BASE + MACIMR);
            MACA0HR_Register = new Register(STM32F4xx.ETH_BASE + MACA0HR);
            MACA0LR_Register = new Register(STM32F4xx.ETH_BASE + MACA0LR);
            MACA1HR_Register = new Register(STM32F4xx.ETH_BASE + MACA1HR);
            MACA1LR_Register = new Register(STM32F4xx.ETH_BASE + MACA1LR);
            MACA2HR_Register = new Register(STM32F4xx.ETH_BASE + MACA2HR);
            MACA2LR_Register = new Register(STM32F4xx.ETH_BASE + MACA2LR);
            MACA3HR_Register = new Register(STM32F4xx.ETH_BASE + MACA3HR);
            MACA3LR_Register = new Register(STM32F4xx.ETH_BASE + MACA3LR);
            MMCCR_Register = new Register(STM32F4xx.ETH_BASE + MMCCR);
            MMCRIR_Register = new Register(STM32F4xx.ETH_BASE + MMCRIR);
            MMCTIR_Register = new Register(STM32F4xx.ETH_BASE + MMCTIR);
            MMCRIMR_Register = new Register(STM32F4xx.ETH_BASE + MMCRIMR);
            MMCTIMR_Register = new Register(STM32F4xx.ETH_BASE + MMCTIMR);
            MMCTGFSCCR_Register = new Register(STM32F4xx.ETH_BASE + MMCTGFSCCR);
            MMCTGFMSCCR_Register = new Register(STM32F4xx.ETH_BASE + MMCTGFMSCCR);
            MMCTGFCR_Register = new Register(STM32F4xx.ETH_BASE + MMCTGFCR);
            MMCRFCECR_Register = new Register(STM32F4xx.ETH_BASE + MMCRFCECR);
            MMCRFAECR_Register = new Register(STM32F4xx.ETH_BASE + MMCRFAECR);
            MMCRGUFCR_Register = new Register(STM32F4xx.ETH_BASE + MMCRGUFCR);
            PTPTSCR_Register = new Register(STM32F4xx.ETH_BASE + PTPTSCR);
            PTPSSIR_Register = new Register(STM32F4xx.ETH_BASE + PTPSSIR);
            PTPTSHR_Register = new Register(STM32F4xx.ETH_BASE + PTPTSHR);
            PTPTSLR_Register = new Register(STM32F4xx.ETH_BASE + PTPTSLR);
            PTPTSHUR_Register = new Register(STM32F4xx.ETH_BASE + PTPTSHUR);
            PTPTSLUR_Register = new Register(STM32F4xx.ETH_BASE + PTPTSLUR);
            PTPTSAR_Register = new Register(STM32F4xx.ETH_BASE + PTPTSAR);
            PTPTTHR_Register = new Register(STM32F4xx.ETH_BASE + PTPTTHR);
            PTPTTLR_Register = new Register(STM32F4xx.ETH_BASE + PTPTTLR);
            // RESERVED8_Register = new Register(STM32F4xx.ETH_BASE + RESERVED8);
            PTPTSSR_Register = new Register(STM32F4xx.ETH_BASE + PTPTSSR);
            DMABMR_Register = new Register(STM32F4xx.ETH_BASE + DMABMR);
            DMATPDR_Register = new Register(STM32F4xx.ETH_BASE + DMATPDR);
            DMARPDR_Register = new Register(STM32F4xx.ETH_BASE + DMARPDR);
            DMARDLAR_Register = new Register(STM32F4xx.ETH_BASE + DMARDLAR);
            DMATDLAR_Register = new Register(STM32F4xx.ETH_BASE + DMATDLAR);
            DMASR_Register = new Register(STM32F4xx.ETH_BASE + DMASR);
            DMAOMR_Register = new Register(STM32F4xx.ETH_BASE + DMAOMR);
            DMAIER_Register = new Register(STM32F4xx.ETH_BASE + DMAIER);
            DMAMFBOCR_Register = new Register(STM32F4xx.ETH_BASE + DMAMFBOCR);
            DMARSWTR_Register = new Register(STM32F4xx.ETH_BASE + DMARSWTR);
            DMACHTDR_Register = new Register(STM32F4xx.ETH_BASE + DMACHTDR);
            DMACHRDR_Register = new Register(STM32F4xx.ETH_BASE + DMACHRDR);
            DMACHTBAR_Register = new Register(STM32F4xx.ETH_BASE + DMACHTBAR);
            DMACHRBAR_Register = new Register(STM32F4xx.ETH_BASE + DMACHRBAR);
        }
    } // END of Class...

    /// <summary>
    /// Digital camera interface (DCMI) - See: P.452
    /// </summary>
    public static class DCMI
    {
        #region Private's...

        /// <summary>
        /// DCMI control register 1, Address offset: 0x00 
        /// </summary>
        private static uint _CR = 0x00;

        /// <summary>
        /// DCMI status register,Address offset: 0x04 
        /// </summary>
        private static uint _SR = 0x04;

        /// <summary>
        /// DCMI raw interrupt status register,Address offset: 0x08 
        /// </summary>
        private static uint _RISR = 0x08;

        /// <summary>
        /// DCMI interrupt enable register,Address offset: 0x0C 
        /// </summary>
        private static uint _IER = 0x0C;

        /// <summary>
        /// DCMI masked interrupt status register, Address offset: 0x10 
        /// </summary>
        private static uint _MISR = 0x10;

        /// <summary>
        /// DCMI interrupt clear register, Address offset: 0x14 
        /// </summary>
        private static uint _ICR = 0x14;

        /// <summary>
        /// DCMI embedded synchronization code register, Address offset: 0x18 
        /// </summary>
        private static uint _ESCR = 0x18;

        /// <summary>
        /// DCMI embedded synchronization unmask register, Address offset: 0x1C 
        /// </summary>
        private static uint _ESUR = 0x1C;

        /// <summary>
        /// DCMI crop window start,Address offset: 0x20 
        /// </summary>
        private static uint _CWSTRTR = 0x20;

        /// <summary>
        /// DCMI crop window size, Address offset: 0x24 
        /// </summary>
        private static uint _CWSIZER = 0x24;

        /// <summary>
        /// DCMI data register,Address offset: 0x28 
        /// </summary>
        private static uint _DR = 0x28;

        #endregion

        #region Properties...

        /// <summary>
        /// DCMI control register 1, Address offset: 0x00 
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// DCMI status register,Address offset: 0x04 
        /// </summary>
        public static uint SR { get { return _SR; } set { _SR = value; } }

        /// <summary>
        /// DCMI raw interrupt status register,Address offset: 0x08 
        /// </summary>
        public static uint RISR { get { return _RISR; } set { _RISR = value; } }

        /// <summary>
        /// DCMI interrupt enable register,Address offset: 0x0C 
        /// </summary>
        public static uint IER { get { return _IER; } set { _IER = value; } }

        /// <summary>
        /// DCMI masked interrupt status register, Address offset: 0x10 
        /// </summary>
        public static uint MISR { get { return _MISR; } set { _MISR = value; } }

        /// <summary>
        /// DCMI interrupt clear register, Address offset: 0x14 
        /// </summary>
        public static uint ICR { get { return _ICR; } set { _ICR = value; } }

        /// <summary>
        /// DCMI embedded synchronization code register, Address offset: 0x18 
        /// </summary>
        public static uint ESCR { get { return _ESCR; } set { _ESCR = value; } }

        /// <summary>
        /// DCMI embedded synchronization unmask register, Address offset: 0x1C 
        /// </summary>
        public static uint ESUR { get { return _ESUR; } set { _ESUR = value; } }

        /// <summary>
        /// DCMI crop window start,Address offset: 0x20 
        /// </summary>
        public static uint CWSTRTR { get { return _CWSTRTR; } set { _CWSTRTR = value; } }

        /// <summary>
        /// DCMI crop window size, Address offset: 0x24 
        /// </summary>
        public static uint CWSIZER { get { return _CWSIZER; } set { _CWSIZER = value; } }

        /// <summary>
        /// DCMI data register,Address offset: 0x28 
        /// </summary>
        public static uint DR { get { return _DR; } set { _DR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// DCMI control register 1, Address offset: 0x00 
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// DCMI status register,Address offset: 0x04 
        /// </summary>
        public static Register SR_Register;

        /// <summary>
        /// DCMI raw interrupt status register,Address offset: 0x08 
        /// </summary>
        public static Register RISR_Register;

        /// <summary>
        /// DCMI interrupt enable register,Address offset: 0x0C 
        /// </summary>
        public static Register IER_Register;

        /// <summary>
        /// DCMI masked interrupt status register, Address offset: 0x10 
        /// </summary>
        public static Register MISR_Register;

        /// <summary>
        /// DCMI interrupt clear register, Address offset: 0x14 
        /// </summary>
        public static Register ICR_Register;

        /// <summary>
        /// DCMI embedded synchronization code register, Address offset: 0x18 
        /// </summary>
        public static Register ESCR_Register;

        /// <summary>
        /// DCMI embedded synchronization unmask register, Address offset: 0x1C 
        /// </summary>
        public static Register ESUR_Register;

        /// <summary>
        /// DCMI crop window start,Address offset: 0x20 
        /// </summary>
        public static Register CWSTRTR_Register;

        /// <summary>
        /// DCMI crop window size, Address offset: 0x24 
        /// </summary>
        public static Register CWSIZER_Register;

        /// <summary>
        /// DCMI data register,Address offset: 0x28 
        /// </summary>
        public static Register DR_Register;

        #endregion

        static DCMI()
        {
            CR_Register = new Register(STM32F4xx.DCMI_BASE + CR);
            SR_Register = new Register(STM32F4xx.DCMI_BASE + SR);
            RISR_Register = new Register(STM32F4xx.DCMI_BASE + RISR);
            IER_Register = new Register(STM32F4xx.DCMI_BASE + IER);
            MISR_Register = new Register(STM32F4xx.DCMI_BASE + MISR);
            ICR_Register = new Register(STM32F4xx.DCMI_BASE + ICR);
            ESCR_Register = new Register(STM32F4xx.DCMI_BASE + ESCR);
            ESUR_Register = new Register(STM32F4xx.DCMI_BASE + ESUR);
            CWSTRTR_Register = new Register(STM32F4xx.DCMI_BASE + CWSTRTR);
            CWSIZER_Register = new Register(STM32F4xx.DCMI_BASE + CWSIZER);
            DR_Register = new Register(STM32F4xx.DCMI_BASE + DR);
        }
    } // END of Class...

    /// <summary>
    /// Cryptographic processor (CRYP) - See: P.704
    /// </summary>
    public static class CRYP
    {
        #region Private's...

        /// <summary>
        /// CRYP control register,Address offset: 0x00 
        /// </summary>
        private static uint _CR = 0x00;

        /// <summary>
        /// CRYP status register, Address offset: 0x04 
        /// </summary>
        private static uint _SR = 0x04;

        /// <summary>
        /// CRYP data input register, Address offset: 0x08 
        /// </summary>
        private static uint _DR = 0x08;

        /// <summary>
        /// CRYP data output register,Address offset: 0x0C 
        /// </summary>
        private static uint _DOUT = 0x0C;

        /// <summary>
        /// CRYP DMA control register,Address offset: 0x10 
        /// </summary>
        private static uint _DMACR = 0x10;

        /// <summary>
        /// CRYP interrupt mask set/clear register, Address offset: 0x14 
        /// </summary>
        private static uint _IMSCR = 0x14;

        /// <summary>
        /// CRYP raw interrupt status register, Address offset: 0x18 
        /// </summary>
        private static uint _RISR = 0x18;

        /// <summary>
        /// CRYP masked interrupt status register,Address offset: 0x1C 
        /// </summary>
        private static uint _MISR = 0x1C;

        /// <summary>
        /// CRYP key leftregister 0,Address offset: 0x20 
        /// </summary>
        private static uint _K0LR = 0x20;

        /// <summary>
        /// CRYP key right register 0,Address offset: 0x24 
        /// </summary>
        private static uint _K0RR = 0x24;

        /// <summary>
        /// CRYP key leftregister 1,Address offset: 0x28 
        /// </summary>
        private static uint _K1LR = 0x28;

        /// <summary>
        /// CRYP key right register 1,Address offset: 0x2C 
        /// </summary>
        private static uint _K1RR = 0x2C;

        /// <summary>
        /// CRYP key leftregister 2,Address offset: 0x30 
        /// </summary>
        private static uint _K2LR = 0x30;

        /// <summary>
        /// CRYP key right register 2,Address offset: 0x34 
        /// </summary>
        private static uint _K2RR = 0x34;

        /// <summary>
        /// CRYP key leftregister 3,Address offset: 0x38 
        /// </summary>
        private static uint _K3LR = 0x38;

        /// <summary>
        /// CRYP key right register 3,Address offset: 0x3C 
        /// </summary>
        private static uint _K3RR = 0x3C;

        /// <summary>
        /// CRYP initialization vector left-wordregister 0, Address offset: 0x40 
        /// </summary>
        private static uint _IV0LR = 0x40;

        /// <summary>
        /// CRYP initialization vector right-word register 0, Address offset: 0x44 
        /// </summary>
        private static uint _IV0RR = 0x44;

        /// <summary>
        /// CRYP initialization vector left-wordregister 1, Address offset: 0x48 
        /// </summary>
        private static uint _IV1LR = 0x48;

        /// <summary>
        /// CRYP initialization vector right-word register 1, Address offset: 0x4C 
        /// </summary>
        private static uint _IV1RR = 0x4C;

        #endregion

        #region Properties...

        /// <summary>
        /// CRYP control register,Address offset: 0x00 
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// CRYP status register, Address offset: 0x04 
        /// </summary>
        public static uint SR { get { return _SR; } set { _SR = value; } }

        /// <summary>
        /// CRYP data input register, Address offset: 0x08 
        /// </summary>
        public static uint DR { get { return _DR; } set { _DR = value; } }

        /// <summary>
        /// CRYP data output register,Address offset: 0x0C 
        /// </summary>
        public static uint DOUT { get { return _DOUT; } set { _DOUT = value; } }

        /// <summary>
        /// CRYP DMA control register,Address offset: 0x10 
        /// </summary>
        public static uint DMACR { get { return _DMACR; } set { _DMACR = value; } }

        /// <summary>
        /// CRYP interrupt mask set/clear register, Address offset: 0x14 
        /// </summary>
        public static uint IMSCR { get { return _IMSCR; } set { _IMSCR = value; } }

        /// <summary>
        /// CRYP raw interrupt status register, Address offset: 0x18 
        /// </summary>
        public static uint RISR { get { return _RISR; } set { _RISR = value; } }

        /// <summary>
        /// CRYP masked interrupt status register,Address offset: 0x1C 
        /// </summary>
        public static uint MISR { get { return _MISR; } set { _MISR = value; } }

        /// <summary>
        /// CRYP key leftregister 0,Address offset: 0x20 
        /// </summary>
        public static uint K0LR { get { return _K0LR; } set { _K0LR = value; } }

        /// <summary>
        /// CRYP key right register 0,Address offset: 0x24 
        /// </summary>
        public static uint K0RR { get { return _K0RR; } set { _K0RR = value; } }

        /// <summary>
        /// CRYP key leftregister 1,Address offset: 0x28 
        /// </summary>
        public static uint K1LR { get { return _K1LR; } set { _K1LR = value; } }

        /// <summary>
        /// CRYP key right register 1,Address offset: 0x2C 
        /// </summary>
        public static uint K1RR { get { return _K1RR; } set { _K1RR = value; } }

        /// <summary>
        /// CRYP key leftregister 2,Address offset: 0x30 
        /// </summary>
        public static uint K2LR { get { return _K2LR; } set { _K2LR = value; } }

        /// <summary>
        /// CRYP key right register 2,Address offset: 0x34 
        /// </summary>
        public static uint K2RR { get { return _K2RR; } set { _K2RR = value; } }

        /// <summary>
        /// CRYP key leftregister 3,Address offset: 0x38 
        /// </summary>
        public static uint K3LR { get { return _K3LR; } set { _K3LR = value; } }

        /// <summary>
        /// CRYP key right register 3,Address offset: 0x3C 
        /// </summary>
        public static uint K3RR { get { return _K3RR; } set { _K3RR = value; } }

        /// <summary>
        /// CRYP initialization vector left-wordregister 0, Address offset: 0x40 
        /// </summary>
        public static uint IV0LR { get { return _IV0LR; } set { _IV0LR = value; } }

        /// <summary>
        /// CRYP initialization vector right-word register 0, Address offset: 0x44 
        /// </summary>
        public static uint IV0RR { get { return _IV0RR; } set { _IV0RR = value; } }

        /// <summary>
        /// CRYP initialization vector left-wordregister 1, Address offset: 0x48 
        /// </summary>
        public static uint IV1LR { get { return _IV1LR; } set { _IV1LR = value; } }

        /// <summary>
        /// CRYP initialization vector right-word register 1, Address offset: 0x4C 
        /// </summary>
        public static uint IV1RR { get { return _IV1RR; } set { _IV1RR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// CRYP control register,Address offset: 0x00 
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// CRYP status register, Address offset: 0x04 
        /// </summary>
        public static Register SR_Register;

        /// <summary>
        /// CRYP data input register, Address offset: 0x08 
        /// </summary>
        public static Register DR_Register;

        /// <summary>
        /// CRYP data output register,Address offset: 0x0C 
        /// </summary>
        public static Register DOUT_Register;

        /// <summary>
        /// CRYP DMA control register,Address offset: 0x10 
        /// </summary>
        public static Register DMACR_Register;

        /// <summary>
        /// CRYP interrupt mask set/clear register, Address offset: 0x14 
        /// </summary>
        public static Register IMSCR_Register;

        /// <summary>
        /// CRYP raw interrupt status register, Address offset: 0x18 
        /// </summary>
        public static Register RISR_Register;

        /// <summary>
        /// CRYP masked interrupt status register,Address offset: 0x1C 
        /// </summary>
        public static Register MISR_Register;

        /// <summary>
        /// CRYP key leftregister 0,Address offset: 0x20 
        /// </summary>
        public static Register K0LR_Register;

        /// <summary>
        /// CRYP key right register 0,Address offset: 0x24 
        /// </summary>
        public static Register K0RR_Register;

        /// <summary>
        /// CRYP key leftregister 1,Address offset: 0x28 
        /// </summary>
        public static Register K1LR_Register;

        /// <summary>
        /// CRYP key right register 1,Address offset: 0x2C 
        /// </summary>
        public static Register K1RR_Register;

        /// <summary>
        /// CRYP key leftregister 2,Address offset: 0x30 
        /// </summary>
        public static Register K2LR_Register;

        /// <summary>
        /// CRYP key right register 2,Address offset: 0x34 
        /// </summary>
        public static Register K2RR_Register;

        /// <summary>
        /// CRYP key leftregister 3,Address offset: 0x38 
        /// </summary>
        public static Register K3LR_Register;

        /// <summary>
        /// CRYP key right register 3,Address offset: 0x3C 
        /// </summary>
        public static Register K3RR_Register;

        /// <summary>
        /// CRYP initialization vector left-wordregister 0, Address offset: 0x40 
        /// </summary>
        public static Register IV0LR_Register;

        /// <summary>
        /// CRYP initialization vector right-word register 0, Address offset: 0x44 
        /// </summary>
        public static Register IV0RR_Register;

        /// <summary>
        /// CRYP initialization vector left-wordregister 1, Address offset: 0x48 
        /// </summary>
        public static Register IV1LR_Register;

        /// <summary>
        /// CRYP initialization vector right-word register 1, Address offset: 0x4C 
        /// </summary>
        public static Register IV1RR_Register;

        #endregion

        static CRYP()
        {
            CR_Register = new Register(STM32F4xx.CRYP_BASE + CR);
            SR_Register = new Register(STM32F4xx.CRYP_BASE + SR);
            DR_Register = new Register(STM32F4xx.CRYP_BASE + DR);
            DOUT_Register = new Register(STM32F4xx.CRYP_BASE + DOUT);
            DMACR_Register = new Register(STM32F4xx.CRYP_BASE + DMACR);
            IMSCR_Register = new Register(STM32F4xx.CRYP_BASE + IMSCR);
            RISR_Register = new Register(STM32F4xx.CRYP_BASE + RISR);
            MISR_Register = new Register(STM32F4xx.CRYP_BASE + MISR);
            K0LR_Register = new Register(STM32F4xx.CRYP_BASE + K0LR);
            K0RR_Register = new Register(STM32F4xx.CRYP_BASE + K0RR);
            K1LR_Register = new Register(STM32F4xx.CRYP_BASE + K1LR);
            K1RR_Register = new Register(STM32F4xx.CRYP_BASE + K1RR);
            K2LR_Register = new Register(STM32F4xx.CRYP_BASE + K2LR);
            K2RR_Register = new Register(STM32F4xx.CRYP_BASE + K2RR);
            K3LR_Register = new Register(STM32F4xx.CRYP_BASE + K3LR);
            K3RR_Register = new Register(STM32F4xx.CRYP_BASE + K3RR);
            IV0LR_Register = new Register(STM32F4xx.CRYP_BASE + IV0LR);
            IV0RR_Register = new Register(STM32F4xx.CRYP_BASE + IV0RR);
            IV1LR_Register = new Register(STM32F4xx.CRYP_BASE + IV1LR);
            IV1RR_Register = new Register(STM32F4xx.CRYP_BASE + IV1RR);
        }
    } // END of Class...

    /// <summary>
    /// Hash processor (HASH) - See: P.755
    /// </summary>
    public static class HASH
    {
        #region Private's...

        /// <summary>
        /// HASH control register,Address offset: 0x00
        /// </summary>
        private static uint _CR = 0x00;

        /// <summary>
        /// HASH data input register, Address offset: 0x04
        /// </summary>
        private static uint _DIN = 0x04;

        /// <summary>
        /// HASH start register,Address offset: 0x08
        /// </summary>
        private static uint _STR = 0x08;

        /// <summary>
        /// HASH digest registers,Address offset: 0x0C-0x1C 
        /// </summary>
        // private static uint _HR[5] =  0x0C-0x1C ;

        /// <summary>
        /// HASH interrupt enable register, Address offset: 0x20
        /// </summary>
        private static uint _IMR = 0x20;

        /// <summary>
        /// HASH status register, Address offset: 0x24
        /// </summary>
        private static uint _SR = 0x24;

        /// <summary>
        /// HASH context swap registers,Address offset: 0x0F8-0x1C0 
        /// </summary>
        // private static uint _CSR[51] =  0x0F8-0x1C0 ;

        #endregion

        #region Properties...

        /// <summary>
        /// HASH control register,Address offset: 0x00
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// HASH data input register, Address offset: 0x04
        /// </summary>
        public static uint DIN { get { return _DIN; } set { _DIN = value; } }

        /// <summary>
        /// HASH start register,Address offset: 0x08
        /// </summary>
        public static uint STR { get { return _STR; } set { _STR = value; } }

        /// <summary>
        /// HASH digest registers,Address offset: 0x0C-0x1C 
        /// </summary>
        // public static uint HR[5]  { get { return _HR[5]; } set { _HR[5] = value; } }

        /// <summary>
        /// HASH interrupt enable register, Address offset: 0x20
        /// </summary>
        public static uint IMR { get { return _IMR; } set { _IMR = value; } }

        /// <summary>
        /// HASH status register, Address offset: 0x24
        /// </summary>
        public static uint SR { get { return _SR; } set { _SR = value; } }

        /// <summary>
        /// HASH context swap registers,Address offset: 0x0F8-0x1C0 
        /// </summary>
        // public static uint CSR[51]  { get { return _CSR[51]; } set { _CSR[51] = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// HASH control register,Address offset: 0x00
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// HASH data input register, Address offset: 0x04
        /// </summary>
        public static Register DIN_Register;

        /// <summary>
        /// HASH start register,Address offset: 0x08
        /// </summary>
        public static Register STR_Register;

        /// <summary>
        /// HASH digest registers,Address offset: 0x0C-0x1C 
        /// </summary>
        // public static Register HR[5]_Register;

        /// <summary>
        /// HASH interrupt enable register, Address offset: 0x20
        /// </summary>
        public static Register IMR_Register;

        /// <summary>
        /// HASH status register, Address offset: 0x24
        /// </summary>
        public static Register SR_Register;

        /// <summary>
        /// HASH context swap registers,Address offset: 0x0F8-0x1C0 
        /// </summary>
        // public Register CSR[51]_Register;

        #endregion

        static HASH()
        {
            CR_Register = new Register(STM32F4xx.HASH_BASE + CR);
            DIN_Register = new Register(STM32F4xx.HASH_BASE + DIN);
            STR_Register = new Register(STM32F4xx.HASH_BASE + STR);
            //HR[5]_Register = new Register(STM32F4xx.HASH_BASE + HR[5]);
            IMR_Register = new Register(STM32F4xx.HASH_BASE + IMR);
            SR_Register = new Register(STM32F4xx.HASH_BASE + SR);
            //CSR[51]_Register = new Register(STM32F4xx.HASH_BASE + CSR[51]);
        }
    } // END of Class...

    /// <summary>
    /// Random number generator (RNG) - See: P.750
    /// </summary>
    public static class RNG
    {
        #region Private's...

        /// <summary>
        /// RNG control register, Address offset: 0x00 
        /// </summary>
        private static uint _CR = 0x00;

        /// <summary>
        /// RNG status register,Address offset: 0x04 
        /// </summary>
        private static uint _SR = 0x04;

        /// <summary>
        /// RNG data register,Address offset: 0x08 
        /// </summary>
        private static uint _DR = 0x08;

        #endregion

        #region Properties...

        /// <summary>
        /// RNG control register, Address offset: 0x00 
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// RNG status register,Address offset: 0x04 
        /// </summary>
        public static uint SR { get { return _SR; } set { _SR = value; } }

        /// <summary>
        /// RNG data register,Address offset: 0x08 
        /// </summary>
        public static uint DR { get { return _DR; } set { _DR = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// RNG control register, Address offset: 0x00 
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// RNG status register,Address offset: 0x04 
        /// </summary>
        public static Register SR_Register;

        /// <summary>
        /// RNG data register,Address offset: 0x08 
        /// </summary>
        public static Register DR_Register;

        #endregion

        static RNG()
        {
            CR_Register = new Register(STM32F4xx.RNG_BASE + CR);
            SR_Register = new Register(STM32F4xx.RNG_BASE + SR);
            DR_Register = new Register(STM32F4xx.RNG_BASE + DR);
        }
    } // END of Class...

    /// <summary>
    /// Flexible static memory controller (FSMC) - See: P.1522
    /// NOT Implimented...
    /// </summary>
    public static class FSMC_Bank1
    {
        #region Private's...

        /// <summary>
        /// NOR/PSRAM chip-select control register(BCR) and chip-select timing register(BTR), Address offset: 0x00-1C
        /// </summary>
        // private static uint _BTCR[8] =  0x00-1C;

        #endregion

        #region Properties...

        /// <summary>
        /// NOR/PSRAM chip-select control register(BCR) and chip-select timing register(BTR), Address offset: 0x00-1C
        /// </summary>
        // public static uint BTCR[8]  { get { return _BTCR[8]; } set { _BTCR[8] = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// NOR/PSRAM chip-select control register(BCR) and chip-select timing register(BTR), Address offset: 0x00-1C
        /// </summary>
        // public static Register BTCR[8]_Register;

        #endregion

        static FSMC_Bank1()
        {
            // BTCR[8]_Register = new Register(STM32F4xx.FSMC_Bank1_R_BASE + BTCR[8]);
        }
    } // END of Class...

    /// <summary>
    /// Flexible static memory controller (FSMC) - See: P.1522
    /// NOT Implimented...
    /// </summary>
    public static class FSMC_Bank1E
    {
        #region Private's...

        /// <summary>
        /// NOR/PSRAM write timing registers, Address offset: 0x104-0x11C 
        /// </summary>
        // private static uint _BWTR[7] =  0x104-0x11C ;

        #endregion

        #region Properties...

        /// <summary>
        /// NOR/PSRAM write timing registers, Address offset: 0x104-0x11C 
        /// </summary>
        // public static uint BWTR[7]  { get { return _BWTR[7]; } set { _BWTR[7] = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// NOR/PSRAM write timing registers, Address offset: 0x104-0x11C 
        /// </summary>
        // public static Register BWTR[7]_Register;

        #endregion

        static FSMC_Bank1E()
        {
            // BWTR[7]_Register = new Register(XX_BASE + BWTR[7]);
        }
    } // END of Class...

    /// <summary>
    /// Flexible static memory controller (FSMC) - See: P.1522
    /// </summary>
    public static class FSMC_Bank2
    {
        #region Private's...

        /// <summary>
        /// NAND Flash control register 2, Address offset: 0x60 
        /// </summary>
        private static uint _PCR2 = 0x60;

        /// <summary>
        /// NAND Flash FIFO status and interrupt register 2, Address offset: 0x64 
        /// </summary>
        private static uint _SR2 = 0x64;

        /// <summary>
        /// NAND Flash Common memory space timing register 2,Address offset: 0x68 
        /// </summary>
        private static uint _PMEM2 = 0x68;

        /// <summary>
        /// NAND Flash Attribute memory space timing register 2, Address offset: 0x6C 
        /// </summary>
        private static uint _PATT2 = 0x6C;

        /// <summary>
        /// NAND Flash ECC result registers 2, Address offset: 0x74 
        /// </summary>
        private static uint _ECCR2 = 0x74;

        #endregion

        #region Properties...

        /// <summary>
        /// NAND Flash control register 2, Address offset: 0x60 
        /// </summary>
        public static uint PCR2 { get { return _PCR2; } set { _PCR2 = value; } }

        /// <summary>
        /// NAND Flash FIFO status and interrupt register 2, Address offset: 0x64 
        /// </summary>
        public static uint SR2 { get { return _SR2; } set { _SR2 = value; } }

        /// <summary>
        /// NAND Flash Common memory space timing register 2,Address offset: 0x68 
        /// </summary>
        public static uint PMEM2 { get { return _PMEM2; } set { _PMEM2 = value; } }

        /// <summary>
        /// NAND Flash Attribute memory space timing register 2, Address offset: 0x6C 
        /// </summary>
        public static uint PATT2 { get { return _PATT2; } set { _PATT2 = value; } }

        /// <summary>
        /// NAND Flash ECC result registers 2, Address offset: 0x74 
        /// </summary>
        public static uint ECCR2 { get { return _ECCR2; } set { _ECCR2 = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// NAND Flash control register 2, Address offset: 0x60 
        /// </summary>
        public static Register PCR2_Register;

        /// <summary>
        /// NAND Flash FIFO status and interrupt register 2, Address offset: 0x64 
        /// </summary>
        public static Register SR2_Register;

        /// <summary>
        /// NAND Flash Common memory space timing register 2,Address offset: 0x68 
        /// </summary>
        public static Register PMEM2_Register;

        /// <summary>
        /// NAND Flash Attribute memory space timing register 2, Address offset: 0x6C 
        /// </summary>
        public static Register PATT2_Register;

        /// <summary>
        /// NAND Flash ECC result registers 2, Address offset: 0x74 
        /// </summary>
        public static Register ECCR2_Register;

        #endregion

        static FSMC_Bank2()
        {
            PCR2_Register = new Register(STM32F4xx.FSMC_Bank2_R_BASE + PCR2);
            SR2_Register = new Register(STM32F4xx.FSMC_Bank2_R_BASE + SR2);
            PMEM2_Register = new Register(STM32F4xx.FSMC_Bank2_R_BASE + PMEM2);
            PATT2_Register = new Register(STM32F4xx.FSMC_Bank2_R_BASE + PATT2);
            ECCR2_Register = new Register(STM32F4xx.FSMC_Bank2_R_BASE + ECCR2);
        }
    } // END of Class...

    /// <summary>
    /// Flexible static memory controller (FSMC) - See: P.1522
    /// </summary>
    public static class FSMC_Bank3
    {
        #region Private's...

        /// <summary>
        /// NAND Flash control register 3, Address offset: 0x80 
        /// </summary>
        private static uint _PCR3 = 0x80;

        /// <summary>
        /// NAND Flash FIFO status and interrupt register 3, Address offset: 0x84 
        /// </summary>
        private static uint _SR3 = 0x84;

        /// <summary>
        /// NAND Flash Common memory space timing register 3,Address offset: 0x88 
        /// </summary>
        private static uint _PMEM3 = 0x88;

        /// <summary>
        /// NAND Flash Attribute memory space timing register 3, Address offset: 0x8C 
        /// </summary>
        private static uint _PATT3 = 0x8C;

        /// <summary>
        /// NAND Flash ECC result registers 3, Address offset: 0x94 
        /// </summary>
        private static uint _ECCR3 = 0x94;

        #endregion

        #region Properties...

        /// <summary>
        /// NAND Flash control register 3, Address offset: 0x80 
        /// </summary>
        public static uint PCR3 { get { return _PCR3; } set { _PCR3 = value; } }

        /// <summary>
        /// NAND Flash FIFO status and interrupt register 3, Address offset: 0x84 
        /// </summary>
        public static uint SR3 { get { return _SR3; } set { _SR3 = value; } }

        /// <summary>
        /// NAND Flash Common memory space timing register 3,Address offset: 0x88 
        /// </summary>
        public static uint PMEM3 { get { return _PMEM3; } set { _PMEM3 = value; } }

        /// <summary>
        /// NAND Flash Attribute memory space timing register 3, Address offset: 0x8C 
        /// </summary>
        public static uint PATT3 { get { return _PATT3; } set { _PATT3 = value; } }

        /// <summary>
        /// NAND Flash ECC result registers 3, Address offset: 0x94 
        /// </summary>
        public static uint ECCR3 { get { return _ECCR3; } set { _ECCR3 = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// NAND Flash control register 3, Address offset: 0x80 
        /// </summary>
        public static Register PCR3_Register;

        /// <summary>
        /// NAND Flash FIFO status and interrupt register 3, Address offset: 0x84 
        /// </summary>
        public static Register SR3_Register;

        /// <summary>
        /// NAND Flash Common memory space timing register 3,Address offset: 0x88 
        /// </summary>
        public static Register PMEM3_Register;

        /// <summary>
        /// NAND Flash Attribute memory space timing register 3, Address offset: 0x8C 
        /// </summary>
        public static Register PATT3_Register;

        /// <summary>
        /// NAND Flash ECC result registers 3, Address offset: 0x94 
        /// </summary>
        public static Register ECCR3_Register;

        #endregion

        static FSMC_Bank3()
        {
            PCR3_Register = new Register(STM32F4xx.FSMC_Bank3_R_BASE + PCR3);
            SR3_Register = new Register(STM32F4xx.FSMC_Bank3_R_BASE + SR3);
            PMEM3_Register = new Register(STM32F4xx.FSMC_Bank3_R_BASE + PMEM3);
            PATT3_Register = new Register(STM32F4xx.FSMC_Bank3_R_BASE + PATT3);
            ECCR3_Register = new Register(STM32F4xx.FSMC_Bank3_R_BASE + ECCR3);
        }
    } // END of Class...

    /// <summary>
    /// Flexible static memory controller (FSMC) - See: P.1522
    /// </summary>
    public static class FSMC_Bank4
    {
        #region Private's...

        /// <summary>
        /// PC Cardcontrol register 4, Address offset: 0xA0 
        /// </summary>
        private static uint _PCR4 = 0xA0;

        /// <summary>
        /// PC CardFIFO status and interrupt register 4, Address offset: 0xA4 
        /// </summary>
        private static uint _SR4 = 0xA4;

        /// <summary>
        /// PC CardCommon memory space timing register 4,Address offset: 0xA8 
        /// </summary>
        private static uint _PMEM4 = 0xA8;

        /// <summary>
        /// PC CardAttribute memory space timing register 4, Address offset: 0xAC 
        /// </summary>
        private static uint _PATT4 = 0xAC;

        /// <summary>
        /// PC CardI/O space timing register 4,Address offset: 0xB0 
        /// </summary>
        private static uint _PIO4 = 0xB0;

        #endregion

        #region Properties...

        /// <summary>
        /// PC Cardcontrol register 4, Address offset: 0xA0 
        /// </summary>
        public static uint PCR4 { get { return _PCR4; } set { _PCR4 = value; } }

        /// <summary>
        /// PC CardFIFO status and interrupt register 4, Address offset: 0xA4 
        /// </summary>
        public static uint SR4 { get { return _SR4; } set { _SR4 = value; } }

        /// <summary>
        /// PC CardCommon memory space timing register 4,Address offset: 0xA8 
        /// </summary>
        public static uint PMEM4 { get { return _PMEM4; } set { _PMEM4 = value; } }

        /// <summary>
        /// PC CardAttribute memory space timing register 4, Address offset: 0xAC 
        /// </summary>
        public static uint PATT4 { get { return _PATT4; } set { _PATT4 = value; } }

        /// <summary>
        /// PC CardI/O space timing register 4,Address offset: 0xB0 
        /// </summary>
        public static uint PIO4 { get { return _PIO4; } set { _PIO4 = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// PC Cardcontrol register 4, Address offset: 0xA0 
        /// </summary>
        public static Register PCR4_Register;

        /// <summary>
        /// PC CardFIFO status and interrupt register 4, Address offset: 0xA4 
        /// </summary>
        public static Register SR4_Register;

        /// <summary>
        /// PC CardCommon memory space timing register 4,Address offset: 0xA8 
        /// </summary>
        public static Register PMEM4_Register;

        /// <summary>
        /// PC CardAttribute memory space timing register 4, Address offset: 0xAC 
        /// </summary>
        public static Register PATT4_Register;

        /// <summary>
        /// PC CardI/O space timing register 4,Address offset: 0xB0 
        /// </summary>
        public static Register PIO4_Register;

        #endregion

        static FSMC_Bank4()
        {
            PCR4_Register = new Register(STM32F4xx.FSMC_Bank4_R_BASE + PCR4);
            SR4_Register = new Register(STM32F4xx.FSMC_Bank4_R_BASE + SR4);
            PMEM4_Register = new Register(STM32F4xx.FSMC_Bank4_R_BASE + PMEM4);
            PATT4_Register = new Register(STM32F4xx.FSMC_Bank4_R_BASE + PATT4);
            PIO4_Register = new Register(STM32F4xx.FSMC_Bank4_R_BASE + PIO4);
        }
    } // END of Class...

    /// <summary>
    /// MCU debug component (DBGMCU) - See: P.1680
    /// </summary>
    public sealed class DBGMCU
    {
        #region Private's...

        /// <summary>
        /// MCU device ID code, Address offset: 0x00 
        /// </summary>
        private static uint _IDCODE = 0x00;

        /// <summary>
        /// Debug MCU configuration register, Address offset: 0x04 
        /// </summary>
        private static uint _CR = 0x04;

        /// <summary>
        /// Debug MCU APB1 freeze register, Address offset: 0x08 
        /// </summary>
        private static uint _APB1FZ = 0x08;

        /// <summary>
        /// Debug MCU APB2 freeze register, Address offset: 0x0C 
        /// </summary>
        private static uint _APB2FZ = 0x0C;

        #endregion

        #region Properties...

        /// <summary>
        /// MCU device ID code, Address offset: 0x00 
        /// </summary>
        public static uint IDCODE { get { return _IDCODE; } set { _IDCODE = value; } }

        /// <summary>
        /// Debug MCU configuration register, Address offset: 0x04 
        /// </summary>
        public static uint CR { get { return _CR; } set { _CR = value; } }

        /// <summary>
        /// Debug MCU APB1 freeze register, Address offset: 0x08 
        /// </summary>
        public static uint APB1FZ { get { return _APB1FZ; } set { _APB1FZ = value; } }

        /// <summary>
        /// Debug MCU APB2 freeze register, Address offset: 0x0C 
        /// </summary>
        public static uint APB2FZ { get { return _APB2FZ; } set { _APB2FZ = value; } }

        #endregion

        #region Registers...

        /// <summary>
        /// MCU device ID code, Address offset: 0x00 
        /// </summary>
        public static Register IDCODE_Register;

        /// <summary>
        /// Debug MCU configuration register, Address offset: 0x04 
        /// </summary>
        public static Register CR_Register;

        /// <summary>
        /// Debug MCU APB1 freeze register, Address offset: 0x08 
        /// </summary>
        public static Register APB1FZ_Register;

        /// <summary>
        /// Debug MCU APB2 freeze register, Address offset: 0x0C 
        /// </summary>
        public static Register APB2FZ_Register;

        #endregion

        public DBGMCU()
        {
            IDCODE_Register = new Register(STM32F4xx.DBGMCU_BASE + IDCODE);
            CR_Register = new Register(STM32F4xx.DBGMCU_BASE + CR);
            APB1FZ_Register = new Register(STM32F4xx.DBGMCU_BASE + APB1FZ);
            APB2FZ_Register = new Register(STM32F4xx.DBGMCU_BASE + APB2FZ);
        }
    } // END of Class...

}
